@@ -34,7 +34,6 @@ static const char szKeySet[] = "wine_test_keyset";
 static const char szBadKeySet[] = "wine_test_bad_keyset";
 #define NON_DEF_PROV_TYPE 999
 
-static HMODULE hadvapi32;
 static BOOL (WINAPI *pCryptAcquireContextA)(HCRYPTPROV*,LPCSTR,LPCSTR,DWORD,DWORD);
 static BOOL (WINAPI *pCryptEnumProviderTypesA)(DWORD, DWORD*, DWORD, DWORD*, LPSTR, DWORD*);
 static BOOL (WINAPI *pCryptEnumProvidersA)(DWORD, DWORD*, DWORD, DWORD*, LPSTR, DWORD*);
@@ -68,41 +67,38 @@ static BOOL (WINAPI *pCryptVerifySignatureW)(HCRYPTHASH, BYTE*, DWORD, HCRYPTKEY
 
 static void init_function_pointers(void)
 {
-    hadvapi32 = GetModuleHandleA("advapi32.dll");
+    HMODULE hadvapi32 = GetModuleHandleA("advapi32.dll");
 
-    if(hadvapi32)
-    {
-        pCryptAcquireContextA = (void*)GetProcAddress(hadvapi32, "CryptAcquireContextA");
-        pCryptEnumProviderTypesA = (void*)GetProcAddress(hadvapi32, "CryptEnumProviderTypesA");
-        pCryptEnumProvidersA = (void*)GetProcAddress(hadvapi32, "CryptEnumProvidersA");
-        pCryptGetDefaultProviderA = (void*)GetProcAddress(hadvapi32, "CryptGetDefaultProviderA");
-        pCryptReleaseContext = (void*)GetProcAddress(hadvapi32, "CryptReleaseContext");
-        pCryptSetProviderExA = (void*)GetProcAddress(hadvapi32, "CryptSetProviderExA");
-        pCryptCreateHash = (void*)GetProcAddress(hadvapi32, "CryptCreateHash");
-        pCryptDestroyHash = (void*)GetProcAddress(hadvapi32, "CryptDestroyHash");
-        pCryptGenRandom = (void*)GetProcAddress(hadvapi32, "CryptGenRandom");
-        pCryptContextAddRef = (void*)GetProcAddress(hadvapi32, "CryptContextAddRef");
-        pCryptGenKey = (void*)GetProcAddress(hadvapi32, "CryptGenKey");
-        pCryptDestroyKey = (void*)GetProcAddress(hadvapi32, "CryptDestroyKey");
-        pCryptDecrypt = (void*)GetProcAddress(hadvapi32, "CryptDecrypt");
-        pCryptDeriveKey = (void*)GetProcAddress(hadvapi32, "CryptDeriveKey");
-        pCryptDuplicateHash = (void*)GetProcAddress(hadvapi32, "CryptDuplicateHash");
-        pCryptDuplicateKey = (void*)GetProcAddress(hadvapi32, "CryptDuplicateKey");
-        pCryptEncrypt = (void*)GetProcAddress(hadvapi32, "CryptEncrypt");
-        pCryptExportKey = (void*)GetProcAddress(hadvapi32, "CryptExportKey");
-        pCryptGetHashParam = (void*)GetProcAddress(hadvapi32, "CryptGetHashParam");
-        pCryptGetKeyParam = (void*)GetProcAddress(hadvapi32, "CryptGetKeyParam");
-        pCryptGetProvParam = (void*)GetProcAddress(hadvapi32, "CryptGetProvParam");
-        pCryptGetUserKey = (void*)GetProcAddress(hadvapi32, "CryptGetUserKey");
-        pCryptHashData = (void*)GetProcAddress(hadvapi32, "CryptHashData");
-        pCryptHashSessionKey = (void*)GetProcAddress(hadvapi32, "CryptHashSessionKey");
-        pCryptImportKey = (void*)GetProcAddress(hadvapi32, "CryptImportKey");
-        pCryptSignHashW = (void*)GetProcAddress(hadvapi32, "CryptSignHashW");
-        pCryptSetHashParam = (void*)GetProcAddress(hadvapi32, "CryptSetHashParam");
-        pCryptSetKeyParam = (void*)GetProcAddress(hadvapi32, "CryptSetKeyParam");
-        pCryptSetProvParam = (void*)GetProcAddress(hadvapi32, "CryptSetProvParam");
-        pCryptVerifySignatureW = (void*)GetProcAddress(hadvapi32, "CryptVerifySignatureW");
-    }
+    pCryptAcquireContextA = (void*)GetProcAddress(hadvapi32, "CryptAcquireContextA");
+    pCryptEnumProviderTypesA = (void*)GetProcAddress(hadvapi32, "CryptEnumProviderTypesA");
+    pCryptEnumProvidersA = (void*)GetProcAddress(hadvapi32, "CryptEnumProvidersA");
+    pCryptGetDefaultProviderA = (void*)GetProcAddress(hadvapi32, "CryptGetDefaultProviderA");
+    pCryptReleaseContext = (void*)GetProcAddress(hadvapi32, "CryptReleaseContext");
+    pCryptSetProviderExA = (void*)GetProcAddress(hadvapi32, "CryptSetProviderExA");
+    pCryptCreateHash = (void*)GetProcAddress(hadvapi32, "CryptCreateHash");
+    pCryptDestroyHash = (void*)GetProcAddress(hadvapi32, "CryptDestroyHash");
+    pCryptGenRandom = (void*)GetProcAddress(hadvapi32, "CryptGenRandom");
+    pCryptContextAddRef = (void*)GetProcAddress(hadvapi32, "CryptContextAddRef");
+    pCryptGenKey = (void*)GetProcAddress(hadvapi32, "CryptGenKey");
+    pCryptDestroyKey = (void*)GetProcAddress(hadvapi32, "CryptDestroyKey");
+    pCryptDecrypt = (void*)GetProcAddress(hadvapi32, "CryptDecrypt");
+    pCryptDeriveKey = (void*)GetProcAddress(hadvapi32, "CryptDeriveKey");
+    pCryptDuplicateHash = (void*)GetProcAddress(hadvapi32, "CryptDuplicateHash");
+    pCryptDuplicateKey = (void*)GetProcAddress(hadvapi32, "CryptDuplicateKey");
+    pCryptEncrypt = (void*)GetProcAddress(hadvapi32, "CryptEncrypt");
+    pCryptExportKey = (void*)GetProcAddress(hadvapi32, "CryptExportKey");
+    pCryptGetHashParam = (void*)GetProcAddress(hadvapi32, "CryptGetHashParam");
+    pCryptGetKeyParam = (void*)GetProcAddress(hadvapi32, "CryptGetKeyParam");
+    pCryptGetProvParam = (void*)GetProcAddress(hadvapi32, "CryptGetProvParam");
+    pCryptGetUserKey = (void*)GetProcAddress(hadvapi32, "CryptGetUserKey");
+    pCryptHashData = (void*)GetProcAddress(hadvapi32, "CryptHashData");
+    pCryptHashSessionKey = (void*)GetProcAddress(hadvapi32, "CryptHashSessionKey");
+    pCryptImportKey = (void*)GetProcAddress(hadvapi32, "CryptImportKey");
+    pCryptSignHashW = (void*)GetProcAddress(hadvapi32, "CryptSignHashW");
+    pCryptSetHashParam = (void*)GetProcAddress(hadvapi32, "CryptSetHashParam");
+    pCryptSetKeyParam = (void*)GetProcAddress(hadvapi32, "CryptSetKeyParam");
+    pCryptSetProvParam = (void*)GetProcAddress(hadvapi32, "CryptSetProvParam");
+    pCryptVerifySignatureW = (void*)GetProcAddress(hadvapi32, "CryptVerifySignatureW");
 }
 
 static void init_environment(void)
@@ -327,10 +323,15 @@ static void test_incorrect_api_usage(void)
     result = pCryptImportKey(hProv, &temp, 1, (HCRYPTKEY)NULL, 0, &hKey2);
     ok (!result && GetLastError() == ERROR_INVALID_PARAMETER, "%d\n", GetLastError());
 
-    dwLen = 1;
-    result = pCryptSignHashW(hHash, 0, NULL, 0, &temp, &dwLen);
-    ok (!result && (GetLastError() == ERROR_INVALID_PARAMETER ||
-        GetLastError() == ERROR_CALL_NOT_IMPLEMENTED), "%d\n", GetLastError());
+    if (pCryptSignHashW)
+    {
+        dwLen = 1;
+        result = pCryptSignHashW(hHash, 0, NULL, 0, &temp, &dwLen);
+        ok (!result && (GetLastError() == ERROR_INVALID_PARAMETER ||
+            GetLastError() == ERROR_CALL_NOT_IMPLEMENTED), "%d\n", GetLastError());
+    }
+    else
+        skip("CryptSignHashW is not available\n");
 
     result = pCryptSetKeyParam(hKey, 0, &temp, 1);
     ok (!result && GetLastError() == ERROR_INVALID_PARAMETER, "%d\n", GetLastError());
@@ -341,9 +342,14 @@ static void test_incorrect_api_usage(void)
     result = pCryptSetProvParam(hProv, 0, &temp, 1);
     ok (!result && GetLastError() == ERROR_INVALID_PARAMETER, "%d\n", GetLastError());
 
-    result = pCryptVerifySignatureW(hHash, &temp, 1, hKey, NULL, 0);
-    ok (!result && (GetLastError() == ERROR_INVALID_PARAMETER ||
-        GetLastError() == ERROR_CALL_NOT_IMPLEMENTED), "%d\n", GetLastError());
+    if (pCryptVerifySignatureW)
+    {
+        result = pCryptVerifySignatureW(hHash, &temp, 1, hKey, NULL, 0);
+        ok (!result && (GetLastError() == ERROR_INVALID_PARAMETER ||
+            GetLastError() == ERROR_CALL_NOT_IMPLEMENTED), "%d\n", GetLastError());
+    }
+    else
+        skip("CryptVerifySignatureW is not available\n");
 
     result = pCryptDestroyHash(hHash);
     ok (!result && GetLastError() == ERROR_INVALID_PARAMETER, "%d\n", GetLastError());
@@ -352,6 +358,96 @@ static void test_incorrect_api_usage(void)
     ok (!result && GetLastError() == ERROR_INVALID_PARAMETER, "%d\n", GetLastError());
 }
 
+static const BYTE privKey[] = {
+ 0x07, 0x02, 0x00, 0x00, 0x00, 0x24, 0x00, 0x00, 0x52, 0x53, 0x41, 0x32, 0x00,
+ 0x02, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x79, 0x10, 0x1c, 0xd0, 0x6b, 0x10,
+ 0x18, 0x30, 0x94, 0x61, 0xdc, 0x0e, 0xcb, 0x96, 0x4e, 0x21, 0x3f, 0x79, 0xcd,
+ 0xa9, 0x17, 0x62, 0xbc, 0xbb, 0x61, 0x4c, 0xe0, 0x75, 0x38, 0x6c, 0xf3, 0xde,
+ 0x60, 0x86, 0x03, 0x97, 0x65, 0xeb, 0x1e, 0x6b, 0xdb, 0x53, 0x85, 0xad, 0x68,
+ 0x21, 0xf1, 0x5d, 0xe7, 0x1f, 0xe6, 0x53, 0xb4, 0xbb, 0x59, 0x3e, 0x14, 0x27,
+ 0xb1, 0x83, 0xa7, 0x3a, 0x54, 0xe2, 0x8f, 0x65, 0x8e, 0x6a, 0x4a, 0xcf, 0x3b,
+ 0x1f, 0x65, 0xff, 0xfe, 0xf1, 0x31, 0x3a, 0x37, 0x7a, 0x8b, 0xcb, 0xc6, 0xd4,
+ 0x98, 0x50, 0x36, 0x67, 0xe4, 0xa1, 0xe8, 0x7e, 0x8a, 0xc5, 0x23, 0xf2, 0x77,
+ 0xf5, 0x37, 0x61, 0x49, 0x72, 0x59, 0xe8, 0x3d, 0xf7, 0x60, 0xb2, 0x77, 0xca,
+ 0x78, 0x54, 0x6d, 0x65, 0x9e, 0x03, 0x97, 0x1b, 0x61, 0xbd, 0x0c, 0xd8, 0x06,
+ 0x63, 0xe2, 0xc5, 0x48, 0xef, 0xb3, 0xe2, 0x6e, 0x98, 0x7d, 0xbd, 0x4e, 0x72,
+ 0x91, 0xdb, 0x31, 0x57, 0xe3, 0x65, 0x3a, 0x49, 0xca, 0xec, 0xd2, 0x02, 0x4e,
+ 0x22, 0x7e, 0x72, 0x8e, 0xf9, 0x79, 0x84, 0x82, 0xdf, 0x7b, 0x92, 0x2d, 0xaf,
+ 0xc9, 0xe4, 0x33, 0xef, 0x89, 0x5c, 0x66, 0x99, 0xd8, 0x80, 0x81, 0x47, 0x2b,
+ 0xb1, 0x66, 0x02, 0x84, 0x59, 0x7b, 0xc3, 0xbe, 0x98, 0x45, 0x4a, 0x3d, 0xdd,
+ 0xea, 0x2b, 0xdf, 0x4e, 0xb4, 0x24, 0x6b, 0xec, 0xe7, 0xd9, 0x0c, 0x45, 0xb8,
+ 0xbe, 0xca, 0x69, 0x37, 0x92, 0x4c, 0x38, 0x6b, 0x96, 0x6d, 0xcd, 0x86, 0x67,
+ 0x5c, 0xea, 0x54, 0x94, 0xa4, 0xca, 0xa4, 0x02, 0xa5, 0x21, 0x4d, 0xae, 0x40,
+ 0x8f, 0x9d, 0x51, 0x83, 0xf2, 0x3f, 0x33, 0xc1, 0x72, 0xb4, 0x1d, 0x94, 0x6e,
+ 0x7d, 0xe4, 0x27, 0x3f, 0xea, 0xff, 0xe5, 0x9b, 0xa7, 0x5e, 0x55, 0x8e, 0x0d,
+ 0x69, 0x1c, 0x7a, 0xff, 0x81, 0x9d, 0x53, 0x52, 0x97, 0x9a, 0x76, 0x79, 0xda,
+ 0x93, 0x32, 0x16, 0xec, 0x69, 0x51, 0x1a, 0x4e, 0xc3, 0xf1, 0x72, 0x80, 0x78,
+ 0x5e, 0x66, 0x4a, 0x8d, 0x85, 0x2f, 0x3f, 0xb2, 0xa7 };
+
+static void test_verify_sig(void)
+{
+	BOOL ret;
+	HCRYPTPROV prov;
+	HCRYPTKEY key;
+	HCRYPTHASH hash;
+	BYTE bogus[] = { 0 };
+
+	if (!pCryptVerifySignatureW)
+	{
+		skip("CryptVerifySignatureW is not available\n");
+		return;
+	}
+
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(0, NULL, 0, 0, NULL, 0);
+	if (!ret && GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
+	{
+		skip("CryptVerifySignatureW is not implemented\n");
+		return;
+	}
+	ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
+	 "Expected ERROR_INVALID_PARAMETER, got %08x\n", GetLastError());
+	ret = pCryptAcquireContextA(&prov, szKeySet, NULL, PROV_RSA_FULL,
+	 CRYPT_NEWKEYSET);
+	if (!ret && GetLastError() == NTE_EXISTS)
+		ret = pCryptAcquireContextA(&prov, szKeySet, NULL, PROV_RSA_FULL, 0);
+	ret = pCryptImportKey(prov, (LPBYTE)privKey, sizeof(privKey), 0, 0, &key);
+	ok(ret, "CryptImportKey failed: %08x\n", GetLastError());
+	ret = pCryptCreateHash(prov, CALG_MD5, 0, 0, &hash);
+	ok(ret, "CryptCreateHash failed: %08x\n", GetLastError());
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(hash, NULL, 0, 0, NULL, 0);
+	ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
+	 "Expected ERROR_INVALID_PARAMETER, got %08x\n", GetLastError());
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(0, NULL, 0, key, NULL, 0);
+	ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
+	 "Expected ERROR_INVALID_PARAMETER, got %08x\n", GetLastError());
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(hash, NULL, 0, key, NULL, 0);
+	ok(!ret && (GetLastError() == NTE_BAD_SIGNATURE ||
+	 GetLastError() == ERROR_INVALID_PARAMETER),
+	 "Expected NTE_BAD_SIGNATURE or ERROR_INVALID_PARAMETER, got %08x\n",
+	 GetLastError());
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(hash, NULL, sizeof(bogus), key, NULL, 0);
+	ok(!ret && (GetLastError() == NTE_BAD_SIGNATURE ||
+	 GetLastError() == ERROR_INVALID_PARAMETER),
+	 "Expected NTE_BAD_SIGNATURE or ERROR_INVALID_PARAMETER, got %08x\n",
+	 GetLastError());
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(hash, bogus, 0, key, NULL, 0);
+	ok(!ret && GetLastError() == NTE_BAD_SIGNATURE,
+	 "Expected NTE_BAD_SIGNATURE, got %08x\n", GetLastError());
+	SetLastError(0xdeadbeef);
+	ret = pCryptVerifySignatureW(hash, bogus, sizeof(bogus), key, NULL, 0);
+	ok(!ret && GetLastError() == NTE_BAD_SIGNATURE,
+	 "Expected NTE_BAD_SIGNATURE, got %08x\n", GetLastError());
+	pCryptDestroyKey(key);
+	pCryptDestroyHash(hash);
+	pCryptReleaseContext(prov, 0);
+}
+
 static BOOL FindProvRegVals(DWORD dwIndex, DWORD *pdwProvType, LPSTR *pszProvName, 
 			    DWORD *pcbProvName, DWORD *pdwProvCount)
 {
@@ -401,13 +497,13 @@ static void test_enum_providers(void)
 	
 	if(!pCryptEnumProvidersA)
 	{
-	    trace("skipping CryptEnumProviders tests\n");
+	    skip("CryptEnumProvidersA is not available\n");
 	    return;
 	}
 	
 	if (!FindProvRegVals(dwIndex, &dwType, &pszProvName, &cbName, &provCount))
 	{
-	    trace("could not find providers in registry, skipping the test\n");
+	    skip("Could not find providers in registry\n");
 	    return;
 	}
 	
@@ -529,13 +625,13 @@ static void test_enum_provider_types(void)
 	
 	if(!pCryptEnumProviderTypesA)
 	{
-	    trace("skipping CryptEnumProviderTypes tests\n");
+	    skip("CryptEnumProviderTypesA is not available\n");
 	    return;
 	}
 	
 	if (!FindProvTypesRegVals(index, &dwProvType, &pszTypeName, &cbTypeName, &dwTypeCount))
 	{
-	    trace("could not find provider types in registry, skipping the test\n");
+	    skip("Could not find provider types in registry\n");
 	    return;
 	}
 	
@@ -667,13 +763,13 @@ static void test_get_default_provider(void)
 	
 	if(!pCryptGetDefaultProviderA)
 	{
-	    trace("skipping CryptGetDefaultProvider tests\n");
+	    skip("CryptGetDefaultProviderA is not available\n");
 	    return;
 	}
 	
 	if(!FindDfltProvRegVals(dwProvType, dwFlags, &pszProvName, &cbProvName))
 	{
-	    trace("could not find default provider in registry, skipping the test\n");
+	    skip("Could not find default provider in registry\n");
 	    return;
 	}
 	
@@ -741,7 +837,7 @@ static void test_set_provider_ex(void)
 	
 	if(!pCryptGetDefaultProviderA || !pCryptSetProviderExA)
 	{
-	    trace("skipping CryptSetProviderEx tests\n");
+	    skip("CryptGetDefaultProviderA and/or CryptSetProviderExA are not available\n");
 	    return;
 	}
 
@@ -751,7 +847,13 @@ static void test_set_provider_ex(void)
 		ERROR_INVALID_PARAMETER, GetLastError());
 
 	/* remove the default provider and then set it to MS_DEF_PROV/PROV_RSA_FULL */
+        SetLastError(0xdeadbeef);
 	result = pCryptSetProviderExA(MS_DEF_PROV, PROV_RSA_FULL, NULL, CRYPT_MACHINE_DEFAULT | CRYPT_DELETE_DEFAULT);
+	if (!result && (GetLastError() == ERROR_ACCESS_DENIED))
+	{
+		skip("Not enough rights to remove the default provider\n");
+		return;
+	}
 	ok(result, "%d\n", GetLastError());
 
 	result = pCryptSetProviderExA(MS_DEF_PROV, PROV_RSA_FULL, NULL, CRYPT_MACHINE_DEFAULT);
@@ -769,6 +871,51 @@ static void test_set_provider_ex(void)
 	LocalFree(pszProvName);
 }
 
+static void test_machine_guid(void)
+{
+   char originalGuid[40], guid[40];
+   LONG r;
+   HKEY key;
+   DWORD size;
+   HCRYPTPROV hCryptProv;
+   BOOL restoreGuid = FALSE, ret;
+
+   r = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Cryptography",
+                     0, KEY_ALL_ACCESS, &key);
+   if (r != ERROR_SUCCESS)
+   {
+       skip("couldn't open HKLM\\Software\\Microsoft\\Cryptography\n");
+       return;
+   }
+   /* Cache existing MachineGuid, and delete it */
+   size = sizeof(originalGuid);
+   r = RegQueryValueExA(key, "MachineGuid", NULL, NULL, (BYTE *)originalGuid,
+                        &size);
+   if (r == ERROR_SUCCESS)
+   {
+       restoreGuid = TRUE;
+       r = RegDeleteValueA(key, "MachineGuid");
+       ok(!r, "RegDeleteValueA failed: %d\n", r);
+   }
+   else
+       ok(r == ERROR_FILE_NOT_FOUND, "expected ERROR_FILE_NOT_FOUND, got %d\n",
+          r);
+   /* Create and release a provider */
+   ret = pCryptAcquireContextA(&hCryptProv, szKeySet, NULL, PROV_RSA_FULL, 0);
+   ok(ret, "CryptAcquireContextA failed: %08x\n", GetLastError());
+   CryptReleaseContext(hCryptProv, 0);
+   /* Check that MachineGuid was created */
+   size = sizeof(guid);
+   r = RegQueryValueExA(key, "MachineGuid", NULL, NULL, (BYTE *)guid, &size);
+   ok(!r, "expected to find MachineGuid: %d\n", r);
+   r = RegDeleteValueA(key, "MachineGuid");
+   ok(!r, "RegDeleteValueA failed: %d\n", r);
+   if (restoreGuid)
+       RegSetValueExA(key, "MachineGuid", 0, REG_SZ, (const BYTE *)originalGuid,
+                      strlen(originalGuid)+1);
+   RegCloseKey(key);
+}
+
 START_TEST(crypt)
 {
 	init_function_pointers();
@@ -776,6 +923,8 @@ START_TEST(crypt)
 	init_environment();
 	test_acquire_context();
 	test_incorrect_api_usage();
+	test_verify_sig();
+	test_machine_guid();
 	clean_up_environment();
 	}
 	
