@@ -20,6 +20,7 @@
  */
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <windows.h>
 #include <advpub.h>
 #include <assert.h>
@@ -34,6 +35,7 @@
 #define REG_VAL_EXISTS(key, value)   !RegQueryValueEx(key, value, NULL, NULL, NULL, NULL)
 #define OPEN_GUID_KEY() !RegOpenKey(HKEY_LOCAL_MACHINE, GUID_KEY, &guid)
 
+static HMODULE hAdvPack;
 static HRESULT (WINAPI *pCloseINFEngine)(HINF);
 static HRESULT (WINAPI *pDelNode)(LPCSTR,DWORD);
 static HRESULT (WINAPI *pGetVersionFromFile)(LPCSTR,LPDWORD,LPDWORD,BOOL);
@@ -45,7 +47,8 @@ static HRESULT (WINAPI *pTranslateInfStringEx)(HINF,PCSTR,PCSTR,PCSTR,PSTR,DWORD
 static CHAR inf_file[MAX_PATH];
 static CHAR PROG_FILES_ROOT[MAX_PATH];
 static CHAR PROG_FILES[MAX_PATH];
-static DWORD PROG_FILES_LEN;
+static CHAR APP_PATH[MAX_PATH];
+static DWORD APP_PATH_LEN;
 
 static void get_progfiles_dir(void)
 {
@@ -56,14 +59,15 @@ static void get_progfiles_dir(void)
     RegQueryValueExA(hkey, "ProgramFilesDir", NULL, NULL, (LPBYTE)PROG_FILES_ROOT, &size);
     RegCloseKey(hkey);
 
-    lstrcpyA(PROG_FILES, PROG_FILES_ROOT);
-    lstrcatA(PROG_FILES, TEST_STRING1);
-    PROG_FILES_LEN = lstrlenA(PROG_FILES) + 1;
+    lstrcpyA(PROG_FILES, PROG_FILES_ROOT + 3); /* skip C:\ */
+    lstrcpyA(APP_PATH, PROG_FILES_ROOT);
+    lstrcatA(APP_PATH, TEST_STRING1);
+    APP_PATH_LEN = lstrlenA(APP_PATH) + 1;
 }
 
 static BOOL init_function_pointers(void)
 {
-    HMODULE hAdvPack = LoadLibraryA("advpack.dll");
+    hAdvPack = LoadLibraryA("advpack.dll");
 
     if (!hAdvPack)
         return FALSE;
@@ -182,10 +186,14 @@ static void delnode_test(void)
     currDir[currDirLen] = '\0';
 }
 
-static void append_str(char **str, const char *data)
+static void append_str(char **str, const char *data, ...)
 {
-    sprintf(*str, data);
+    va_list valist;
+
+    va_start(valist, data);
+    vsprintf(*str, data, valist);
     *str += strlen(*str);
+    va_end(valist);
 }
 
 static void create_inf_file(void)
@@ -218,7 +226,7 @@ static void create_inf_file(void)
     append_str(&ptr, "CustomHDestination=CustInstDestSection\n");
     append_str(&ptr, "[Strings]\n");
     append_str(&ptr, "DefaultAppPath=\"Application Name\"\n");
-    append_str(&ptr, "LProgramF=\"Program Files\"\n");
+    append_str(&ptr, "LProgramF=\"%s\"\n", PROG_FILES);
     append_str(&ptr, "[DestA]\n");
     append_str(&ptr, "HKLM,\"Software\\Garbage\",\"ProgramFilesDir\",,'%%24%%\\%%LProgramF%%'\n");
     append_str(&ptr, "[DestB]\n");
@@ -287,8 +295,8 @@ static void translateinfstring_test(void)
 
     if(hr == ERROR_SUCCESS)
     {
-        ok(!strcmp(buffer, PROG_FILES), "Expected '%s', got '%s'\n", PROG_FILES, buffer);
-        ok(dwSize == PROG_FILES_LEN, "Expected size %d, got %d\n", PROG_FILES_LEN, dwSize);
+        ok(!strcmp(buffer, APP_PATH), "Expected '%s', got '%s'\n", APP_PATH, buffer);
+        ok(dwSize == APP_PATH_LEN, "Expected size %d, got %d\n", APP_PATH_LEN, dwSize);
     }
 
     buffer[0] = 0;
@@ -403,18 +411,18 @@ static void translateinfstringex_test(void)
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* translate the string with the install section specified */
-    memset(buffer, 'a', PROG_FILES_LEN);
-    buffer[PROG_FILES_LEN - 1] = '\0';
+    memset(buffer, 'a', APP_PATH_LEN);
+    buffer[APP_PATH_LEN - 1] = '\0';
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "InstallDir",
                               buffer, size, &size, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
-    ok(!strcmp(buffer, PROG_FILES), "Expected %s, got %s\n", PROG_FILES, buffer);
-    ok(size == PROG_FILES_LEN, "Expected size %d, got %d\n", PROG_FILES_LEN, size);
+    ok(!strcmp(buffer, APP_PATH), "Expected %s, got %s\n", APP_PATH, buffer);
+    ok(size == APP_PATH_LEN, "Expected size %d, got %d\n", APP_PATH_LEN, size);
 
     /* Single quote test (Note size includes null on return from call) */
-    memset(buffer, 'a', PROG_FILES_LEN);
-    buffer[PROG_FILES_LEN - 1] = '\0';
+    memset(buffer, 'a', APP_PATH_LEN);
+    buffer[APP_PATH_LEN - 1] = '\0';
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result1",
                               buffer, size, &size, NULL);
@@ -424,8 +432,8 @@ static void translateinfstringex_test(void)
     ok(size == lstrlenA(PROG_FILES_ROOT)+1, "Expected size %d, got %d\n",
            lstrlenA(PROG_FILES_ROOT)+1, size);
 
-    memset(buffer, 'a', PROG_FILES_LEN);
-    buffer[PROG_FILES_LEN - 1] = '\0';
+    memset(buffer, 'a', APP_PATH_LEN);
+    buffer[APP_PATH_LEN - 1] = '\0';
     size = MAX_PATH;
     hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result2",
                               buffer, size, &size, NULL);
@@ -440,8 +448,8 @@ static void translateinfstringex_test(void)
         lstrcpy(drive, PROG_FILES_ROOT);
         drive[3] = 0x00; /* Just keep the system drive plus '\' */
 
-        memset(buffer, 'a', PROG_FILES_LEN);
-        buffer[PROG_FILES_LEN - 1] = '\0';
+        memset(buffer, 'a', APP_PATH_LEN);
+        buffer[APP_PATH_LEN - 1] = '\0';
         size = MAX_PATH;
         hr = pTranslateInfStringEx(hinf, inf_file, "Options.NTx86", "Result3",
                                   buffer, size, &size, NULL);
@@ -514,7 +522,7 @@ static void setperusersecvalues_test(void)
     /* set initial values */
     lstrcpy(peruser.szGUID, "guid");
     hr = pSetPerUserSecValues(&peruser);
-    ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
     ok(OPEN_GUID_KEY(), "Expected guid key to exist\n");
     ok(check_reg_str(guid, NULL, "displayname"), "Expected displayname\n");
     ok(check_reg_str(guid, "ComponentID", "compid"), "Expected compid\n");
@@ -531,7 +539,7 @@ static void setperusersecvalues_test(void)
     /* raise the version, but bRollback is FALSE, so vals not saved */
     lstrcpy(peruser.szVersion, "2,1,1,1");
     hr = pSetPerUserSecValues(&peruser);
-    ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
     ok(check_reg_str(guid, NULL, "displayname"), "Expected displayname\n");
     ok(check_reg_str(guid, "ComponentID", "compid"), "Expected compid\n");
     ok(check_reg_str(guid, "Locale", "locale"), "Expected locale\n");
@@ -548,7 +556,7 @@ static void setperusersecvalues_test(void)
     peruser.bRollback = TRUE;
     lstrcpy(peruser.szVersion, "3,1,1,1");
     hr = pSetPerUserSecValues(&peruser);
-    ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
+    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
     ok(check_reg_str(guid, NULL, "displayname"), "Expected displayname\n");
     ok(check_reg_str(guid, "ComponentID", "compid"), "Expected compid\n");
     ok(check_reg_str(guid, "Locale", "locale"), "Expected locale\n");
@@ -587,4 +595,6 @@ START_TEST(advpack)
     setperusersecvalues_test();
     translateinfstring_test();
     translateinfstringex_test();
+
+    FreeLibrary(hAdvPack);
 }
