@@ -112,17 +112,23 @@ static BOOL check_ini_contents(LPSTR filename, BOOL add)
 
     GetPrivateProfileStringA("backup", "one", NULL, field, FIELD_LEN, filename);
     match = !lstrcmpA(field, "-1,0,0,0,0,0,-1");
-    if ((add && !match) || (!add && match))
+    if ((add && !match) || (!add && match)) {
+        trace("first test: got %s\n", field);
         ret = FALSE;
+    }
 
     GetPrivateProfileStringA("backup", "two", NULL, field, FIELD_LEN, filename);
-    if (lstrcmpA(field, "-1,0,0,0,0,0,-1"))
+    if (lstrcmpA(field, "-1,0,0,0,0,0,-1")) {
+        trace("second test: got %s\n", field);
         ret = FALSE;
+    }
 
     GetPrivateProfileStringA("backup", "three", NULL, field, FIELD_LEN, filename);
     match = !lstrcmpA(field, "-1,0,0,0,0,0,-1");
-    if ((add && !match) || (!add && match))
+    if ((add && !match) || (!add && match)) {
+        trace("third test: got %s\n", field);
         ret = FALSE;
+    }
 
     return ret;
 }
@@ -131,6 +137,7 @@ static void test_AddDelBackupEntry(void)
 {
     HRESULT res;
     CHAR path[MAX_PATH];
+    CHAR windir[MAX_PATH];
 
     lstrcpyA(path, CURR_DIR);
     lstrcatA(path, "\\backup\\basename.INI");
@@ -187,9 +194,10 @@ static void test_AddDelBackupEntry(void)
     ok(DeleteFileA(path), "Expected path to exist\n");
     RemoveDirectoryA("backup");
 
-    lstrcpyA(path, "c:\\windows\\basename.INI");
+    GetWindowsDirectoryA(windir, sizeof(windir));
+    sprintf(path, "%s\\basename.INI", windir);
 
-    /* try a NULL backup dir, INI is created in c:\windows */
+    /* try a NULL backup dir, INI is created in the windows directory */
     res = pAddDelBackupEntry("one\0two\0three\0", NULL, "basename", AADBE_ADD_ENTRY);
     ok(res == S_OK, "Expected S_OK, got %d\n", res);
     ok(check_ini_contents(path, TRUE), "Expected ini contents to match\n");
@@ -552,4 +560,6 @@ START_TEST(files)
     test_AdvInstallFile();
 
     delete_test_files();
+
+    FreeLibrary(hAdvPack);
 }
