@@ -111,6 +111,10 @@ static const struct message test_dtm_set_range_swap_min_max_seq[] = {
 };
 
 static const struct message test_dtm_set_and_get_system_time_seq[] = {
+    { DTM_SETSYSTEMTIME, sent|wparam, 0x00000001 },
+    { 0x0090, sent|optional }, /* Vista */
+    { WM_DESTROY, sent|wparam|lparam, 0x00000000, 0x00000000 },
+    { WM_NCDESTROY, sent|wparam|lparam, 0x00000000, 0x00000000 },
     { DTM_SETSYSTEMTIME, sent|wparam, 0x00000001 },
     { DTM_GETSYSTEMTIME, sent|wparam, 0x00000000 },
     { DTM_SETSYSTEMTIME, sent|wparam, 0x00000000 },
@@ -122,6 +126,7 @@ static const struct message test_dtm_set_and_get_system_time_seq[] = {
 };
 
 static const struct message destroy_window_seq[] = {
+    { 0x0090, sent|optional }, /* Vista */
     { WM_DESTROY, sent|wparam|lparam, 0x00000000, 0x00000000 },
     { WM_NCDESTROY, sent|wparam|lparam, 0x00000000, 0x00000000 },
     { 0 }
@@ -134,12 +139,12 @@ struct subclass_info
 
 static LRESULT WINAPI datetime_subclass_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
-    struct subclass_info *info = (struct subclass_info *)GetWindowLongA(hwnd, GWL_USERDATA);
+    struct subclass_info *info = (struct subclass_info *)GetWindowLongPtrA(hwnd, GWLP_USERDATA);
     static long defwndproc_counter = 0;
     LRESULT ret;
     struct message msg;
 
-    trace("datetime: %p, %04x, %08x, %08lx\n", hwnd, message, wParam, lParam);
+    trace("datetime: %p, %04x, %08lx, %08lx\n", hwnd, message, wParam, lParam);
 
     msg.message = message;
     msg.flags = sent|wparam|lparam;
@@ -179,15 +184,17 @@ static HWND create_datetime_control(DWORD style, DWORD exstyle)
         return NULL;
     }
 
-    info->oldproc = (WNDPROC)SetWindowLongA(hWndDateTime, GWL_WNDPROC,
-                                            (LONG)datetime_subclass_proc);
-    SetWindowLongA(hWndDateTime, GWL_USERDATA, (LONG)info);
+    info->oldproc = (WNDPROC)SetWindowLongPtrA(hWndDateTime, GWLP_WNDPROC,
+                                            (LONG_PTR)datetime_subclass_proc);
+    SetWindowLongPtrA(hWndDateTime, GWLP_USERDATA, (LONG_PTR)info);
 
     return hWndDateTime;
 }
 
 static void test_dtm_set_format(HWND hWndDateTime)
 {
+    CHAR txt[256];
+    SYSTEMTIME systime;
     LRESULT r;
 
     r = SendMessage(hWndDateTime, DTM_SETFORMAT, 0, (LPARAM)NULL);
@@ -198,6 +205,17 @@ static void test_dtm_set_format(HWND hWndDateTime)
     expect(1, r);
 
     ok_sequence(sequences, DATETIME_SEQ_INDEX, test_dtm_set_format_seq, "test_dtm_set_format", FALSE);
+
+    r = SendMessage(hWndDateTime, DTM_SETFORMAT, 0,
+		    (LPARAM)"'hh' hh");
+    expect(1, r);
+    ZeroMemory(&systime, sizeof(systime));
+    systime.wYear = 2000;
+    systime.wMonth = systime.wDay = 1;
+    r = SendMessage(hWndDateTime, DTM_SETSYSTEMTIME, 0, (LPARAM)&systime);
+    expect(1, r);
+    GetWindowText(hWndDateTime, txt, 256);
+    todo_wine ok(strcmp(txt, "hh 12") == 0, "String mismatch (\"%s\" vs \"hh 12\")\n", txt);
     flush_sequences(sequences, NUM_MSG_SEQUENCES);
 }
 
@@ -308,7 +326,7 @@ static void test_dtm_set_and_get_range(HWND hWndDateTime)
 
     /* initialize st[0] to lowest possible value */
     fill_systime_struct(&st[0], 1601, 1, 0, 1, 0, 0, 0, 0);
-    /* intialize st[1] to all invalid numbers */
+    /* initialize st[1] to all invalid numbers */
     fill_systime_struct(&st[1], 0, 0, 7, 0, 24, 60, 60, 1000);
 
     r = SendMessage(hWndDateTime, DTM_SETRANGE, GDTR_MIN, (LPARAM)st);
@@ -477,6 +495,23 @@ static void test_dtm_set_and_get_system_time(HWND hWndDateTime)
     LRESULT r;
     SYSTEMTIME st;
     SYSTEMTIME getSt;
+    HWND hWndDateTime_test_gdt_none;
+
+    hWndDateTime_test_gdt_none = create_datetime_control(0, 0);
+
+    ok(hWndDateTime_test_gdt_none!=NULL, "Expected non NULL, got %p\n", hWndDateTime_test_gdt_none);
+    if(hWndDateTime_test_gdt_none) {
+        r = SendMessage(hWndDateTime_test_gdt_none, DTM_SETSYSTEMTIME, GDT_NONE, (LPARAM)&st);
+        expect(0, r);
+    }
+    else {
+        skip("hWndDateTime_test_gdt_none is NULL\n");
+        flush_sequences(sequences, NUM_MSG_SEQUENCES);
+
+        return;
+    }
+
+    DestroyWindow(hWndDateTime_test_gdt_none);
 
     r = SendMessage(hWndDateTime, DTM_SETSYSTEMTIME, GDT_NONE, (LPARAM)&st);
     expect(1, r);
@@ -540,11 +575,21 @@ static void test_datetime_control(void)
 
 START_TEST(datetime)
 {
-    INITCOMMONCONTROLSEX icex;
+    HMODULE hComctl32;
+    BOOL (WINAPI *pInitCommonControlsEx)(const INITCOMMONCONTROLSEX*);
+    INITCOMMONCONTROLSEX iccex;
+
+    hComctl32 = GetModuleHandleA("comctl32.dll");
+    pInitCommonControlsEx = (void*)GetProcAddress(hComctl32, "InitCommonControlsEx");
+    if (!pInitCommonControlsEx)
+    {
+        skip("InitCommonControlsEx() is missing. Skipping the tests\n");
+        return;
+    }
+    iccex.dwSize = sizeof(iccex);
+    iccex.dwICC  = ICC_DATE_CLASSES;
+    pInitCommonControlsEx(&iccex);
 
-    icex.dwSize = sizeof(icex);
-    icex.dwICC = ICC_DATE_CLASSES;
-    InitCommonControlsEx(&icex);
     init_msg_sequences(sequences, NUM_MSG_SEQUENCES);
 
     test_datetime_control();
