@@ -112,60 +112,9 @@ BOOL FD31_CallWindowProc(const FD31_DATA *lfs, UINT wMsg, WPARAM wParam,
     return lfs->callbacks->CWP(lfs, wMsg, wParam, lParam);
 }
 
-/***********************************************************************
- * 				FD31_ScanDir                 [internal]
- */
-static BOOL FD31_ScanDir(HWND hWnd, LPCWSTR newPath)
-{
-    WCHAR		buffer[BUFFILE];
-    HWND 		hdlg, hdlgDir;
-    LRESULT             lRet = TRUE;
-    HCURSOR             hCursorWait, oldCursor;
-
-    TRACE("Trying to change to %s\n", debugstr_w(newPath));
-    if  ( newPath[0] && !SetCurrentDirectoryW( newPath ))
-        return FALSE;
-    lstrcpynW(buffer, newPath, sizeof(buffer)/sizeof(WCHAR));
-
-    /* get the list of spec files */
-    GetDlgItemTextW(hWnd, edt1, buffer, sizeof(buffer)/sizeof(WCHAR));
-
-    hCursorWait = LoadCursorA(0, (LPSTR)IDC_WAIT);
-    oldCursor = SetCursor(hCursorWait);
-
-    /* list of files */
-    if ((hdlg = GetDlgItem(hWnd, lst1)) != 0) {
-        WCHAR*	scptr; /* ptr on semi-colon */
-	WCHAR*	filter = buffer;
-
-	TRACE("Using filter %s\n", debugstr_w(filter));
-	SendMessageW(hdlg, LB_RESETCONTENT, 0, 0);
-	while (filter) {
-	    scptr = strchrW(filter, ';');
-	    if (scptr)	*scptr = 0;
-            while (*filter == ' ') filter++;
-	    TRACE("Using file spec %s\n", debugstr_w(filter));
-	    if (SendMessageW(hdlg, LB_DIR, 0, (LPARAM)filter) == LB_ERR)
-	        return FALSE;
-	    if (scptr) *scptr = ';';
-	        filter = (scptr) ? (scptr + 1) : 0;
-	 }
-    }
-
-    /* list of directories */
-    strcpyW(buffer, FILE_star);
-
-    if ((hdlgDir = GetDlgItem(hWnd, lst2)) != 0) {
-        lRet = DlgDirListW(hWnd, buffer, lst2, stc1, DDL_EXCLUSIVE | DDL_DIRECTORY);
-    }
-    SetCursor(oldCursor);
-    return lRet;
-}
-
 /***********************************************************************
  * 				FD31_GetFileType		[internal]
  */
-
 static LPCWSTR FD31_GetFileType(LPCWSTR cfptr, LPCWSTR fptr, const WORD index)
 {
   int n, i;
@@ -189,6 +138,55 @@ static LPCWSTR FD31_GetFileType(LPCWSTR cfptr, LPCWSTR fptr, const WORD index)
   return FILE_star; /* FIXME */
 }
 
+/***********************************************************************
+ * 				FD31_ScanDir                 [internal]
+ */
+static BOOL FD31_ScanDir(const OPENFILENAMEW *ofn, HWND hWnd, LPCWSTR newPath)
+{
+    WCHAR		buffer[BUFFILE];
+    HWND 		hdlg, hdlgDir;
+    LRESULT             lRet = TRUE;
+    HCURSOR             hCursorWait, oldCursor;
+
+    TRACE("Trying to change to %s\n", debugstr_w(newPath));
+    if  ( newPath[0] && !SetCurrentDirectoryW( newPath ))
+        return FALSE;
+
+    /* get the list of spec files */
+    lstrcpynW(buffer, FD31_GetFileType(ofn->lpstrCustomFilter,
+              ofn->lpstrFilter, ofn->nFilterIndex - 1), BUFFILE);
+
+    hCursorWait = LoadCursorA(0, (LPSTR)IDC_WAIT);
+    oldCursor = SetCursor(hCursorWait);
+
+    /* list of files */
+    if ((hdlg = GetDlgItem(hWnd, lst1)) != 0) {
+        WCHAR*	scptr; /* ptr on semi-colon */
+	WCHAR*	filter = buffer;
+
+	TRACE("Using filter %s\n", debugstr_w(filter));
+	SendMessageW(hdlg, LB_RESETCONTENT, 0, 0);
+	while (filter) {
+	    scptr = strchrW(filter, ';');
+	    if (scptr)	*scptr = 0;
+            while (*filter == ' ') filter++;
+	    TRACE("Using file spec %s\n", debugstr_w(filter));
+	    SendMessageW(hdlg, LB_DIR, 0, (LPARAM)filter);
+	    if (scptr) *scptr = ';';
+	        filter = (scptr) ? (scptr + 1) : 0;
+	 }
+    }
+
+    /* list of directories */
+    strcpyW(buffer, FILE_star);
+
+    if ((hdlgDir = GetDlgItem(hWnd, lst2)) != 0) {
+        lRet = DlgDirListW(hWnd, buffer, lst2, stc1, DDL_EXCLUSIVE | DDL_DIRECTORY);
+    }
+    SetCursor(oldCursor);
+    return lRet;
+}
+
 /***********************************************************************
  *                              FD31_WMDrawItem              [internal]
  */
@@ -375,7 +373,7 @@ static LRESULT FD31_DirListDblClick( const FD31_DATA *lfs )
     }
   strcatW(tmpstr, FILE_bslash);
 
-  FD31_ScanDir(hWnd, tmpstr);
+  FD31_ScanDir(lfs->ofnW, hWnd, tmpstr);
   /* notify the app */
   if (lfs->hook)
     {
@@ -450,7 +448,7 @@ static LRESULT FD31_TestPath( const FD31_DATA *lfs, LPWSTR path )
 
         TRACE("path=%s, tmpstr2=%s\n", debugstr_w(path), debugstr_w(tmpstr2));
         SetDlgItemTextW( hWnd, edt1, tmpstr2 );
-        FD31_ScanDir(hWnd, path);
+        FD31_ScanDir(lfs->ofnW, hWnd, path);
         return (lfs->ofnW->Flags & OFN_NOVALIDATE) ? TRUE : FALSE;
     }
 
@@ -462,7 +460,7 @@ static LRESULT FD31_TestPath( const FD31_DATA *lfs, LPWSTR path )
         strcatW(path, FILE_bslash);
 
     /* if ScanDir succeeds, we have changed the directory */
-    if (FD31_ScanDir(hWnd, path))
+    if (FD31_ScanDir(lfs->ofnW, hWnd, path))
         return FALSE; /* and path is not a valid file name */
 
     /* if not, this must be a filename */
@@ -477,7 +475,7 @@ static LRESULT FD31_TestPath( const FD31_DATA *lfs, LPWSTR path )
 
         lstrcpynW(tmpstr2, pBeginFileName + 1, sizeof(tmpstr2)/sizeof(WCHAR) );
         /* Should we MessageBox() if this fails? */
-        if (!FD31_ScanDir(hWnd, path))
+        if (!FD31_ScanDir(lfs->ofnW, hWnd, path))
         {
             return FALSE;
         }
@@ -531,7 +529,7 @@ static LRESULT FD31_Validate( const FD31_DATA *lfs, LPCWSTR path, UINT control,
     FD31_UpdateFileTitle(lfs);
     if (lfs->hook)
     {
-        lRet = (BOOL)FD31_CallWindowProc(lfs, lfs->fileokstring,
+        lRet = FD31_CallWindowProc(lfs, lfs->fileokstring,
                   0, lfs->lParam );
         if (lRet)
         {
@@ -543,7 +541,7 @@ static LRESULT FD31_Validate( const FD31_DATA *lfs, LPCWSTR path, UINT control,
     {
         if (ofnW->lpstrFile)
         {
-            LPWSTR str = (LPWSTR)ofnW->lpstrFile;
+            LPWSTR str = ofnW->lpstrFile;
             LPWSTR ptr = strrchrW(str, '\\');
 	    str[lstrlenW(str) + 1] = '\0';
 	    *ptr = 0;
@@ -590,7 +588,6 @@ static LRESULT FD31_FileTypeChange( const FD31_DATA *lfs )
         return TRUE;
     pstr = (LPWSTR)SendDlgItemMessageW(lfs->hwnd, cmb1, CB_GETITEMDATA, lRet, 0);
     TRACE("Selected filter : %s\n", debugstr_w(pstr));
-    SetDlgItemTextW( lfs->hwnd, edt1, pstr );
 
     return FD31_Validate( lfs, NULL, cmb1, lRet, TRUE );
 }
@@ -783,7 +780,7 @@ void FD31_DestroyPrivate(PFD31_DATA lfs)
 /************************************************************************
  *                              FD31_AllocPrivate            [internal]
  *      allocate a private object to hold 32 bits Unicode
- *      structure that will be used throughtout the calls, while
+ *      structure that will be used throughout the calls, while
  *      keeping available the original structures and a few variables
  *      On entry : type = dialog procedure type (16,32A,32W)
  *                 dlgType = dialog type (open or save)
@@ -871,11 +868,19 @@ LONG FD31_WMInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
   if (ofn->nFilterIndex == 0 && ofn->lpstrCustomFilter == NULL)
   	ofn->nFilterIndex = 1;
   SendDlgItemMessageW(hWnd, cmb1, CB_SETCURSEL, ofn->nFilterIndex - 1, 0);
-  lstrcpynW(tmpstr, FD31_GetFileType(ofn->lpstrCustomFilter,
-	     (LPWSTR)ofn->lpstrFilter, ofn->nFilterIndex - 1),BUFFILE);
-  TRACE("nFilterIndex = %d, SetText of edt1 to %s\n",
+  if (ofn->lpstrFile && ofn->lpstrFile[0])
+  {
+    TRACE( "SetText of edt1 to %s\n", debugstr_w(ofn->lpstrFile) );
+    SetDlgItemTextW( hWnd, edt1, ofn->lpstrFile );
+  }
+  else
+  {
+    lstrcpynW(tmpstr, FD31_GetFileType(ofn->lpstrCustomFilter,
+	     ofn->lpstrFilter, ofn->nFilterIndex - 1),BUFFILE);
+    TRACE("nFilterIndex = %d, SetText of edt1 to %s\n",
   			ofn->nFilterIndex, debugstr_w(tmpstr));
-  SetDlgItemTextW( hWnd, edt1, tmpstr );
+    SetDlgItemTextW( hWnd, edt1, tmpstr );
+  }
   /* get drive list */
   *tmpstr = 0;
   DlgDirListComboBoxW(hWnd, tmpstr, cmb2, 0, DDL_DRIVES | DDL_EXCLUSIVE);
@@ -896,9 +901,9 @@ LONG FD31_WMInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
     }
   else
     *tmpstr = 0;
-  if (!FD31_ScanDir(hWnd, tmpstr)) {
+  if (!FD31_ScanDir(ofn, hWnd, tmpstr)) {
     *tmpstr = 0;
-    if (!FD31_ScanDir(hWnd, tmpstr))
+    if (!FD31_ScanDir(ofn, hWnd, tmpstr))
       WARN("Couldn't read initial directory %s!\n", debugstr_w(tmpstr));
   }
   /* select current drive in combo 2, omit missing drives */
@@ -919,7 +924,7 @@ LONG FD31_WMInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
   if (ofn->Flags & OFN_HIDEREADONLY)
     ShowWindow(GetDlgItem(hWnd, chx1), SW_HIDE);
   if (lfs->hook)
-      return (BOOL) FD31_CallWindowProc(lfs, WM_INITDIALOG, wParam, lfs->lParam);
+      return FD31_CallWindowProc(lfs, WM_INITDIALOG, wParam, lfs->lParam);
   return TRUE;
 }
 
