@@ -19,5 +19,15 @@
 #define IDD_CREDDIALOG      100
 #define IDC_USERNAME        101
 #define IDC_PASSWORD        102
-#define IDC_DOMAIN          103
+#define IDC_MESSAGE         103
 #define IDC_STATIC          104
+#define IDC_SAVE            105
+
+#define IDB_BANNER          200
+
+#define IDS_TITLEFORMAT     300
+#define IDS_MESSAGEFORMAT   301
+#define IDS_INCORRECTPASSWORDTITLE  302
+#define IDS_INCORRECTPASSWORD       303
+#define IDS_CAPSLOCKON      304
+#define IDS_CAPSLOCKONTITLE 305
