@@ -42,18 +42,17 @@ static HRESULT WINAPI IDirect3DSurface8Impl_QueryInterface(LPDIRECT3DSURFACE8 if
 
 static ULONG WINAPI IDirect3DSurface8Impl_AddRef(LPDIRECT3DSURFACE8 iface) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
-    IUnknown *containerParent = NULL;
 
     TRACE("(%p)\n", This);
 
-    IWineD3DSurface_GetContainerParent(This->wineD3DSurface, &containerParent);
-    if (containerParent) {
-        /* Forward to the containerParent */
-        TRACE("(%p) : Forwarding to %p\n", This, containerParent);
-        return IUnknown_AddRef(containerParent);
+    if (This->forwardReference) {
+        /* Forward refcounting */
+        TRACE("(%p) : Forwarding to %p\n", This, This->forwardReference);
+        return IUnknown_AddRef(This->forwardReference);
     } else {
         /* No container, handle our own refcounting */
         ULONG ref = InterlockedIncrement(&This->ref);
+        if(ref == 1 && This->parentDevice) IUnknown_AddRef(This->parentDevice);
         TRACE("(%p) : AddRef from %d\n", This, ref - 1);
         return ref;
     }
@@ -61,24 +60,27 @@ static ULONG WINAPI IDirect3DSurface8Impl_AddRef(LPDIRECT3DSURFACE8 iface) {
 
 static ULONG WINAPI IDirect3DSurface8Impl_Release(LPDIRECT3DSURFACE8 iface) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
-    IUnknown *containerParent = NULL;
 
     TRACE("(%p)\n", This);
 
-    IWineD3DSurface_GetContainerParent(This->wineD3DSurface, &containerParent);
-    if (containerParent) {
-        /* Forward to the containerParent */
-        TRACE("(%p) : Forwarding to %p\n", This, containerParent);
-        return IUnknown_Release(containerParent);
+    if (This->forwardReference) {
+        /* Forward refcounting */
+        TRACE("(%p) : Forwarding to %p\n", This, This->forwardReference);
+        return IUnknown_Release(This->forwardReference);
     } else {
         /* No container, handle our own refcounting */
         ULONG ref = InterlockedDecrement(&This->ref);
         TRACE("(%p) : ReleaseRef to %d\n", This, ref);
 
         if (ref == 0) {
-            IWineD3DSurface_Release(This->wineD3DSurface);
             if (This->parentDevice) IUnknown_Release(This->parentDevice);
-            HeapFree(GetProcessHeap(), 0, This);
+            /* Implicit surfaces are destroyed with the device, not if refcount reaches 0. */
+            if (!This->isImplicit) {
+                EnterCriticalSection(&d3d8_cs);
+                IWineD3DSurface_Release(This->wineD3DSurface);
+                LeaveCriticalSection(&d3d8_cs);
+                HeapFree(GetProcessHeap(), 0, This);
+            }
         }
 
         return ref;
@@ -88,76 +90,67 @@ static ULONG WINAPI IDirect3DSurface8Impl_Release(LPDIRECT3DSURFACE8 iface) {
 /* IDirect3DSurface8 IDirect3DResource8 Interface follow: */
 static HRESULT WINAPI IDirect3DSurface8Impl_GetDevice(LPDIRECT3DSURFACE8 iface, IDirect3DDevice8 **ppDevice) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
-    return IDirect3DResource8Impl_GetDevice((LPDIRECT3DRESOURCE8) This, ppDevice);
+    HRESULT hr;
+    TRACE("(%p)->(%p)\n", This, ppDevice);
+
+    EnterCriticalSection(&d3d8_cs);
+    hr = IDirect3DResource8Impl_GetDevice((LPDIRECT3DRESOURCE8) This, ppDevice);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 static HRESULT WINAPI IDirect3DSurface8Impl_SetPrivateData(LPDIRECT3DSURFACE8 iface, REFGUID refguid, CONST void *pData, DWORD SizeOfData, DWORD Flags) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
+    HRESULT hr;
     TRACE("(%p) Relay\n", This);
-    return IWineD3DSurface_SetPrivateData(This->wineD3DSurface, refguid, pData, SizeOfData, Flags);
+
+    EnterCriticalSection(&d3d8_cs);
+    hr = IWineD3DSurface_SetPrivateData(This->wineD3DSurface, refguid, pData, SizeOfData, Flags);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 static HRESULT WINAPI IDirect3DSurface8Impl_GetPrivateData(LPDIRECT3DSURFACE8 iface, REFGUID refguid, void *pData, DWORD *pSizeOfData) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
+    HRESULT hr;
     TRACE("(%p) Relay\n", This);
-    return IWineD3DSurface_GetPrivateData(This->wineD3DSurface, refguid, pData, pSizeOfData);
+
+    EnterCriticalSection(&d3d8_cs);
+    hr = IWineD3DSurface_GetPrivateData(This->wineD3DSurface, refguid, pData, pSizeOfData);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 static HRESULT WINAPI IDirect3DSurface8Impl_FreePrivateData(LPDIRECT3DSURFACE8 iface, REFGUID refguid) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
+    HRESULT hr;
     TRACE("(%p) Relay\n", This);
-    return IWineD3DSurface_FreePrivateData(This->wineD3DSurface, refguid);
+
+    EnterCriticalSection(&d3d8_cs);
+    hr = IWineD3DSurface_FreePrivateData(This->wineD3DSurface, refguid);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 /* IDirect3DSurface8 Interface follow: */
 static HRESULT WINAPI IDirect3DSurface8Impl_GetContainer(LPDIRECT3DSURFACE8 iface, REFIID riid, void **ppContainer) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
     HRESULT res;
-    IUnknown *IWineContainer = NULL;
 
     TRACE("(%p) Relay\n", This);
 
-    /* The container returned from IWineD3DSurface_GetContainer is either an IWineD3DDevice,
-       one of the subclasses of IWineD3DBaseTexture or an IWineD3DSwapChain  */
-    /* Get the IUnknown container. */
-    res = IWineD3DSurface_GetContainer(This->wineD3DSurface, &IID_IUnknown, (void **)&IWineContainer);
-    if (res == D3D_OK && IWineContainer != NULL) {
+    if (!This->container) return E_NOINTERFACE;
 
-        /* Now find out what kind of container it is (so that we can get its parent)*/
-        IUnknown  *IUnknownParent = NULL;
-        IUnknown  *myContainer    = NULL;
-        if (D3D_OK == IUnknown_QueryInterface(IWineContainer, &IID_IWineD3DDevice, (void **)&myContainer)) {
-            IWineD3DDevice_GetParent((IWineD3DDevice *)IWineContainer, &IUnknownParent);
-            IUnknown_Release(myContainer);
-        } else 
-        if (D3D_OK == IUnknown_QueryInterface(IWineContainer, &IID_IWineD3DBaseTexture, (void **)&myContainer)) {
-            IWineD3DBaseTexture_GetParent((IWineD3DBaseTexture *)IWineContainer, &IUnknownParent);
-            IUnknown_Release(myContainer);
-        } else
-        if (D3D_OK == IUnknown_QueryInterface(IWineContainer, &IID_IWineD3DSwapChain, (void **)&myContainer)) {
-            IWineD3DSwapChain_GetParent((IWineD3DSwapChain *)IWineContainer, &IUnknownParent);
-            IUnknown_Release(myContainer);
-        } else {
-            FIXME("Container is of unknown interface\n");
-        }
-        /* Tidy up.. */
-        IUnknown_Release((IWineD3DDevice *)IWineContainer);
+    res = IUnknown_QueryInterface(This->container, riid, ppContainer);
 
-        /* Now, query the interface of the parent for the riid */
-        if (IUnknownParent != NULL) {
-            res = IUnknown_QueryInterface(IUnknownParent, riid, ppContainer);
-            /* Tidy up.. */
-            IUnknown_Release(IUnknownParent);
-        }
-    }
-
-    TRACE("(%p) : returning %p\n", This, *ppContainer);    
+    TRACE("(%p) : returning %p\n", This, *ppContainer);
     return res;
 }
 
 static HRESULT WINAPI IDirect3DSurface8Impl_GetDesc(LPDIRECT3DSURFACE8 iface, D3DSURFACE_DESC *pDesc) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
     WINED3DSURFACE_DESC    wined3ddesc;
+    HRESULT hr;
     TRACE("(%p) Relay\n", This);
 
     /* As d3d8 and d3d9 structures differ, pass in ptrs to where data needs to go */
@@ -171,20 +164,49 @@ static HRESULT WINAPI IDirect3DSurface8Impl_GetDesc(LPDIRECT3DSURFACE8 iface, D3
     wined3ddesc.Width               = &pDesc->Width;
     wined3ddesc.Height              = &pDesc->Height;
 
-    return IWineD3DSurface_GetDesc(This->wineD3DSurface, &wined3ddesc);
+    EnterCriticalSection(&d3d8_cs);
+    hr = IWineD3DSurface_GetDesc(This->wineD3DSurface, &wined3ddesc);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 static HRESULT WINAPI IDirect3DSurface8Impl_LockRect(LPDIRECT3DSURFACE8 iface, D3DLOCKED_RECT *pLockedRect, CONST RECT *pRect, DWORD Flags) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
+    HRESULT hr;
     TRACE("(%p) Relay\n", This);
     TRACE("(%p) calling IWineD3DSurface_LockRect %p %p %p %d\n", This, This->wineD3DSurface, pLockedRect, pRect, Flags);
-    return IWineD3DSurface_LockRect(This->wineD3DSurface, (WINED3DLOCKED_RECT *) pLockedRect, pRect, Flags);
+
+    EnterCriticalSection(&d3d8_cs);
+    if (pRect) {
+        D3DSURFACE_DESC desc;
+        IDirect3DSurface8_GetDesc(iface, &desc);
+
+        if ((pRect->left < 0)
+                || (pRect->top < 0)
+                || (pRect->left >= pRect->right)
+                || (pRect->top >= pRect->bottom)
+                || (pRect->right > desc.Width)
+                || (pRect->bottom > desc.Height)) {
+            WARN("Trying to lock an invalid rectangle, returning D3DERR_INVALIDCALL\n");
+            LeaveCriticalSection(&d3d8_cs);
+            return D3DERR_INVALIDCALL;
+        }
+    }
+
+    hr = IWineD3DSurface_LockRect(This->wineD3DSurface, (WINED3DLOCKED_RECT *) pLockedRect, pRect, Flags);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 static HRESULT WINAPI IDirect3DSurface8Impl_UnlockRect(LPDIRECT3DSURFACE8 iface) {
     IDirect3DSurface8Impl *This = (IDirect3DSurface8Impl *)iface;
+    HRESULT hr;
     TRACE("(%p) Relay\n", This);
-    return IWineD3DSurface_UnlockRect(This->wineD3DSurface);
+
+    EnterCriticalSection(&d3d8_cs);
+    hr = IWineD3DSurface_UnlockRect(This->wineD3DSurface);
+    LeaveCriticalSection(&d3d8_cs);
+    return hr;
 }
 
 const IDirect3DSurface8Vtbl Direct3DSurface8_Vtbl =
