@@ -164,7 +164,7 @@ static void test_swapchain(void)
     d3dpp.BackBufferFormat = d3ddm.Format;
     d3dpp.BackBufferCount  = 0;
 
-    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_NULLREF, hwnd,
+    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                   D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &pDevice );
     ok(SUCCEEDED(hr), "Failed to create IDirect3D9Device (%s)\n", DXGetErrorString9(hr));
     if (FAILED(hr)) goto cleanup;
@@ -183,7 +183,7 @@ static void test_swapchain(void)
     ok(backbuffer != NULL, "The back buffer is NULL\n");
     if(backbuffer) IDirect3DSurface9_Release(backbuffer);
 
-    /* Try to get a nonexistant swapchain */
+    /* Try to get a nonexistent swapchain */
     hr = IDirect3DDevice9_GetSwapChain(pDevice, 1, &swapchainX);
     ok(hr == D3DERR_INVALIDCALL, "GetSwapChain on an nonexistent swapchain returned (%s)\n", DXGetErrorString9(hr));
     ok(swapchainX == NULL, "Swapchain 1 is %p\n", swapchainX);
@@ -279,6 +279,15 @@ static void test_swapchain(void)
     DestroyWindow( hwnd );
 }
 
+/* Shared between two functions */
+static const DWORD simple_vs[] = {0xFFFE0101,       /* vs_1_1               */
+    0x0000001F, 0x80000000, 0x900F0000,             /* dcl_position0 v0     */
+    0x00000009, 0xC0010000, 0x90E40000, 0xA0E40000, /* dp4 oPos.x, v0, c0   */
+    0x00000009, 0xC0020000, 0x90E40000, 0xA0E40001, /* dp4 oPos.y, v0, c1   */
+    0x00000009, 0xC0040000, 0x90E40000, 0xA0E40002, /* dp4 oPos.z, v0, c2   */
+    0x00000009, 0xC0080000, 0x90E40000, 0xA0E40003, /* dp4 oPos.w, v0, c3   */
+    0x0000FFFF};                                    /* END                  */
+
 static void test_refcount(void)
 {
     HRESULT                      hr;
@@ -313,13 +322,6 @@ static void test_refcount(void)
     {
 	D3DDECL_END()
     };
-    static DWORD simple_vs[] = {0xFFFE0101,             /* vs_1_1               */
-        0x0000001F, 0x80000000, 0x900F0000,             /* dcl_position0 v0     */
-        0x00000009, 0xC0010000, 0x90E40000, 0xA0E40000, /* dp4 oPos.x, v0, c0   */
-        0x00000009, 0xC0020000, 0x90E40000, 0xA0E40001, /* dp4 oPos.y, v0, c1   */
-        0x00000009, 0xC0040000, 0x90E40000, 0xA0E40002, /* dp4 oPos.z, v0, c2   */
-        0x00000009, 0xC0080000, 0x90E40000, 0xA0E40003, /* dp4 oPos.w, v0, c3   */
-        0x0000FFFF};                                    /* END                  */
     static DWORD simple_ps[] = {0xFFFF0101,                                     /* ps_1_1                       */
         0x00000051, 0xA00F0001, 0x3F800000, 0x00000000, 0x00000000, 0x00000000, /* def c1 = 1.0, 0.0, 0.0, 0.0  */
         0x00000042, 0xB00F0000,                                                 /* tex t0                       */
@@ -343,7 +345,7 @@ static void test_refcount(void)
     d3dpp.EnableAutoDepthStencil = TRUE;
     d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
 
-    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_NULLREF, hwnd,
+    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                   D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &pDevice );
     ok(SUCCEEDED(hr), "Failed to create IDirect3D9Device (%s)\n", DXGetErrorString9(hr));
     if (FAILED(hr)) goto cleanup;
@@ -354,8 +356,8 @@ static void test_refcount(void)
     /**
      * Check refcount of implicit surfaces and implicit swapchain. Findings:
      *   - the container is the device OR swapchain
-     *   - they hold a refernce to the device
-     *   - they are created with a refcount of 0 (Get/Release returns orignial refcount)
+     *   - they hold a reference to the device
+     *   - they are created with a refcount of 0 (Get/Release returns original refcount)
      *   - they are not freed if refcount reaches 0.
      *   - the refcount is not forwarded to the container.
      */
@@ -653,7 +655,7 @@ static void test_cursor(void)
     d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
     d3dpp.BackBufferFormat = d3ddm.Format;
 
-    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_NULLREF, hwnd,
+    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
                                   D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &pDevice );
     ok(SUCCEEDED(hr), "Failed to create IDirect3D9Device (%s)\n", DXGetErrorString9(hr));
     if (FAILED(hr)) goto cleanup;
@@ -714,11 +716,16 @@ static void test_reset(void)
     IDirect3D9                  *pD3d               = NULL;
     IDirect3DDevice9            *pDevice            = NULL;
     D3DPRESENT_PARAMETERS        d3dpp;
-    D3DDISPLAYMODE               d3ddm;
+    D3DDISPLAYMODE               d3ddm, d3ddm2;
     D3DVIEWPORT9                 vp;
     DWORD                        width, orig_width = GetSystemMetrics(SM_CXSCREEN);
     DWORD                        height, orig_height = GetSystemMetrics(SM_CYSCREEN);
     IDirect3DSwapChain9          *pSwapchain;
+    IDirect3DSurface9            *surface;
+    IDirect3DTexture9            *texture;
+    IDirect3DVertexShader9       *shader;
+    BOOL                         support_800x600 = FALSE;
+    UINT                         i;
 
     pD3d = pDirect3DCreate9( D3D_SDK_VERSION );
     ok(pD3d != NULL, "Failed to create IDirect3D9 object\n");
@@ -734,6 +741,27 @@ static void test_reset(void)
     d3dpp.BackBufferHeight  = 600;
     d3dpp.BackBufferFormat = d3ddm.Format;
 
+    for(i = 0; i < IDirect3D9_GetAdapterModeCount(pD3d, D3DADAPTER_DEFAULT, d3ddm.Format); i++) {
+        ZeroMemory( &d3ddm2, sizeof(d3ddm2) );
+        hr = IDirect3D9_EnumAdapterModes(pD3d, D3DADAPTER_DEFAULT, d3ddm.Format, i, &d3ddm2);
+        ok(hr == D3D_OK, "IDirect3D9_EnumAdapterModes returned %#x\n", hr);
+
+        if(d3ddm2.Width == 800 && d3ddm2.Height == 600) {
+            support_800x600 = TRUE;
+        }
+        /* We use them as invalid modes */
+        if((d3ddm2.Width == 801 && d3ddm2.Height == 600) ||
+           (d3ddm2.Width == 32 && d3ddm2.Height == 32)) {
+            skip("This system supports a screen resolution of %dx%d, not running mode tests\n",
+                 d3ddm2.Width, d3ddm2.Height);
+            goto cleanup;
+        }
+    }
+    if(!support_800x600) {
+        skip("Mode 800x600 not supported, skipping mode tests\n");
+        goto cleanup;
+    }
+
     hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL /* no NULLREF here */, hwnd,
                                   D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &pDevice );
 
@@ -742,6 +770,8 @@ static void test_reset(void)
         skip("could not create device, IDirect3D9_CreateDevice returned %#x\n", hr);
         goto cleanup;
     }
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after creation returned %#x\n", hr);
 
     width = GetSystemMetrics(SM_CXSCREEN);
     height = GetSystemMetrics(SM_CYSCREEN);
@@ -774,6 +804,8 @@ static void test_reset(void)
     d3dpp.BackBufferFormat = d3ddm.Format;
     hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
     ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
 
     ZeroMemory(&vp, sizeof(vp));
     hr = IDirect3DDevice9_GetViewport(pDevice, &vp);
@@ -815,6 +847,8 @@ static void test_reset(void)
     d3dpp.BackBufferHeight  = 300;
     hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
     ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
 
     width = GetSystemMetrics(SM_CXSCREEN);
     height = GetSystemMetrics(SM_CYSCREEN);
@@ -849,6 +883,92 @@ static void test_reset(void)
         IDirect3DSwapChain9_Release(pSwapchain);
     }
 
+    ZeroMemory( &d3dpp, sizeof(d3dpp) );
+    d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
+    d3dpp.Windowed         = TRUE;
+    d3dpp.BackBufferWidth  = 400;
+    d3dpp.BackBufferHeight  = 300;
+
+    /* _Reset fails if there is a resource in the default pool */
+    hr = IDirect3DDevice9_CreateOffscreenPlainSurface(pDevice, 16, 16, D3DFMT_R5G6B5, D3DPOOL_DEFAULT, &surface, NULL);
+    ok(hr == D3D_OK, "IDirect3DDevice9_CreateOffscreenPlainSurface returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3DERR_INVALIDCALL, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3DERR_DEVICENOTRESET, "IDirect3DDevice9_TestCooperativeLevel after a failed reset returned %#x\n", hr);
+    IDirect3DSurface9_Release(surface);
+    /* Reset again to get the device out of the lost state */
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
+
+    /* Scratch, sysmem and managed pools are fine */
+    hr = IDirect3DDevice9_CreateOffscreenPlainSurface(pDevice, 16, 16, D3DFMT_R5G6B5, D3DPOOL_SCRATCH, &surface, NULL);
+    ok(hr == D3D_OK, "IDirect3DDevice9_CreateOffscreenPlainSurface returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
+    IDirect3DSurface9_Release(surface);
+
+    hr = IDirect3DDevice9_CreateOffscreenPlainSurface(pDevice, 16, 16, D3DFMT_R5G6B5, D3DPOOL_SYSTEMMEM, &surface, NULL);
+    ok(hr == D3D_OK, "IDirect3DDevice9_CreateOffscreenPlainSurface returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
+    IDirect3DSurface9_Release(surface);
+
+    hr = IDirect3DDevice9_CreateTexture(pDevice, 16, 16, 0, 0, D3DFMT_R5G6B5, D3DPOOL_MANAGED, &texture, NULL);
+    ok(hr == D3D_OK, "IDirect3DDevice9_CreateTexture returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
+    IDirect3DTexture9_Release(texture);
+
+    /* A reference held to an implicit surface causes failures as well */
+    hr = IDirect3DDevice9_GetBackBuffer(pDevice, 0, 0, D3DBACKBUFFER_TYPE_MONO, &surface);
+    ok(hr == D3D_OK, "IDirect3DDevice9_GetBackBuffer returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3DERR_INVALIDCALL, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3DERR_DEVICENOTRESET, "IDirect3DDevice9_TestCooperativeLevel after a failed reset returned %#x\n", hr);
+    IDirect3DSurface9_Release(surface);
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3D_OK, "IDirect3DDevice9_TestCooperativeLevel after a successful reset returned %#x\n", hr);
+
+    /* Shaders are fine as well */
+    hr = IDirect3DDevice9_CreateVertexShader(pDevice, simple_vs, &shader);
+    ok(hr == D3D_OK, "IDirect3DDevice9_CreateVertexShader returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3D_OK, "IDirect3DDevice9_Reset failed with %s\n", DXGetErrorString9(hr));
+    IDirect3DVertexShader9_Release(shader);
+
+    /* Try setting invalid modes */
+    ZeroMemory( &d3dpp, sizeof(d3dpp) );
+    d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
+    d3dpp.Windowed         = FALSE;
+    d3dpp.BackBufferWidth  = 32;
+    d3dpp.BackBufferHeight  = 32;
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3DERR_INVALIDCALL, "IDirect3DDevice9_Reset to w=32, h=32, windowed=FALSE failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3DERR_DEVICENOTRESET, "IDirect3DDevice9_TestCooperativeLevel after a failed reset returned %#x\n", hr);
+
+    ZeroMemory( &d3dpp, sizeof(d3dpp) );
+    d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
+    d3dpp.Windowed         = FALSE;
+    d3dpp.BackBufferWidth  = 801;
+    d3dpp.BackBufferHeight  = 600;
+    hr = IDirect3DDevice9_Reset(pDevice, &d3dpp);
+    ok(hr == D3DERR_INVALIDCALL, "IDirect3DDevice9_Reset to w=801, h=600, windowed=FALSE failed with %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_TestCooperativeLevel(pDevice);
+    ok(hr == D3DERR_DEVICENOTRESET, "IDirect3DDevice9_TestCooperativeLevel after a failed reset returned %#x\n", hr);
+
 cleanup:
     if(pD3d) IDirect3D9_Release(pD3d);
     if(pDevice) IDirect3D9_Release(pDevice);
@@ -1175,7 +1295,8 @@ static void test_depthstenciltest(void)
     D3DPRESENT_PARAMETERS        d3dpp;
     D3DDISPLAYMODE               d3ddm;
     IDirect3DSurface9           *pDepthStencil           = NULL;
-    DWORD                        state;
+    IDirect3DSurface9           *pDepthStencil2          = NULL;
+    D3DZBUFFERTYPE               state;
 
     pD3d = pDirect3DCreate9( D3D_SDK_VERSION );
     ok(pD3d != NULL, "Failed to create IDirect3D9 object\n");
@@ -1212,6 +1333,11 @@ static void test_depthstenciltest(void)
     hr = IDirect3DDevice9_SetDepthStencilSurface(pDevice, NULL);
     ok(hr == D3D_OK, "IDirect3DDevice9_SetDepthStencilSurface failed with %s\n", DXGetErrorString9(hr));
 
+    /* Check if the set buffer is returned on a get. WineD3D had a bug with that once, prevent it from coming back */
+    hr = IDirect3DDevice9_GetDepthStencilSurface(pDevice, &pDepthStencil2);
+    ok(hr == D3DERR_NOTFOUND && pDepthStencil2 == NULL, "IDirect3DDevice9_GetDepthStencilSurface failed with %s\n", DXGetErrorString9(hr));
+    if(pDepthStencil2) IDirect3DSurface9_Release(pDepthStencil2);
+
     /* This left the render states untouched! */
     hr = IDirect3DDevice9_GetRenderState(pDevice, D3DRS_ZENABLE, &state);
     ok(hr == D3D_OK, "IDirect3DDevice9_GetRenderState failed with %s\n", DXGetErrorString9(hr));
@@ -1244,6 +1370,74 @@ static void test_depthstenciltest(void)
     hr = IDirect3DDevice9_Clear(pDevice, 0, NULL, D3DCLEAR_ZBUFFER, 0x00000000, 1.0, 0);
     ok(hr == D3D_OK, "IDirect3DDevice9_Clear failed with %s\n", DXGetErrorString9(hr));
 
+    if(pDepthStencil) IDirect3DSurface9_Release(pDepthStencil);
+    if(pDevice) IDirect3D9_Release(pDevice);
+
+    /* Now see if autodepthstencil disable is honored. First, without a format set */
+    ZeroMemory( &d3dpp, sizeof(d3dpp) );
+    d3dpp.Windowed         = TRUE;
+    d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
+    d3dpp.BackBufferWidth  = 800;
+    d3dpp.BackBufferHeight  = 600;
+    d3dpp.BackBufferFormat = d3ddm.Format;
+    d3dpp.EnableAutoDepthStencil = FALSE;
+    d3dpp.AutoDepthStencilFormat = D3DFMT_UNKNOWN;
+
+    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL /* no NULLREF here */, hwnd,
+                                  D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &pDevice );
+    ok(hr == D3D_OK || hr == D3DERR_NOTAVAILABLE, "IDirect3D9_CreateDevice failed with %s\n", DXGetErrorString9(hr));
+    if(!pDevice)
+    {
+        skip("Failed to create a d3d device\n");
+        goto cleanup;
+    }
+
+    pDepthStencil = NULL;
+    hr = IDirect3DDevice9_GetDepthStencilSurface(pDevice, &pDepthStencil);
+    ok(hr == D3DERR_NOTFOUND && pDepthStencil == NULL, "IDirect3DDevice9_GetDepthStencilSurface returned %s, surface = %p\n", DXGetErrorString9(hr), pDepthStencil);
+    if(pDepthStencil) {
+        IDirect3DSurface9_Release(pDepthStencil);
+        pDepthStencil = NULL;
+    }
+
+    /* Check the depth test state */
+    hr = IDirect3DDevice9_GetRenderState(pDevice, D3DRS_ZENABLE, &state);
+    ok(hr == D3D_OK, "IDirect3DDevice9_GetRenderState failed with %s\n", DXGetErrorString9(hr));
+    ok(state == D3DZB_FALSE, "D3DRS_ZENABLE is %s\n", state == D3DZB_FALSE ? "D3DZB_FALSE" : (state == D3DZB_TRUE ? "D3DZB_TRUE" : "D3DZB_USEW"));
+
+    if(pDevice) IDirect3D9_Release(pDevice);
+
+    /* Next, try EnableAutoDepthStencil FALSE with a depth stencil format set */
+    ZeroMemory( &d3dpp, sizeof(d3dpp) );
+    d3dpp.Windowed         = TRUE;
+    d3dpp.SwapEffect       = D3DSWAPEFFECT_DISCARD;
+    d3dpp.BackBufferWidth  = 800;
+    d3dpp.BackBufferHeight  = 600;
+    d3dpp.BackBufferFormat = d3ddm.Format;
+    d3dpp.EnableAutoDepthStencil = FALSE;
+    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
+
+    hr = IDirect3D9_CreateDevice( pD3d, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL /* no NULLREF here */, hwnd,
+                                  D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &pDevice );
+    ok(hr == D3D_OK || hr == D3DERR_NOTAVAILABLE, "IDirect3D9_CreateDevice failed with %s\n", DXGetErrorString9(hr));
+    if(!pDevice)
+    {
+        skip("Failed to create a d3d device\n");
+        goto cleanup;
+    }
+
+    pDepthStencil = NULL;
+    hr = IDirect3DDevice9_GetDepthStencilSurface(pDevice, &pDepthStencil);
+    ok(hr == D3DERR_NOTFOUND && pDepthStencil == NULL, "IDirect3DDevice9_GetDepthStencilSurface returned %s, surface = %p\n", DXGetErrorString9(hr), pDepthStencil);
+    if(pDepthStencil) {
+        IDirect3DSurface9_Release(pDepthStencil);
+        pDepthStencil = NULL;
+    }
+
+    hr = IDirect3DDevice9_GetRenderState(pDevice, D3DRS_ZENABLE, &state);
+    ok(hr == D3D_OK, "IDirect3DDevice9_GetRenderState failed with %s\n", DXGetErrorString9(hr));
+    ok(state == D3DZB_FALSE, "D3DRS_ZENABLE is %s\n", state == D3DZB_FALSE ? "D3DZB_FALSE" : (state == D3DZB_TRUE ? "D3DZB_TRUE" : "D3DZB_USEW"));
+
 cleanup:
     if(pDepthStencil) IDirect3DSurface9_Release(pDepthStencil);
     if(pD3d) IDirect3D9_Release(pD3d);
@@ -1375,8 +1569,8 @@ static void test_null_stream(void)
     IDirect3D9 *d3d9;
     HWND hwnd;
     HRESULT hr;
-    IDirect3DVertexShader9 *shader;
-    IDirect3DVertexDeclaration9 *decl;
+    IDirect3DVertexShader9 *shader = NULL;
+    IDirect3DVertexDeclaration9 *decl = NULL;
     DWORD shader_code[] = {
         0xfffe0101,                             /* vs_1_1           */
         0x0000001f, 0x80000000, 0x900f0000,     /* dcl_position v0  */
@@ -1416,8 +1610,16 @@ static void test_null_stream(void)
     }
     hr = IDirect3DDevice9_CreateVertexDeclaration(device, decl_elements, &decl);
     ok(SUCCEEDED(hr), "IDirect3DDevice9_CreateVertexDeclaration failed (0x%08x)\n", hr);
+    if (!SUCCEEDED(hr)) {
+	skip("Vertex declaration handling not possible.\n");
+	goto cleanup;
+    }
     hr = IDirect3DDevice9_CreateVertexBuffer(device, 12 * sizeof(float), 0, 0, D3DPOOL_MANAGED, &buffer, NULL);
     ok(SUCCEEDED(hr), "IDirect3DDevice9_CreateVertexBuffer failed (0x%08x)\n", hr);
+    if (!SUCCEEDED(hr)) {
+	skip("Vertex buffer handling not possible.\n");
+	goto cleanup;
+    }
 
     hr = IDirect3DDevice9_SetStreamSource(device, 0, buffer, 0, sizeof(float) * 3);
     ok(SUCCEEDED(hr), "IDirect3DDevice9_SetStreamSource failed (0x%08x)\n", hr);
@@ -1442,13 +1644,348 @@ static void test_null_stream(void)
     IDirect3DDevice9_SetVertexShader(device, NULL);
     IDirect3DDevice9_SetVertexDeclaration(device, NULL);
 
-    cleanup:
+cleanup:
     if(decl) IDirect3DVertexDeclaration9_Release(decl);
     if(shader) IDirect3DVertexShader9_Release(shader);
     if(device) IDirect3DDevice9_Release(device);
     if(d3d9) IDirect3D9_Release(d3d9);
 }
 
+static inline const char *debug_d3dpool(D3DPOOL pool) {
+    switch(pool) {
+        case D3DPOOL_DEFAULT: return "D3DPOOL_DEFAULT";
+        case D3DPOOL_SYSTEMMEM: return "D3DPOOL_SYSTEMMEM";
+        case D3DPOOL_SCRATCH: return "D3DPOOL_SCRATCH";
+        case D3DPOOL_MANAGED: return "D3DPOOL_MANAGED";
+        default:
+        return "unknown pool";
+    }
+}
+
+static void test_vertex_buffer_alignment(void)
+{
+    IDirect3DVertexBuffer9 *buffer = NULL;
+    D3DPRESENT_PARAMETERS present_parameters;
+    IDirect3DDevice9 *device = NULL;
+    IDirect3D9 *d3d9;
+    HWND hwnd;
+    HRESULT hr;
+    D3DPOOL pools[] = {D3DPOOL_DEFAULT, D3DPOOL_SYSTEMMEM, D3DPOOL_SCRATCH, D3DPOOL_MANAGED};
+    DWORD sizes[] = {1, 4, 16, 17, 32, 33, 64, 65, 1024, 1025, 1048576, 1048577};
+    unsigned int i, j;
+    void *data;
+
+    d3d9 = pDirect3DCreate9( D3D_SDK_VERSION );
+    ok(d3d9 != NULL, "Failed to create IDirect3D9 object\n");
+    hwnd = CreateWindow( "static", "d3d9_test", WS_OVERLAPPEDWINDOW, 100, 100, 160, 160, NULL, NULL, NULL, NULL );
+    ok(hwnd != NULL, "Failed to create window\n");
+    if (!d3d9 || !hwnd) goto cleanup;
+
+    ZeroMemory(&present_parameters, sizeof(present_parameters));
+    present_parameters.Windowed = TRUE;
+    present_parameters.hDeviceWindow = hwnd;
+    present_parameters.SwapEffect = D3DSWAPEFFECT_DISCARD;
+
+    hr = IDirect3D9_CreateDevice( d3d9, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL /* no NULLREF here */, hwnd,
+                                  D3DCREATE_SOFTWARE_VERTEXPROCESSING, &present_parameters, &device );
+    ok(hr == D3D_OK || hr == D3DERR_NOTAVAILABLE, "IDirect3D9_CreateDevice failed with %s\n", DXGetErrorString9(hr));
+    if(!device)
+    {
+        skip("Failed to create a d3d device\n");
+        goto cleanup;
+    }
+
+    for(i = 0; i < (sizeof(sizes) / sizeof(sizes[0])); i++) {
+        for(j = 0; j < (sizeof(pools) / sizeof(pools[0])); j++) {
+            hr = IDirect3DDevice9_CreateVertexBuffer(device, sizes[i], 0, 0, pools[j], &buffer, NULL);
+            if(pools[j] == D3DPOOL_SCRATCH) {
+                ok(hr == D3DERR_INVALIDCALL, "Creating a D3DPOOL_SCRATCH buffer returned (0x%08x)\n", hr);
+            } else {
+                ok(SUCCEEDED(hr), "IDirect3DDevice9_CreateVertexBuffer failed (0x%08x). Pool = %s, size %d\n", hr,
+                   debug_d3dpool(pools[j]), sizes[i]);
+            }
+            if(FAILED(hr)) continue;
+
+            hr = IDirect3DVertexBuffer9_Lock(buffer, 0, 0, (void **) &data, 0);
+            ok(SUCCEEDED(hr), "IDirect3DVertexBuffer9_Lock failed (0x%08x)\n", hr);
+            ok(((DWORD_PTR) data & 31) == 0, "Vertex buffer start address is not 32 byte aligned(size: %d, pool: %s, data: %p)\n",
+               sizes[i], debug_d3dpool(pools[j]), data);
+            hr = IDirect3DVertexBuffer9_Unlock(buffer);
+            ok(SUCCEEDED(hr), "IDirect3DVertexBuffer9_Unlock failed (0x%08x)\n", hr);
+
+            if(buffer) IDirect3DVertexBuffer9_Release(buffer);
+        }
+    }
+
+    cleanup:
+    if(d3d9) IDirect3D9_Release(d3d9);
+}
+
+static void test_lights(void)
+{
+    D3DPRESENT_PARAMETERS present_parameters;
+    IDirect3DDevice9 *device = NULL;
+    IDirect3D9 *d3d9;
+    HWND hwnd;
+    HRESULT hr;
+    unsigned int i;
+    BOOL enabled;
+    D3DCAPS9 caps;
+
+    d3d9 = pDirect3DCreate9( D3D_SDK_VERSION );
+    ok(d3d9 != NULL, "Failed to create IDirect3D9 object\n");
+    hwnd = CreateWindow( "static", "d3d9_test", WS_OVERLAPPEDWINDOW, 100, 100, 160, 160, NULL, NULL, NULL, NULL );
+    ok(hwnd != NULL, "Failed to create window\n");
+    if (!d3d9 || !hwnd) goto cleanup;
+
+    ZeroMemory(&present_parameters, sizeof(present_parameters));
+    present_parameters.Windowed = TRUE;
+    present_parameters.hDeviceWindow = hwnd;
+    present_parameters.SwapEffect = D3DSWAPEFFECT_DISCARD;
+
+    hr = IDirect3D9_CreateDevice( d3d9, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
+                                  D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE, &present_parameters, &device );
+    ok(hr == D3D_OK || hr == D3DERR_NOTAVAILABLE, "IDirect3D9_CreateDevice failed with %s\n", DXGetErrorString9(hr));
+    if(!device)
+    {
+        skip("Failed to create a d3d device\n");
+        goto cleanup;
+    }
+
+    memset(&caps, 0, sizeof(caps));
+    hr = IDirect3DDevice9_GetDeviceCaps(device, &caps);
+    ok(hr == D3D_OK, "IDirect3DDevice9_GetDeviceCaps failed with %s\n", DXGetErrorString9(hr));
+
+    for(i = 1; i <= caps.MaxActiveLights; i++) {
+        hr = IDirect3DDevice9_LightEnable(device, i, TRUE);
+        ok(hr == D3D_OK, "Enabling light %u failed with %s\n", i, DXGetErrorString9(hr));
+        hr = IDirect3DDevice9_GetLightEnable(device, i, &enabled);
+        ok(hr == D3D_OK, "GetLightEnable on light %u failed with %s\n", i, DXGetErrorString9(hr));
+        ok(enabled, "Light %d is %s\n", i, enabled ? "enabled" : "disabled");
+    }
+
+    /* TODO: Test the rendering results in this situation */
+    hr = IDirect3DDevice9_LightEnable(device, i + 1, TRUE);
+    ok(hr == D3D_OK, "Enabling one light more than supported returned %s\n", DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_GetLightEnable(device, i + 1, &enabled);
+    ok(hr == D3D_OK, "GetLightEnable on light %u failed with %s\n", i + 1, DXGetErrorString9(hr));
+    ok(enabled, "Light %d is %s\n", i + 1, enabled ? "enabled" : "disabled");
+    hr = IDirect3DDevice9_LightEnable(device, i + 1, FALSE);
+    ok(hr == D3D_OK, "Disabling the additional returned %s\n", DXGetErrorString9(hr));
+
+    for(i = 1; i <= caps.MaxActiveLights; i++) {
+        hr = IDirect3DDevice9_LightEnable(device, i, FALSE);
+        ok(hr == D3D_OK, "Disabling light %u failed with %s\n", i, DXGetErrorString9(hr));
+    }
+
+    cleanup:
+    if(device) IDirect3DDevice9_Release(device);
+    if(d3d9) IDirect3D9_Release(d3d9);
+}
+
+static void test_set_stream_source(void)
+{
+    D3DPRESENT_PARAMETERS present_parameters;
+    IDirect3DDevice9 *device = NULL;
+    IDirect3D9 *d3d9;
+    HWND hwnd;
+    HRESULT hr;
+    IDirect3DVertexBuffer9 *pVertexBuffer = NULL;
+
+    d3d9 = pDirect3DCreate9( D3D_SDK_VERSION );
+    ok(d3d9 != NULL, "Failed to create IDirect3D9 object\n");
+    hwnd = CreateWindow( "static", "d3d9_test", WS_OVERLAPPEDWINDOW, 100, 100, 160, 160, NULL, NULL, NULL, NULL );
+    ok(hwnd != NULL, "Failed to create window\n");
+    if (!d3d9 || !hwnd) goto cleanup;
+
+    ZeroMemory(&present_parameters, sizeof(present_parameters));
+    present_parameters.Windowed = TRUE;
+    present_parameters.hDeviceWindow = hwnd;
+    present_parameters.SwapEffect = D3DSWAPEFFECT_DISCARD;
+
+    hr = IDirect3D9_CreateDevice( d3d9, D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd,
+                                  D3DCREATE_HARDWARE_VERTEXPROCESSING, &present_parameters, &device );
+    ok(hr == D3D_OK || hr == D3DERR_NOTAVAILABLE, "IDirect3D9_CreateDevice failed with %s\n", DXGetErrorString9(hr));
+    if(!device)
+    {
+        hr = IDirect3D9_CreateDevice( d3d9, D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, hwnd,
+                                      D3DCREATE_HARDWARE_VERTEXPROCESSING, &present_parameters, &device );
+        ok(hr == D3D_OK || hr == D3DERR_NOTAVAILABLE, "IDirect3D9_CreateDevice failed with %s\n", DXGetErrorString9(hr));
+        if(!device)
+        {
+            skip("Failed to create a d3d device\n");
+            goto cleanup;
+        }
+    }
+
+    hr = IDirect3DDevice9_CreateVertexBuffer( device, 512, 0, 0, D3DPOOL_DEFAULT, &pVertexBuffer, NULL );
+    ok(hr == D3D_OK, "Failed to create a vertex buffer, hr = %s\n", DXGetErrorString9(hr));
+    if (SUCCEEDED(hr)) {
+        /* Some cards(Geforce 7400 at least) accept non-aligned offsets, others(radeon 9000 verified) reject it,
+         * so accept both results. Wine currently rejects this to be able to optimize the vbo conversion, but writes
+         * a WARN
+         */
+        hr = IDirect3DDevice9_SetStreamSource(device, 0, pVertexBuffer, 0, 32);
+        ok(hr == D3D_OK, "Failed to set the stream source, offset 0, hr = %s\n",
+           DXGetErrorString9(hr));
+        hr = IDirect3DDevice9_SetStreamSource(device, 0, pVertexBuffer, 1, 32);
+        ok(hr == D3DERR_INVALIDCALL || hr == D3D_OK, "Unexpected result when setting the stream source, offset 1, hr = %s\n",
+           DXGetErrorString9(hr));
+        hr = IDirect3DDevice9_SetStreamSource(device, 0, pVertexBuffer, 2, 32);
+        ok(hr == D3DERR_INVALIDCALL || hr == D3D_OK, "Unexpected result when setting the stream source, offset 2, hr = %s\n",
+           DXGetErrorString9(hr));
+        hr = IDirect3DDevice9_SetStreamSource(device, 0, pVertexBuffer, 3, 32);
+        ok(hr == D3DERR_INVALIDCALL || hr == D3D_OK, "Unexpected result when setting the stream source, offset 3, hr = %s\n",
+           DXGetErrorString9(hr));
+        hr = IDirect3DDevice9_SetStreamSource(device, 0, pVertexBuffer, 4, 32);
+        ok(hr == D3D_OK, "Failed to set the stream source, offset 4, hr = %s\n",
+          DXGetErrorString9(hr));
+    }
+    /* Try to set the NULL buffer with an offset and stride 0 */
+    hr = IDirect3DDevice9_SetStreamSource(device, 0, NULL, 0, 0);
+    ok(hr == D3D_OK, "Failed to set the stream source, offset 0, hr = %s\n",
+       DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_SetStreamSource(device, 0, NULL, 1, 0);
+    ok(hr == D3DERR_INVALIDCALL || hr == D3D_OK, "Unexpected result when setting the stream source, offset 1, hr = %s\n",
+       DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_SetStreamSource(device, 0, NULL, 2, 0);
+    ok(hr == D3DERR_INVALIDCALL || hr == D3D_OK, "Unexpected result when setting the stream source, offset 2, hr = %s\n",
+       DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_SetStreamSource(device, 0, NULL, 3, 0);
+    ok(hr == D3DERR_INVALIDCALL || hr == D3D_OK, "Unexpected result when setting the stream source, offset 3, hr = %s\n",
+       DXGetErrorString9(hr));
+    hr = IDirect3DDevice9_SetStreamSource(device, 0, NULL, 4, 0);
+    ok(hr == D3D_OK, "Failed to set the stream source, offset 4, hr = %s\n",
+       DXGetErrorString9(hr));
+
+    hr = IDirect3DDevice9_SetStreamSource(device, 0, NULL, 0, 0);
+    ok(hr == D3D_OK, "Failed to set the stream source, offset 4, hr = %s\n", DXGetErrorString9(hr));
+
+    if(pVertexBuffer) IDirect3DDevice9_Release(pVertexBuffer);
+cleanup:
+    if(device) IDirect3DDevice9_Release(device);
+    if(d3d9) IDirect3D9_Release(d3d9);
+}
+
+struct formats {
+    D3DFORMAT DisplayFormat;
+    D3DFORMAT BackBufferFormat;
+    BOOL shouldPass;
+};
+
+struct formats r5g6b5_format_list[] =
+{
+    { D3DFMT_R5G6B5, D3DFMT_R5G6B5, TRUE },
+    { D3DFMT_R5G6B5, D3DFMT_X1R5G5B5, FALSE },
+    { D3DFMT_R5G6B5, D3DFMT_A1R5G5B5, FALSE },
+    { D3DFMT_R5G6B5, D3DFMT_X8R8G8B8, FALSE },
+    { D3DFMT_R5G6B5, D3DFMT_A8R8G8B8, FALSE },
+    { 0, 0, 0}
+};
+
+struct formats x1r5g5b5_format_list[] =
+{
+    { D3DFMT_X1R5G5B5, D3DFMT_R5G6B5, FALSE },
+    { D3DFMT_X1R5G5B5, D3DFMT_X1R5G5B5, TRUE },
+    { D3DFMT_X1R5G5B5, D3DFMT_A1R5G5B5, TRUE },
+    { D3DFMT_X1R5G5B5, D3DFMT_X8R8G8B8, FALSE },
+    { D3DFMT_X1R5G5B5, D3DFMT_A8R8G8B8, FALSE },
+
+    /* A1R5G5B5 should not be usable as a display format, it is backbuffer-only */
+    { D3DFMT_A1R5G5B5, D3DFMT_R5G6B5, FALSE },
+    { D3DFMT_A1R5G5B5, D3DFMT_X1R5G5B5, FALSE },
+    { D3DFMT_A1R5G5B5, D3DFMT_A1R5G5B5, FALSE },
+    { D3DFMT_A1R5G5B5, D3DFMT_X8R8G8B8, FALSE },
+    { D3DFMT_A1R5G5B5, D3DFMT_A8R8G8B8, FALSE },
+    { 0, 0, 0}
+};
+
+struct formats x8r8g8b8_format_list[] =
+{
+    { D3DFMT_X8R8G8B8, D3DFMT_R5G6B5, FALSE },
+    { D3DFMT_X8R8G8B8, D3DFMT_X1R5G5B5, FALSE },
+    { D3DFMT_X8R8G8B8, D3DFMT_A1R5G5B5, FALSE },
+    { D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, TRUE },
+    { D3DFMT_X8R8G8B8, D3DFMT_A8R8G8B8, TRUE },
+
+    /* A1R8G8B8 should not be usable as a display format, it is backbuffer-only */
+    { D3DFMT_A8R8G8B8, D3DFMT_R5G6B5, FALSE },
+    { D3DFMT_A8R8G8B8, D3DFMT_X1R5G5B5, FALSE },
+    { D3DFMT_A8R8G8B8, D3DFMT_A1R5G5B5, FALSE },
+    { D3DFMT_A8R8G8B8, D3DFMT_X8R8G8B8, FALSE },
+    { D3DFMT_A8R8G8B8, D3DFMT_A8R8G8B8, FALSE },
+    { 0, 0, 0}
+};
+
+static void test_display_formats()
+{
+    /* Direct3D9 offers 4 display formats R5G6B5, X1R5G5B5, X8R8G8B8 and A2R10G10B10.
+     * Next to these there are 6 different backbuffer formats. Only a fixed number of
+     * combinations are possible in FULLSCREEN mode. In windowed mode more combinations are
+     * allowed due to depth conversion and this is likely driver dependent.
+     * This test checks which combinations are possible in fullscreen mode and this should not be driver dependent.
+     * TODO: handle A2R10G10B10 but what hardware supports it? Parhelia? It is very rare. */
+
+    UINT Adapter = D3DADAPTER_DEFAULT;
+    D3DDEVTYPE DeviceType = D3DDEVTYPE_HAL;
+    int i, nmodes;
+    HRESULT hr;
+
+    IDirect3D9 *d3d9 = pDirect3DCreate9( D3D_SDK_VERSION );
+    ok(d3d9 != NULL, "Failed to create IDirect3D9 object\n");
+    if(!d3d9) return;
+
+    nmodes = IDirect3D9_GetAdapterModeCount(d3d9, D3DADAPTER_DEFAULT, D3DFMT_R5G6B5);
+    if(!nmodes) {
+        skip("Display format R5G6B5 not supported, skipping\n");
+    } else {
+        trace("Testing display format R5G6B5\n");
+        for(i=0; r5g6b5_format_list[i].DisplayFormat != 0; i++)
+        {
+            hr = IDirect3D9_CheckDeviceType(d3d9, Adapter, DeviceType, r5g6b5_format_list[i].DisplayFormat, r5g6b5_format_list[i].BackBufferFormat, FALSE);
+
+            if(r5g6b5_format_list[i].shouldPass)
+                ok(hr == D3D_OK, "format %d %d didn't pass with hr=%#08x\n", r5g6b5_format_list[i].DisplayFormat, r5g6b5_format_list[i].BackBufferFormat, hr);
+            else
+                ok(hr != D3D_OK, "format %d %d didn't pass while it was expected to\n", r5g6b5_format_list[i].DisplayFormat, r5g6b5_format_list[i].BackBufferFormat);
+        }
+    }
+
+    nmodes = IDirect3D9_GetAdapterModeCount(d3d9, D3DADAPTER_DEFAULT, D3DFMT_X1R5G5B5);
+    if(!nmodes) {
+        skip("Display format X1R5G5B5 not supported, skipping\n");
+    } else {
+        trace("Testing display format X1R5G5B5\n");
+        for(i=0; x1r5g5b5_format_list[i].DisplayFormat != 0; i++)
+        {
+            hr = IDirect3D9_CheckDeviceType(d3d9, Adapter, DeviceType, x1r5g5b5_format_list[i].DisplayFormat, x1r5g5b5_format_list[i].BackBufferFormat, FALSE);
+
+            if(x1r5g5b5_format_list[i].shouldPass)
+                ok(hr == D3D_OK, "format %d %d didn't pass with hr=%#08x\n", x1r5g5b5_format_list[i].DisplayFormat, x1r5g5b5_format_list[i].BackBufferFormat, hr);
+            else
+                ok(hr != D3D_OK, "format %d %d didn't pass while it was expected to\n", x1r5g5b5_format_list[i].DisplayFormat, x1r5g5b5_format_list[i].BackBufferFormat);
+        }
+    }
+
+    nmodes = IDirect3D9_GetAdapterModeCount(d3d9, D3DADAPTER_DEFAULT, D3DFMT_X8R8G8B8);
+    if(!nmodes) {
+        skip("Display format X8R8G8B8 not supported, skipping\n");
+    } else {
+        trace("Testing display format X8R8G8B8\n");
+        for(i=0; x8r8g8b8_format_list[i].DisplayFormat != 0; i++)
+        {
+            hr = IDirect3D9_CheckDeviceType(d3d9, Adapter, DeviceType, x8r8g8b8_format_list[i].DisplayFormat, x8r8g8b8_format_list[i].BackBufferFormat, FALSE);
+
+            if(x8r8g8b8_format_list[i].shouldPass)
+                ok(hr == D3D_OK, "format %d %d didn't pass with hr=%#08x\n", x8r8g8b8_format_list[i].DisplayFormat, x8r8g8b8_format_list[i].BackBufferFormat, hr);
+            else
+                ok(hr != D3D_OK, "format %d %d didn't pass while it was expected to\n", x8r8g8b8_format_list[i].DisplayFormat, x8r8g8b8_format_list[i].BackBufferFormat);
+        }
+    }
+
+    if(d3d9) IDirect3D9_Release(d3d9);
+}
+
 START_TEST(device)
 {
     HMODULE d3d9_handle = LoadLibraryA( "d3d9.dll" );
@@ -1462,16 +1999,21 @@ START_TEST(device)
     ok(pDirect3DCreate9 != NULL, "Failed to get address of Direct3DCreate9\n");
     if (pDirect3DCreate9)
     {
+        test_display_formats();
         test_display_modes();
         test_swapchain();
         test_refcount();
         test_mipmap_levels();
         test_cursor();
         test_reset();
+        test_reset();
         test_scene();
         test_limits();
         test_depthstenciltest();
         test_draw_indexed();
         test_null_stream();
+        test_vertex_buffer_alignment();
+        test_lights();
+        test_set_stream_source();
     }
 }
