@@ -54,18 +54,18 @@ WINE_DEFAULT_DEBUG_CHANNEL(d3d7);
  *****************************************************************************/
 
 static void _dump_executedata(const D3DEXECUTEDATA *lpData) {
-    DPRINTF("dwSize : %d\n", lpData->dwSize);
-    DPRINTF("Vertex      Offset : %d  Count  : %d\n", lpData->dwVertexOffset, lpData->dwVertexCount);
-    DPRINTF("Instruction Offset : %d  Length : %d\n", lpData->dwInstructionOffset, lpData->dwInstructionLength);
-    DPRINTF("HVertex     Offset : %d\n", lpData->dwHVertexOffset);
+    TRACE("dwSize : %d\n", lpData->dwSize);
+    TRACE("Vertex      Offset : %d  Count  : %d\n", lpData->dwVertexOffset, lpData->dwVertexCount);
+    TRACE("Instruction Offset : %d  Length : %d\n", lpData->dwInstructionOffset, lpData->dwInstructionLength);
+    TRACE("HVertex     Offset : %d\n", lpData->dwHVertexOffset);
 }
 
 static void _dump_D3DEXECUTEBUFFERDESC(const D3DEXECUTEBUFFERDESC *lpDesc) {
-    DPRINTF("dwSize       : %d\n", lpDesc->dwSize);
-    DPRINTF("dwFlags      : %x\n", lpDesc->dwFlags);
-    DPRINTF("dwCaps       : %x\n", lpDesc->dwCaps);
-    DPRINTF("dwBufferSize : %d\n", lpDesc->dwBufferSize);
-    DPRINTF("lpData       : %p\n", lpDesc->lpData);
+    TRACE("dwSize       : %d\n", lpDesc->dwSize);
+    TRACE("dwFlags      : %x\n", lpDesc->dwFlags);
+    TRACE("dwCaps       : %x\n", lpDesc->dwCaps);
+    TRACE("dwBufferSize : %d\n", lpDesc->dwBufferSize);
+    TRACE("lpData       : %p\n", lpDesc->lpData);
 }
 
 /*****************************************************************************
@@ -317,7 +317,7 @@ IDirect3DExecuteBufferImpl_Execute(IDirect3DExecuteBufferImpl *This,
 		for (i = 0; i < count; i++) {
 		    LPD3DSTATE ci = (LPD3DSTATE) instr;
 		    
-		    IDirect3DDevice7_SetRenderState(ICOM_INTERFACE(lpDevice, IDirect3DDevice7),
+		    IDirect3DDevice2_SetRenderState(ICOM_INTERFACE(lpDevice, IDirect3DDevice2),
 						    ci->u1.drstRenderStateType, ci->u2.dwArg[0]);
 
 		    instr += size;
@@ -437,9 +437,9 @@ IDirect3DExecuteBufferImpl_Execute(IDirect3DExecuteBufferImpl *This,
 			    dst->u3.sz = (src->u1.x * mat._13) + (src->u2.y * mat._23) + (src->u3.z * mat._33) + (1.0 * mat._43);
 			    dst->u4.rhw = (src->u1.x * mat._14) + (src->u2.y * mat._24) + (src->u3.z * mat._34) + (1.0 * mat._44);
 
-			    dst->u1.sx = dst->u1.sx / dst->u4.rhw * Viewport->dwWidth / 2
+			    dst->u1.sx = dst->u1.sx / dst->u4.rhw * Viewport->dvScaleX
 				       + Viewport->dwX + Viewport->dwWidth / 2;
-			    dst->u2.sy = (-dst->u2.sy) / dst->u4.rhw * Viewport->dwHeight / 2
+			    dst->u2.sy = (-dst->u2.sy) / dst->u4.rhw * Viewport->dvScaleY
 				       + Viewport->dwY + Viewport->dwHeight / 2;
 			    dst->u3.sz /= dst->u4.rhw;
 			    dst->u4.rhw = 1 / dst->u4.rhw;
@@ -460,8 +460,8 @@ IDirect3DExecuteBufferImpl_Execute(IDirect3DExecuteBufferImpl *This,
 			    dump_D3DMATRIX(&proj_mat);
 			    TRACE("  View       Matrix : (%p)\n",&view_mat);
 			    dump_D3DMATRIX(&view_mat);
-			    TRACE("  World Matrix : (%p)\n", &mat);
-			    dump_D3DMATRIX(&mat);
+			    TRACE("  World Matrix : (%p)\n", &world_mat);
+			    dump_D3DMATRIX(&world_mat);
 			}
 
 			multiply_matrix(&mat,&view_mat,&world_mat);
@@ -479,8 +479,11 @@ IDirect3DExecuteBufferImpl_Execute(IDirect3DExecuteBufferImpl *This,
 			    dst->u3.sz = (src->u1.x * mat._13) + (src->u2.y * mat._23) + (src->u3.z * mat._33) + (1.0 * mat._43);
 			    dst->u4.rhw = (src->u1.x * mat._14) + (src->u2.y * mat._24) + (src->u3.z * mat._34) + (1.0 * mat._44);
 
-			    dst->u1.sx /= dst->u4.rhw * Viewport->dvScaleX * Viewport->dwWidth / 2 + Viewport->dwX;
-			    dst->u2.sy /= dst->u4.rhw * Viewport->dvScaleY * Viewport->dwHeight / 2 + Viewport->dwY;
+			    dst->u1.sx = dst->u1.sx / dst->u4.rhw * Viewport->dvScaleX
+				       + Viewport->dwX + Viewport->dwWidth / 2;
+			    dst->u2.sy = (-dst->u2.sy) / dst->u4.rhw * Viewport->dvScaleY
+				       + Viewport->dwY + Viewport->dwHeight / 2;
+
 			    dst->u3.sz /= dst->u4.rhw;
 			    dst->u4.rhw = 1 / dst->u4.rhw;
 
@@ -524,12 +527,18 @@ IDirect3DExecuteBufferImpl_Execute(IDirect3DExecuteBufferImpl *This,
 		    if ((This->data.dsStatus.dwStatus & ci->dwMask) == ci->dwValue) {
 		        if (!ci->bNegate) {
                             TRACE(" Branch to %d\n", ci->dwOffset);
-			    instr = (char*)current + ci->dwOffset;
+                            if (ci->dwOffset) {
+                                instr = (char*)current + ci->dwOffset;
+                                break;
+                            }
 			}
 		    } else {
 		        if (ci->bNegate) {
                             TRACE(" Branch to %d\n", ci->dwOffset);
-			    instr = (char*)current + ci->dwOffset;
+                            if (ci->dwOffset) {
+                                instr = (char*)current + ci->dwOffset;
+                                break;
+                            }
 			}
 		    }
 
