@@ -32,11 +32,9 @@
 
 #include "windef.h"
 #include "winbase.h"
-#include "winnls.h"
 #include "winerror.h"
 #include "wingdi.h"
 #include "wine/exception.h"
-#include "excpt.h"
 
 #include "ddraw.h"
 #include "d3d.h"
@@ -47,9 +45,9 @@
 WINE_DEFAULT_DEBUG_CHANNEL(d3d7);
 WINE_DECLARE_DEBUG_CHANNEL(ddraw_thunk);
 
-static void dump_material(LPD3DMATERIAL mat)
+static void dump_material(const D3DMATERIAL *mat)
 {
-    DPRINTF("  dwSize : %d\n", mat->dwSize);
+    TRACE("  dwSize : %d\n", mat->dwSize);
 }
 
 /*****************************************************************************
@@ -151,8 +149,10 @@ IDirect3DMaterialImpl_Release(IDirect3DMaterial3 *iface)
     {
         if(This->Handle)
         {
+            EnterCriticalSection(&ddraw_cs);
             This->ddraw->d3ddevice->Handles[This->Handle - 1].ptr = NULL;
             This->ddraw->d3ddevice->Handles[This->Handle - 1].type = DDrawHandle_Unknown;
+            LeaveCriticalSection(&ddraw_cs);
         }
 
         HeapFree(GetProcessHeap(), 0, This);
@@ -248,9 +248,11 @@ IDirect3DMaterialImpl_SetMaterial(IDirect3DMaterial3 *iface,
         dump_material(lpMat);
 
     /* Stores the material */
+    EnterCriticalSection(&ddraw_cs);
     memset(&This->mat, 0, sizeof(This->mat));
     memcpy(&This->mat, lpMat, lpMat->dwSize);
-    
+    LeaveCriticalSection(&ddraw_cs);
+
     return DD_OK;
 }
 
@@ -280,9 +282,11 @@ IDirect3DMaterialImpl_GetMaterial(IDirect3DMaterial3 *iface,
     }
 
     /* Copies the material structure */
+    EnterCriticalSection(&ddraw_cs);
     dwSize = lpMat->dwSize;
     memset(lpMat, 0, dwSize);
     memcpy(lpMat, &This->mat, dwSize);
+    LeaveCriticalSection(&ddraw_cs);
 
     return DD_OK;
 }
@@ -311,6 +315,7 @@ IDirect3DMaterialImpl_GetHandle(IDirect3DMaterial3 *iface,
     IDirect3DDeviceImpl *device = ICOM_OBJECT(IDirect3DDeviceImpl, IDirect3DDevice3, lpDirect3DDevice3);
     TRACE("(%p/%p)->(%p,%p)\n", This, iface, device, lpHandle);
 
+    EnterCriticalSection(&ddraw_cs);
     This->active_device = device;
     if(!This->Handle)
     {
@@ -318,6 +323,7 @@ IDirect3DMaterialImpl_GetHandle(IDirect3DMaterial3 *iface,
         if(!This->Handle)
         {
             ERR("Error creating a handle\n");
+            LeaveCriticalSection(&ddraw_cs);
             return DDERR_INVALIDPARAMS;   /* Unchecked */
         }
         device->Handles[This->Handle - 1].ptr = This;
@@ -325,6 +331,7 @@ IDirect3DMaterialImpl_GetHandle(IDirect3DMaterial3 *iface,
     }
     *lpHandle = This->Handle;
     TRACE(" returning handle %08x.\n", *lpHandle);
+    LeaveCriticalSection(&ddraw_cs);
 
     return D3D_OK;
 }
