@@ -74,7 +74,7 @@ const GUID DInput_Wine_Mouse_GUID = { /* 9e573ed8-7734-11d2-8d4a-23903fb6bdf7 */
     0x9e573ed8, 0x7734, 0x11d2, {0x8d, 0x4a, 0x23, 0x90, 0x3f, 0xb6, 0xbd, 0xf7}
 };
 
-void _dump_mouse_state(DIMOUSESTATE2 *m_state)
+static void _dump_mouse_state(DIMOUSESTATE2 *m_state)
 {
     int i;
 
@@ -173,14 +173,14 @@ static SysMouseImpl *alloc_device(REFGUID rguid, const void *mvt, IDirectInputIm
 {
     SysMouseImpl* newDevice;
     LPDIDATAFORMAT df = NULL;
-    int i;
+    unsigned i;
 
     newDevice = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(SysMouseImpl));
     if (!newDevice) return NULL;
     newDevice->base.lpVtbl = mvt;
     newDevice->base.ref = 1;
     newDevice->base.dwCoopLevel = DISCL_NONEXCLUSIVE | DISCL_BACKGROUND;
-    memcpy(&newDevice->base.guid, rguid, sizeof(*rguid));
+    newDevice->base.guid = *rguid;
     InitializeCriticalSection(&newDevice->base.crit);
     newDevice->base.crit.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": SysMouseImpl*->base.crit");
     newDevice->base.dinput = dinput;
@@ -301,7 +301,7 @@ static void dinput_mouse_hook( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARA
                 /* Already have X, need to queue it */
                 if (inst_id != -1)
                     queue_event((LPDIRECTINPUTDEVICE8A)This, id_to_offset(&This->base.data_format, inst_id),
-                                wdata, hook->time, This->base.dinput->evsequence);
+                                wdata, GetCurrentTime(), This->base.dinput->evsequence);
                 inst_id = DIDFT_MAKEINSTANCE(WINE_MOUSE_Y_AXIS_INSTANCE) | DIDFT_RELAXIS;
                 wdata = pt1.y;
             }
@@ -352,7 +352,7 @@ static void dinput_mouse_hook( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARA
     {
         _dump_mouse_state(&This->m_state);
         queue_event((LPDIRECTINPUTDEVICE8A)This, id_to_offset(&This->base.data_format, inst_id),
-                    wdata, hook->time, This->base.dinput->evsequence++);
+                    wdata, GetCurrentTime(), This->base.dinput->evsequence++);
     }
 
     LeaveCriticalSection(&This->base.crit);
@@ -410,7 +410,18 @@ static HRESULT WINAPI SysMouseAImpl_Acquire(LPDIRECTINPUTDEVICE8A iface)
     
     /* Install our mouse hook */
     if (This->base.dwCoopLevel & DISCL_EXCLUSIVE)
+    {
+      RECT rc;
+
       ShowCursor(FALSE); /* hide cursor */
+      if (GetWindowRect(This->base.win, &rc))
+      {
+        FIXME("Clipping cursor to %s\n", wine_dbgstr_rect( &rc ));
+        ClipCursor(&rc);
+      }
+      else
+        ERR("Failed to get RECT: %d\n", GetLastError());
+    }
     
     /* Get the window dimension and find the center */
     GetWindowRect(This->base.win, &rect);
@@ -446,7 +457,10 @@ static HRESULT WINAPI SysMouseAImpl_Unacquire(LPDIRECTINPUTDEVICE8A iface)
     if ((res = IDirectInputDevice2AImpl_Unacquire(iface)) != DI_OK) return res;
 
     if (This->base.dwCoopLevel & DISCL_EXCLUSIVE)
+    {
+        ClipCursor(NULL);
         ShowCursor(TRUE); /* show cursor */
+    }
 
     /* And put the mouse cursor back where it was at acquire time */
     if (This->base.dwCoopLevel & DISCL_EXCLUSIVE)
@@ -576,7 +590,7 @@ static HRESULT WINAPI SysMouseAImpl_GetProperty(LPDIRECTINPUTDEVICE8A iface,
 }
 
 /******************************************************************************
-  *     GetCapabilities : get the device capablitites
+  *     GetCapabilities : get the device capabilities
   */
 static HRESULT WINAPI SysMouseAImpl_GetCapabilities(
 	LPDIRECTINPUTDEVICE8A iface,
