@@ -3,27 +3,30 @@
  * Copyright (C) 2004 Rok Mandeljc
  * Copyright (C) 2004 Raphael Junqueira
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
- 
+
+#define COBJMACROS
+
+#include "config.h"
+#include "wine/port.h"
+
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
 
-#define COBJMACROS
-
 #include "windef.h"
 #include "winbase.h"
 #include "winnt.h"
@@ -31,9 +34,7 @@
 #include "winuser.h"
 
 #include "wine/debug.h"
-#include "wine/list.h"
 #include "wine/unicode.h"
-#include "winreg.h"
 #include "objbase.h"
 
 #include "dmusici.h"
@@ -116,7 +117,7 @@ static const char *debugstr_month (DWORD dwMonth) {
 }
 
 /* FILETIME struct to string conversion for debug messages */
-static const char *debugstr_filetime (LPFILETIME time) {
+static const char *debugstr_filetime (const FILETIME *time) {
 	SYSTEMTIME sysTime;
 
 	if (!time) return "'null'";
@@ -129,7 +130,7 @@ static const char *debugstr_filetime (LPFILETIME time) {
 }
 
 /* DMUS_VERSION struct to string conversion for debug messages */
-static const char *debugstr_dmversion (LPDMUS_VERSION version) {
+static const char *debugstr_dmversion (const DMUS_VERSION *version) {
 	if (!version) return "'null'";
 	return wine_dbg_sprintf ("\'%i,%i,%i,%i\'",
 		HIWORD(version->dwVersionMS),LOWORD(version->dwVersionMS),
@@ -311,9 +312,9 @@ HRESULT IDirectMusicUtils_IPersistStream_ParseReference (LPPERSISTSTREAM iface,
     }
     TRACE_(dmfile)(": ListCount[0] = %d < ListSize[0] = %d\n", ListCount[0], ListSize[0]);
   } while (ListCount[0] < ListSize[0]);
-  
+
   ref_desc.dwValidData |= DMUS_OBJ_CLASS;
-  memcpy(&ref_desc.guidClass, &ref.guidClassID, sizeof(ref.guidClassID));
+  ref_desc.guidClass = ref.guidClassID;
 
   TRACE_(dmfile)("** DM Reference Begin of Load ***\n");
   TRACE_(dmfile)("With Desc:\n");
