@@ -20,9 +20,9 @@
  *
  * NOTES: 
  *  o Implementation of all things which are associated with dplay on
- *    the computer - ie shared resources and such. Methods in this
- *    compilation unit should not call anything out side this unit
- *    excepting base windows services and an interface to start the
+ *    the computer - i.e. shared resources and such. Methods in this
+ *    compilation unit should not call anything outside of this unit
+ *    except base windows services and an interface to start the
  *    messaging thread.
  *  o Methods that begin with DPLAYX_ are used for dealing with
  *    dplayx.dll data which is accessible from all processes.
@@ -199,10 +199,10 @@ static DPSESSIONDESC2* sessionData = NULL;
 /* static DPSESSIONDESC2* sessionData[ numSupportedSessions ]; */
 
 /* Function prototypes */
-static DWORD DPLAYX_SizeOfLobbyDataA( LPDPLCONNECTION lpDplData );
-static DWORD DPLAYX_SizeOfLobbyDataW( LPDPLCONNECTION lpDplData );
-static void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, LPDPLCONNECTION src );
-static void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src );
+static DWORD DPLAYX_SizeOfLobbyDataA( const DPLCONNECTION *lpDplData );
+static DWORD DPLAYX_SizeOfLobbyDataW( const DPLCONNECTION *lpDplData );
+static void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, const DPLCONNECTION *src );
+static void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, const DPLCONNECTION *src );
 static BOOL DPLAYX_IsAppIdLobbied( DWORD dwAppId, LPDPLAYX_LOBBYDATA* dplData );
 static void DPLAYX_InitializeLobbyDataEntry( LPDPLAYX_LOBBYDATA lpData );
 static BOOL DPLAYX_CopyIntoSessionDesc2A( LPDPSESSIONDESC2  lpSessionDest,
@@ -333,7 +333,7 @@ BOOL DPLAYX_ConstructData(void)
       sessionData[i].dwSize = 0;
     }
 
-    /* Zero out the dynmaic area */
+    /* Zero out the dynamic area */
     ZeroMemory( lpMemArea, dwDynamicSharedSize );
 
     /* Just for fun sync the whole data area */
@@ -433,7 +433,7 @@ BOOL DPLAYX_IsAppIdLobbied( DWORD dwAppID, LPDPLAYX_LOBBYDATA* lplpDplData )
   return FALSE;
 }
 
-/* Reserve a spot for the new appliction. TRUE means success and FALSE failure.  */
+/* Reserve a spot for the new application. TRUE means success and FALSE failure.  */
 BOOL DPLAYX_CreateLobbyApplication( DWORD dwAppID )
 {
   UINT i;
@@ -506,7 +506,7 @@ BOOL DPLAYX_SetLobbyHandles( DWORD dwAppID,
 {
   LPDPLAYX_LOBBYDATA lpLData;
 
-  /* Need to explictly give lobby application. Can't set for yourself */
+  /* Need to explicitly give lobby application. Can't set for yourself */
   if( dwAppID == 0 )
   {
     return FALSE;
@@ -658,11 +658,11 @@ HRESULT DPLAYX_GetConnectionSettingsA
 }
 
 /* Assumption: Enough contiguous space was allocated at dest */
-void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, LPDPLCONNECTION src )
+void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, const DPLCONNECTION *src )
 {
   BYTE* lpStartOfFreeSpace;
 
-  CopyMemory( dest, src, sizeof( DPLCONNECTION ) );
+  *dest = *src;
 
   lpStartOfFreeSpace = ((BYTE*)dest) + sizeof( DPLCONNECTION );
 
@@ -671,7 +671,7 @@ void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, LPDPLCONNECTION src )
   {
     dest->lpSessionDesc = (LPDPSESSIONDESC2)lpStartOfFreeSpace;
     lpStartOfFreeSpace += sizeof( DPSESSIONDESC2 );
-    CopyMemory( dest->lpSessionDesc, src->lpSessionDesc, sizeof( DPSESSIONDESC2 ) );
+    *dest->lpSessionDesc = *src->lpSessionDesc;
 
     /* Session names may or may not exist */
     if( src->lpSessionDesc->u1.lpszSessionNameA )
@@ -679,7 +679,7 @@ void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, LPDPLCONNECTION src )
       strcpy( (LPSTR)lpStartOfFreeSpace, src->lpSessionDesc->u1.lpszSessionNameA );
       dest->lpSessionDesc->u1.lpszSessionNameA = (LPSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=
-        strlen( (LPSTR)dest->lpSessionDesc->u1.lpszSessionNameA ) + 1;
+        strlen( dest->lpSessionDesc->u1.lpszSessionNameA ) + 1;
     }
 
     if( src->lpSessionDesc->u2.lpszPasswordA )
@@ -687,7 +687,7 @@ void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, LPDPLCONNECTION src )
       strcpy( (LPSTR)lpStartOfFreeSpace, src->lpSessionDesc->u2.lpszPasswordA );
       dest->lpSessionDesc->u2.lpszPasswordA = (LPSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=
-        strlen( (LPSTR)dest->lpSessionDesc->u2.lpszPasswordA ) + 1;
+        strlen( dest->lpSessionDesc->u2.lpszPasswordA ) + 1;
     }
   }
 
@@ -696,14 +696,14 @@ void DPLAYX_CopyConnStructA( LPDPLCONNECTION dest, LPDPLCONNECTION src )
   {
     dest->lpPlayerName = (LPDPNAME)lpStartOfFreeSpace;
     lpStartOfFreeSpace += sizeof( DPNAME );
-    CopyMemory( dest->lpPlayerName, src->lpPlayerName, sizeof( DPNAME ) );
+    *dest->lpPlayerName = *src->lpPlayerName;
 
     if( src->lpPlayerName->u1.lpszShortNameA )
     {
       strcpy( (LPSTR)lpStartOfFreeSpace, src->lpPlayerName->u1.lpszShortNameA );
       dest->lpPlayerName->u1.lpszShortNameA = (LPSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=
-        strlen( (LPSTR)dest->lpPlayerName->u1.lpszShortNameA ) + 1;
+        strlen( dest->lpPlayerName->u1.lpszShortNameA ) + 1;
     }
 
     if( src->lpPlayerName->u2.lpszLongNameA )
@@ -780,11 +780,11 @@ HRESULT DPLAYX_GetConnectionSettingsW
 }
 
 /* Assumption: Enough contiguous space was allocated at dest */
-void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src )
+void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, const DPLCONNECTION *src )
 {
   BYTE*              lpStartOfFreeSpace;
 
-  CopyMemory( dest, src, sizeof( DPLCONNECTION ) );
+  *dest = *src;
 
   lpStartOfFreeSpace = ( (BYTE*)dest) + sizeof( DPLCONNECTION );
 
@@ -793,15 +793,15 @@ void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src )
   {
     dest->lpSessionDesc = (LPDPSESSIONDESC2)lpStartOfFreeSpace;
     lpStartOfFreeSpace += sizeof( DPSESSIONDESC2 );
-    CopyMemory( dest->lpSessionDesc, src->lpSessionDesc, sizeof( DPSESSIONDESC2 ) );
+    *dest->lpSessionDesc = *src->lpSessionDesc;
 
     /* Session names may or may not exist */
     if( src->lpSessionDesc->u1.lpszSessionName )
     {
-      strcpyW( (LPWSTR)lpStartOfFreeSpace, dest->lpSessionDesc->u1.lpszSessionName );
-      src->lpSessionDesc->u1.lpszSessionName = (LPWSTR)lpStartOfFreeSpace;
+      strcpyW( (LPWSTR)lpStartOfFreeSpace, src->lpSessionDesc->u1.lpszSessionName );
+      dest->lpSessionDesc->u1.lpszSessionName = (LPWSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=  sizeof(WCHAR) *
-        ( strlenW( (LPWSTR)dest->lpSessionDesc->u1.lpszSessionName ) + 1 );
+        ( strlenW( dest->lpSessionDesc->u1.lpszSessionName ) + 1 );
     }
 
     if( src->lpSessionDesc->u2.lpszPassword )
@@ -809,7 +809,7 @@ void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src )
       strcpyW( (LPWSTR)lpStartOfFreeSpace, src->lpSessionDesc->u2.lpszPassword );
       dest->lpSessionDesc->u2.lpszPassword = (LPWSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=  sizeof(WCHAR) *
-        ( strlenW( (LPWSTR)dest->lpSessionDesc->u2.lpszPassword ) + 1 );
+        ( strlenW( dest->lpSessionDesc->u2.lpszPassword ) + 1 );
     }
   }
 
@@ -818,14 +818,14 @@ void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src )
   {
     dest->lpPlayerName = (LPDPNAME)lpStartOfFreeSpace;
     lpStartOfFreeSpace += sizeof( DPNAME );
-    CopyMemory( dest->lpPlayerName, src->lpPlayerName, sizeof( DPNAME ) );
+    *dest->lpPlayerName = *src->lpPlayerName;
 
     if( src->lpPlayerName->u1.lpszShortName )
     {
       strcpyW( (LPWSTR)lpStartOfFreeSpace, src->lpPlayerName->u1.lpszShortName );
       dest->lpPlayerName->u1.lpszShortName = (LPWSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=  sizeof(WCHAR) *
-        ( strlenW( (LPWSTR)dest->lpPlayerName->u1.lpszShortName ) + 1 );
+        ( strlenW( dest->lpPlayerName->u1.lpszShortName ) + 1 );
     }
 
     if( src->lpPlayerName->u2.lpszLongName )
@@ -833,7 +833,7 @@ void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src )
       strcpyW( (LPWSTR)lpStartOfFreeSpace, src->lpPlayerName->u2.lpszLongName );
       dest->lpPlayerName->u2.lpszLongName = (LPWSTR)lpStartOfFreeSpace;
       lpStartOfFreeSpace +=  sizeof(WCHAR) *
-        ( strlenW( (LPWSTR)dest->lpPlayerName->u2.lpszLongName ) + 1 );
+        ( strlenW( dest->lpPlayerName->u2.lpszLongName ) + 1 );
     }
 
   }
@@ -848,14 +848,14 @@ void DPLAYX_CopyConnStructW( LPDPLCONNECTION dest, LPDPLCONNECTION src )
 
 }
 
-/* Store the structure into the shared data structre. Ensure that allocs for
+/* Store the structure into the shared data structure. Ensure that allocs for
  * variable length strings come from the shared data structure.
- * FIXME: We need to free information as well
+ * FIXME: We need to free information as well.
  */
 HRESULT DPLAYX_SetConnectionSettingsA
 ( DWORD dwFlags,
   DWORD dwAppID,
-  LPDPLCONNECTION lpConn )
+  const DPLCONNECTION *lpConn )
 {
   LPDPLAYX_LOBBYDATA lpDplData;
 
@@ -917,7 +917,7 @@ HRESULT DPLAYX_SetConnectionSettingsA
 HRESULT DPLAYX_SetConnectionSettingsW
 ( DWORD dwFlags,
   DWORD dwAppID,
-  LPDPLCONNECTION lpConn )
+  const DPLCONNECTION *lpConn )
 {
   LPDPLAYX_LOBBYDATA lpDplData;
 
@@ -961,7 +961,7 @@ HRESULT DPLAYX_SetConnectionSettingsW
   return DP_OK;
 }
 
-DWORD DPLAYX_SizeOfLobbyDataA( LPDPLCONNECTION lpConn )
+DWORD DPLAYX_SizeOfLobbyDataA( const DPLCONNECTION *lpConn )
 {
   DWORD dwTotalSize = sizeof( DPLCONNECTION );
 
@@ -1008,7 +1008,7 @@ DWORD DPLAYX_SizeOfLobbyDataA( LPDPLCONNECTION lpConn )
   return dwTotalSize;
 }
 
-DWORD DPLAYX_SizeOfLobbyDataW( LPDPLCONNECTION lpConn )
+DWORD DPLAYX_SizeOfLobbyDataW( const DPLCONNECTION *lpConn )
 {
   DWORD dwTotalSize = sizeof( DPLCONNECTION );
 
