@@ -27,7 +27,6 @@
 #include "winbase.h"
 #include "winuser.h"
 #include "mmsystem.h"
-#include "winreg.h"
 #include "winternl.h"
 #include "wine/debug.h"
 #include "dsound.h"
@@ -217,11 +216,14 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetVolume(
 	}
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceExclusive(&This->lock, TRUE);
 
 	if (This->dsbd.dwFlags & DSBCAPS_CTRL3D) {
 		oldVol = This->ds3db_lVolume;
 		This->ds3db_lVolume = vol;
+		if (vol != oldVol)
+			/* recalc 3d volume, which in turn recalcs the pans */
+			DSOUND_Calc3DBuffer(This);
 	} else {
 		oldVol = This->volpan.lVolume;
 		This->volpan.lVolume = vol;
@@ -234,11 +236,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetVolume(
 			hres = IDsDriverBuffer_SetVolumePan(This->hwbuf, &(This->volpan));
 	    		if (hres != DS_OK)
 		    		WARN("IDsDriverBuffer_SetVolumePan failed\n");
-		} else
-			DSOUND_ForceRemix(This);
+		}
 	}
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
 	return hres;
@@ -287,19 +288,18 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetFrequency(
 	}
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceExclusive(&This->lock, TRUE);
 
 	oldFreq = This->freq;
 	This->freq = freq;
 	if (freq != oldFreq) {
-		This->freqAdjust = (freq << DSOUND_FREQSHIFT) / This->device->pwfx->nSamplesPerSec;
+		This->freqAdjust = ((DWORD64)This->freq << DSOUND_FREQSHIFT) / This->device->pwfx->nSamplesPerSec;
 		This->nAvgBytesPerSec = freq * This->pwfx->nBlockAlign;
 		DSOUND_RecalcFormat(This);
-		if (!This->hwbuf)
-			DSOUND_ForceRemix(This);
+		DSOUND_MixToTemporary(This, 0, This->buflen, FALSE);
 	}
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
 	return DS_OK;
@@ -313,12 +313,11 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Play(
 	TRACE("(%p,%08x,%08x,%08x)\n",This,reserved1,reserved2,flags);
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceExclusive(&This->lock, TRUE);
 
 	This->playflags = flags;
-	if (This->state == STATE_STOPPED) {
+	if (This->state == STATE_STOPPED && !This->hwbuf) {
 		This->leadin = TRUE;
-		This->startpos = This->buf_mixpos;
 		This->state = STATE_STARTING;
 	} else if (This->state == STATE_STOPPING)
 		This->state = STATE_PLAYING;
@@ -330,7 +329,7 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Play(
 			This->state = STATE_PLAYING;
 	}
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
 	return hres;
@@ -343,12 +342,15 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Stop(LPDIRECTSOUNDBUFFER8 iface)
 	TRACE("(%p)\n",This);
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceExclusive(&This->lock, TRUE);
 
 	if (This->state == STATE_PLAYING)
 		This->state = STATE_STOPPING;
 	else if (This->state == STATE_STARTING)
+	{
 		This->state = STATE_STOPPED;
+		DSOUND_CheckEvent(This, 0, 0);
+	}
 	if (This->hwbuf) {
 		hres = IDsDriverBuffer_Stop(This->hwbuf);
 		if (hres != DS_OK)
@@ -356,9 +358,8 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Stop(LPDIRECTSOUNDBUFFER8 iface)
 		else
 			This->state = STATE_STOPPED;
 	}
-	DSOUND_CheckEvent(This, 0);
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
 	return hres;
@@ -380,14 +381,13 @@ static ULONG WINAPI IDirectSoundBufferImpl_Release(LPDIRECTSOUNDBUFFER8 iface)
 
     if (!ref) {
 	DirectSoundDevice_RemoveBuffer(This->device, This);
-
-	This->lock.DebugInfo->Spare[0] = 0;
-	DeleteCriticalSection(&(This->lock));
+	RtlDeleteResource(&This->lock);
 
 	if (This->hwbuf) {
 		IDsDriverBuffer_Release(This->hwbuf);
 		if (This->device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY) {
 			This->buffer->ref--;
+			list_remove(&This->entry);
 			if (This->buffer->ref==0) {
 				HeapFree(GetProcessHeap(),0,This->buffer->memory);
 				HeapFree(GetProcessHeap(),0,This->buffer);
@@ -395,12 +395,14 @@ static ULONG WINAPI IDirectSoundBufferImpl_Release(LPDIRECTSOUNDBUFFER8 iface)
 		}
 	} else {
 		This->buffer->ref--;
+		list_remove(&This->entry);
 		if (This->buffer->ref==0) {
 			HeapFree(GetProcessHeap(),0,This->buffer->memory);
 			HeapFree(GetProcessHeap(),0,This->buffer);
 		}
 	}
 
+	HeapFree(GetProcessHeap(), 0, This->tmp_buffer);
 	HeapFree(GetProcessHeap(), 0, This->notifies);
 	HeapFree(GetProcessHeap(), 0, This->pwfx);
 	HeapFree(GetProcessHeap(), 0, This);
@@ -410,51 +412,14 @@ static ULONG WINAPI IDirectSoundBufferImpl_Release(LPDIRECTSOUNDBUFFER8 iface)
     return ref;
 }
 
-DWORD DSOUND_CalcPlayPosition(IDirectSoundBufferImpl *This, DWORD pplay, DWORD pwrite)
-{
-	DWORD bplay = This->buf_mixpos;
-	DWORD pmix = This->primary_mixpos;
-        DirectSoundDevice * device = This->device;
-	TRACE("(%p, pplay=%u, pwrite=%u)\n", This, pplay, pwrite);
-
-	/* the actual primary play position (pplay) is always behind last mixed (pmix),
-	 * unless the computer is too slow or something */
-	/* we need to know how far away we are from there */
-	if (pmix < pplay) pmix += device->buflen; /* wraparound */
-	pmix -= pplay;
-	/* detect buffer underrun */
-	if (pwrite < pplay) pwrite += device->buflen; /* wraparound */
-	pwrite -= pplay;
-	if (pmix > (ds_snd_queue_max * device->fraglen + pwrite + device->writelead)) {
-		WARN("detected an underrun: primary queue was %d\n",pmix);
-		pmix = 0;
-	}
-	/* divide the offset by its sample size */
-	pmix /= device->pwfx->nBlockAlign;
-	TRACE("primary back-samples=%d\n",pmix);
-	/* adjust for our frequency */
-	pmix = (pmix * This->freqAdjust) >> DSOUND_FREQSHIFT;
-	/* multiply by our own sample size */
-	pmix *= This->pwfx->nBlockAlign;
-	TRACE("this back-offset=%d\n", pmix);
-	/* subtract from our last mixed position */
-	while (bplay < pmix) bplay += This->buflen; /* wraparound */
-	bplay -= pmix;
-	if (This->leadin && ((bplay < This->startpos) || (bplay > This->buf_mixpos))) {
-		/* seems we haven't started playing yet */
-		TRACE("this still in lead-in phase\n");
-		bplay = This->startpos;
-	}
-	/* return the result */
-	return bplay;
-}
-
 static HRESULT WINAPI IDirectSoundBufferImpl_GetCurrentPosition(
 	LPDIRECTSOUNDBUFFER8 iface,LPDWORD playpos,LPDWORD writepos
 ) {
 	HRESULT	hres;
 	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
 	TRACE("(%p,%p,%p)\n",This,playpos,writepos);
+
+	RtlAcquireResourceShared(&This->lock, TRUE);
 	if (This->hwbuf) {
 		hres=IDsDriverBuffer_GetPosition(This->hwbuf,playpos,writepos);
 		if (hres != DS_OK) {
@@ -462,47 +427,29 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetCurrentPosition(
 		    return hres;
 		}
 	} else {
-		if (playpos && (This->state != STATE_PLAYING)) {
-			/* we haven't been merged into the primary buffer (yet) */
-			*playpos = This->buf_mixpos;
-		} else if (playpos) {
-			DWORD pplay, pwrite;
-			/* let's get this exact; first, recursively call GetPosition on the primary */
-			EnterCriticalSection(&(This->device->mixlock));
-			if (DSOUND_PrimaryGetPosition(This->device, &pplay, &pwrite) != DS_OK)
-				WARN("DSOUND_PrimaryGetPosition failed\n");
-			/* detect HEL mode underrun */
-			if (!(This->device->hwbuf || This->device->pwqueue))
-				TRACE("detected an underrun\n");
-			if ((This->dsbd.dwFlags & DSBCAPS_GETCURRENTPOSITION2) || This->device->hwbuf) {
-				/* calculate play position using this */
-				*playpos = DSOUND_CalcPlayPosition(This, pplay, pwrite);
-			} else {
-				/* (unless the app isn't using GETCURRENTPOSITION2) */
-				/* don't know exactly how this should be handled...
-				 * the docs says that play cursor is reported as directly
-				 * behind write cursor, hmm... */
-				/* let's just do what might work for Half-Life */
-				DWORD wp;
-				wp = (This->device->pwplay + ds_hel_margin) * This->device->fraglen;
-				wp %= This->device->buflen;
-				*playpos = DSOUND_CalcPlayPosition(This, wp, pwrite);
-			}
-			LeaveCriticalSection(&(This->device->mixlock));
+		DWORD pos = This->sec_mixpos;
+
+		/* sanity */
+		if (pos >= This->buflen){
+			FIXME("Bad play position. playpos: %d, buflen: %d\n", pos, This->buflen);
+			pos %= This->buflen;
 		}
+
+		if (playpos)
+			*playpos = pos;
 		if (writepos)
-                    *writepos = This->buf_mixpos;
+			*writepos = pos;
 	}
-	if (writepos) {
-		if (This->state != STATE_STOPPED) {
-			/* apply the documented 10ms lead to writepos */
-			*writepos += This->writelead;
-		}
+	if (writepos && This->state != STATE_STOPPED && (!This->hwbuf || !(This->device->drvdesc.dwFlags & DSDDESC_DONTNEEDWRITELEAD))) {
+		/* apply the documented 10ms lead to writepos */
+		*writepos += This->writelead;
 		*writepos %= This->buflen;
 	}
-	if (playpos)
-            This->last_playpos = *playpos;
-	TRACE("playpos = %d, writepos = %d (%p, time=%d)\n", playpos?*playpos:0, writepos?*writepos:0, This, GetTickCount());
+	RtlReleaseResource(&This->lock);
+
+	TRACE("playpos = %d, writepos = %d, buflen=%d (%p, time=%d)\n",
+		playpos?*playpos:-1, writepos?*writepos:-1, This->buflen, This, GetTickCount());
+
 	return DS_OK;
 }
 
@@ -548,8 +495,9 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetFormat(
                 *wfwritten = size;
         } else {
             WARN("invalid parameter: wfsize too small\n");
+            CopyMemory(lpwf,This->pwfx,wfsize);
             if (wfwritten)
-                *wfwritten = 0;
+                *wfwritten = wfsize;
             return DSERR_INVALIDPARAM;
         }
     } else {
@@ -608,18 +556,8 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Lock(
 		return DSERR_INVALIDPARAM;
         }
 
-	EnterCriticalSection(&(This->lock));
-
-	if ((writebytes == This->buflen) &&
-	    ((This->state == STATE_STARTING) ||
-	     (This->state == STATE_PLAYING)))
-		/* some games, like Half-Life, try to be clever (not) and
-		 * keep one secondary buffer, and mix sounds into it itself,
-		 * locking the entire buffer every time... so we can just forget
-		 * about tracking the last-written-to-position... */
-		This->probably_valid_to = (DWORD)-1;
-	else
-		This->probably_valid_to = writecursor;
+	/* **** */
+	RtlAcquireResourceShared(&This->lock, TRUE);
 
 	if (!(This->device->drvdesc.dwFlags & DSDDESC_DONTNEEDSECONDARYLOCK) && This->hwbuf) {
 		hres = IDsDriverBuffer_Lock(This->hwbuf,
@@ -629,50 +567,40 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Lock(
 				     0);
 		if (hres != DS_OK) {
 			WARN("IDsDriverBuffer_Lock failed\n");
-			LeaveCriticalSection(&(This->lock));
+			RtlReleaseResource(&This->lock);
 			return hres;
 		}
 	} else {
-		BOOL remix = FALSE;
 		if (writecursor+writebytes <= This->buflen) {
 			*(LPBYTE*)lplpaudioptr1 = This->buffer->memory+writecursor;
+			if (This->sec_mixpos >= writecursor && This->sec_mixpos < writecursor + writebytes && This->state == STATE_PLAYING)
+				WARN("Overwriting mixing position, case 1\n");
 			*audiobytes1 = writebytes;
 			if (lplpaudioptr2)
 				*(LPBYTE*)lplpaudioptr2 = NULL;
 			if (audiobytes2)
 				*audiobytes2 = 0;
+			TRACE("Locked %p(%i bytes) and %p(%i bytes) writecursor=%d\n",
+			  *(LPBYTE*)lplpaudioptr1, *audiobytes1, lplpaudioptr2 ? *(LPBYTE*)lplpaudioptr2 : NULL, audiobytes2 ? *audiobytes2: 0, writecursor);
 			TRACE("->%d.0\n",writebytes);
 		} else {
+			DWORD remainder = writebytes + writecursor - This->buflen;
 			*(LPBYTE*)lplpaudioptr1 = This->buffer->memory+writecursor;
 			*audiobytes1 = This->buflen-writecursor;
+			if (This->sec_mixpos >= writecursor && This->sec_mixpos < writecursor + writebytes && This->state == STATE_PLAYING)
+				WARN("Overwriting mixing position, case 2\n");
 			if (lplpaudioptr2)
 				*(LPBYTE*)lplpaudioptr2 = This->buffer->memory;
 			if (audiobytes2)
 				*audiobytes2 = writebytes-(This->buflen-writecursor);
-			TRACE("->%d.%d\n",*audiobytes1,audiobytes2?*audiobytes2:0);
-		}
-		if (This->state == STATE_PLAYING) {
-			/* if the segment between playpos and buf_mixpos is touched,
-			 * we need to cancel some mixing */
-			/* we'll assume that the app always calls GetCurrentPosition before
-			 * locking a playing buffer, so that last_playpos is up-to-date */
-			if (This->buf_mixpos >= This->last_playpos) {
-				if (This->buf_mixpos > writecursor &&
-				    This->last_playpos < writecursor+writebytes)
-					remix = TRUE;
-			} else {
-				if (This->buf_mixpos > writecursor ||
-				    This->last_playpos < writecursor+writebytes)
-					remix = TRUE;
-			}
-			if (remix) {
-				TRACE("locking prebuffered region, ouch\n");
-				DSOUND_MixCancelAt(This, writecursor);
-			}
+			if (audiobytes2 && This->sec_mixpos < remainder && This->state == STATE_PLAYING)
+				WARN("Overwriting mixing position, case 3\n");
+			TRACE("Locked %p(%i bytes) and %p(%i bytes) writecursor=%d\n", *(LPBYTE*)lplpaudioptr1, *audiobytes1, lplpaudioptr2 ? *(LPBYTE*)lplpaudioptr2 : NULL, audiobytes2 ? *audiobytes2: 0, writecursor);
 		}
 	}
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
+	/* **** */
 
 	return DS_OK;
 }
@@ -682,20 +610,33 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetCurrentPosition(
 ) {
 	HRESULT hres = DS_OK;
 	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
+	DWORD oldpos;
 	TRACE("(%p,%d)\n",This,newpos);
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceExclusive(&This->lock, TRUE);
 
+	oldpos = This->sec_mixpos;
+
+	/* start mixing from this new location instead */
 	newpos %= This->buflen;
-	This->buf_mixpos = newpos;
+	newpos -= newpos%This->pwfx->nBlockAlign;
+	This->sec_mixpos = newpos;
+
+	/* at this point, do not attempt to reset buffers, mess with primary mix position,
+           or anything like that to reduce latancy. The data already prebuffered cannot be changed */
+
+	/* position HW buffer if applicable, else just start mixing from new location instead */
 	if (This->hwbuf) {
 		hres = IDsDriverBuffer_SetPosition(This->hwbuf, This->buf_mixpos);
 		if (hres != DS_OK)
 			WARN("IDsDriverBuffer_SetPosition failed\n");
 	}
+	else if (oldpos != newpos)
+		/* FIXME: Perhaps add a call to DSOUND_MixToTemporary here? Not sure it's needed */
+		This->buf_mixpos = DSOUND_secpos_to_bufpos(This, newpos, 0, NULL);
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
 	return hres;
@@ -722,7 +663,7 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetPan(
 	}
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceExclusive(&This->lock, TRUE);
 
 	if (This->volpan.lPan != pan) {
 		This->volpan.lPan = pan;
@@ -732,11 +673,10 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetPan(
 			hres = IDsDriverBuffer_SetVolumePan(This->hwbuf, &(This->volpan));
 			if (hres != DS_OK)
 				WARN("IDsDriverBuffer_SetVolumePan failed\n");
-		} else
-			DSOUND_ForceRemix(This);
+		}
 	}
 
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
 	return hres;
@@ -766,14 +706,13 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetPan(
 static HRESULT WINAPI IDirectSoundBufferImpl_Unlock(
 	LPDIRECTSOUNDBUFFER8 iface,LPVOID p1,DWORD x1,LPVOID p2,DWORD x2
 ) {
-	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
-	DWORD probably_valid_to;
+	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface, *iter;
 	HRESULT hres = DS_OK;
 
 	TRACE("(%p,%p,%d,%p,%d)\n", This,p1,x1,p2,x2);
 
 	/* **** */
-	EnterCriticalSection(&(This->lock));
+	RtlAcquireResourceShared(&This->lock, TRUE);
 
 	if (!(This->device->drvdesc.dwFlags & DSDDESC_DONTNEEDSECONDARYLOCK) && This->hwbuf) {
 		hres = IDsDriverBuffer_Unlock(This->hwbuf, p1, x1, p2, x2);
@@ -781,22 +720,27 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Unlock(
 			WARN("IDsDriverBuffer_Unlock failed\n");
 	}
 
-        if (hres == DS_OK) {
-		if (p2) probably_valid_to = (((LPBYTE)p2)-This->buffer->memory) + x2;
-		else probably_valid_to = (((LPBYTE)p1)-This->buffer->memory) + x1;
-		probably_valid_to %= This->buflen;
-		if ((probably_valid_to == 0) && ((x1+x2) == This->buflen) &&
-		    ((This->state == STATE_STARTING) ||
-		     (This->state == STATE_PLAYING)))
-			/* see IDirectSoundBufferImpl_Lock */
-			probably_valid_to = (DWORD)-1;
-		This->probably_valid_to = probably_valid_to;
-	}
-
-	LeaveCriticalSection(&(This->lock));
+	RtlReleaseResource(&This->lock);
 	/* **** */
 
-	TRACE("probably_valid_to=%d\n", This->probably_valid_to);
+	if (!p2)
+		x2 = 0;
+
+	if (!This->hwbuf && (x1 || x2))
+	{
+		RtlAcquireResourceShared(&This->device->buffer_list_lock, TRUE);
+		LIST_FOR_EACH_ENTRY(iter, &This->buffer->buffers, IDirectSoundBufferImpl, entry )
+		{
+			RtlAcquireResourceShared(&iter->lock, TRUE);
+			if (x1)
+				DSOUND_MixToTemporary(iter, (DWORD_PTR)p1 - (DWORD_PTR)iter->buffer->memory, x1, FALSE);
+			if (x2)
+				DSOUND_MixToTemporary(iter, 0, x2, FALSE);
+			RtlReleaseResource(&iter->lock);
+		}
+		RtlReleaseResource(&This->device->buffer_list_lock);
+	}
+
 	return hres;
 }
 
@@ -870,9 +814,7 @@ static HRESULT WINAPI IDirectSoundBufferImpl_Initialize(
 	LPDIRECTSOUNDBUFFER8 iface,LPDIRECTSOUND dsound,LPCDSBUFFERDESC dbsd
 ) {
 	IDirectSoundBufferImpl *This = (IDirectSoundBufferImpl *)iface;
-	FIXME("(%p,%p,%p):stub\n",This,dsound,dbsd);
-	DPRINTF("Re-Init!!!\n");
-	WARN("already initialized\n");
+	WARN("(%p) already initialized\n", This);
 	return DSERR_ALREADYINITIALIZED;
 }
 
@@ -898,11 +840,8 @@ static HRESULT WINAPI IDirectSoundBufferImpl_GetCaps(
 
 	caps->dwBufferBytes = This->buflen;
 
-	/* This value represents the speed of the "unlock" command.
-	   As unlock is quite fast (it does not do anything), I put
-	   4096 ko/s = 4 Mo / s */
-	/* FIXME: hwbuf speed */
-	caps->dwUnlockTransferRate = 4096;
+	/* According to windows, this is zero*/
+	caps->dwUnlockTransferRate = 0;
 	caps->dwPlayCpuOverhead = 0;
 
 	return DS_OK;
@@ -1083,8 +1022,19 @@ HRESULT IDirectSoundBufferImpl_Create(
 	if (wfex->wBitsPerSample==16) capf |= DSCAPS_SECONDARY16BIT;
 	else capf |= DSCAPS_SECONDARY8BIT;
 
-	use_hw = (device->drvcaps.dwFlags & capf) == capf;
-	TRACE("use_hw = 0x%08x, capf = 0x%08x, device->drvcaps.dwFlags = 0x%08x\n", use_hw, capf, device->drvcaps.dwFlags);
+	use_hw = !!(dsbd->dwFlags & DSBCAPS_LOCHARDWARE);
+	TRACE("use_hw = %d, capf = 0x%08x, device->drvcaps.dwFlags = 0x%08x\n", use_hw, capf, device->drvcaps.dwFlags);
+	if (use_hw && ((device->drvcaps.dwFlags & capf) != capf || !device->driver))
+	{
+		if (device->driver)
+			WARN("Format not supported for hardware buffer\n");
+		HeapFree(GetProcessHeap(),0,dsb->pwfx);
+		HeapFree(GetProcessHeap(),0,dsb);
+		*pdsb = NULL;
+		if ((device->drvcaps.dwFlags & capf) != capf)
+			return DSERR_BADFORMAT;
+		return DSERR_GENERIC;
+	}
 
 	/* FIXME: check hardware sample rate mixing capabilities */
 	/* FIXME: check app hints for software/hardware buffer (STATIC, LOCHARDWARE, etc) */
@@ -1112,8 +1062,6 @@ HRESULT IDirectSoundBufferImpl_Create(
 			*pdsb = NULL;
 			return DSERR_OUTOFMEMORY;
 		}
-		dsb->buffer->ref = 1;
-		FillMemory(dsb->buffer->memory, dsb->buflen, dsbd->lpwfxFormat->wBitsPerSample == 8 ? 128 : 0);
 	}
 
 	/* Allocate the hardware buffer */
@@ -1121,41 +1069,36 @@ HRESULT IDirectSoundBufferImpl_Create(
 		err = IDsDriver_CreateSoundBuffer(device->driver,wfex,dsbd->dwFlags,0,
 						  &(dsb->buflen),&(dsb->buffer->memory),
 						  (LPVOID*)&(dsb->hwbuf));
-                /* fall back to software buffer on failure */
-		if (err != DS_OK) {
-			TRACE("IDsDriver_CreateSoundBuffer failed, falling back to software buffer\n");
-			use_hw = 0;
-			if (device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY) {
-				dsb->buffer->memory = HeapAlloc(GetProcessHeap(),0,dsb->buflen);
-				if (dsb->buffer->memory == NULL) {
-					WARN("out of memory\n");
-					HeapFree(GetProcessHeap(),0,dsb->buffer);
-					HeapFree(GetProcessHeap(),0,dsb->pwfx);
-					HeapFree(GetProcessHeap(),0,dsb);
-					*pdsb = NULL;
-					return DSERR_OUTOFMEMORY;
-				}
-				dsb->buffer->ref = 1;
-				FillMemory(dsb->buffer->memory, dsb->buflen, dsbd->lpwfxFormat->wBitsPerSample == 8 ? 128 : 0);
-			}
-			err = DS_OK;
+		if (FAILED(err))
+		{
+			WARN("Failed to create hardware secondary buffer: %08x\n", err);
+			if (device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY)
+				HeapFree(GetProcessHeap(),0,dsb->buffer->memory);
+			HeapFree(GetProcessHeap(),0,dsb->buffer);
+			HeapFree(GetProcessHeap(),0,dsb->pwfx);
+			HeapFree(GetProcessHeap(),0,dsb);
+			*pdsb = NULL;
+			return DSERR_GENERIC;
 		}
 	}
 
-	/* calculate fragment size and write lead */
-	DSOUND_RecalcFormat(dsb);
+	dsb->buffer->ref = 1;
+	list_init(&dsb->buffer->buffers);
+	list_add_head(&dsb->buffer->buffers, &dsb->entry);
+	FillMemory(dsb->buffer->memory, dsb->buflen, dsbd->lpwfxFormat->wBitsPerSample == 8 ? 128 : 0);
 
 	/* It's not necessary to initialize values to zero since */
 	/* we allocated this structure with HEAP_ZERO_MEMORY... */
-	dsb->playpos = 0;
-	dsb->buf_mixpos = 0;
+	dsb->buf_mixpos = dsb->sec_mixpos = 0;
 	dsb->state = STATE_STOPPED;
 
-	dsb->freqAdjust = (dsb->freq << DSOUND_FREQSHIFT) /
-		device->pwfx->nSamplesPerSec;
+	dsb->freqAdjust = ((DWORD64)dsb->freq << DSOUND_FREQSHIFT) / device->pwfx->nSamplesPerSec;
 	dsb->nAvgBytesPerSec = dsb->freq *
 		dsbd->lpwfxFormat->nBlockAlign;
 
+	/* calculate fragment size and write lead */
+	DSOUND_RecalcFormat(dsb);
+
 	if (dsb->dsbd.dwFlags & DSBCAPS_CTRL3D) {
 		dsb->ds3db_ds3db.dwSize = sizeof(DS3DBUFFER);
 		dsb->ds3db_ds3db.vPosition.x = 0.0;
@@ -1179,8 +1122,7 @@ HRESULT IDirectSoundBufferImpl_Create(
 	} else
 		DSOUND_RecalcVolPan(&(dsb->volpan));
 
-	InitializeCriticalSection(&(dsb->lock));
-        dsb->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": IDirectSoundBufferImpl.lock");
+	RtlInitializeResource(&dsb->lock);
 
 	/* register buffer if not primary */
 	if (!(dsbd->dwFlags & DSBCAPS_PRIMARYBUFFER)) {
@@ -1188,8 +1130,7 @@ HRESULT IDirectSoundBufferImpl_Create(
 		if (err != DS_OK) {
 			HeapFree(GetProcessHeap(),0,dsb->buffer->memory);
 			HeapFree(GetProcessHeap(),0,dsb->buffer);
-        		dsb->lock.DebugInfo->Spare[0] = 0;
-			DeleteCriticalSection(&(dsb->lock));
+			RtlDeleteResource(&dsb->lock);
 			HeapFree(GetProcessHeap(),0,dsb->pwfx);
 			HeapFree(GetProcessHeap(),0,dsb);
 			dsb = NULL;
@@ -1263,46 +1204,26 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
 
         hres = IDsDriver_DuplicateSoundBuffer(device->driver, pdsb->hwbuf,
                                               (LPVOID *)&dsb->hwbuf);
-        if (hres != DS_OK) {
-            TRACE("IDsDriver_DuplicateSoundBuffer failed, falling back to "
-                  "software buffer\n");
-            dsb->hwbuf = NULL;
-            /* allocate buffer */
-            if (device->drvdesc.dwFlags & DSDDESC_USESYSTEMMEMORY) {
-                dsb->buffer = HeapAlloc(GetProcessHeap(),0,sizeof(*(dsb->buffer)));
-                if (dsb->buffer == NULL) {
-                    WARN("out of memory\n");
-                    HeapFree(GetProcessHeap(),0,dsb);
-                    *ppdsb = NULL;
-                    return DSERR_OUTOFMEMORY;
-                }
-
-                dsb->buffer->memory = HeapAlloc(GetProcessHeap(),0,dsb->buflen);
-                if (dsb->buffer->memory == NULL) {
-                    WARN("out of memory\n");
-                    HeapFree(GetProcessHeap(),0,dsb->buffer);
-                    HeapFree(GetProcessHeap(),0,dsb);
-                    *ppdsb = NULL;
-                    return DSERR_OUTOFMEMORY;
-                }
-                dsb->buffer->ref = 1;
-
-                /* FIXME: copy buffer ? */
-            }
+        if (FAILED(hres)) {
+            WARN("IDsDriver_DuplicateSoundBuffer failed (%08x)\n", hres);
+            HeapFree(GetProcessHeap(),0,dsb);
+            *ppdsb = NULL;
+            return hres;
         }
-    } else {
-        dsb->hwbuf = NULL;
-        dsb->buffer->ref++;
     }
 
+    dsb->buffer->ref++;
+    list_add_head(&dsb->buffer->buffers, &dsb->entry);
     dsb->ref = 0;
     dsb->state = STATE_STOPPED;
-    dsb->playpos = 0;
-    dsb->buf_mixpos = 0;
+    dsb->buf_mixpos = dsb->sec_mixpos = 0;
     dsb->device = device;
     dsb->ds3db = NULL;
     dsb->iks = NULL; /* FIXME? */
     dsb->secondary = NULL;
+    dsb->tmp_buffer = NULL;
+    DSOUND_RecalcFormat(dsb);
+    DSOUND_MixToTemporary(dsb, 0, dsb->buflen, FALSE);
 
     /* variable sized struct so calculate size based on format */
     size = sizeof(WAVEFORMATEX) + pdsb->pwfx->cbSize;
@@ -1318,14 +1239,13 @@ HRESULT IDirectSoundBufferImpl_Duplicate(
 
     CopyMemory(dsb->pwfx, pdsb->pwfx, size);
 
-    InitializeCriticalSection(&(dsb->lock));
-    dsb->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": IDirectSoundBufferImpl.lock");
+    RtlInitializeResource(&dsb->lock);
 
     /* register buffer */
     hres = DirectSoundDevice_AddBuffer(device, dsb);
     if (hres != DS_OK) {
-        dsb->lock.DebugInfo->Spare[0] = 0;
-        DeleteCriticalSection(&(dsb->lock));
+        RtlDeleteResource(&dsb->lock);
+        HeapFree(GetProcessHeap(),0,dsb->tmp_buffer);
         HeapFree(GetProcessHeap(),0,dsb->buffer);
         HeapFree(GetProcessHeap(),0,dsb->pwfx);
         HeapFree(GetProcessHeap(),0,dsb);
