@@ -44,7 +44,6 @@ static BOOL DC_DeleteObject( HGDIOBJ handle, void *obj );
 static const struct gdi_obj_funcs dc_funcs =
 {
     NULL,             /* pSelectObject */
-    NULL,             /* pGetObject16 */
     NULL,             /* pGetObjectA */
     NULL,             /* pGetObjectW */
     NULL,             /* pUnrealizeObject */
@@ -155,40 +154,6 @@ DC *alloc_dc_ptr( const DC_FUNCTIONS *funcs, WORD magic )
 
 
 
-/***********************************************************************
- *           DC_GetDCPtr
- */
-DC *DC_GetDCPtr( HDC hdc )
-{
-    DC *dc = get_dc_obj( hdc );
-    if (!dc) return NULL;
-
-    if (!InterlockedCompareExchange( &dc->refcount, 1, 0 ))
-    {
-        dc->thread = GetCurrentThreadId();
-    }
-    else if (dc->thread != GetCurrentThreadId())
-    {
-        GDI_ReleaseObj( hdc );
-        SetLastError( ERROR_ACCESS_DENIED );
-        return NULL;
-    }
-    else InterlockedIncrement( &dc->refcount );
-
-    return dc;
-}
-
-
-/***********************************************************************
- *           DC_ReleaseDCPtr
- */
-void DC_ReleaseDCPtr( DC *dc )
-{
-    release_dc_ptr( dc );
-    GDI_ReleaseObj( dc->hSelf );
-}
-
-
 /***********************************************************************
  *           free_dc_ptr
  */
@@ -472,7 +437,7 @@ void WINAPI SetDCState( HDC hdc, HDC hdcs )
         release_dc_ptr( dc );
         return;
     }
-    if (!dcs->flags & DC_SAVED)
+    if (!(dcs->flags & DC_SAVED))
     {
         release_dc_ptr( dc );
         release_dc_ptr( dcs );
@@ -1429,6 +1394,24 @@ BOOL WINAPI SetDCHook( HDC hdc, DCHOOKPROC hookProc, DWORD_PTR dwHookData )
 }
 
 
+/***********************************************************************
+ *           GetDCHook   (GDI32.@)
+ *
+ * Note: this doesn't exist in Win32, we add it here because user32 needs it.
+ */
+DWORD_PTR WINAPI GetDCHook( HDC hdc, DCHOOKPROC *proc )
+{
+    DC *dc = get_dc_ptr( hdc );
+    DWORD_PTR ret;
+
+    if (!dc) return 0;
+    if (proc) *proc = dc->hookThunk;
+    ret = dc->dwHookData;
+    release_dc_ptr( dc );
+    return ret;
+}
+
+
 /* relay function to call the 16-bit DC hook proc */
 static BOOL WINAPI call_dc_hook16( HDC hdc, WORD code, DWORD_PTR data, LPARAM lParam )
 {
@@ -1488,11 +1471,12 @@ DWORD WINAPI GetDCHook16( HDC16 hdc16, FARPROC16 *phookProc )
 
 
 /***********************************************************************
- *           SetHookFlags   (GDI.192)
+ *           SetHookFlags   (GDI32.@)
+ *
+ * Note: this doesn't exist in Win32, we add it here because user32 needs it.
  */
-WORD WINAPI SetHookFlags16(HDC16 hdc16, WORD flags)
+WORD WINAPI SetHookFlags( HDC hdc, WORD flags )
 {
-    HDC hdc = HDC_32( hdc16 );
     DC *dc = get_dc_obj( hdc );  /* not get_dc_ptr, this needs to work from any thread */
     LONG ret = 0;
 
@@ -1516,7 +1500,7 @@ WORD WINAPI SetHookFlags16(HDC16 hdc16, WORD flags)
  */
 INT WINAPI SetICMMode(HDC hdc, INT iEnableICM)
 {
-/*FIXME  Asuming that ICM is always off, and cannot be turned on */
+/*FIXME:  Assume that ICM is always off, and cannot be turned on */
     if (iEnableICM == ICM_OFF) return ICM_OFF;
     if (iEnableICM == ICM_ON) return 0;
     if (iEnableICM == ICM_QUERY) return ICM_OFF;
