@@ -82,7 +82,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(bitmap);
  *           DIB_GetDIBWidthBytes
  *
  * Return the width of a DIB bitmap in bytes. DIB bitmap data is 32-bit aligned.
- * http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/bitmaps_87eb.asp
  */
 int DIB_GetDIBWidthBytes( int width, int depth )
 {
@@ -189,31 +188,31 @@ INT WINAPI StretchDIBits(HDC hdc, INT xDst, INT yDst, INT widthDst,
                        const BITMAPINFO *info, UINT wUsage, DWORD dwRop )
 {
     DC *dc;
+    INT ret;
 
     if (!bits || !info)
 	return 0;
 
-    dc = DC_GetDCUpdate( hdc );
-    if(!dc) return FALSE;
+    if (!(dc = get_dc_ptr( hdc ))) return 0;
 
     if(dc->funcs->pStretchDIBits)
     {
-        heightSrc = dc->funcs->pStretchDIBits(dc->physDev, xDst, yDst, widthDst,
-                                              heightDst, xSrc, ySrc, widthSrc,
-                                              heightSrc, bits, info, wUsage, dwRop);
-        GDI_ReleaseObj( hdc );
+        update_dc( dc );
+        ret = dc->funcs->pStretchDIBits(dc->physDev, xDst, yDst, widthDst,
+                                        heightDst, xSrc, ySrc, widthSrc,
+                                        heightSrc, bits, info, wUsage, dwRop);
+        release_dc_ptr( dc );
     }
     else /* use StretchBlt */
     {
-        HBITMAP hBitmap, hOldBitmap;
-        HPALETTE hpal = NULL;
-        HDC hdcMem;
         LONG height;
         LONG width;
         WORD planes, bpp;
         DWORD compr, size;
+        HBITMAP hBitmap;
+        BOOL fastpath = FALSE;
 
-        GDI_ReleaseObj( hdc );
+        release_dc_ptr( dc );
 
         if (DIB_GetBitmapInfo( &info->bmiHeader, &width, &height, &planes, &bpp, &compr, &size ) == -1)
         {
@@ -227,53 +226,86 @@ INT WINAPI StretchDIBits(HDC hdc, INT xDst, INT yDst, INT widthDst,
             return 0;
         }
 
-	hdcMem = CreateCompatibleDC( hdc );
-        hBitmap = CreateCompatibleBitmap(hdc, width, height);
-        hOldBitmap = SelectObject( hdcMem, hBitmap );
-        if(wUsage == DIB_PAL_COLORS)
+        hBitmap = GetCurrentObject(hdc, OBJ_BITMAP);
+
+        if (xDst == 0 && yDst == 0 && xSrc == 0 && ySrc == 0 &&
+            widthDst == widthSrc && heightDst == heightSrc &&
+            info->bmiHeader.biCompression == BI_RGB &&
+            dwRop == SRCCOPY)
         {
-            hpal = GetCurrentObject(hdc, OBJ_PAL);
-            hpal = SelectPalette(hdcMem, hpal, FALSE);
+            BITMAPOBJ *bmp;
+            if ((bmp = (BITMAPOBJ *)GDI_GetObjPtr( hBitmap, BITMAP_MAGIC )))
+            {
+                if (bmp->bitmap.bmBitsPixel == bpp &&
+                    bmp->bitmap.bmWidth == widthSrc &&
+                    bmp->bitmap.bmHeight == heightSrc &&
+                    bmp->bitmap.bmPlanes == planes)
+                    fastpath = TRUE;
+                GDI_ReleaseObj( hBitmap );
+            }
         }
 
-	if (info->bmiHeader.biCompression == BI_RLE4 ||
-	    info->bmiHeader.biCompression == BI_RLE8) {
-
-	   /* when RLE compression is used, there may be some gaps (ie the DIB doesn't
-	    * contain all the rectangle described in bmiHeader, but only part of it.
-	    * This mean that those undescribed pixels must be left untouched.
-	    * So, we first copy on a memory bitmap the current content of the
-	    * destination rectangle, blit the DIB bits on top of it - hence leaving
-	    * the gaps untouched -, and blitting the rectangle back.
-	    * This insure that gaps are untouched on the destination rectangle
-	    * Not doing so leads to trashed images (the gaps contain what was on the
-	    * memory bitmap => generally black or garbage)
-	    * Unfortunately, RLE DIBs without gaps will be slowed down. But this is
-	    * another speed vs correctness issue. Anyway, if speed is needed, then the
-	    * pStretchDIBits function shall be implemented.
-	    * ericP (2000/09/09)
-	    */
-
-            /* copy existing bitmap from destination dc */
-            StretchBlt( hdcMem, xSrc, abs(height) - heightSrc - ySrc,
-                        widthSrc, heightSrc, hdc, xDst, yDst, widthDst, heightDst,
-                        dwRop );
+        if (fastpath)
+        {
+            /* fast path */
+            TRACE("using fast path\n");
+            ret = SetDIBits( hdc, hBitmap, 0, height, bits, info, wUsage);
         }
+        else
+        {
+            /* slow path - need to use StretchBlt */
+            HBITMAP hOldBitmap;
+            HPALETTE hpal = NULL;
+            HDC hdcMem;
 
-        SetDIBits(hdcMem, hBitmap, 0, height, bits, info, wUsage);
+            hdcMem = CreateCompatibleDC( hdc );
+            hBitmap = CreateCompatibleBitmap(hdc, width, height);
+            hOldBitmap = SelectObject( hdcMem, hBitmap );
+            if(wUsage == DIB_PAL_COLORS)
+            {
+                hpal = GetCurrentObject(hdc, OBJ_PAL);
+                hpal = SelectPalette(hdcMem, hpal, FALSE);
+            }
 
-        /* Origin for DIBitmap may be bottom left (positive biHeight) or top
-           left (negative biHeight) */
-        StretchBlt( hdc, xDst, yDst, widthDst, heightDst,
-		    hdcMem, xSrc, abs(height) - heightSrc - ySrc,
-		    widthSrc, heightSrc, dwRop );
-        if(hpal)
-            SelectPalette(hdcMem, hpal, FALSE);
-	SelectObject( hdcMem, hOldBitmap );
-	DeleteDC( hdcMem );
-	DeleteObject( hBitmap );
+            if (info->bmiHeader.biCompression == BI_RLE4 ||
+	            info->bmiHeader.biCompression == BI_RLE8) {
+
+                /* when RLE compression is used, there may be some gaps (ie the DIB doesn't
+                 * contain all the rectangle described in bmiHeader, but only part of it.
+                 * This mean that those undescribed pixels must be left untouched.
+                 * So, we first copy on a memory bitmap the current content of the
+                 * destination rectangle, blit the DIB bits on top of it - hence leaving
+                 * the gaps untouched -, and blitting the rectangle back.
+                 * This insure that gaps are untouched on the destination rectangle
+                 * Not doing so leads to trashed images (the gaps contain what was on the
+                 * memory bitmap => generally black or garbage)
+                 * Unfortunately, RLE DIBs without gaps will be slowed down. But this is
+                 * another speed vs correctness issue. Anyway, if speed is needed, then the
+                 * pStretchDIBits function shall be implemented.
+                 * ericP (2000/09/09)
+                 */
+
+                /* copy existing bitmap from destination dc */
+                StretchBlt( hdcMem, xSrc, abs(height) - heightSrc - ySrc,
+                            widthSrc, heightSrc, hdc, xDst, yDst, widthDst, heightDst,
+                            dwRop );
+            }
+
+            ret = SetDIBits(hdcMem, hBitmap, 0, height, bits, info, wUsage);
+
+            /* Origin for DIBitmap may be bottom left (positive biHeight) or top
+               left (negative biHeight) */
+            if (ret) StretchBlt( hdc, xDst, yDst, widthDst, heightDst,
+                                 hdcMem, xSrc, abs(height) - heightSrc - ySrc,
+                                 widthSrc, heightSrc, dwRop );
+            if(hpal)
+                SelectPalette(hdcMem, hpal, FALSE);
+            SelectObject( hdcMem, hOldBitmap );
+            DeleteDC( hdcMem );
+            DeleteObject( hBitmap );
+        }
     }
-    return heightSrc;
+    return ret;
 }
 
 
@@ -303,29 +335,35 @@ INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
     BITMAPOBJ *bitmap;
     INT result = 0;
 
-    if (!(dc = DC_GetDCUpdate( hdc )))
+    if (!(dc = get_dc_ptr( hdc )))
     {
         if (coloruse == DIB_RGB_COLORS) FIXME( "shouldn't require a DC for DIB_RGB_COLORS\n" );
         return 0;
     }
 
+    update_dc( dc );
+
     if (!(bitmap = GDI_GetObjPtr( hbitmap, BITMAP_MAGIC )))
     {
-        GDI_ReleaseObj( hdc );
+        release_dc_ptr( dc );
         return 0;
     }
 
     if (!bitmap->funcs && !BITMAP_SetOwnerDC( hbitmap, dc )) goto done;
 
-    if (bitmap->funcs && bitmap->funcs->pSetDIBits)
-        result = bitmap->funcs->pSetDIBits( dc->physDev, hbitmap, startscan, lines,
+    result = lines;
+    if (bitmap->funcs)
+    {
+        if (bitmap->funcs != dc->funcs)
+            ERR( "not supported: DDB bitmap %p not belonging to device %p\n", hbitmap, hdc );
+        else if (dc->funcs->pSetDIBits)
+            result = dc->funcs->pSetDIBits( dc->physDev, hbitmap, startscan, lines,
                                             bits, info, coloruse );
-    else
-        result = lines;
+    }
 
  done:
     GDI_ReleaseObj( hbitmap );
-    GDI_ReleaseObj( hdc );
+    release_dc_ptr( dc );
     return result;
 }
 
@@ -343,18 +381,21 @@ INT WINAPI SetDIBitsToDevice(HDC hdc, INT xDest, INT yDest, DWORD cx,
 
     if (!bits) return 0;
 
-    if (!(dc = DC_GetDCUpdate( hdc ))) return 0;
+    if (!(dc = get_dc_ptr( hdc ))) return 0;
 
     if(dc->funcs->pSetDIBitsToDevice)
+    {
+        update_dc( dc );
         ret = dc->funcs->pSetDIBitsToDevice( dc->physDev, xDest, yDest, cx, cy, xSrc,
 					     ySrc, startscan, lines, bits,
 					     info, coloruse );
+    }
     else {
         FIXME("unimplemented on hdc %p\n", hdc);
 	ret = 0;
     }
 
-    GDI_ReleaseObj( hdc );
+    release_dc_ptr( dc );
     return ret;
 }
 
@@ -367,7 +408,7 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, CONST RGBQUA
     UINT result = 0;
     BITMAPOBJ * bitmap;
 
-    if (!(dc = DC_GetDCPtr( hdc ))) return 0;
+    if (!(dc = get_dc_ptr( hdc ))) return 0;
 
     if ((bitmap = GDI_GetObjPtr( dc->hBitmap, BITMAP_MAGIC )))
     {
@@ -387,7 +428,7 @@ UINT WINAPI SetDIBColorTable( HDC hdc, UINT startpos, UINT entries, CONST RGBQUA
     if (dc->funcs->pSetDIBColorTable)
         dc->funcs->pSetDIBColorTable(dc->physDev, startpos, entries, colors);
 
-    GDI_ReleaseObj( hdc );
+    release_dc_ptr( dc );
     return result;
 }
 
@@ -400,7 +441,7 @@ UINT WINAPI GetDIBColorTable( HDC hdc, UINT startpos, UINT entries, RGBQUAD *col
     DC * dc;
     UINT result = 0;
 
-    if (!(dc = DC_GetDCPtr( hdc ))) return 0;
+    if (!(dc = get_dc_ptr( hdc ))) return 0;
 
     if (dc->funcs->pGetDIBColorTable)
         result = dc->funcs->pGetDIBColorTable(dc->physDev, startpos, entries, colors);
@@ -422,7 +463,7 @@ UINT WINAPI GetDIBColorTable( HDC hdc, UINT startpos, UINT entries, RGBQUAD *col
             GDI_ReleaseObj( dc->hBitmap );
         }
     }
-    GDI_ReleaseObj( hdc );
+    release_dc_ptr( dc );
     return result;
 }
 
@@ -529,8 +570,6 @@ static const RGBTRIPLE DefLogPaletteTriples[20] = { /* Copy of Default Logical P
  * RETURNS
  *    Success: Number of scan lines copied from bitmap
  *    Failure: 0
- *
- * http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/bitmaps_87eb.asp
  */
 INT WINAPI GetDIBits(
     HDC hdc,         /* [in]  Handle to device context */
@@ -563,14 +602,15 @@ INT WINAPI GetDIBits(
         return 0;
     }
     core_header = (bitmap_type == 0);
-    if (!(dc = DC_GetDCUpdate( hdc )))
+    if (!(dc = get_dc_ptr( hdc )))
     {
         SetLastError( ERROR_INVALID_PARAMETER );
         return 0;
     }
+    update_dc( dc );
     if (!(bmp = (BITMAPOBJ *)GDI_GetObjPtr( hbitmap, BITMAP_MAGIC )))
     {
-        GDI_ReleaseObj( hdc );
+        release_dc_ptr( dc );
 	return 0;
     }
 
@@ -622,7 +662,7 @@ INT WINAPI GetDIBits(
                 memset( palEntry, 0, sizeof(palEntry) );
                 if (!GetPaletteEntries( dc->hPalette, 0, 1 << bmp->bitmap.bmBitsPixel, palEntry ))
                 {
-                    GDI_ReleaseObj( hdc );
+                    release_dc_ptr( dc );
                     GDI_ReleaseObj( hbitmap );
                     return 0;
                 }
@@ -822,7 +862,7 @@ INT WINAPI GetDIBits(
                                     *dstbits++ = (BYTE)(((val >> 2) & 0xf8) | ((val >> 7) & 0x07));
                                     *dstbits++ = (BYTE)(((val >> 7) & 0xf8) | ((val >> 12) & 0x07));
                                 }
-                                dstbits = (LPBYTE)(dbits+=dstwidthb);
+                                dstbits = dbits+=dstwidthb;
                                 srcbits = (LPWORD)(sbits+=srcwidthb);
                             }
                         }
@@ -838,7 +878,7 @@ INT WINAPI GetDIBits(
 
                     case 32: /* 32 bpp srcDIB -> 24 bpp dstDIB */
                         {
-                            LPBYTE srcbits = (LPBYTE)sbits;
+                            LPBYTE srcbits = sbits;
 
                             width = min(srcwidth, dstwidth);
                             for( y = 0; y < lines; y++) {
@@ -847,8 +887,8 @@ INT WINAPI GetDIBits(
                                     *dstbits++ = *srcbits++;
                                     *dstbits++ = *srcbits++;
                                 }
-                                dstbits=(LPBYTE)(dbits+=dstwidthb);
-                                srcbits = (LPBYTE)(sbits+=srcwidthb);
+                                dstbits = dbits+=dstwidthb;
+                                srcbits = sbits+=srcwidthb;
                             }
                         }
                         break;
@@ -972,11 +1012,15 @@ INT WINAPI GetDIBits(
                     break;
                     
                 case 16:
+                    if (bits)
+                    {
+                        /* Add color only when bits is given, as per MSDN */
+                        ((PDWORD)info->bmiColors)[0] = 0xf800;
+                        ((PDWORD)info->bmiColors)[1] = 0x07e0;
+                        ((PDWORD)info->bmiColors)[2] = 0x001f;
+                    }
                     info->bmiHeader.biBitCount = 16;
                     info->bmiHeader.biCompression = BI_BITFIELDS;
-                    ((PDWORD)info->bmiColors)[0] = 0xf800;
-                    ((PDWORD)info->bmiColors)[1] = 0x07e0;
-                    ((PDWORD)info->bmiColors)[2] = 0x001f;
                     break;
     
                 default:
@@ -1015,7 +1059,7 @@ INT WINAPI GetDIBits(
     }
     TRACE("biWidth = %d, biHeight = %d\n", width, height);
 
-    GDI_ReleaseObj( hdc );
+    release_dc_ptr( dc );
     GDI_ReleaseObj( hbitmap );
     return lines;
 }
@@ -1061,14 +1105,14 @@ HBITMAP WINAPI CreateDIBitmap( HDC hdc, const BITMAPINFOHEADER *header,
     {
         if (init == CBM_INIT) SetDIBits( hdc, handle, 0, height, bits, data, coloruse );
 
-        else if (hdc && ((dc = DC_GetDCPtr( hdc )) != NULL) )
+        else if (hdc && ((dc = get_dc_ptr( hdc )) != NULL) )
         {
             if (!BITMAP_SetOwnerDC( handle, dc ))
             {
                 DeleteObject( handle );
                 handle = 0;
             }
-            GDI_ReleaseObj( hdc );
+            release_dc_ptr( dc );
         }
     }
 
@@ -1123,7 +1167,7 @@ HBITMAP16 WINAPI CreateDIBSection16 (HDC16 hdc, const BITMAPINFO *bmi, UINT16 us
     return HBITMAP_16(hbitmap);
 }
 
-/* Copy/synthetize RGB palette from BITMAPINFO. Ripped from dlls/winex11.drv/dib.c */
+/* Copy/synthesize RGB palette from BITMAPINFO. Ripped from dlls/winex11.drv/dib.c */
 static void DIB_CopyColorTable( DC *dc, BITMAPOBJ *bmp, WORD coloruse, const BITMAPINFO *info )
 {
     RGBQUAD *colorTable;
@@ -1309,7 +1353,7 @@ HBITMAP WINAPI CreateDIBSection(HDC hdc, CONST BITMAPINFO *bmi, UINT usage,
         bDesktopDC = TRUE;
     }
 
-    if (!(dc = DC_GetDCPtr( hdc ))) goto error;
+    if (!(dc = get_dc_ptr( hdc ))) goto error;
 
     /* create Device Dependent Bitmap and add DIB pointer */
     ret = CreateBitmap( dib->dsBm.bmWidth, dib->dsBm.bmHeight, 1,
@@ -1333,7 +1377,7 @@ HBITMAP WINAPI CreateDIBSection(HDC hdc, CONST BITMAPINFO *bmi, UINT usage,
         }
     }
 
-    GDI_ReleaseObj(hdc);
+    release_dc_ptr( dc );
     if (bDesktopDC) DeleteDC( hdc );
     if (ret && bits) *bits = dib->dsBm.bmBits;
     return ret;
