@@ -31,7 +31,7 @@
  *
  * Memory-based metafiles are just stored as a continuous block of memory with
  * a METAHEADER at the head with METARECORDs appended to it.  mtType is
- * METAFILE_MEMORY (1).  Note this is indentical to the disk image of a
+ * METAFILE_MEMORY (1).  Note this is identical to the disk image of a
  * disk-based metafile - even mtType is METAFILE_MEMORY.
  * 16bit HMETAFILE16s are global handles to this block
  * 32bit HMETAFILEs are GDI handles METAFILEOBJs, which contains a ptr to
@@ -294,7 +294,7 @@ METAHEADER *MF_LoadDiskBasedMetaFile(METAHEADER *mh)
  *         MF_CreateMetaHeaderDisk
  *
  * Take a memory based METAHEADER and change it to a disk based METAHEADER
- * assosiated with filename.  Note: Trashes contents of old one.
+ * associated with filename.  Note: Trashes contents of old one.
  */
 METAHEADER *MF_CreateMetaHeaderDisk(METAHEADER *mh, LPCVOID filename, BOOL uni )
 {
@@ -427,7 +427,12 @@ BOOL MF_PlayMetaFile( HDC hdc, METAHEADER *mh)
                   mr->rdSize,offset,mh->mtSize*2);
             break;
 	}
+
 	offset += mr->rdSize * 2;
+	if (mr->rdFunction == META_EOF) {
+	    TRACE("Got META_EOF so stopping\n");
+	    break;
+	}
 	PlayMetaFileRecord( hdc, ht, mr, mh->mtNoObjects );
     }
 
@@ -1265,19 +1270,19 @@ static BOOL MF_Play_MetaExtTextOut(HDC hdc, METARECORD *mr)
     }
 
     if (mr->rdSize == len / 2)
-        dxx = NULL;                      /* determine if array present */
+        dxx = NULL;                      /* determine if array is present */
     else
         if (mr->rdSize == (len + s1 * sizeof(INT16)) / 2)
         {
             dxx = (LPINT16)(sot+(((s1+1)>>1)*2));
             dx = HeapAlloc( GetProcessHeap(), 0, s1*sizeof(INT));
-            if (dx) for (i = 0; i < s1; i++) dx[i] = (SHORT)dxx[i];
+            if (dx) for (i = 0; i < s1; i++) dx[i] = dxx[i];
         }
 	else {
             TRACE("%s  len: %d\n",  sot, mr->rdSize);
             WARN("Please report: ExtTextOut len=%d slen=%d rdSize=%d opt=%04x\n",
 		 len, s1, mr->rdSize, mr->rdParm[3]);
-	    dxx = NULL; /* should't happen -- but if, we continue with NULL */
+	    dxx = NULL; /* shouldn't happen -- but if, we continue with NULL */
 	}
     ExtTextOutA( hdc,
                  (SHORT)mr->rdParm[1],       /* X position */
