@@ -179,7 +179,7 @@ GpStatus WINGDIPAPI GdipAddPathEllipse(GpPath *path, REAL x, REAL y, REAL width,
     memset(&path->pathdata.Types[old_count + 1], PathPointTypeBezier,
            MAX_ARC_PTS - 1);
 
-    /* An ellipse is an instrinsic figure (always its own subpath). */
+    /* An ellipse is an intrinsic figure (always is its own subpath). */
     path->pathdata.Types[old_count] = PathPointTypeStart;
     path->pathdata.Types[old_count + MAX_ARC_PTS - 1] |= PathPointTypeCloseSubpath;
     path->newfigure = TRUE;
@@ -281,7 +281,7 @@ GpStatus WINGDIPAPI GdipClonePath(GpPath* path, GpPath **clone)
     *clone = GdipAlloc(sizeof(GpPath));
     if(!*clone) return OutOfMemory;
 
-    memcpy(*clone, path, sizeof(GpPath));
+    **clone = *path;
 
     (*clone)->pathdata.Points = GdipAlloc(path->datalen * sizeof(PointF));
     (*clone)->pathdata.Types = GdipAlloc(path->datalen);
@@ -570,6 +570,6 @@ GpStatus WINGDIPAPI GdipTransformPath(GpPath *path, GpMatrix *matrix)
     if(path->pathdata.Count == 0)
         return Ok;
 
-    return GdipTransformMatrixPoints(matrix, (GpPointF*) path->pathdata.Points,
+    return GdipTransformMatrixPoints(matrix, path->pathdata.Points,
                                      path->pathdata.Count);
 }
