@@ -182,7 +182,10 @@ DWORD getInterfaceStatsByName(const char *name, PMIB_IFROW entry)
     fclose(fp);
   }
   else
+  {
      ERR ("unimplemented!\n");
+     return ERROR_NOT_SUPPORTED;
+  }
 
   return NO_ERROR;
 }
@@ -312,7 +315,10 @@ DWORD getICMPStats(MIB_ICMP *stats)
     fclose(fp);
   }
   else
+  {
      ERR ("unimplemented!\n");
+     return ERROR_NOT_SUPPORTED;
+  }
 
   return NO_ERROR;
 }
@@ -426,7 +432,10 @@ DWORD getIPStats(PMIB_IPSTATS stats)
     fclose(fp);
   }
   else
+  {
      ERR ("unimplemented!\n");
+     return ERROR_NOT_SUPPORTED;
+  }
 
   return NO_ERROR;
 }
@@ -519,7 +528,10 @@ DWORD getTCPStats(MIB_TCPSTATS *stats)
     fclose(fp);
   }
   else
+  {
      ERR ("unimplemented!\n");
+     return ERROR_NOT_SUPPORTED;
+  }
 
   return NO_ERROR;
 }
@@ -575,7 +587,10 @@ DWORD getUDPStats(MIB_UDPSTATS *stats)
     fclose(fp);
   }
   else
+  {
      ERR ("unimplemented!\n");
+     return ERROR_NOT_SUPPORTED;
+  }
 
   return NO_ERROR;
 }
@@ -762,9 +777,12 @@ DWORD getRouteTable(PMIB_IPFORWARDTABLE *ppIpForwardTable, HANDLE heap,
     ret = ERROR_INVALID_PARAMETER;
   else {
     DWORD numRoutes = getNumRoutes();
-    PMIB_IPFORWARDTABLE table = HeapAlloc(heap, flags,
-     sizeof(MIB_IPFORWARDTABLE) + (numRoutes - 1) * sizeof(MIB_IPFORWARDROW));
+    DWORD size = sizeof(MIB_IPFORWARDTABLE);
+    PMIB_IPFORWARDTABLE table;
 
+    if (numRoutes > 1)
+      size += (numRoutes - 1) * sizeof(MIB_IPFORWARDROW);
+    table = HeapAlloc(heap, flags, size);
     if (table) {
 #if defined(HAVE_SYS_SYSCTL_H) && defined(NET_RT_DUMP)
        int mib[6] = {CTL_NET, PF_ROUTE, 0, PF_INET, NET_RT_DUMP, 0};
@@ -963,7 +981,7 @@ DWORD getRouteTable(PMIB_IPFORWARDTABLE *ppIpForwardTable, HANDLE heap,
       else
       {
         ERR ("unimplemented!\n");
-        return ERROR_INVALID_PARAMETER;
+        return ERROR_NOT_SUPPORTED;
       }
 #endif
     }
@@ -984,16 +1002,19 @@ DWORD getArpTable(PMIB_IPNETTABLE *ppIpNetTable, HANDLE heap, DWORD flags)
 
 #if defined(HAVE_SYS_SYSCTL_H) && defined(NET_RT_DUMP)
   ERR ("unimplemented!\n");
-  return ERROR_INVALID_PARAMETER;
+  return ERROR_NOT_SUPPORTED;
 #endif
 
   if (!ppIpNetTable)
     ret = ERROR_INVALID_PARAMETER;
   else {
     DWORD numEntries = getNumArpEntries();
-    PMIB_IPNETTABLE table = HeapAlloc(heap, flags,
-     sizeof(MIB_IPNETTABLE) + (numEntries - 1) * sizeof(MIB_IPNETROW));
+    DWORD size = sizeof(MIB_IPNETTABLE);
+    PMIB_IPNETTABLE table;
 
+    if (numEntries > 1)
+      size += (numEntries - 1) * sizeof(MIB_IPNETROW);
+    table = HeapAlloc(heap, flags, size);
     if (table) {
       FILE *fp;
 
@@ -1064,6 +1085,8 @@ DWORD getArpTable(PMIB_IPNETTABLE *ppIpNetTable, HANDLE heap, DWORD flags)
         }
         fclose(fp);
       }
+      else
+        ret = ERROR_NOT_SUPPORTED;
     }
     else
       ret = ERROR_OUTOFMEMORY;
@@ -1082,16 +1105,19 @@ DWORD getUdpTable(PMIB_UDPTABLE *ppUdpTable, HANDLE heap, DWORD flags)
 
 #if defined(HAVE_SYS_SYSCTL_H) && defined(NET_RT_DUMP)
   ERR ("unimplemented!\n");
-  return ERROR_INVALID_PARAMETER;
+  return ERROR_NOT_SUPPORTED;
 #endif
 
   if (!ppUdpTable)
     ret = ERROR_INVALID_PARAMETER;
   else {
     DWORD numEntries = getNumUdpEntries();
-    PMIB_UDPTABLE table = HeapAlloc(heap, flags,
-     sizeof(MIB_UDPTABLE) + (numEntries - 1) * sizeof(MIB_UDPROW));
+    DWORD size = sizeof(MIB_UDPTABLE);
+    PMIB_UDPTABLE table;
 
+    if (numEntries > 1)
+      size += (numEntries - 1) * sizeof(MIB_UDPROW);
+    table = HeapAlloc(heap, flags, size);
     if (table) {
       FILE *fp;
 
@@ -1132,6 +1158,8 @@ DWORD getUdpTable(PMIB_UDPTABLE *ppUdpTable, HANDLE heap, DWORD flags)
         }
         fclose(fp);
       }
+      else
+        ret = ERROR_NOT_SUPPORTED;
     }
     else
       ret = ERROR_OUTOFMEMORY;
@@ -1193,9 +1221,11 @@ DWORD getTcpTable(PMIB_TCPTABLE *ppTcpTable, DWORD maxEntries, HANDLE heap,
 
    if (!*ppTcpTable)
    {
-      *ppTcpTable = HeapAlloc (heap, flags,
-                               sizeof (MIB_TCPTABLE) +
-                               (numEntries - 1) * sizeof (MIB_TCPROW));
+      DWORD size = sizeof(MIB_TCPTABLE);
+
+      if (numEntries > 1)
+         size += (numEntries - 1) * sizeof (MIB_TCPROW);
+      *ppTcpTable = HeapAlloc (heap, flags, size);
       if (!*ppTcpTable)
       {
          ERR ("Out of memory!\n");
@@ -1294,7 +1324,7 @@ DWORD getTcpTable(PMIB_TCPTABLE *ppTcpTable, DWORD maxEntries, HANDLE heap,
    /* get from /proc/net/tcp, no error if can't */
    fp = fopen("/proc/net/tcp", "r");
    if (!fp)
-      return NO_ERROR;
+      return ERROR_NOT_SUPPORTED;
 
    /* skip header line */
    ptr = fgets(buf, sizeof(buf), fp);
