@@ -132,6 +132,9 @@ static HRSRC find_resourceA( HMODULE hModule, LPCSTR type, LPCSTR name, WORD lan
     LDR_RESOURCE_INFO info;
     const IMAGE_RESOURCE_DATA_ENTRY *entry = NULL;
 
+    nameW.Buffer = NULL;
+    typeW.Buffer = NULL;
+
     __TRY
     {
         if ((status = get_res_nameA( name, &nameW )) != STATUS_SUCCESS) goto done;
@@ -586,7 +589,7 @@ LPVOID WINAPI LockResource( HGLOBAL handle )
     TRACE("(%p)\n", handle );
 
     if (HIWORD( handle ))  /* 32-bit memory handle */
-        return (LPVOID)handle;
+        return handle;
 
     /* 16-bit memory handle */
     return LockResource16( HGLOBAL_16(handle) );
