@@ -23,7 +23,6 @@
 
 #include <string.h>
 
-#include "winerror.h"
 #include "wine/winbase16.h"
 #include "wine/server.h"
 #include "wine/debug.h"
@@ -35,7 +34,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(selector);
 #define LDT_SIZE 8192
 
 /* get the number of selectors needed to cover up to the selector limit */
-inline static WORD get_sel_count( WORD sel )
+static inline WORD get_sel_count( WORD sel )
 {
     return (wine_ldt_copy.limit[sel >> __AHSHIFT] >> 16) + 1;
 }
@@ -483,7 +482,7 @@ SEGPTR WINAPI MapLS( LPCVOID ptr )
 
     if (!HIWORD(ptr)) return (SEGPTR)LOWORD(ptr);
 
-    base = (const char *)ptr - ((unsigned int)ptr & 0x7fff);
+    base = (const char *)ptr - ((ULONG_PTR)ptr & 0x7fff);
     HeapLock( GetProcessHeap() );
     for (entry = first_entry; entry; entry = entry->next)
     {
