@@ -23,6 +23,9 @@
 #include <windows.h>
 #include <stdio.h>
 
+static BOOL (WINAPI *pGetConsoleInputExeNameA)(DWORD, LPSTR);
+static BOOL (WINAPI *pSetConsoleInputExeNameA)(LPCSTR);
+
 /* DEFAULT_ATTRIB is used for all initial filling of the console.
  * all modifications are made with TEST_ATTRIB so that we could check
  * what has to be modified or not
@@ -50,6 +53,21 @@
   ok(expect, "At (%d,%d): expecting attr %04x got %04x\n", (c).X, (c).Y, (attr), __attr); \
 } while (0)
 
+static void init_function_pointers(void)
+{
+    HMODULE hKernel32;
+
+#define KERNEL32_GET_PROC(func)                                     \
+    p##func = (void *)GetProcAddress(hKernel32, #func);             \
+    if(!p##func) trace("GetProcAddress(hKernel32, '%s') failed\n", #func);
+
+    hKernel32 = GetModuleHandleA("kernel32.dll");
+    KERNEL32_GET_PROC(GetConsoleInputExeNameA);
+    KERNEL32_GET_PROC(SetConsoleInputExeNameA);
+
+#undef KERNEL32_GET_PROC
+}
+
 /* FIXME: this could be optimized on a speed point of view */
 static void resetContent(HANDLE hCon, COORD sbSize, BOOL content)
 {
@@ -746,12 +764,60 @@ static void testScreenBuffer(HANDLE hConOut)
     SetConsoleOutputCP(oldcp);
 }
 
+static void test_GetSetConsoleInputExeName(void)
+{
+    BOOL ret;
+    DWORD error;
+    char buffer[MAX_PATH], module[MAX_PATH], *p;
+    static char input_exe[MAX_PATH] = "winetest.exe";
+
+    SetLastError(0xdeadbeef);
+    ret = pGetConsoleInputExeNameA(0, NULL);
+    error = GetLastError();
+    ok(ret, "GetConsoleInputExeNameA failed\n");
+    ok(error == ERROR_BUFFER_OVERFLOW, "got %u expected ERROR_BUFFER_OVERFLOW\n", error);
+
+    SetLastError(0xdeadbeef);
+    ret = pGetConsoleInputExeNameA(0, buffer);
+    error = GetLastError();
+    ok(ret, "GetConsoleInputExeNameA failed\n");
+    ok(error == ERROR_BUFFER_OVERFLOW, "got %u expected ERROR_BUFFER_OVERFLOW\n", error);
+
+    GetModuleFileNameA(GetModuleHandle(NULL), module, sizeof(module));
+    p = strrchr(module, '\\') + 1;
+
+    ret = pGetConsoleInputExeNameA(sizeof(buffer)/sizeof(buffer[0]), buffer);
+    ok(ret, "GetConsoleInputExeNameA failed\n");
+    todo_wine ok(!lstrcmpA(buffer, p), "got %s expected %s\n", buffer, p);
+
+    SetLastError(0xdeadbeef);
+    ret = pSetConsoleInputExeNameA(NULL);
+    error = GetLastError();
+    ok(!ret, "SetConsoleInputExeNameA failed\n");
+    ok(error == ERROR_INVALID_PARAMETER, "got %u expected ERROR_INVALID_PARAMETER\n", error);
+
+    SetLastError(0xdeadbeef);
+    ret = pSetConsoleInputExeNameA("");
+    error = GetLastError();
+    ok(!ret, "SetConsoleInputExeNameA failed\n");
+    ok(error == ERROR_INVALID_PARAMETER, "got %u expected ERROR_INVALID_PARAMETER\n", error);
+
+    ret = pSetConsoleInputExeNameA(input_exe);
+    ok(ret, "SetConsoleInputExeNameA failed\n");
+
+    ret = pGetConsoleInputExeNameA(sizeof(buffer)/sizeof(buffer[0]), buffer);
+    ok(ret, "GetConsoleInputExeNameA failed\n");
+    ok(!lstrcmpA(buffer, input_exe), "got %s expected %s\n", buffer, input_exe);
+}
+
 START_TEST(console)
 {
     HANDLE hConIn, hConOut;
     BOOL ret;
     CONSOLE_SCREEN_BUFFER_INFO	sbi;
 
+    init_function_pointers();
+
     /* be sure we have a clean console (and that's our own)
      * FIXME: this will make the test fail (currently) if we don't run
      * under X11
@@ -784,4 +850,12 @@ START_TEST(console)
     testScreenBuffer(hConOut);
     testCtrlHandler();
     /* still to be done: access rights & access on objects */
+
+    if (!pGetConsoleInputExeNameA || !pSetConsoleInputExeNameA)
+    {
+        skip("GetConsoleInputExeNameA and/or SetConsoleInputExeNameA is not available\n");
+        return;
+    }
+    else
+        test_GetSetConsoleInputExeName();
 }
