@@ -207,7 +207,9 @@ static LONG get_tz_bias(const TIME_ZONE_INFORMATION *tzinfo, DWORD tz_id)
     switch (tz_id)
     {
     case TIME_ZONE_ID_DAYLIGHT:
-        return tzinfo->DaylightBias;
+        if (memcmp(&tzinfo->StandardDate, &tzinfo->DaylightDate, sizeof(tzinfo->DaylightDate)) != 0)
+            return tzinfo->DaylightBias;
+        /* fall through */
 
     case TIME_ZONE_ID_STANDARD:
         return tzinfo->StandardBias;
@@ -298,6 +300,12 @@ static void test_GetTimeZoneInformation(void)
     utc = st;
     SetLastError(0xdeadbeef);
     res = pSystemTimeToTzSpecificLocalTime(&tzinfo, &utc, &current);
+    if (!res && GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
+    {
+        skip("SystemTimeToTzSpecificLocalTime is not implemented\n");
+        return;
+    }
+
     ok(res, "SystemTimeToTzSpecificLocalTime error %u\n", GetLastError());
     s_time = system_time_to_minutes(&current);
 
