@@ -21,13 +21,11 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <windef.h>
-#include <winbase.h>
 
-#include "tlhelp32.h"
-#include "wine/test.h"
 #include "windef.h"
 #include "winbase.h"
+#include "tlhelp32.h"
+#include "wine/test.h"
 #include "winuser.h"
 
 static char     selfname[MAX_PATH];
@@ -108,6 +106,7 @@ static void test_process(DWORD curr_pid, DWORD sub_pcs_pid)
     MODULEENTRY32       me;
     unsigned            found = 0;
     int                 num = 0;
+    int			childpos = -1;
 
     hSnapshot = pCreateToolhelp32Snapshot( TH32CS_SNAPPROCESS, 0 );
     ok(hSnapshot != NULL, "Cannot create snapshot\n");
@@ -119,7 +118,7 @@ static void test_process(DWORD curr_pid, DWORD sub_pcs_pid)
         do
         {
             if (pe.th32ProcessID == curr_pid) found++;
-            if (pe.th32ProcessID == sub_pcs_pid) found++;
+            if (pe.th32ProcessID == sub_pcs_pid) { childpos = num; found++; }
             trace("PID=%x %s\n", pe.th32ProcessID, pe.szExeFile);
             num++;
         } while (pProcess32Next( hSnapshot, &pe ));
@@ -141,6 +140,10 @@ static void test_process(DWORD curr_pid, DWORD sub_pcs_pid)
     ok(found == 2, "couldn't find self and/or sub-process in process list\n");
     ok(!num, "mismatch in counting\n");
 
+    /* one broken program does Process32First() and does not expect anything
+     * interesting to be there, especially not the just forked off child */
+    ok (childpos !=0, "child is not expected to be at position 0.\n");
+
     te.dwSize = sizeof(te);
     ok(!pThread32First( hSnapshot, &te ), "shouldn't return a thread\n");
 
@@ -339,8 +342,5 @@ START_TEST(toolhelp)
     test_module(info.dwProcessId, sub_expected_modules, NUM_OF(sub_expected_modules));
 
     SetEvent(ev2);
-    w = WaitForSingleObject(info.hProcess, WAIT_TIME);
-    ok(w == WAIT_OBJECT_0, "Failed to wait on sub-process termination\n");
-    ok(GetExitCodeProcess(info.hProcess, &w), "couldn't get process exit code\n");
-    ok(w == WAIT_OBJECT_0, "Sub-Process failed to terminate properly\n");
+    winetest_wait_child_process( info.hProcess );
 }
