@@ -21,15 +21,12 @@
 #include "wine/test.h"
 #include "winbase.h"
 
-#define CDROM   "CDROM"
-#define FLOPPY  "FLOPPY"
-#define HARDISK "HARDDISK"
-#define LANMAN  "LANMANREDIRECTOR"
-#define RAMDISK "RAMDISK"
-
 static HINSTANCE hdll;
 static BOOL (WINAPI * pGetVolumeNameForVolumeMountPointA)(LPCSTR, LPSTR, DWORD);
 static BOOL (WINAPI * pGetVolumeNameForVolumeMountPointW)(LPCWSTR, LPWSTR, DWORD);
+static HANDLE (WINAPI *pFindFirstVolumeA)(LPSTR,DWORD);
+static BOOL (WINAPI *pFindNextVolumeA)(HANDLE,LPSTR,DWORD);
+static BOOL (WINAPI *pFindVolumeClose)(HANDLE);
 
 /* ############################### */
 
@@ -38,17 +35,49 @@ static void test_query_dos_deviceA(void)
     char drivestr[] = "a:";
     char *p, buffer[2000];
     DWORD ret;
+    BOOL found = FALSE;
+
     for (;drivestr[0] <= 'z'; drivestr[0]++) {
         ret = QueryDosDeviceA( drivestr, buffer, sizeof(buffer));
         if(ret) {
             for (p = buffer; *p; p++) *p = toupper(*p);
-            todo_wine
-            ok( strstr( buffer, CDROM)   || strstr( buffer, FLOPPY) ||
-                strstr( buffer, HARDISK) || strstr( buffer, LANMAN) ||
-                strstr( buffer, RAMDISK), "expect the string %s contains %s,%s,%s,%s or %s\n",
-                buffer, CDROM, FLOPPY, HARDISK, LANMAN, RAMDISK);
+            if (strstr(buffer, "HARDDISK") || strstr(buffer, "RAMDISK")) found = TRUE;
         }
     }
+    todo_wine ok(found, "expected at least one devicename to contain HARDDISK or RAMDISK\n");
+}
+
+static void test_FindFirstVolume(void)
+{
+    char volume[50];
+    HANDLE handle;
+
+    if (!pFindFirstVolumeA) {
+        skip("FindFirstVolumeA not found\n");
+        return;
+    }
+
+    handle = pFindFirstVolumeA( volume, 0 );
+    ok( handle == INVALID_HANDLE_VALUE, "succeeded with short buffer\n" );
+    ok( GetLastError() == ERROR_MORE_DATA ||  /* XP */
+        GetLastError() == ERROR_FILENAME_EXCED_RANGE,  /* Vista */
+        "wrong error %u\n", GetLastError() );
+    handle = pFindFirstVolumeA( volume, 49 );
+    ok( handle == INVALID_HANDLE_VALUE, "succeeded with short buffer\n" );
+    ok( GetLastError() == ERROR_FILENAME_EXCED_RANGE, "wrong error %u\n", GetLastError() );
+    handle = pFindFirstVolumeA( volume, 50 );
+    ok( handle != INVALID_HANDLE_VALUE, "failed err %u\n", GetLastError() );
+    if (handle != INVALID_HANDLE_VALUE)
+    {
+        do
+        {
+            ok( strlen(volume) == 49, "bad volume name %s\n", volume );
+            ok( !memcmp( volume, "\\\\?\\Volume{", 11 ), "bad volume name %s\n", volume );
+            ok( !memcmp( volume + 47, "}\\", 2 ), "bad volume name %s\n", volume );
+        } while (pFindNextVolumeA( handle, volume, MAX_PATH ));
+        ok( GetLastError() == ERROR_NO_MORE_FILES, "wrong error %u\n", GetLastError() );
+        pFindVolumeClose( handle );
+    }
 }
 
 static void test_GetVolumeNameForVolumeMountPointA(void)
@@ -110,8 +139,12 @@ START_TEST(volume)
     hdll = GetModuleHandleA("kernel32.dll");
     pGetVolumeNameForVolumeMountPointA = (void *) GetProcAddress(hdll, "GetVolumeNameForVolumeMountPointA");
     pGetVolumeNameForVolumeMountPointW = (void *) GetProcAddress(hdll, "GetVolumeNameForVolumeMountPointW");
+    pFindFirstVolumeA = (void *) GetProcAddress(hdll, "FindFirstVolumeA");
+    pFindNextVolumeA = (void *) GetProcAddress(hdll, "FindNextVolumeA");
+    pFindVolumeClose = (void *) GetProcAddress(hdll, "FindVolumeClose");
 
     test_query_dos_deviceA();
+    test_FindFirstVolume();
     test_GetVolumeNameForVolumeMountPointA();
     test_GetVolumeNameForVolumeMountPointW();
 }
