@@ -83,7 +83,7 @@ SCODE WINAPI PropCopyMore(LPSPropValue lpDest, LPSPropValue lpSrc,
     case PT_CLSID:
         scode = lpMore(sizeof(GUID), lpOrig, (LPVOID*)&lpDest->Value.lpguid);
         if (SUCCEEDED(scode))
-            memcpy(lpDest->Value.lpguid, lpSrc->Value.lpguid, sizeof(GUID));
+            *lpDest->Value.lpguid = *lpSrc->Value.lpguid;
         break;
     case PT_STRING8:
         ulLen = lstrlenA(lpSrc->Value.lpszA) + 1u;
@@ -810,7 +810,7 @@ SCODE WINAPI ScCopyProps(int cValues, LPSPropValue lpProps, LPVOID lpDst, ULONG
         {
         case PT_CLSID:
             lpDest->Value.lpguid = (LPGUID)lpDataDest;
-            memcpy(lpDest->Value.lpguid, lpProps->Value.lpguid, sizeof(GUID));
+            *lpDest->Value.lpguid = *lpProps->Value.lpguid;
             lpDataDest += sizeof(GUID);
             break;
         case PT_STRING8:
@@ -928,7 +928,7 @@ SCODE WINAPI ScRelocProps(int cValues, LPSPropValue lpProps, LPVOID lpOld,
                           LPVOID lpNew, ULONG *lpCount)
 {
     static const BOOL bBadPtr = TRUE; /* Windows bug - Assumes source is bad */
-    LPSPropValue lpDest = (LPSPropValue)lpProps;
+    LPSPropValue lpDest = lpProps;
     ULONG ulCount = cValues * sizeof(SPropValue);
     ULONG ulLen, i;
     int iter;
@@ -964,7 +964,7 @@ SCODE WINAPI ScRelocProps(int cValues, LPSPropValue lpProps, LPVOID lpOld,
             break;
         case PT_STRING8:
             ulLen = bBadPtr ? 0 : lstrlenA(lpDest->Value.lpszA) + 1u;
-            lpDest->Value.lpszA = (LPSTR)RELOC_PTR(lpDest->Value.lpszA);
+            lpDest->Value.lpszA = RELOC_PTR(lpDest->Value.lpszA);
             if (bBadPtr)
                 ulLen = lstrlenA(lpDest->Value.lpszA) + 1u;
             ulCount += ulLen;
@@ -999,7 +999,7 @@ SCODE WINAPI ScRelocProps(int cValues, LPSPropValue lpProps, LPVOID lpOld,
                     {
                         ULONG ulStrLen = bBadPtr ? 0 : lstrlenA(lpDest->Value.MVszA.lppszA[i]) + 1u;
 
-                        lpDest->Value.MVszA.lppszA[i] = (LPSTR)RELOC_PTR(lpDest->Value.MVszA.lppszA[i]);
+                        lpDest->Value.MVszA.lppszA[i] = RELOC_PTR(lpDest->Value.MVszA.lppszA[i]);
                         if (bBadPtr)
                             ulStrLen = lstrlenA(lpDest->Value.MVszA.lppszA[i]) + 1u;
                         ulCount += ulStrLen;
@@ -1331,7 +1331,7 @@ ULONG WINAPI FBadProp(LPSPropValue lpProp)
         return FBadRglpszW(lpProp->Value.MVszW.lppszW,
                            lpProp->Value.MVszW.cValues);
     case PT_MV_BINARY:
-        return FBadEntryList((LPENTRYLIST)&lpProp->Value.MVbin);
+        return FBadEntryList(&lpProp->Value.MVbin);
     }
     return FALSE;
 }
