@@ -216,7 +216,7 @@ static void test_multibyte_to_unicode_translations(IMultiLanguage2 *iML2)
     ok(lenA == expected_len, "expected lenA %u, got %u\n", expected_len, lenA);
 }
 
-inline static void cpinfo_cmp(MIMECPINFO *cpinfo1, MIMECPINFO *cpinfo2)
+static inline void cpinfo_cmp(MIMECPINFO *cpinfo1, MIMECPINFO *cpinfo2)
 {
     ok(cpinfo1->dwFlags == cpinfo2->dwFlags, "dwFlags mismatch: %08x != %08x\n", cpinfo1->dwFlags, cpinfo2->dwFlags);
     ok(cpinfo1->uiCodePage == cpinfo2->uiCodePage, "uiCodePage mismatch: %u != %u\n", cpinfo1->uiCodePage, cpinfo2->uiCodePage);
@@ -499,7 +499,7 @@ static void test_EnumCodePages(IMultiLanguage2 *iML2, DWORD flags)
     IEnumCodePage_Release(iEnumCP);
 }
 
-inline static void scriptinfo_cmp(SCRIPTINFO *sinfo1, SCRIPTINFO *sinfo2)
+static inline void scriptinfo_cmp(SCRIPTINFO *sinfo1, SCRIPTINFO *sinfo2)
 {
     ok(sinfo1->ScriptId == sinfo2->ScriptId, "ScriptId mismatch: %d != %d\n", sinfo1->ScriptId, sinfo2->ScriptId);
     ok(sinfo1->uiCodePage == sinfo2->uiCodePage, "uiCodePage mismatch: %u != %u\n", sinfo1->uiCodePage, sinfo2->uiCodePage);
@@ -677,6 +677,61 @@ static void test_rfc1766(IMultiLanguage2 *iML2)
         ok(n == 1, "couldn't fetch 1 RFC1766INFO structure\n");
         ok(IsValidLocale(info.lcid, LCID_SUPPORTED), "invalid lcid %04x\n", info.lcid);
     }
+    IEnumRfc1766_Release(pEnumRfc1766);
+}
+
+static void test_GetLcidFromRfc1766(IMultiLanguage2 *iML2)
+{
+    LCID lcid;
+    HRESULT ret;
+
+    static WCHAR e[] = { 'e',0 };
+    static WCHAR en[] = { 'e','n',0 };
+    static WCHAR empty[] = { 0 };
+    static WCHAR dash[] = { '-',0 };
+    static WCHAR e_dash[] = { 'e','-',0 };
+    static WCHAR en_gb[] = { 'e','n','-','g','b',0 };
+    static WCHAR en_us[] = { 'e','n','-','u','s',0 };
+    static WCHAR en_them[] = { 'e','n','-','t','h','e','m',0 };
+    static WCHAR english[] = { 'e','n','g','l','i','s','h',0 };
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, NULL, en);
+    ok(ret == E_INVALIDARG, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, NULL);
+    ok(ret == E_INVALIDARG, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, e);
+    ok(ret == E_FAIL, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, empty);
+    ok(ret == E_FAIL, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, dash);
+    ok(ret == E_FAIL, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, e_dash);
+    ok(ret == E_FAIL, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, en_them);
+    ok(ret == E_FAIL, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, english);
+    ok(ret == E_FAIL, "GetLcidFromRfc1766 returned: %08x\n", ret);
+
+    lcid = 0;
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, en);
+    ok(ret == S_OK, "GetLcidFromRfc1766 returned: %08x\n", ret);
+    ok(lcid == 9, "got wrong lcid: %04x\n", lcid);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, en_gb);
+    ok(ret == S_OK, "GetLcidFromRfc1766 returned: %08x\n", ret);
+    ok(lcid == 0x809, "got wrong lcid: %04x\n", lcid);
+
+    ret = IMultiLanguage2_GetLcidFromRfc1766(iML2, &lcid, en_us);
+    ok(ret == S_OK, "GetLcidFromRfc1766 returned: %08x\n", ret);
+    ok(lcid == 0x409, "got wrong lcid: %04x\n", lcid);
 }
 
 START_TEST(mlang)
@@ -696,6 +751,7 @@ START_TEST(mlang)
     if (ret != S_OK || !iML2) return;
 
     test_rfc1766(iML2);
+    test_GetLcidFromRfc1766(iML2);
 
     test_EnumCodePages(iML2, 0);
     test_EnumCodePages(iML2, MIMECONTF_MIME_LATEST);
