@@ -99,6 +99,24 @@ WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 static const WCHAR wszFont[] = {'f','o','n','t',0};
 static const WCHAR wszSize[] = {'s','i','z','e',0};
 
+void set_dirty(HTMLDocument *This, VARIANT_BOOL dirty)
+{
+    nsresult nsres;
+
+    if(This->usermode != EDITMODE || !This->nscontainer || !This->nscontainer->editor)
+        return;
+
+    if(dirty) {
+        nsres = nsIEditor_IncrementModificationCount(This->nscontainer->editor, 1);
+        if(NS_FAILED(nsres))
+            ERR("IncrementModificationCount failed: %08x\n", nsres);
+    }else {
+        nsres = nsIEditor_ResetModificationCount(This->nscontainer->editor);
+        if(NS_FAILED(nsres))
+            ERR("ResetModificationCount failed: %08x\n", nsres);
+    }
+}
+
 static void do_ns_editor_command(NSContainer *This, const char *cmd)
 {
     nsresult nsres;
@@ -235,7 +253,7 @@ static void remove_child_attr(nsIDOMElement *elem, LPCWSTR tag, nsAString *attr_
 
             nsAString_Init(&tag_str, NULL);
             nsIDOMElement_GetTagName(child_elem, &tag_str);
-            nsAString_GetData(&tag_str, &ctag, NULL);
+            nsAString_GetData(&tag_str, &ctag);
 
             if(!strcmpiW(ctag, tag))
                 /* FIXME: remove node if there are no more attributes */
@@ -282,7 +300,7 @@ static void get_font_size(HTMLDocument *This, WCHAR *ret)
 
             nsAString_Init(&tag_str, NULL);
             nsIDOMElement_GetTagName(elem, &tag_str);
-            nsAString_GetData(&tag_str, &tag, NULL);
+            nsAString_GetData(&tag_str, &tag);
 
             if(!strcmpiW(tag, wszFont)) {
                 nsAString size_str, val_str;
@@ -294,7 +312,7 @@ static void get_font_size(HTMLDocument *This, WCHAR *ret)
                 nsAString_Init(&val_str, NULL);
 
                 nsIDOMElement_GetAttribute(elem, &size_str, &val_str);
-                nsAString_GetData(&val_str, &val, NULL);
+                nsAString_GetData(&val_str, &val);
 
                 if(*val) {
                     TRACE("found size %s\n", debugstr_w(val));
@@ -383,71 +401,8 @@ static void set_font_size(HTMLDocument *This, LPCWSTR size)
 
     nsISelection_Release(nsselection);
     nsIDOMDocument_Release(nsdoc);
-}
 
-static BOOL is_visible_text_node(nsIDOMNode *node)
-{
-    nsIDOMCharacterData *char_data;
-    nsAString data_str;
-    LPCWSTR data, ptr;
-    PRUint32 len;
-
-    nsIDOMNode_QueryInterface(node, &IID_nsIDOMCharacterData, (void**)&char_data);
-
-    nsIDOMCharacterData_GetLength(char_data, &len);
-
-    nsAString_Init(&data_str, NULL);
-    nsIDOMCharacterData_GetData(char_data, &data_str);
-    nsAString_GetData(&data_str, &data, NULL);
-
-    if(*data == '\n') {
-        len--;
-        for(ptr=data+1; ptr && isspaceW(*ptr); ptr++)
-            len--;
-    }
-
-    nsAString_Finish(&data_str);
-
-    nsIDOMCharacterData_Release(char_data);
-
-    return len != 0;
-}
-
-static nsIDOMNode *get_child_text_node(nsIDOMNode *node, BOOL first)
-{
-    nsIDOMNode *iter, *iter2;
-
-    if(first)
-        nsIDOMNode_GetFirstChild(node, &iter);
-    else
-        nsIDOMNode_GetLastChild(node, &iter);
-
-    while(iter) {
-        PRUint16 node_type;
-
-        nsIDOMNode_GetNodeType(iter, &node_type);
-        switch(node_type) {
-        case TEXT_NODE:
-            if(is_visible_text_node(iter))
-                return iter;
-        case ELEMENT_NODE:
-            iter2 = get_child_text_node(iter, first);
-            if(iter2) {
-                nsIDOMNode_Release(iter);
-                return iter2;
-            }
-        }
-
-        if(first)
-            nsIDOMNode_GetNextSibling(iter, &iter2);
-        else
-            nsIDOMNode_GetPreviousSibling(iter, &iter2);
-
-        nsIDOMNode_Release(iter);
-        iter = iter2;
-    }
-
-    return NULL;
+    set_dirty(This, VARIANT_TRUE);
 }
 
 static void handle_arrow_key(HTMLDocument *This, nsIDOMKeyEvent *event, const char * const cmds[4])
@@ -569,30 +524,8 @@ void handle_edit_event(HTMLDocument *This, nsIDOMEvent *event)
 
 void handle_edit_load(HTMLDocument *This)
 {
+    This->nscontainer->reset_focus = GetFocus();
     get_editor_controller(This->nscontainer);
-
-    if(This->ui_active) {
-        OLECHAR wszHTMLDocument[30];
-        RECT rcBorderWidths;
-
-        if(This->ip_window)
-            IOleInPlaceUIWindow_SetActiveObject(This->ip_window, NULL, NULL);
-        if(This->hostui)
-            IDocHostUIHandler_HideUI(This->hostui);
-
-        if(This->hostui)
-            IDocHostUIHandler_ShowUI(This->hostui, DOCHOSTUITYPE_AUTHOR, ACTOBJ(This), CMDTARGET(This),
-                This->frame, This->ip_window);
-
-        LoadStringW(hInst, IDS_HTMLDOCUMENT, wszHTMLDocument,
-                    sizeof(wszHTMLDocument)/sizeof(WCHAR));
-
-        if(This->ip_window)
-            IOleInPlaceUIWindow_SetActiveObject(This->ip_window, ACTOBJ(This), wszHTMLDocument);
-
-        memset(&rcBorderWidths, 0, sizeof(rcBorderWidths));
-        IOleInPlaceFrame_SetBorderSpace(This->frame, &rcBorderWidths);
-    }
 }
 
 static void set_ns_fontname(NSContainer *This, const char *fontname)
@@ -626,7 +559,6 @@ static HRESULT exec_fontname(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
 
     if(in) {
         char *stra;
-        DWORD len;
 
         if(V_VT(in) != VT_BSTR) {
             FIXME("Unsupported vt=%d\n", V_VT(out));
@@ -635,13 +567,9 @@ static HRESULT exec_fontname(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
 
         TRACE("%s\n", debugstr_w(V_BSTR(in)));
 
-        len = WideCharToMultiByte(CP_ACP, 0, V_BSTR(in), -1, NULL, 0, NULL, NULL);
-        stra = mshtml_alloc(len);
-        WideCharToMultiByte(CP_ACP, 0, V_BSTR(in), -1, stra, -1, NULL, NULL);
-
+        stra = heap_strdupWtoA(V_BSTR(in));
         set_ns_fontname(This->nscontainer, stra);
-
-        mshtml_free(stra);
+        heap_free(stra);
 
         update_doc(This, UPDATE_UI);
     }
@@ -653,6 +581,9 @@ static HRESULT exec_fontname(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
         DWORD len;
         nsresult nsres;
 
+        V_VT(out) = VT_BSTR;
+        V_BSTR(out) = NULL;
+
         nsparam = create_nscommand_params();
 
         nsres = get_ns_command_state(This->nscontainer, NSCMD_FONTFACE, nsparam);
@@ -663,13 +594,12 @@ static HRESULT exec_fontname(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
         nsICommandParams_Release(nsparam);
 
         len = MultiByteToWideChar(CP_ACP, 0, stra, -1, NULL, 0);
-        strw = mshtml_alloc(len*sizeof(WCHAR));
+        strw = heap_alloc(len*sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, stra, -1, strw, -1);
         nsfree(stra);
 
-        V_VT(out) = VT_BSTR;
         V_BSTR(out) = SysAllocString(strw);
-        mshtml_free(strw);
+        heap_free(strw);
     }
 
     return S_OK;
@@ -1037,6 +967,21 @@ HRESULT editor_exec_paste(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VAR
     return S_OK;
 }
 
+static HRESULT exec_setdirty(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
+{
+    TRACE("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
+
+    if(!in)
+        return S_OK;
+
+    if(V_VT(in) == VT_BOOL)
+        set_dirty(This, V_BOOL(in));
+    else
+        FIXME("unsupported vt=%d\n", V_VT(in));
+
+    return S_OK;
+}
+
 static HRESULT query_edit_status(HTMLDocument *This, OLECMD *cmd)
 {
     switch(cmd->cmdID) {
@@ -1210,11 +1155,12 @@ static HRESULT exec_hyperlink(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
 {
     BSTR url = NULL;
     INT ret;
-    nsAString ns_url;
-    PRBool insert_link_at_caret;
     nsISelection *nsselection;
+    nsIDOMDocument *nsdoc;
+    nsresult nsres;
+    HRESULT hres = E_FAIL;
 
-    FIXME("%p, 0x%x, %p, %p\n", This, cmdexecopt, in, out);
+    TRACE("%p, 0x%x, %p, %p\n", This, cmdexecopt, in, out);
 
     if (cmdexecopt == OLECMDEXECOPT_DONTPROMPTUSER)
     {
@@ -1236,29 +1182,23 @@ static HRESULT exec_hyperlink(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
     if (!nsselection)
         return E_FAIL;
 
-    nsAString_Init(&ns_url, url);
-
-    nsISelection_GetIsCollapsed(nsselection, &insert_link_at_caret);
-
-    if (insert_link_at_caret)
+    nsres = nsIWebNavigation_GetDocument(This->nscontainer->navigation, &nsdoc);
+    if(NS_SUCCEEDED(nsres))
     {
         static const WCHAR wszA[] = {'a',0};
         static const WCHAR wszHref[] = {'h','r','e','f',0};
         nsIHTMLEditor *html_editor;
-        nsIDOMDocument *nsdoc;
         nsIDOMNode *text_node;
         nsIDOMElement *anchor_elem;
         nsIDOMNode *unused_node;
         nsAString a_str;
         nsAString href_str;
-        nsresult nsres;
-
-        nsres = nsIWebNavigation_GetDocument(This->nscontainer->navigation, &nsdoc);
-        if(NS_FAILED(nsres))
-            return E_FAIL;
+        nsAString ns_url;
+        PRBool insert_link_at_caret;
 
         nsAString_Init(&a_str, wszA);
         nsAString_Init(&href_str, wszHref);
+        nsAString_Init(&ns_url, url);
 
         /* create an element for the link */
         nsIDOMDocument_CreateElement(nsdoc, &a_str, &anchor_elem);
@@ -1267,44 +1207,48 @@ static HRESULT exec_hyperlink(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
         nsAString_Finish(&href_str);
         nsAString_Finish(&a_str);
 
-        /* create an element with text of URL */
-        nsIDOMDocument_CreateTextNode(nsdoc, &ns_url, (nsIDOMText **)&text_node);
+        nsISelection_GetIsCollapsed(nsselection, &insert_link_at_caret);
 
-        /* wrap the <a> tags around the text element */
-        nsIDOMElement_AppendChild(anchor_elem, text_node, &unused_node);
-        nsIDOMNode_Release(text_node);
-        nsIDOMNode_Release(unused_node);
+        /* create an element with text of URL */
+        if (insert_link_at_caret)
+        {
+            nsIDOMDocument_CreateTextNode(nsdoc, &ns_url, (nsIDOMText **)&text_node);
+
+            /* wrap the <a> tags around the text element */
+            nsIDOMElement_AppendChild(anchor_elem, text_node, &unused_node);
+            nsIDOMNode_Release(text_node);
+            nsIDOMNode_Release(unused_node);
+        }
+
+        nsAString_Finish(&ns_url);
 
         nsIEditor_QueryInterface(This->nscontainer->editor, &IID_nsIHTMLEditor, (void **)&html_editor);
         if (html_editor)
         {
-            /* add them to the document at the caret position */
-            nsres = nsIHTMLEditor_InsertElementAtSelection(html_editor, anchor_elem, FALSE);
+            if (insert_link_at_caret)
+            {
+                /* add them to the document at the caret position */
+                nsres = nsIHTMLEditor_InsertElementAtSelection(html_editor, anchor_elem, FALSE);
+                nsISelection_SelectAllChildren(nsselection, (nsIDOMNode*)anchor_elem);
+            }
+            else /* add them around the selection using the magic provided to us by nsIHTMLEditor */
+                nsres = nsIHTMLEditor_InsertLinkAroundSelection(html_editor, anchor_elem);
             nsIHTMLEditor_Release(html_editor);
         }
 
-        nsISelection_SelectAllChildren(nsselection, (nsIDOMNode*)anchor_elem);
-
         nsIDOMElement_Release(anchor_elem);
         nsIDOMDocument_Release(nsdoc);
-    }
-    else
-    {
-        nsICommandParams *nsparam = create_nscommand_params();
 
-        nsICommandParams_SetStringValue(nsparam, NSSTATE_ATTRIBUTE, &ns_url);
-        do_ns_command(This->nscontainer, NSCMD_INSERTLINKNOUI, nsparam);
-        nsICommandParams_Release(nsparam);
+        hres = NS_SUCCEEDED(nsres) ? S_OK : E_FAIL;
     }
 
-    nsAString_Finish(&ns_url);
-
     nsISelection_Release(nsselection);
 
     if (cmdexecopt != OLECMDEXECOPT_DONTPROMPTUSER)
         SysFreeString(url);
 
-    return S_OK;
+    TRACE("-- 0x%08x\n", nsres);
+    return hres;
 }
 
 static HRESULT query_selall_status(HTMLDocument *This, OLECMD *cmd)
@@ -1335,6 +1279,7 @@ const cmdtable_t editmode_cmds[] = {
     {IDM_OUTDENT,         query_edit_status,    exec_outdent},
     {IDM_COMPOSESETTINGS, NULL,                 exec_composesettings},
     {IDM_HYPERLINK,       query_edit_status,    exec_hyperlink},
+    {IDM_SETDIRTY,        NULL,                 exec_setdirty},
     {0,NULL,NULL}
 };
 
