@@ -41,22 +41,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
-#define NSCMD_BOLD "cmd_bold"
 #define NSCMD_COPY "cmd_copy"
-#define NSCMD_ITALIC "cmd_italic"
-#define NSCMD_UNDERLINE "cmd_underline"
-#define NSCMD_ALIGN "cmd_align"
-#define NSCMD_INDENT "cmd_indent"
-#define NSCMD_OUTDENT "cmd_outdent"
-#define NSCMD_INSERTHR "cmd_insertHR"
-#define NSCMD_UL "cmd_ul"
-#define NSCMD_OL "cmd_ol"
-
-#define NSSTATE_ATTRIBUTE "state_attribute"
-
-#define NSALIGN_CENTER "center"
-#define NSALIGN_LEFT   "left"
-#define NSALIGN_RIGHT  "right"
 
 void do_ns_command(NSContainer *This, const char *cmd, nsICommandParams *nsparam)
 {
@@ -416,6 +401,12 @@ static HRESULT exec_stop_download(HTMLDocument *This, DWORD nCmdexecopt, VARIANT
     return E_NOTIMPL;
 }
 
+static HRESULT exec_find(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
+{
+    FIXME("(%p)->(%d %p %p)\n", This, nCmdexecopt, pvaIn, pvaOut);
+    return E_NOTIMPL;
+}
+
 static HRESULT exec_delete(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
 {
     FIXME("(%p)->(%d %p %p)\n", This, nCmdexecopt, pvaIn, pvaOut);
@@ -543,8 +534,6 @@ static HRESULT exec_editmode(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
     IMoniker *mon;
     HRESULT hres;
 
-    static const WCHAR wszAboutBlank[] = {'a','b','o','u','t',':','b','l','a','n','k',0};
-
     TRACE("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
 
     if(in || out)
@@ -555,9 +544,20 @@ static HRESULT exec_editmode(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
 
     This->usermode = EDITMODE;
 
+    if(This->mon) {
+        CLSID clsid = IID_NULL;
+        hres = IMoniker_GetClassID(This->mon, &clsid);
+        if(SUCCEEDED(hres)) {
+            /* We should use IMoniker::Save here */
+            FIXME("Use CLSID %s\n", debugstr_guid(&clsid));
+        }
+    }
+
     if(This->frame)
         IOleInPlaceFrame_SetStatusText(This->frame, NULL);
 
+    This->readystate = READYSTATE_UNINITIALIZED;
+
     if(This->client) {
         IOleCommandTarget *cmdtrg;
 
@@ -590,15 +590,50 @@ static HRESULT exec_editmode(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in,
     if(This->nscontainer)
         set_ns_editmode(This->nscontainer);
 
-    hres = CreateURLMoniker(NULL, wszAboutBlank, &mon);
-    if(FAILED(hres)) {
-        FIXME("CreateURLMoniker failed: %08x\n", hres);
-        return hres;
-    }
-
     update_doc(This, UPDATE_UI);
 
-    return IPersistMoniker_Load(PERSISTMON(This), TRUE, mon, NULL, 0);
+    if(This->mon) {
+        /* FIXME: We should find nicer way to do this */
+        remove_doc_tasks(This);
+
+        mon = This->mon;
+        IMoniker_AddRef(mon);
+    }else {
+        static const WCHAR about_blankW[] = {'a','b','o','u','t',':','b','l','a','n','k',0};
+
+        hres = CreateURLMoniker(NULL, about_blankW, &mon);
+        if(FAILED(hres)) {
+            FIXME("CreateURLMoniker failed: %08x\n", hres);
+            return hres;
+        }
+    }
+
+    hres = IPersistMoniker_Load(PERSISTMON(This), TRUE, mon, NULL, 0);
+    IMoniker_Release(mon);
+    if(FAILED(hres))
+        return hres;
+
+    if(This->ui_active) {
+        RECT rcBorderWidths;
+
+        if(This->ip_window)
+            call_set_active_object(This->ip_window, NULL);
+        if(This->hostui)
+            IDocHostUIHandler_HideUI(This->hostui);
+
+        if(This->hostui)
+            IDocHostUIHandler_ShowUI(This->hostui, DOCHOSTUITYPE_AUTHOR, ACTOBJ(This), CMDTARGET(This),
+                This->frame, This->ip_window);
+
+        if(This->ip_window)
+            call_set_active_object(This->ip_window, ACTOBJ(This));
+
+        memset(&rcBorderWidths, 0, sizeof(rcBorderWidths));
+        if (This->frame)
+            IOleInPlaceFrame_SetBorderSpace(This->frame, &rcBorderWidths);
+    }
+
+    return S_OK;
 }
 
 static HRESULT exec_htmleditmode(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
@@ -607,12 +642,6 @@ static HRESULT exec_htmleditmode(HTMLDocument *This, DWORD cmdexecopt, VARIANT *
     return S_OK;
 }
 
-static HRESULT exec_setdirty(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
-{
-    FIXME("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
-    return E_NOTIMPL;
-}
-
 static HRESULT exec_baselinefont3(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
 {
     FIXME("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
@@ -676,7 +705,8 @@ static const struct {
     { OLECMDF_SUPPORTED|OLECMDF_ENABLED,  exec_stop                 }, /* OLECMDID_STOP */
     {0},{0},{0},{0},{0},{0},
     { OLECMDF_SUPPORTED,                  exec_stop_download        }, /* OLECMDID_STOPDOWNLOAD */
-    {0},{0},
+    {0},
+    { OLECMDF_SUPPORTED|OLECMDF_ENABLED,  exec_find                 }, /* OLECMDID_FIND */
     { OLECMDF_SUPPORTED,                  exec_delete               }, /* OLECMDID_DELETE */
     {0},{0},
     { OLECMDF_SUPPORTED,                  exec_enable_interaction   }, /* OLECMDID_ENABLE_INTERACTION */
@@ -698,7 +728,6 @@ static const cmdtable_t base_cmds[] = {
     {IDM_BROWSEMODE,       NULL,                  exec_browsemode},
     {IDM_EDITMODE,         NULL,                  exec_editmode},
     {IDM_PRINT,            query_enabled_stub,    exec_print},
-    {IDM_SETDIRTY,         NULL,                  exec_setdirty},
     {IDM_HTMLEDITMODE,     NULL,                  exec_htmleditmode},
     {IDM_BASELINEFONT3,    NULL,                  exec_baselinefont3},
     {IDM_BLOCKDIRLTR,      query_enabled_stub,    NULL},
@@ -867,14 +896,14 @@ static const IOleCommandTargetVtbl OleCommandTargetVtbl = {
     OleCommandTarget_Exec
 };
 
-void show_context_menu(HTMLDocument *This, DWORD dwID, POINT *ppt)
+void show_context_menu(HTMLDocument *This, DWORD dwID, POINT *ppt, IDispatch *elem)
 {
     HMENU menu_res, menu;
     DWORD cmdid;
     HRESULT hres;
 
     hres = IDocHostUIHandler_ShowContextMenu(This->hostui, dwID, ppt,
-            (IUnknown*)CMDTARGET(This), (IDispatch*)HTMLDOC(This));
+            (IUnknown*)CMDTARGET(This), elem);
     if(hres == S_OK)
         return;
 
