@@ -42,6 +42,9 @@ typedef struct {
     LONG ref;
 
     nsISelection *nsselection;
+    HTMLDocument *doc;
+
+    struct list entry;
 } HTMLSelectionObject;
 
 #define HTMLSELOBJ(x)  ((IHTMLSelectionObject*) &(x)->lpHTMLSelectionObjectVtbl)
@@ -95,7 +98,9 @@ static ULONG WINAPI HTMLSelectionObject_Release(IHTMLSelectionObject *iface)
     if(!ref) {
         if(This->nsselection)
             nsISelection_Release(This->nsselection);
-        mshtml_free(This);
+        if(This->doc)
+            list_remove(&This->entry);
+        heap_free(This);
     }
 
     return ref;
@@ -139,10 +144,52 @@ static HRESULT WINAPI HTMLSelectionObject_Invoke(IHTMLSelectionObject *iface, DI
 static HRESULT WINAPI HTMLSelectionObject_createRange(IHTMLSelectionObject *iface, IDispatch **range)
 {
     HTMLSelectionObject *This = HTMLSELOBJ_THIS(iface);
+    nsIDOMRange *nsrange = NULL;
 
     TRACE("(%p)->(%p)\n", This, range);
 
-    *range = (IDispatch*)HTMLTxtRange_Create(This->nsselection);
+    if(This->nsselection) {
+        PRInt32 nsrange_cnt = 0;
+        nsresult nsres;
+
+        nsISelection_GetRangeCount(This->nsselection, &nsrange_cnt);
+        if(!nsrange_cnt) {
+            nsIDOMDocument *nsdoc;
+            nsIDOMHTMLDocument *nshtmldoc;
+            nsIDOMHTMLElement *nsbody = NULL;
+
+            TRACE("nsrange_cnt = 0\n");
+
+            nsres = nsIWebNavigation_GetDocument(This->doc->nscontainer->navigation, &nsdoc);
+            if(NS_FAILED(nsres) || !nsdoc) {
+                ERR("GetDocument failed: %08x\n", nsres);
+                return E_FAIL;
+            }
+
+            nsIDOMDocument_QueryInterface(nsdoc, &IID_nsIDOMHTMLDocument, (void**)&nshtmldoc);
+            nsIDOMDocument_Release(nsdoc);
+
+            nsres = nsIDOMHTMLDocument_GetBody(nshtmldoc, &nsbody);
+            nsIDOMHTMLDocument_Release(nshtmldoc);
+            if(NS_FAILED(nsres) || !nsbody) {
+                ERR("Could not get body: %08x\n", nsres);
+                return E_FAIL;
+            }
+
+            nsres = nsISelection_Collapse(This->nsselection, (nsIDOMNode*)nsbody, 0);
+            nsIDOMHTMLElement_Release(nsbody);
+            if(NS_FAILED(nsres))
+                ERR("Collapse failed: %08x\n", nsres);
+        }else if(nsrange_cnt > 1) {
+            FIXME("range_cnt = %d\n", nsrange_cnt);
+        }
+
+        nsres = nsISelection_GetRangeAt(This->nsselection, 0, &nsrange);
+        if(NS_FAILED(nsres))
+            ERR("GetRangeAt failed: %08x\n", nsres);
+    }
+
+    *range = (IDispatch*)HTMLTxtRange_Create(This->doc, nsrange);
     return S_OK;
 }
 
@@ -163,7 +210,7 @@ static HRESULT WINAPI HTMLSelectionObject_clear(IHTMLSelectionObject *iface)
 static HRESULT WINAPI HTMLSelectionObject_get_type(IHTMLSelectionObject *iface, BSTR *p)
 {
     HTMLSelectionObject *This = HTMLSELOBJ_THIS(iface);
-    PRInt32 range_cnt = 0;
+    PRBool collapsed = TRUE;
 
     static const WCHAR wszNone[] = {'N','o','n','e',0};
     static const WCHAR wszText[] = {'T','e','x','t',0};
@@ -171,9 +218,10 @@ static HRESULT WINAPI HTMLSelectionObject_get_type(IHTMLSelectionObject *iface,
     TRACE("(%p)->(%p)\n", This, p);
 
     if(This->nsselection)
-        nsISelection_GetRangeCount(This->nsselection, &range_cnt);
+        nsISelection_GetIsCollapsed(This->nsselection, &collapsed);
 
-    *p = SysAllocString(range_cnt ? wszText : wszNone); /* FIXME: control */
+    *p = SysAllocString(collapsed ? wszNone : wszText); /* FIXME: control */
+    TRACE("ret %s\n", debugstr_w(*p));
     return S_OK;
 }
 
@@ -193,13 +241,25 @@ static const IHTMLSelectionObjectVtbl HTMLSelectionObjectVtbl = {
     HTMLSelectionObject_get_type
 };
 
-IHTMLSelectionObject *HTMLSelectionObject_Create(nsISelection *nsselection)
+IHTMLSelectionObject *HTMLSelectionObject_Create(HTMLDocument *doc, nsISelection *nsselection)
 {
-    HTMLSelectionObject *ret = mshtml_alloc(sizeof(HTMLSelectionObject));
+    HTMLSelectionObject *ret = heap_alloc(sizeof(HTMLSelectionObject));
 
     ret->lpHTMLSelectionObjectVtbl = &HTMLSelectionObjectVtbl;
     ret->ref = 1;
     ret->nsselection = nsselection; /* We shouldn't call AddRef here */
 
+    ret->doc = doc;
+    list_add_head(&doc->selection_list, &ret->entry);
+
     return HTMLSELOBJ(ret);
 }
+
+void detach_selection(HTMLDocument *This)
+{
+    HTMLSelectionObject *iter;
+
+    LIST_FOR_EACH_ENTRY(iter, &This->selection_list, HTMLSelectionObject, entry) {
+        iter->doc = NULL;
+    }
+}
