@@ -28,6 +28,7 @@
 #include "winuser.h"
 #include "winnls.h"
 #include "ole2.h"
+#include "mshtmcid.h"
 
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -37,9 +38,11 @@
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
 static const WCHAR brW[] = {'b','r',0};
+static const WCHAR hrW[] = {'h','r',0};
 
 typedef struct {
     const IHTMLTxtRangeVtbl *lpHTMLTxtRangeVtbl;
+    const IOleCommandTargetVtbl *lpOleCommandTargetVtbl;
 
     LONG ref;
 
@@ -122,7 +125,7 @@ static int string_to_nscmptype(LPCWSTR str)
 
 static PRUint16 get_node_type(nsIDOMNode *node)
 {
-    PRUint16 type = 0xfff;
+    PRUint16 type = 0;
 
     if(node)
         nsIDOMNode_GetNodeType(node, &type);
@@ -130,7 +133,7 @@ static PRUint16 get_node_type(nsIDOMNode *node)
     return type;
 }
 
-static BOOL is_br_node(nsIDOMNode *node)
+static BOOL is_elem_tag(nsIDOMNode *node, LPCWSTR istag)
 {
     nsIDOMElement *elem;
     nsAString tag_str;
@@ -145,10 +148,33 @@ static BOOL is_br_node(nsIDOMNode *node)
     nsAString_Init(&tag_str, NULL);
     nsIDOMElement_GetTagName(elem, &tag_str);
     nsIDOMElement_Release(elem);
-    nsAString_GetData(&tag_str, &tag, 0);
+    nsAString_GetData(&tag_str, &tag);
 
-    if(!strcmpiW(tag, brW))
-        ret = TRUE;
+    ret = !strcmpiW(tag, istag);
+
+    nsAString_Finish(&tag_str);
+
+    return ret;
+}
+
+static BOOL is_space_elem(nsIDOMNode *node)
+{
+    nsIDOMElement *elem;
+    nsAString tag_str;
+    const PRUnichar *tag;
+    BOOL ret = FALSE;
+    nsresult nsres;
+
+    nsres = nsIDOMNode_QueryInterface(node, &IID_nsIDOMElement, (void**)&elem);
+    if(NS_FAILED(nsres))
+        return FALSE;
+
+    nsAString_Init(&tag_str, NULL);
+    nsIDOMElement_GetTagName(elem, &tag_str);
+    nsIDOMElement_Release(elem);
+    nsAString_GetData(&tag_str, &tag);
+
+    ret = !strcmpiW(tag, brW) || !strcmpiW(tag, hrW);
 
     nsAString_Finish(&tag_str);
 
@@ -159,20 +185,20 @@ static inline void wstrbuf_init(wstrbuf_t *buf)
 {
     buf->len = 0;
     buf->size = 16;
-    buf->buf = mshtml_alloc(buf->size * sizeof(WCHAR));
+    buf->buf = heap_alloc(buf->size * sizeof(WCHAR));
     *buf->buf = 0;
 }
 
 static inline void wstrbuf_finish(wstrbuf_t *buf)
 {
-    mshtml_free(buf->buf);
+    heap_free(buf->buf);
 }
 
 static void wstrbuf_append_len(wstrbuf_t *buf, LPCWSTR str, int len)
 {
     if(buf->len+len >= buf->size) {
         buf->size = 2*buf->len+len;
-        buf->buf = mshtml_realloc(buf->buf, buf->size * sizeof(WCHAR));
+        buf->buf = heap_realloc(buf->buf, buf->size * sizeof(WCHAR));
     }
 
     memcpy(buf->buf+buf->len, str, len*sizeof(WCHAR));
@@ -185,6 +211,39 @@ static inline void wstrbuf_append(wstrbuf_t *buf, LPCWSTR str)
     wstrbuf_append_len(buf, str, strlenW(str));
 }
 
+static void wstrbuf_append_nodetxt(wstrbuf_t *buf, LPCWSTR str, int len)
+{
+    const WCHAR *s = str;
+    WCHAR *d;
+
+    TRACE("%s\n", debugstr_wn(str, len));
+
+    if(buf->len+len >= buf->size) {
+        buf->size = 2*buf->len+len;
+        buf->buf = heap_realloc(buf->buf, buf->size * sizeof(WCHAR));
+    }
+
+    if(buf->len && isspaceW(buf->buf[buf->len-1])) {
+        while(s < str+len && isspaceW(*s))
+            s++;
+    }
+
+    d = buf->buf+buf->len;
+    while(s < str+len) {
+        if(isspaceW(*s)) {
+            *d++ = ' ';
+            s++;
+            while(s < str+len && isspaceW(*s))
+                s++;
+        }else {
+            *d++ = *s++;
+        }
+    }
+
+    buf->len = d - buf->buf;
+    *d = 0;
+}
+
 static void wstrbuf_append_node(wstrbuf_t *buf, nsIDOMNode *node)
 {
 
@@ -198,8 +257,8 @@ static void wstrbuf_append_node(wstrbuf_t *buf, nsIDOMNode *node)
 
         nsAString_Init(&data_str, NULL);
         nsIDOMText_GetData(nstext, &data_str);
-        nsAString_GetData(&data_str, &data, NULL);
-        wstrbuf_append(buf, data);
+        nsAString_GetData(&data_str, &data);
+        wstrbuf_append_nodetxt(buf, data, strlenW(data));
         nsAString_Finish(&data_str);
 
        nsIDOMText_Release(nstext);
@@ -207,9 +266,12 @@ static void wstrbuf_append_node(wstrbuf_t *buf, nsIDOMNode *node)
         break;
     }
     case ELEMENT_NODE:
-        if(is_br_node(node)) {
+        if(is_elem_tag(node, brW)) {
             static const WCHAR endlW[] = {'\r','\n'};
             wstrbuf_append_len(buf, endlW, 2);
+        }else if(is_elem_tag(node, hrW)) {
+            static const WCHAR endl2W[] = {'\r','\n','\r','\n'};
+            wstrbuf_append_len(buf, endl2W, 4);
         }
     }
 }
@@ -229,7 +291,10 @@ static BOOL fill_nodestr(dompos_t *pos)
     nsAString_Init(&pos->str, NULL);
     nsIDOMText_GetData(text, &pos->str);
     nsIDOMText_Release(text);
-    nsAString_GetData(&pos->str, &pos->p, NULL);
+    nsAString_GetData(&pos->str, &pos->p);
+
+    if(pos->off == -1)
+        pos->off = *pos->p ? strlenW(pos->p)-1 : 0;
 
     return TRUE;
 }
@@ -385,7 +450,7 @@ static void set_range_pos(HTMLTxtRange *This, BOOL start, dompos_t *pos)
         else
             nsres = nsIDOMRange_SetStartBefore(This->nsrange, pos->node);
     }else {
-        if(pos->type == TEXT_NODE)
+        if(pos->type == TEXT_NODE && pos->p[pos->off+1])
             nsres = nsIDOMRange_SetEnd(This->nsrange, pos->node, pos->off+1);
         else
             nsres = nsIDOMRange_SetEndAfter(This->nsrange, pos->node);
@@ -413,6 +478,11 @@ static inline void dompos_addref(dompos_t *pos)
         fill_nodestr(pos);
 }
 
+static inline BOOL dompos_cmp(const dompos_t *pos1, const dompos_t *pos2)
+{
+    return pos1->node == pos2->node && pos1->off == pos2->off;
+}
+
 static void range_to_string(HTMLTxtRange *This, wstrbuf_t *buf)
 {
     nsIDOMNode *iter, *tmp;
@@ -432,11 +502,11 @@ static void range_to_string(HTMLTxtRange *This, wstrbuf_t *buf)
 
     if(start_pos.type == TEXT_NODE) {
         if(start_pos.node == end_pos.node) {
-            wstrbuf_append_len(buf, start_pos.p+start_pos.off, end_pos.off-start_pos.off+1);
+            wstrbuf_append_nodetxt(buf, start_pos.p+start_pos.off, end_pos.off-start_pos.off+1);
             iter = start_pos.node;
             nsIDOMNode_AddRef(iter);
         }else {
-            wstrbuf_append(buf, start_pos.p+start_pos.off);
+            wstrbuf_append_nodetxt(buf, start_pos.p+start_pos.off, strlenW(start_pos.p+start_pos.off));
             iter = next_node(start_pos.node);
         }
     }else {
@@ -453,12 +523,26 @@ static void range_to_string(HTMLTxtRange *This, wstrbuf_t *buf)
 
     nsIDOMNode_AddRef(end_pos.node);
 
-    if(start_pos.node != end_pos.node && !is_br_node(end_pos.node))
-        wstrbuf_append_len(buf, end_pos.p, end_pos.off+1);
+    if(start_pos.node != end_pos.node) {
+        if(end_pos.type == TEXT_NODE)
+            wstrbuf_append_nodetxt(buf, end_pos.p, end_pos.off+1);
+        else
+            wstrbuf_append_node(buf, end_pos.node);
+    }
 
     nsIDOMNode_Release(iter);
     dompos_release(&start_pos);
     dompos_release(&end_pos);
+
+    if(buf->len) {
+        WCHAR *p;
+
+        for(p = buf->buf+buf->len-1; p >= buf->buf && isspaceW(*p); p--);
+
+        p = strchrW(p, '\r');
+        if(p)
+            *p = 0;
+    }
 }
 
 static WCHAR get_pos_char(const dompos_t *pos)
@@ -467,86 +551,219 @@ static WCHAR get_pos_char(const dompos_t *pos)
     case TEXT_NODE:
         return pos->p[pos->off];
     case ELEMENT_NODE:
-        if(is_br_node(pos->node))
+        if(is_space_elem(pos->node))
             return '\n';
     }
 
     return 0;
 }
 
+static void end_space(const dompos_t *pos, dompos_t *new_pos)
+{
+    const WCHAR *p;
+
+    *new_pos = *pos;
+    dompos_addref(new_pos);
+
+    if(pos->type != TEXT_NODE)
+        return;
+
+    p = new_pos->p+new_pos->off;
+
+    if(!*p || !isspace(*p))
+        return;
+
+    while(p[1] && isspace(p[1]))
+        p++;
+
+    new_pos->off = p - new_pos->p;
+}
+
 static WCHAR next_char(const dompos_t *pos, dompos_t *new_pos)
 {
     nsIDOMNode *iter, *tmp;
+    dompos_t last_space, tmp_pos;
+    const WCHAR *p;
+    WCHAR cspace = 0;
 
-    if(pos->type == TEXT_NODE && pos->off != -1 && pos->p[pos->off+1]) {
-        *new_pos = *pos;
-        new_pos->off++;
-        dompos_addref(new_pos);
-        return new_pos->p[new_pos->off];
+    if(pos->type == TEXT_NODE && pos->off != -1 && pos->p[pos->off]) {
+        p = pos->p+pos->off;
+
+        if(isspace(*p))
+            while(isspaceW(*++p));
+        else
+            p++;
+
+        if(*p && isspaceW(*p)) {
+            cspace = ' ';
+            while(p[1] && isspaceW(p[1]))
+                p++;
+        }
+
+        if(*p) {
+            *new_pos = *pos;
+            new_pos->off = p - pos->p;
+            dompos_addref(new_pos);
+
+            return cspace ? cspace : *p;
+        }else {
+            last_space = *pos;
+            last_space.off = p - pos->p;
+            dompos_addref(&last_space);
+        }
     }
 
     iter = next_node(pos->node);
-    if(!iter)
-        return 0;
 
-    while(1) {
+    while(iter) {
         switch(get_node_type(iter)) {
         case TEXT_NODE:
-            new_pos->node = iter;
-            new_pos->type = TEXT_NODE;
-            new_pos->off = 0;
-            fill_nodestr(new_pos);
-            return *new_pos->p;
+            tmp_pos.node = iter;
+            tmp_pos.type = TEXT_NODE;
+            dompos_addref(&tmp_pos);
+
+            p = tmp_pos.p;
+
+            if(!*p) {
+                dompos_release(&tmp_pos);
+                break;
+            }else if(isspaceW(*p)) {
+                if(cspace)
+                    dompos_release(&last_space);
+                else
+                    cspace = ' ';
+
+                while(p[1] && isspaceW(p[1]))
+                      p++;
+
+                tmp_pos.off = p-tmp_pos.p;
+
+                if(!p[1]) {
+                    last_space = tmp_pos;
+                    break;
+                }
+
+                *new_pos = tmp_pos;
+                nsIDOMNode_Release(iter);
+                return cspace;
+            }else if(cspace) {
+                *new_pos = last_space;
+                dompos_release(&tmp_pos);
+                nsIDOMNode_Release(iter);
+
+                return cspace;
+            }else if(*p) {
+                tmp_pos.off = 0;
+                *new_pos = tmp_pos;
+            }
+
+            nsIDOMNode_Release(iter);
+            return *p;
 
         case ELEMENT_NODE:
-            if(!is_br_node(iter))
-                break;
+            if(is_elem_tag(iter, brW)) {
+                if(cspace)
+                    dompos_release(&last_space);
+                cspace = '\n';
 
-            new_pos->node = iter;
-            new_pos->type = ELEMENT_NODE;
-            new_pos->off = 0;
-            new_pos->p = NULL;
-            return '\n';
+                nsIDOMNode_AddRef(iter);
+                last_space.node = iter;
+                last_space.type = ELEMENT_NODE;
+                last_space.off = 0;
+                last_space.p = NULL;
+            }else if(is_elem_tag(iter, hrW)) {
+                if(cspace) {
+                    *new_pos = last_space;
+                    nsIDOMNode_Release(iter);
+                    return cspace;
+                }
+
+                new_pos->node = iter;
+                new_pos->type = ELEMENT_NODE;
+                new_pos->off = 0;
+                new_pos->p = NULL;
+                return '\n';
+            }
         }
 
         tmp = iter;
         iter = next_node(iter);
         nsIDOMNode_Release(tmp);
-
-        if(!iter)
-            break;
     }
 
-    return 0;
+    if(cspace) {
+        *new_pos = last_space;
+    }else {
+        *new_pos = *pos;
+        dompos_addref(new_pos);
+    }
+
+    return cspace;
 }
 
 static WCHAR prev_char(HTMLTxtRange *This, const dompos_t *pos, dompos_t *new_pos)
 {
     nsIDOMNode *iter, *tmp;
+    const WCHAR *p;
+    BOOL skip_space = FALSE;
 
-    if(pos->type == TEXT_NODE && pos->off > 0) {
-        *new_pos = *pos;
-        new_pos->off--;
-        dompos_addref(new_pos);
-        return new_pos->p[new_pos->off];
+    if(pos->type == TEXT_NODE && isspaceW(pos->p[pos->off]))
+        skip_space = TRUE;
+
+    if(pos->type == TEXT_NODE && pos->off) {
+        p = pos->p+pos->off-1;
+
+        if(skip_space) {
+            while(p >= pos->p && isspace(*p))
+                p--;
+        }
+
+        if(p >= pos->p) {
+            *new_pos = *pos;
+            new_pos->off = p-pos->p;
+            dompos_addref(new_pos);
+            return new_pos->p[new_pos->off];
+        }
     }
 
     iter = prev_node(This, pos->node);
-    if(!iter)
-        return 0;
 
-    while(1) {
+    while(iter) {
         switch(get_node_type(iter)) {
-        case TEXT_NODE:
-            new_pos->node = iter;
-            new_pos->type = TEXT_NODE;
-            fill_nodestr(new_pos);
-            new_pos->off = strlenW(new_pos->p)-1;
-            return new_pos->p[new_pos->off];
+        case TEXT_NODE: {
+            dompos_t tmp_pos;
+
+            tmp_pos.node = iter;
+            tmp_pos.type = TEXT_NODE;
+            dompos_addref(&tmp_pos);
+
+            p = tmp_pos.p + strlenW(tmp_pos.p)-1;
+
+            if(skip_space) {
+                while(p >= tmp_pos.p && isspaceW(*p))
+                    p--;
+            }
+
+            if(p < tmp_pos.p) {
+                dompos_release(&tmp_pos);
+                break;
+            }
+
+            tmp_pos.off = p-tmp_pos.p;
+            *new_pos = tmp_pos;
+            nsIDOMNode_Release(iter);
+            return *p;
+        }
 
         case ELEMENT_NODE:
-            if(!is_br_node(iter))
+            if(is_elem_tag(iter, brW)) {
+                if(skip_space) {
+                    skip_space = FALSE;
+                    break;
+                }
+            }else if(!is_elem_tag(iter, hrW)) {
                 break;
+            }
 
             new_pos->node = iter;
             new_pos->type = ELEMENT_NODE;
@@ -558,9 +775,6 @@ static WCHAR prev_char(HTMLTxtRange *This, const dompos_t *pos, dompos_t *new_po
         tmp = iter;
         iter = prev_node(This, iter);
         nsIDOMNode_Release(tmp);
-
-        if(!iter)
-            break;
     }
 
     *new_pos = *pos;
@@ -568,18 +782,21 @@ static WCHAR prev_char(HTMLTxtRange *This, const dompos_t *pos, dompos_t *new_po
     return 0;
 }
 
-static long move_next_chars(long cnt, const dompos_t *pos, BOOL col, dompos_t *new_pos)
+static long move_next_chars(long cnt, const dompos_t *pos, BOOL col, const dompos_t *bound_pos,
+        BOOL *bounded, dompos_t *new_pos)
 {
     dompos_t iter, tmp;
     long ret = 0;
     WCHAR c;
 
+    if(bounded)
+        *bounded = FALSE;
+
     if(col)
         ret++;
 
     if(ret >= cnt) {
-        *new_pos = *pos;
-        dompos_addref(new_pos);
+        end_space(pos, new_pos);
         return ret;
     }
 
@@ -589,24 +806,32 @@ static long move_next_chars(long cnt, const dompos_t *pos, BOOL col, dompos_t *n
     while(ret < cnt) {
         tmp = iter;
         c = next_char(&tmp, &iter);
-        if(!c) {
-            iter = tmp;
-            break;
-        }
-        ret++;
         dompos_release(&tmp);
+        if(!c)
+            break;
+
+        ret++;
+        if(bound_pos && dompos_cmp(&tmp, bound_pos)) {
+            *bounded = TRUE;
+            ret++;
+        }
     }
 
     *new_pos = iter;
     return ret;
 }
 
-static long move_prev_chars(HTMLTxtRange *This, long cnt, const dompos_t *pos, BOOL end, dompos_t *new_pos)
+static long move_prev_chars(HTMLTxtRange *This, long cnt, const dompos_t *pos, BOOL end,
+        const dompos_t *bound_pos, BOOL *bounded, dompos_t *new_pos)
 {
     dompos_t iter, tmp;
     long ret = 0;
+    BOOL prev_eq = FALSE;
     WCHAR c;
 
+    if(bounded)
+        *bounded = FALSE;
+
     c = prev_char(This, pos, &iter);
     if(c)
         ret++;
@@ -614,55 +839,27 @@ static long move_prev_chars(HTMLTxtRange *This, long cnt, const dompos_t *pos, B
     while(c && ret < cnt) {
         tmp = iter;
         c = prev_char(This, &tmp, &iter);
+        dompos_release(&tmp);
         if(!c) {
-            iter = tmp;
             if(end)
                 ret++;
             break;
         }
+
         ret++;
-        dompos_release(&tmp);
+
+        if(prev_eq) {
+            *bounded = TRUE;
+            ret++;
+        }
+
+        prev_eq = bound_pos && dompos_cmp(&iter, bound_pos);
     }
 
     *new_pos = iter;
     return ret;
 }
 
-static BOOL find_next_space(const dompos_t *pos, BOOL first_space, dompos_t *ret)
-{
-    dompos_t iter, tmp;
-    WCHAR c;
-
-    if(first_space) {
-        c = get_pos_char(pos);
-        if(c && isspaceW(c)) {
-            *ret = *pos;
-            dompos_addref(ret);
-            return FALSE;
-        }
-    }
-
-    c = next_char(pos, &iter);
-    if(!c) {
-        *ret = *pos;
-        dompos_addref(ret);
-        return FALSE;
-    }
-
-    while(!isspaceW(c)) {
-        tmp = iter;
-        c = next_char(&tmp, &iter);
-        if(!c) {
-            iter = tmp;
-            break;
-        }
-        dompos_release(&tmp);
-    }
-
-    *ret = iter;
-    return TRUE;
-}
-
 static long find_prev_space(HTMLTxtRange *This, const dompos_t *pos, BOOL first_space, dompos_t *ret)
 {
     dompos_t iter, tmp;
@@ -670,8 +867,7 @@ static long find_prev_space(HTMLTxtRange *This, const dompos_t *pos, BOOL first_
 
     c = prev_char(This, pos, &iter);
     if(!c || (first_space && isspaceW(c))) {
-        *ret = *pos;
-        dompos_addref(ret);
+        *ret = iter;
         return FALSE;
     }
 
@@ -689,21 +885,67 @@ static long find_prev_space(HTMLTxtRange *This, const dompos_t *pos, BOOL first_
     return TRUE;
 }
 
+static int find_word_end(const dompos_t *pos, dompos_t *ret)
+{
+    dompos_t iter, tmp;
+    int cnt = 1;
+    WCHAR c;
+    c = get_pos_char(pos);
+    if(isspaceW(c)) {
+        *ret = *pos;
+        dompos_addref(ret);
+        return 0;
+    }
+
+    c = next_char(pos, &iter);
+    if(!c) {
+        *ret = iter;
+        return 0;
+    }
+    if(c == '\n') {
+        *ret = *pos;
+        dompos_addref(ret);
+        return 0;
+    }
+
+    while(c && !isspaceW(c)) {
+        tmp = iter;
+        c = next_char(&tmp, &iter);
+        if(c == '\n') {
+            dompos_release(&iter);
+            iter = tmp;
+        }else {
+            cnt++;
+            dompos_release(&tmp);
+        }
+    }
+
+    *ret = iter;
+    return cnt;
+}
+
 static long move_next_words(long cnt, const dompos_t *pos, dompos_t *new_pos)
 {
     dompos_t iter, tmp;
     long ret = 0;
+    WCHAR c;
 
-    iter = *pos;
-    dompos_addref(&iter);
-
-    while(ret < cnt) {
-        if(!find_next_space(&iter, FALSE, &tmp))
-            break;
-
+    c = get_pos_char(pos);
+    if(isspaceW(c)) {
+        end_space(pos, &iter);
         ret++;
-        dompos_release(&iter);
-        iter = tmp;
+    }else {
+        c = next_char(pos, &iter);
+        if(c && isspaceW(c))
+            ret++;
+    }
+
+    while(c && ret < cnt) {
+        tmp = iter;
+        c = next_char(&tmp, &iter);
+        dompos_release(&tmp);
+        if(isspaceW(c))
+            ret++;
     }
 
     *new_pos = iter;
@@ -748,6 +990,9 @@ static HRESULT WINAPI HTMLTxtRange_QueryInterface(IHTMLTxtRange *iface, REFIID r
     }else if(IsEqualGUID(&IID_IHTMLTxtRange, riid)) {
         TRACE("(%p)->(IID_IHTMLTxtRange %p)\n", This, ppv);
         *ppv = HTMLTXTRANGE(This);
+    }else if(IsEqualGUID(&IID_IOleCommandTarget, riid)) {
+        TRACE("(%p)->(IID_IOleCommandTarget %p)\n", This, ppv);
+        *ppv = CMDTARGET(This);
     }
 
     if(*ppv) {
@@ -781,7 +1026,7 @@ static ULONG WINAPI HTMLTxtRange_Release(IHTMLTxtRange *iface)
             nsISelection_Release(This->nsrange);
         if(This->doc)
             list_remove(&This->entry);
-        mshtml_free(This);
+        heap_free(This);
     }
 
     return ref;
@@ -843,7 +1088,7 @@ static HRESULT WINAPI HTMLTxtRange_get_htmlText(IHTMLTxtRange *iface, BSTR *p)
             nsnode_to_nsstring((nsIDOMNode*)fragment, &nsstr);
             nsIDOMDocumentFragment_Release(fragment);
 
-            nsAString_GetData(&nsstr, &nstext, NULL);
+            nsAString_GetData(&nsstr, &nstext);
             *p = SysAllocString(nstext);
 
             nsAString_Finish(&nsstr);
@@ -880,6 +1125,7 @@ static HRESULT WINAPI HTMLTxtRange_put_text(IHTMLTxtRange *iface, BSTR v)
 
     nsAString_Init(&text_str, v);
     nsres = nsIDOMDocument_CreateTextNode(nsdoc, &text_str, &text_node);
+    nsIDOMDocument_Release(nsdoc);
     nsAString_Finish(&text_str);
     if(NS_FAILED(nsres)) {
         ERR("CreateTextNode failed: %08x\n", nsres);
@@ -893,7 +1139,11 @@ static HRESULT WINAPI HTMLTxtRange_put_text(IHTMLTxtRange *iface, BSTR v)
     if(NS_FAILED(nsres))
         ERR("InsertNode failed: %08x\n", nsres);
 
-    return S_OK;
+    nsres = nsIDOMRange_SetEndAfter(This->nsrange, (nsIDOMNode*)text_node);
+    if(NS_FAILED(nsres))
+        ERR("SetEndAfter failed: %08x\n", nsres);
+
+    return IHTMLTxtRange_collapse(HTMLTXTRANGE(This), VARIANT_FALSE);
 }
 
 static HRESULT WINAPI HTMLTxtRange_get_text(IHTMLTxtRange *iface, BSTR *p)
@@ -1047,30 +1297,71 @@ static HRESULT WINAPI HTMLTxtRange_expand(IHTMLTxtRange *iface, BSTR Unit, VARIA
     if(unit == RU_UNKNOWN)
         return E_INVALIDARG;
 
+    *Success = VARIANT_FALSE;
+
     switch(unit) {
     case RU_WORD: {
-        dompos_t end_pos, start_pos, new_pos;
+        dompos_t end_pos, start_pos, new_start_pos, new_end_pos;
+        PRBool collapsed;
 
-        *Success = VARIANT_FALSE;
+        nsIDOMRange_GetCollapsed(This->nsrange, &collapsed);
 
         get_cur_pos(This, TRUE, &start_pos);
         get_cur_pos(This, FALSE, &end_pos);
-        if(find_next_space(&end_pos, TRUE, &new_pos)) {
-            set_range_pos(This, FALSE, &new_pos);
-            *Success = VARIANT_TRUE;
-        }
-        dompos_release(&new_pos);
 
-        if(find_prev_space(This, &start_pos, TRUE, &new_pos)) {
-            set_range_pos(This, TRUE, &new_pos);
+        if(find_word_end(&end_pos, &new_end_pos) || collapsed) {
+            set_range_pos(This, FALSE, &new_end_pos);
             *Success = VARIANT_TRUE;
         }
 
-        dompos_release(&new_pos);
+        if(start_pos.type && (get_pos_char(&end_pos) || !dompos_cmp(&new_end_pos, &end_pos))) {
+            if(find_prev_space(This, &start_pos, TRUE, &new_start_pos)) {
+                set_range_pos(This, TRUE, &new_start_pos);
+                *Success = VARIANT_TRUE;
+            }
+            dompos_release(&new_start_pos);
+        }
+
+        dompos_release(&new_end_pos);
         dompos_release(&end_pos);
+        dompos_release(&start_pos);
 
         break;
     }
+
+    case RU_TEXTEDIT: {
+        nsIDOMDocument *nsdoc;
+        nsIDOMHTMLDocument *nshtmldoc;
+        nsIDOMHTMLElement *nsbody = NULL;
+        nsresult nsres;
+
+        nsres = nsIWebNavigation_GetDocument(This->doc->nscontainer->navigation, &nsdoc);
+        if(NS_FAILED(nsres) || !nsdoc) {
+            ERR("GetDocument failed: %08x\n", nsres);
+            break;
+        }
+
+        nsIDOMDocument_QueryInterface(nsdoc, &IID_nsIDOMHTMLDocument, (void**)&nshtmldoc);
+        nsIDOMDocument_Release(nsdoc);
+
+        nsres = nsIDOMHTMLDocument_GetBody(nshtmldoc, &nsbody);
+        nsIDOMHTMLDocument_Release(nshtmldoc);
+        if(NS_FAILED(nsres) || !nsbody) {
+            ERR("Could not get body: %08x\n", nsres);
+            break;
+        }
+
+        nsres = nsIDOMRange_SelectNodeContents(This->nsrange, (nsIDOMNode*)nsbody);
+        nsIDOMHTMLElement_Release(nsbody);
+        if(NS_FAILED(nsres)) {
+            ERR("Collapse failed: %08x\n", nsres);
+            break;
+        }
+
+        *Success = VARIANT_TRUE;
+        break;
+    }
+
     default:
         FIXME("Unimplemented unit %s\n", debugstr_w(Unit));
     }
@@ -1090,11 +1381,9 @@ static HRESULT WINAPI HTMLTxtRange_move(IHTMLTxtRange *iface, BSTR Unit,
     if(unit == RU_UNKNOWN)
         return E_INVALIDARG;
 
-    IHTMLTxtRange_collapse(HTMLTXTRANGE(This), TRUE);
-
     if(!Count) {
         *ActualCount = 0;
-        return S_OK;
+        return IHTMLTxtRange_collapse(HTMLTXTRANGE(This), TRUE);
     }
 
     switch(unit) {
@@ -1104,12 +1393,13 @@ static HRESULT WINAPI HTMLTxtRange_move(IHTMLTxtRange *iface, BSTR Unit,
         get_cur_pos(This, TRUE, &cur_pos);
 
         if(Count > 0) {
-            *ActualCount = move_next_chars(Count, &cur_pos, TRUE, &new_pos);
+            *ActualCount = move_next_chars(Count, &cur_pos, TRUE, NULL, NULL, &new_pos);
             set_range_pos(This, FALSE, &new_pos);
-            IHTMLTxtRange_collapse(HTMLTXTRANGE(This), FALSE);
             dompos_release(&new_pos);
+
+            IHTMLTxtRange_collapse(HTMLTXTRANGE(This), FALSE);
         }else {
-            *ActualCount = -move_prev_chars(This, -Count, &cur_pos, FALSE, &new_pos);
+            *ActualCount = -move_prev_chars(This, -Count, &cur_pos, FALSE, NULL, NULL, &new_pos);
             set_range_pos(This, TRUE, &new_pos);
             IHTMLTxtRange_collapse(HTMLTXTRANGE(This), TRUE);
             dompos_release(&new_pos);
@@ -1127,8 +1417,8 @@ static HRESULT WINAPI HTMLTxtRange_move(IHTMLTxtRange *iface, BSTR Unit,
         if(Count > 0) {
             *ActualCount = move_next_words(Count, &cur_pos, &new_pos);
             set_range_pos(This, FALSE, &new_pos);
-            IHTMLTxtRange_collapse(HTMLTXTRANGE(This), FALSE);
             dompos_release(&new_pos);
+            IHTMLTxtRange_collapse(HTMLTXTRANGE(This), FALSE);
         }else {
             *ActualCount = -move_prev_words(This, -Count, &cur_pos, &new_pos);
             set_range_pos(This, TRUE, &new_pos);
@@ -1152,8 +1442,51 @@ static HRESULT WINAPI HTMLTxtRange_moveStart(IHTMLTxtRange *iface, BSTR Unit,
         long Count, long *ActualCount)
 {
     HTMLTxtRange *This = HTMLTXTRANGE_THIS(iface);
-    FIXME("(%p)->(%s %ld %p)\n", This, debugstr_w(Unit), Count, ActualCount);
-    return E_NOTIMPL;
+    range_unit_t unit;
+
+    TRACE("(%p)->(%s %ld %p)\n", This, debugstr_w(Unit), Count, ActualCount);
+
+    unit = string_to_unit(Unit);
+    if(unit == RU_UNKNOWN)
+        return E_INVALIDARG;
+
+    if(!Count) {
+        *ActualCount = 0;
+        return S_OK;
+    }
+
+    switch(unit) {
+    case RU_CHAR: {
+        dompos_t start_pos, end_pos, new_pos;
+        PRBool collapsed;
+
+        get_cur_pos(This, TRUE, &start_pos);
+        get_cur_pos(This, FALSE, &end_pos);
+        nsIDOMRange_GetCollapsed(This->nsrange, &collapsed);
+
+        if(Count > 0) {
+            BOOL bounded;
+
+            *ActualCount = move_next_chars(Count, &start_pos, collapsed, &end_pos, &bounded, &new_pos);
+            set_range_pos(This, !bounded, &new_pos);
+            if(bounded)
+                IHTMLTxtRange_collapse(HTMLTXTRANGE(This), FALSE);
+        }else {
+            *ActualCount = -move_prev_chars(This, -Count, &start_pos, FALSE, NULL, NULL, &new_pos);
+            set_range_pos(This, TRUE, &new_pos);
+        }
+
+        dompos_release(&start_pos);
+        dompos_release(&end_pos);
+        dompos_release(&new_pos);
+        break;
+    }
+
+    default:
+        FIXME("unimplemented unit %s\n", debugstr_w(Unit));
+    }
+
+    return S_OK;
 }
 
 static HRESULT WINAPI HTMLTxtRange_moveEnd(IHTMLTxtRange *iface, BSTR Unit,
@@ -1175,24 +1508,27 @@ static HRESULT WINAPI HTMLTxtRange_moveEnd(IHTMLTxtRange *iface, BSTR Unit,
 
     switch(unit) {
     case RU_CHAR: {
-        dompos_t cur_pos, new_pos;
+        dompos_t start_pos, end_pos, new_pos;
         PRBool collapsed;
 
-        get_cur_pos(This, FALSE, &cur_pos);
+        get_cur_pos(This, TRUE, &start_pos);
+        get_cur_pos(This, FALSE, &end_pos);
         nsIDOMRange_GetCollapsed(This->nsrange, &collapsed);
 
         if(Count > 0) {
-            *ActualCount = move_next_chars(Count, &cur_pos, collapsed, &new_pos);
+            *ActualCount = move_next_chars(Count, &end_pos, collapsed, NULL, NULL, &new_pos);
             set_range_pos(This, FALSE, &new_pos);
         }else {
-            *ActualCount = -move_prev_chars(This, -Count, &cur_pos, TRUE,  &new_pos);
-            if(*ActualCount == Count)
-                set_range_pos(This, FALSE, &new_pos);
-            else
+            BOOL bounded;
+
+            *ActualCount = -move_prev_chars(This, -Count, &end_pos, TRUE, &start_pos, &bounded, &new_pos);
+            set_range_pos(This, bounded, &new_pos);
+            if(bounded)
                 IHTMLTxtRange_collapse(HTMLTXTRANGE(This), TRUE);
         }
 
-        dompos_release(&cur_pos);
+        dompos_release(&start_pos);
+        dompos_release(&end_pos);
         dompos_release(&new_pos);
         break;
     }
@@ -1412,11 +1748,112 @@ static const IHTMLTxtRangeVtbl HTMLTxtRangeVtbl = {
     HTMLTxtRange_execCommandShowHelp
 };
 
+#define OLECMDTRG_THIS(iface) DEFINE_THIS(HTMLTxtRange, OleCommandTarget, iface)
+
+static HRESULT WINAPI RangeCommandTarget_QueryInterface(IOleCommandTarget *iface, REFIID riid, void **ppv)
+{
+    HTMLTxtRange *This = OLECMDTRG_THIS(iface);
+    return IHTMLTxtRange_QueryInterface(HTMLTXTRANGE(This), riid, ppv);
+}
+
+static ULONG WINAPI RangeCommandTarget_AddRef(IOleCommandTarget *iface)
+{
+    HTMLTxtRange *This = OLECMDTRG_THIS(iface);
+    return IHTMLTxtRange_AddRef(HTMLTXTRANGE(This));
+}
+
+static ULONG WINAPI RangeCommandTarget_Release(IOleCommandTarget *iface)
+{
+    HTMLTxtRange *This = OLECMDTRG_THIS(iface);
+    return IHTMLTxtRange_Release(HTMLTXTRANGE(This));
+}
+
+static HRESULT WINAPI RangeCommandTarget_QueryStatus(IOleCommandTarget *iface, const GUID *pguidCmdGroup,
+        ULONG cCmds, OLECMD prgCmds[], OLECMDTEXT *pCmdText)
+{
+    HTMLTxtRange *This = OLECMDTRG_THIS(iface);
+    FIXME("(%p)->(%s %d %p %p)\n", This, debugstr_guid(pguidCmdGroup), cCmds, prgCmds, pCmdText);
+    return E_NOTIMPL;
+}
+
+static HRESULT exec_indent(HTMLTxtRange *This, VARIANT *in, VARIANT *out)
+{
+    nsIDOMDocumentFragment *fragment;
+    nsIDOMElement *blockquote_elem, *p_elem;
+    nsIDOMDocument *nsdoc;
+    nsIDOMNode *tmp;
+    nsAString tag_str;
+
+    static const PRUnichar blockquoteW[] = {'B','L','O','C','K','Q','U','O','T','E',0};
+    static const PRUnichar pW[] = {'P',0};
+
+    TRACE("(%p)->(%p %p)\n", This, in, out);
+
+    nsIWebNavigation_GetDocument(This->doc->nscontainer->navigation, &nsdoc);
+
+    nsAString_Init(&tag_str, blockquoteW);
+    nsIDOMDocument_CreateElement(nsdoc, &tag_str, &blockquote_elem);
+    nsAString_Finish(&tag_str);
+
+    nsAString_Init(&tag_str, pW);
+    nsIDOMDocument_CreateElement(nsdoc, &tag_str, &p_elem);
+    nsAString_Finish(&tag_str);
+
+    nsIDOMDocument_Release(nsdoc);
+
+    nsIDOMRange_ExtractContents(This->nsrange, &fragment);
+    nsIDOMElement_AppendChild(p_elem, (nsIDOMNode*)fragment, &tmp);
+    nsIDOMDocumentFragment_Release(fragment);
+    nsIDOMNode_Release(tmp);
+
+    nsIDOMElement_AppendChild(blockquote_elem, (nsIDOMNode*)p_elem, &tmp);
+    nsIDOMElement_Release(p_elem);
+    nsIDOMNode_Release(tmp);
+
+    nsIDOMRange_InsertNode(This->nsrange, (nsIDOMNode*)blockquote_elem);
+    nsIDOMElement_Release(blockquote_elem);
+
+    return S_OK;
+}
+
+static HRESULT WINAPI RangeCommandTarget_Exec(IOleCommandTarget *iface, const GUID *pguidCmdGroup,
+        DWORD nCmdID, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
+{
+    HTMLTxtRange *This = OLECMDTRG_THIS(iface);
+
+    TRACE("(%p)->(%s %d %x %p %p)\n", This, debugstr_guid(pguidCmdGroup), nCmdID,
+          nCmdexecopt, pvaIn, pvaOut);
+
+    if(pguidCmdGroup && IsEqualGUID(&CGID_MSHTML, pguidCmdGroup)) {
+        switch(nCmdID) {
+        case IDM_INDENT:
+            return exec_indent(This, pvaIn, pvaOut);
+        default:
+            FIXME("Unsupported cmdid %d of CGID_MSHTML\n", nCmdID);
+        }
+    }else {
+        FIXME("Unsupported cmd %d of group %s\n", nCmdID, debugstr_guid(pguidCmdGroup));
+    }
+
+    return E_NOTIMPL;
+}
+
+#undef OLECMDTRG_THIS
+
+static const IOleCommandTargetVtbl OleCommandTargetVtbl = {
+    RangeCommandTarget_QueryInterface,
+    RangeCommandTarget_AddRef,
+    RangeCommandTarget_Release,
+    RangeCommandTarget_QueryStatus,
+    RangeCommandTarget_Exec
+};
+
 IHTMLTxtRange *HTMLTxtRange_Create(HTMLDocument *doc, nsIDOMRange *nsrange)
 {
-    HTMLTxtRange *ret = mshtml_alloc(sizeof(HTMLTxtRange));
+    HTMLTxtRange *ret = heap_alloc(sizeof(HTMLTxtRange));
 
     ret->lpHTMLTxtRangeVtbl = &HTMLTxtRangeVtbl;
+    ret->lpOleCommandTargetVtbl = &OleCommandTargetVtbl;
     ret->ref = 1;
 
     if(nsrange)
