@@ -18,11 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-
-/*
-http://msdn.microsoft.com/library/default.asp?url=/library/en-us/msi/setup/controlevent_overview.asp
-*/
-
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -168,7 +163,7 @@ static UINT ControlEvent_SpawnWaitDialog(MSIPACKAGE* package, LPCWSTR argument,
 static UINT ControlEvent_DoAction(MSIPACKAGE* package, LPCWSTR argument, 
                                   msi_dialog* dialog)
 {
-    ACTION_PerformAction(package,argument,TRUE);
+    ACTION_PerformAction(package,argument,-1,TRUE);
     return ERROR_SUCCESS;
 }
 
@@ -406,6 +401,13 @@ static UINT ControlEvent_DirectoryListUp(MSIPACKAGE *package, LPCWSTR argument,
     return msi_dialog_directorylist_up( dialog );
 }
 
+static UINT ControlEvent_ReinstallMode(MSIPACKAGE *package, LPCWSTR argument,
+                                       msi_dialog *dialog)
+{
+    static const WCHAR szReinstallMode[] = {'R','E','I','N','S','T','A','L','L','M','O','D','E',0};
+    return MSI_SetPropertyW( package, szReinstallMode, argument );
+}
+
 static const struct _events Events[] = {
     { "EndDialog",ControlEvent_EndDialog },
     { "NewDialog",ControlEvent_NewDialog },
@@ -420,6 +422,7 @@ static const struct _events Events[] = {
     { "SetInstallLevel",ControlEvent_SetInstallLevel },
     { "DirectoryListUp",ControlEvent_DirectoryListUp },
     { "SelectionBrowse",ControlEvent_SpawnDialog },
+    { "ReinstallMode",ControlEvent_ReinstallMode },
     { NULL,NULL },
 };
 
