@@ -123,15 +123,16 @@ UINT msi_clone_properties(MSIPACKAGE *package)
             break;
 
         rc = MSI_DatabaseOpenViewW(package->db, Insert, &view2);
-        if (rc!= ERROR_SUCCESS)
+        if (rc != ERROR_SUCCESS)
+        {
+            msiobj_release(&row->hdr);
             continue;
+        }
 
-        rc = MSI_ViewExecute(view2, row);
+        MSI_ViewExecute(view2, row);
         MSI_ViewClose(view2);
         msiobj_release(&view2->hdr);
-
-        if (rc == ERROR_SUCCESS)
-            msiobj_release(&row->hdr);
+        msiobj_release(&row->hdr);
     }
 
     MSI_ViewClose(view);
@@ -331,12 +332,6 @@ done:
     msi_free(version);
 }
 
-/*
- * There are a whole slew of these we need to set
- *
- *
-http://msdn.microsoft.com/library/default.asp?url=/library/en-us/msi/setup/properties.asp
- */
 static VOID set_installer_properties(MSIPACKAGE *package)
 {
     WCHAR pth[MAX_PATH];
@@ -346,11 +341,11 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     DWORD verval;
     WCHAR verstr[10], bufstr[20];
     HDC dc;
-    LPWSTR check;
     HKEY hkey;
-    LONG res;
+    LPWSTR username, companyname;
     SYSTEM_INFO sys_info;
     SYSTEMTIME systemtime;
+    LANGID langid;
 
     static const WCHAR cszbs[]={'\\',0};
     static const WCHAR CFF[] = 
@@ -419,9 +414,17 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     static const WCHAR szScreenX[] = {'S','c','r','e','e','n','X',0};
     static const WCHAR szScreenY[] = {'S','c','r','e','e','n','Y',0};
     static const WCHAR szColorBits[] = {'C','o','l','o','r','B','i','t','s',0};
-    static const WCHAR szScreenFormat[] = {'%','d',0};
+    static const WCHAR szIntFormat[] = {'%','d',0};
     static const WCHAR szIntel[] = { 'I','n','t','e','l',0 };
     static const WCHAR szAllUsers[] = { 'A','L','L','U','S','E','R','S',0 };
+    static const WCHAR szUserInfo[] = {
+        'S','O','F','T','W','A','R','E','\\',
+        'M','i','c','r','o','s','o','f','t','\\',
+        'M','S',' ','S','e','t','u','p',' ','(','A','C','M','E',')','\\',
+        'U','s','e','r',' ','I','n','f','o',0
+    };
+    static const WCHAR szDefName[] = { 'D','e','f','N','a','m','e',0 };
+    static const WCHAR szDefCompany[] = { 'D','e','f','C','o','m','p','a','n','y',0 };
     static const WCHAR szCurrentVersion[] = {
         'S','O','F','T','W','A','R','E','\\',
         'M','i','c','r','o','s','o','f','t','\\',
@@ -436,6 +439,7 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     static const WCHAR szCOMPANYNAME[] = {'C','O','M','P','A','N','Y','N','A','M','E',0};
     static const WCHAR szDate[] = {'D','a','t','e',0};
     static const WCHAR szTime[] = {'T','i','m','e',0};
+    static const WCHAR szUserLangID[] = {'U','s','e','r','L','a','n','g','u','a','g','e','I','D',0};
 
     /*
      * Other things that probably should be set:
@@ -522,7 +526,7 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     /* Physical Memory is specified in MB. Using total amount. */
     msex.dwLength = sizeof(msex);
     GlobalMemoryStatusEx( &msex );
-    sprintfW( bufstr, szScreenFormat, (int)(msex.ullTotalPhys/1024/1024));
+    sprintfW( bufstr, szIntFormat, (int)(msex.ullTotalPhys/1024/1024));
     MSI_SetPropertyW(package, szPhysicalMemory, bufstr);
 
     SHGetFolderPathW(NULL,CSIDL_WINDOWS,NULL,0,pth);
@@ -569,42 +573,48 @@ static VOID set_installer_properties(MSIPACKAGE *package)
     GetSystemInfo( &sys_info );
     if (sys_info.u.s.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL)
     {
-        sprintfW( bufstr, szScreenFormat, sys_info.wProcessorLevel );
+        sprintfW( bufstr, szIntFormat, sys_info.wProcessorLevel );
         MSI_SetPropertyW( package, szIntel, bufstr );
     }
 
     /* Screen properties. */
     dc = GetDC(0);
-    sprintfW( bufstr, szScreenFormat, GetDeviceCaps( dc, HORZRES ) );
+    sprintfW( bufstr, szIntFormat, GetDeviceCaps( dc, HORZRES ) );
     MSI_SetPropertyW( package, szScreenX, bufstr );
-    sprintfW( bufstr, szScreenFormat, GetDeviceCaps( dc, VERTRES ));
+    sprintfW( bufstr, szIntFormat, GetDeviceCaps( dc, VERTRES ));
     MSI_SetPropertyW( package, szScreenY, bufstr );
-    sprintfW( bufstr, szScreenFormat, GetDeviceCaps( dc, BITSPIXEL ));
+    sprintfW( bufstr, szIntFormat, GetDeviceCaps( dc, BITSPIXEL ));
     MSI_SetPropertyW( package, szColorBits, bufstr );
     ReleaseDC(0, dc);
 
     /* USERNAME and COMPANYNAME */
-    res = RegOpenKeyW( HKEY_LOCAL_MACHINE, szCurrentVersion, &hkey );
-    if (res != ERROR_SUCCESS)
-        return;
+    username = msi_dup_property( package, szUSERNAME );
+    companyname = msi_dup_property( package, szCOMPANYNAME );
 
-    check = msi_dup_property( package, szUSERNAME );
-    if (!check)
+    if ((!username || !companyname) &&
+        RegOpenKeyW( HKEY_CURRENT_USER, szUserInfo, &hkey ) == ERROR_SUCCESS)
     {
-        LPWSTR user = msi_reg_get_val_str( hkey, szRegisteredUser );
-        MSI_SetPropertyW( package, szUSERNAME, user );
-        msi_free( user );
+        if (!username &&
+            (username = msi_reg_get_val_str( hkey, szDefName )))
+            MSI_SetPropertyW( package, szUSERNAME, username );
+        if (!companyname &&
+            (companyname = msi_reg_get_val_str( hkey, szDefCompany )))
+            MSI_SetPropertyW( package, szCOMPANYNAME, companyname );
+        CloseHandle( hkey );
     }
-
-    msi_free( check );
-
-    check = msi_dup_property( package, szCOMPANYNAME );
-    if (!check)
+    if ((!username || !companyname) &&
+        RegOpenKeyW( HKEY_LOCAL_MACHINE, szCurrentVersion, &hkey ) == ERROR_SUCCESS)
     {
-        LPWSTR company = msi_reg_get_val_str( hkey, szRegisteredOrg );
-        MSI_SetPropertyW( package, szCOMPANYNAME, company );
-        msi_free( company );
+        if (!username &&
+            (username = msi_reg_get_val_str( hkey, szRegisteredUser )))
+            MSI_SetPropertyW( package, szUSERNAME, username );
+        if (!companyname &&
+            (companyname = msi_reg_get_val_str( hkey, szRegisteredOrg )))
+            MSI_SetPropertyW( package, szCOMPANYNAME, companyname );
+        CloseHandle( hkey );
     }
+    msi_free( username );
+    msi_free( companyname );
 
     if ( set_user_sid_prop( package ) != ERROR_SUCCESS)
         ERR("Failed to set the UserSID property\n");
@@ -627,8 +637,10 @@ static VOID set_installer_properties(MSIPACKAGE *package)
 
     set_msi_assembly_prop( package );
 
-    msi_free( check );
-    CloseHandle( hkey );
+    langid = GetUserDefaultLangID();
+    sprintfW(bufstr, szIntFormat, langid);
+
+    MSI_SetPropertyW( package, szUserLangID, bufstr );
 }
 
 static UINT msi_load_summary_properties( MSIPACKAGE *package )
@@ -636,7 +648,7 @@ static UINT msi_load_summary_properties( MSIPACKAGE *package )
     UINT rc;
     MSIHANDLE suminfo;
     MSIHANDLE hdb = alloc_msihandle( &package->db->hdr );
-    INT word_count;
+    INT count;
     DWORD len;
     LPWSTR package_code;
     static const WCHAR szPackageCode[] = {
@@ -644,8 +656,9 @@ static UINT msi_load_summary_properties( MSIPACKAGE *package )
 
     if (!hdb) {
         ERR("Unable to allocate handle\n");
-        return 0;
+        return ERROR_OUTOFMEMORY;
     }
+
     rc = MsiGetSummaryInformationW( hdb, NULL, 0, &suminfo );
     MsiCloseHandle(hdb);
     if (rc != ERROR_SUCCESS)
@@ -654,35 +667,47 @@ static UINT msi_load_summary_properties( MSIPACKAGE *package )
         return rc;
     }
 
-    /* load package attributes */
-    rc = MsiSummaryInfoGetPropertyW( suminfo, PID_WORDCOUNT, NULL,
-                                     &word_count, NULL, NULL, NULL );
-    if (rc == ERROR_SUCCESS)
-        package->WordCount = word_count;
-    else
-        WARN("Unable to query word count\n");
+    rc = MsiSummaryInfoGetPropertyW( suminfo, PID_PAGECOUNT, NULL,
+                                     &count, NULL, NULL, NULL );
+    if (rc != ERROR_SUCCESS)
+    {
+        WARN("Unable to query page count: %d\n", rc);
+        goto done;
+    }
 
     /* load package code property */
     len = 0;
     rc = MsiSummaryInfoGetPropertyW( suminfo, PID_REVNUMBER, NULL,
                                      NULL, NULL, NULL, &len );
-    if (rc == ERROR_MORE_DATA)
+    if (rc != ERROR_MORE_DATA)
     {
-        len++;
-        package_code = msi_alloc( len * sizeof(WCHAR) );
-        rc = MsiSummaryInfoGetPropertyW( suminfo, PID_REVNUMBER, NULL,
-                                         NULL, NULL, package_code, &len );
-        if (rc == ERROR_SUCCESS)
-            MSI_SetPropertyW( package, szPackageCode, package_code );
-        else
-            WARN("Unable to query rev number, %d\n", rc);
-        msi_free( package_code );
+        WARN("Unable to query revision number: %d\n", rc);
+        rc = ERROR_FUNCTION_FAILED;
+        goto done;
     }
-    else
-        WARN("Unable to query rev number, %d\n", rc);
 
+    len++;
+    package_code = msi_alloc( len * sizeof(WCHAR) );
+    rc = MsiSummaryInfoGetPropertyW( suminfo, PID_REVNUMBER, NULL,
+                                     NULL, NULL, package_code, &len );
+    if (rc != ERROR_SUCCESS)
+    {
+        WARN("Unable to query rev number: %d\n", rc);
+        goto done;
+    }
+
+    MSI_SetPropertyW( package, szPackageCode, package_code );
+    msi_free( package_code );
+
+    /* load package attributes */
+    count = 0;
+    MsiSummaryInfoGetPropertyW( suminfo, PID_WORDCOUNT, NULL,
+                                &count, NULL, NULL, NULL );
+    package->WordCount = count;
+
+done:
     MsiCloseHandle(suminfo);
-    return ERROR_SUCCESS;
+    return rc;
 }
 
 static MSIPACKAGE *msi_alloc_package( void )
@@ -745,6 +770,7 @@ MSIPACKAGE *MSI_CreatePackage( MSIDATABASE *db, LPCWSTR base_url )
         'P','r','o','d','u','c','t','C','o','d','e',0};
     MSIPACKAGE *package;
     WCHAR uilevel[10];
+    UINT r;
 
     TRACE("%p\n", db);
 
@@ -766,7 +792,12 @@ MSIPACKAGE *MSI_CreatePackage( MSIDATABASE *db, LPCWSTR base_url )
 
         package->ProductCode = msi_dup_property( package, szProductCode );
         set_installed_prop( package );
-        msi_load_summary_properties( package );
+        r = msi_load_summary_properties( package );
+        if (r != ERROR_SUCCESS)
+        {
+            msiobj_release( &package->hdr );
+            return NULL;
+        }
 
         if (package->WordCount & MSIWORDCOUNT_ADMINISTRATIVE)
             msi_load_admin_properties( package );
@@ -856,7 +887,18 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
         handle = atoiW(&szPackage[1]);
         db = msihandle2msiinfo( handle, MSIHANDLETYPE_DATABASE );
         if( !db )
-            return ERROR_INVALID_HANDLE;
+        {
+            IWineMsiRemoteDatabase *remote_database;
+
+            remote_database = (IWineMsiRemoteDatabase *)msi_get_remote( handle );
+            if ( !remote_database )
+                return ERROR_INVALID_HANDLE;
+
+            IWineMsiRemoteDatabase_Release( remote_database );
+            WARN("MsiOpenPackage not allowed during a custom action!\n");
+
+            return ERROR_FUNCTION_FAILED;
+        }
     }
     else
     {
@@ -877,11 +919,12 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
         r = MSI_OpenDatabaseW( file, MSIDBOPEN_READONLY, &db );
         if( r != ERROR_SUCCESS )
         {
-            if (GetLastError() == ERROR_FILE_NOT_FOUND)
-                msi_ui_error( 4, MB_OK | MB_ICONWARNING );
             if (file != szPackage)
                 DeleteFileW( file );
 
+            if (GetFileAttributesW(szPackage) == INVALID_FILE_ATTRIBUTES)
+                return ERROR_FILE_NOT_FOUND;
+
             return r;
         }
     }
@@ -893,21 +936,25 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, MSIPACKAGE **pPackage)
     {
         if (file != szPackage)
             DeleteFileW( file );
-        return ERROR_FUNCTION_FAILED;
+
+        return ERROR_INSTALL_PACKAGE_INVALID;
     }
 
     if( file != szPackage )
         track_tempfile( package, file );
 
-    if( szPackage[0] != '#' )
-    {
+    MSI_SetPropertyW( package, Database, db->path );
+
+    if( UrlIsW( szPackage, URLIS_URL ) )
         MSI_SetPropertyW( package, OriginalDatabase, szPackage );
-        MSI_SetPropertyW( package, Database, szPackage );
-    }
+    else if( szPackage[0] == '#' )
+        MSI_SetPropertyW( package, OriginalDatabase, db->path );
     else
     {
-        MSI_SetPropertyW( package, OriginalDatabase, db->path );
-        MSI_SetPropertyW( package, Database, db->path );
+        WCHAR fullpath[MAX_PATH];
+
+        GetFullPathNameW( szPackage, MAX_PATH, fullpath, NULL );
+        MSI_SetPropertyW( package, OriginalDatabase, fullpath );
     }
 
     *pPackage = package;
@@ -922,9 +969,15 @@ UINT WINAPI MsiOpenPackageExW(LPCWSTR szPackage, DWORD dwOptions, MSIHANDLE *phP
 
     TRACE("%s %08x %p\n", debugstr_w(szPackage), dwOptions, phPackage );
 
-    if( szPackage == NULL )
+    if( !szPackage || !phPackage )
         return ERROR_INVALID_PARAMETER;
 
+    if ( !*szPackage )
+    {
+        FIXME("Should create an empty database and package\n");
+        return ERROR_FUNCTION_FAILED;
+    }
+
     if( dwOptions )
         FIXME("dwOptions %08x not supported\n", dwOptions);
 
@@ -1306,7 +1359,7 @@ UINT WINAPI MsiSetPropertyW( MSIHANDLE hInstall, LPCWSTR szName, LPCWSTR szValue
     if( !package )
     {
         HRESULT hr;
-        BSTR name, value;
+        BSTR name = NULL, value = NULL;
         IWineMsiRemotePackage *remote_package;
 
         remote_package = (IWineMsiRemotePackage *)msi_get_remote( hInstall );
@@ -1315,7 +1368,7 @@ UINT WINAPI MsiSetPropertyW( MSIHANDLE hInstall, LPCWSTR szName, LPCWSTR szValue
 
         name = SysAllocString( szName );
         value = SysAllocString( szValue );
-        if (!name || !value)
+        if ((!name && szName) || (!value && szValue))
         {
             SysFreeString( name );
             SysFreeString( value );
@@ -1477,7 +1530,7 @@ static UINT MSI_GetProperty( MSIHANDLE handle, LPCWSTR name,
         r = msi_strcpy_to_awstring( value, szValueBuf, pchValueBuf );
 
         /* Bug required by Adobe installers */
-        if (!szValueBuf->unicode)
+        if (!szValueBuf->unicode && !szValueBuf->str.a)
             *pchValueBuf *= sizeof(WCHAR);
 
 done:
@@ -1595,14 +1648,34 @@ static HRESULT WINAPI mrp_SetMsiHandle( IWineMsiRemotePackage *iface, MSIHANDLE
     return S_OK;
 }
 
-HRESULT WINAPI mrp_GetActiveDatabase( IWineMsiRemotePackage *iface, MSIHANDLE *handle )
+static HRESULT WINAPI mrp_GetActiveDatabase( IWineMsiRemotePackage *iface, MSIHANDLE *handle )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
-    *handle = MsiGetActiveDatabase(This->package);
+    IWineMsiRemoteDatabase *rdb = NULL;
+    HRESULT hr;
+    MSIHANDLE hdb;
+
+    hr = create_msi_remote_database( NULL, (LPVOID *)&rdb );
+    if (FAILED(hr) || !rdb)
+    {
+        ERR("Failed to create remote database\n");
+        return hr;
+    }
+
+    hdb = MsiGetActiveDatabase(This->package);
+
+    hr = IWineMsiRemoteDatabase_SetMsiHandle( rdb, hdb );
+    if (FAILED(hr))
+    {
+        ERR("Failed to set the database handle\n");
+        return hr;
+    }
+
+    *handle = alloc_msi_remote_handle( (IUnknown *)rdb );
     return S_OK;
 }
 
-HRESULT WINAPI mrp_GetProperty( IWineMsiRemotePackage *iface, BSTR property, BSTR *value, DWORD *size )
+static HRESULT WINAPI mrp_GetProperty( IWineMsiRemotePackage *iface, BSTR property, BSTR *value, DWORD *size )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r;
@@ -1614,63 +1687,63 @@ HRESULT WINAPI mrp_GetProperty( IWineMsiRemotePackage *iface, BSTR property, BST
     return S_OK;
 }
 
-HRESULT WINAPI mrp_SetProperty( IWineMsiRemotePackage *iface, BSTR property, BSTR value )
+static HRESULT WINAPI mrp_SetProperty( IWineMsiRemotePackage *iface, BSTR property, BSTR value )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiSetPropertyW(This->package, (LPWSTR)property, (LPWSTR)value);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_ProcessMessage( IWineMsiRemotePackage *iface, INSTALLMESSAGE message, MSIHANDLE record )
+static HRESULT WINAPI mrp_ProcessMessage( IWineMsiRemotePackage *iface, INSTALLMESSAGE message, MSIHANDLE record )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiProcessMessage(This->package, message, record);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_DoAction( IWineMsiRemotePackage *iface, BSTR action )
+static HRESULT WINAPI mrp_DoAction( IWineMsiRemotePackage *iface, BSTR action )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiDoActionW(This->package, (LPWSTR)action);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_Sequence( IWineMsiRemotePackage *iface, BSTR table, int sequence )
+static HRESULT WINAPI mrp_Sequence( IWineMsiRemotePackage *iface, BSTR table, int sequence )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiSequenceW(This->package, (LPWSTR)table, sequence);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_GetTargetPath( IWineMsiRemotePackage *iface, BSTR folder, BSTR *value, DWORD *size )
+static HRESULT WINAPI mrp_GetTargetPath( IWineMsiRemotePackage *iface, BSTR folder, BSTR *value, DWORD *size )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiGetTargetPathW(This->package, (LPWSTR)folder, (LPWSTR)value, size);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_SetTargetPath( IWineMsiRemotePackage *iface, BSTR folder, BSTR value)
+static HRESULT WINAPI mrp_SetTargetPath( IWineMsiRemotePackage *iface, BSTR folder, BSTR value)
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiSetTargetPathW(This->package, (LPWSTR)folder, (LPWSTR)value);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_GetSourcePath( IWineMsiRemotePackage *iface, BSTR folder, BSTR *value, DWORD *size )
+static HRESULT WINAPI mrp_GetSourcePath( IWineMsiRemotePackage *iface, BSTR folder, BSTR *value, DWORD *size )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiGetSourcePathW(This->package, (LPWSTR)folder, (LPWSTR)value, size);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_GetMode( IWineMsiRemotePackage *iface, MSIRUNMODE mode, BOOL *ret )
+static HRESULT WINAPI mrp_GetMode( IWineMsiRemotePackage *iface, MSIRUNMODE mode, BOOL *ret )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     *ret = MsiGetMode(This->package, mode);
     return S_OK;
 }
 
-HRESULT WINAPI mrp_GetFeatureState( IWineMsiRemotePackage *iface, BSTR feature,
+static HRESULT WINAPI mrp_GetFeatureState( IWineMsiRemotePackage *iface, BSTR feature,
                                     INSTALLSTATE *installed, INSTALLSTATE *action )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
@@ -1678,14 +1751,14 @@ HRESULT WINAPI mrp_GetFeatureState( IWineMsiRemotePackage *iface, BSTR feature,
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_SetFeatureState( IWineMsiRemotePackage *iface, BSTR feature, INSTALLSTATE state )
+static HRESULT WINAPI mrp_SetFeatureState( IWineMsiRemotePackage *iface, BSTR feature, INSTALLSTATE state )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiSetFeatureStateW(This->package, (LPWSTR)feature, state);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_GetComponentState( IWineMsiRemotePackage *iface, BSTR component,
+static HRESULT WINAPI mrp_GetComponentState( IWineMsiRemotePackage *iface, BSTR component,
                                       INSTALLSTATE *installed, INSTALLSTATE *action )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
@@ -1693,28 +1766,28 @@ HRESULT WINAPI mrp_GetComponentState( IWineMsiRemotePackage *iface, BSTR compone
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_SetComponentState( IWineMsiRemotePackage *iface, BSTR component, INSTALLSTATE state )
+static HRESULT WINAPI mrp_SetComponentState( IWineMsiRemotePackage *iface, BSTR component, INSTALLSTATE state )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiSetComponentStateW(This->package, (LPWSTR)component, state);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_GetLanguage( IWineMsiRemotePackage *iface, LANGID *language )
+static HRESULT WINAPI mrp_GetLanguage( IWineMsiRemotePackage *iface, LANGID *language )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     *language = MsiGetLanguage(This->package);
     return S_OK;
 }
 
-HRESULT WINAPI mrp_SetInstallLevel( IWineMsiRemotePackage *iface, int level )
+static HRESULT WINAPI mrp_SetInstallLevel( IWineMsiRemotePackage *iface, int level )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiSetInstallLevel(This->package, level);
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_FormatRecord( IWineMsiRemotePackage *iface, MSIHANDLE record,
+static HRESULT WINAPI mrp_FormatRecord( IWineMsiRemotePackage *iface, MSIHANDLE record,
                                  BSTR value, DWORD *size )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
@@ -1722,7 +1795,7 @@ HRESULT WINAPI mrp_FormatRecord( IWineMsiRemotePackage *iface, MSIHANDLE record,
     return HRESULT_FROM_WIN32(r);
 }
 
-HRESULT WINAPI mrp_EvaluateCondition( IWineMsiRemotePackage *iface, BSTR condition )
+static HRESULT WINAPI mrp_EvaluateCondition( IWineMsiRemotePackage *iface, BSTR condition )
 {
     msi_remote_package_impl* This = mrp_from_IWineMsiRemotePackage( iface );
     UINT r = MsiEvaluateConditionW(This->package, (LPWSTR)condition);
