@@ -21,6 +21,7 @@
 
 #include <stdio.h>
 #include <windows.h>
+#include <shlwapi.h>
 #include <msi.h>
 #include <msiquery.h>
 
@@ -517,7 +518,6 @@ static void test_formatrecord(void)
     ok( sz == 7, "size wrong\n");
     ok( 0 == strcmp(buffer,"boo hoo"), "wrong output\n");
 
-
     /* empty string */
     r = MsiRecordSetString(hrec, 0, "");
     sz = sizeof buffer;
@@ -734,11 +734,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-
-    todo_wine{
     ok( sz == 16, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,"{{2hey}1[dummy]}"), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{{[1][2]}[3]{[4][dummy]}}");
     r = MsiRecordSetString(hrec, 1, "2");
@@ -750,10 +747,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 0, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{{[1][2]}[3]} {[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -765,10 +760,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 12, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,"{{12}3} {12}"), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{[1][2]} {{[1][2]}[3]} {[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -780,10 +773,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 15, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,"12 {{12}3} {12}"), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{[4]}{[1][2]} {{[1][2]}[3]} {[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -795,11 +786,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 15, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,"12 {{12}3} {12}"), "wrong output (%s)\n",buffer);
-    }
-
 
     r = MsiRecordSetString(hrec, 0, "{blah} {[4]}{[1][2]} {{[1][2]}[3]} {[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -811,10 +799,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 22, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,"{blah} 12 {{12}3} {12}"), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{{[1]}[2]} {[4]}{[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -826,10 +812,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 13, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer,"{{1}2} {}{12}"), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{{[1]}} {[4]}{[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -841,10 +825,8 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine{
     ok( sz == 3, "size wrong,(%i)\n",sz);
     ok( 0 == strcmp(buffer," 12"), "wrong output (%s)\n",buffer);
-    }
 
     r = MsiRecordSetString(hrec, 0, "{{{[1]}} {[4]}{[1][2]}");
     r = MsiRecordSetString(hrec, 1, "1");
@@ -1015,61 +997,54 @@ static void test_formatrecord(void)
     ok( 0 == strcmp(buffer,"boo hoo"), "wrong output\n");
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{[1]}}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 0, "size wrong\n");
     ok( 0 == strcmp(buffer,""), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{ {[1]}}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( 0 == strcmp(buffer," {hoo}"), "wrong output\n");
-    ok( sz == 6, "size wrong\n");
+    todo_wine
+    {
+        ok( 0 == strcmp(buffer," {hoo}"), "wrong output\n");
+        ok( sz == 6, "size wrong\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{[1]} }");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 8, "size wrong\n");
     ok( 0 == strcmp(buffer,"{{hoo} }"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{ [1]}}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 0, "size wrong\n");
     ok( 0 == strcmp(buffer,""), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{[1] }}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 0, "size wrong\n");
     ok( 0 == strcmp(buffer,""), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{a}{b}{c }{ d}{any text}}");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 0, "size wrong\n");
     ok( 0 == strcmp(buffer,""), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{{a} }");
@@ -1079,13 +1054,11 @@ static void test_formatrecord(void)
     ok( 0 == strcmp(buffer,"{{a} }"), "wrong output\n");
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{a} {b}}");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 0, "size wrong\n");
     ok( 0 == strcmp(buffer,""), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     todo_wine {
@@ -1097,13 +1070,11 @@ static void test_formatrecord(void)
     }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{a b}}");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 0, "size wrong\n");
     ok( 0 == strcmp(buffer,""), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{ }");
@@ -1113,13 +1084,11 @@ static void test_formatrecord(void)
     ok( 0 == strcmp(buffer,"{ }"), "wrong output\n");
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, " {{a}}}");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 2, "size wrong\n");
     ok( 0 == strcmp(buffer," }"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     todo_wine {
@@ -1174,6 +1143,14 @@ static void test_formatrecord(void)
     ok( 0 == strcmp(buffer,"[\\[]"), "wrong output\n");
     ok( r == ERROR_SUCCESS, "format failed\n");
 
+    MsiRecordSetString(hrec, 0, "[\\[]");
+    MsiRecordSetString(hrec, 1, "hoo");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(0, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 4, "Expected 4, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "[\\[]"), "Expected \"[\\[]\", got \"%s\"\n", buffer);
+
     r = MsiRecordSetString(hrec, 0, "[foo]");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
@@ -1198,44 +1175,36 @@ static void test_formatrecord(void)
     ok( 0 == strcmp(buffer,"[%FOO]"), "wrong output\n");
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{[1]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 6, "size wrong\n");
     ok( 0 == strcmp(buffer,"{{hoo}"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{ {[1]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 8, "size wrong\n");
     ok( 0 == strcmp(buffer,"{{ {hoo}"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{ {[1]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 8, "size wrong\n");
     ok( 0 == strcmp(buffer,"{{ {hoo}"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{ {{[1]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 9, "size wrong\n");
     ok( 0 == strcmp(buffer,"{{ {{hoo}"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "[1]}");
@@ -1264,24 +1233,33 @@ static void test_formatrecord(void)
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 19, "size wrong\n");
-    ok( 0 == strcmp(buffer,"01{2{3{4hoo56}7}8}9"), "wrong output\n");
+    todo_wine
+    {
+        ok( sz == 19, "size wrong\n");
+        ok( 0 == strcmp(buffer,"01{2{3{4hoo56}7}8}9"), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "0{1{2[1]3}4");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 9, "size wrong\n");
-    ok( 0 == strcmp(buffer,"01{2hoo34"), "wrong output\n");
+    todo_wine
+    {
+        ok( sz == 9, "size wrong\n");
+        ok( 0 == strcmp(buffer,"01{2hoo34"), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "0{1{2[1]3}4");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 9, "size wrong\n");
-    ok( 0 == strcmp(buffer,"01{2hoo34"), "wrong output\n");
+    todo_wine
+    {
+        ok( sz == 9, "size wrong\n");
+        ok( 0 == strcmp(buffer,"01{2hoo34"), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{[1.} [1]");
@@ -1289,7 +1267,10 @@ static void test_formatrecord(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 9, "size wrong\n");
-    ok( 0 == strcmp(buffer,"{[1.} hoo"), "wrong output\n");
+    todo_wine
+    {
+        ok( 0 == strcmp(buffer,"{[1.} hoo"), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{[{[1]}]}");
@@ -1297,8 +1278,11 @@ static void test_formatrecord(void)
     r = MsiRecordSetString(hrec, 2, "foo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 9, "size wrong\n");
-    ok( 0 == strcmp(buffer,"{[{[1]}]}"), "wrong output\n");
+    todo_wine
+    {
+        ok( sz == 9, "size wrong\n");
+        ok( 0 == strcmp(buffer,"{[{[1]}]}"), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{[1][}");
@@ -1306,8 +1290,11 @@ static void test_formatrecord(void)
     r = MsiRecordSetString(hrec, 2, "foo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 2, "size wrong\n");
-    ok( 0 == strcmp(buffer,"2["), "wrong output\n");
+    todo_wine
+    {
+        ok( sz == 2, "size wrong\n");
+        ok( 0 == strcmp(buffer,"2["), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "[1]");
@@ -1319,17 +1306,17 @@ static void test_formatrecord(void)
     ok( 0 == strcmp(buffer,"[2]"), "wrong output\n");
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "[{{boo}}1]");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 3, "size wrong\n");
-    ok( 0 == strcmp(buffer,"[1]"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
+    ok( sz == 3, "size wrong\n");
+    todo_wine
+    {
+        ok( 0 == strcmp(buffer,"[1]"), "wrong output: %s\n", buffer);
+    }
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "[{{boo}}1]");
     r = MsiRecordSetString(hrec, 0, "[1{{boo}}]");
     r = MsiRecordSetString(hrec, 1, "hoo");
@@ -1337,33 +1324,36 @@ static void test_formatrecord(void)
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 3, "size wrong\n");
     ok( 0 == strcmp(buffer,"[1]"), "wrong output\n");
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{[1]{{boo} }}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 11, "size wrong\n");
-    ok( 0 == strcmp(buffer,"hoo{{boo }}"), "wrong output\n");
+    todo_wine
+    {
+        ok( sz == 11, "size wrong\n");
+        ok( 0 == strcmp(buffer,"hoo{{boo }}"), "wrong output\n");
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{[1{{boo}}]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 12, "size wrong: got %u, expected 12\n", sz);
-    ok( 0 == strcmp(buffer,"{[1{{boo}}]}"), "wrong output: got %s, expected [1]\n", buffer);
+    todo_wine
+    {
+        ok( sz == 12, "size wrong: got %u, expected 12\n", sz);
+        ok( 0 == strcmp(buffer,"{[1{{boo}}]}"), "wrong output: got %s, expected [1]\n", buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
-    todo_wine {
     r = MsiRecordSetString(hrec, 0, "{{[1]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( sz == 6, "size wrong: got %u, expected 3\n", sz);
     ok( 0 == strcmp(buffer,"{{hoo}"), "wrong output: got %s, expected [1]\n", buffer);
-    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{[1{{bo}o}}]}");
@@ -1377,16 +1367,22 @@ static void test_formatrecord(void)
     r = MsiRecordSetString(hrec, 0, "{[1{{b{o}o}}]}");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 14, "size wrong\n");
-    ok( 0 == strcmp(buffer,"{[1{{b{o}o}}]}"), "wrong output %s\n",buffer);
+    todo_wine
+    {
+        ok( sz == 14, "size wrong\n");
+        ok( 0 == strcmp(buffer,"{[1{{b{o}o}}]}"), "wrong output %s\n",buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "{ {[1]}");
     r = MsiRecordSetString(hrec, 1, "hoo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 5, "size wrong\n");
-    ok( 0 == strcmp(buffer," {hoo"), "wrong output %s\n",buffer);
+    todo_wine
+    {
+        ok( sz == 5, "size wrong\n");
+        ok( 0 == strcmp(buffer," {hoo"), "wrong output %s\n",buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     /* {} inside a substitution does strange things... */
@@ -1394,24 +1390,33 @@ static void test_formatrecord(void)
     r = MsiRecordSetString(hrec, 1, "2");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 5, "size wrong\n");
-    ok( 0 == strcmp(buffer,"[[1]]"), "wrong output %s\n",buffer);
+    todo_wine
+    {
+        ok( sz == 5, "size wrong\n");
+        ok( 0 == strcmp(buffer,"[[1]]"), "wrong output %s\n",buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "[[1]{}[1]]");
     r = MsiRecordSetString(hrec, 1, "2");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 6, "size wrong\n");
-    ok( 0 == strcmp(buffer,"[[1]2]"), "wrong output %s\n",buffer);
+    todo_wine
+    {
+        ok( sz == 6, "size wrong\n");
+        ok( 0 == strcmp(buffer,"[[1]2]"), "wrong output %s\n",buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "[a[1]b[1]c{}d[1]e]");
     r = MsiRecordSetString(hrec, 1, "2");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 14, "size wrong\n");
-    ok( 0 == strcmp(buffer,"[a[1]b[1]cd2e]"), "wrong output %s\n",buffer);
+    todo_wine
+    {
+        ok( sz == 14, "size wrong\n");
+        ok( 0 == strcmp(buffer,"[a[1]b[1]cd2e]"), "wrong output %s\n",buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     r = MsiRecordSetString(hrec, 0, "[a[1]b");
@@ -1489,8 +1494,11 @@ static void test_formatrecord(void)
     r = MsiRecordSetString(hrec, 1, "foo");
     sz = sizeof buffer;
     r = MsiFormatRecord(0, hrec, buffer, &sz);
-    ok( sz == 12, "size wrong\n");
-    ok( 0 == strcmp(buffer,"{foo[-1]foo}"), "wrong output %s\n",buffer);
+    todo_wine
+    {
+        ok( sz == 12, "size wrong\n");
+        ok( 0 == strcmp(buffer,"{foo[-1]foo}"), "wrong output %s\n",buffer);
+    }
     ok( r == ERROR_SUCCESS, "format failed\n");
 
     /* nested braces */
@@ -1540,31 +1548,22 @@ static void test_formatrecord(void)
     MsiRecordSetString(hrec, 0, "{abc}{{def}hi{j[one]k}}");
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 5, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,"{abc}"), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 5, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,"{abc}"), "wrong output (%s)\n",buffer);
 
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{{def}hi{j[one]k}}");
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 0, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 0, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
 
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{{def}hi{jk}}");
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 0, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 0, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
 
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{{{def}}hi{jk}}");
@@ -1590,21 +1589,15 @@ static void test_formatrecord(void)
     MsiRecordSetString(hrec, 0, "{{def}{jk}}");
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 0, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 0, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
 
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{{def}}");
     r = MsiFormatRecord(0, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 0, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 0, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
 
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{a{b}c}");
@@ -1646,6 +1639,33 @@ static void test_formatrecord(void)
     ok( sz == 8, "size wrong(%i)\n",sz);
     ok( 0 == strcmp(buffer,"100 -100"), "wrong output (%s)\n",buffer);
 
+    sz = sizeof(buffer);
+    MsiRecordSetString(hrec, 0, "[1] {[noprop] [twoprop]} {abcdef}");
+    MsiRecordSetString(hrec, 1, "one");
+    r = MsiFormatRecord(0, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 33, "Expected 33, got %d\n",sz);
+    ok(!lstrcmpA(buffer, "one {[noprop] [twoprop]} {abcdef}"),
+       "Expected \"one {[noprop] [twoprop]} {abcdef}\", got \"%s\"\n", buffer);
+
+    sz = sizeof(buffer);
+    MsiRecordSetString(hrec, 0, "[1] {[noprop] [one]} {abcdef}");
+    MsiRecordSetString(hrec, 1, "one");
+    r = MsiFormatRecord(0, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 29, "Expected 29, got %d\n",sz);
+    ok(!lstrcmpA(buffer, "one {[noprop] [one]} {abcdef}"),
+       "Expected \"one {[noprop] [one]} {abcdef}\", got \"%s\"\n", buffer);
+
+    sz = sizeof(buffer);
+    MsiRecordSetString(hrec, 0, "[1] {[one]} {abcdef}");
+    MsiRecordSetString(hrec, 1, "one");
+    r = MsiFormatRecord(0, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 20, "Expected 20, got %d\n",sz);
+    ok(!lstrcmpA(buffer, "one {[one]} {abcdef}"),
+       "Expected \"one {[one]} {abcdef}\", got \"%s\"\n", buffer);
+
     MsiCloseHandle( hrec );
 }
 
@@ -1706,7 +1726,9 @@ static void test_formatrecord_package(void)
     todo_wine
     {
         ok( sz == 66, "size wrong (%i)\n",sz);
-        ok( 0 == strcmp(buffer,"1: [2] 2: stuff 3: prop 4: val 5:  6:  7:  8:  9:  10:  11:  12:  "), "wrong output(%s)\n",buffer);
+        ok( !lstrcmpA(buffer,
+            "1: [2] 2: stuff 3: prop 4: val 5:  6:  7:  8:  9:  10:  11:  12:  "),
+            "wrong output(%s)\n",buffer);
     }
 
     /* now put play games with escaping */
@@ -1758,6 +1780,101 @@ static void test_formatrecord_package(void)
     ok( sz == 0, "size wrong: %d\n", sz);
     ok( 0 == strcmp(buffer,""), "wrong output: %s\n", buffer);
 
+    MsiRecordSetString(hrec, 0, "\\x");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 2, "Expected 2, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "\\x"), "Expected \"\\x\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\[");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 3, "Expected 3, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "[\\["), "Expected \"[\\[\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\[]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 1, "Expected 1, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "["), "Expected \"[\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[[]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 3, "Expected 3, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "[[]"), "Expected \"[]\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\[]]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 2, "Expected 2, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "[]"), "Expected \"[]\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\[a]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 1, "Expected 1, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "["), "Expected \"[\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\a[]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    todo_wine
+    {
+        ok(sz == 1, "Expected 1, got %d\n", sz);
+        ok(!lstrcmpA(buffer, "a"), "Expected \"a\", got \"%s\"\n", buffer);
+    }
+
+    MsiRecordSetString(hrec, 0, "[prop]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 3, "Expected 3, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "val"), "Expected \"val\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[prop] [pro\\pblah] [prop]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 8, "Expected 8, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "val  val"), "Expected \"val  val\", got \"%s\"\n", buffer);
+
+    MsiSetPropertyA(package, "b", "ball");
+    MsiRecordSetString(hrec, 0, "[\\b]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 1, "Expected 1, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "b"), "Expected \"b\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\c]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 1, "Expected 1, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "c"), "Expected \"c\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\[]prop]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 6, "Expected 6, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "[prop]"), "Expected \"[prop]\", got \"%s\"\n", buffer);
+
+    MsiRecordSetString(hrec, 0, "[\\a]prop]");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 6, "Expected 6, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "aprop]"), "Expected \"aprop]\", got \"%s\"\n", buffer);
+
     MsiRecordSetString(hrec, 0, "[\\[]Bracket Text[\\]]");
     sz = sizeof buffer;
     r = MsiFormatRecord(package, hrec, buffer, &sz);
@@ -1773,8 +1890,19 @@ static void test_formatrecord_package(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    ok( sz == 9, "size wrong\n");
-    ok( 0 == strcmp(buffer,"boo "), "wrong output\n");
+    ok( sz == 9, "size wrong: %d\n", sz);
+    ok( 0 == strcmp(buffer,"boo "), "wrong output: %s\n", buffer);
+    ok(!lstrcmpA(&buffer[5], " hoo"),
+       "Expected \" hoo\", got \"%s\"\n", &buffer[5]);
+
+    MsiRecordSetString(hrec, 0, "[1] [~abc] [2]");
+    MsiRecordSetString(hrec, 1, "boo");
+    MsiRecordSetString(hrec, 2, "hoo");
+    sz = sizeof(buffer);
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 8, "Expected 8, got %d\n", sz);
+    ok(!lstrcmpA(buffer, "boo  hoo"), "Expected \"boo  hoo\", got \"%s\"\n", buffer);
 
     /* properties */
     r = MsiSetProperty(package,"dummy","Bork");
@@ -1799,7 +1927,6 @@ static void test_formatrecord_package(void)
     ok( sz == 8, "size wrong\n");
     ok( 0 == strcmp(buffer,"boo  hoo"), "wrong output\n");
 
-
     /* nesting tests */
     r = MsiSetProperty(package,"dummya","foo");
     r = MsiSetProperty(package,"dummyb","baa");
@@ -1816,7 +1943,6 @@ static void test_formatrecord_package(void)
     ok( sz == 12, "size wrong(%i)\n",sz);
     ok( 0 == strcmp(buffer,"foo baa whoa"), "wrong output (%s)\n",buffer);
 
-
     r = MsiSetProperty(package,"dummya","1");
     r = MsiSetProperty(package,"dummyb","[2]");
     ok( r == ERROR_SUCCESS, "set property failed\n");
@@ -1828,8 +1954,11 @@ static void test_formatrecord_package(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    ok( sz == 9, "size wrong(%i)\n",sz);
-    ok( 0 == strcmp(buffer,"1 [1] [2]"), "wrong output (%s)\n",buffer);
+    todo_wine
+    {
+        ok( sz == 9, "size wrong(%i)\n",sz);
+        ok( 0 == strcmp(buffer,"1 [1] [2]"), "wrong output (%s)\n",buffer);
+    }
 
     r = MsiSetProperty(package,"dummya","1");
     r = MsiSetProperty(package,"dummyb","a");
@@ -1855,8 +1984,11 @@ static void test_formatrecord_package(void)
     sz = sizeof buffer;
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    ok( sz == 11, "size wrong(%i)\n",sz);
-    ok( 0 == strcmp(buffer,"boo hoo [3]"), "wrong output (%s)\n",buffer);
+    todo_wine
+    {
+        ok( sz == 11, "size wrong(%i)\n",sz);
+        ok( 0 == strcmp(buffer,"boo hoo [3]"), "wrong output (%s)\n",buffer);
+    }
 
     r = MsiRecordSetString(hrec, 0, "[1] [2] [[3]]");
     r = MsiRecordSetString(hrec, 1, "boo");
@@ -1943,7 +2075,7 @@ static void test_formatrecord_package(void)
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{a[one]bc[two]de[one]f}");
     r = MsiFormatRecord(package, hrec, buffer, &sz);
-    ok( r == ERROR_SUCCESS, "format failed\n");
+    ok( r == ERROR_SUCCESS, "format failed: %d\n", r);
     ok( sz == 25, "size wrong(%i)\n",sz);
     ok( 0 == strcmp(buffer,"amercurybcvenusdemercuryf"), "wrong output (%s)\n",buffer);
 
@@ -1970,38 +2102,66 @@ static void test_formatrecord_package(void)
     MsiRecordSetString(hrec, 0, "{abc{d[one]ef}"); /* missing final brace */
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    ok( sz == 14, "size wrong(%i)\n",sz);
-    ok( 0 == strcmp(buffer,"abc{dmercuryef"), "wrong output (%s)\n",buffer);
+    todo_wine
+    {
+        ok( sz == 14, "size wrong(%i)\n",sz);
+        ok( 0 == strcmp(buffer,"abc{dmercuryef"), "wrong output (%s)\n",buffer);
+    }
 
     MsiSetProperty(package, "one", "mercury");
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{abc{d[one]ef}}");
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    ok( sz == 15, "size wrong(%i)\n",sz);
-    ok( 0 == strcmp(buffer,"abc{dmercuryef}"), "wrong output (%s)\n",buffer);
+    todo_wine
+    {
+        ok( sz == 15, "size wrong(%i)\n",sz);
+        ok( 0 == strcmp(buffer,"abc{dmercuryef}"), "wrong output (%s)\n",buffer);
+    }
 
     MsiSetProperty(package, "one", "mercury");
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{abc}{{def}hi{j[one]k}}");
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 5, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,"{abc}"), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 5, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,"{abc}"), "wrong output (%s)\n",buffer);
 
     MsiSetProperty(package, "one", "mercury");
+
     sz = sizeof buffer;
     MsiRecordSetString(hrec, 0, "{{def}hi{j[one]k}}");
     r = MsiFormatRecord(package, hrec, buffer, &sz);
     ok( r == ERROR_SUCCESS, "format failed\n");
-    todo_wine
-    {
-        ok( sz == 0, "size wrong(%i)\n",sz);
-        ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
-    }
+    ok( sz == 0, "size wrong(%i)\n",sz);
+    ok( 0 == strcmp(buffer,""), "wrong output (%s)\n",buffer);
+
+    sz = sizeof(buffer);
+    MsiRecordSetString(hrec, 0, "[1] {[noprop] [twoprop]} {abcdef}");
+    MsiRecordSetString(hrec, 1, "one");
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 13, "Expected 13, got %d\n",sz);
+    ok(!lstrcmpA(buffer, "one  {abcdef}"),
+       "Expected \"one  {abcdef}\", got \"%s\"\n", buffer);
+
+    sz = sizeof(buffer);
+    MsiRecordSetString(hrec, 0, "[1] {[noprop] [one]} {abcdef}");
+    MsiRecordSetString(hrec, 1, "one");
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 13, "Expected 13, got %d\n",sz);
+    ok(!lstrcmpA(buffer, "one  {abcdef}"),
+       "Expected \"one  {abcdef}\", got \"%s\"\n", buffer);
+
+    sz = sizeof(buffer);
+    MsiRecordSetString(hrec, 0, "[1] {[one]} {abcdef}");
+    MsiRecordSetString(hrec, 1, "one");
+    r = MsiFormatRecord(package, hrec, buffer, &sz);
+    ok(r == ERROR_SUCCESS, "Expected ERROR_SUCCESS, got %d\n", r);
+    ok(sz == 20, "Expected 20, got %d\n",sz);
+    ok(!lstrcmpA(buffer, "one mercury {abcdef}"),
+       "Expected \"one mercury {abcdef}\", got \"%s\"\n", buffer);
 
     MsiCloseHandle(hrec);
 
@@ -2017,6 +2177,7 @@ static void test_formatrecord_tables(void)
     CHAR buf[MAX_PATH];
     CHAR curr_dir[MAX_PATH];
     CHAR expected[MAX_PATH];
+    CHAR root[MAX_PATH];
     DWORD size;
     UINT r;
 
@@ -2145,48 +2306,57 @@ static void test_formatrecord_tables(void)
     r = MsiDoAction(hpkg, "CostFinalize");
     ok( r == ERROR_SUCCESS, "CostFinalize failed: %d\n", r);
 
+    size = MAX_PATH;
+    MsiGetProperty( hpkg, "ROOTDRIVE", root, &size );
+
+    sprintf( expected, "1: %sfrontal.txt ", root);
+
     /* frontal full file key */
     size = MAX_PATH;
     MsiRecordSetString( hrec, 1, "[#frontal_file]" );
     r = MsiFormatRecord( hpkg, hrec, buf, &size );
     ok( r == ERROR_SUCCESS, "format record failed: %d\n", r);
-    ok( !lstrcmp( buf, "1: C:\\frontal.txt " ), "Expected '1: C:\\frontal.txt ', got %s\n", buf);
+    ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
 
     /* frontal short file key */
     size = MAX_PATH;
     MsiRecordSetString( hrec, 1, "[!frontal_file]" );
     r = MsiFormatRecord( hpkg, hrec, buf, &size );
     ok( r == ERROR_SUCCESS, "format record failed: %d\n", r);
-    ok( !lstrcmp( buf, "1: C:\\frontal.txt " ), "Expected '1: C:\\frontal.txt ', got %s\n", buf);
+    ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
+
+    sprintf( expected, "1: %sI am a really long directory\\temporal.txt ", root);
 
     /* temporal full file key */
     size = MAX_PATH;
     MsiRecordSetString( hrec, 1, "[#temporal_file]" );
     r = MsiFormatRecord( hpkg, hrec, buf, &size );
     ok( r == ERROR_SUCCESS, "format record failed: %d\n", r);
-    ok( !lstrcmp( buf, "1: C:\\I am a really long directory\\temporal.txt " ),
-        "Expected '1: C:\\I am a really long directory\\temporal.txt ', got %s\n", buf);
+    ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
 
     /* temporal short file key */
     size = MAX_PATH;
     MsiRecordSetString( hrec, 1, "[!temporal_file]" );
     r = MsiFormatRecord( hpkg, hrec, buf, &size );
     ok( r == ERROR_SUCCESS, "format record failed: %d\n", r);
-    ok( !lstrcmp( buf, "1: C:\\I am a really long directory\\temporal.txt " ),
-        "Expected '1: C:\\I am a really long directory\\temporal.txt ', got %s\n", buf);
+    ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
 
     /* custom action 51, files don't exist */
     r = MsiDoAction( hpkg, "MyCustom" );
     ok( r == ERROR_SUCCESS, "MyCustom failed: %d\n", r);
 
+    sprintf( expected, "%sI am a really long directory\\temporal.txt", root);
+
     size = MAX_PATH;
     r = MsiGetProperty( hpkg, "prop", buf, &size );
     ok( r == ERROR_SUCCESS, "get property failed: %d\n", r);
-    ok( !lstrcmp( buf, "C:\\I am a really long directory\\temporal.txt" ),
-        "Expected 'C:\\I am a really long directory\\temporal.txt', got %s\n", buf);
+    ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
 
-    CreateDirectory( "C:\\I am a really long directory", NULL );
-    create_test_file( "C:\\I am a really long directory\\temporal.txt" );
+    sprintf( buf, "%sI am a really long directory", root );
+    CreateDirectory( buf, NULL );
+
+    lstrcat( buf, "\\temporal.txt" );
+    create_test_file( buf );
 
     /* custom action 51, files exist */
     r = MsiDoAction( hpkg, "MyCustom" );
@@ -2197,8 +2367,7 @@ static void test_formatrecord_tables(void)
     ok( r == ERROR_SUCCESS, "get property failed: %d\n", r);
     todo_wine
     {
-        ok( !lstrcmp( buf, "C:\\I am a really long directory\\temporal.txt" ),
-            "Expected 'C:\\I am a really long directory\\temporal.txt', got %s\n", buf);
+        ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
     }
 
     /* custom action 51, escaped text 1 */
@@ -2228,13 +2397,14 @@ static void test_formatrecord_tables(void)
     ok( r == ERROR_SUCCESS, "get property failed: %d\n", r);
     ok( !lstrcmp( buf, "" ), "Expected '', got %s\n", buf);
 
+    sprintf( expected, "1: %sI am a really long directory\\ ", root);
+
     /* component with INSTALLSTATE_LOCAL */
     size = MAX_PATH;
     MsiRecordSetString( hrec, 1, "[$temporal]" );
     r = MsiFormatRecord( hpkg, hrec, buf, &size );
     ok( r == ERROR_SUCCESS, "format record failed: %d\n", r);
-    ok( !lstrcmp( buf, "1: C:\\I am a really long directory\\ " ),
-        "Expected '1: C:\\I am a really long directory\\ ', got %s\n", buf);
+    ok( !lstrcmp( buf, expected ), "Expected \"%s\", got \"%s\"\n", expected, buf);
 
     r = MsiSetComponentState( hpkg, "temporal", INSTALLSTATE_SOURCE );
     ok( r == ERROR_SUCCESS, "failed to set install state: %d\n", r);
@@ -2249,8 +2419,11 @@ static void test_formatrecord_tables(void)
     ok( r == ERROR_SUCCESS, "format record failed: %d\n", r);
     ok( !lstrcmp( buf, expected ), "Expected '%s', got %s\n", expected, buf);
 
-    DeleteFile( "C:\\I am a really long directory\\temporal.txt" );
-    RemoveDirectory( "C:\\I am a really long directory" );
+    sprintf( buf, "%sI am a really long directory\\temporal.txt", root );
+    DeleteFile( buf );
+
+    sprintf( buf, "%sI am a really long directory", root );
+    RemoveDirectory( buf );
 
     MsiCloseHandle( hrec );
     MsiCloseHandle( hpkg );
