@@ -35,6 +35,14 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(msidb);
 
+#define MSI_HASH_TABLE_SIZE 37
+
+typedef struct tagMSIHASHENTRY
+{
+    struct tagMSIHASHENTRY *next;
+    UINT value;
+    UINT row;
+} MSIHASHENTRY;
 
 /* below is the query interface to a table */
 
@@ -44,13 +52,78 @@ typedef struct tagMSIWHEREVIEW
     MSIDATABASE   *db;
     MSIVIEW       *table;
     UINT           row_count;
-    UINT          *reorder;
+    MSIHASHENTRY **reorder;
     struct expr   *cond;
+    UINT           rec_index;
 } MSIWHEREVIEW;
 
+static void free_hash_table(MSIHASHENTRY **table)
+{
+    MSIHASHENTRY *new, *old;
+    int i;
+
+    if (!table)
+        return;
+
+    for (i = 0; i < MSI_HASH_TABLE_SIZE; i++)
+    {
+        new = table[i];
+
+        while (new)
+        {
+            old = new;
+            new = old->next;
+            msi_free(old);
+        }
+
+        table[i] = NULL;
+    }
+
+    msi_free(table);
+}
+
+static UINT find_entry_in_hash(MSIHASHENTRY **table, UINT row, UINT *val)
+{
+    MSIHASHENTRY *entry;
+
+    if (!(entry = table[row % MSI_HASH_TABLE_SIZE]))
+    {
+        WARN("Row not found in hash table!\n");
+        return ERROR_FUNCTION_FAILED;
+    }
+
+    while (entry && entry->row != row)
+        entry = entry->next;
+
+    if (entry) *val = entry->value;
+    return ERROR_SUCCESS;
+}
+
+static UINT add_entry_to_hash(MSIHASHENTRY **table, UINT row, UINT val)
+{
+    MSIHASHENTRY *new = msi_alloc(sizeof(MSIHASHENTRY));
+    MSIHASHENTRY *prev;
+
+    if (!new)
+        return ERROR_OUTOFMEMORY;
+
+    new->next = NULL;
+    new->value = val;
+    new->row = row;
+
+    prev = table[row % MSI_HASH_TABLE_SIZE];
+    if (prev)
+        new->next = prev;
+
+    table[row % MSI_HASH_TABLE_SIZE] = new;
+
+    return ERROR_SUCCESS;
+}
+
 static UINT WHERE_fetch_int( struct tagMSIVIEW *view, UINT row, UINT col, UINT *val )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
+    UINT r;
 
     TRACE("%p %d %d %p\n", wv, row, col, val );
 
@@ -60,7 +133,9 @@ static UINT WHERE_fetch_int( struct tagMSIVIEW *view, UINT row, UINT col, UINT *
     if( row > wv->row_count )
         return ERROR_NO_MORE_ITEMS;
 
-    row = wv->reorder[ row ];
+    r = find_entry_in_hash(wv->reorder, row, &row);
+    if (r != ERROR_SUCCESS)
+        return r;
 
     return wv->table->ops->fetch_int( wv->table, row, col, val );
 }
@@ -68,6 +143,7 @@ static UINT WHERE_fetch_int( struct tagMSIVIEW *view, UINT row, UINT col, UINT *
 static UINT WHERE_fetch_stream( struct tagMSIVIEW *view, UINT row, UINT col, IStream **stm )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
+    UINT r;
 
     TRACE("%p %d %d %p\n", wv, row, col, stm );
 
@@ -77,14 +153,37 @@ static UINT WHERE_fetch_stream( struct tagMSIVIEW *view, UINT row, UINT col, ISt
     if( row > wv->row_count )
         return ERROR_NO_MORE_ITEMS;
 
-    row = wv->reorder[ row ];
+    r = find_entry_in_hash(wv->reorder, row, &row);
+    if (r != ERROR_SUCCESS)
+        return r;
 
     return wv->table->ops->fetch_stream( wv->table, row, col, stm );
 }
 
+static UINT WHERE_get_row( struct tagMSIVIEW *view, UINT row, MSIRECORD **rec )
+{
+    MSIWHEREVIEW *wv = (MSIWHEREVIEW *)view;
+    UINT r;
+
+    TRACE("%p %d %p\n", wv, row, rec );
+
+    if (!wv->table)
+        return ERROR_FUNCTION_FAILED;
+
+    if (row > wv->row_count)
+        return ERROR_NO_MORE_ITEMS;
+
+    r = find_entry_in_hash(wv->reorder, row, &row);
+    if (r != ERROR_SUCCESS)
+        return r;
+
+    return wv->table->ops->get_row(view, row, rec);
+}
+
 static UINT WHERE_set_row( struct tagMSIVIEW *view, UINT row, MSIRECORD *rec, UINT mask )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
+    UINT r;
 
     TRACE("%p %d %p %08x\n", wv, row, rec, mask );
 
@@ -94,11 +193,33 @@ static UINT WHERE_set_row( struct tagMSIVIEW *view, UINT row, MSIRECORD *rec, UI
     if( row > wv->row_count )
         return ERROR_NO_MORE_ITEMS;
 
-    row = wv->reorder[ row ];
+    r = find_entry_in_hash(wv->reorder, row, &row);
+    if (r != ERROR_SUCCESS)
+        return r;
 
     return wv->table->ops->set_row( wv->table, row, rec, mask );
 }
 
+static UINT WHERE_delete_row(struct tagMSIVIEW *view, UINT row)
+{
+    MSIWHEREVIEW *wv = (MSIWHEREVIEW *)view;
+    UINT r;
+
+    TRACE("(%p %d)\n", view, row);
+
+    if ( !wv->table )
+        return ERROR_FUNCTION_FAILED;
+
+    if ( row > wv->row_count )
+        return ERROR_NO_MORE_ITEMS;
+
+    r = find_entry_in_hash( wv->reorder, row, &row );
+    if ( r != ERROR_SUCCESS )
+        return r;
+
+    return wv->table->ops->delete_row( wv->table, row );
+}
+
 static INT INT_evaluate_binary( INT lval, UINT op, INT rval )
 {
     switch( op )
@@ -139,24 +260,25 @@ static INT INT_evaluate_unary( INT lval, UINT op )
     return 0;
 }
 
-static const WCHAR *STRING_evaluate( string_table *st,
-              MSIVIEW *table, UINT row, struct expr *expr, MSIRECORD *record )
+static const WCHAR *STRING_evaluate( MSIWHEREVIEW *wv, UINT row,
+                                     const struct expr *expr,
+                                     const MSIRECORD *record )
 {
     UINT val = 0, r;
 
     switch( expr->type )
     {
     case EXPR_COL_NUMBER_STRING:
-        r = table->ops->fetch_int( table, row, expr->u.col_number, &val );
+        r = wv->table->ops->fetch_int( wv->table, row, expr->u.col_number, &val );
         if( r != ERROR_SUCCESS )
             return NULL;
-        return msi_string_lookup_id( st, val );
+        return msi_string_lookup_id( wv->db->strings, val );
 
     case EXPR_SVAL:
         return expr->u.sval;
 
     case EXPR_WILDCARD:
-        return MSI_RecordGetString( record, 1 );
+        return MSI_RecordGetString( record, ++wv->rec_index );
 
     default:
         ERR("Invalid expression type\n");
@@ -165,15 +287,16 @@ static const WCHAR *STRING_evaluate( string_table *st,
     return NULL;
 }
 
-static UINT STRCMP_Evaluate( string_table *st, MSIVIEW *table, UINT row, 
-                             struct expr *cond, INT *val, MSIRECORD *record )
+static UINT STRCMP_Evaluate( MSIWHEREVIEW *wv, UINT row, const struct expr *cond,
+                             INT *val, const MSIRECORD *record )
 {
     int sr;
     const WCHAR *l_str, *r_str;
 
-    l_str = STRING_evaluate( st, table, row, cond->u.expr.left, record );
-    r_str = STRING_evaluate( st, table, row, cond->u.expr.right, record );
-    if( l_str == r_str )
+    l_str = STRING_evaluate( wv, row, cond->u.expr.left, record );
+    r_str = STRING_evaluate( wv, row, cond->u.expr.right, record );
+    if( l_str == r_str ||
+        ((!l_str || !*l_str) && (!r_str || !*r_str)) )
         sr = 0;
     else if( l_str && ! r_str )
         sr = 1;
@@ -183,14 +306,15 @@ static UINT STRCMP_Evaluate( string_table *st, MSIVIEW *table, UINT row,
         sr = lstrcmpW( l_str, r_str );
 
     *val = ( cond->u.expr.op == OP_EQ && ( sr == 0 ) ) ||
+           ( cond->u.expr.op == OP_NE && ( sr != 0 ) ) ||
            ( cond->u.expr.op == OP_LT && ( sr < 0 ) ) ||
            ( cond->u.expr.op == OP_GT && ( sr > 0 ) );
 
     return ERROR_SUCCESS;
 }
 
-static UINT WHERE_evaluate( MSIDATABASE *db, MSIVIEW *table, UINT row, 
-                             struct expr *cond, INT *val, MSIRECORD *record )
+static UINT WHERE_evaluate( MSIWHEREVIEW *wv, UINT row,
+                            struct expr *cond, INT *val, MSIRECORD *record )
 {
     UINT r, tval;
     INT lval, rval;
@@ -201,12 +325,12 @@ static UINT WHERE_evaluate( MSIDATABASE *db, MSIVIEW *table, UINT row,
     switch( cond->type )
     {
     case EXPR_COL_NUMBER:
-        r = table->ops->fetch_int( table, row, cond->u.col_number, &tval );
+        r = wv->table->ops->fetch_int( wv->table, row, cond->u.col_number, &tval );
         *val = tval - 0x8000;
         return ERROR_SUCCESS;
 
     case EXPR_COL_NUMBER32:
-        r = table->ops->fetch_int( table, row, cond->u.col_number, &tval );
+        r = wv->table->ops->fetch_int( wv->table, row, cond->u.col_number, &tval );
         *val = tval - 0x80000000;
         return r;
 
@@ -215,27 +339,27 @@ static UINT WHERE_evaluate( MSIDATABASE *db, MSIVIEW *table, UINT row,
         return ERROR_SUCCESS;
 
     case EXPR_COMPLEX:
-        r = WHERE_evaluate( db, table, row, cond->u.expr.left, &lval, record );
+        r = WHERE_evaluate( wv, row, cond->u.expr.left, &lval, record );
         if( r != ERROR_SUCCESS )
             return r;
-        r = WHERE_evaluate( db, table, row, cond->u.expr.right, &rval, record );
+        r = WHERE_evaluate( wv, row, cond->u.expr.right, &rval, record );
         if( r != ERROR_SUCCESS )
             return r;
         *val = INT_evaluate_binary( lval, cond->u.expr.op, rval );
         return ERROR_SUCCESS;
 
     case EXPR_UNARY:
-        r = table->ops->fetch_int( table, row, cond->u.expr.left->u.col_number, &tval );
+        r = wv->table->ops->fetch_int( wv->table, row, cond->u.expr.left->u.col_number, &tval );
         if( r != ERROR_SUCCESS )
             return r;
         *val = INT_evaluate_unary( tval, cond->u.expr.op );
         return ERROR_SUCCESS;
 
     case EXPR_STRCMP:
-        return STRCMP_Evaluate( db->strings, table, row, cond, val, record );
+        return STRCMP_Evaluate( wv, row, cond, val, record );
 
     case EXPR_WILDCARD:
-        *val = MSI_RecordGetInteger( record, 1 );
+        *val = MSI_RecordGetInteger( record, ++wv->rec_index );
         return ERROR_SUCCESS;
 
     default:
@@ -244,7 +368,6 @@ static UINT WHERE_evaluate( MSIDATABASE *db, MSIVIEW *table, UINT row,
     }
 
     return ERROR_SUCCESS;
-
 }
 
 static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
@@ -267,10 +390,10 @@ static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
     if( r != ERROR_SUCCESS )
         return r;
 
-    msi_free( wv->reorder );
-    wv->reorder = msi_alloc( count*sizeof(UINT) );
+    free_hash_table(wv->reorder);
+    wv->reorder = msi_alloc_zero(MSI_HASH_TABLE_SIZE * sizeof(MSIHASHENTRY *));
     if( !wv->reorder )
-        return ERROR_FUNCTION_FAILED;
+        return ERROR_OUTOFMEMORY;
 
     wv->row_count = 0;
     if (wv->cond->type == EXPR_STRCMP)
@@ -307,7 +430,7 @@ static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
             {
                 r = table->ops->find_matching_rows(table, col, value, &row, &handle);
                 if (r == ERROR_SUCCESS)
-                    wv->reorder[ wv->row_count ++ ] = row;
+                    add_entry_to_hash(wv->reorder, wv->row_count++, row);
             } while (r == ERROR_SUCCESS);
 
             if (r == ERROR_NO_MORE_ITEMS)
@@ -321,11 +444,12 @@ static UINT WHERE_execute( struct tagMSIVIEW *view, MSIRECORD *record )
     for( i=0; i<count; i++ )
     {
         val = 0;
-        r = WHERE_evaluate( wv->db, table, i, wv->cond, &val, record );
+        wv->rec_index = 0;
+        r = WHERE_evaluate( wv, i, wv->cond, &val, record );
         if( r != ERROR_SUCCESS )
             return r;
         if( val )
-            wv->reorder[ wv->row_count ++ ] = i;
+            add_entry_to_hash( wv->reorder, wv->row_count++, i );
     }
 
     return ERROR_SUCCESS;
@@ -338,10 +462,7 @@ static UINT WHERE_close( struct tagMSIVIEW *view )
     TRACE("%p\n", wv );
 
     if( !wv->table )
-         return ERROR_FUNCTION_FAILED;
-
-    msi_free( wv->reorder );
-    wv->reorder = NULL;
+        return ERROR_FUNCTION_FAILED;
 
     return wv->table->ops->close( wv->table );
 }
@@ -379,16 +500,19 @@ static UINT WHERE_get_column_info( struct tagMSIVIEW *view,
 }
 
 static UINT WHERE_modify( struct tagMSIVIEW *view, MSIMODIFY eModifyMode,
-                MSIRECORD *rec )
+                          MSIRECORD *rec, UINT row )
 {
     MSIWHEREVIEW *wv = (MSIWHEREVIEW*)view;
+    UINT r;
 
-    TRACE("%p %d %p\n", wv, eModifyMode, rec );
+    TRACE("%p %d %p\n", wv, eModifyMode, rec);
 
-    if( !wv->table )
-         return ERROR_FUNCTION_FAILED;
+    r = WHERE_execute(view, NULL);
+    if (r != ERROR_SUCCESS)
+        return r;
 
-    return wv->table->ops->modify( wv->table, eModifyMode, rec );
+    find_entry_in_hash(wv->reorder, row - 1, &row);
+    return wv->table->ops->modify( wv->table, eModifyMode, rec, row );
 }
 
 static UINT WHERE_delete( struct tagMSIVIEW *view )
@@ -401,7 +525,7 @@ static UINT WHERE_delete( struct tagMSIVIEW *view )
         wv->table->ops->delete( wv->table );
     wv->table = 0;
 
-    msi_free( wv->reorder );
+    free_hash_table(wv->reorder);
     wv->reorder = NULL;
     wv->row_count = 0;
 
@@ -423,29 +547,44 @@ static UINT WHERE_find_matching_rows( struct tagMSIVIEW *view, UINT col,
          return ERROR_FUNCTION_FAILED;
 
     r = wv->table->ops->find_matching_rows( wv->table, col, val, row, handle );
+    if (r != ERROR_SUCCESS)
+        return r;
 
     if( *row > wv->row_count )
         return ERROR_NO_MORE_ITEMS;
 
-    *row = wv->reorder[ *row ];
-
-    return r;
+    return find_entry_in_hash(wv->reorder, *row, row);
 }
 
+static UINT WHERE_sort(struct tagMSIVIEW *view, column_info *columns)
+{
+    MSIWHEREVIEW *wv = (MSIWHEREVIEW *)view;
+
+    TRACE("%p %p\n", view, columns);
+
+    return wv->table->ops->sort(wv->table, columns);
+}
 
 static const MSIVIEWOPS where_ops =
 {
     WHERE_fetch_int,
     WHERE_fetch_stream,
+    WHERE_get_row,
     WHERE_set_row,
     NULL,
+    WHERE_delete_row,
     WHERE_execute,
     WHERE_close,
     WHERE_get_dimensions,
     WHERE_get_column_info,
     WHERE_modify,
     WHERE_delete,
-    WHERE_find_matching_rows
+    WHERE_find_matching_rows,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    WHERE_sort,
 };
 
 static UINT WHERE_VerifyCondition( MSIDATABASE *db, MSIVIEW *table, struct expr *cond,
@@ -478,7 +617,7 @@ static UINT WHERE_VerifyCondition( MSIDATABASE *db, MSIVIEW *table, struct expr
         else
         {
             *valid = 0;
-            ERR("Couldn't find column %s\n", debugstr_w( cond->u.column ) );
+            WARN("Couldn't find column %s\n", debugstr_w( cond->u.column ) );
         }
         break;
     case EXPR_COMPLEX:
@@ -502,6 +641,7 @@ static UINT WHERE_VerifyCondition( MSIDATABASE *db, MSIVIEW *table, struct expr
             case OP_EQ:
             case OP_GT:
             case OP_LT:
+            case OP_NE:
                 break;
             default:
                 *valid = FALSE;
@@ -580,6 +720,7 @@ UINT WHERE_CreateView( MSIDATABASE *db, MSIVIEW **view, MSIVIEW *table,
     wv->row_count = 0;
     wv->reorder = NULL;
     wv->cond = cond;
+    wv->rec_index = 0;
     *view = (MSIVIEW*) wv;
 
     return ERROR_SUCCESS;
