@@ -28,7 +28,6 @@
 #include "winnls.h"
 #include "wingdi.h"
 #include "winuser.h"
-#include "winreg.h"
 #include "winternl.h"
 #include "vfw.h"
 #include "digitalv.h"
@@ -244,7 +243,7 @@ static void MCIWND_UpdateState(MCIWndInfo *mwi)
         strcatW(buffer, r_braceW);
     }
 
-    TRACE("=> '%s'\n", debugstr_w(buffer));
+    TRACE("=> %s\n", debugstr_w(buffer));
     SetWindowTextW(mwi->hWnd, buffer);
 }
 
@@ -448,7 +447,7 @@ static LRESULT WINAPI MCIWndProc(HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lPa
 {
     MCIWndInfo *mwi;
 
-    TRACE("%p %04x %08x %08lx\n", hWnd, wMsg, wParam, lParam);
+    TRACE("%p %04x %08lx %08lx\n", hWnd, wMsg, wParam, lParam);
 
     mwi = (MCIWndInfo*)GetWindowLongW(hWnd, 0);
     if (!mwi && wMsg != WM_CREATE)
@@ -1057,7 +1056,7 @@ end_of_mci_open:
         return mwi->inactive_timer;
 
     case MCIWNDM_CHANGESTYLES:
-        TRACE("MCIWNDM_CHANGESTYLES mask %08x, set %08lx\n", wParam, lParam);
+        TRACE("MCIWNDM_CHANGESTYLES mask %08lx, set %08lx\n", wParam, lParam);
         /* FIXME: update the visual window state as well:
          * add/remove trackbar, autosize, etc.
          */
@@ -1121,7 +1120,7 @@ end_of_mci_open:
         {
             MCI_STATUS_PARMS mci_status;
 
-            TRACE("MCIWNDM_GETTIMEFORMAT %08x %08lx\n", wParam, lParam);
+            TRACE("MCIWNDM_GETTIMEFORMAT %08lx %08lx\n", wParam, lParam);
 
             /* get format string if requested */
             if (wParam && lParam)
@@ -1186,6 +1185,8 @@ end_of_mci_open:
                 if (!mwi->lasterror)
                     SendDlgItemMessageW(hWnd, CTL_TRACKBAR, TBM_SETRANGEMAX, 1,
                                         SendMessageW(hWnd, MCIWNDM_GETLENGTH, 0, 0));
+
+                HeapFree(GetProcessHeap(), 0, cmdW);
             }
 
             if (wMsg == MCIWNDM_SETTIMEFORMATA)
