@@ -127,6 +127,7 @@ static void test_xmldoc(void)
     hr = IXMLElement_get_tagName(element, &name);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
     ok(!lstrcmpW(name, szBankAccount), "Expected BANKACCOUNT\n");
+    SysFreeString(name);
 
     hr = IXMLElement_get_children(element, &collection);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
@@ -151,6 +152,7 @@ static void test_xmldoc(void)
     hr = IXMLElement_get_tagName(child, &name);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
     ok(!lstrcmpW(name, szNumber), "Expected NUMBER\n");
+    SysFreeString(name);
 
     hr = IXMLElement_get_children(child, &inner);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
@@ -171,6 +173,7 @@ static void test_xmldoc(void)
     hr = IXMLElement_get_text(value, &name);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
     ok(!lstrcmpW(name, szNumVal), "Expected '1234'\n");
+    SysFreeString(name);
 
     IXMLElementCollection_Release(inner);
 
@@ -196,6 +199,7 @@ static void test_xmldoc(void)
     hr = IXMLElement_get_tagName(child, &name);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
     ok(!lstrcmpW(name, szName), "Expected NAME\n");
+    SysFreeString(name);
 
     hr = IXMLElement_get_children(child, &inner);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
@@ -217,6 +221,7 @@ static void test_xmldoc(void)
     hr = IXMLElement_get_text(value, &name);
     ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
     ok(!lstrcmpW(name, szNameVal), "Expected 'Captain Ahab'\n");
+    SysFreeString(name);
 
     IXMLElementCollection_Release(inner);
 
@@ -265,16 +270,25 @@ static void test_createElement(void)
     V_I4(&vType) = -1;
     V_VT(&vName) = VT_NULL;
     hr = IXMLDocument_createElement(doc, vType, vName, &element);
-    ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
-    ok(element != NULL, "Expected non-NULL element\n");
-
-    if (element != NULL)
+    /* Up to and including SP7, createElement returns an element. */
+    if(hr == S_OK)
     {
-        hr = IXMLElement_get_type(element, &type);
-        ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
-        ok(type == XMLELEMTYPE_OTHER, "Expected XMLELEMTYPE_OTHER, got %ld\n", type);
+        ok(element != NULL, "Expected element\n");
+        if (element != NULL)
+        {
+            hr = IXMLElement_get_type(element, &type);
+            ok(hr == S_OK, "Expected S_OK, got %d\n", hr);
+            /* SP7 returns an XMLELEMTYPE_ELEMENT */
+            ok(type == XMLELEMTYPE_OTHER || type == XMLELEMTYPE_ELEMENT,
+                         "Expected XMLELEMTYPE_OTHER || XMLELEMTYPE_ELEMENT, got %ld\n", type);
 
-        IXMLElement_Release(element);
+            IXMLElement_Release(element);
+        }
+    }
+    else
+    {
+        ok(hr == E_NOTIMPL, "Expected E_NOTIMPL, got %d\n", hr);
+        ok(element == NULL, "Expected NULL element\n");
     }
 
     /* invalid vName type */
