@@ -39,9 +39,6 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-#ifdef HAVE_TERMIOS_H
-#include <termios.h>
-#endif
 #include <fcntl.h>
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
@@ -191,24 +188,29 @@ static NTSTATUS get_baud_rate(int fd, SERIAL_BAUD_RATE* sbr)
 
 static NTSTATUS get_hand_flow(int fd, SERIAL_HANDFLOW* shf)
 {
-    int stat;
+    int stat = 0;
     struct termios port;
-    
+
     if (tcgetattr(fd, &port) == -1)
     {
         ERR("tcgetattr error '%s'\n", strerror(errno));
         return FILE_GetNtStatus();
     }
+    /* termios does not support DTR/DSR flow control */
+    shf->ControlHandShake = 0;
+    shf->FlowReplace = 0;
 #ifdef TIOCMGET
     if (ioctl(fd, TIOCMGET, &stat) == -1)
     {
         WARN("ioctl error '%s'\n", strerror(errno));
-        stat = DTR_CONTROL_ENABLE | RTS_CONTROL_ENABLE;
+        shf->ControlHandShake |= SERIAL_DTR_CONTROL;
+        shf->FlowReplace |= SERIAL_RTS_CONTROL;
     }
+#else
+    WARN("Setting DTR/RTS to enabled by default\n");
+    shf->ControlHandShake |= SERIAL_DTR_CONTROL;
+    shf->FlowReplace |= SERIAL_RTS_CONTROL;
 #endif
-    /* termios does not support DTR/DSR flow control */
-    shf->ControlHandShake = 0;
-    shf->FlowReplace = 0;
 #ifdef TIOCM_DTR
     if (stat & TIOCM_DTR)
 #endif
@@ -216,7 +218,7 @@ static NTSTATUS get_hand_flow(int fd, SERIAL_HANDFLOW* shf)
 #ifdef CRTSCTS
     if (port.c_cflag & CRTSCTS)
     {
-        shf->ControlHandShake |= SERIAL_DTR_CONTROL | SERIAL_DTR_HANDSHAKE;
+        shf->FlowReplace |= SERIAL_RTS_CONTROL;
         shf->ControlHandShake |= SERIAL_CTS_HANDSHAKE;
     }
     else
@@ -225,11 +227,11 @@ static NTSTATUS get_hand_flow(int fd, SERIAL_HANDFLOW* shf)
 #ifdef TIOCM_RTS
         if (stat & TIOCM_RTS)
 #endif
-            shf->ControlHandShake |= SERIAL_RTS_CONTROL;
+            shf->FlowReplace |= SERIAL_RTS_CONTROL;
     }
-    if (port.c_iflag & IXON)
-        shf->FlowReplace |= SERIAL_AUTO_RECEIVE;
     if (port.c_iflag & IXOFF)
+        shf->FlowReplace |= SERIAL_AUTO_RECEIVE;
+    if (port.c_iflag & IXON)
         shf->FlowReplace |= SERIAL_AUTO_TRANSMIT;
 
     shf->XonLimit = 10;
@@ -259,7 +261,6 @@ static NTSTATUS get_line_control(int fd, SERIAL_LINE_CONTROL* slc)
 #ifdef CMSPAR
     case PARENB|CMSPAR:         slc->Parity = MARKPARITY;       break;
     case PARENB|PARODD|CMSPAR:  slc->Parity = SPACEPARITY;      break;
-        break;
 #endif
     }
     switch (port.c_cflag & CSIZE)
@@ -270,7 +271,7 @@ static NTSTATUS get_line_control(int fd, SERIAL_LINE_CONTROL* slc)
     case CS8:	slc->WordLength = 8;	break;
     default: ERR("unknown size %x\n", (UINT)(port.c_cflag & CSIZE));
     }
-    
+
     if (port.c_cflag & CSTOPB)
     {
         if (slc->WordLength == 5)
@@ -493,9 +494,10 @@ static NTSTATUS set_baud_rate(int fd, const SERIAL_BAUD_RATE* sbr)
             port.c_cflag |= B38400;
         }
         break;
-#endif    /* Don't have linux/serial.h or lack TIOCSSERIAL */
+#else     /* Don't have linux/serial.h or lack TIOCSSERIAL */
         ERR("baudrate %d\n", sbr->BaudRate);
         return STATUS_NOT_SUPPORTED;
+#endif    /* Don't have linux/serial.h or lack TIOCSSERIAL */
     }
 #elif !defined(__EMX__)
     switch (sbr->BaudRate)
@@ -595,13 +597,13 @@ static NTSTATUS set_handflow(int fd, const SERIAL_HANDFLOW* shf)
     if (shf->ControlHandShake & SERIAL_DTR_HANDSHAKE)
     {
         WARN("DSR/DTR flow control not supported\n");
-    } else if (shf->ControlHandShake & SERIAL_DTR_CONTROL)
+    } else if (!(shf->ControlHandShake & SERIAL_DTR_CONTROL))
         whack_modem(fd, ~TIOCM_DTR, 0);
-    else    
+    else
         whack_modem(fd, 0, TIOCM_DTR);
 #endif
 #ifdef TIOCM_RTS
-    if (!(shf->ControlHandShake & SERIAL_DSR_HANDSHAKE))
+    if (!(shf->ControlHandShake & SERIAL_CTS_HANDSHAKE))
     {
         if ((shf->FlowReplace & (SERIAL_RTS_CONTROL|SERIAL_RTS_HANDSHAKE)) == 0)
             whack_modem(fd, ~TIOCM_RTS, 0);
@@ -611,13 +613,13 @@ static NTSTATUS set_handflow(int fd, const SERIAL_HANDFLOW* shf)
 #endif
 
     if (shf->FlowReplace & SERIAL_AUTO_RECEIVE)
-        port.c_iflag |= IXON;
-    else
-        port.c_iflag &= ~IXON;
-    if (shf->FlowReplace & SERIAL_AUTO_TRANSMIT)
         port.c_iflag |= IXOFF;
     else
         port.c_iflag &= ~IXOFF;
+    if (shf->FlowReplace & SERIAL_AUTO_TRANSMIT)
+        port.c_iflag |= IXON;
+    else
+        port.c_iflag &= ~IXON;
     if (tcsetattr(fd, TCSANOW, &port) == -1)
     {
         ERR("tcsetattr error '%s'\n", strerror(errno));
@@ -911,9 +913,10 @@ static NTSTATUS get_irq_info(int fd, serial_irq_info *irq_info)
     }
     TRACE("TIOCGICOUNT err %s\n", strerror(errno));
     return FILE_GetNtStatus();
-#endif
+#else
     memset(irq_info,0, sizeof(serial_irq_info));
     return STATUS_NOT_IMPLEMENTED;
+#endif
 }
 
 
@@ -982,7 +985,7 @@ static DWORD CALLBACK wait_for_event(LPVOID arg)
     async_commio *commio = (async_commio*) arg;
     int fd, needs_close;
 
-    if (!server_get_unix_fd( commio->hDevice, FILE_READ_DATA | FILE_WRITE_DATA, &fd, &needs_close, NULL ))
+    if (!server_get_unix_fd( commio->hDevice, FILE_READ_DATA | FILE_WRITE_DATA, &fd, &needs_close, NULL, NULL ))
     {
         serial_irq_info new_irq_info;
         DWORD new_mstat, new_evtmask;
@@ -1097,7 +1100,7 @@ out_now:
     return status;
 }
 
-static NTSTATUS xmit_immediate(HANDLE hDevice, int fd, char* ptr)
+static NTSTATUS xmit_immediate(HANDLE hDevice, int fd, const char* ptr)
 {
     /* FIXME: not perfect as it should bypass the in-queue */
     WARN("(%p,'%c') not perfect!\n", hDevice, *ptr);
@@ -1130,7 +1133,7 @@ static inline NTSTATUS io_control(HANDLE hDevice,
     piosb->Information = 0;
 
     if (dwIoControlCode != IOCTL_SERIAL_GET_TIMEOUTS)
-        if ((status = server_get_unix_fd( hDevice, access, &fd, &needs_close, NULL )))
+        if ((status = server_get_unix_fd( hDevice, access, &fd, &needs_close, NULL, NULL )))
             goto error;
 
     switch (dwIoControlCode)
@@ -1362,7 +1365,7 @@ NTSTATUS COMM_DeviceIoControl(HANDLE hDevice,
 
         /* this is an ioctl we implement in a non blocking way if hEvent is not
          * null
-         * so we have to explicitely wait if no hEvent is provided
+         * so we have to explicitly wait if no hEvent is provided
          */
         if (!hev)
         {
