@@ -174,7 +174,7 @@ static const struct exception
 static int got_exception;
 static BOOL have_vectored_api;
 
-static void run_exception_test(const void *handler, const void* context,
+static void run_exception_test(void *handler, const void* context,
                                const void *code, unsigned int code_size)
 {
     struct {
@@ -215,10 +215,8 @@ LONG CALLBACK rtlraiseexception_vectored_handler(EXCEPTION_POINTERS *ExceptionIn
      */
     if(rec->ExceptionCode == EXCEPTION_BREAKPOINT)
     {
-        todo_wine {
         ok(context->Eip == (DWORD)code_mem + 0xa, "Eip at %x instead of %x\n",
            context->Eip, (DWORD)code_mem + 0xa);
-        }
     }
     else
     {
@@ -247,10 +245,8 @@ static DWORD rtlraiseexception_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTR
      */
     if(rec->ExceptionCode == EXCEPTION_BREAKPOINT)
     {
-        todo_wine {
         ok(context->Eip == (DWORD)code_mem + 0xa, "Eip at %x instead of %x\n",
            context->Eip, (DWORD)code_mem + 0xa);
-        }
     }
     else
     {
@@ -473,6 +469,30 @@ static DWORD align_check_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_
     return ExceptionContinueExecution;
 }
 
+/* Test the direction flag handling. */
+static const BYTE direction_flag_code[] = {
+    0x55,                  	/* push   %ebp */
+    0x89,0xe5,             	/* mov    %esp,%ebp */
+    0xfd,                  	/* std */
+    0xfa,                  	/* cli - cause exception */
+    0x5d,                  	/* pop    %ebp */
+    0xc3,                  	/* ret     */
+};
+
+static DWORD direction_flag_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *frame,
+                                     CONTEXT *context, EXCEPTION_REGISTRATION_RECORD **dispatcher )
+{
+#ifdef __GNUC__
+    unsigned int flags;
+    __asm__("pushfl; popl %0" : "=r" (flags) );
+    ok( !(flags & 0x400), "eflags has DF bit set\n" );
+#endif
+    ok( context->EFlags & 0x400, "context eflags has DF bit cleared\n" );
+    got_exception++;
+    context->Eip++;  /* skip cli */
+    return ExceptionContinueExecution;
+}
+
 /* test single stepping over hardware breakpoint */
 static DWORD bpx_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *frame,
                           CONTEXT *context, EXCEPTION_REGISTRATION_RECORD **dispatcher )
@@ -493,14 +513,15 @@ static DWORD bpx_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *
         /* single step exception on second nop */
         ok( context->Eip == (DWORD)code_mem + 1, "eip is wrong: %x instead of %x\n",
                                                  context->Eip, (DWORD)code_mem + 1);
-        todo_wine{ ok( (context->Dr6 & 0x4000), "BS flag is not set in Dr6\n"); };
-        ok( (context->Dr6 & 0xf) == 0, "B0...3 flags in Dr6 shouldn't be set\n");
+        ok( (context->Dr6 & 0x4000), "BS flag is not set in Dr6\n");
+       /* depending on the win version the B0 bit is already set here as well
+        ok( (context->Dr6 & 0xf) == 0, "B0...3 flags in Dr6 shouldn't be set\n"); */
         context->EFlags |= 0x100;
     } else if( got_exception == 3) {
         /* hw bp exception on second nop */
         ok( context->Eip == (DWORD)code_mem + 1, "eip is wrong: %x instead of %x\n",
                                                  context->Eip, (DWORD)code_mem + 1);
-        todo_wine{ ok( (context->Dr6 & 0xf) == 1, "B0 flag is not set in Dr6\n"); };
+        ok( (context->Dr6 & 0xf) == 1, "B0 flag is not set in Dr6\n");
         ok( !(context->Dr6 & 0x4000), "BS flag is set in Dr6\n");
         context->Dr0 = 0;       /* clear breakpoint */
         context->EFlags |= 0x100;
@@ -509,7 +530,7 @@ static DWORD bpx_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD *
         ok( context->Eip == (DWORD)code_mem + 2, "eip is wrong: %x instead of %x\n",
                                                  context->Eip, (DWORD)code_mem + 2);
         ok( (context->Dr6 & 0xf) == 0, "B0...3 flags in Dr6 shouldn't be set\n");
-        todo_wine{ ok( (context->Dr6 & 0x4000), "BS flag is not set in Dr6\n"); };
+        ok( (context->Dr6 & 0x4000), "BS flag is not set in Dr6\n");
     }
 
     context->Dr6 = 0;  /* clear status register */
@@ -524,9 +545,7 @@ static DWORD int3_handler( EXCEPTION_RECORD *rec, EXCEPTION_REGISTRATION_RECORD
 {
     ok( rec->ExceptionAddress == code_mem, "exception address not at: %p, but at %p\n",
                                            code_mem,  rec->ExceptionAddress);
-    todo_wine {
-        ok( context->Eip == (DWORD)code_mem, "eip not at: %p, but at %#x\n", code_mem, context->Eip);
-    }
+    ok( context->Eip == (DWORD)code_mem, "eip not at: %p, but at %#x\n", code_mem, context->Eip);
     if(context->Eip == (DWORD)code_mem) context->Eip++; /* skip breakpoint */
 
     return ExceptionContinueExecution;
@@ -542,7 +561,7 @@ static void test_exceptions(void)
 
     if (!pNtGetContextThread || !pNtSetContextThread)
     {
-        trace( "NtGetContextThread/NtSetContextThread not found, skipping tests\n" );
+        skip( "NtGetContextThread/NtSetContextThread not found\n" );
         return;
     }
 
@@ -565,6 +584,11 @@ static void test_exceptions(void)
     run_exception_test(align_check_handler, NULL, align_check_code, sizeof(align_check_code));
     ok(got_exception == 0, "got %d alignment faults, expected 0\n", got_exception);
 
+    /* test direction flag */
+    got_exception = 0;
+    run_exception_test(direction_flag_handler, NULL, direction_flag_code, sizeof(direction_flag_code));
+    ok(got_exception == 1, "got %d exceptions, expected 1\n", got_exception);
+
     /* test single stepping over hardware breakpoint */
     memset(&ctx, 0, sizeof(ctx));
     ctx.Dr0 = (DWORD) code_mem;  /* set hw bp on first nop */
@@ -601,7 +625,7 @@ static void test_debugger(void)
         return;
     }
 
-    sprintf(cmdline, "%s %s %s", my_argv[0], my_argv[1], "debuggee");
+    sprintf(cmdline, "%s %s %s %p", my_argv[0], my_argv[1], "debuggee", &test_stage);
     ret = CreateProcess(NULL, cmdline, NULL, NULL, FALSE, DEBUG_PROCESS, NULL, NULL, &si, &pi);
     ok(ret, "could not create child process error: %u\n", GetLastError());
     if (!ret)
@@ -624,7 +648,7 @@ static void test_debugger(void)
             if(de.u.CreateProcessInfo.lpBaseOfImage != pNtCurrentTeb()->Peb->ImageBaseAddress)
             {
                 skip("child process loaded at different address, terminating it\n");
-                pNtTerminateProcess(pi.hProcess, 1);
+                pNtTerminateProcess(pi.hProcess, 0);
             }
         }
         else if (de.dwDebugEventCode == EXCEPTION_DEBUG_EVENT)
@@ -691,10 +715,8 @@ static void test_debugger(void)
                         /* ctx.Eip is the same value the exception handler got */
                         if (de.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT)
                         {
-                            todo_wine{
                             ok((char *)ctx.Eip == (char *)code_mem_address + 0xa, "Eip at 0x%x instead of %p\n",
                                 ctx.Eip, (char *)code_mem_address + 0xa);
-                            }
                             /* need to fixup Eip for debuggee */
                             if ((char *)ctx.Eip == (char *)code_mem_address + 0xa)
                                 ctx.Eip += 1;
@@ -706,7 +728,7 @@ static void test_debugger(void)
                     }
                 }
                 else
-                    ok(FALSE, "unexpected stage %d\n", stage);
+                    ok(FALSE, "unexpected stage %x\n", stage);
 
                 status = pNtSetContextThread(pi.hThread, &ctx);
                 ok(!status, "NtSetContextThread failed with 0x%x\n", status);
@@ -717,6 +739,7 @@ static void test_debugger(void)
 
     } while (de.dwDebugEventCode != EXIT_PROCESS_DEBUG_EVENT);
 
+    winetest_wait_child_process( pi.hProcess );
     ok(CloseHandle(pi.hThread) != 0, "error %u\n", GetLastError());
     ok(CloseHandle(pi.hProcess) != 0, "error %u\n", GetLastError());
 
@@ -804,19 +827,21 @@ static void test_simd_exceptions(void)
 START_TEST(exception)
 {
 #ifdef __i386__
-    pNtCurrentTeb = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"), "NtCurrentTeb" );
-    pNtGetContextThread  = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"), "NtGetContextThread" );
-    pNtSetContextThread  = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"), "NtSetContextThread" );
-    pNtReadVirtualMemory = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"), "NtReadVirtualMemory" );
-    pRtlRaiseException   = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"), "RtlRaiseException" );
-    pNtTerminateProcess  = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"), "NtTerminateProcess" );
-    pRtlAddVectoredExceptionHandler    = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"),
+    HMODULE hntdll = GetModuleHandleA("ntdll.dll");
+
+    pNtCurrentTeb        = (void *)GetProcAddress( hntdll, "NtCurrentTeb" );
+    pNtGetContextThread  = (void *)GetProcAddress( hntdll, "NtGetContextThread" );
+    pNtSetContextThread  = (void *)GetProcAddress( hntdll, "NtSetContextThread" );
+    pNtReadVirtualMemory = (void *)GetProcAddress( hntdll, "NtReadVirtualMemory" );
+    pRtlRaiseException   = (void *)GetProcAddress( hntdll, "RtlRaiseException" );
+    pNtTerminateProcess  = (void *)GetProcAddress( hntdll, "NtTerminateProcess" );
+    pRtlAddVectoredExceptionHandler    = (void *)GetProcAddress( hntdll,
                                                                  "RtlAddVectoredExceptionHandler" );
-    pRtlRemoveVectoredExceptionHandler = (void *)GetProcAddress( GetModuleHandleA("ntdll.dll"),
+    pRtlRemoveVectoredExceptionHandler = (void *)GetProcAddress( hntdll,
                                                                  "RtlRemoveVectoredExceptionHandler" );
     if (!pNtCurrentTeb)
     {
-        trace( "NtCurrentTeb not found, skipping tests\n" );
+        skip( "NtCurrentTeb not found\n" );
         return;
     }
 
@@ -833,8 +858,17 @@ START_TEST(exception)
     }
 
     my_argc = winetest_get_mainargs( &my_argv );
-    if (my_argc >= 3)
+    if (my_argc >= 4)
     {
+        void *addr;
+        sscanf( my_argv[3], "%p", &addr );
+
+        if (addr != &test_stage)
+        {
+            skip( "child process not mapped at same address (%p/%p)\n", &test_stage, addr);
+            return;
+        }
+
         /* child must be run under a debugger */
         if (!pNtCurrentTeb()->Peb->BeingDebugged)
         {
