@@ -1273,7 +1273,7 @@ static HRESULT WINAPI OLEClipbrd_IDataObject_GetData(
    * Return the clipboard data in the storage medium structure
    */
   pmedium->tymed = (hData == 0) ? TYMED_NULL : TYMED_HGLOBAL;
-  pmedium->u.hGlobal = (HGLOBAL)hData;
+  pmedium->u.hGlobal = hData;
   pmedium->pUnkForRelease = NULL;
 
   hr = S_OK;
@@ -1365,7 +1365,7 @@ static HRESULT WINAPI OLEClipbrd_IDataObject_GetCanonicalFormatEtc(
   if ( !pformatectIn || !pformatetcOut )
     return E_INVALIDARG;
 
-  memcpy(pformatetcOut, pformatectIn, sizeof(FORMATETC));
+  *pformatetcOut = *pformatectIn;
   return DATA_S_SAMEFORMATETC;
 }
 
