@@ -28,12 +28,14 @@ DEFINE_OLEGUID( CLSID_ItemMoniker,      0x304, 0, 0 );
 DEFINE_OLEGUID( CLSID_AntiMoniker,      0x305, 0, 0 );
 DEFINE_OLEGUID( CLSID_CompositeMoniker, 0x309, 0, 0 );
 DEFINE_OLEGUID( CLSID_ClassMoniker,     0x31a, 0, 0 );
+DEFINE_OLEGUID( CLSID_PointerMoniker,   0x306, 0, 0 );
 
 HRESULT FileMonikerCF_Create(REFIID riid, LPVOID *ppv);
 HRESULT ItemMonikerCF_Create(REFIID riid, LPVOID *ppv);
 HRESULT AntiMonikerCF_Create(REFIID riid, LPVOID *ppv);
 HRESULT CompositeMonikerCF_Create(REFIID riid, LPVOID *ppv);
 HRESULT ClassMonikerCF_Create(REFIID riid, LPVOID *ppv);
+HRESULT PointerMonikerCF_Create(REFIID riid, LPVOID *ppv);
 
 /* This function decomposes a String path to a String Table containing all the elements ("\" or "subDirectory" or "Directory" or "FileName") of the path */
 int FileMonikerImpl_DecomposePath(LPCOLESTR str, LPOLESTR** stringTable);
