@@ -111,12 +111,6 @@ static const ULONG PROPERTY_NULL             = 0xFFFFFFFF;
      STGM_TRANSACTED | STGM_CONVERT | STGM_PRIORITY | STGM_NOSCRATCH | \
      STGM_NOSNAPSHOT | STGM_DIRECT_SWMR | STGM_DELETEONRELEASE | STGM_SIMPLE)
 
-/*
- * These are signatures to detect the type of Document file.
- */
-static const BYTE STORAGE_magic[8]    ={0xd0,0xcf,0x11,0xe0,0xa1,0xb1,0x1a,0xe1};
-static const BYTE STORAGE_oldmagic[8] ={0xd0,0xcf,0x11,0xe0,0x0e,0x11,0xfc,0x0d};
-
 /*
  * Forward declarations of all the structures used by the storage
  * module.
@@ -191,11 +185,11 @@ BigBlockFile*  BIGBLOCKFILE_Construct(HANDLE hFile,
                                       BOOL fileBased);
 void           BIGBLOCKFILE_Destructor(LPBIGBLOCKFILE This);
 void           BIGBLOCKFILE_EnsureExists(LPBIGBLOCKFILE This, ULONG index);
-void*          BIGBLOCKFILE_GetBigBlock(LPBIGBLOCKFILE This, ULONG index);
-void*          BIGBLOCKFILE_GetROBigBlock(LPBIGBLOCKFILE This, ULONG index);
-void           BIGBLOCKFILE_ReleaseBigBlock(LPBIGBLOCKFILE This, void *pBlock);
 void           BIGBLOCKFILE_SetSize(LPBIGBLOCKFILE This, ULARGE_INTEGER newSize);
-ULARGE_INTEGER BIGBLOCKFILE_GetSize(LPBIGBLOCKFILE This);
+HRESULT        BIGBLOCKFILE_ReadAt(LPBIGBLOCKFILE This, ULARGE_INTEGER offset,
+           void* buffer, ULONG size, ULONG* bytesRead);
+HRESULT        BIGBLOCKFILE_WriteAt(LPBIGBLOCKFILE This, ULARGE_INTEGER offset,
+           const void* buffer, ULONG size, ULONG* bytesRead);
 
 /*************************************************************************
  * Ole Convert support
@@ -251,6 +245,11 @@ struct StorageBaseImpl
    * flags that this storage was opened or created with
    */
   DWORD openFlags;
+
+  /*
+   * State bits appear to only be preserved while running. No in the stream
+   */
+  DWORD stateBits;
 };
 
 /****************************************************************************
@@ -312,14 +311,14 @@ struct StorageImpl
 };
 
 BOOL StorageImpl_ReadProperty(
-            StorageImpl* This,
-	    ULONG          index,
-	    StgProperty*    buffer);
+            StorageImpl*    This,
+            ULONG           index,
+            StgProperty*    buffer);
 
 BOOL StorageImpl_WriteProperty(
-            StorageImpl* This,
-	    ULONG          index,
-	    StgProperty*   buffer);
+            StorageImpl*        This,
+            ULONG               index,
+            const StgProperty*  buffer);
 
 BlockChainStream* Storage32Impl_SmallBlocksToBigBlocks(
                       StorageImpl* This,
@@ -424,9 +423,8 @@ void StorageUtl_WriteULargeInteger(BYTE* buffer, ULONG offset,
  const ULARGE_INTEGER *value);
 void StorageUtl_ReadGUID(const BYTE* buffer, ULONG offset, GUID* value);
 void StorageUtl_WriteGUID(BYTE* buffer, ULONG offset, const GUID* value);
-void StorageUtl_CopyPropertyToSTATSTG(STATSTG*     destination,
-					     StgProperty* source,
-					     int          statFlags);
+void StorageUtl_CopyPropertyToSTATSTG(STATSTG* destination, const StgProperty* source,
+ int statFlags);
 
 /****************************************************************************
  * BlockChainStream definitions.
