@@ -29,6 +29,7 @@
 #include "winbase.h"
 #include "objbase.h"
 #include "shlguid.h"
+#include "shobjidl.h"
 
 #include "wine/test.h"
 
@@ -48,6 +49,14 @@ static const IID IID_IWineTest =
     {0xa1, 0xa2, 0x5d, 0x5a, 0x36, 0x54, 0xd3, 0xbd}
 }; /* 5201163f-8164-4fd0-a1a2-5d5a3654d3bd */
 
+static const IID IID_IRemUnknown =
+{
+    0x00000131,
+    0x0000,
+    0x0000,
+    {0xc0,0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46}
+};
+
 #define EXTENTID_WineTest IID_IWineTest
 #define CLSID_WineTest IID_IWineTest
 
@@ -70,7 +79,6 @@ static void test_cocreateinstance_proxy(void)
     hr = CoCreateInstance(&CLSID_ShellDesktop, NULL, CLSCTX_INPROC, &IID_IUnknown, (void **)&pProxy);
     ok_ole_success(hr, CoCreateInstance);
     hr = IUnknown_QueryInterface(pProxy, &IID_IMultiQI, (void **)&pMQI);
-    todo_wine
     ok(hr == S_OK, "created object is not a proxy, so was created in the wrong apartment\n");
     if (hr == S_OK)
         IMultiQI_Release(pMQI);
@@ -141,7 +149,9 @@ static HRESULT WINAPI Test_IClassFactory_QueryInterface(
     if (ppvObj == NULL) return E_POINTER;
 
     if (IsEqualGUID(riid, &IID_IUnknown) ||
-        IsEqualGUID(riid, &IID_IClassFactory))
+        IsEqualGUID(riid, &IID_IClassFactory) ||
+        /* the only other interface Wine is currently able to marshal (for testing two proxies) */
+        IsEqualGUID(riid, &IID_IRemUnknown))
     {
         *ppvObj = (LPVOID)iface;
         IClassFactory_AddRef(iface);
@@ -232,7 +242,6 @@ static DWORD CALLBACK host_object_proc(LPVOID p)
     {
         if (msg.hwnd == NULL && msg.message == RELEASEMARSHALDATA)
         {
-            trace("releasing marshal data\n");
             CoReleaseMarshalData(data->stream);
             SetEvent((HANDLE)msg.lParam);
         }
@@ -426,6 +435,8 @@ static void test_same_apartment_unmarshal_failure(void)
     ok(hr == E_NOINTERFACE, "CoUnmarshalInterface should have returned E_NOINTERFACE instead of 0x%08x\n", hr);
 
     ok_no_locks();
+
+    IStream_Release(pStream);
 }
 
 /* tests success case of an interthread marshal */
@@ -514,14 +525,6 @@ static void test_proxy_marshal_and_unmarshal(void)
 
     ok_more_than_one_lock();
 
-    /* now the proxies should be as follows:
-     *  pProxy2 -> &Test_ClassFactory
-     * they should NOT be as follows:
-     *  pProxy -> &Test_ClassFactory
-     *  pProxy2 -> pProxy
-     * the above can only really be tested by looking in +ole traces
-     */
-
     IUnknown_Release(pProxy2);
 
     /* unmarshal all of the proxies to check that the object stub still exists */
@@ -600,6 +603,112 @@ static void test_proxy_marshal_and_unmarshal2(void)
     end_host_object(tid, thread);
 }
 
+/* tests success case of an interthread marshal and then table-weak-marshaling the proxy */
+static void test_proxy_marshal_and_unmarshal_weak(void)
+{
+    HRESULT hr;
+    IStream *pStream = NULL;
+    IUnknown *pProxy = NULL;
+    IUnknown *pProxy2 = NULL;
+    DWORD tid;
+    HANDLE thread;
+
+    cLocks = 0;
+
+    hr = CreateStreamOnHGlobal(NULL, TRUE, &pStream);
+    ok_ole_success(hr, CreateStreamOnHGlobal);
+    tid = start_host_object(pStream, &IID_IClassFactory, (IUnknown*)&Test_ClassFactory, MSHLFLAGS_NORMAL, &thread);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy);
+    ok_ole_success(hr, CoUnmarshalInterface);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    /* marshal the proxy */
+    hr = CoMarshalInterface(pStream, &IID_IClassFactory, pProxy, MSHCTX_INPROC, NULL, MSHLFLAGS_TABLEWEAK);
+    ok_ole_success(hr, CoMarshalInterface);
+
+    ok_more_than_one_lock();
+
+    /* release the original proxy to test that we successfully keep the
+     * original object alive */
+    IUnknown_Release(pProxy);
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy2);
+    todo_wine
+    ok(hr == CO_E_OBJNOTREG, "CoUnmarshalInterface should return CO_E_OBJNOTREG instead of 0x%08x\n", hr);
+
+    ok_no_locks();
+
+    IStream_Release(pStream);
+
+    end_host_object(tid, thread);
+}
+
+/* tests success case of an interthread marshal and then table-strong-marshaling the proxy */
+static void test_proxy_marshal_and_unmarshal_strong(void)
+{
+    HRESULT hr;
+    IStream *pStream = NULL;
+    IUnknown *pProxy = NULL;
+    IUnknown *pProxy2 = NULL;
+    DWORD tid;
+    HANDLE thread;
+
+    cLocks = 0;
+
+    hr = CreateStreamOnHGlobal(NULL, TRUE, &pStream);
+    ok_ole_success(hr, CreateStreamOnHGlobal);
+    tid = start_host_object(pStream, &IID_IClassFactory, (IUnknown*)&Test_ClassFactory, MSHLFLAGS_NORMAL, &thread);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy);
+    ok_ole_success(hr, CoUnmarshalInterface);
+
+    ok_more_than_one_lock();
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    /* marshal the proxy */
+    hr = CoMarshalInterface(pStream, &IID_IClassFactory, pProxy, MSHCTX_INPROC, NULL, MSHLFLAGS_TABLESTRONG);
+    ok(hr == S_OK /* WinNT */ || hr == E_INVALIDARG /* Win9x */,
+        "CoMarshalInterface should have return S_OK or E_INVALIDARG instead of 0x%08x\n", hr);
+    if (FAILED(hr))
+    {
+        IUnknown_Release(pProxy);
+        goto end;
+    }
+
+    ok_more_than_one_lock();
+
+    /* release the original proxy to test that we successfully keep the
+     * original object alive */
+    IUnknown_Release(pProxy);
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy2);
+    ok_ole_success(hr, CoUnmarshalInterface);
+
+    ok_more_than_one_lock();
+
+    IUnknown_Release(pProxy2);
+
+    ok_more_than_one_lock();
+
+end:
+    IStream_Release(pStream);
+
+    end_host_object(tid, thread);
+
+    ok_no_locks();
+}
+
 /* tests that stubs are released when the containing apartment is destroyed */
 static void test_marshal_stub_apartment_shutdown(void)
 {
@@ -1168,6 +1277,24 @@ static DWORD CALLBACK bad_thread_proc(LPVOID p)
     HRESULT hr;
     IUnknown * proxy = NULL;
 
+    hr = IClassFactory_CreateInstance(cf, NULL, &IID_IUnknown, (LPVOID*)&proxy);
+    todo_wine
+    ok(hr == CO_E_NOTINITIALIZED,
+       "COM should have failed with CO_E_NOTINITIALIZED on using proxy without apartment, but instead returned 0x%08x\n",
+       hr);
+
+    hr = IClassFactory_QueryInterface(cf, &IID_IMultiQI, (LPVOID *)&proxy);
+    /* Win9x returns S_OK, whilst NT returns RPC_E_WRONG_THREAD */
+    trace("call to proxy's QueryInterface for local interface without apartment returned 0x%08x\n", hr);
+    if (SUCCEEDED(hr))
+        IUnknown_Release(proxy);
+
+    hr = IClassFactory_QueryInterface(cf, &IID_IStream, (LPVOID *)&proxy);
+    /* Win9x returns E_NOINTERFACE, whilst NT returns RPC_E_WRONG_THREAD */
+    trace("call to proxy's QueryInterface without apartment returned 0x%08x\n", hr);
+    if (SUCCEEDED(hr))
+        IUnknown_Release(proxy);
+
     pCoInitializeEx(NULL, COINIT_MULTITHREADED);
 
     hr = IClassFactory_CreateInstance(cf, NULL, &IID_IUnknown, (LPVOID*)&proxy);
@@ -1176,6 +1303,19 @@ static DWORD CALLBACK bad_thread_proc(LPVOID p)
         "COM should have failed with RPC_E_WRONG_THREAD on using proxy from wrong apartment, but instead returned 0x%08x\n",
         hr);
 
+    hr = IClassFactory_QueryInterface(cf, &IID_IStream, (LPVOID *)&proxy);
+    /* Win9x returns E_NOINTERFACE, whilst NT returns RPC_E_WRONG_THREAD */
+    trace("call to proxy's QueryInterface from wrong apartment returned 0x%08x\n", hr);
+
+    /* this statement causes Win9x DCOM to crash during CoUninitialize of
+     * other apartment, so don't test this on Win9x (signified by NT-only
+     * export of CoRegisterSurrogateEx) */
+    if (GetProcAddress(GetModuleHandle("ole32"), "CoRegisterSurrogateEx"))
+        /* now be really bad and release the proxy from the wrong apartment */
+        IUnknown_Release(cf);
+    else
+        skip("skipping test for releasing proxy from wrong apartment that will succeed, but cause a crash during CoUninitialize\n");
+
     CoUninitialize();
 
     return 0;
@@ -1206,13 +1346,18 @@ static void test_proxy_used_in_wrong_thread(void)
 
     ok_more_than_one_lock();
 
+    /* do a call that will fail, but result in IRemUnknown being used by the proxy */
+    IClassFactory_QueryInterface(pProxy, &IID_IStream, (LPVOID *)&pStream);
+
     /* create a thread that we can misbehave in */
     thread = CreateThread(NULL, 0, bad_thread_proc, (LPVOID)pProxy, 0, &tid2);
 
     WaitForSingleObject(thread, INFINITE);
     CloseHandle(thread);
 
-    IUnknown_Release(pProxy);
+    /* do release statement on Win9x that we should have done above */
+    if (!GetProcAddress(GetModuleHandle("ole32"), "CoRegisterSurrogateEx"))
+        IUnknown_Release(pProxy);
 
     ok_no_locks();
 
@@ -1405,7 +1550,7 @@ static void test_proxy_interfaces(void)
     if (hr == S_OK) IUnknown_Release(pOtherUnknown);
 
     hr = IUnknown_QueryInterface(pProxy, &IID_IClientSecurity, (LPVOID*)&pOtherUnknown);
-    todo_wine { ok_ole_success(hr, IUnknown_QueryInterface IID_IClientSecurity); }
+    ok_ole_success(hr, IUnknown_QueryInterface IID_IClientSecurity);
     if (hr == S_OK) IUnknown_Release(pOtherUnknown);
 
     hr = IUnknown_QueryInterface(pProxy, &IID_IMultiQI, (LPVOID*)&pOtherUnknown);
@@ -1473,7 +1618,7 @@ static void test_proxybuffer(REFIID riid)
     LPVOID lpvtbl;
     ULONG refs;
     CLSID clsid;
-    HeapUnknown *pUnkOuter = (HeapUnknown *)HeapAlloc(GetProcessHeap(), 0, sizeof(*pUnkOuter));
+    HeapUnknown *pUnkOuter = HeapAlloc(GetProcessHeap(), 0, sizeof(*pUnkOuter));
 
     pUnkOuter->lpVtbl = &HeapUnknown_Vtbl;
     pUnkOuter->refs = 1;
@@ -1687,15 +1832,19 @@ static LRESULT CALLBACK window_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM l
     }
 }
 
-static void test_message_reentrancy(void)
+static void register_test_window(void)
 {
     WNDCLASS wndclass;
-    MSG msg;
 
     memset(&wndclass, 0, sizeof(wndclass));
     wndclass.lpfnWndProc = window_proc;
     wndclass.lpszClassName = "WineCOMTest";
     RegisterClass(&wndclass);
+}
+
+static void test_message_reentrancy(void)
+{
+    MSG msg;
 
     hwnd_app = CreateWindow("WineCOMTest", NULL, 0, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, NULL, 0);
     ok(hwnd_app != NULL, "Window creation failed\n");
@@ -1708,6 +1857,7 @@ static void test_message_reentrancy(void)
         TranslateMessage(&msg);
         DispatchMessage(&msg);
     }
+    DestroyWindow(hwnd_app);
 }
 
 static HRESULT WINAPI TestMsg_IClassFactory_CreateInstance(
@@ -1773,6 +1923,7 @@ static void test_call_from_message(void)
         TranslateMessage(&msg);
         DispatchMessage(&msg);
     }
+    DestroyWindow(hwnd_app);
 }
 
 static void test_WM_QUIT_handling(void)
@@ -2110,6 +2261,94 @@ static void test_handler_marshaling(void)
 }
 
 
+static void test_client_security(void)
+{
+    HRESULT hr;
+    IStream *pStream = NULL;
+    IClassFactory *pProxy = NULL;
+    IUnknown *pProxy2 = NULL;
+    IUnknown *pUnknown1 = NULL;
+    IUnknown *pUnknown2 = NULL;
+    IClientSecurity *pCliSec = NULL;
+    IMarshal *pMarshal;
+    DWORD tid;
+    HANDLE thread;
+    static const LARGE_INTEGER ullZero;
+    DWORD dwAuthnSvc;
+    DWORD dwAuthzSvc;
+    OLECHAR *pServerPrincName;
+    DWORD dwAuthnLevel;
+    DWORD dwImpLevel;
+    void *pAuthInfo;
+    DWORD dwCapabilities;
+    void *pv;
+
+    cLocks = 0;
+
+    hr = CreateStreamOnHGlobal(NULL, TRUE, &pStream);
+    ok_ole_success(hr, "CreateStreamOnHGlobal");
+    tid = start_host_object(pStream, &IID_IClassFactory, (IUnknown*)&Test_ClassFactory, MSHLFLAGS_NORMAL, &thread);
+
+    IStream_Seek(pStream, ullZero, STREAM_SEEK_SET, NULL);
+    hr = CoUnmarshalInterface(pStream, &IID_IClassFactory, (void **)&pProxy);
+    ok_ole_success(hr, "CoUnmarshalInterface");
+    IStream_Release(pStream);
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IUnknown, (LPVOID*)&pUnknown1);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IUnknown");
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IRemUnknown, (LPVOID*)&pProxy2);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IStream");
+
+    hr = IUnknown_QueryInterface(pProxy2, &IID_IUnknown, (LPVOID*)&pUnknown2);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IUnknown");
+
+    ok(pUnknown1 == pUnknown2, "both proxy's IUnknowns should be the same - %p, %p\n", pUnknown1, pUnknown2);
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IMarshal, (LPVOID*)&pMarshal);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IMarshal");
+
+    hr = IUnknown_QueryInterface(pProxy, &IID_IClientSecurity, (LPVOID*)&pCliSec);
+    ok_ole_success(hr, "IUnknown_QueryInterface IID_IClientSecurity");
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pProxy, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    todo_wine ok_ole_success(hr, "IClientSecurity_QueryBlanket (all NULLs)");
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pMarshal, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    todo_wine ok(hr == E_NOINTERFACE, "IClientSecurity_QueryBlanket with local interface should have returned E_NOINTERFACE instead of 0x%08x\n", hr);
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pProxy, &dwAuthnSvc, &dwAuthzSvc, &pServerPrincName, &dwAuthnLevel, &dwImpLevel, &pAuthInfo, &dwCapabilities);
+    todo_wine ok_ole_success(hr, "IClientSecurity_QueryBlanket");
+
+    hr = IClientSecurity_SetBlanket(pCliSec, (IUnknown *)pProxy, dwAuthnSvc, dwAuthzSvc, pServerPrincName, dwAuthnLevel, RPC_C_IMP_LEVEL_IMPERSONATE, pAuthInfo, dwCapabilities);
+    todo_wine ok_ole_success(hr, "IClientSecurity_SetBlanket");
+
+    hr = IClassFactory_CreateInstance(pProxy, NULL, &IID_IWineTest, &pv);
+    ok(hr == E_NOINTERFACE, "COM call should have succeeded instead of returning 0x%08x\n", hr);
+
+    hr = IClientSecurity_SetBlanket(pCliSec, (IUnknown *)pMarshal, dwAuthnSvc, dwAuthzSvc, pServerPrincName, dwAuthnLevel, dwImpLevel, pAuthInfo, dwCapabilities);
+    todo_wine ok(hr == E_NOINTERFACE, "IClientSecurity_SetBlanket with local interface should have returned E_NOINTERFACE instead of 0x%08x\n", hr);
+
+    hr = IClientSecurity_SetBlanket(pCliSec, (IUnknown *)pProxy, 0xdeadbeef, dwAuthzSvc, pServerPrincName, dwAuthnLevel, dwImpLevel, pAuthInfo, dwCapabilities);
+    todo_wine ok(hr == E_INVALIDARG, "IClientSecurity_SetBlanke with invalid dwAuthnSvc should have returned E_INVALIDARG instead of 0x%08x\n", hr);
+
+    CoTaskMemFree(pServerPrincName);
+
+    hr = IClientSecurity_QueryBlanket(pCliSec, (IUnknown *)pUnknown1, &dwAuthnSvc, &dwAuthzSvc, &pServerPrincName, &dwAuthnLevel, &dwImpLevel, &pAuthInfo, &dwCapabilities);
+    todo_wine ok_ole_success(hr, "IClientSecurity_QueryBlanket(IUnknown)");
+
+    CoTaskMemFree(pServerPrincName);
+
+    IClassFactory_Release(pProxy);
+    IUnknown_Release(pProxy2);
+    IUnknown_Release(pUnknown1);
+    IUnknown_Release(pUnknown2);
+    IMarshal_Release(pMarshal);
+    IClientSecurity_Release(pCliSec);
+
+    end_host_object(tid, thread);
+}
+
 static HANDLE heventShutdown;
 
 static void LockModuleOOP(void)
@@ -2239,11 +2478,13 @@ static HANDLE create_target_process(const char *arg)
     STARTUPINFO si = { 0 };
     si.cb = sizeof(si);
 
+    pi.hThread = NULL;
+    pi.hProcess = NULL;
     winetest_get_mainargs( &argv );
     sprintf(cmdline, "%s %s %s", argv[0], argv[1], arg);
     ok(CreateProcess(argv[0], cmdline, NULL, NULL, FALSE, 0, NULL, NULL,
-                     &si, &pi) != 0, "error: %u\n", GetLastError());
-    ok(CloseHandle(pi.hThread) != 0, "error %u\n", GetLastError());
+                     &si, &pi) != 0, "CreateProcess failed with error: %u\n", GetLastError());
+    if (pi.hThread) CloseHandle(pi.hThread);
     return pi.hProcess;
 }
 
@@ -2321,7 +2562,7 @@ static void test_local_server(void)
     ok(process != NULL, "couldn't start local server process, error was %d\n", GetLastError());
 
     ready_event = CreateEvent(NULL, FALSE, FALSE, "Wine COM Test Ready Event");
-    WaitForSingleObject(ready_event, 1000);
+    WaitForSingleObject(ready_event, INFINITE);
     CloseHandle(ready_event);
 
     hr = CoCreateInstance(&CLSID_WineOOPTest, NULL, CLSCTX_LOCAL_SERVER, &IID_IClassFactory, (void **)&cf);
@@ -2335,7 +2576,7 @@ static void test_local_server(void)
     quit_event = CreateEvent(NULL, FALSE, FALSE, "Wine COM Test Quit Event");
     SetEvent(quit_event);
 
-    WaitForSingleObject(process, INFINITE);
+    winetest_wait_child_process( process );
     CloseHandle(quit_event);
     CloseHandle(process);
 }
@@ -2358,10 +2599,11 @@ static DWORD CALLBACK get_global_interface_proc(LPVOID pv)
 		hr);
 
 	CoInitialize(NULL);
+
 	hr = IGlobalInterfaceTable_GetInterfaceFromGlobal(params->git, params->cookie, &IID_IClassFactory, (void **)&cf);
 	ok_ole_success(hr, IGlobalInterfaceTable_GetInterfaceFromGlobal);
 
-	IGlobalInterfaceTable_Release(params->git);
+	IClassFactory_Release(cf);
 
 	CoUninitialize();
 
@@ -2377,6 +2619,10 @@ static void test_globalinterfacetable(void)
 	DWORD tid;
 	struct git_params params;
 	DWORD ret;
+        IUnknown *object;
+
+        trace("test_globalinterfacetable\n");
+	cLocks = 0;
 
 	hr = CoCreateInstance(&CLSID_StdGlobalInterfaceTable, NULL, CLSCTX_INPROC_SERVER, &IID_IGlobalInterfaceTable, (void **)&git);
 	ok_ole_success(hr, CoCreateInstance);
@@ -2384,6 +2630,8 @@ static void test_globalinterfacetable(void)
 	hr = IGlobalInterfaceTable_RegisterInterfaceInGlobal(git, (IUnknown *)&Test_ClassFactory, &IID_IClassFactory, &cookie);
 	ok_ole_success(hr, IGlobalInterfaceTable_RegisterInterfaceInGlobal);
 
+	ok_more_than_one_lock();
+
 	params.cookie = cookie;
 	params.git = git;
 	/* note: params is on stack so we MUST wait for get_global_interface_proc
@@ -2400,6 +2648,23 @@ static void test_globalinterfacetable(void)
 	}
 
 	CloseHandle(thread);
+
+	/* test getting interface from global with different iid */
+	hr = IGlobalInterfaceTable_GetInterfaceFromGlobal(git, cookie, &IID_IUnknown, (void **)&object);
+	ok_ole_success(hr, IGlobalInterfaceTable_GetInterfaceFromGlobal);
+	IUnknown_Release(object);
+
+	/* test getting interface from global with same iid */
+	hr = IGlobalInterfaceTable_GetInterfaceFromGlobal(git, cookie, &IID_IClassFactory, (void **)&object);
+	ok_ole_success(hr, IGlobalInterfaceTable_GetInterfaceFromGlobal);
+	IUnknown_Release(object);
+
+	hr = IGlobalInterfaceTable_RevokeInterfaceFromGlobal(git, cookie);
+	ok_ole_success(hr, IGlobalInterfaceTable_RevokeInterfaceFromGlobal);
+
+	ok_no_locks();
+
+	IGlobalInterfaceTable_Release(git);
 }
 
 static const char *debugstr_iid(REFIID riid)
@@ -2460,9 +2725,7 @@ static void WINAPI TestChannelHook_ClientGetSize(
     trace("\t%s method %d\n", debugstr_iid(riid), info->iMethod);
     trace("\tcid: %s\n", debugstr_iid(&info->uCausality));
     ok(info->cbSize == sizeof(*info), "info->cbSize was %d instead of %d\n", info->cbSize, (int)sizeof(*info));
-    todo_wine {
     ok(info->dwServerPid == GetCurrentProcessId(), "info->dwServerPid was 0x%x instead of 0x%x\n", info->dwServerPid, GetCurrentProcessId());
-    }
     ok(!info->pObject, "info->pObject should be NULL\n");
     ok(IsEqualGUID(uExtent, &EXTENTID_WineTest), "uExtent wasn't correct\n");
 
@@ -2479,9 +2742,7 @@ static void WINAPI TestChannelHook_ClientFillBuffer(
     SChannelHookCallInfo *info = (SChannelHookCallInfo *)riid;
     trace("TestChannelHook_ClientFillBuffer\n");
     ok(info->cbSize == sizeof(*info), "info->cbSize was %d instead of %d\n", info->cbSize, (int)sizeof(*info));
-    todo_wine {
     ok(info->dwServerPid == GetCurrentProcessId(), "info->dwServerPid was 0x%x instead of 0x%x\n", info->dwServerPid, GetCurrentProcessId());
-    }
     ok(!info->pObject, "info->pObject should be NULL\n");
     ok(IsEqualGUID(uExtent, &EXTENTID_WineTest), "uExtent wasn't correct\n");
 
@@ -2501,8 +2762,8 @@ static void WINAPI TestChannelHook_ClientNotify(
     SChannelHookCallInfo *info = (SChannelHookCallInfo *)riid;
     trace("TestChannelHook_ClientNotify hrFault = 0x%08x\n", hrFault);
     ok(info->cbSize == sizeof(*info), "info->cbSize was %d instead of %d\n", info->cbSize, (int)sizeof(*info));
-    todo_wine {
     ok(info->dwServerPid == GetCurrentProcessId(), "info->dwServerPid was 0x%x instead of 0x%x\n", info->dwServerPid, GetCurrentProcessId());
+    todo_wine {
     ok(info->pObject != NULL, "info->pObject shouldn't be NULL\n");
     }
     ok(IsEqualGUID(uExtent, &EXTENTID_WineTest), "uExtent wasn't correct\n");
@@ -2619,7 +2880,6 @@ static void test_channel_hook(void)
 
 START_TEST(marshal)
 {
-    WNDCLASS wndclass;
     HMODULE hOle32 = GetModuleHandle("ole32");
     int argc;
     char **argv;
@@ -2636,11 +2896,7 @@ START_TEST(marshal)
         return;
     }
 
-    /* register a window class used in several tests */
-    memset(&wndclass, 0, sizeof(wndclass));
-    wndclass.lpfnWndProc = window_proc;
-    wndclass.lpszClassName = "WineCOMTest";
-    RegisterClass(&wndclass);
+    register_test_window();
 
     test_cocreateinstance_proxy();
 
@@ -2657,6 +2913,8 @@ START_TEST(marshal)
     test_interthread_marshal_and_unmarshal();
     test_proxy_marshal_and_unmarshal();
     test_proxy_marshal_and_unmarshal2();
+    test_proxy_marshal_and_unmarshal_weak();
+    test_proxy_marshal_and_unmarshal_strong();
     test_marshal_stub_apartment_shutdown();
     test_marshal_proxy_apartment_shutdown();
     test_marshal_proxy_mta_apartment_shutdown();
@@ -2682,6 +2940,7 @@ START_TEST(marshal)
     test_freethreadedmarshaler();
     test_inproc_handler();
     test_handler_marshaling();
+    test_client_security();
 
     test_local_server();
 
