@@ -21,7 +21,6 @@
 #include "config.h"
 
 #include "quartz_private.h"
-#include "control_private.h"
 #include "pin.h"
 
 #include "uuids.h"
@@ -31,7 +30,6 @@
 #include "dshow.h"
 #include "strmif.h"
 #include "vfwmsgs.h"
-#include "evcode.h"
 #include "msacm.h"
 
 #include <assert.h>
@@ -43,37 +41,39 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
-/* FIXME: Improve buffers management */
-#define OUTPUT_BUFFER_SIZE 15000
-#define INPUT_BUFFER_SIZE 4096
-
 typedef struct ACMWrapperImpl
 {
     TransformFilterImpl tf;
     HACMSTREAM has;
     LPWAVEFORMATEX pWfIn;
     LPWAVEFORMATEX pWfOut;
-    BYTE buffer[INPUT_BUFFER_SIZE];
-    DWORD max_size;
-    DWORD current_size;
-    BOOL reinit_codec; /* FIXME: Should use sync points instead */
 } ACMWrapperImpl;
 
-static HRESULT ACMWrapper_ProcessSampleData(TransformFilterImpl* pTransformFilter, LPBYTE data, DWORD size)
+static HRESULT ACMWrapper_ProcessSampleData(TransformFilterImpl* pTransformFilter, IMediaSample *pSample)
 {
     ACMWrapperImpl* This = (ACMWrapperImpl*)pTransformFilter;
     AM_MEDIA_TYPE amt;
-    HRESULT hr;
-    IMediaSample* pSample = NULL;
-    DWORD cbDstStream;
+    IMediaSample* pOutSample = NULL;
+    DWORD cbDstStream, cbSrcStream;
     LPBYTE pbDstStream;
+    LPBYTE pbSrcStream = NULL;
     ACMSTREAMHEADER ash;
-    DWORD offset = 0;
-    BOOL stop = FALSE;
     BOOL unprepare_header = FALSE;
     MMRESULT res;
+    HRESULT hr;
+    LONGLONG tStart = -1, tStop = -1, tMed;
 
-    TRACE("(%p)->(%p,%d)\n", This, data, size);
+    hr = IMediaSample_GetPointer(pSample, &pbSrcStream);
+    if (FAILED(hr))
+    {
+        ERR("Cannot get pointer to sample data (%x)\n", hr);
+	return hr;
+    }
+
+    IMediaSample_GetTime(pSample, &tStart, &tStop);
+    cbSrcStream = IMediaSample_GetActualDataLength(pSample);
+
+    TRACE("Sample data ptr = %p, size = %ld\n", pbSrcStream, (long)cbSrcStream);
 
     hr = IPin_ConnectionMediaType(This->tf.ppPins[0], &amt);
     if (FAILED(hr)) {
@@ -81,42 +81,30 @@ static HRESULT ACMWrapper_ProcessSampleData(TransformFilterImpl* pTransformFilte
 	return hr;
     }
 
-    while(!stop)
+    ash.pbSrc = pbSrcStream;
+    ash.cbSrcLength = cbSrcStream;
+
+    while(hr == S_OK && ash.cbSrcLength)
     {
-	DWORD rem_buf = This->max_size - This->current_size;
-	DWORD rem_smp = size - offset;
-	DWORD copy_size = min(rem_buf, rem_smp);
-
-	memcpy(This->buffer + This->current_size, data + offset, copy_size);
-	This->current_size += copy_size;
-	offset += copy_size;
-
-	if (offset >= size)
-	    stop = TRUE;
-	if (This->current_size < This->max_size)
-	    break;
-  
-	hr = OutputPin_GetDeliveryBuffer((OutputPin*)This->tf.ppPins[1], &pSample, NULL, NULL, 0);
+	hr = OutputPin_GetDeliveryBuffer((OutputPin*)This->tf.ppPins[1], &pOutSample, NULL, NULL, 0);
 	if (FAILED(hr)) {
 	    ERR("Unable to get delivery buffer (%x)\n", hr);
 	    return hr;
 	}
 
-	hr = IMediaSample_SetActualDataLength(pSample, 0);
+	hr = IMediaSample_SetActualDataLength(pOutSample, 0);
 	assert(hr == S_OK);
 
-	hr = IMediaSample_GetPointer(pSample, &pbDstStream);
+	hr = IMediaSample_GetPointer(pOutSample, &pbDstStream);
 	if (FAILED(hr)) {
 	    ERR("Unable to get pointer to buffer (%x)\n", hr);
 	    goto error;
 	}
-	cbDstStream = IMediaSample_GetSize(pSample);
+	cbDstStream = IMediaSample_GetSize(pOutSample);
 
 	ash.cbStruct = sizeof(ash);
 	ash.fdwStatus = 0;
 	ash.dwUser = 0;
-	ash.pbSrc = This->buffer;
-	ash.cbSrcLength = This->current_size;
 	ash.pbDst = pbDstStream;
 	ash.cbDstLength = cbDstStream;
 
@@ -124,41 +112,74 @@ static HRESULT ACMWrapper_ProcessSampleData(TransformFilterImpl* pTransformFilte
 	    ERR("Cannot prepare header %d\n", res);
 	    goto error;
 	}
-
 	unprepare_header = TRUE;
 
-	if ((res = acmStreamConvert(This->has, &ash, This->reinit_codec ? ACM_STREAMCONVERTF_START : 0))) {
-	    ERR("Cannot convert data header %d\n", res);
+        if (IMediaSample_IsDiscontinuity(pSample) == S_OK)
+            res = acmStreamConvert(This->has, &ash, ACM_STREAMCONVERTF_START);
+        else
+            res = acmStreamConvert(This->has, &ash, 0);
+
+
+        if (res)
+        {
+	    if(res != MMSYSERR_MOREDATA)
+	        ERR("Cannot convert data header %d\n", res);
 	    goto error;
 	}
-	This->reinit_codec = FALSE;
 
-	TRACE("used in %u, used out %u\n", ash.cbSrcLengthUsed, ash.cbDstLengthUsed);
+	TRACE("used in %u/%u, used out %u/%u\n", ash.cbSrcLengthUsed, ash.cbSrcLength, ash.cbDstLengthUsed, ash.cbDstLength);
 
-	hr = IMediaSample_SetActualDataLength(pSample, ash.cbDstLengthUsed);
+	hr = IMediaSample_SetActualDataLength(pOutSample, ash.cbDstLengthUsed);
 	assert(hr == S_OK);
 
-	if (ash.cbSrcLengthUsed < ash.cbSrcLength) {
-	    This->current_size = ash.cbSrcLength - ash.cbSrcLengthUsed;
-	    memmove(This->buffer, This->buffer + ash.cbSrcLengthUsed, This->current_size);
-	}
-	else
-	    This->current_size = 0;
+        if (!ash.cbSrcLengthUsed)
+        {
+            WARN("Sample was skipped\n");
+            ash.cbSrcLength = 0;
+            goto error;
+        }
 
-	hr = OutputPin_SendSample((OutputPin*)This->tf.ppPins[1], pSample);
-	if (hr != S_OK && hr != VFW_E_NOT_CONNECTED) {
-	    ERR("Error sending sample (%x)\n", hr);
-	    goto error;
+        TRACE("Sample start time: %u.%03u\n", (DWORD)(tStart/10000000), (DWORD)((tStart/10000)%1000));
+        if (ash.cbSrcLengthUsed == cbSrcStream)
+        {
+            IMediaSample_SetTime(pOutSample, &tStart, &tStop);
+            tStart = tStop;
+        }
+        else if (tStop != tStart)
+        {
+            tMed = tStop - tStart;
+            tMed = tStart + tMed * ash.cbSrcLengthUsed / cbSrcStream;
+            IMediaSample_SetTime(pOutSample, &tStart, &tMed);
+            tStart = tMed;
+        }
+        else
+        {
+            ERR("No valid timestamp found\n");
+            IMediaSample_SetTime(pOutSample, NULL, NULL);
+        }
+        TRACE("Sample stop time: %u.%03u\n", (DWORD)(tStart/10000000), (DWORD)((tStart/10000)%1000));
+
+        if (IMediaSample_IsDiscontinuity(pSample) == S_FALSE)
+            hr = OutputPin_SendSample((OutputPin*)This->tf.ppPins[1], pOutSample);
+        else
+            TRACE("Skipping first sample: Discontinuity\n");
+
+        if (hr != S_OK && hr != VFW_E_NOT_CONNECTED) {
+            if (FAILED(hr))
+                ERR("Error sending sample (%x)\n", hr);
+            goto error;
         }
 
 error:
         if (unprepare_header && (res = acmStreamUnprepareHeader(This->has, &ash, 0)))
             ERR("Cannot unprepare header %d\n", res);
         unprepare_header = FALSE;
+        ash.pbSrc += ash.cbSrcLengthUsed;
+        ash.cbSrcLength -= ash.cbSrcLengthUsed;
 
-        if (pSample)
-            IMediaSample_Release(pSample);
-        pSample = NULL;
+        if (pOutSample)
+            IMediaSample_Release(pOutSample);
+        pOutSample = NULL;
     }
 
     return hr;
@@ -199,16 +220,8 @@ static HRESULT ACMWrapper_ConnectInput(TransformFilterImpl* pTransformFilter, co
         {
             This->has = drv;
 
-	    if ((res = acmStreamSize(drv, OUTPUT_BUFFER_SIZE, &This->max_size, ACM_STREAMSIZEF_DESTINATION))) {
-		ERR("Cannot retrieve input buffer size error %d!\n", res);
-		This->max_size = INPUT_BUFFER_SIZE;
-	    }
-
-	    TRACE("input buffer size %d\n", This->max_size);
-
             /* Update buffer size of media samples in output */
-            ((OutputPin*)This->tf.ppPins[1])->allocProps.cbBuffer = OUTPUT_BUFFER_SIZE;
-	    
+            ((OutputPin*)This->tf.ppPins[1])->allocProps.cbBuffer = This->pWfOut->nAvgBytesPerSec / 2;
             TRACE("Connection accepted\n");
             return S_OK;
         }
@@ -219,7 +232,7 @@ static HRESULT ACMWrapper_ConnectInput(TransformFilterImpl* pTransformFilter, co
     }
 
     TRACE("Connection refused\n");
-    return S_FALSE;
+    return VFW_E_TYPE_NOT_ACCEPTED;
 }
 
 static HRESULT ACMWrapper_Cleanup(TransformFilterImpl* pTransformFilter)
@@ -240,6 +253,7 @@ static const TransformFuncsTable ACMWrapper_FuncsTable = {
     NULL,
     ACMWrapper_ProcessSampleData,
     NULL,
+    NULL,
     ACMWrapper_ConnectInput,
     ACMWrapper_Cleanup
 };
@@ -260,9 +274,7 @@ HRESULT ACMWrapper_create(IUnknown * pUnkOuter, LPVOID * ppv)
     This = CoTaskMemAlloc(sizeof(ACMWrapperImpl));
     ZeroMemory(This, sizeof(ACMWrapperImpl));
 
-    This->reinit_codec = TRUE;
-
-    hr = TransformFilter_Create(&(This->tf), &CLSID_ACMWrapper, &ACMWrapper_FuncsTable);
+    hr = TransformFilter_Create(&(This->tf), &CLSID_ACMWrapper, &ACMWrapper_FuncsTable, NULL, NULL, NULL);
 
     if (FAILED(hr))
         return hr;
