@@ -45,6 +45,7 @@ static const IPinVtbl DSoundRender_InputPin_Vtbl;
 static const IMemInputPinVtbl MemInputPin_Vtbl; 
 static const IBasicAudioVtbl IBasicAudio_Vtbl;
 static const IReferenceClockVtbl IReferenceClock_Vtbl;
+static const IMediaSeekingVtbl IMediaSeeking_Vtbl;
 
 typedef struct DSoundRenderImpl
 {
@@ -71,11 +72,31 @@ typedef struct DSoundRenderImpl
     DWORD last_play_pos;
     DWORD play_loops;
     REFERENCE_TIME play_time;
+    MediaSeekingImpl mediaSeeking;
 
     long volume;
     long pan;
 } DSoundRenderImpl;
 
+static HRESULT sound_mod_stop(IBaseFilter *iface)
+{
+    FIXME("(%p) stub\n", iface);
+    return S_OK;
+}
+
+static HRESULT sound_mod_start(IBaseFilter *iface)
+{
+    FIXME("(%p) stub\n", iface);
+    return S_OK;
+}
+
+static HRESULT sound_mod_rate(IBaseFilter *iface)
+{
+    FIXME("(%p) stub\n", iface);
+    return S_OK;
+}
+
+
 static HRESULT DSoundRender_InputPin_Construct(const PIN_INFO * pPinInfo, SAMPLEPROC pSampleProc, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, LPCRITICAL_SECTION pCritSec, IPin ** ppPin)
 {
     InputPin * pPinImpl;
@@ -97,10 +118,12 @@ static HRESULT DSoundRender_InputPin_Construct(const PIN_INFO * pPinInfo, SAMPLE
     {
         pPinImpl->pin.lpVtbl = &DSoundRender_InputPin_Vtbl;
         pPinImpl->lpVtblMemInput = &MemInputPin_Vtbl;
-        
+
         *ppPin = (IPin *)(&pPinImpl->pin.lpVtbl);
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
@@ -229,7 +252,6 @@ static HRESULT DSoundRender_Sample(LPVOID iface, IMediaSample * pSample)
         ERR("Cannot get sample time (%x)\n", hr);
 
     cbSrcStream = IMediaSample_GetActualDataLength(pSample);
-
     TRACE("Sample data ptr = %p, size = %ld\n", pbSrcStream, cbSrcStream);
 
 #if 0 /* For debugging purpose */
@@ -313,6 +335,9 @@ HRESULT DSoundRender_create(IUnknown * pUnkOuter, LPVOID * ppv)
 
     if (SUCCEEDED(hr))
     {
+        MediaSeekingImpl_Init((IBaseFilter*)pDSoundRender, sound_mod_stop, sound_mod_start, sound_mod_rate, &pDSoundRender->mediaSeeking, &pDSoundRender->csFilter);
+        pDSoundRender->mediaSeeking.lpVtbl = &IMediaSeeking_Vtbl;
+
         pDSoundRender->ppPins[0] = (IPin *)pDSoundRender->pInputPin;
         *ppv = (LPVOID)pDSoundRender;
     }
@@ -348,6 +373,8 @@ static HRESULT WINAPI DSoundRender_QueryInterface(IBaseFilter * iface, REFIID ri
         *ppv = (LPVOID)&(This->IBasicAudio_vtbl);
     else if (IsEqualIID(riid, &IID_IReferenceClock))
         *ppv = (LPVOID)&(This->IReferenceClock_vtbl);
+    else if (IsEqualIID(riid, &IID_IMediaSeeking))
+        *ppv = &This->mediaSeeking.lpVtbl;
 
     if (*ppv)
     {
@@ -355,7 +382,8 @@ static HRESULT WINAPI DSoundRender_QueryInterface(IBaseFilter * iface, REFIID ri
         return S_OK;
     }
 
-    FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
+    if (!IsEqualIID(riid, &IID_IPin))
+        FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
 
     return E_NOINTERFACE;
 }
@@ -496,6 +524,15 @@ static HRESULT WINAPI DSoundRender_Run(IBaseFilter * iface, REFERENCE_TIME tStar
         /* It's okay if there's no buffer yet. It'll start when it's created */
         if (This->dsbuffer)
         {
+            if (This->state == State_Stopped)
+            {
+                char *buf1;
+                DWORD size1;
+
+                IDirectSoundBuffer_Lock(This->dsbuffer, 0, 0, (void**)&buf1, &size1, NULL, NULL, DSBLOCK_ENTIREBUFFER);
+                memset(buf1, 0, size1);
+                IDirectSoundBuffer_Unlock(This->dsbuffer, buf1, size1, NULL, 0);
+            }
             hr = IDirectSoundBuffer_Play(This->dsbuffer, 0, 0, DSBPLAY_LOOPING);
             if (FAILED(hr))
                 ERR("Can't start playing! (%x)\n", hr);
@@ -759,15 +796,26 @@ static HRESULT WINAPI DSoundRender_InputPin_Disconnect(IPin * iface)
 static HRESULT WINAPI DSoundRender_InputPin_EndOfStream(IPin * iface)
 {
     InputPin* This = (InputPin*)iface;
+    DSoundRenderImpl *me = (DSoundRenderImpl*)This->pin.pinInfo.pFilter;
     IMediaEventSink* pEventSink;
     HRESULT hr;
 
     TRACE("(%p/%p)->()\n", This, iface);
+    InputPin_EndOfStream(iface);
 
-    hr = IFilterGraph_QueryInterface(((DSoundRenderImpl*)This->pin.pinInfo.pFilter)->filterInfo.pGraph, &IID_IMediaEventSink, (LPVOID*)&pEventSink);
+    hr = IFilterGraph_QueryInterface(me->filterInfo.pGraph, &IID_IMediaEventSink, (LPVOID*)&pEventSink);
     if (SUCCEEDED(hr))
     {
-        /* FIXME: We should wait that all audio data has been played */
+        BYTE * silence;
+
+        silence = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, me->buf_size);
+        if (silence)
+        {
+            memset(silence, 0, me->buf_size);
+            DSoundRender_SendSampleData((DSoundRenderImpl*)This->pin.pinInfo.pFilter, silence, me->buf_size);
+            HeapFree(GetProcessHeap(), 0, silence);
+        }
+
         hr = IMediaEventSink_Notify(pEventSink, EC_COMPLETE, S_OK, 0);
         IMediaEventSink_Release(pEventSink);
     }
@@ -1009,7 +1057,7 @@ static HRESULT WINAPI ReferenceClock_GetTime(IReferenceClock *iface,
     if (This->dsbuffer)
         hr = DSoundRender_GetPos(This, &play_pos, NULL, pTime);
     if (FAILED(hr))
-        ERR("Could not get refreence time (%x)!\n", hr);
+        ERR("Could not get reference time (%x)!\n", hr);
 
     return hr;
 }
@@ -1060,3 +1108,53 @@ static const IReferenceClockVtbl IReferenceClock_Vtbl =
     ReferenceClock_AdvisePeriodic,
     ReferenceClock_Unadvise
 };
+
+static inline DSoundRenderImpl *impl_from_IMediaSeeking( IMediaSeeking *iface )
+{
+    return (DSoundRenderImpl *)((char*)iface - FIELD_OFFSET(DSoundRenderImpl, mediaSeeking.lpVtbl));
+}
+
+static HRESULT WINAPI sound_seek_QueryInterface(IMediaSeeking * iface, REFIID riid, LPVOID * ppv)
+{
+    DSoundRenderImpl *This = impl_from_IMediaSeeking(iface);
+
+    return IUnknown_QueryInterface((IUnknown *)This, riid, ppv);
+}
+
+static ULONG WINAPI sound_seek_AddRef(IMediaSeeking * iface)
+{
+    DSoundRenderImpl *This = impl_from_IMediaSeeking(iface);
+
+    return IUnknown_AddRef((IUnknown *)This);
+}
+
+static ULONG WINAPI sound_seek_Release(IMediaSeeking * iface)
+{
+    DSoundRenderImpl *This = impl_from_IMediaSeeking(iface);
+
+    return IUnknown_Release((IUnknown *)This);
+}
+
+static const IMediaSeekingVtbl IMediaSeeking_Vtbl =
+{
+    sound_seek_QueryInterface,
+    sound_seek_AddRef,
+    sound_seek_Release,
+    MediaSeekingImpl_GetCapabilities,
+    MediaSeekingImpl_CheckCapabilities,
+    MediaSeekingImpl_IsFormatSupported,
+    MediaSeekingImpl_QueryPreferredFormat,
+    MediaSeekingImpl_GetTimeFormat,
+    MediaSeekingImpl_IsUsingTimeFormat,
+    MediaSeekingImpl_SetTimeFormat,
+    MediaSeekingImpl_GetDuration,
+    MediaSeekingImpl_GetStopPosition,
+    MediaSeekingImpl_GetCurrentPosition,
+    MediaSeekingImpl_ConvertTimeFormat,
+    MediaSeekingImpl_SetPositions,
+    MediaSeekingImpl_GetPositions,
+    MediaSeekingImpl_GetAvailable,
+    MediaSeekingImpl_SetRate,
+    MediaSeekingImpl_GetRate,
+    MediaSeekingImpl_GetPreroll
+};
