@@ -30,6 +30,7 @@
 #include "vfwmsgs.h"
 #include "winbase.h"
 #include "winreg.h"
+#include "shlwapi.h"
 #include <assert.h>
 
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
@@ -62,10 +63,47 @@ static inline AsyncReader *impl_from_IFileSourceFilter( IFileSourceFilter *iface
     return (AsyncReader *)((char*)iface - FIELD_OFFSET(AsyncReader, lpVtblFSF));
 }
 
+static WCHAR const mediatype_name[11] = {
+    'M', 'e', 'd', 'i', 'a', ' ', 'T', 'y', 'p', 'e', 0 };
+static WCHAR const subtype_name[8] = {
+    'S', 'u', 'b', 't', 'y', 'p', 'e', 0 };
+
 static HRESULT process_extensions(HKEY hkeyExtensions, LPCOLESTR pszFileName, GUID * majorType, GUID * minorType)
 {
-    /* FIXME: implement */
-    return E_NOTIMPL;
+    WCHAR *extension;
+    LONG l;
+    HKEY hsub;
+    WCHAR keying[39];
+    DWORD size;
+
+    if (!pszFileName)
+        return E_POINTER;
+
+    /* Get the part of the name that matters */
+    extension = PathFindExtensionW(pszFileName);
+    if (*extension != '.')
+        return E_FAIL;
+
+    l = RegOpenKeyExW(hkeyExtensions, extension, 0, KEY_READ, &hsub);
+    if (l)
+        return E_FAIL;
+
+    size = sizeof(keying);
+    l = RegQueryValueExW(hsub, mediatype_name, NULL, NULL, (LPBYTE)keying, &size);
+    if (!l)
+        CLSIDFromString(keying, majorType);
+
+    size = sizeof(keying);
+    if (!l)
+        l = RegQueryValueExW(hsub, subtype_name, NULL, NULL, (LPBYTE)keying, &size);
+    if (!l)
+        CLSIDFromString(keying, minorType);
+
+    RegCloseKey(hsub);
+
+    if (!l)
+        return S_OK;
+    return E_FAIL;
 }
 
 static unsigned char byte_from_hex_char(WCHAR wHex)
@@ -197,8 +235,10 @@ static HRESULT GetClassMediaFile(IAsyncReader * pReader, LPCOLESTR pszFileName,
     BOOL bFound = FALSE;
     static const WCHAR wszMediaType[] = {'M','e','d','i','a',' ','T','y','p','e',0};
 
-    CopyMemory(majorType, &GUID_NULL, sizeof(*majorType));
-    CopyMemory(minorType, &GUID_NULL, sizeof(*minorType));
+    TRACE("(%p, %s, %p, %p)\n", pReader, debugstr_w(pszFileName), majorType, minorType);
+
+    *majorType = GUID_NULL;
+    *minorType = GUID_NULL;
 
     lRet = RegOpenKeyExW(HKEY_CLASSES_ROOT, wszMediaType, 0, KEY_READ, &hkeyMediaType);
     hr = HRESULT_FROM_WIN32(lRet);
@@ -213,7 +253,7 @@ static HRESULT GetClassMediaFile(IAsyncReader * pReader, LPCOLESTR pszFileName,
             WCHAR wszMajorKeyName[CHARS_IN_GUID];
             DWORD dwKeyNameLength = sizeof(wszMajorKeyName) / sizeof(wszMajorKeyName[0]);
             static const WCHAR wszExtensions[] = {'E','x','t','e','n','s','i','o','n','s',0};
-    
+
             if (RegEnumKeyExW(hkeyMediaType, indexMajor, wszMajorKeyName, &dwKeyNameLength, NULL, NULL, NULL, NULL) != ERROR_SUCCESS)
                 break;
             if (RegOpenKeyExW(hkeyMediaType, wszMajorKeyName, 0, KEY_READ, &hkeyMajor) != ERROR_SUCCESS)
@@ -353,7 +393,8 @@ static HRESULT WINAPI AsyncReader_QueryInterface(IBaseFilter * iface, REFIID rii
         return S_OK;
     }
 
-    FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
+    if (!IsEqualIID(riid, &IID_IPin) && !IsEqualIID(riid, &IID_IMediaSeeking))
+        FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
 
     return E_NOINTERFACE;
 }
@@ -609,7 +650,7 @@ static HRESULT WINAPI FileSource_Load(IFileSourceFilter * iface, LPCOLESTR pszFi
             This->pmt->pbFormat = NULL;
             This->pmt->pUnk = NULL;
             This->pmt->lSampleSize = 0;
-            memcpy(&This->pmt->formattype, &FORMAT_None, sizeof(FORMAT_None));
+            This->pmt->formattype = FORMAT_None;
             hr = GetClassMediaFile(pReader, pszFileName, &This->pmt->majortype, &This->pmt->subtype);
             if (FAILED(hr))
             {
@@ -752,7 +793,8 @@ static HRESULT WINAPI FileAsyncReaderPin_QueryInterface(IPin * iface, REFIID rii
         return S_OK;
     }
 
-    FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
+    if (!IsEqualIID(riid, &IID_IPin) && !IsEqualIID(riid, &IID_IMediaSeeking))
+        FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
 
     return E_NOINTERFACE;
 }
@@ -880,6 +922,8 @@ static HRESULT FileAsyncReader_Construct(HANDLE hFile, IBaseFilter * pBaseFilter
         *ppPin = (IPin *)(&pPinImpl->pin.pin.lpVtbl);
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
