@@ -32,7 +32,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(quartz);
 
-void dump_AM_SAMPLE2_PROPERTIES(AM_SAMPLE2_PROPERTIES * pProps)
+void dump_AM_SAMPLE2_PROPERTIES(const AM_SAMPLE2_PROPERTIES * pProps)
 {
     if (!pProps)
     {
@@ -188,9 +188,9 @@ static HRESULT WINAPI BaseMemAllocator_SetProperties(IMemAllocator * iface, ALLO
                 hr = E_OUTOFMEMORY;
             else
             {
-                memcpy(This->pProps, pRequest, sizeof(*This->pProps));
-                    
-                memcpy(pActual, pRequest, sizeof(*pActual));
+                *This->pProps = *pRequest;
+
+                *pActual = *pRequest;
 
                 hr = S_OK;
             }
