@@ -43,19 +43,19 @@ static const IPinVtbl Parser_OutputPin_Vtbl;
 static const IPinVtbl Parser_InputPin_Vtbl;
 
 static HRESULT Parser_OutputPin_QueryAccept(LPVOID iface, const AM_MEDIA_TYPE * pmt);
-static HRESULT Parser_ChangeStart(LPVOID iface);
-static HRESULT Parser_ChangeStop(LPVOID iface);
-static HRESULT Parser_ChangeRate(LPVOID iface);
+static HRESULT Parser_ChangeCurrent(IBaseFilter *iface);
+static HRESULT Parser_ChangeStop(IBaseFilter *iface);
+static HRESULT Parser_ChangeRate(IBaseFilter *iface);
 
 static HRESULT Parser_InputPin_Construct(const PIN_INFO * pPinInfo, SAMPLEPROC pSampleProc, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, LPCRITICAL_SECTION pCritSec, IPin ** ppPin);
 
-static inline Parser_OutputPin *impl_from_IMediaSeeking( IMediaSeeking *iface )
+static inline ParserImpl *impl_from_IMediaSeeking( IMediaSeeking *iface )
 {
-    return (Parser_OutputPin *)((char*)iface - FIELD_OFFSET(Parser_OutputPin, mediaSeeking.lpVtbl));
+    return (ParserImpl *)((char*)iface - FIELD_OFFSET(ParserImpl, mediaSeeking.lpVtbl));
 }
 
 
-HRESULT Parser_Create(ParserImpl* pParser, const CLSID* pClsid, PFN_PROCESS_SAMPLE fnProcessSample, PFN_QUERY_ACCEPT fnQueryAccept, PFN_PRE_CONNECT fnPreConnect, PFN_CLEANUP fnCleanup)
+HRESULT Parser_Create(ParserImpl* pParser, const CLSID* pClsid, PFN_PROCESS_SAMPLE fnProcessSample, PFN_QUERY_ACCEPT fnQueryAccept, PFN_PRE_CONNECT fnPreConnect, PFN_CLEANUP fnCleanup, CHANGEPROC stop, CHANGEPROC current, CHANGEPROC rate)
 {
     HRESULT hr;
     PIN_INFO piInput;
@@ -80,6 +80,18 @@ HRESULT Parser_Create(ParserImpl* pParser, const CLSID* pClsid, PFN_PROCESS_SAMP
     piInput.pFilter = (IBaseFilter *)pParser;
     lstrcpynW(piInput.achName, wcsInputPinName, sizeof(piInput.achName) / sizeof(piInput.achName[0]));
 
+    if (!current)
+        current = Parser_ChangeCurrent;
+
+    if (!stop)
+        stop = Parser_ChangeStop;
+
+    if (!rate)
+        rate = Parser_ChangeRate;
+
+    MediaSeekingImpl_Init((IBaseFilter*)pParser, stop, current, rate, &pParser->mediaSeeking, &pParser->csFilter);
+    pParser->mediaSeeking.lpVtbl = &Parser_Seeking_Vtbl;
+
     hr = Parser_InputPin_Construct(&piInput, fnProcessSample, (LPVOID)pParser, fnQueryAccept, &pParser->csFilter, (IPin **)&pParser->pInputPin);
 
     if (SUCCEEDED(hr))
@@ -98,21 +110,16 @@ HRESULT Parser_Create(ParserImpl* pParser, const CLSID* pClsid, PFN_PROCESS_SAMP
     return hr;
 }
 
-static HRESULT Parser_OutputPin_Init(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES * props, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, const AM_MEDIA_TYPE * pmt, float fSamplesPerSec, LPCRITICAL_SECTION pCritSec, Parser_OutputPin * pPinImpl)
+static HRESULT Parser_OutputPin_Init(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES * props, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, const AM_MEDIA_TYPE * pmt, LPCRITICAL_SECTION pCritSec, Parser_OutputPin * pPinImpl)
 {
     pPinImpl->pmt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
     CopyMediaType(pPinImpl->pmt, pmt);
     pPinImpl->dwSamplesProcessed = 0;
-    pPinImpl->dwSampleSize = 0;
-    pPinImpl->fSamplesPerSec = fSamplesPerSec;
-
-    MediaSeekingImpl_Init((LPVOID)pPinInfo->pFilter, Parser_ChangeStop, Parser_ChangeStart, Parser_ChangeRate, &pPinImpl->mediaSeeking);
-    pPinImpl->mediaSeeking.lpVtbl = &Parser_Seeking_Vtbl;
 
     return OutputPin_Init(pPinInfo, props, pUserData, pQueryAccept, pCritSec, &pPinImpl->pin);
 }
 
-static HRESULT Parser_OutputPin_Construct(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES * props, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, const AM_MEDIA_TYPE * pmt, float fSamplesPerSec, LPCRITICAL_SECTION pCritSec, IPin ** ppPin)
+static HRESULT Parser_OutputPin_Construct(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES * props, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, const AM_MEDIA_TYPE * pmt, LPCRITICAL_SECTION pCritSec, IPin ** ppPin)
 {
     Parser_OutputPin * pPinImpl;
 
@@ -125,13 +132,15 @@ static HRESULT Parser_OutputPin_Construct(const PIN_INFO * pPinInfo, ALLOCATOR_P
     if (!pPinImpl)
         return E_OUTOFMEMORY;
 
-    if (SUCCEEDED(Parser_OutputPin_Init(pPinInfo, props, pUserData, pQueryAccept, pmt, fSamplesPerSec, pCritSec, pPinImpl)))
+    if (SUCCEEDED(Parser_OutputPin_Init(pPinInfo, props, pUserData, pQueryAccept, pmt, pCritSec, pPinImpl)))
     {
         pPinImpl->pin.pin.lpVtbl = &Parser_OutputPin_Vtbl;
-        
+
         *ppPin = (IPin *)pPinImpl;
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
@@ -150,6 +159,8 @@ static HRESULT WINAPI Parser_QueryInterface(IBaseFilter * iface, REFIID riid, LP
         *ppv = (LPVOID)This;
     else if (IsEqualIID(riid, &IID_IBaseFilter))
         *ppv = (LPVOID)This;
+    else if (IsEqualIID(riid, &IID_IMediaSeeking))
+        *ppv = (LPVOID)&This->mediaSeeking;
 
     if (*ppv)
     {
@@ -157,7 +168,8 @@ static HRESULT WINAPI Parser_QueryInterface(IBaseFilter * iface, REFIID riid, LP
         return S_OK;
     }
 
-    FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
+    if (!IsEqualIID(riid, &IID_IPin))
+        FIXME("No interface for %s!\n", qzdebugstr_guid(riid));
 
     return E_NOINTERFACE;
 }
@@ -247,11 +259,11 @@ static HRESULT WINAPI Parser_Stop(IBaseFilter * iface)
             LeaveCriticalSection(&This->csFilter);
             return S_OK;
         }
-        hr = PullPin_StopProcessing(This->pInputPin);
         This->state = State_Stopped;
     }
     LeaveCriticalSection(&This->csFilter);
-    
+
+    hr = PullPin_StopProcessing(This->pInputPin);
     return hr;
 }
 
@@ -279,8 +291,6 @@ static HRESULT WINAPI Parser_Pause(IBaseFilter * iface)
     {
         unsigned int i;
 
-        hr = PullPin_Seek(This->pInputPin, 0, ((LONGLONG)0x7fffffff << 32) | 0xffffffff);
-
         if (SUCCEEDED(hr))
             hr = PullPin_InitProcessing(This->pInputPin);
 
@@ -288,10 +298,12 @@ static HRESULT WINAPI Parser_Pause(IBaseFilter * iface)
         {
             for (i = 1; i < This->cStreams + 1; i++)
             {
-                Parser_OutputPin* StreamPin = (Parser_OutputPin *)This->ppPins[i];
-                OutputPin_DeliverNewSegment((OutputPin *)This->ppPins[i], 0, (LONGLONG)ceil(10000000.0 * (float)StreamPin->dwLength / StreamPin->fSamplesPerSec), 1.0);
-                StreamPin->mediaSeeking.llDuration = (LONGLONG)ceil(10000000.0 * (float)StreamPin->dwLength / StreamPin->fSamplesPerSec);
-                StreamPin->mediaSeeking.llStop = (LONGLONG)ceil(10000000.0 * (float)StreamPin->dwLength / StreamPin->fSamplesPerSec);
+                LONGLONG duration;
+                DOUBLE speed;
+
+                IMediaSeeking_GetDuration((IMediaSeeking *)&This->mediaSeeking, &duration);
+                IMediaSeeking_GetRate((IMediaSeeking *)&This->mediaSeeking, &speed);
+                OutputPin_DeliverNewSegment((OutputPin *)This->ppPins[i], 0, duration, speed);
                 OutputPin_CommitAllocator((OutputPin *)This->ppPins[i]);
             }
 
@@ -328,11 +340,11 @@ static HRESULT WINAPI Parser_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
 
         This->rtStreamStart = tStart;
 
-        hr = PullPin_Seek(This->pInputPin, tStart, ((LONGLONG)0x7fffffff << 32) | 0xffffffff);
-
         if (SUCCEEDED(hr) && (This->state == State_Stopped))
         {
+            LeaveCriticalSection(&This->csFilter);
             hr = PullPin_InitProcessing(This->pInputPin);
+            EnterCriticalSection(&This->csFilter);
 
             if (SUCCEEDED(hr))
             { 
@@ -344,7 +356,11 @@ static HRESULT WINAPI Parser_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
         }
 
         if (SUCCEEDED(hr))
+        {
+            LeaveCriticalSection(&This->csFilter);
             hr = PullPin_StartProcessing(This->pInputPin);
+            EnterCriticalSection(&This->csFilter);
+        }
 
         if (SUCCEEDED(hr))
             This->state = State_Running;
@@ -495,8 +511,7 @@ static const IBaseFilterVtbl Parser_Vtbl =
     Parser_QueryVendorInfo
 };
 
-HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PROPERTIES * props,
-                      const AM_MEDIA_TYPE * amt, float fSamplesPerSec, DWORD dwSampleSize, DWORD dwLength)
+HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PROPERTIES * props, const AM_MEDIA_TYPE * amt)
 {
     IPin ** ppOldPins;
     HRESULT hr;
@@ -506,12 +521,10 @@ HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PR
     This->ppPins = CoTaskMemAlloc((This->cStreams + 2) * sizeof(IPin *));
     memcpy(This->ppPins, ppOldPins, (This->cStreams + 1) * sizeof(IPin *));
 
-    hr = Parser_OutputPin_Construct(piOutput, props, NULL, Parser_OutputPin_QueryAccept, amt, fSamplesPerSec, &This->csFilter, This->ppPins + This->cStreams + 1);
+    hr = Parser_OutputPin_Construct(piOutput, props, NULL, Parser_OutputPin_QueryAccept, amt, &This->csFilter, This->ppPins + This->cStreams + 1);
 
     if (SUCCEEDED(hr))
     {
-        ((Parser_OutputPin *)(This->ppPins[This->cStreams + 1]))->dwSampleSize = dwSampleSize;
-        ((Parser_OutputPin *)(This->ppPins[This->cStreams + 1]))->dwLength = dwLength;
         ((Parser_OutputPin *)(This->ppPins[This->cStreams + 1]))->pin.pin.pUserData = (LPVOID)This->ppPins[This->cStreams + 1];
         This->cStreams++;
         CoTaskMemFree(ppOldPins);
@@ -549,42 +562,42 @@ static HRESULT Parser_RemoveOutputPins(ParserImpl * This)
     return S_OK;
 }
 
-static HRESULT Parser_ChangeStart(LPVOID iface)
+static HRESULT Parser_ChangeCurrent(IBaseFilter *iface)
 {
-    FIXME("(%p)\n", iface);
+    FIXME("(%p) filter hasn't implemented current position change!\n", iface);
     return S_OK;
 }
 
-static HRESULT Parser_ChangeStop(LPVOID iface)
+static HRESULT Parser_ChangeStop(IBaseFilter *iface)
 {
-    FIXME("(%p)\n", iface);
+    FIXME("(%p) filter hasn't implemented stop position change!\n", iface);
     return S_OK;
 }
 
-static HRESULT Parser_ChangeRate(LPVOID iface)
+static HRESULT Parser_ChangeRate(IBaseFilter *iface)
 {
-    FIXME("(%p)\n", iface);
+    FIXME("(%p) filter hasn't implemented rate change!\n", iface);
     return S_OK;
 }
 
 
 static HRESULT WINAPI Parser_Seeking_QueryInterface(IMediaSeeking * iface, REFIID riid, LPVOID * ppv)
 {
-    Parser_OutputPin *This = impl_from_IMediaSeeking(iface);
+    ParserImpl *This = impl_from_IMediaSeeking(iface);
 
     return IUnknown_QueryInterface((IUnknown *)This, riid, ppv);
 }
 
 static ULONG WINAPI Parser_Seeking_AddRef(IMediaSeeking * iface)
 {
-    Parser_OutputPin *This = impl_from_IMediaSeeking(iface);
+    ParserImpl *This = impl_from_IMediaSeeking(iface);
 
     return IUnknown_AddRef((IUnknown *)This);
 }
 
 static ULONG WINAPI Parser_Seeking_Release(IMediaSeeking * iface)
 {
-    Parser_OutputPin *This = impl_from_IMediaSeeking(iface);
+    ParserImpl *This = impl_from_IMediaSeeking(iface);
 
     return IUnknown_Release((IUnknown *)This);
 }
@@ -626,7 +639,9 @@ static HRESULT WINAPI Parser_OutputPin_QueryInterface(IPin * iface, REFIID riid,
     else if (IsEqualIID(riid, &IID_IPin))
         *ppv = (LPVOID)iface;
     else if (IsEqualIID(riid, &IID_IMediaSeeking))
-        *ppv = (LPVOID)&This->mediaSeeking;
+    {
+        return IBaseFilter_QueryInterface(This->pin.pin.pinInfo.pFilter, &IID_IMediaSeeking, ppv);
+    }
 
     if (*ppv)
     {
@@ -723,10 +738,12 @@ static HRESULT Parser_InputPin_Construct(const PIN_INFO * pPinInfo, SAMPLEPROC p
     if (SUCCEEDED(PullPin_Init(pPinInfo, pSampleProc, pUserData, pQueryAccept, pCritSec, pPinImpl)))
     {
         pPinImpl->pin.lpVtbl = &Parser_InputPin_Vtbl;
-        
+
         *ppPin = (IPin *)(&pPinImpl->pin.lpVtbl);
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
