@@ -41,6 +41,7 @@ struct ParserImpl
     PullPin * pInputPin;
     IPin ** ppPins;
     ULONG cStreams;
+    MediaSeekingImpl mediaSeeking;
 };
 
 typedef struct Parser_OutputPin
@@ -48,12 +49,9 @@ typedef struct Parser_OutputPin
     OutputPin pin;
 
     AM_MEDIA_TYPE * pmt;
-    float fSamplesPerSec;
     DWORD dwSamplesProcessed;
-    DWORD dwSampleSize;
-    DWORD dwLength;
-    MediaSeekingImpl mediaSeeking;
 } Parser_OutputPin;
 
-HRESULT Parser_AddPin(ParserImpl * This, PIN_INFO * piOutput, ALLOCATOR_PROPERTIES * props, AM_MEDIA_TYPE * amt, float fSamplesPerSec, DWORD dwSampleSize, DWORD dwLength);
-HRESULT Parser_Create(ParserImpl*, const CLSID*, PFN_PROCESS_SAMPLE, PFN_QUERY_ACCEPT, PFN_PRE_CONNECT, PFN_CLEANUP);
+HRESULT Parser_AddPin(ParserImpl * This, const PIN_INFO * piOutput, ALLOCATOR_PROPERTIES * props, const AM_MEDIA_TYPE * amt);
+
+HRESULT Parser_Create(ParserImpl*, const CLSID*, PFN_PROCESS_SAMPLE, PFN_QUERY_ACCEPT, PFN_PRE_CONNECT, PFN_CLEANUP, CHANGEPROC stop, CHANGEPROC current, CHANGEPROC rate);
