@@ -37,6 +37,105 @@ static const IPinVtbl PullPin_Vtbl;
 #define ALIGNDOWN(value,boundary) ((value)/(boundary)*(boundary))
 #define ALIGNUP(value,boundary) (ALIGNDOWN((value)+(boundary)-1, (boundary)))
 
+typedef HRESULT (*SendPinFunc)( IPin *to, LPVOID arg );
+
+/** Helper function, there are a lot of places where the error code is inherited
+ * The following rules apply:
+ *
+ * Return the first received error code (E_NOTIMPL is ignored)
+ * If no errors occur: return the first received non-error-code that isn't S_OK
+ */
+HRESULT updatehres( HRESULT original, HRESULT new )
+{
+    if (FAILED( original ) || new == E_NOTIMPL)
+        return original;
+
+    if (FAILED( new ) || original == S_OK)
+        return new;
+
+    return original;
+}
+
+/** Sends a message from a pin further to other, similar pins
+ * fnMiddle is called on each pin found further on the stream.
+ * fnEnd (can be NULL) is called when the message can't be sent any further (this is a renderer or source)
+ *
+ * If the pin given is an input pin, the message will be sent downstream to other input pins
+ * If the pin given is an output pin, the message will be sent upstream to other output pins
+ */
+static HRESULT SendFurther( IPin *from, SendPinFunc fnMiddle, LPVOID arg, SendPinFunc fnEnd )
+{
+    PIN_INFO pin_info;
+    ULONG amount = 0;
+    HRESULT hr = S_OK;
+    HRESULT hr_return = S_OK;
+    IEnumPins *enumpins = NULL;
+    BOOL foundend = TRUE;
+    PIN_DIRECTION from_dir;
+
+    IPin_QueryDirection( from, &from_dir );
+
+    hr = IPin_QueryInternalConnections( from, NULL, &amount );
+    if (hr != E_NOTIMPL && amount)
+        FIXME("Use QueryInternalConnections!\n");
+     hr = S_OK;
+
+    pin_info.pFilter = NULL;
+    hr = IPin_QueryPinInfo( from, &pin_info );
+    if (FAILED(hr))
+        goto out;
+
+    hr = IBaseFilter_EnumPins( pin_info.pFilter, &enumpins );
+    if (FAILED(hr))
+        goto out;
+
+    hr = IEnumPins_Reset( enumpins );
+    while (hr == S_OK) {
+        IPin *pin = NULL;
+        hr = IEnumPins_Next( enumpins, 1, &pin, NULL );
+        if (hr == VFW_E_ENUM_OUT_OF_SYNC)
+        {
+            hr = IEnumPins_Reset( enumpins );
+            continue;
+        }
+        if (pin)
+        {
+            PIN_DIRECTION dir;
+
+            IPin_QueryDirection( pin, &dir );
+            if (dir != from_dir)
+            {
+                IPin *connected = NULL;
+
+                foundend = FALSE;
+                IPin_ConnectedTo( pin, &connected );
+                if (connected)
+                {
+                    HRESULT hr_local;
+
+                    hr_local = fnMiddle( connected, arg );
+                    hr_return = updatehres( hr_return, hr_local );
+                    IPin_Release(connected);
+                }
+            }
+            IPin_Release( pin );
+        }
+    } while (hr == S_OK);
+    if (!foundend)
+        hr = hr_return;
+    else if (fnEnd) {
+        HRESULT hr_local;
+
+        hr_local = fnEnd( from, arg );
+        hr_return = updatehres( hr_return, hr_local );
+    }
+
+out:
+    if (pin_info.pFilter)
+        IBaseFilter_Release( pin_info.pFilter );
+    return hr;
+}
+
 static inline InputPin *impl_from_IMemInputPin( IMemInputPin *iface )
 {
     return (InputPin *)((char*)iface - FIELD_OFFSET(InputPin, lpVtblMemInput));
@@ -147,10 +246,12 @@ HRESULT InputPin_Construct(const PIN_INFO * pPinInfo, SAMPLEPROC pSampleProc, LP
     {
         pPinImpl->pin.lpVtbl = &InputPin_Vtbl;
         pPinImpl->lpVtblMemInput = &MemInputPin_Vtbl;
-        
+
         *ppPin = (IPin *)(&pPinImpl->pin.lpVtbl);
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
@@ -178,7 +279,8 @@ HRESULT InputPin_Init(const PIN_INFO * pPinInfo, SAMPLEPROC pSampleProc, LPVOID
     return S_OK;
 }
 
-HRESULT OutputPin_Init(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES * props, LPVOID pUserData, QUERYACCEPTPROC pQueryAccept, LPCRITICAL_SECTION pCritSec, OutputPin * pPinImpl)
+HRESULT OutputPin_Init(const PIN_INFO * pPinInfo, const ALLOCATOR_PROPERTIES * props, LPVOID pUserData,
+                       QUERYACCEPTPROC pQueryAccept, LPCRITICAL_SECTION pCritSec, OutputPin * pPinImpl)
 {
     TRACE("\n");
 
@@ -197,7 +299,7 @@ HRESULT OutputPin_Init(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES * props,
     pPinImpl->pConnectSpecific = OutputPin_ConnectSpecific;
     if (props)
     {
-        memcpy(&pPinImpl->allocProps, props, sizeof(pPinImpl->allocProps));
+        pPinImpl->allocProps = *props;
         if (pPinImpl->allocProps.cbAlign == 0)
             pPinImpl->allocProps.cbAlign = 1;
     }
@@ -227,10 +329,12 @@ HRESULT OutputPin_Construct(const PIN_INFO * pPinInfo, ALLOCATOR_PROPERTIES *pro
     if (SUCCEEDED(OutputPin_Init(pPinInfo, props, pUserData, pQueryAccept, pCritSec, pPinImpl)))
     {
         pPinImpl->pin.lpVtbl = &OutputPin_Vtbl;
-        
+
         *ppPin = (IPin *)(&pPinImpl->pin.lpVtbl);
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
@@ -274,7 +378,7 @@ HRESULT WINAPI IPinImpl_ConnectedTo(IPin * iface, IPin ** ppPin)
     HRESULT hr;
     IPinImpl *This = (IPinImpl *)iface;
 
-/*  TRACE("(%p)\n", ppPin);*/
+    TRACE("(%p)\n", ppPin);
 
     EnterCriticalSection(This->pCritSec);
     {
@@ -403,6 +507,10 @@ HRESULT WINAPI InputPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
         *ppv = (LPVOID)iface;
     else if (IsEqualIID(riid, &IID_IMemInputPin))
         *ppv = (LPVOID)&This->lpVtblMemInput;
+    else if (IsEqualIID(riid, &IID_IMediaSeeking))
+    {
+        return IBaseFilter_QueryInterface(This->pin.pinInfo.pFilter, &IID_IMediaSeeking, ppv);
+    }
 
     if (*ppv)
     {
@@ -483,36 +591,73 @@ HRESULT WINAPI InputPin_ReceiveConnection(IPin * iface, IPin * pReceivePin, cons
     return hr;
 }
 
+static HRESULT deliver_endofstream(IPin* pin, LPVOID unused)
+{
+    return IPin_EndOfStream( pin );
+}
+
 HRESULT WINAPI InputPin_EndOfStream(IPin * iface)
 {
-    TRACE("()\n");
+    FIXME("() stub\n");
 
-    return S_OK;
+    /* Should do an end of stream notification?
+     * Also, don't accept any more samples from now!
+     * TODO: Don't accept any more packets
+     */
+
+    return SendFurther( iface, deliver_endofstream, NULL, NULL );
+}
+
+static HRESULT deliver_beginflush(IPin* pin, LPVOID unused)
+{
+    return IPin_BeginFlush( pin );
 }
 
 HRESULT WINAPI InputPin_BeginFlush(IPin * iface)
 {
-    FIXME("()\n");
-    return E_NOTIMPL;
+    FIXME("() stub\n");
+
+    /* TODO: Drop all cached packets, and don't accept any more samples! */
+    return SendFurther( iface, deliver_beginflush, NULL, NULL );
+}
+
+static HRESULT deliver_endflush(IPin* pin, LPVOID unused)
+{
+    return IPin_EndFlush( pin );
 }
 
 HRESULT WINAPI InputPin_EndFlush(IPin * iface)
 {
-    FIXME("()\n");
-    return E_NOTIMPL;
+    FIXME("() stub\n");
+
+    /* TODO: Accept any samples again */
+    return SendFurther( iface, deliver_endflush, NULL, NULL );
+}
+
+typedef struct newsegmentargs
+{
+    REFERENCE_TIME tStart, tStop;
+    double rate;
+} newsegmentargs;
+
+static HRESULT deliver_newsegment(IPin *pin, LPVOID data)
+{
+    newsegmentargs *args = data;
+    return IPin_NewSegment(pin, args->tStart, args->tStop, args->rate);
 }
 
 HRESULT WINAPI InputPin_NewSegment(IPin * iface, REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
 {
     InputPin *This = (InputPin *)iface;
+    newsegmentargs args;
 
     TRACE("(%x%08x, %x%08x, %e)\n", (ULONG)(tStart >> 32), (ULONG)tStart, (ULONG)(tStop >> 32), (ULONG)tStop, dRate);
 
-    This->tStart = tStart;
-    This->tStop = tStop;
-    This->dRate = dRate;
+    args.tStart = This->tStart = tStart;
+    args.tStop = This->tStop = tStop;
+    args.rate = This->dRate = dRate;
 
-    return S_OK;
+    return SendFurther( iface, deliver_newsegment, &args, NULL );
 }
 
 static const IPinVtbl InputPin_Vtbl = 
@@ -662,6 +807,10 @@ HRESULT WINAPI OutputPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
         *ppv = (LPVOID)iface;
     else if (IsEqualIID(riid, &IID_IPin))
         *ppv = (LPVOID)iface;
+    else if (IsEqualIID(riid, &IID_IMediaSeeking))
+    {
+        return IBaseFilter_QueryInterface(This->pin.pinInfo.pFilter, &IID_IMediaSeeking, ppv);
+    }
 
     if (*ppv)
     {
@@ -1020,10 +1169,12 @@ HRESULT PullPin_Construct(const PIN_INFO * pPinInfo, SAMPLEPROC pSampleProc, LPV
     if (SUCCEEDED(PullPin_Init(pPinInfo, pSampleProc, pUserData, pQueryAccept, pCritSec, pPinImpl)))
     {
         pPinImpl->pin.lpVtbl = &PullPin_Vtbl;
-        
+
         *ppPin = (IPin *)(&pPinImpl->pin.lpVtbl);
         return S_OK;
     }
+
+    CoTaskMemFree(pPinImpl);
     return E_FAIL;
 }
 
@@ -1044,11 +1195,12 @@ HRESULT PullPin_Init(const PIN_INFO * pPinInfo, SAMPLEPROC pSampleProc, LPVOID p
     pPinImpl->pAlloc = NULL;
     pPinImpl->pReader = NULL;
     pPinImpl->hThread = NULL;
-    pPinImpl->hEventStateChanged = CreateEventW(NULL, FALSE, TRUE, NULL);
+    pPinImpl->hEventStateChanged = CreateEventW(NULL, TRUE, TRUE, NULL);
 
     pPinImpl->rtStart = 0;
     pPinImpl->rtCurrent = 0;
     pPinImpl->rtStop = ((LONGLONG)0x7fffffff << 32) | 0xffffffff;
+    pPinImpl->state = State_Stopped;
 
     return S_OK;
 }
@@ -1136,6 +1288,10 @@ HRESULT WINAPI PullPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
         *ppv = (LPVOID)iface;
     else if (IsEqualIID(riid, &IID_IPin))
         *ppv = (LPVOID)iface;
+    else if (IsEqualIID(riid, &IID_IMediaSeeking))
+    {
+        return IBaseFilter_QueryInterface(This->pin.pinInfo.pFilter, &IID_IMediaSeeking, ppv);
+    }
 
     if (*ppv)
     {
@@ -1182,8 +1338,11 @@ static void CALLBACK PullPin_Thread_Process(ULONG_PTR iface)
     ALLOCATOR_PROPERTIES allocProps;
 
     CoInitializeEx(NULL, COINIT_MULTITHREADED);
-    
+
+    EnterCriticalSection(This->pin.pCritSec);
     SetEvent(This->hEventStateChanged);
+    This->state = State_Running;
+    LeaveCriticalSection(This->pin.pCritSec);
 
     hr = IMemAllocator_GetProperties(This->pAlloc, &allocProps);
 
@@ -1192,7 +1351,7 @@ static void CALLBACK PullPin_Thread_Process(ULONG_PTR iface)
 
     TRACE("Start\n");
 
-    while (This->rtCurrent < This->rtStop && hr == S_OK)
+    while (This->rtCurrent < This->rtStop && hr == S_OK && !This->stop_playback)
     {
         /* FIXME: to improve performance by quite a bit this should be changed
          * so that one sample is processed while one sample is fetched. However,
@@ -1220,7 +1379,7 @@ static void CALLBACK PullPin_Thread_Process(ULONG_PTR iface)
             hr = IAsyncReader_Request(This->pReader, pSample, (ULONG_PTR)0);
 
         if (SUCCEEDED(hr))
-            hr = IAsyncReader_WaitForNext(This->pReader, 10000, &pSample, &dwUser);
+            hr = IAsyncReader_WaitForNext(This->pReader, 1000, &pSample, &dwUser);
 
         if (SUCCEEDED(hr))
         {
@@ -1240,10 +1399,27 @@ static void CALLBACK PullPin_Thread_Process(ULONG_PTR iface)
     }
 
     CoUninitialize();
-
+    EnterCriticalSection(This->pin.pCritSec);
+    This->state = State_Paused;
+    LeaveCriticalSection(This->pin.pCritSec);
     TRACE("End\n");
 }
 
+static void CALLBACK PullPin_Thread_Pause(ULONG_PTR iface)
+{
+    PullPin *This = (PullPin *)iface;
+
+    TRACE("(%p/%p)->()\n", This, (LPVOID)iface);
+
+    EnterCriticalSection(This->pin.pCritSec);
+    {
+        This->state = State_Paused;
+        SetEvent(This->hEventStateChanged);
+    }
+    LeaveCriticalSection(This->pin.pCritSec);
+}
+
+
 static void CALLBACK PullPin_Thread_Stop(ULONG_PTR iface)
 {
     PullPin *This = (PullPin *)iface;
@@ -1258,11 +1434,12 @@ static void CALLBACK PullPin_Thread_Stop(ULONG_PTR iface)
         This->hThread = NULL;
         if (FAILED(hr = IMemAllocator_Decommit(This->pAlloc)))
             ERR("Allocator decommit failed with error %x. Possible memory leak\n", hr);
+
+        SetEvent(This->hEventStateChanged);
+        This->state = State_Stopped;
     }
     LeaveCriticalSection(This->pin.pCritSec);
 
-    SetEvent(This->hEventStateChanged);
-
     IBaseFilter_Release(This->pin.pinInfo.pFilter);
 
     ExitThread(0);
@@ -1274,16 +1451,16 @@ HRESULT PullPin_InitProcessing(PullPin * This)
 
     TRACE("(%p)->()\n", This);
 
-    assert(!This->hThread);
-
     /* if we are connected */
     if (This->pAlloc)
     {
+        WaitForSingleObject(This->hEventStateChanged, INFINITE);
         EnterCriticalSection(This->pin.pCritSec);
+        if (This->state == State_Stopped)
         {
             DWORD dwThreadId;
             assert(!This->hThread);
-        
+
             /* AddRef the filter to make sure it and it's pins will be around
              * as long as the thread */
             IBaseFilter_AddRef(This->pin.pinInfo.pFilter);
@@ -1296,8 +1473,13 @@ HRESULT PullPin_InitProcessing(PullPin * This)
             }
 
             if (SUCCEEDED(hr))
+            {
                 hr = IMemAllocator_Commit(This->pAlloc);
+                This->state = State_Paused;
+                SetEvent(This->hEventStateChanged);
+            }
         }
+        else assert(This->hThread);
         LeaveCriticalSection(This->pin.pCritSec);
     }
 
@@ -1313,8 +1495,10 @@ HRESULT PullPin_StartProcessing(PullPin * This)
     if(This->pAlloc)
     {
         assert(This->hThread);
-        
+
+        PullPin_WaitForStateChange(This, INFINITE);
         ResetEvent(This->hEventStateChanged);
+        This->stop_playback = 0;
 
         if (!QueueUserAPC(PullPin_Thread_Process, This->hThread, (ULONG_PTR)This))
             return HRESULT_FROM_WIN32(GetLastError());
@@ -1325,8 +1509,21 @@ HRESULT PullPin_StartProcessing(PullPin * This)
 
 HRESULT PullPin_PauseProcessing(PullPin * This)
 {
-    /* make the processing function exit its loop */
-    This->rtStop = 0;
+    /* if we are connected */
+    TRACE("(%p)->()\n", This);
+    if(This->pAlloc)
+    {
+        assert(This->hThread);
+
+        PullPin_WaitForStateChange(This, INFINITE);
+        EnterCriticalSection(This->pin.pCritSec);
+        This->stop_playback = 1;
+        LeaveCriticalSection(This->pin.pCritSec);
+        ResetEvent(This->hEventStateChanged);
+
+        if (!QueueUserAPC(PullPin_Thread_Pause, This->hThread, (ULONG_PTR)This))
+            return HRESULT_FROM_WIN32(GetLastError());
+    }
 
     return S_OK;
 }
@@ -1334,14 +1531,13 @@ HRESULT PullPin_PauseProcessing(PullPin * This)
 HRESULT PullPin_StopProcessing(PullPin * This)
 {
     /* if we are connected */
-    if (This->pAlloc)
+    if (This->pAlloc && This->hThread)
     {
-        assert(This->hThread);
+        PullPin_WaitForStateChange(This, INFINITE);
 
+        This->stop_playback = 1;
         ResetEvent(This->hEventStateChanged);
 
-        PullPin_PauseProcessing(This);
-
         if (!QueueUserAPC(PullPin_Thread_Stop, This->hThread, (ULONG_PTR)This))
             return HRESULT_FROM_WIN32(GetLastError());
     }
@@ -1356,41 +1552,46 @@ HRESULT PullPin_WaitForStateChange(PullPin * This, DWORD dwMilliseconds)
     return S_OK;
 }
 
-HRESULT PullPin_Seek(PullPin * This, REFERENCE_TIME rtStart, REFERENCE_TIME rtStop)
-{
-    FIXME("(%p)->(%x%08x, %x%08x)\n", This, (LONG)(rtStart >> 32), (LONG)rtStart, (LONG)(rtStop >> 32), (LONG)rtStop);
-
-    PullPin_BeginFlush((IPin *)This);
-    /* FIXME: need critical section? */
-    This->rtStart = rtStart;
-    This->rtStop = rtStop;
-    PullPin_EndFlush((IPin *)This);
-
-    return S_OK;
-}
-
 HRESULT WINAPI PullPin_EndOfStream(IPin * iface)
 {
-    FIXME("(%p)->()\n", iface);
-    return E_NOTIMPL;
+    FIXME("(%p)->() stub\n", iface);
+
+    return SendFurther( iface, deliver_endofstream, NULL, NULL );
 }
 
 HRESULT WINAPI PullPin_BeginFlush(IPin * iface)
 {
-    FIXME("(%p)->()\n", iface);
-    return E_NOTIMPL;
+    PullPin *This = (PullPin *)iface;
+    FIXME("(%p)->() stub\n", iface);
+
+    SendFurther( iface, deliver_beginflush, NULL, NULL );
+
+    if (This->state == State_Running)
+        return PullPin_PauseProcessing(This);
+    return S_OK;
 }
 
 HRESULT WINAPI PullPin_EndFlush(IPin * iface)
 {
-    FIXME("(%p)->()\n", iface);
-    return E_NOTIMPL;
+    FILTER_STATE state;
+    PullPin *This = (PullPin *)iface;
+
+    FIXME("(%p)->() stub\n", iface);
+    SendFurther( iface, deliver_endflush, NULL, NULL );
+
+    return IBaseFilter_GetState(This->pin.pinInfo.pFilter, INFINITE, &state);
 }
 
 HRESULT WINAPI PullPin_NewSegment(IPin * iface, REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
 {
-    FIXME("(%p)->(%s, %s, %g)\n", iface, wine_dbgstr_longlong(tStart), wine_dbgstr_longlong(tStop), dRate);
-    return E_NOTIMPL;
+    newsegmentargs args;
+    FIXME("(%p)->(%s, %s, %g) stub\n", iface, wine_dbgstr_longlong(tStart), wine_dbgstr_longlong(tStop), dRate);
+
+    args.tStart = tStart;
+    args.tStop = tStop;
+    args.rate = dRate;
+
+    return SendFurther( iface, deliver_newsegment, &args, NULL );
 }
 
 static const IPinVtbl PullPin_Vtbl = 
