@@ -27,7 +27,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(richedit);
 static BOOL
 ME_MoveCursorChars(ME_TextEditor *editor, ME_Cursor *pCursor, int nRelOfs);
 
-
 void ME_GetSelection(ME_TextEditor *editor, int *from, int *to)
 {
   *from = ME_GetCursorOfs(editor, 0);
@@ -59,7 +58,7 @@ int ME_GetTextLengthEx(ME_TextEditor *editor, const GETTEXTLENGTHEX *how)
   length = ME_GetTextLength(editor);
 
   if ((GetWindowLongW(editor->hWnd, GWL_STYLE) & ES_MULTILINE) && (how->flags & GTL_USECRLF))
-    length += editor->nParagraphs;
+    length += editor->nParagraphs - 1;
   
   if (how->flags & GTL_NUMBYTES)
   {
@@ -203,19 +202,21 @@ ME_GetCursorCoordinates(ME_TextEditor *editor, ME_Cursor *pCursor,
           pSizeRun = run = tmp;
           assert(run);
           assert(run->type == diRun);
-          sz = ME_GetRunSize(&c, &para->member.para, &run->member.run, ME_StrLen(run->member.run.strText));
+          sz = ME_GetRunSize(&c, &para->member.para,
+                             &run->member.run, ME_StrLen(run->member.run.strText),
+                             row->member.row.nLMargin);
         }
       }
       if (pCursor->nOffset && !(run->member.run.nFlags & MERF_SKIPPED)) {
-        sz = ME_GetRunSize(&c, &para->member.para, &run->member.run, pCursor->nOffset);
+        sz = ME_GetRunSize(&c, &para->member.para, &run->member.run, pCursor->nOffset,
+                           row->member.row.nLMargin);
       }
 
       *height = pSizeRun->member.run.nAscent + pSizeRun->member.run.nDescent;
       *x = run->member.run.pt.x + sz.cx;
       *y = para->member.para.nYPos + row->member.row.nBaseline + pSizeRun->member.run.pt.y - pSizeRun->member.run.nAscent - ME_GetYScrollPos(editor);
-      
-      ME_DestroyContext(&c);
-      ReleaseDC(editor->hWnd, hDC);
+
+      ME_DestroyContext(&c, editor->hWnd);
       return;
     }
   }
@@ -394,22 +395,39 @@ ME_InternalInsertTextFromCursor(ME_TextEditor *editor, int nCursor,
 }
 
 
-/* FIXME this is temporary, just to have something to test how bad graphics handler is */
-void ME_InsertGraphicsFromCursor(ME_TextEditor *editor, int nCursor)
+void ME_InsertOLEFromCursor(ME_TextEditor *editor, const REOBJECT* reo, int nCursor)
 {
-  ME_Style *pStyle = ME_GetInsertStyle(editor, nCursor);
-  WCHAR space = ' ';
+  ME_Style              *pStyle = ME_GetInsertStyle(editor, nCursor);
+  ME_DisplayItem        *di;
+  WCHAR                 space = ' ';
   
   /* FIXME no no no */
   if (ME_IsSelection(editor))
     ME_DeleteSelection(editor);
 
-  ME_InternalInsertTextFromCursor(editor, nCursor, &space, 1, pStyle,
-                                  MERF_GRAPHICS);
+  di = ME_InternalInsertTextFromCursor(editor, nCursor, &space, 1, pStyle,
+                                       MERF_GRAPHICS);
+  di->member.run.ole_obj = ALLOC_OBJ(*reo);
+  ME_CopyReObject(di->member.run.ole_obj, reo);
   ME_SendSelChange(editor);
 }
 
 
+void ME_InsertEndRowFromCursor(ME_TextEditor *editor, int nCursor)
+{
+  ME_Style              *pStyle = ME_GetInsertStyle(editor, nCursor);
+  ME_DisplayItem        *di;
+  WCHAR                 space = ' ';
+
+  /* FIXME no no no */
+  if (ME_IsSelection(editor))
+    ME_DeleteSelection(editor);
+
+  di = ME_InternalInsertTextFromCursor(editor, nCursor, &space, 1, pStyle,
+                                       MERF_ENDROW);
+  ME_SendSelChange(editor);
+}
+
 void
 ME_InsertTableCellFromCursor(ME_TextEditor *editor, int nCursor)
 {
@@ -485,6 +503,22 @@ void ME_InsertTextFromCursor(ME_TextEditor *editor, int nCursor,
         continue;
       }
     }
+    /* handle special \r\r\n sequence (richedit 2.x and higher only) */
+    if (!editor->bEmulateVersion10 && pos-str < len-2 && pos[0] == '\r' && pos[1] == '\r' && pos[2] == '\n') {
+      WCHAR space = ' ';
+
+      if (pos!=str)
+        ME_InternalInsertTextFromCursor(editor, nCursor, str, pos-str, style, 0);
+
+      ME_InternalInsertTextFromCursor(editor, nCursor, &space, 1, style, 0);
+
+      pos+=3;
+      if(pos-str <= len) {
+        len -= pos - str;
+        str = pos;
+        continue;
+      }
+    }
     if (pos-str < len) {   /* handle EOLs */
       ME_DisplayItem *tp, *end_run;
       ME_Style *tmp_style;
@@ -685,107 +719,87 @@ ME_SelectWord(ME_TextEditor *editor)
 int ME_GetCursorOfs(ME_TextEditor *editor, int nCursor)
 {
   ME_Cursor *pCursor = &editor->pCursors[nCursor];
-  
   return ME_GetParagraph(pCursor->pRun)->member.para.nCharOfs
     + pCursor->pRun->member.run.nCharOfs + pCursor->nOffset;
 }
 
-int ME_FindPixelPos(ME_TextEditor *editor, int x, int y, ME_Cursor *result, BOOL *is_eol)
+static void ME_FindPixelPos(ME_TextEditor *editor, int x, int y, ME_Cursor *result, BOOL *is_eol)
 {
   ME_DisplayItem *p = editor->pBuffer->pFirst->member.para.next_para;
+  ME_DisplayItem *last = NULL;
   int rx = 0;
-  
+
   if (is_eol)
     *is_eol = 0;
 
-  while(p != editor->pBuffer->pLast)
+  /* find paragraph */
+  for (; p != editor->pBuffer->pLast; p = p->member.para.next_para)
   {
-    if (p->type == diParagraph)
+    assert(p->type == diParagraph);
+    if (y < p->member.para.nYPos + p->member.para.nHeight)
     {
-      int ry = y - p->member.para.nYPos;
-      if (ry < 0)
-      {
-        result->pRun = ME_FindItemFwd(p, diRun);
-        result->nOffset = 0;
-        return 0;
-      }
-      if (ry >= p->member.para.nHeight)
-      {
-        p = p->member.para.next_para;
-        continue;
-      }
+      y -= p->member.para.nYPos;
       p = ME_FindItemFwd(p, diStartRow);
-      y = ry;
-      continue;
+      break;
     }
-    if (p->type == diStartRow)
+  }
+  /* find row */
+  for (; p != editor->pBuffer->pLast; )
+  {
+    ME_DisplayItem *pp;
+    assert(p->type == diStartRow);
+    if (y < p->member.row.nYPos + p->member.row.nHeight)
     {
-      int ry = y - p->member.row.nYPos;
-      if (ry < 0)
-        return 0;
-      if (ry >= p->member.row.nHeight)
-      {
-        p = ME_FindItemFwd(p, diStartRowOrParagraphOrEnd);
-        if (p->type != diStartRow)
-          return 0;
-        continue;
-      }
-      p = ME_FindItemFwd(p, diRun);
-      continue;
+        p = ME_FindItemFwd(p, diRun);
+        break;
     }
-    if (p->type == diRun)
+    pp = ME_FindItemFwd(p, diStartRowOrParagraphOrEnd);
+    if (pp->type != diStartRow)
     {
-      ME_DisplayItem *pp;
+        p = ME_FindItemFwd(p, diRun);
+        break;
+    }
+    p = pp;
+  }
+  for (; p != editor->pBuffer->pLast; p = p->next)
+  {
+    switch (p->type)
+    {
+    case diRun:
       rx = x - p->member.run.pt.x;
-      if (rx < 0)
-        rx = 0;
-      if (rx >= p->member.run.nWidth) /* not this run yet... find next item */
+      if (rx < p->member.run.nWidth)
       {
-        pp = p;
-        do {
-          p = p->next;
-          if (p->type == diRun)
-          {
-            rx = x - p->member.run.pt.x;
-            goto continue_search;
-          }
-          if (p->type == diStartRow)
-          {
-            p = ME_FindItemFwd(p, diRun);
-            if (is_eol)
-              *is_eol = 1;
-            rx = 0; /* FIXME not sure */
-            goto found_here;
-          }
-          if (p->type == diParagraph || p->type == diTextEnd)
-          {
-            rx = 0; /* FIXME not sure */
-            p = pp;
-            goto found_here;
-          }
-        } while(1);
-        continue;
+      found_here:
+        assert(p->type == diRun);
+        if ((p->member.run.nFlags & MERF_ENDPARA) || rx < 0)
+          rx = 0;
+        result->pRun = p;
+        result->nOffset = ME_CharFromPointCursor(editor, rx, &p->member.run);
+        if (editor->pCursors[0].nOffset == p->member.run.strText->nLen && rx)
+        {
+          result->pRun = ME_FindItemFwd(editor->pCursors[0].pRun, diRun);
+          result->nOffset = 0;
+        }
+        return;
       }
-    found_here:
-      if (p->member.run.nFlags & MERF_ENDPARA)
-        rx = 0;
-      result->pRun = p;
-      result->nOffset = ME_CharFromPointCursor(editor, rx, &p->member.run);
-      if (editor->pCursors[0].nOffset == p->member.run.strText->nLen && rx)
-      {
-        result->pRun = ME_FindItemFwd(editor->pCursors[0].pRun, diRun);
-        result->nOffset = 0;
-      }
-      return 1;
+      break;
+    case diStartRow:
+      p = ME_FindItemFwd(p, diRun);
+      if (is_eol) *is_eol = 1;
+      rx = 0; /* FIXME not sure */
+      goto found_here;
+    case diParagraph:
+    case diTextEnd:
+      rx = 0; /* FIXME not sure */
+      p = last;
+      goto found_here;
+    default: assert(0);
     }
-    assert(0);
-  continue_search:
-    ;
+    last = p;
   }
   result->pRun = ME_FindItemBack(p, diRun);
   result->nOffset = 0;
   assert(result->pRun->member.run.nFlags & MERF_ENDPARA);
-  return 0;
 }
 
 
@@ -817,25 +831,64 @@ void ME_LButtonDown(ME_TextEditor *editor, int x, int y)
   tmp_cursor = editor->pCursors[0];
   is_selection = ME_IsSelection(editor);
 
-  ME_FindPixelPos(editor, x, y, &editor->pCursors[0], &editor->bCaretAtEnd);
-  
-  if (GetKeyState(VK_SHIFT)>=0)
+  if (x >= editor->selofs)
   {
-    editor->pCursors[1] = editor->pCursors[0];
-  }
-  else
-  {
-    if (!is_selection) {
+    ME_FindPixelPos(editor, x, y, &editor->pCursors[0], &editor->bCaretAtEnd);
+    if (GetKeyState(VK_SHIFT)>=0)
+    {
+      editor->pCursors[1] = editor->pCursors[0];
+    }
+    else if (!is_selection) {
       editor->pCursors[1] = tmp_cursor;
       is_selection = 1;
     }
+
+    ME_InvalidateSelection(editor);
+    HideCaret(editor->hWnd);
+    ME_MoveCaret(editor);
+    ShowCaret(editor->hWnd);
+    ME_ClearTempStyle(editor);
+    ME_SendSelChange(editor);
+  }
+  else
+  {
+    ME_DisplayItem *pRow;
+
+    editor->linesel = 1;
+    editor->sely = y;
+    /* Set pCursors[0] to beginning of line */
+    ME_FindPixelPos(editor, x, y, &editor->pCursors[1], &editor->bCaretAtEnd);
+    /* Set pCursors[1] to end of line */
+    pRow = ME_FindItemFwd(editor->pCursors[1].pRun, diStartRowOrParagraphOrEnd);
+    assert(pRow);
+    /* pCursor[0] is the position where the cursor will be drawn,
+     * pCursor[1] is the other end of the selection range
+     * pCursor[2] and [3] are backups of [0] and [1] so I
+     * don't have to look them up again
+     */
+
+    if (pRow->type == diStartRow) {
+      /* FIXME WTF was I thinking about here ? */
+      ME_DisplayItem *pRun = ME_FindItemFwd(pRow, diRun);
+      assert(pRun);
+      editor->pCursors[0].pRun = pRun;
+      editor->pCursors[0].nOffset = 0;
+      editor->bCaretAtEnd = 1;
+    } else {
+      editor->pCursors[0].pRun = ME_FindItemBack(pRow, diRun);
+      assert(editor->pCursors[0].pRun && editor->pCursors[0].pRun->member.run.nFlags & MERF_ENDPARA);
+      editor->pCursors[0].nOffset = 0;
+      editor->bCaretAtEnd = 0;
+    }
+    editor->pCursors[2] = editor->pCursors[0];
+    editor->pCursors[3] = editor->pCursors[1];
+    ME_InvalidateSelection(editor);
+    HideCaret(editor->hWnd);
+    ME_MoveCaret(editor);
+    ShowCaret(editor->hWnd);
+    ME_ClearTempStyle(editor);
+    ME_SendSelChange(editor);
   }
-  ME_InvalidateSelection(editor);
-  HideCaret(editor->hWnd);
-  ME_MoveCaret(editor);
-  ShowCaret(editor->hWnd);
-  ME_ClearTempStyle(editor);
-  ME_SendSelChange(editor);
 }
 
 void ME_MouseMove(ME_TextEditor *editor, int x, int y)
@@ -846,14 +899,33 @@ void ME_MouseMove(ME_TextEditor *editor, int x, int y)
 
   tmp_cursor = editor->pCursors[0];
   /* FIXME: do something with the return value of ME_FindPixelPos */
-  ME_FindPixelPos(editor, x, y, &tmp_cursor, &editor->bCaretAtEnd);
-  
-  if (tmp_cursor.pRun == editor->pCursors[0].pRun && 
-      tmp_cursor.nOffset == editor->pCursors[0].nOffset)
+  if (!editor->linesel)
+    ME_FindPixelPos(editor, x, y, &tmp_cursor, &editor->bCaretAtEnd);
+  else ME_FindPixelPos(editor, (y > editor->sely) * editor->rcFormat.right, y, &tmp_cursor, &editor->bCaretAtEnd);
+
+  if (!memcmp(&tmp_cursor, editor->pCursors, sizeof(tmp_cursor)))
     return;
-  
+
   ME_InvalidateSelection(editor);
-  editor->pCursors[0] = tmp_cursor;
+  if (!editor->linesel)
+    editor->pCursors[0] = tmp_cursor;
+  else if (!memcmp(&tmp_cursor, editor->pCursors+2, sizeof(tmp_cursor)) ||
+           !memcmp(&tmp_cursor, editor->pCursors+3, sizeof(tmp_cursor)))
+  {
+    editor->pCursors[0] = editor->pCursors[2];
+    editor->pCursors[1] = editor->pCursors[3];
+  }
+  else if (y < editor->sely)
+  {
+    editor->pCursors[0] = tmp_cursor;
+    editor->pCursors[1] = editor->pCursors[2];
+  }
+  else
+  {
+    editor->pCursors[0] = tmp_cursor;
+    editor->pCursors[1] = editor->pCursors[3];
+  }
+
   HideCaret(editor->hWnd);
   ME_MoveCaret(editor);
   ME_InvalidateSelection(editor);
@@ -1191,11 +1263,11 @@ ME_Style *ME_GetSelectionInsertStyle(ME_TextEditor *editor)
 {
   ME_Style *style;
   int from, to;
-  ME_Cursor c;
-  
+
   ME_GetSelection(editor, &from, &to);
-  ME_CursorFromCharOfs(editor, from, &c);
   if (from != to) {
+    ME_Cursor c;
+    ME_CursorFromCharOfs(editor, from, &c);
     style = c.pRun->member.run.style;
     ME_AddRefStyle(style); /* ME_GetInsertStyle has already done that */
   }
@@ -1225,7 +1297,6 @@ void ME_SendSelChange(ME_TextEditor *editor)
   SendMessageW(GetParent(editor->hWnd), WM_NOTIFY, sc.nmhdr.idFrom, (LPARAM)&sc);
 }
 
-
 BOOL
 ME_ArrowKey(ME_TextEditor *editor, int nVKey, BOOL extend, BOOL ctrl)
 {
