@@ -267,9 +267,9 @@ void ME_JoinRuns(ME_TextEditor *editor, ME_DisplayItem *p)
  * Splits a run into two in a given place. It also updates the screen position
  * and size (extent) of the newly generated runs.  
  */    
-ME_DisplayItem *ME_SplitRun(ME_Context *c, ME_DisplayItem *item, int nVChar)
+ME_DisplayItem *ME_SplitRun(ME_WrapContext *wc, ME_DisplayItem *item, int nVChar)
 {
-  ME_TextEditor *editor = c->editor;
+  ME_TextEditor *editor = wc->context->editor;
   ME_DisplayItem *item2 = NULL;
   ME_Run *run, *run2;
   ME_Paragraph *para = &ME_GetParagraph(item)->member.para;
@@ -291,8 +291,8 @@ ME_DisplayItem *ME_SplitRun(ME_Context *c, ME_DisplayItem *item, int nVChar)
 
   run2 = &item2->member.run;
 
-  ME_CalcRunExtent(c, para, run);
-  ME_CalcRunExtent(c, para, run2);
+  ME_CalcRunExtent(wc->context, para, wc->nRow ? wc->nLeftMargin : wc->nFirstMargin, run);
+  ME_CalcRunExtent(wc->context, para, wc->nRow ? wc->nLeftMargin : wc->nFirstMargin, run2);
 
   run2->pt.x = run->pt.x+run->nWidth;
   run2->pt.y = run->pt.y;
@@ -359,6 +359,7 @@ ME_DisplayItem *ME_MakeRun(ME_Style *s, ME_String *strData, int nFlags)
 {
   ME_DisplayItem *item = ME_MakeDI(diRun);
   item->member.run.style = s;
+  item->member.run.ole_obj = NULL;
   item->member.run.strText = strData;
   item->member.run.nFlags = nFlags;
   item->member.run.nCharOfs = -1;
@@ -467,19 +468,6 @@ void ME_UpdateRunFlags(ME_TextEditor *editor, ME_Run *run)
     run->nFlags &= ~(MERF_WHITESPACE | MERF_STARTWHITE | MERF_ENDWHITE);
 }
 
-/******************************************************************************
- * ME_GetGraphicsSize
- * 
- * Sets run extent for graphics runs. This functionality is just a placeholder
- * for future OLE object support, and will be removed.
- */     
-void ME_GetGraphicsSize(ME_TextEditor *editor, ME_Run *run, SIZE *pSize)
-{
-  assert(run->nFlags & MERF_GRAPHICS);
-  pSize->cx = 64;
-  pSize->cy = 64;
-}
-
 /******************************************************************************
  * ME_CharFromPoint
  * 
@@ -487,11 +475,10 @@ void ME_GetGraphicsSize(ME_TextEditor *editor, ME_Run *run, SIZE *pSize)
  * pixel horizontal position. This version rounds left (ie. if the second
  * character is at pixel position 8, then for cx=0..7 it returns 0).  
  */     
-int ME_CharFromPoint(ME_TextEditor *editor, int cx, ME_Run *run)
+int ME_CharFromPoint(ME_Context *c, int cx, ME_Run *run)
 {
   int fit = 0;
   HGDIOBJ hOldFont;
-  HDC hDC;
   SIZE sz;
   if (!run->strText->nLen)
     return 0;
@@ -505,29 +492,28 @@ int ME_CharFromPoint(ME_TextEditor *editor, int cx, ME_Run *run)
   if (run->nFlags & MERF_GRAPHICS)
   {
     SIZE sz;
-    ME_GetGraphicsSize(editor, run, &sz);
+    ME_GetOLEObjectSize(c, run, &sz);
     if (cx < sz.cx)
       return 0;
     return 1;
   }
-  hDC = GetDC(editor->hWnd);
-  hOldFont = ME_SelectStyleFont(editor, hDC, run->style);
+  hOldFont = ME_SelectStyleFont(c, run->style);
   
-  if (editor->cPasswordMask)
+  if (c->editor->cPasswordMask)
   {
-    ME_String *strMasked = ME_MakeStringR(editor->cPasswordMask,ME_StrVLen(run->strText));
-    GetTextExtentExPointW(hDC, strMasked->szData, run->strText->nLen,
+    ME_String *strMasked = ME_MakeStringR(c->editor->cPasswordMask,ME_StrVLen(run->strText));
+    GetTextExtentExPointW(c->hDC, strMasked->szData, run->strText->nLen,
       cx, &fit, NULL, &sz);
     ME_DestroyString(strMasked);
   }
   else
   {
-    GetTextExtentExPointW(hDC, run->strText->szData, run->strText->nLen,
+    GetTextExtentExPointW(c->hDC, run->strText->szData, run->strText->nLen,
       cx, &fit, NULL, &sz);
   }
   
-  ME_UnselectStyleFont(editor, hDC, run->style, hOldFont);
-  ReleaseDC(editor->hWnd, hDC);
+  ME_UnselectStyleFont(c, run->style, hOldFont);
+
   return fit;
 }
 
@@ -548,8 +534,8 @@ int ME_CharFromPointCursor(ME_TextEditor *editor, int cx, ME_Run *run)
   /* This could point to either the run's real text, or it's masked form in a password control */
 	
   int fit = 0, fit1 = 0;
+  ME_Context c;
   HGDIOBJ hOldFont;
-  HDC hDC;
   SIZE sz, sz2, sz3;
   if (!run->strText->nLen)
     return 0;
@@ -560,31 +546,32 @@ int ME_CharFromPointCursor(ME_TextEditor *editor, int cx, ME_Run *run)
       return 0;
     return 1;
   }
+  ME_InitContext(&c, editor, GetDC(editor->hWnd));
   if (run->nFlags & MERF_GRAPHICS)
   {
     SIZE sz;
-    ME_GetGraphicsSize(editor, run, &sz);
+    ME_GetOLEObjectSize(&c, run, &sz);
+    ME_DestroyContext(&c, editor->hWnd);
     if (cx < sz.cx/2)
       return 0;
     return 1;
   }
-  
+
   if (editor->cPasswordMask)
     strRunText = ME_MakeStringR(editor->cPasswordMask,ME_StrVLen(run->strText));
   else
     strRunText = run->strText;
 
-  hDC = GetDC(editor->hWnd);
-  hOldFont = ME_SelectStyleFont(editor, hDC, run->style);
-  GetTextExtentExPointW(hDC, strRunText->szData, strRunText->nLen,
-    cx, &fit, NULL, &sz);
+  hOldFont = ME_SelectStyleFont(&c, run->style);
+  GetTextExtentExPointW(c.hDC, strRunText->szData, strRunText->nLen,
+                        cx, &fit, NULL, &sz);
   if (fit != strRunText->nLen)
   {
     int chars = 1;
 
-    GetTextExtentPoint32W(hDC, strRunText->szData, fit, &sz2);
+    GetTextExtentPoint32W(c.hDC, strRunText->szData, fit, &sz2);
     fit1 = ME_StrRelPos(strRunText, fit, &chars);
-    GetTextExtentPoint32W(hDC, strRunText->szData, fit1, &sz3);
+    GetTextExtentPoint32W(c.hDC, strRunText->szData, fit1, &sz3);
     if (cx >= (sz2.cx+sz3.cx)/2)
       fit = fit1;
   }
@@ -592,11 +579,24 @@ int ME_CharFromPointCursor(ME_TextEditor *editor, int cx, ME_Run *run)
   if (editor->cPasswordMask)
     ME_DestroyString(strRunText);
   
-  ME_UnselectStyleFont(editor, hDC, run->style, hOldFont);
-  ReleaseDC(editor->hWnd, hDC);
+  ME_UnselectStyleFont(&c, run->style, hOldFont);
+  ME_DestroyContext(&c, editor->hWnd);
   return fit;
 }
 
+/******************************************************************************
+ * ME_GetTextExtent
+ *
+ * Finds a width and a height of the text using a specified style
+ */
+static void ME_GetTextExtent(ME_Context *c, LPCWSTR szText, int nChars, ME_Style *s, SIZE *size)
+{
+  HGDIOBJ hOldFont;
+  hOldFont = ME_SelectStyleFont(c, s);
+  GetTextExtentPoint32W(c->hDC, szText, nChars, size);
+  ME_UnselectStyleFont(c, s, hOldFont);
+}
+
 /******************************************************************************
  * ME_PointFromChar
  * 
@@ -606,46 +606,31 @@ int ME_CharFromPointCursor(ME_TextEditor *editor, int cx, ME_Run *run)
 int ME_PointFromChar(ME_TextEditor *editor, ME_Run *pRun, int nOffset)
 {
   SIZE size;
-  HDC hDC = GetDC(editor->hWnd);
-  HGDIOBJ hOldFont;
+  ME_Context c;
   ME_String *strRunText;
   /* This could point to either the run's real text, or it's masked form in a password control */
 
+  ME_InitContext(&c, editor, GetDC(editor->hWnd));
   if (pRun->nFlags & MERF_GRAPHICS)
   {
-    if (!nOffset) return 0;
-    ME_GetGraphicsSize(editor, pRun, &size);
-    return 1;
+    if (nOffset)
+      ME_GetOLEObjectSize(&c, pRun, &size);
+    ReleaseDC(editor->hWnd, c.hDC);
+    return nOffset != 0;
   }
   
    if (editor->cPasswordMask)
     strRunText = ME_MakeStringR(editor->cPasswordMask,ME_StrVLen(pRun->strText));
   else
     strRunText = pRun->strText;
-  
-  hOldFont = ME_SelectStyleFont(editor, hDC, pRun->style);
-  GetTextExtentPoint32W(hDC, strRunText->szData, nOffset, &size);
-  ME_UnselectStyleFont(editor, hDC, pRun->style, hOldFont);
-  ReleaseDC(editor->hWnd, hDC);
+
+  ME_GetTextExtent(&c,  strRunText->szData, nOffset, pRun->style, &size);
+  ReleaseDC(editor->hWnd, c.hDC);
   if (editor->cPasswordMask)
     ME_DestroyString(strRunText);
   return size.cx;
 }
 
-/******************************************************************************
- * ME_GetTextExtent
- *
- * Finds a width and a height of the text using a specified style
- */
-static void ME_GetTextExtent(ME_Context *c, LPCWSTR szText, int nChars, ME_Style *s, SIZE *size)
-{
-  HDC hDC = c->hDC;
-  HGDIOBJ hOldFont;
-  hOldFont = ME_SelectStyleFont(c->editor, hDC, s);
-  GetTextExtentPoint32W(hDC, szText, nChars, size);
-  ME_UnselectStyleFont(c->editor, hDC, s, hOldFont);
-}
-
 /******************************************************************************
  * ME_GetRunSizeCommon
  * 
@@ -653,7 +638,7 @@ static void ME_GetTextExtent(ME_Context *c, LPCWSTR szText, int nChars, ME_Style
  * (nLen).
  */
 static SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run *run, int nLen,
-                                int *pAscent, int *pDescent)
+                                int startx, int *pAscent, int *pDescent)
 {
   SIZE size;
   int nMaxLen = ME_StrVLen(run->strText);
@@ -683,8 +668,8 @@ static SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run
   if (run->nFlags & MERF_TAB)
   {
     int pos = 0, i = 0, ppos;
-    int lpsx = GetDeviceCaps(c->hDC, LOGPIXELSX);
     PARAFORMAT2 *pFmt = para->pFmt;
+
     do {
       if (i < pFmt->cTabCount)
       {
@@ -695,9 +680,9 @@ static SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run
       {
         pos += 720-(pos%720);
       }
-      ppos = pos*lpsx/1440;
-      if (ppos>run->pt.x) {
-        size.cx = ppos - run->pt.x;
+      ppos = ME_twips2pointsX(c, pos);
+      if (ppos > startx + run->pt.x) {
+        size.cx = ppos - startx - run->pt.x;
         break;
       }
     } while(1);
@@ -706,7 +691,7 @@ static SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run
   }
   if (run->nFlags & MERF_GRAPHICS)
   {
-    ME_GetGraphicsSize(c->editor, run, &size);
+    ME_GetOLEObjectSize(c, run, &size);
     if (size.cy > *pAscent)
       *pAscent = size.cy;
     /* descent is unchanged */
@@ -714,9 +699,7 @@ static SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run
   }
   if (run->nFlags & MERF_CELL)
   {
-    int lpsx = GetDeviceCaps(c->hDC, LOGPIXELSX);
-
-    size.cx = run->pCell->nRightBoundary * lpsx / 1440 - run->pt.x;
+    size.cx = ME_twips2pointsX(c, run->pCell->nRightBoundary) - run->pt.x;
     return size;
   }
   return size;
@@ -728,10 +711,11 @@ static SIZE ME_GetRunSizeCommon(ME_Context *c, const ME_Paragraph *para, ME_Run
  * Finds width and height (but not ascent and descent) of a part of the run
  * up to given character.    
  */     
-SIZE ME_GetRunSize(ME_Context *c, const ME_Paragraph *para, ME_Run *run, int nLen)
+SIZE ME_GetRunSize(ME_Context *c, const ME_Paragraph *para,
+                   ME_Run *run, int nLen, int startx)
 {
   int asc, desc;
-  return ME_GetRunSizeCommon(c, para, run, nLen, &asc, &desc);
+  return ME_GetRunSizeCommon(c, para, run, nLen, startx, &asc, &desc);
 }
 
 /******************************************************************************
@@ -741,14 +725,15 @@ SIZE ME_GetRunSize(ME_Context *c, const ME_Paragraph *para, ME_Run *run, int nLe
  * is calculated based on whole row's ascent and descent anyway, so no need
  * to use it here.        
  */     
-void ME_CalcRunExtent(ME_Context *c, const ME_Paragraph *para, ME_Run *run)
+void ME_CalcRunExtent(ME_Context *c, const ME_Paragraph *para, int startx, ME_Run *run)
 {
   if (run->nFlags & MERF_HIDDEN)
     run->nWidth = 0;
   else
   {
     int nEnd = ME_StrVLen(run->strText);
-    SIZE size = ME_GetRunSizeCommon(c, para, run, nEnd, &run->nAscent, &run->nDescent);
+    SIZE size = ME_GetRunSizeCommon(c, para, run, nEnd, startx,
+                                    &run->nAscent, &run->nDescent);
     run->nWidth = size.cx;
     if (!size.cx)
       WARN("size.cx == 0\n");
@@ -938,8 +923,8 @@ void ME_GetCharFormat(ME_TextEditor *editor, int nFrom, int nTo, CHARFORMAT2W *p
 
   do {
     /* FIXME add more style feature comparisons */
-    int nAttribs = CFM_SIZE | CFM_FACE | CFM_COLOR;
-    int nEffects = CFM_BOLD | CFM_ITALIC | CFM_UNDERLINE;
+    int nAttribs = CFM_SIZE | CFM_FACE | CFM_COLOR | CFM_UNDERLINETYPE;
+    int nEffects = CFM_BOLD | CFM_ITALIC;
 
     run = ME_FindItemFwd(run, diRun);
 
@@ -957,11 +942,14 @@ void ME_GetCharFormat(ME_TextEditor *editor, int nFrom, int nTo, CHARFORMAT2W *p
     {
       if (!(tmp.dwMask & CFM_FACE))
         pFmt->dwMask &= ~CFM_FACE;
-      else if (lstrcmpW(pFmt->szFaceName, tmp.szFaceName))
+      else if (lstrcmpW(pFmt->szFaceName, tmp.szFaceName) ||
+          pFmt->bPitchAndFamily != tmp.bPitchAndFamily)
         pFmt->dwMask &= ~CFM_FACE;
     }
     if (pFmt->yHeight != tmp.yHeight)
       pFmt->dwMask &= ~CFM_SIZE;
+    if (pFmt->bUnderlineType != tmp.bUnderlineType)
+      pFmt->dwMask &= ~CFM_UNDERLINETYPE;
     if (pFmt->dwMask & CFM_COLOR)
     {
       if (!((pFmt->dwEffects&CFE_AUTOCOLOR) & (tmp.dwEffects&CFE_AUTOCOLOR)))
