@@ -62,15 +62,21 @@ ME_UndoItem *ME_AddUndoItem(ME_TextEditor *editor, ME_DIType type, const ME_Disp
       break;
     case diUndoSetParagraphFormat:
       assert(pdi);
-      CopyMemory(&pItem->member.para, &pdi->member.para, sizeof(ME_Paragraph));
+      pItem->member.para = pdi->member.para;
       pItem->member.para.pFmt = ALLOC_OBJ(PARAFORMAT2);
-      CopyMemory(pItem->member.para.pFmt, pdi->member.para.pFmt, sizeof(PARAFORMAT2));
+      *pItem->member.para.pFmt = *pdi->member.para.pFmt;
       break;
     case diUndoInsertRun:
       assert(pdi);
-      CopyMemory(&pItem->member.run, &pdi->member.run, sizeof(ME_Run));
+      pItem->member.run = pdi->member.run;
       pItem->member.run.strText = ME_StrDup(pItem->member.run.strText);
       ME_AddRefStyle(pItem->member.run.style);
+      if (pdi->member.run.ole_obj)
+      {
+        pItem->member.run.ole_obj = ALLOC_OBJ(*pItem->member.run.ole_obj);
+        ME_CopyReObject(pItem->member.run.ole_obj, pdi->member.run.ole_obj);
+      }
+      else pItem->member.run.ole_obj = NULL;
       break;
     case diUndoSetCharFormat:
     case diUndoSetDefaultCharFormat:
@@ -221,7 +227,7 @@ static void ME_PlayUndoItem(ME_TextEditor *editor, ME_DisplayItem *pItem)
       tmp.pRun = ME_SplitRunSimple(editor, tmp.pRun, tmp.nOffset);
     new_para = ME_SplitParagraph(editor, tmp.pRun, tmp.pRun->member.run.style);
     assert(pItem->member.para.pFmt->cbSize == sizeof(PARAFORMAT2));
-    CopyMemory(new_para->member.para.pFmt, pItem->member.para.pFmt, sizeof(PARAFORMAT2));
+    *new_para->member.para.pFmt = *pItem->member.para.pFmt;
     break;
   }
   default:
