@@ -36,6 +36,17 @@ static const char *progname;
 
 static HANDLE stop_event;
 
+static void (WINAPI *pNDRSContextMarshall2)(RPC_BINDING_HANDLE, NDR_SCONTEXT, void*, NDR_RUNDOWN, void*, ULONG);
+static NDR_SCONTEXT (WINAPI *pNDRSContextUnmarshall2)(RPC_BINDING_HANDLE, void*, ULONG, void*, ULONG);
+
+static void InitFunctionPointers(void)
+{
+    HMODULE hrpcrt4 = GetModuleHandleA("rpcrt4.dll");
+
+    pNDRSContextMarshall2 = (void *)GetProcAddress(hrpcrt4, "NDRSContextMarshall2");
+    pNDRSContextUnmarshall2 = (void *)GetProcAddress(hrpcrt4, "NDRSContextUnmarshall2");
+}
+
 void __RPC_FAR *__RPC_USER
 midl_user_allocate(size_t n)
 {
@@ -48,6 +59,14 @@ midl_user_free(void __RPC_FAR *p)
   free(p);
 }
 
+static char *
+xstrdup(const char *s)
+{
+  char *d = HeapAlloc(GetProcessHeap(), 0, strlen(s) + 1);
+  strcpy(d, s);
+  return d;
+}
+
 int
 s_int_return(void)
 {
@@ -84,6 +103,12 @@ s_str_length(const char *s)
   return strlen(s);
 }
 
+int
+s_str_t_length(str_t s)
+{
+  return strlen(s);
+}
+
 int
 s_cstr_length(const char *s, int n)
 {
@@ -263,6 +288,26 @@ s_square_puint(puint_t p)
   return n * n;
 }
 
+int
+s_sum_puints(puints_t *p)
+{
+  int sum = 0;
+  int i;
+  for (i = 0; i < p->n; ++i)
+    sum += atoi(p->ps[i]);
+  return sum;
+}
+
+int
+s_sum_cpuints(cpuints_t *p)
+{
+  int sum = 0;
+  int i;
+  for (i = 0; i < p->n; ++i)
+    sum += atoi(p->ps[i]);
+  return sum;
+}
+
 int
 s_dot_copy_vectors(vector_t u, vector_t v)
 {
@@ -288,6 +333,12 @@ s_square_encu(encu_t *eu)
   }
 }
 
+void
+s_check_se2(se_t *s)
+{
+  ok(s->f == E2, "check_se2\n");
+}
+
 int
 s_sum_parr(int *a[3])
 {
@@ -402,6 +453,206 @@ s_wstr_struct_len(wstr_struct_t *s)
   return lstrlenW(s->s);
 }
 
+int
+s_sum_doub_carr(doub_carr_t *dc)
+{
+  int i, j;
+  int sum = 0;
+  for (i = 0; i < dc->n; ++i)
+    for (j = 0; j < dc->a[i]->n; ++j)
+      sum += dc->a[i]->a[j];
+  return sum;
+}
+
+void
+s_make_pyramid_doub_carr(unsigned char n, doub_carr_t **dc)
+{
+  doub_carr_t *t;
+  int i, j;
+  t = MIDL_user_allocate(FIELD_OFFSET(doub_carr_t, a[n]));
+  t->n = n;
+  for (i = 0; i < n; ++i)
+  {
+    int v = i + 1;
+    t->a[i] = MIDL_user_allocate(FIELD_OFFSET(doub_carr_1_t, a[v]));
+    t->a[i]->n = v;
+    for (j = 0; j < v; ++j)
+      t->a[i]->a[j] = j + 1;
+  }
+  *dc = t;
+}
+
+unsigned
+s_hash_bstr(bstr_t b)
+{
+  short n = b[-1];
+  short *s = b;
+  unsigned hash = 0;
+  short i;
+  for (i = 0; i < n; ++i)
+    hash = 5 * hash + (unsigned) s[i];
+  return hash;
+}
+
+void
+s_get_name(name_t *name)
+{
+  const char bossman[] = "Jeremy White";
+  memcpy(name->name, bossman, min(name->size, sizeof(bossman)));
+  /* ensure nul-termination */
+  if (name->size < sizeof(bossman))
+    name->name[name->size - 1] = 0;
+}
+
+int
+s_sum_pcarr2(int n, int **pa)
+{
+  return s_sum_conf_array(*pa, n);
+}
+
+int
+s_sum_L1_norms(int n, vector_t *vs)
+{
+  int i;
+  int sum = 0;
+  for (i = 0; i < n; ++i)
+    sum += abs(vs[i].x) + abs(vs[i].y) + abs(vs[i].z);
+  return sum;
+}
+
+s123_t *
+s_get_s123(void)
+{
+  s123_t *s = MIDL_user_allocate(sizeof *s);
+  s->f1 = 1;
+  s->f2 = 2;
+  s->f3 = 3;
+  return s;
+}
+
+str_t
+s_get_filename(void)
+{
+    return (char *)__FILE__;
+}
+
+void
+s_context_handle_test(void)
+{
+    NDR_SCONTEXT h;
+    RPC_BINDING_HANDLE binding;
+    RPC_STATUS status;
+    unsigned char buf[20];
+    static RPC_SERVER_INTERFACE server_if =
+    {
+        sizeof(RPC_SERVER_INTERFACE),
+        {{0x00000000,0x4114,0x0704,{0x23,0x01,0x00,0x00,0x00,0x00,0x00,0x00}},{1,0}},
+        {{0x8a885d04,0x1ceb,0x11c9,{0x9f,0xe8,0x08,0x00,0x2b,0x10,0x48,0x60}},{2,0}},
+        NULL,
+        0,
+        0,
+        0,
+        0,
+        0,
+    };
+
+    binding = I_RpcGetCurrentCallHandle();
+    ok(binding != NULL, "I_RpcGetCurrentCallHandle returned NULL\n");
+
+    h = pNDRSContextUnmarshall2(binding, NULL, NDR_LOCAL_DATA_REPRESENTATION, NULL, 0);
+    ok(h != NULL, "NDRSContextUnmarshall2 returned NULL\n");
+
+    /* marshal a context handle with NULL userContext */
+    memset(buf, 0xcc, sizeof(buf));
+    pNDRSContextMarshall2(binding, h, buf, NULL, NULL, 0);
+    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%x\n", *(ULONG *)buf);
+    ok(UuidIsNil((UUID *)&buf[4], &status), "uuid should have been nil\n");
+
+    h = pNDRSContextUnmarshall2(binding, NULL, NDR_LOCAL_DATA_REPRESENTATION, NULL, 0);
+    ok(h != NULL, "NDRSContextUnmarshall2 returned NULL\n");
+
+    /* marshal a context handle with non-NULL userContext */
+    memset(buf, 0xcc, sizeof(buf));
+    h->userContext = (void *)0xdeadbeef;
+    pNDRSContextMarshall2(binding, h, buf, NULL, NULL, 0);
+    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%x\n", *(ULONG *)buf);
+    ok(!UuidIsNil((UUID *)&buf[4], &status), "uuid should not have been nil\n");
+
+    h = pNDRSContextUnmarshall2(binding, buf, NDR_LOCAL_DATA_REPRESENTATION, NULL, 0);
+    ok(h != NULL, "NDRSContextUnmarshall2 returned NULL\n");
+    ok(h->userContext == (void *)0xdeadbeef, "userContext of interface didn't unmarshal properly: %p\n", h->userContext);
+
+    /* marshal a context handle with an interface specified */
+    h = pNDRSContextUnmarshall2(binding, NULL, NDR_LOCAL_DATA_REPRESENTATION, &server_if.InterfaceId, 0);
+    ok(h != NULL, "NDRSContextUnmarshall2 returned NULL\n");
+
+    memset(buf, 0xcc, sizeof(buf));
+    h->userContext = (void *)0xcafebabe;
+    pNDRSContextMarshall2(binding, h, buf, NULL, &server_if.InterfaceId, 0);
+    ok(*(ULONG *)buf == 0, "attributes should have been set to 0 instead of 0x%x\n", *(ULONG *)buf);
+    ok(!UuidIsNil((UUID *)&buf[4], &status), "uuid should not have been nil\n");
+
+    h = pNDRSContextUnmarshall2(binding, buf, NDR_LOCAL_DATA_REPRESENTATION, &server_if.InterfaceId, 0);
+    ok(h != NULL, "NDRSContextUnmarshall2 returned NULL\n");
+    ok(h->userContext == (void *)0xcafebabe, "userContext of interface didn't unmarshal properly: %p\n", h->userContext);
+
+    /* test same interface data, but different pointer */
+    /* raises ERROR_INVALID_HANDLE exception */
+    if (0)
+    {
+        RPC_SERVER_INTERFACE server_if_clone = server_if;
+
+        pNDRSContextUnmarshall2(binding, buf, NDR_LOCAL_DATA_REPRESENTATION, &server_if_clone.InterfaceId, 0);
+    }
+
+    /* test different interface data, but different pointer */
+    /* raises ERROR_INVALID_HANDLE exception */
+    if (0)
+    {
+        static RPC_SERVER_INTERFACE server_if2 =
+        {
+            sizeof(RPC_SERVER_INTERFACE),
+            {{0x00000000,0x4114,0x0704,{0x23,0x01,0x00,0x00,0x00,0x00,0x00,0x00}},{1,0}},
+            {{0x8a885d04,0x1ceb,0x11c9,{0x9f,0xe8,0x08,0x00,0x2b,0x10,0x48,0x60}},{2,0}},
+            NULL,
+            0,
+            0,
+            0,
+            0,
+            0,
+        };
+        pNDRSContextMarshall2(binding, h, buf, NULL, &server_if.InterfaceId, 0);
+
+        pNDRSContextUnmarshall2(binding, buf, NDR_LOCAL_DATA_REPRESENTATION, &server_if2.InterfaceId, 0);
+    }
+}
+
+void
+s_get_5numbers(int count, pints_t n[5])
+{
+    int i;
+    for (i = 0; i < count; i++)
+    {
+        n[i].pi = midl_user_allocate(sizeof(*n[i].pi));
+        *n[i].pi = i;
+        n[i].ppi = NULL;
+        n[i].pppi = NULL;
+    }
+}
+
+void
+s_get_numbers(int length, int size, pints_t n[])
+{
+    int i;
+    for (i = 0; i < length; i++)
+    {
+        n[i].pi = midl_user_allocate(sizeof(*n[i].pi));
+        *n[i].pi = i;
+        n[i].ppi = NULL;
+        n[i].pppi = NULL;
+    }
+}
+
 void
 s_stop(void)
 {
@@ -416,25 +667,21 @@ make_cmdline(char buffer[MAX_PATH], const char *test)
   sprintf(buffer, "%s server %s", progname, test);
 }
 
-static int
+static void
 run_client(const char *test)
 {
   char cmdline[MAX_PATH];
   PROCESS_INFORMATION info;
   STARTUPINFOA startup;
-  DWORD exitcode;
 
   memset(&startup, 0, sizeof startup);
   startup.cb = sizeof startup;
 
   make_cmdline(cmdline, test);
   ok(CreateProcessA(NULL, cmdline, NULL, NULL, FALSE, 0L, NULL, NULL, &startup, &info), "CreateProcess\n");
-  ok(WaitForSingleObject(info.hProcess, 30000) == WAIT_OBJECT_0, "Child process termination\n");
-  ok(GetExitCodeProcess(info.hProcess, &exitcode), "GetExitCodeProcess\n");
+  winetest_wait_child_process( info.hProcess );
   ok(CloseHandle(info.hProcess), "CloseHandle\n");
   ok(CloseHandle(info.hThread), "CloseHandle\n");
-
-  return exitcode == 0;
 }
 
 static void
@@ -448,7 +695,7 @@ basic_tests(void)
   pvectors_t pvecs = {&vec1, &pvec2};
   sp_inner_t spi = {42};
   sp_t sp = {-13, &spi};
-  aligns_t aligns = {3, 4, 5, 6.0};
+  aligns_t aligns;
   pints_t pints;
   ptypes_t ptypes;
   padded_t padded;
@@ -463,6 +710,8 @@ basic_tests(void)
   int x;
   str_struct_t ss = {string};
   wstr_struct_t ws = {wstring};
+  str_t str;
+  se_t se;
 
   ok(int_return() == INT_CODE, "RPC int_return\n");
 
@@ -478,6 +727,7 @@ basic_tests(void)
   ok(x == 25, "RPC square_ref\n");
 
   ok(str_length(string) == strlen(string), "RPC str_length\n");
+  ok(str_t_length(string) == strlen(string), "RPC str_length\n");
   ok(dot_self(&a) == 59, "RPC dot_self\n");
 
   ok(str_struct_len(&ss) == lstrlenA(string), "RPC str_struct_len\n");
@@ -531,6 +781,14 @@ basic_tests(void)
   ok(enum_ord(E3) == 3, "RPC enum_ord\n");
   ok(enum_ord(E4) == 4, "RPC enum_ord\n");
 
+  se.f = E2;
+  check_se2(&se);
+
+  memset(&aligns, 0, sizeof(aligns));
+  aligns.c = 3;
+  aligns.i = 4;
+  aligns.s = 5;
+  aligns.d = 6.0;
   ok(sum_aligns(&aligns) == 18.0, "RPC sum_aligns\n");
 
   padded.i = -3;
@@ -557,6 +815,10 @@ basic_tests(void)
   ok(sum_bogus(&bogus) == 12, "RPC sum_bogus\n");
 
   check_null(NULL);
+
+  str = get_filename();
+  ok(!strcmp(str, __FILE__), "get_filename() returned %s instead of %s\n", str, __FILE__);
+  midl_user_free(str);
 }
 
 static void
@@ -606,6 +868,7 @@ null_list(void)
 {
   test_list_t *n = HeapAlloc(GetProcessHeap(), 0, sizeof *n);
   n->t = TL_NULL;
+  n->u.x = 0;
   return n;
 }
 
@@ -687,6 +950,38 @@ us_t_UserFree(ULONG *flags, us_t *pus)
   HeapFree(GetProcessHeap(), 0, pus->x);
 }
 
+ULONG __RPC_USER
+bstr_t_UserSize(ULONG *flags, ULONG start, bstr_t *b)
+{
+  return start + FIELD_OFFSET(wire_bstr_t, data[(*b)[-1]]);
+}
+
+unsigned char * __RPC_USER
+bstr_t_UserMarshal(ULONG *flags, unsigned char *buffer, bstr_t *b)
+{
+  wire_bstr_t *wb = (wire_bstr_t *) buffer;
+  wb->n = (*b)[-1];
+  memcpy(&wb->data, *b, wb->n * sizeof wb->data[0]);
+  return buffer + FIELD_OFFSET(wire_bstr_t, data[wb->n]);
+}
+
+unsigned char * __RPC_USER
+bstr_t_UserUnmarshal(ULONG *flags, unsigned char *buffer, bstr_t *b)
+{
+  wire_bstr_t *wb = (wire_bstr_t *) buffer;
+  short *data = HeapAlloc(GetProcessHeap(), 0, (wb->n + 1) * sizeof *data);
+  data[0] = wb->n;
+  memcpy(&data[1], wb->data, wb->n * sizeof data[1]);
+  *b = &data[1];
+  return buffer + FIELD_OFFSET(wire_bstr_t, data[wb->n]);
+}
+
+void __RPC_USER
+bstr_t_UserFree(ULONG *flags, bstr_t *b)
+{
+  HeapFree(GetProcessHeap(), 0, &((*b)[-1]));
+}
+
 static void
 pointer_tests(void)
 {
@@ -695,10 +990,42 @@ pointer_tests(void)
   test_list_t *list = make_list(make_list(make_list(null_list())));
   test_us_t tus = {{p1}};
   int *pa[4];
+  puints_t pus;
+  cpuints_t cpus;
+  short bstr_data[] = { 5, 'H', 'e', 'l', 'l', 'o' };
+  bstr_t bstr = &bstr_data[1];
+  name_t name;
+  void *buffer;
+  int *pa2;
+  s123_t *s123;
 
   ok(test_list_length(list) == 3, "RPC test_list_length\n");
   ok(square_puint(p1) == 121, "RPC square_puint\n");
-  todo_wine ok(square_test_us(&tus) == 121, "RPC square_test_us\n");
+  pus.n = 4;
+  pus.ps = HeapAlloc(GetProcessHeap(), 0, pus.n * sizeof pus.ps[0]);
+  pus.ps[0] = xstrdup("5");
+  pus.ps[1] = xstrdup("6");
+  pus.ps[2] = xstrdup("7");
+  pus.ps[3] = xstrdup("8");
+  ok(sum_puints(&pus) == 26, "RPC sum_puints\n");
+  HeapFree(GetProcessHeap(), 0, pus.ps[0]);
+  HeapFree(GetProcessHeap(), 0, pus.ps[1]);
+  HeapFree(GetProcessHeap(), 0, pus.ps[2]);
+  HeapFree(GetProcessHeap(), 0, pus.ps[3]);
+  HeapFree(GetProcessHeap(), 0, pus.ps);
+  cpus.n = 4;
+  cpus.ps = HeapAlloc(GetProcessHeap(), 0, cpus.n * sizeof cpus.ps[0]);
+  cpus.ps[0] = xstrdup("5");
+  cpus.ps[1] = xstrdup("6");
+  cpus.ps[2] = xstrdup("7");
+  cpus.ps[3] = xstrdup("8");
+  ok(sum_cpuints(&cpus) == 26, "RPC sum_puints\n");
+  HeapFree(GetProcessHeap(), 0, cpus.ps[0]);
+  HeapFree(GetProcessHeap(), 0, cpus.ps[1]);
+  HeapFree(GetProcessHeap(), 0, cpus.ps[2]);
+  HeapFree(GetProcessHeap(), 0, cpus.ps[3]);
+  HeapFree(GetProcessHeap(), 0, cpus.ps);
+  ok(square_test_us(&tus) == 121, "RPC square_test_us\n");
 
   pa[0] = &a[0];
   pa[1] = &a[1];
@@ -711,7 +1038,43 @@ pointer_tests(void)
   pa[3] = &a[3];
   ok(sum_pcarr(pa, 4) == 10, "RPC sum_pcarr\n");
 
+  ok(hash_bstr(bstr) == s_hash_bstr(bstr), "RPC hash_bstr_data\n");
+
   free_list(list);
+
+  name.size = 10;
+  name.name = buffer = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, name.size);
+  get_name(&name);
+  ok(name.name == buffer, "[in,out] pointer should have stayed as %p but instead changed to %p\n", name.name, buffer);
+  ok(!strcmp(name.name, "Jeremy Wh"), "name didn't unmarshall properly, expected \"Jeremy Wh\", but got \"%s\"\n", name.name);
+  HeapFree(GetProcessHeap(), 0, name.name);
+
+  pa2 = a;
+  ok(sum_pcarr2(4, &pa2) == 10, "RPC sum_pcarr2\n");
+
+  s123 = get_s123();
+  ok(s123->f1 == 1 && s123->f2 == 2 && s123->f3 == 3, "RPC get_s123\n");
+  MIDL_user_free(s123);
+}
+
+static int
+check_pyramid_doub_carr(doub_carr_t *dc)
+{
+  int i, j;
+  for (i = 0; i < dc->n; ++i)
+    for (j = 0; j < dc->a[i]->n; ++j)
+      if (dc->a[i]->a[j] != j + 1)
+        return FALSE;
+  return TRUE;
+}
+
+static void
+free_pyramid_doub_carr(doub_carr_t *dc)
+{
+  int i;
+  for (i = 0; i < dc->n; ++i)
+    MIDL_user_free(dc->a[i]);
+  MIDL_user_free(dc);
 }
 
 static void
@@ -730,6 +1093,9 @@ array_tests(void)
   cs_t *cs;
   int n;
   int ca[5] = {1, -2, 3, -4, 5};
+  doub_carr_t *dc;
+  int *pi;
+  pints_t api[5];
 
   ok(cstr_length(str1, sizeof str1) == strlen(str1), "RPC cstr_length\n");
 
@@ -765,22 +1131,59 @@ array_tests(void)
   cps.ca1 = &c[2];
   cps.n = 3;
   cps.ca2 = &c[3];
-  todo_wine ok(sum_cps(&cps) == 53, "RPC sum_cps\n");
+  ok(sum_cps(&cps) == 53, "RPC sum_cps\n");
 
   cpsc.a = 4;
   cpsc.b = 5;
   cpsc.c = 1;
   cpsc.ca = c;
-  todo_wine ok(sum_cpsc(&cpsc) == 6, "RPC sum_cpsc\n");
+  ok(sum_cpsc(&cpsc) == 6, "RPC sum_cpsc\n");
   cpsc.a = 4;
   cpsc.b = 5;
   cpsc.c = 0;
   cpsc.ca = c;
-  todo_wine ok(sum_cpsc(&cpsc) == 10, "RPC sum_cpsc\n");
+  ok(sum_cpsc(&cpsc) == 10, "RPC sum_cpsc\n");
 
   ok(sum_toplev_conf_2n(c, 3) == 15, "RPC sum_toplev_conf_2n\n");
   ok(sum_toplev_conf_cond(c, 5, 6, 1) == 10, "RPC sum_toplev_conf_cond\n");
   ok(sum_toplev_conf_cond(c, 5, 6, 0) == 15, "RPC sum_toplev_conf_cond\n");
+
+  dc = malloc(FIELD_OFFSET(doub_carr_t, a[2]));
+  dc->n = 2;
+  dc->a[0] = malloc(FIELD_OFFSET(doub_carr_1_t, a[3]));
+  dc->a[0]->n = 3;
+  dc->a[0]->a[0] = 5;
+  dc->a[0]->a[1] = 1;
+  dc->a[0]->a[2] = 8;
+  dc->a[1] = malloc(FIELD_OFFSET(doub_carr_1_t, a[2]));
+  dc->a[1]->n = 2;
+  dc->a[1]->a[0] = 2;
+  dc->a[1]->a[1] = 3;
+  ok(sum_doub_carr(dc) == 19, "RPC sum_doub_carr\n");
+  free(dc->a[0]);
+  free(dc->a[1]);
+  free(dc);
+
+  dc = NULL;
+  make_pyramid_doub_carr(4, &dc);
+  ok(check_pyramid_doub_carr(dc), "RPC make_pyramid_doub_carr\n");
+  free_pyramid_doub_carr(dc);
+
+  ok(sum_L1_norms(2, vs) == 21, "RPC sum_L1_norms\n");
+
+  memset(api, 0, sizeof(api));
+  pi = HeapAlloc(GetProcessHeap(), 0, sizeof(*pi));
+  *pi = -1;
+  api[0].pi = pi;
+  get_5numbers(1, api);
+  ok(api[0].pi == pi, "RPC varying array [out] pointer changed from %p to %p\n", pi, api[0].pi);
+  ok(*api[0].pi == 0, "pi unmarshalled incorrectly %d\n", *pi);
+
+  api[0].pi = pi;
+  get_numbers(1, 1, api);
+  ok(api[0].pi == pi, "RPC conformant varying array [out] pointer changed from %p to %p\n", pi, api[0].pi);
+  ok(*api[0].pi == 0, "pi unmarshalled incorrectly %d\n", *pi);
+  HeapFree(GetProcessHeap(), 0, pi);
 }
 
 static void
@@ -790,6 +1193,7 @@ run_tests(void)
   union_tests();
   pointer_tests();
   array_tests();
+  context_handle_test();
 }
 
 static void
@@ -843,10 +1247,10 @@ server(void)
   stop_event = CreateEvent(NULL, FALSE, FALSE, NULL);
   ok(stop_event != NULL, "CreateEvent failed\n");
 
-  ok(run_client("tcp_basic"), "tcp_basic client test failed\n");
+  run_client("tcp_basic");
 
   ok(RPC_S_OK == RpcServerUseProtseqEp(np, 0, pipe, NULL), "RpcServerUseProtseqEp\n");
-  ok(run_client("np_basic"), "np_basic client test failed\n");
+  run_client("np_basic");
 
   ok(WAIT_OBJECT_0 == WaitForSingleObject(stop_event, 60000), "WaitForSingleObject\n");
   todo_wine {
@@ -859,6 +1263,8 @@ START_TEST(server)
   int argc;
   char **argv;
 
+  InitFunctionPointers();
+
   argc = winetest_get_mainargs(&argv);
   progname = argv[0];
 
