@@ -24,11 +24,13 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winnls.h"
+#include "wincred.h"
 #include "rpc.h"
 #include "sspi.h"
 #include "lm.h"
 #include "secur32_priv.h"
 #include "hmac_md5.h"
+#include "wine/unicode.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(ntlm);
@@ -93,6 +95,42 @@ static SECURITY_STATUS SEC_ENTRY ntlm_QueryCredentialsAttributesW(
     return ret;
 }
 
+static char *ntlm_GetUsernameArg(LPCWSTR userW, INT userW_length)
+{
+    static const char username_arg[] = "--username=";
+    char *user;
+    int unixcp_size;
+
+    unixcp_size =  WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
+        userW, userW_length, NULL, 0, NULL, NULL) + sizeof(username_arg);
+    user = HeapAlloc(GetProcessHeap(), 0, unixcp_size);
+    if (!user) return NULL;
+    memcpy(user, username_arg, sizeof(username_arg) - 1);
+    WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, userW, userW_length,
+        user + sizeof(username_arg) - 1,
+        unixcp_size - sizeof(username_arg) + 1, NULL, NULL);
+    user[unixcp_size - 1] = '\0';
+    return user;
+}
+
+static char *ntlm_GetDomainArg(LPCWSTR domainW, INT domainW_length)
+{
+    static const char domain_arg[] = "--domain=";
+    char *domain;
+    int unixcp_size;
+
+    unixcp_size = WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
+        domainW, domainW_length, NULL, 0,  NULL, NULL) + sizeof(domain_arg);
+    domain = HeapAlloc(GetProcessHeap(), 0, unixcp_size);
+    if (!domain) return NULL;
+    memcpy(domain, domain_arg, sizeof(domain_arg) - 1);
+    WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, domainW,
+        domainW_length, domain + sizeof(domain_arg) - 1,
+        unixcp_size - sizeof(domain) + 1, NULL, NULL);
+    domain[unixcp_size - 1] = '\0';
+    return domain;
+}
+
 /***********************************************************************
  *              AcquireCredentialsHandleW
  */
@@ -129,54 +167,6 @@ static SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleW(
             break;
         case SECPKG_CRED_OUTBOUND:
             {
-                static const char username_arg[] = "--username=";
-                static const char domain_arg[] = "--domain=";
-                int unixcp_size;
-
-                if(pAuthData == NULL)
-                {
-                    LPWKSTA_USER_INFO_1 ui = NULL;
-                    NET_API_STATUS status;
-
-                    status = NetWkstaUserGetInfo(NULL, 1, (LPBYTE *)&ui);
-                    if (status != NERR_Success || ui == NULL)
-                    {
-                        ret = SEC_E_NO_CREDENTIALS;
-                        phCredential = NULL;
-                        break;
-                    }
-                    
-                    username = HeapAlloc(GetProcessHeap(), 0, 
-                            (lstrlenW(ui->wkui1_username)+1) * 
-                            sizeof(SEC_WCHAR));
-                    lstrcpyW(username, ui->wkui1_username);
-                        
-                    /* same for the domain */
-                    domain = HeapAlloc(GetProcessHeap(), 0, 
-                            (lstrlenW(ui->wkui1_logon_domain)+1) * 
-                            sizeof(SEC_WCHAR));
-                    lstrcpyW(domain, ui->wkui1_logon_domain);
-                    NetApiBufferFree(ui);
-                }
-                else
-                {
-                    PSEC_WINNT_AUTH_IDENTITY_W auth_data = 
-                        (PSEC_WINNT_AUTH_IDENTITY_W)pAuthData;
-
-                    /* Get username and domain from pAuthData */
-                    username = HeapAlloc(GetProcessHeap(), 0, 
-                            (auth_data->UserLength + 1) * sizeof(SEC_WCHAR));
-                    memcpy(username, auth_data->User,
-                           auth_data->UserLength * sizeof(SEC_WCHAR));
-                    username[auth_data->UserLength] = '\0';
-
-                    domain = HeapAlloc(GetProcessHeap(), 0,
-                            (auth_data->DomainLength + 1) * sizeof(SEC_WCHAR));
-                    memcpy(domain, auth_data->Domain,
-                           auth_data->DomainLength * sizeof(SEC_WCHAR));
-                    domain[auth_data->DomainLength] = '\0';
-                }
-
                 ntlm_cred = HeapAlloc(GetProcessHeap(), 0, sizeof(*ntlm_cred));
                 if (!ntlm_cred)
                 {
@@ -184,31 +174,21 @@ static SECURITY_STATUS SEC_ENTRY ntlm_AcquireCredentialsHandleW(
                     break;
                 }
                 ntlm_cred->mode = NTLM_CLIENT;
+                ntlm_cred->username_arg = NULL;
+                ntlm_cred->domain_arg = NULL;
                 ntlm_cred->password = NULL;
                 ntlm_cred->pwlen = 0;
 
-                TRACE("Username is %s\n", debugstr_w(username));
-                unixcp_size =  WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
-                        username, -1, NULL, 0, NULL, NULL) + sizeof(username_arg);
-                ntlm_cred->username_arg = HeapAlloc(GetProcessHeap(), 0, unixcp_size);
-                memcpy(ntlm_cred->username_arg, username_arg, sizeof(username_arg) - 1);
-                WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, username, -1,
-                        ntlm_cred->username_arg + sizeof(username_arg) - 1,
-                        unixcp_size - sizeof(username_arg) + 1, NULL, NULL);
-
-                TRACE("Domain name is %s\n", debugstr_w(domain));
-                unixcp_size = WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
-                        domain, -1, NULL, 0,  NULL, NULL) + sizeof(domain_arg);
-                ntlm_cred->domain_arg = HeapAlloc(GetProcessHeap(), 0, unixcp_size);
-                memcpy(ntlm_cred->domain_arg, domain_arg, sizeof(domain_arg) - 1);
-                WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS, domain,
-                        -1, ntlm_cred->domain_arg + sizeof(domain_arg) - 1,
-                        unixcp_size - sizeof(domain) + 1, NULL, NULL);
-
                 if(pAuthData != NULL)
                 {
                     PSEC_WINNT_AUTH_IDENTITY_W auth_data =
-                    (PSEC_WINNT_AUTH_IDENTITY_W)pAuthData;
+                        (PSEC_WINNT_AUTH_IDENTITY_W)pAuthData;
+
+                    TRACE("Username is %s\n", debugstr_wn(auth_data->User, auth_data->UserLength));
+                    TRACE("Domain name is %s\n", debugstr_wn(auth_data->Domain, auth_data->DomainLength));
+
+                    ntlm_cred->username_arg = ntlm_GetUsernameArg(auth_data->User, auth_data->UserLength);
+                    ntlm_cred->domain_arg = ntlm_GetDomainArg(auth_data->Domain, auth_data->DomainLength);
 
                     if(auth_data->PasswordLength != 0)
                     {
@@ -380,6 +360,69 @@ static int ntlm_GetTokenBufferIndex(PSecBufferDesc pMessage)
     return -1;
 }
 
+/*************************************************************************
+ *             ntlm_GetDataBufferIndex
+ * Calculates the index of the first secbuffer with BufferType == SECBUFFER_DATA
+ * Returns index if found or -1 if not found.
+ */
+static int ntlm_GetDataBufferIndex(PSecBufferDesc pMessage)
+{
+    UINT i;
+
+    TRACE("%p\n", pMessage);
+
+    for( i = 0; i < pMessage->cBuffers; ++i )
+    {
+        if(pMessage->pBuffers[i].BufferType == SECBUFFER_DATA)
+            return i;
+    }
+
+    return -1;
+}
+
+static BOOL ntlm_GetCachedCredential(const SEC_WCHAR *pszTargetName, PCREDENTIALW *cred)
+{
+    LPCWSTR p;
+    LPCWSTR pszHost;
+    LPWSTR pszHostOnly;
+    BOOL ret;
+
+    if (!pszTargetName)
+        return FALSE;
+
+    /* try to get the start of the hostname from service principal name (SPN) */
+    pszHost = strchrW(pszTargetName, '/');
+    if (pszHost)
+    {
+        /* skip slash character */
+        pszHost++;
+
+        /* find end of host by detecting start of instance port or start of referrer */
+        p = strchrW(pszHost, ':');
+        if (!p)
+            p = strchrW(pszHost, '/');
+        if (!p)
+            p = pszHost + strlenW(pszHost);
+    }
+    else /* otherwise not an SPN, just a host */
+    {
+        pszHost = pszTargetName;
+        p = pszHost + strlenW(pszHost);
+    }
+
+    pszHostOnly = HeapAlloc(GetProcessHeap(), 0, (p - pszHost + 1) * sizeof(WCHAR));
+    if (!pszHostOnly)
+        return FALSE;
+
+    memcpy(pszHostOnly, pszHost, (p - pszHost) * sizeof(WCHAR));
+    pszHostOnly[p - pszHost] = '\0';
+
+    ret = CredReadW(pszHostOnly, CRED_TYPE_DOMAIN_PASSWORD, 0, cred);
+
+    HeapFree(GetProcessHeap(), 0, pszHostOnly);
+    return ret;
+}
+
 /***********************************************************************
  *              InitializeSecurityContextW
  */
@@ -397,6 +440,9 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
     PBYTE bin;
     int buffer_len, bin_len, max_len = NTLM_MAX_BUF;
     int token_idx;
+    SEC_CHAR *username = NULL;
+    SEC_CHAR *domain = NULL;
+    SEC_CHAR *password = NULL;
 
     TRACE("%p %p %s %d %d %d %p %d %p %p %p %p\n", phCredential, phContext,
      debugstr_w(pszTargetName), fContextReq, Reserved1, TargetDataRep, pInput,
@@ -416,11 +462,6 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
      */
     /* The squid cache size is 2010 chars, and that's what ntlm_auth uses */
 
-    if (pszTargetName)
-    {
-        TRACE("According to a MS whitepaper pszTargetName is ignored.\n");
-    }
-
     if(TargetDataRep == SECURITY_NETWORK_DREP){
         TRACE("Setting SECURITY_NETWORK_DREP\n");
     }
@@ -432,12 +473,16 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
     {
         static char helper_protocol[] = "--helper-protocol=ntlmssp-client-1";
         static CHAR credentials_argv[] = "--use-cached-creds";
-        SEC_CHAR *client_argv[6];
+        SEC_CHAR *client_argv[5];
+        int pwlen = 0;
 
         TRACE("First time in ISC()\n");
 
         if(!phCredential)
-            return SEC_E_INVALID_HANDLE;
+        {
+            ret = SEC_E_INVALID_HANDLE;
+            goto isc_end;
+        }
 
         /* As the server side of sspi never calls this, make sure that
          * the handler is a client handler.
@@ -446,15 +491,79 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         if(ntlm_cred->mode != NTLM_CLIENT)
         {
             TRACE("Cred mode = %d\n", ntlm_cred->mode);
-            return SEC_E_INVALID_HANDLE;
+            ret = SEC_E_INVALID_HANDLE;
+            goto isc_end;
         }
 
         client_argv[0] = ntlm_auth;
         client_argv[1] = helper_protocol;
-        client_argv[2] = ntlm_cred->username_arg;
-        client_argv[3] = ntlm_cred->domain_arg;
-        client_argv[4] = credentials_argv;
-        client_argv[5] = NULL;
+        if (!ntlm_cred->username_arg && !ntlm_cred->domain_arg)
+        {
+            LPWKSTA_USER_INFO_1 ui = NULL;
+            NET_API_STATUS status;
+            PCREDENTIALW cred;
+
+            if (ntlm_GetCachedCredential(pszTargetName, &cred))
+            {
+                LPWSTR p;
+                p = strchrW(cred->UserName, '\\');
+                if (p)
+                {
+                    domain = ntlm_GetDomainArg(cred->UserName, p - cred->UserName);
+                    p++;
+                }
+                else
+                {
+                    domain = ntlm_GetDomainArg(NULL, 0);
+                    p = cred->UserName;
+                }
+
+                username = ntlm_GetUsernameArg(p, -1);
+
+                if(cred->CredentialBlobSize != 0)
+                {
+                    pwlen = WideCharToMultiByte(CP_UNIXCP,
+                        WC_NO_BEST_FIT_CHARS, (LPWSTR)cred->CredentialBlob,
+                        cred->CredentialBlobSize / sizeof(WCHAR), NULL, 0,
+                        NULL, NULL);
+
+                    password = HeapAlloc(GetProcessHeap(), 0, pwlen);
+
+                    WideCharToMultiByte(CP_UNIXCP, WC_NO_BEST_FIT_CHARS,
+                                        (LPWSTR)cred->CredentialBlob,
+                                        cred->CredentialBlobSize / sizeof(WCHAR),
+                                        password, pwlen, NULL, NULL);
+                }
+
+                CredFree(cred);
+
+                client_argv[2] = username;
+                client_argv[3] = domain;
+                client_argv[4] = NULL;
+            }
+            else
+            {
+                status = NetWkstaUserGetInfo(NULL, 1, (LPBYTE *)&ui);
+                if (status != NERR_Success || ui == NULL)
+                {
+                    ret = SEC_E_NO_CREDENTIALS;
+                    goto isc_end;
+                }
+                username = ntlm_GetUsernameArg(ui->wkui1_username, -1);
+
+                TRACE("using cached credentials\n");
+
+                client_argv[2] = username;
+                client_argv[3] = credentials_argv;
+                client_argv[4] = NULL;
+            }
+        }
+        else
+        {
+            client_argv[2] = ntlm_cred->username_arg;
+            client_argv[3] = ntlm_cred->domain_arg;
+            client_argv[4] = NULL;
+        }
 
         if((ret = fork_helper(&helper, ntlm_auth, client_argv)) != SEC_E_OK)
             goto isc_end;
@@ -469,19 +578,20 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         }
 
         /* Generate the dummy session key = MD4(MD4(password))*/
-        if(ntlm_cred->password)
+        if(password || ntlm_cred->password)
         {
             SEC_WCHAR *unicode_password;
             int passwd_lenW;
 
             TRACE("Converting password to unicode.\n");
             passwd_lenW = MultiByteToWideChar(CP_ACP, 0,
-                                              (LPCSTR)ntlm_cred->password, ntlm_cred->pwlen,
+                                              password ? (LPCSTR)password : (LPCSTR)ntlm_cred->password,
+                                              password ? pwlen : ntlm_cred->pwlen,
                                               NULL, 0);
             unicode_password = HeapAlloc(GetProcessHeap(), 0,
                                          passwd_lenW * sizeof(SEC_WCHAR));
-            MultiByteToWideChar(CP_ACP, 0, (LPCSTR)ntlm_cred->password,
-                                ntlm_cred->pwlen, unicode_password, passwd_lenW);
+            MultiByteToWideChar(CP_ACP, 0, password ? (LPCSTR)password : (LPCSTR)ntlm_cred->password,
+                                password ? pwlen : ntlm_cred->pwlen, unicode_password, passwd_lenW);
 
             SECUR32_CreateNTLMv1SessionKey((PBYTE)unicode_password,
                                            passwd_lenW * sizeof(SEC_WCHAR), helper->session_key);
@@ -542,7 +652,7 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
 
         /* If no password is given, try to use cached credentials. Fall back to an empty
          * password if this failed. */
-        if(ntlm_cred->password == NULL)
+        if(!password && !ntlm_cred->password)
         {
             lstrcpynA(buffer, "OK", max_len-1);
             if((ret = run_helper(helper, buffer, max_len, &buffer_len)) != SEC_E_OK)
@@ -553,8 +663,9 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
             /* If the helper replied with "PW", using cached credentials failed */
             if(!strncmp(buffer, "PW", 2))
             {
-                TRACE("Using cached credentials failed. Using empty password.\n");
-                lstrcpynA(buffer, "PW AA==", max_len-1);
+                TRACE("Using cached credentials failed.\n");
+                ret = SEC_E_NO_CREDENTIALS;
+                goto isc_end;
             }
             else /* Just do a noop on the next run */
                 lstrcpynA(buffer, "OK", max_len-1);
@@ -562,8 +673,8 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         else
         {
             lstrcpynA(buffer, "PW ", max_len-1);
-            if((ret = encodeBase64((unsigned char*)ntlm_cred->password,
-                        ntlm_cred->pwlen, buffer+3,
+            if((ret = encodeBase64(password ? (unsigned char *)password : (unsigned char *)ntlm_cred->password,
+                        password ? pwlen : ntlm_cred->pwlen, buffer+3,
                         max_len-3, &buffer_len)) != SEC_E_OK)
             {
                 cleanup_helper(helper);
@@ -637,7 +748,10 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         }
 
         if(!phContext)
-            return SEC_E_INVALID_HANDLE;
+        {
+            ret = SEC_E_INVALID_HANDLE;
+            goto isc_end;
+        }
 
         /* As the server side of sspi never calls this, make sure that
          * the handler is a client handler.
@@ -646,7 +760,8 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         if(helper->mode != NTLM_CLIENT)
         {
             TRACE("Helper mode = %d\n", helper->mode);
-            return SEC_E_INVALID_HANDLE;
+            ret = SEC_E_INVALID_HANDLE;
+            goto isc_end;
         }
 
         if (!pInput->pBuffers[input_token_idx].pvBuffer)
@@ -720,7 +835,7 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
 
     if (fContextReq & ISC_REQ_ALLOCATE_MEMORY)
     {
-        pOutput->pBuffers[token_idx].pvBuffer = SECUR32_ALLOC(bin_len);
+        pOutput->pBuffers[token_idx].pvBuffer = HeapAlloc(GetProcessHeap(), 0, bin_len);
         pOutput->pBuffers[token_idx].cbBuffer = bin_len;
     }
     else if (pOutput->pBuffers[token_idx].cbBuffer < bin_len)
@@ -804,14 +919,17 @@ static SECURITY_STATUS SEC_ENTRY ntlm_InitializeSecurityContextW(
         helper->crypt.ntlm2.send_a4i = SECUR32_arc4Alloc();
         helper->crypt.ntlm2.recv_a4i = SECUR32_arc4Alloc();
         SECUR32_arc4Init(helper->crypt.ntlm2.send_a4i,
-                (BYTE *)helper->crypt.ntlm2.send_seal_key, 16);
+                         helper->crypt.ntlm2.send_seal_key, 16);
         SECUR32_arc4Init(helper->crypt.ntlm2.recv_a4i,
-               (BYTE *)helper->crypt.ntlm2.recv_seal_key, 16);
+                         helper->crypt.ntlm2.recv_seal_key, 16);
         helper->crypt.ntlm2.send_seq_no = 0l;
         helper->crypt.ntlm2.recv_seq_no = 0l;
     }
 
 isc_end:
+    HeapFree(GetProcessHeap(), 0, username);
+    HeapFree(GetProcessHeap(), 0, domain);
+    HeapFree(GetProcessHeap(), 0, password);
     HeapFree(GetProcessHeap(), 0, want_flags);
     HeapFree(GetProcessHeap(), 0, buffer);
     HeapFree(GetProcessHeap(), 0, bin);
@@ -1110,6 +1228,10 @@ static SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
 
         TRACE("Reply from ntlm_auth: %s\n", debugstr_a(buffer));
 
+        /* At this point, we get a NA if the user didn't authenticate, but a BH
+         * if ntlm_auth could not connect to winbindd. Apart from running Wine
+         * as root, there is no way to fix this for now, so just handle this as
+         * a failed login. */
         if(strncmp(buffer, "AF ", 3) != 0)
         {
             if(strncmp(buffer, "NA ", 3) == 0)
@@ -1119,7 +1241,18 @@ static SECURITY_STATUS SEC_ENTRY ntlm_AcceptSecurityContext(
             }
             else
             {
-                ret = SEC_E_INTERNAL_ERROR;
+                size_t ntlm_pipe_err_len = strlen("BH NT_STATUS_ACCESS_DENIED");
+
+                if( (buffer_len >= ntlm_pipe_err_len) &&
+                    (strncmp(buffer, "BH NT_STATUS_ACCESS_DENIED",
+                             ntlm_pipe_err_len) == 0))
+                {
+                    TRACE("Connection to winbindd failed\n");
+                    ret = SEC_E_LOGON_DENIED;
+                }
+                else
+                    ret = SEC_E_INTERNAL_ERROR;
+
                 goto asc_end;
             }
         }
@@ -1609,7 +1742,7 @@ static SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
         ULONG fQOP, PSecBufferDesc pMessage, ULONG MessageSeqNo)
 {
     PNegoHelper helper;
-    int token_idx;
+    int token_idx, data_idx;
 
     TRACE("(%p %d %p %d)\n", phContext, fQOP, pMessage, MessageSeqNo);
 
@@ -1628,6 +1761,9 @@ static SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
     if((token_idx = ntlm_GetTokenBufferIndex(pMessage)) == -1)
         return SEC_E_INVALID_TOKEN;
 
+    if((data_idx = ntlm_GetDataBufferIndex(pMessage)) ==-1 )
+        return SEC_E_INVALID_TOKEN;
+
     if(pMessage->pBuffers[token_idx].cbBuffer < 16)
         return SEC_E_BUFFER_TOO_SMALL;
 
@@ -1638,8 +1774,8 @@ static SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
     { 
         ntlm_CreateSignature(helper, pMessage, token_idx, NTLM_SEND, FALSE);
         SECUR32_arc4Process(helper->crypt.ntlm2.send_a4i,
-                (BYTE *)pMessage->pBuffers[1].pvBuffer,
-                pMessage->pBuffers[1].cbBuffer);
+                (BYTE *)pMessage->pBuffers[data_idx].pvBuffer,
+                pMessage->pBuffers[data_idx].cbBuffer);
 
         if(helper->neg_flags & NTLMSSP_NEGOTIATE_KEY_EXCHANGE)
             SECUR32_arc4Process(helper->crypt.ntlm2.send_a4i,
@@ -1662,8 +1798,9 @@ static SECURITY_STATUS SEC_ENTRY ntlm_EncryptMessage(PCtxtHandle phContext,
 
         sig = pMessage->pBuffers[token_idx].pvBuffer;
 
-        SECUR32_arc4Process(helper->crypt.ntlm.a4i, pMessage->pBuffers[1].pvBuffer,
-                pMessage->pBuffers[1].cbBuffer);
+        SECUR32_arc4Process(helper->crypt.ntlm.a4i,
+                pMessage->pBuffers[data_idx].pvBuffer,
+                pMessage->pBuffers[data_idx].cbBuffer);
         SECUR32_arc4Process(helper->crypt.ntlm.a4i, sig+4, 12);
 
         if(helper->neg_flags & NTLMSSP_NEGOTIATE_ALWAYS_SIGN || helper->neg_flags == 0)
@@ -1683,7 +1820,7 @@ static SECURITY_STATUS SEC_ENTRY ntlm_DecryptMessage(PCtxtHandle phContext,
     SECURITY_STATUS ret;
     ULONG ntlmssp_flags_save;
     PNegoHelper helper;
-    int token_idx;
+    int token_idx, data_idx;
     TRACE("(%p %p %d %p)\n", phContext, pMessage, MessageSeqNo, pfQOP);
 
     if(!phContext)
@@ -1698,6 +1835,9 @@ static SECURITY_STATUS SEC_ENTRY ntlm_DecryptMessage(PCtxtHandle phContext,
     if((token_idx = ntlm_GetTokenBufferIndex(pMessage)) == -1)
         return SEC_E_INVALID_TOKEN;
 
+    if((data_idx = ntlm_GetDataBufferIndex(pMessage)) ==-1)
+        return SEC_E_INVALID_TOKEN;
+
     if(pMessage->pBuffers[token_idx].cbBuffer < 16)
         return SEC_E_BUFFER_TOO_SMALL;
 
@@ -1706,12 +1846,14 @@ static SECURITY_STATUS SEC_ENTRY ntlm_DecryptMessage(PCtxtHandle phContext,
     if(helper->neg_flags & NTLMSSP_NEGOTIATE_NTLM2 && helper->neg_flags & NTLMSSP_NEGOTIATE_SEAL)
     {
         SECUR32_arc4Process(helper->crypt.ntlm2.recv_a4i,
-                pMessage->pBuffers[1].pvBuffer, pMessage->pBuffers[1].cbBuffer);
+                pMessage->pBuffers[data_idx].pvBuffer,
+                pMessage->pBuffers[data_idx].cbBuffer);
     }
     else
     {
         SECUR32_arc4Process(helper->crypt.ntlm.a4i,
-                pMessage->pBuffers[1].pvBuffer, pMessage->pBuffers[1].cbBuffer);
+                pMessage->pBuffers[data_idx].pvBuffer,
+                pMessage->pBuffers[data_idx].cbBuffer);
     }
 
     /* Make sure we use a session key for the signature check, EncryptMessage
