@@ -1065,3 +1065,31 @@ void WINAPI InstallHinfSectionA( HWND hwnd, HINSTANCE handle, LPCSTR cmdline, IN
         RtlFreeUnicodeString( &cmdlineW );
     }
 }
+
+/***********************************************************************
+ *              SetupInstallServicesFromInfSectionW  (SETUPAPI.@)
+ */
+BOOL WINAPI SetupInstallServicesFromInfSectionW( HINF Inf, PCWSTR SectionName, DWORD Flags)
+{
+    FIXME("(%p, %s, %d) stub!\n", Inf, debugstr_w(SectionName), Flags);
+    return FALSE;
+}
+
+/***********************************************************************
+ *              SetupInstallServicesFromInfSectionA  (SETUPAPI.@)
+ */
+BOOL WINAPI SetupInstallServicesFromInfSectionA( HINF Inf, PCSTR SectionName, DWORD Flags)
+{
+    UNICODE_STRING SectionNameW;
+    BOOL ret = FALSE;
+
+    if (RtlCreateUnicodeStringFromAsciiz( &SectionNameW, SectionName ))
+    {
+        ret = SetupInstallServicesFromInfSectionW( Inf, SectionNameW.Buffer, Flags );
+        RtlFreeUnicodeString( &SectionNameW );
+    }
+    else
+        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+
+    return ret;
+}
