@@ -20,7 +20,7 @@
 
 #include <assert.h>
 #include <stdarg.h>
-
+#include <stdio.h>
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
@@ -36,14 +36,20 @@ static HMODULE hSetupAPI;
 static HDEVINFO (WINAPI *pSetupDiCreateDeviceInfoList)(GUID*,HWND);
 static HDEVINFO (WINAPI *pSetupDiCreateDeviceInfoListExW)(GUID*,HWND,PCWSTR,PVOID);
 static BOOL     (WINAPI *pSetupDiCreateDeviceInterfaceA)(HDEVINFO, PSP_DEVINFO_DATA, const GUID *, PCSTR, DWORD, PSP_DEVICE_INTERFACE_DATA);
+static BOOL     (WINAPI *pSetupDiCallClassInstaller)(DI_FUNCTION, HDEVINFO, PSP_DEVINFO_DATA);
 static BOOL     (WINAPI *pSetupDiDestroyDeviceInfoList)(HDEVINFO);
 static BOOL     (WINAPI *pSetupDiEnumDeviceInfo)(HDEVINFO, DWORD, PSP_DEVINFO_DATA);
 static BOOL     (WINAPI *pSetupDiEnumDeviceInterfaces)(HDEVINFO, PSP_DEVINFO_DATA, const GUID *, DWORD, PSP_DEVICE_INTERFACE_DATA);
+static BOOL     (WINAPI *pSetupDiInstallClassA)(HWND, PCSTR, DWORD, HSPFILEQ);
 static HKEY     (WINAPI *pSetupDiOpenClassRegKeyExA)(GUID*,REGSAM,DWORD,PCSTR,PVOID);
+static HKEY     (WINAPI *pSetupDiOpenDevRegKey)(HDEVINFO, PSP_DEVINFO_DATA, DWORD, DWORD, DWORD, REGSAM);
+static HKEY     (WINAPI *pSetupDiCreateDevRegKeyW)(HDEVINFO, PSP_DEVINFO_DATA, DWORD, DWORD, DWORD, HINF, PCWSTR);
 static BOOL     (WINAPI *pSetupDiCreateDeviceInfoA)(HDEVINFO, PCSTR, GUID *, PCSTR, HWND, DWORD, PSP_DEVINFO_DATA);
 static BOOL     (WINAPI *pSetupDiGetDeviceInstanceIdA)(HDEVINFO, PSP_DEVINFO_DATA, PSTR, DWORD, PDWORD);
 static BOOL     (WINAPI *pSetupDiGetDeviceInterfaceDetailA)(HDEVINFO, PSP_DEVICE_INTERFACE_DATA, PSP_DEVICE_INTERFACE_DETAIL_DATA_A, DWORD, PDWORD, PSP_DEVINFO_DATA);
+static BOOL     (WINAPI *pSetupDiGetDeviceInterfaceDetailW)(HDEVINFO, PSP_DEVICE_INTERFACE_DATA, PSP_DEVICE_INTERFACE_DETAIL_DATA_W, DWORD, PDWORD, PSP_DEVINFO_DATA);
 static BOOL     (WINAPI *pSetupDiRegisterDeviceInfo)(HDEVINFO, PSP_DEVINFO_DATA, DWORD, PSP_DETSIG_CMPPROC, PVOID, PSP_DEVINFO_DATA);
+static HDEVINFO (WINAPI *pSetupDiGetClassDevsA)(CONST GUID *, LPCSTR, HWND, DWORD);
 
 static void init_function_pointers(void)
 {
@@ -54,14 +60,88 @@ static void init_function_pointers(void)
     pSetupDiCreateDeviceInfoListExW = (void *)GetProcAddress(hSetupAPI, "SetupDiCreateDeviceInfoListExW");
     pSetupDiCreateDeviceInterfaceA = (void *)GetProcAddress(hSetupAPI, "SetupDiCreateDeviceInterfaceA");
     pSetupDiDestroyDeviceInfoList = (void *)GetProcAddress(hSetupAPI, "SetupDiDestroyDeviceInfoList");
+    pSetupDiCallClassInstaller = (void *)GetProcAddress(hSetupAPI, "SetupDiCallClassInstaller");
     pSetupDiEnumDeviceInfo = (void *)GetProcAddress(hSetupAPI, "SetupDiEnumDeviceInfo");
     pSetupDiEnumDeviceInterfaces = (void *)GetProcAddress(hSetupAPI, "SetupDiEnumDeviceInterfaces");
     pSetupDiGetDeviceInstanceIdA = (void *)GetProcAddress(hSetupAPI, "SetupDiGetDeviceInstanceIdA");
     pSetupDiGetDeviceInterfaceDetailA = (void *)GetProcAddress(hSetupAPI, "SetupDiGetDeviceInterfaceDetailA");
+    pSetupDiGetDeviceInterfaceDetailW = (void *)GetProcAddress(hSetupAPI, "SetupDiGetDeviceInterfaceDetailW");
+    pSetupDiInstallClassA = (void *)GetProcAddress(hSetupAPI, "SetupDiInstallClassA");
     pSetupDiOpenClassRegKeyExA = (void *)GetProcAddress(hSetupAPI, "SetupDiOpenClassRegKeyExA");
+    pSetupDiOpenDevRegKey = (void *)GetProcAddress(hSetupAPI, "SetupDiOpenDevRegKey");
+    pSetupDiCreateDevRegKeyW = (void *)GetProcAddress(hSetupAPI, "SetupDiCreateDevRegKeyW");
     pSetupDiRegisterDeviceInfo = (void *)GetProcAddress(hSetupAPI, "SetupDiRegisterDeviceInfo");
+    pSetupDiGetClassDevsA = (void *)GetProcAddress(hSetupAPI, "SetupDiGetClassDevsA");
 }
 
+/* RegDeleteTreeW from dlls/advapi32/registry.c */
+LSTATUS WINAPI devinst_RegDeleteTreeW(HKEY hKey, LPCWSTR lpszSubKey)
+{
+    LONG ret;
+    DWORD dwMaxSubkeyLen, dwMaxValueLen;
+    DWORD dwMaxLen, dwSize;
+    WCHAR szNameBuf[MAX_PATH], *lpszName = szNameBuf;
+    HKEY hSubKey = hKey;
+
+    if(lpszSubKey)
+    {
+        ret = RegOpenKeyExW(hKey, lpszSubKey, 0, KEY_READ, &hSubKey);
+        if (ret) return ret;
+    }
+
+    /* Get highest length for keys, values */
+    ret = RegQueryInfoKeyW(hSubKey, NULL, NULL, NULL, NULL,
+            &dwMaxSubkeyLen, NULL, NULL, &dwMaxValueLen, NULL, NULL, NULL);
+    if (ret) goto cleanup;
+
+    dwMaxSubkeyLen++;
+    dwMaxValueLen++;
+    dwMaxLen = max(dwMaxSubkeyLen, dwMaxValueLen);
+    if (dwMaxLen > sizeof(szNameBuf)/sizeof(WCHAR))
+    {
+        /* Name too big: alloc a buffer for it */
+        if (!(lpszName = HeapAlloc( GetProcessHeap(), 0, dwMaxLen*sizeof(WCHAR))))
+        {
+            ret = ERROR_NOT_ENOUGH_MEMORY;
+            goto cleanup;
+        }
+    }
+
+
+    /* Recursively delete all the subkeys */
+    while (TRUE)
+    {
+        dwSize = dwMaxLen;
+        if (RegEnumKeyExW(hSubKey, 0, lpszName, &dwSize, NULL,
+                          NULL, NULL, NULL)) break;
+
+        ret = devinst_RegDeleteTreeW(hSubKey, lpszName);
+        if (ret) goto cleanup;
+    }
+
+    if (lpszSubKey)
+        ret = RegDeleteKeyW(hKey, lpszSubKey);
+    else
+        while (TRUE)
+        {
+            dwSize = dwMaxLen;
+            if (RegEnumValueW(hKey, 0, lpszName, &dwSize,
+                  NULL, NULL, NULL, NULL)) break;
+
+            ret = RegDeleteValueW(hKey, lpszName);
+            if (ret) goto cleanup;
+        }
+
+cleanup:
+    /* Free buffer if allocated */
+    if (lpszName != szNameBuf)
+        HeapFree( GetProcessHeap(), 0, lpszName);
+    if(lpszSubKey)
+        RegCloseKey(hSubKey);
+    return ret;
+}
+
+
 static void test_SetupDiCreateDeviceInfoListEx(void) 
 {
     HDEVINFO devlist;
@@ -139,6 +219,86 @@ static void test_SetupDiOpenClassRegKeyExA(void)
         trace("failed to open classes key\n");
 }
 
+static void append_str(char **str, const char *data)
+{
+    sprintf(*str, data);
+    *str += strlen(*str);
+}
+
+static void create_inf_file(LPCSTR filename)
+{
+    char data[1024];
+    char *ptr = data;
+    DWORD dwNumberOfBytesWritten;
+    HANDLE hf = CreateFile(filename, GENERIC_WRITE, 0, NULL,
+                           CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+
+    append_str(&ptr, "[Version]\n");
+    append_str(&ptr, "Signature=\"$Chicago$\"\n");
+    append_str(&ptr, "Class=Bogus\n");
+    append_str(&ptr, "ClassGUID={6a55b5a4-3f65-11db-b704-0011955c2bdb}\n");
+    append_str(&ptr, "[ClassInstall32]\n");
+    append_str(&ptr, "AddReg=BogusClass.NT.AddReg\n");
+    append_str(&ptr, "[BogusClass.NT.AddReg]\n");
+    append_str(&ptr, "HKR,,,,\"Wine test devices\"\n");
+
+    WriteFile(hf, data, ptr - data, &dwNumberOfBytesWritten, NULL);
+    CloseHandle(hf);
+}
+
+static void get_temp_filename(LPSTR path)
+{
+    static char curr[MAX_PATH] = { 0 };
+    char temp[MAX_PATH];
+    LPSTR ptr;
+
+    if (!*curr)
+        GetCurrentDirectoryA(MAX_PATH, curr);
+    GetTempFileNameA(curr, "set", 0, temp);
+    ptr = strrchr(temp, '\\');
+
+    lstrcpyA(path, ptr + 1);
+}
+
+static void testInstallClass(void)
+{
+    static const WCHAR classKey[] = {'S','y','s','t','e','m','\\',
+     'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
+     'C','o','n','t','r','o','l','\\','C','l','a','s','s','\\',
+     '{','6','a','5','5','b','5','a','4','-','3','f','6','5','-',
+     '1','1','d','b','-','b','7','0','4','-',
+     '0','0','1','1','9','5','5','c','2','b','d','b','}',0};
+    char tmpfile[MAX_PATH];
+    BOOL ret;
+
+    if (!pSetupDiInstallClassA)
+    {
+        skip("No SetupDiInstallClassA\n");
+        return;
+    }
+    tmpfile[0] = '.';
+    tmpfile[1] = '\\';
+    get_temp_filename(tmpfile + 2);
+    create_inf_file(tmpfile + 2);
+
+    ret = pSetupDiInstallClassA(NULL, NULL, 0, NULL);
+    ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
+     "Expected ERROR_INVALID_PARAMETER, got %08x\n", GetLastError());
+    ret = pSetupDiInstallClassA(NULL, NULL, DI_NOVCP, NULL);
+    ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
+     "Expected ERROR_INVALID_PARAMETER, got %08x\n", GetLastError());
+    ret = pSetupDiInstallClassA(NULL, tmpfile + 2, DI_NOVCP, NULL);
+    ok(!ret && GetLastError() == ERROR_INVALID_PARAMETER,
+     "Expected ERROR_INVALID_PARAMETER, got %08x\n", GetLastError());
+    ret = pSetupDiInstallClassA(NULL, tmpfile + 2, 0, NULL);
+    ok(!ret && GetLastError() == ERROR_FILE_NOT_FOUND,
+     "Expected ERROR_FILE_NOT_FOUND, got %08x\n", GetLastError());
+    ret = pSetupDiInstallClassA(NULL, tmpfile, 0, NULL);
+    ok(ret, "SetupDiInstallClassA failed: %08x\n", GetLastError());
+    RegDeleteKeyW(HKEY_LOCAL_MACHINE, classKey);
+    DeleteFile(tmpfile);
+}
+
 static void testCreateDeviceInfo(void)
 {
     BOOL ret;
@@ -471,6 +631,7 @@ static void testGetDeviceInterfaceDetail(void)
             LPBYTE buf = HeapAlloc(GetProcessHeap(), 0, size);
             SP_DEVICE_INTERFACE_DETAIL_DATA_A *detail =
                 (SP_DEVICE_INTERFACE_DETAIL_DATA_A *)buf;
+            DWORD expectedsize = offsetof(SP_DEVICE_INTERFACE_DETAIL_DATA_W, DevicePath) + sizeof(WCHAR)*(1 + strlen(path));
 
             detail->cbSize = 0;
             SetLastError(0xdeadbeef);
@@ -485,18 +646,204 @@ static void testGetDeviceInterfaceDetail(void)
             ok(!ret && GetLastError() == ERROR_INVALID_USER_BUFFER,
              "Expected ERROR_INVALID_USER_BUFFER, got %08x\n", GetLastError());
             detail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA_A);
+            SetLastError(0xdeadbeef);
+            ret = pSetupDiGetDeviceInterfaceDetailA(set, &interfaceData, detail,
+                    size, &size, NULL);
+            ok(!ret && GetLastError() == ERROR_INVALID_USER_BUFFER,
+             "Expected ERROR_INVALID_USER_BUFFER, got %08x\n", GetLastError());
+            /* Windows 2000 and up check for the exact size. Win9x returns ERROR_INVALID_PARAMETER
+             * on every call (so doesn't get here) and NT4 doesn't have this function.
+             */
+            detail->cbSize = FIELD_OFFSET(SP_DEVICE_INTERFACE_DETAIL_DATA_A, DevicePath[1]);
             ret = pSetupDiGetDeviceInterfaceDetailA(set, &interfaceData, detail,
                     size, &size, NULL);
             ok(ret, "SetupDiGetDeviceInterfaceDetailA failed: %d\n",
                     GetLastError());
             ok(!lstrcmpiA(path, detail->DevicePath), "Unexpected path %s\n",
                     detail->DevicePath);
+            /* Check SetupDiGetDeviceInterfaceDetailW */
+            if (pSetupDiGetDeviceInterfaceDetailW)
+            {
+                ret = pSetupDiGetDeviceInterfaceDetailW(set, &interfaceData, NULL, 0, &size, NULL);
+                ok(!ret && GetLastError() == ERROR_INSUFFICIENT_BUFFER, "Expected ERROR_INSUFFICIENT_BUFFER, got error code: %d\n", GetLastError());
+                ok(expectedsize == size, "SetupDiGetDeviceInterfaceDetailW returned wrong reqsize: expected %d, got %d\n", expectedsize, size);
+            }
+            else
+                skip("SetupDiGetDeviceInterfaceDetailW is not available\n");
+
             HeapFree(GetProcessHeap(), 0, buf);
         }
         pSetupDiDestroyDeviceInfoList(set);
     }
 }
 
+static void testDevRegKey(void)
+{
+    static const WCHAR classKey[] = {'S','y','s','t','e','m','\\',
+     'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
+     'C','o','n','t','r','o','l','\\','C','l','a','s','s','\\',
+     '{','6','a','5','5','b','5','a','4','-','3','f','6','5','-',
+     '1','1','d','b','-','b','7','0','4','-',
+     '0','0','1','1','9','5','5','c','2','b','d','b','}',0};
+    BOOL ret;
+    GUID guid = {0x6a55b5a4, 0x3f65, 0x11db, {0xb7,0x04,
+        0x00,0x11,0x95,0x5c,0x2b,0xdb}};
+    HDEVINFO set;
+
+    if (!pSetupDiCreateDeviceInfoList || !pSetupDiDestroyDeviceInfoList ||
+     !pSetupDiCreateDeviceInfoA || !pSetupDiOpenDevRegKey ||
+     !pSetupDiRegisterDeviceInfo || !pSetupDiCreateDevRegKeyW ||
+     !pSetupDiCallClassInstaller)
+    {
+        skip("No SetupDiOpenDevRegKey\n");
+        return;
+    }
+    set = pSetupDiCreateDeviceInfoList(&guid, NULL);
+    ok(set != NULL, "SetupDiCreateDeviceInfoList failed: %d\n", GetLastError());
+    if (set)
+    {
+        SP_DEVINFO_DATA devInfo = { sizeof(devInfo), { 0 } };
+        HKEY key = INVALID_HANDLE_VALUE;
+
+        ret = pSetupDiCreateDeviceInfoA(set, "ROOT\\LEGACY_BOGUS\\0000", &guid,
+                NULL, NULL, 0, &devInfo);
+        ok(ret, "SetupDiCreateDeviceInfoA failed: %08x\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(NULL, NULL, 0, 0, 0, 0);
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_INVALID_HANDLE,
+         "Expected ERROR_INVALID_HANDLE, got %d\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, NULL, 0, 0, 0, 0);
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_INVALID_PARAMETER,
+         "Expected ERROR_INVALID_PARAMETER, got %d\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, 0, 0, 0, 0);
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_INVALID_FLAGS,
+         "Expected ERROR_INVALID_FLAGS, got %d\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0, 0, 0);
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_INVALID_FLAGS,
+         "Expected ERROR_INVALID_FLAGS, got %d\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_BOTH, 0);
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_INVALID_FLAGS,
+         "Expected ERROR_INVALID_FLAGS, got %d\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_DRV, 0);
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_DEVINFO_NOT_REGISTERED,
+         "Expected ERROR_DEVINFO_NOT_REGISTERED, got %08x\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        ret = pSetupDiRegisterDeviceInfo(set, &devInfo, 0, NULL, NULL, NULL);
+        ok(ret, "SetupDiRegisterDeviceInfo failed: %08x\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_DRV, 0);
+        /* The software key isn't created by default */
+        todo_wine
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_KEY_DOES_NOT_EXIST,
+         "Expected ERROR_KEY_DOES_NOT_EXIST_EXIST, got %08x\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_DEV, 0);
+        todo_wine
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_KEY_DOES_NOT_EXIST,
+         "Expected ERROR_KEY_DOES_NOT_EXIST_EXIST, got %08x\n", GetLastError());
+        SetLastError(0xdeadbeef);
+        key = pSetupDiCreateDevRegKeyW(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_DRV, NULL, NULL);
+        ok(key != INVALID_HANDLE_VALUE, "SetupDiCreateDevRegKey failed: %08x\n",
+         GetLastError());
+        RegCloseKey(key);
+        SetLastError(0xdeadbeef);
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_DRV, 0);
+        todo_wine
+        ok(key == INVALID_HANDLE_VALUE &&
+         GetLastError() == ERROR_INVALID_DATA,
+         "Expected ERROR_INVALID_DATA, got %08x\n", GetLastError());
+        key = pSetupDiOpenDevRegKey(set, &devInfo, DICS_FLAG_GLOBAL, 0,
+         DIREG_DRV, KEY_READ);
+        ok(key != INVALID_HANDLE_VALUE, "SetupDiOpenDevRegKey failed: %08x\n",
+         GetLastError());
+        ret = pSetupDiCallClassInstaller(DIF_REMOVE, set, &devInfo);
+        pSetupDiDestroyDeviceInfoList(set);
+    }
+    devinst_RegDeleteTreeW(HKEY_LOCAL_MACHINE, classKey);
+}
+
+static void testRegisterAndGetDetail(void)
+{
+    HDEVINFO set;
+    BOOL ret;
+    GUID guid = {0x6a55b5a4, 0x3f65, 0x11db, {0xb7,0x04,
+        0x00,0x11,0x95,0x5c,0x2b,0xdb}};
+    SP_DEVINFO_DATA devInfo = { sizeof(SP_DEVINFO_DATA), { 0 } };
+    SP_DEVICE_INTERFACE_DATA interfaceData = { sizeof(interfaceData), { 0 } };
+    DWORD dwSize = 0;
+
+    SetLastError(0xdeadbeef);
+    set = pSetupDiGetClassDevsA(&guid, NULL, 0, DIGCF_ALLCLASSES);
+    ok(set != INVALID_HANDLE_VALUE, "SetupDiGetClassDevsA failed: %08x\n",
+     GetLastError());
+
+    SetLastError(0xdeadbeef);
+    ret = pSetupDiCreateDeviceInfoA(set, "LEGACY_BOGUS", &guid, NULL, 0,
+     DICD_GENERATE_ID, &devInfo);
+    ok(ret, "SetupDiCreateDeviceInfoA failed: %08x\n", GetLastError());
+    SetLastError(0xdeadbeef);
+    ret = pSetupDiCreateDeviceInterfaceA(set, &devInfo, &guid, NULL, 0, &interfaceData);
+    ok(ret, "SetupDiCreateDeviceInterfaceA failed: %08x\n", GetLastError());
+    SetLastError(0xdeadbeef);
+    ret = pSetupDiRegisterDeviceInfo(set, &devInfo, 0, NULL, NULL, NULL);
+    ok(ret, "SetupDiRegisterDeviceInfo failed: %08x\n", GetLastError());
+
+    pSetupDiDestroyDeviceInfoList(set);
+
+    SetLastError(0xdeadbeef);
+    set = pSetupDiGetClassDevsA(&guid, NULL, 0, DIGCF_DEVICEINTERFACE);
+    ok(set != INVALID_HANDLE_VALUE, "SetupDiGetClassDevsA failed: %08x\n",
+     GetLastError());
+
+    SetLastError(0xdeadbeef);
+    ret = pSetupDiEnumDeviceInterfaces(set, NULL, &guid, 0, &interfaceData);
+    ok(ret, "SetupDiEnumDeviceInterfaces failed: %08x\n", GetLastError());
+    SetLastError(0xdeadbeef);
+    ret = pSetupDiGetDeviceInterfaceDetailA(set, &interfaceData, NULL, 0, &dwSize, NULL);
+    ok(!ret && GetLastError() == ERROR_INSUFFICIENT_BUFFER,
+     "Expected ERROR_INSUFFICIENT_BUFFER, got %08x\n", GetLastError());
+    if (!ret && GetLastError() == ERROR_INSUFFICIENT_BUFFER)
+    {
+        static const char path[] =
+            "\\\\?\\root#legacy_bogus#0000#{6a55b5a4-3f65-11db-b704-0011955c2bdb}";
+        PSP_DEVICE_INTERFACE_DETAIL_DATA_A detail = NULL;
+
+        detail = (PSP_DEVICE_INTERFACE_DETAIL_DATA_A)HeapAlloc(GetProcessHeap(), 0, dwSize);
+        if (detail)
+        {
+            detail->cbSize = offsetof(SP_DEVICE_INTERFACE_DETAIL_DATA_A, DevicePath) + sizeof(char);
+            SetLastError(0xdeadbeef);
+            ret = pSetupDiGetDeviceInterfaceDetailA(set, &interfaceData,
+             detail, dwSize, &dwSize, NULL);
+            ok(ret, "SetupDiGetDeviceInterfaceDetailA failed: %08x\n", GetLastError());
+            ok(!lstrcmpiA(path, detail->DevicePath), "Unexpected path %s\n",
+                    detail->DevicePath);
+            HeapFree(GetProcessHeap(), 0, detail);
+        }
+    }
+
+    pSetupDiDestroyDeviceInfoList(set);
+}
+
 START_TEST(devinst)
 {
     init_function_pointers();
@@ -510,9 +857,12 @@ START_TEST(devinst)
         test_SetupDiOpenClassRegKeyExA();
     else
         skip("SetupDiOpenClassRegKeyExA is not available\n");
+    testInstallClass();
     testCreateDeviceInfo();
     testGetDeviceInstanceId();
     testRegisterDeviceInfo();
     testCreateDeviceInterface();
     testGetDeviceInterfaceDetail();
+    testDevRegKey();
+    testRegisterAndGetDetail();
 }
