@@ -146,7 +146,7 @@ static void test_get_set(void)
     ok(SUCCEEDED(r), "GetWorkingDirectory failed (0x%08x)\n", r);
     ok(lstrcmpi(buffer,str)==0, "GetWorkingDirectory returned '%s'\n", buffer);
 
-    /* Test Getting / Setting the work directory */
+    /* Test Getting / Setting the path */
     strcpy(buffer,"garbage");
     r = IShellLinkA_GetPath(sl, buffer, sizeof(buffer), NULL, SLGP_RAWPATH);
     ok(SUCCEEDED(r), "GetPath failed (0x%08x)\n", r);
@@ -283,7 +283,7 @@ static void test_get_set(void)
     i=0xdeadbeef;
     r = IShellLinkA_GetIconLocation(sl, buffer, sizeof(buffer), &i);
     ok(SUCCEEDED(r), "GetIconLocation failed (0x%08x)\n", r);
-    ok(lstrcmpi(buffer,str)==0, "GetArguments returned '%s'\n", buffer);
+    ok(lstrcmpi(buffer,str)==0, "GetIconLocation returned '%s'\n", buffer);
     ok(i==0xbabecafe, "GetIconLocation returned %d'\n", i);
 
     /* Test Getting / Setting the hot key */
@@ -527,7 +527,7 @@ static void test_load_save(void)
     check_lnk(lnkfile, &desc, 0x0);
 
     r=GetModuleFileName(NULL, mypath, sizeof(mypath));
-    ok(r>=0 && r<sizeof(mypath), "GetModuleFileName failed (%d)\n", r);
+    ok(r<sizeof(mypath), "GetModuleFileName failed (%d)\n", r);
     strcpy(mydir, mypath);
     p=strrchr(mydir, '\\');
     if (p)
@@ -564,9 +564,14 @@ static void test_load_save(void)
 
     /* Create a temporary non-executable file */
     r=GetTempPath(sizeof(mypath), mypath);
-    ok(r>=0 && r<sizeof(mypath), "GetTempPath failed (%d), err %d\n", r, GetLastError());
+    ok(r<sizeof(mypath), "GetTempPath failed (%d), err %d\n", r, GetLastError());
+    if (!pGetLongPathNameA)
+    {
+        skip("GetLongPathNameA is not available\n");
+        goto cleanup;
+    }
     r=pGetLongPathNameA(mypath, mydir, sizeof(mydir));
-    ok(r>=0 && r<sizeof(mydir), "GetLongPathName failed (%d), err %d\n", r, GetLastError());
+    ok(r<sizeof(mydir), "GetLongPathName failed (%d), err %d\n", r, GetLastError());
     p=strrchr(mydir, '\\');
     if (p)
         *p='\0';
@@ -597,6 +602,7 @@ static void test_load_save(void)
      * represented as a path.
      */
 
+cleanup:
     /* DeleteFileW is not implemented on Win9x */
     r=DeleteFileA("c:\\test.lnk");
     ok(r, "failed to delete link (%d)\n", GetLastError());
@@ -644,12 +650,12 @@ static void test_datalink(void)
     }
 
     flags = 0;
-    r = dl->lpVtbl->GetFlags( dl, &flags );
+    r = IShellLinkDataList_GetFlags( dl, &flags );
     ok( r == S_OK, "GetFlags failed\n");
     ok( flags == 0, "GetFlags returned wrong flags\n");
 
     dar = (void*)-1;
-    r = dl->lpVtbl->CopyDataBlock( dl, EXP_DARWIN_ID_SIG, (LPVOID*) &dar );
+    r = IShellLinkDataList_CopyDataBlock( dl, EXP_DARWIN_ID_SIG, (LPVOID*) &dar );
     ok( r == E_FAIL, "CopyDataBlock failed\n");
     ok( dar == NULL, "should be null\n");
 
@@ -658,17 +664,17 @@ static void test_datalink(void)
 
     /*
      * The following crashes:
-     * r = dl->lpVtbl->GetFlags( dl, NULL );
+     * r = IShellLinkDataList_GetFlags( dl, NULL );
      */
 
     flags = 0;
-    r = dl->lpVtbl->GetFlags( dl, &flags );
+    r = IShellLinkDataList_GetFlags( dl, &flags );
     ok( r == S_OK, "GetFlags failed\n");
     ok( flags == (SLDF_HAS_DARWINID|SLDF_HAS_LOGO3ID),
         "GetFlags returned wrong flags\n");
 
     dar = NULL;
-    r = dl->lpVtbl->CopyDataBlock( dl, EXP_DARWIN_ID_SIG, (LPVOID*) &dar );
+    r = IShellLinkDataList_CopyDataBlock( dl, EXP_DARWIN_ID_SIG, (LPVOID*) &dar );
     ok( r == S_OK, "CopyDataBlock failed\n");
 
     ok( dar && ((DATABLOCK_HEADER*)dar)->dwSignature == EXP_DARWIN_ID_SIG, "signature wrong\n");
