@@ -43,6 +43,9 @@
 static IMalloc *ppM;
 
 static HRESULT (WINAPI *pSHBindToParent)(LPCITEMIDLIST, REFIID, LPVOID*, LPCITEMIDLIST*);
+static HRESULT (WINAPI *pSHGetFolderPathA)(HWND, int, HANDLE, DWORD, LPSTR);
+static HRESULT (WINAPI *pSHGetFolderPathAndSubDirA)(HWND, int, HANDLE, DWORD, LPCSTR, LPSTR);
+static BOOL (WINAPI *pSHGetPathFromIDListW)(LPCITEMIDLIST,LPWSTR);
 static BOOL (WINAPI *pSHGetSpecialFolderPathW)(HWND, LPWSTR, int, BOOL);
 static HRESULT (WINAPI *pStrRetToBufW)(STRRET*,LPCITEMIDLIST,LPWSTR,UINT);
 static LPITEMIDLIST (WINAPI *pILFindLastID)(LPCITEMIDLIST);
@@ -55,20 +58,17 @@ static void init_function_pointers(void)
     HRESULT hr;
 
     hmod = GetModuleHandleA("shell32.dll");
-    if(hmod)
-    {
-        pSHBindToParent = (void*)GetProcAddress(hmod, "SHBindToParent");
-        pSHGetSpecialFolderPathW = (void*)GetProcAddress(hmod, "SHGetSpecialFolderPathW");
-        pILFindLastID = (void *)GetProcAddress(hmod, (LPCSTR)16);
-        pILFree = (void*)GetProcAddress(hmod, (LPSTR)155);
-        pILIsEqual = (void*)GetProcAddress(hmod, (LPSTR)21);
-    }
+    pSHBindToParent = (void*)GetProcAddress(hmod, "SHBindToParent");
+    pSHGetFolderPathA = (void*)GetProcAddress(hmod, "SHGetFolderPathA");
+    pSHGetFolderPathAndSubDirA = (void*)GetProcAddress(hmod, "SHGetFolderPathAndSubDirA");
+    pSHGetPathFromIDListW = (void*)GetProcAddress(hmod, "SHGetPathFromIDListW");
+    pSHGetSpecialFolderPathW = (void*)GetProcAddress(hmod, "SHGetSpecialFolderPathW");
+    pILFindLastID = (void *)GetProcAddress(hmod, (LPCSTR)16);
+    pILFree = (void*)GetProcAddress(hmod, (LPSTR)155);
+    pILIsEqual = (void*)GetProcAddress(hmod, (LPSTR)21);
 
     hmod = GetModuleHandleA("shlwapi.dll");
-    if(hmod)
-    {
-        pStrRetToBufW = (void*)GetProcAddress(hmod, "StrRetToBufW");
-    }
+    pStrRetToBufW = (void*)GetProcAddress(hmod, "StrRetToBufW");
 
     hr = SHGetMalloc(&ppM);
     ok(hr == S_OK, "SHGetMalloc failed %08x\n", hr);
@@ -330,7 +330,27 @@ static void test_BindToObject(void)
 
     IShellFolder_Release(psfSystemDir);
 }
-  
+
+/* Based on PathAddBackslashW from dlls/shlwapi/path.c */
+static LPWSTR myPathAddBackslashW( LPWSTR lpszPath )
+{
+  size_t iLen;
+
+  if (!lpszPath || (iLen = lstrlenW(lpszPath)) >= MAX_PATH)
+    return NULL;
+
+  if (iLen)
+  {
+    lpszPath += iLen;
+    if (lpszPath[-1] != '\\')
+    {
+      *lpszPath++ = '\\';
+      *lpszPath = '\0';
+    }
+  }
+  return lpszPath;
+}
+
 static void test_GetDisplayName(void)
 {
     BOOL result;
@@ -349,10 +369,10 @@ static void test_GetDisplayName(void)
     static const WCHAR wszDirName[] = { 'w','i','n','e','t','e','s','t',0 };
 
     /* I'm trying to figure if there is a functional difference between calling
-     * SHGetPathFromIDList and calling GetDisplayNameOf(SHGDN_FORPARSING) after
+     * SHGetPathFromIDListW and calling GetDisplayNameOf(SHGDN_FORPARSING) after
      * binding to the shellfolder. One thing I thought of was that perhaps 
-     * SHGetPathFromIDList would be able to get the path to a file, which does
-     * not exist anymore, while the other method would'nt. It turns out there's
+     * SHGetPathFromIDListW would be able to get the path to a file, which does
+     * not exist anymore, while the other method wouldn't. It turns out there's
      * no functional difference in this respect.
      */
 
@@ -363,7 +383,7 @@ static void test_GetDisplayName(void)
     ok(result, "SHGetSpecialFolderPathW failed! Last error: %u\n", GetLastError());
     if (!result) return;
 
-    PathAddBackslashW(wszTestDir);
+    myPathAddBackslashW(wszTestDir);
     lstrcatW(wszTestDir, wszDirName);
     /* Use ANSI file functions so this works on Windows 9x */
     WideCharToMultiByte(CP_ACP, 0, wszTestDir, -1, szTestDir, MAX_PATH, 0, 0);
@@ -376,7 +396,7 @@ static void test_GetDisplayName(void)
     }
 
     lstrcpyW(wszTestFile, wszTestDir);
-    PathAddBackslashW(wszTestFile);
+    myPathAddBackslashW(wszTestFile);
     lstrcatW(wszTestFile, wszFileName);
     WideCharToMultiByte(CP_ACP, 0, wszTestFile, -1, szTestFile, MAX_PATH, 0, 0);
 
@@ -447,9 +467,12 @@ static void test_GetDisplayName(void)
     RemoveDirectoryA(szTestDir);
 
     /* SHGetPathFromIDListW still works, although the file is not present anymore. */
-    result = SHGetPathFromIDListW(pidlTestFile, wszTestFile2);
-    ok (result, "SHGetPathFromIDListW failed! Last error: %u\n", GetLastError());
-    ok (!lstrcmpiW(wszTestFile, wszTestFile2), "SHGetPathFromIDListW returns incorrect path!\n");
+    if (pSHGetPathFromIDListW)
+    {
+        result = pSHGetPathFromIDListW(pidlTestFile, wszTestFile2);
+        ok (result, "SHGetPathFromIDListW failed! Last error: %u\n", GetLastError());
+        ok (!lstrcmpiW(wszTestFile, wszTestFile2), "SHGetPathFromIDListW returns incorrect path!\n");
+    }
 
     if(!pSHBindToParent) return;
 
@@ -779,26 +802,30 @@ static void test_SHGetPathFromIDList(void)
 	HMODULE hShell32;
 	LPITEMIDLIST pidlPrograms;
 
-    if(!pSHGetSpecialFolderPathW) return;
+    if(!pSHGetPathFromIDListW || !pSHGetSpecialFolderPathW)
+    {
+        skip("SHGetPathFromIDListW() or SHGetSpecialFolderPathW() is missing\n");
+        return;
+    }
 
-    /* Calling SHGetPathFromIDList with no pidl should return the empty string */
+    /* Calling SHGetPathFromIDListW with no pidl should return the empty string */
     wszPath[0] = 'a';
     wszPath[1] = '\0';
-    result = SHGetPathFromIDListW(NULL, wszPath);
+    result = pSHGetPathFromIDListW(NULL, wszPath);
     ok(!result, "Expected failure\n");
     ok(!wszPath[0], "Expected empty string\n");
 
-    /* Calling SHGetPathFromIDList with an empty pidl should return the desktop folder's path. */
+    /* Calling SHGetPathFromIDListW with an empty pidl should return the desktop folder's path. */
     result = pSHGetSpecialFolderPathW(NULL, wszDesktop, CSIDL_DESKTOP, FALSE);
     ok(result, "SHGetSpecialFolderPathW(CSIDL_DESKTOP) failed! Last error: %u\n", GetLastError());
     if (!result) return;
     
-    result = SHGetPathFromIDListW(pidlEmpty, wszPath);
+    result = pSHGetPathFromIDListW(pidlEmpty, wszPath);
     ok(result, "SHGetPathFromIDListW failed! Last error: %u\n", GetLastError());
     if (!result) return;
-    ok(!lstrcmpiW(wszDesktop, wszPath), "SHGetPathFromIDList didn't return desktop path for empty pidl!\n");
+    ok(!lstrcmpiW(wszDesktop, wszPath), "SHGetPathFromIDListW didn't return desktop path for empty pidl!\n");
 
-    /* MyComputer does not map to a filesystem path. SHGetPathFromIDList should fail. */
+    /* MyComputer does not map to a filesystem path. SHGetPathFromIDListW should fail. */
     hr = SHGetDesktopFolder(&psfDesktop);
     ok (SUCCEEDED(hr), "SHGetDesktopFolder failed! hr = %08x\n", hr);
     if (FAILED(hr)) return;
@@ -813,9 +840,9 @@ static void test_SHGetPathFromIDList(void)
     SetLastError(0xdeadbeef);
     wszPath[0] = 'a';
     wszPath[1] = '\0';
-    result = SHGetPathFromIDListW(pidlMyComputer, wszPath);
-    ok (!result, "SHGetPathFromIDList succeeded where it shouldn't!\n");
-    ok (GetLastError()==0xdeadbeef, "SHGetPathFromIDList shouldn't set last error! Last error: %u\n", GetLastError());
+    result = pSHGetPathFromIDListW(pidlMyComputer, wszPath);
+    ok (!result, "SHGetPathFromIDListW succeeded where it shouldn't!\n");
+    ok (GetLastError()==0xdeadbeef, "SHGetPathFromIDListW shouldn't set last error! Last error: %u\n", GetLastError());
     ok (!wszPath[0], "Expected empty path\n");
     if (result) {
         IShellFolder_Release(psfDesktop);
@@ -830,7 +857,7 @@ static void test_SHGetPathFromIDList(void)
         IShellFolder_Release(psfDesktop);
         return;
     }
-    PathAddBackslashW(wszFileName);
+    myPathAddBackslashW(wszFileName);
     lstrcatW(wszFileName, wszTestFile);
     hTestFile = CreateFileW(wszFileName, GENERIC_WRITE, 0, NULL, CREATE_NEW, 0, NULL);
     ok(hTestFile != INVALID_HANDLE_VALUE, "CreateFileW failed! Last error: %u\n", GetLastError());
@@ -867,7 +894,7 @@ static void test_SHGetPathFromIDList(void)
            "returned incorrect path for file placed on desktop\n");
     }
 
-    result = SHGetPathFromIDListW(pidlTestFile, wszPath);
+    result = pSHGetPathFromIDListW(pidlTestFile, wszPath);
     ok(result, "SHGetPathFromIDListW failed! Last error: %u\n", GetLastError());
     IMalloc_Free(ppM, pidlTestFile);
     if (!result) return;
@@ -882,9 +909,9 @@ static void test_SHGetPathFromIDList(void)
     ok(SUCCEEDED(hr), "SHGetFolderLocation failed: 0x%08x\n", hr);
 
     SetLastError(0xdeadbeef);
-    result = SHGetPathFromIDListW(pidlPrograms, wszPath);
+    result = pSHGetPathFromIDListW(pidlPrograms, wszPath);
 	IMalloc_Free(ppM, pidlPrograms);
-    ok(result, "SHGetPathFromIDList failed\n");
+    ok(result, "SHGetPathFromIDListW failed\n");
 }
 
 static void test_EnumObjects_and_CompareIDs(void)
@@ -1159,7 +1186,7 @@ static void test_FolderShortcut(void) {
 
     /* Next few lines are meant to show that children of FolderShortcuts are not FolderShortcuts,
      * but ShellFSFolders. */
-    PathAddBackslashW(wszDesktopPath);
+    myPathAddBackslashW(wszDesktopPath);
     lstrcatW(wszDesktopPath, wszSomeSubFolder);
     if (!CreateDirectoryW(wszDesktopPath, NULL)) {
         IShellFolder_Release(pShellFolder);
@@ -1342,6 +1369,113 @@ static void test_ITEMIDLIST_format(void) {
     }
 }
 
+static void testSHGetFolderPathAndSubDirA(void)
+{
+    HRESULT ret;
+    BOOL delret;
+    DWORD dwret;
+    int i;
+    static char wine[] = "wine";
+    static char winetemp[] = "wine\\temp";
+    static char appdata[MAX_PATH];
+    static char testpath[MAX_PATH];
+    static char toolongpath[MAX_PATH+1];
+
+    if(!pSHGetFolderPathA) {
+        skip("SHGetFolderPathA not present!\n");
+        return;
+    }
+    if(!SUCCEEDED(pSHGetFolderPathA(NULL, CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, appdata)))
+    {
+        skip("SHGetFolderPathA failed for CSIDL_LOCAL_APPDATA!\n");
+        return;
+    }
+
+    sprintf(testpath, "%s\\%s", appdata, winetemp);
+    delret = RemoveDirectoryA(testpath);
+    if(!delret && (ERROR_PATH_NOT_FOUND != GetLastError()) ) {
+        skip("RemoveDirectoryA(%s) failed with error %u\n", testpath, GetLastError());
+        return;
+    }
+
+    sprintf(testpath, "%s\\%s", appdata, wine);
+    delret = RemoveDirectoryA(testpath);
+    if(!delret && (ERROR_PATH_NOT_FOUND != GetLastError()) && (ERROR_FILE_NOT_FOUND != GetLastError())) {
+        skip("RemoveDirectoryA(%s) failed with error %u\n", testpath, GetLastError());
+        return;
+    }
+    for(i=0; i< MAX_PATH; i++)
+        toolongpath[i] = '0' + i % 10;
+    toolongpath[MAX_PATH] = '\0';
+
+    /* test invalid second parameter */
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | 0xff, NULL, SHGFP_TYPE_CURRENT, wine, testpath);
+    ok(E_INVALIDARG == ret, "expected E_INVALIDARG, got  %x\n", ret);
+
+    /* test invalid forth parameter */
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | CSIDL_LOCAL_APPDATA, NULL, 2, wine, testpath);
+    switch(ret) {
+        case S_OK: /* winvista */
+            ok(!strncmp(appdata, testpath, strlen(appdata)),
+                "expected %s to start with %s\n", testpath, appdata);
+            ok(!lstrcmpA(&testpath[1 + strlen(appdata)], winetemp),
+                "expected %s to end with %s\n", testpath, winetemp);
+            break;
+        case E_INVALIDARG: /* winxp, win2k3 */
+            break;
+        default:
+            ok(0, "expected S_OK or E_INVALIDARG, got  %x\n", ret);
+    }
+
+    /* test fifth parameter */
+    testpath[0] = '\0';
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, NULL, testpath);
+    ok(S_OK == ret, "expected S_OK, got %x\n", ret);
+    ok(!lstrcmpA(appdata, testpath), "expected %s, got %s\n", appdata, testpath);
+
+    testpath[0] = '\0';
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, "", testpath);
+    ok(S_OK == ret, "expected S_OK, got %x\n", ret);
+    ok(!lstrcmpA(appdata, testpath), "expected %s, got %s\n", appdata, testpath);
+
+    testpath[0] = '\0';
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, "\\", testpath);
+    ok(S_OK == ret, "expected S_OK, got %x\n", ret);
+    ok(!lstrcmpA(appdata, testpath), "expected %s, got %s\n", appdata, testpath);
+
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, toolongpath, testpath);
+    ok(HRESULT_FROM_WIN32(ERROR_FILENAME_EXCED_RANGE) == ret,
+        "expected %x, got %x\n", HRESULT_FROM_WIN32(ERROR_FILENAME_EXCED_RANGE), ret);
+
+    testpath[0] = '\0';
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_DONT_VERIFY | CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, wine, NULL);
+    ok((S_OK == ret) || (E_INVALIDARG == ret), "expected S_OK or E_INVALIDARG, got %x\n", ret);
+
+    /* test a not existing path */
+    testpath[0] = '\0';
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, winetemp, testpath);
+    ok(HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND) == ret,
+        "expected %x, got %x\n", HRESULT_FROM_WIN32(ERROR_PATH_NOT_FOUND), ret);
+
+    /* create a directory inside a not existing directory */
+    testpath[0] = '\0';
+    ret = pSHGetFolderPathAndSubDirA(NULL, CSIDL_FLAG_CREATE | CSIDL_LOCAL_APPDATA, NULL, SHGFP_TYPE_CURRENT, winetemp, testpath);
+    ok(S_OK == ret, "expected S_OK, got %x\n", ret);
+    ok(!strncmp(appdata, testpath, strlen(appdata)),
+        "expected %s to start with %s\n", testpath, appdata);
+    ok(!lstrcmpA(&testpath[1 + strlen(appdata)], winetemp),
+        "expected %s to end with %s\n", testpath, winetemp);
+    dwret = GetFileAttributes(testpath);
+    ok(FILE_ATTRIBUTE_DIRECTORY | dwret, "expected %x to contain FILE_ATTRIBUTE_DIRECTORY\n", dwret);
+
+    /* cleanup */
+    sprintf(testpath, "%s\\%s", appdata, winetemp);
+    RemoveDirectoryA(testpath);
+    sprintf(testpath, "%s\\%s", appdata, wine);
+    RemoveDirectoryA(testpath);
+}
+
+
 START_TEST(shlfolder)
 {
     init_function_pointers();
@@ -1358,6 +1492,10 @@ START_TEST(shlfolder)
     test_CallForAttributes();
     test_FolderShortcut();
     test_ITEMIDLIST_format();
+    if(pSHGetFolderPathAndSubDirA)
+        testSHGetFolderPathAndSubDirA();
+    else
+        skip("SHGetFolderPathAndSubDirA not present\n");
 
     OleUninitialize();
 }
