@@ -35,16 +35,24 @@
     ok(ret == val, "Unexpected value of '" #expr "': " #fmt " instead of " #val "\n", ret); \
 } while (0);
 
-static HMODULE hShlwapi;
-static LPSTR   (WINAPI *pStrCpyNXA)(LPSTR,LPCSTR,int);
-static LPWSTR  (WINAPI *pStrCpyNXW)(LPWSTR,LPCWSTR,int);
-static HRESULT (WINAPI *pStrRetToBSTR)(STRRET*,void*,BSTR*);
+static BOOL    (WINAPI *pIntlStrEqWorkerA)(BOOL,LPCSTR,LPCSTR,int);
+static BOOL    (WINAPI *pIntlStrEqWorkerW)(BOOL,LPCWSTR,LPCWSTR,int);
 static DWORD   (WINAPI *pSHAnsiToAnsi)(LPCSTR,LPSTR,int);
 static DWORD   (WINAPI *pSHUnicodeToUnicode)(LPCWSTR,LPWSTR,int);
+static LPSTR   (WINAPI *pStrCatBuffA)(LPSTR,LPCSTR,INT);
+static LPWSTR  (WINAPI *pStrCatBuffW)(LPWSTR,LPCWSTR,INT);
+static LPSTR   (WINAPI *pStrCpyNXA)(LPSTR,LPCSTR,int);
+static LPWSTR  (WINAPI *pStrCpyNXW)(LPWSTR,LPCWSTR,int);
+static LPSTR   (WINAPI *pStrFormatByteSize64A)(LONGLONG,LPSTR,UINT);
+static LPSTR   (WINAPI *pStrFormatKBSizeA)(LONGLONG,LPSTR,UINT);
+static LPWSTR  (WINAPI *pStrFormatKBSizeW)(LONGLONG,LPWSTR,UINT);
 static BOOL    (WINAPI *pStrIsIntlEqualA)(BOOL,LPCSTR,LPCSTR,int);
-static BOOL    (WINAPI *pIntlStrEqWorkerA)(BOOL,LPCSTR,LPCSTR,int);
 static BOOL    (WINAPI *pStrIsIntlEqualW)(BOOL,LPCWSTR,LPCWSTR,int);
-static BOOL    (WINAPI *pIntlStrEqWorkerW)(BOOL,LPCWSTR,LPCWSTR,int);
+static HRESULT (WINAPI *pStrRetToBSTR)(STRRET*,void*,BSTR*);
+static HRESULT (WINAPI *pStrRetToBufA)(STRRET*,LPCITEMIDLIST,LPSTR,UINT);
+static HRESULT (WINAPI *pStrRetToBufW)(STRRET*,LPCITEMIDLIST,LPWSTR,UINT);
+static INT     (WINAPIV *pwnsprintfA)(LPSTR,INT,LPCSTR, ...);
+static INT     (WINAPIV *pwnsprintfW)(LPWSTR,INT,LPCWSTR, ...);
 
 static int strcmpW(const WCHAR *str1, const WCHAR *str2)
 {
@@ -483,6 +491,7 @@ static void test_StrDupA(void)
    */
   lpszStr = StrDupA(NULL);
   ok(lpszStr == NULL || *lpszStr == '\0', "NULL string returned %p\n", lpszStr);
+  LocalFree((HLOCAL)lpszStr);
 }
 
 static void test_StrFormatByteSize64A(void)
@@ -490,9 +499,15 @@ static void test_StrFormatByteSize64A(void)
   char szBuff[256];
   const StrFormatSizeResult* result = StrFormatSize_results;
 
+  if (!pStrFormatByteSize64A)
+  {
+    skip("StrFormatByteSize64A() is not available. Tests skipped\n");
+    return;
+  }
+
   while(result->value)
   {
-    StrFormatByteSize64A(result->value, szBuff, 256);
+    pStrFormatByteSize64A(result->value, szBuff, 256);
 
     ok(!strcmp(result->byte_size_64, szBuff),
         "Formatted %x%08x wrong: got %s, expected %s\n",
@@ -508,9 +523,15 @@ static void test_StrFormatKBSizeW(void)
   char szBuff[256];
   const StrFormatSizeResult* result = StrFormatSize_results;
 
+  if (!pStrFormatKBSizeW)
+  {
+    skip("StrFormatKBSizeW() is not available. Tests skipped\n");
+    return;
+  }
+
   while(result->value)
   {
-    StrFormatKBSizeW(result->value, szBuffW, 256);
+    pStrFormatKBSizeW(result->value, szBuffW, 256);
     WideCharToMultiByte(0,0,szBuffW,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR),0,0);
     ok(!strcmp(result->kb_size, szBuff),
         "Formatted %x%08x wrong: got %s, expected %s\n",
@@ -524,9 +545,15 @@ static void test_StrFormatKBSizeA(void)
   char szBuff[256];
   const StrFormatSizeResult* result = StrFormatSize_results;
 
+  if (!pStrFormatKBSizeA)
+  {
+    skip("StrFormatKBSizeA() is not available. Tests skipped\n");
+    return;
+  }
+
   while(result->value)
   {
-    StrFormatKBSizeA(result->value, szBuff, 256);
+    pStrFormatKBSizeA(result->value, szBuff, 256);
 
     ok(!strcmp(result->kb_size, szBuff),
         "Formatted %x%08x wrong: got %s, expected %s\n",
@@ -560,20 +587,21 @@ static void test_StrCmpA(void)
   ok(!ChrCmpIA('b', 'B'), "ChrCmpIA is not case-insensitive\n");
   ok(ChrCmpIA('a', 'z'), "ChrCmpIA believes that a == z!\n");
 
-  pStrIsIntlEqualA = (void *)GetProcAddress(hShlwapi, "StrIsIntlEqualA");
-  pIntlStrEqWorkerA = (void *)GetProcAddress(hShlwapi, "IntlStrEqWorkerA");
+  if (pStrIsIntlEqualA)
+  {
+    ok(pStrIsIntlEqualA(FALSE, str1, str2, 5), "StrIsIntlEqualA(FALSE,...) isn't case-insensitive\n");
+    ok(!pStrIsIntlEqualA(TRUE, str1, str2, 5), "StrIsIntlEqualA(TRUE,...) isn't case-sensitive\n");
+  }
+  else
+    skip("StrIsIntlEqualA() is not available. Tests skipped\n");
 
-  if (!pStrIsIntlEqualA)
-    return;
-
-  ok(pStrIsIntlEqualA(FALSE, str1, str2, 5), "StrIsIntlEqualA(FALSE,...) isn't case-insensitive\n");
-  ok(!pStrIsIntlEqualA(TRUE, str1, str2, 5), "StrIsIntlEqualA(TRUE,...) isn't case-sensitive\n");
-
-  if (!pIntlStrEqWorkerA)
-    return;
-
-  ok(pIntlStrEqWorkerA(FALSE, str1, str2, 5), "IntlStrEqWorkerA(FALSE,...) isn't case-insensitive\n");
-  ok(!pIntlStrEqWorkerA(TRUE, str1, str2, 5), "pIntlStrEqWorkerA(TRUE,...) isn't case-sensitive\n");
+  if (pIntlStrEqWorkerA)
+  {
+    ok(pIntlStrEqWorkerA(FALSE, str1, str2, 5), "IntlStrEqWorkerA(FALSE,...) isn't case-insensitive\n");
+    ok(!pIntlStrEqWorkerA(TRUE, str1, str2, 5), "pIntlStrEqWorkerA(TRUE,...) isn't case-sensitive\n");
+  }
+  else
+    skip("IntlStrEqWorkerA() is not available. Tests skipped\n");
 }
 
 static void test_StrCmpW(void)
@@ -586,26 +614,27 @@ static void test_StrCmpW(void)
   ok(!ChrCmpIW('b', 'B'), "ChrCmpIW is not case-insensitive\n");
   ok(ChrCmpIW('a', 'z'), "ChrCmpIW believes that a == z!\n");
 
-  pStrIsIntlEqualW = (void *)GetProcAddress(hShlwapi, "StrIsIntlEqualW");
-  pIntlStrEqWorkerW = (void *)GetProcAddress(hShlwapi, "IntlStrEqWorkerW");
+  if (pStrIsIntlEqualW)
+  {
+    ok(pStrIsIntlEqualW(FALSE, str1, str2, 5), "StrIsIntlEqualW(FALSE,...) isn't case-insensitive\n");
+    ok(!pStrIsIntlEqualW(TRUE, str1, str2, 5), "StrIsIntlEqualW(TRUE,...) isn't case-sensitive\n");
+  }
+  else
+    skip("StrIsIntlEqualW() is not available. Tests skipped\n");
 
-  if (!pStrIsIntlEqualW)
-    return;
-
-  ok(pStrIsIntlEqualW(FALSE, str1, str2, 5), "StrIsIntlEqualW(FALSE,...) isn't case-insensitive\n");
-  ok(!pStrIsIntlEqualW(TRUE, str1, str2, 5), "StrIsIntlEqualW(TRUE,...) isn't case-sensitive\n");
-
-  if (!pIntlStrEqWorkerW)
-    return;
-
-  ok(pIntlStrEqWorkerW(FALSE, str1, str2, 5), "IntlStrEqWorkerW(FALSE,...) isn't case-insensitive\n");
-  ok(!pIntlStrEqWorkerW(TRUE, str1, str2, 5), "IntlStrEqWorkerW(TRUE,...) isn't case-sensitive\n");
+  if (pIntlStrEqWorkerW)
+  {
+    ok(pIntlStrEqWorkerW(FALSE, str1, str2, 5), "IntlStrEqWorkerW(FALSE,...) isn't case-insensitive\n");
+    ok(!pIntlStrEqWorkerW(TRUE, str1, str2, 5), "IntlStrEqWorkerW(TRUE,...) isn't case-sensitive\n");
+  }
+  else
+    skip("IntlStrEqWorkerW() is not available. Tests skipped\n");
 }
 
 static WCHAR *CoDupStrW(const char* src)
 {
   INT len = MultiByteToWideChar(CP_ACP, 0, src, -1, NULL, 0);
-  WCHAR* szTemp = (WCHAR*)CoTaskMemAlloc(len * sizeof(WCHAR));
+  WCHAR* szTemp = CoTaskMemAlloc(len * sizeof(WCHAR));
   MultiByteToWideChar(CP_ACP, 0, src, -1, szTemp, len);
   return szTemp;
 }
@@ -618,8 +647,11 @@ static void test_StrRetToBSTR(void)
     STRRET strret;
     HRESULT ret;
 
-    pStrRetToBSTR = (void *)GetProcAddress(hShlwapi, "StrRetToBSTR");
-    if (!pStrRetToBSTR) return;
+    if (!pStrRetToBSTR)
+    {
+        skip("StrRetToBSTR() is not available. Tests skipped\n");
+        return;
+    }
 
     strret.uType = STRRET_WSTR;
     U(strret).pOleStr = CoDupStrW("Test");
@@ -656,9 +688,11 @@ static void test_StrCpyNXA(void)
   LPSTR lpszRes;
   char dest[8];
 
-  pStrCpyNXA = (void *)GetProcAddress(hShlwapi, (LPSTR)399);
   if (!pStrCpyNXA)
+  {
+    skip("StrCpyNXA() is not available. Tests skipped\n");
     return;
+  }
 
   memset(dest, '\n', sizeof(dest));
   lpszRes = pStrCpyNXA(dest, lpSrc, sizeof(dest)/sizeof(dest[0]));
@@ -675,14 +709,16 @@ static void test_StrCpyNXW(void)
   LPWSTR lpszRes;
   WCHAR dest[8];
 
-  pStrCpyNXW = (void *)GetProcAddress(hShlwapi, (LPSTR)400);
   if (!pStrCpyNXW)
+  {
+    skip("StrCpyNXW() is not available. Tests skipped\n");
     return;
+  }
 
   memcpy(dest, lpInit, sizeof(lpInit));
   lpszRes = pStrCpyNXW(dest, lpSrc, sizeof(dest)/sizeof(dest[0]));
   ok(lpszRes == dest + 5 && !memcmp(dest, lpRes, sizeof(dest)),
-       "StrCpyNXA: expected %p, \"hello\\0\\n\\n\", got %p, \"%d,%d,%d,%d,%d,%d,%d,%d\"\n",
+       "StrCpyNXW: expected %p, \"hello\\0\\n\\n\", got %p, \"%d,%d,%d,%d,%d,%d,%d,%d\"\n",
        dest + 5, lpszRes, dest[0], dest[1], dest[2], dest[3], dest[4], dest[5], dest[6], dest[7]);
 }
 
@@ -730,9 +766,11 @@ static void test_SHAnsiToAnsi(void)
   char dest[8];
   DWORD dwRet;
 
-  pSHAnsiToAnsi = (void *)GetProcAddress(hShlwapi, (LPSTR)345);
   if (!pSHAnsiToAnsi)
+  {
+    skip("SHAnsiToAnsi() is not available. Tests skipped\n");
     return;
+  }
 
   memset(dest, '\n', sizeof(dest));
   dwRet = pSHAnsiToAnsi("hello", dest, sizeof(dest)/sizeof(dest[0]));
@@ -749,9 +787,11 @@ static void test_SHUnicodeToUnicode(void)
   WCHAR dest[8];
   DWORD dwRet;
 
-  pSHUnicodeToUnicode = (void *)GetProcAddress(hShlwapi, (LPSTR)346);
   if (!pSHUnicodeToUnicode)
+  {
+    skip("SHUnicodeToUnicode() is not available. Tests skipped\n");
     return;
+  }
 
   memcpy(dest, lpInit, sizeof(lpInit));
   dwRet = pSHUnicodeToUnicode(lpSrc, dest, sizeof(dest)/sizeof(dest[0]));
@@ -781,46 +821,80 @@ static void test_StrXXX_overflows(void)
     expect_eq(StrCpyNA(buf, str1, 10), buf, PCHAR, "%p");
     expect_eq(buf[9], 0, CHAR, "%x");
     expect_eq(buf[10], '\xbf', CHAR, "%x");
-    expect_eq(StrCatBuffA(buf, str1, 100), buf, PCHAR, "%p");
-    expect_eq(buf[99], 0, CHAR, "%x");
-    expect_eq(buf[100], '\xbf', CHAR, "%x");
+
+    if (pStrCatBuffA)
+    {
+        expect_eq(pStrCatBuffA(buf, str1, 100), buf, PCHAR, "%p");
+        expect_eq(buf[99], 0, CHAR, "%x");
+        expect_eq(buf[100], '\xbf', CHAR, "%x");
+    }
+    else
+        skip("StrCatBuffA() is not available. Tests skipped\n");
 
     memset(wbuf, 0xbf, sizeof(wbuf));
     expect_eq(StrCpyNW(wbuf, wstr1, 10), wbuf, PWCHAR, "%p");
     expect_eq(wbuf[9], 0, WCHAR, "%x");
     expect_eq(wbuf[10], (WCHAR)0xbfbf, WCHAR, "%x");
-    expect_eq(StrCatBuffW(wbuf, wstr1, 100), wbuf, PWCHAR, "%p");
-    expect_eq(wbuf[99], 0, WCHAR, "%x");
-    expect_eq(wbuf[100], (WCHAR)0xbfbf, WCHAR, "%x");
 
-    memset(wbuf, 0xbf, sizeof(wbuf));
-    strret.uType = STRRET_WSTR;
-    U(strret).pOleStr = StrDupW(wstr1);
-    expect_eq(StrRetToBufW(&strret, NULL, wbuf, 10), S_OK, HRESULT, "%x");
-    expect_eq(wbuf[9], 0, WCHAR, "%x");
-    expect_eq(wbuf[10], (WCHAR)0xbfbf, WCHAR, "%x");
+    if (pStrCatBuffW)
+    {
+        expect_eq(pStrCatBuffW(wbuf, wstr1, 100), wbuf, PWCHAR, "%p");
+        expect_eq(wbuf[99], 0, WCHAR, "%x");
+        expect_eq(wbuf[100], (WCHAR)0xbfbf, WCHAR, "%x");
+    }
+    else
+        skip("StrCatBuffW() is not available. Tests skipped\n");
 
-    memset(buf, 0xbf, sizeof(buf));
-    strret.uType = STRRET_CSTR;
-    StrCpyN(U(strret).cStr, str1, MAX_PATH);
-    expect_eq(StrRetToBufA(&strret, NULL, buf, 10), S_OK, HRESULT, "%x");
-    expect_eq(buf[9], 0, CHAR, "%x");
-    expect_eq(buf[10], (CHAR)0xbf, CHAR, "%x");
+    if (pStrRetToBufW)
+    {
+        memset(wbuf, 0xbf, sizeof(wbuf));
+        strret.uType = STRRET_WSTR;
+        U(strret).pOleStr = StrDupW(wstr1);
+        expect_eq(pStrRetToBufW(&strret, NULL, wbuf, 10), S_OK, HRESULT, "%x");
+        expect_eq(wbuf[9], 0, WCHAR, "%x");
+        expect_eq(wbuf[10], (WCHAR)0xbfbf, WCHAR, "%x");
+    }
+    else
+        skip("StrRetToBufW() is not available. Tests skipped\n");
 
-    memset(buf, 0xbf, sizeof(buf));
-    ret = wnsprintfA(buf, 10, "%s", str1);
-    todo_wine ok(ret == 9, "Unexpected wsnprintfA return %d, expected 9\n", ret);
-    expect_eq(buf[9], 0, CHAR, "%x");
-    expect_eq(buf[10], (CHAR)0xbf, CHAR, "%x");
-    memset(wbuf, 0xbf, sizeof(wbuf));
-    ret = wnsprintfW(wbuf, 10, fmt, wstr1);
-    todo_wine ok(ret == 9, "Unexpected wsnprintfW return %d, expected 9\n", ret);
-    expect_eq(wbuf[9], 0, WCHAR, "%x");
-    expect_eq(wbuf[10], (WCHAR)0xbfbf, WCHAR, "%x");
+    if (pStrRetToBufA)
+    {
+        memset(buf, 0xbf, sizeof(buf));
+        strret.uType = STRRET_CSTR;
+        StrCpyN(U(strret).cStr, str1, MAX_PATH);
+        expect_eq(pStrRetToBufA(&strret, NULL, buf, 10), S_OK, HRESULT, "%x");
+        expect_eq(buf[9], 0, CHAR, "%x");
+        expect_eq(buf[10], (CHAR)0xbf, CHAR, "%x");
+    }
+    else
+        skip("StrRetToBufA() is not available. Tests skipped\n");
+
+    if (pwnsprintfA)
+    {
+        memset(buf, 0xbf, sizeof(buf));
+        ret = pwnsprintfA(buf, 10, "%s", str1);
+        todo_wine ok(ret == 9, "Unexpected wsnprintfA return %d, expected 9\n", ret);
+        expect_eq(buf[9], 0, CHAR, "%x");
+        expect_eq(buf[10], (CHAR)0xbf, CHAR, "%x");
+    }
+    else
+        skip("wnsprintfA() is not available. Tests skipped\n");
+
+    if (pwnsprintfW)
+    {
+        memset(wbuf, 0xbf, sizeof(wbuf));
+        ret = pwnsprintfW(wbuf, 10, fmt, wstr1);
+        todo_wine ok(ret == 9, "Unexpected wsnprintfW return %d, expected 9\n", ret);
+        expect_eq(wbuf[9], 0, WCHAR, "%x");
+        expect_eq(wbuf[10], (WCHAR)0xbfbf, WCHAR, "%x");
+    }
+    else
+        skip("wnsprintfW() is not available. Tests skipped\n");
 }
 
 START_TEST(string)
 {
+  HMODULE hShlwapi;
   TCHAR thousandDelim[8];
   TCHAR decimalDelim[8];
   CoInitialize(0);
@@ -829,8 +903,24 @@ START_TEST(string)
   GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SDECIMAL, decimalDelim, 8);
 
   hShlwapi = GetModuleHandleA("shlwapi");
-  if (!hShlwapi)
-     return;
+  pIntlStrEqWorkerA = (void *)GetProcAddress(hShlwapi, "IntlStrEqWorkerA");
+  pIntlStrEqWorkerW = (void *)GetProcAddress(hShlwapi, "IntlStrEqWorkerW");
+  pSHAnsiToAnsi = (void *)GetProcAddress(hShlwapi, (LPSTR)345);
+  pSHUnicodeToUnicode = (void *)GetProcAddress(hShlwapi, (LPSTR)346);
+  pStrCatBuffA = (void *)GetProcAddress(hShlwapi, "StrCatBuffA");
+  pStrCatBuffW = (void *)GetProcAddress(hShlwapi, "StrCatBuffW");
+  pStrCpyNXA = (void *)GetProcAddress(hShlwapi, (LPSTR)399);
+  pStrCpyNXW = (void *)GetProcAddress(hShlwapi, (LPSTR)400);
+  pStrFormatByteSize64A = (void *)GetProcAddress(hShlwapi, "StrFormatByteSize64A");
+  pStrFormatKBSizeA = (void *)GetProcAddress(hShlwapi, "StrFormatKBSizeA");
+  pStrFormatKBSizeW = (void *)GetProcAddress(hShlwapi, "StrFormatKBSizeW");
+  pStrIsIntlEqualA = (void *)GetProcAddress(hShlwapi, "StrIsIntlEqualA");
+  pStrIsIntlEqualW = (void *)GetProcAddress(hShlwapi, "StrIsIntlEqualW");
+  pStrRetToBSTR = (void *)GetProcAddress(hShlwapi, "StrRetToBSTR");
+  pStrRetToBufA = (void *)GetProcAddress(hShlwapi, "StrRetToBufA");
+  pStrRetToBufW = (void *)GetProcAddress(hShlwapi, "StrRetToBufW");
+  pwnsprintfA = (void *)GetProcAddress(hShlwapi, "wnsprintfA");
+  pwnsprintfW = (void *)GetProcAddress(hShlwapi, "wnsprintfW");
 
   test_StrChrA();
   test_StrChrW();
