@@ -24,8 +24,6 @@
 /*
  * Theory:
  *
- * http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnwui/html/msdn_icons.asp
- *
  * Cursors and icons are stored in a global heap block, with the
  * following layout:
  *
@@ -39,8 +37,6 @@
  * the X client instead of in the server like other bitmaps; however,
  * some programs (notably Paint Brush) expect to be able to manipulate
  * the bits directly :-(
- *
- * FIXME: what are we going to do with animation and color (bpp > 1) cursors ?!
  */
 
 #include "config.h"
@@ -53,9 +49,7 @@
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
-#include "wownt32.h"
 #include "winerror.h"
-#include "excpt.h"
 #include "wine/winbase16.h"
 #include "wine/winuser16.h"
 #include "wine/exception.h"
@@ -691,7 +685,7 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
     hotspot.y = ICON_HOTSPOT;
 
     TRACE_(cursor)("%p (%u bytes), ver %08x, %ix%i %s %s\n",
-                   bits, cbSize, (unsigned)dwVersion, width, height,
+                   bits, cbSize, dwVersion, width, height,
                                   bIcon ? "icon" : "cursor", (cFlag & LR_MONOCHROME) ? "mono" : "" );
     if (dwVersion == 0x00020000)
     {
@@ -707,12 +701,6 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
         hotspot = *pt;
         bmi = (BITMAPINFO *)(pt + 1);
     }
-    size = bitmap_info_size( bmi, DIB_RGB_COLORS );
-
-    if (!width) width = bmi->bmiHeader.biWidth;
-    if (!height) height = bmi->bmiHeader.biHeight/2;
-    DoStretch = (bmi->bmiHeader.biHeight/2 != height) ||
-      (bmi->bmiHeader.biWidth != width);
 
     /* Check bitmap header */
 
@@ -724,6 +712,13 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
           return 0;
     }
 
+    size = bitmap_info_size( bmi, DIB_RGB_COLORS );
+
+    if (!width) width = bmi->bmiHeader.biWidth;
+    if (!height) height = bmi->bmiHeader.biHeight/2;
+    DoStretch = (bmi->bmiHeader.biHeight/2 != height) ||
+      (bmi->bmiHeader.biWidth != width);
+
     if (!screen_dc) screen_dc = CreateDCW( DISPLAYW, NULL, NULL, NULL );
     if (screen_dc)
     {
@@ -873,7 +868,7 @@ HICON WINAPI CreateIconFromResourceEx( LPBYTE bits, UINT cbSize,
 
     DeleteObject( hAndBits );
     DeleteObject( hXorBits );
-    return HICON_32((HICON16)hObj);
+    return HICON_32(hObj);
 }
 
 
@@ -903,6 +898,13 @@ static HICON CURSORICON_LoadFromFile( LPCWSTR filename,
     if (!bits)
         return hIcon;
 
+    /* Check for .ani. */
+    if (memcmp( bits, "RIFF", 4 ) == 0)
+    {
+        FIXME("No support for .ani cursors.\n");
+        goto end;
+    }
+
     dir = (CURSORICONFILEDIR*) bits;
     if ( filesize < sizeof(*dir) )
         goto end;
@@ -911,7 +913,7 @@ static HICON CURSORICON_LoadFromFile( LPCWSTR filename,
         goto end;
 
     if ( fCursor )
-        entry = CURSORICON_FindBestCursorFile( dir, width, height, 1 );
+        entry = CURSORICON_FindBestCursorFile( dir, width, height, colors );
     else
         entry = CURSORICON_FindBestIconFile( dir, width, height, colors );
 
@@ -1129,7 +1131,7 @@ static HICON CURSORICON_ExtCopy(HICON hIcon, UINT nType,
             }
             else
             {
-                pDirEntry = (CURSORICONDIRENTRY *)CURSORICON_FindBestCursorRes(
+                pDirEntry = CURSORICON_FindBestCursorRes(
                                 pDir, iDesiredCX, iDesiredCY, 1);
             }
 
@@ -1349,7 +1351,7 @@ WORD WINAPI DestroyIcon32( HGLOBAL16 handle, UINT16 flags )
     if ( get_user_thread_info()->cursor == HICON_32(handle) )
     {
         WARN_(cursor)("Destroying active cursor!\n" );
-        SetCursor( 0 );
+        return FALSE;
     }
 
     /* Try shared cursor/icon first */
@@ -1796,6 +1798,8 @@ HICON WINAPI CreateIconIndirect(PICONINFO iconinfo)
            iconinfo->hbmColor, iconinfo->hbmMask,
            iconinfo->xHotspot, iconinfo->yHotspot, iconinfo->fIcon);
 
+    if (!iconinfo->hbmMask) return 0;
+
     if (iconinfo->hbmColor)
     {
         GetObjectW( iconinfo->hbmColor, sizeof(bmpXor), &bmpXor );
