@@ -576,8 +576,7 @@ static BOOL UITOOLS95_DrawRectEdge(HDC hdc, LPRECT rc,
  */
 BOOL WINAPI DrawEdge( HDC hdc, LPRECT rc, UINT edge, UINT flags )
 {
-    TRACE("%p %d,%d-%d,%d %04x %04x\n",
-          hdc, rc->left, rc->top, rc->right, rc->bottom, edge, flags );
+    TRACE("%p %s %04x %04x\n", hdc, wine_dbgstr_rect(rc), edge, flags );
 
     if(flags & BF_DIAGONAL)
       return UITOOLS95_DrawDiagEdge(hdc, rc, edge, flags);
