@@ -908,6 +908,15 @@ void WINAPI ControlPanelInfo16( INT16 nInfoType, WORD wData, LPSTR lpBuffer )
 }
 
 
+/***********************************************************************
+ *           OldSetDeskPattern   (USER.279)
+ */
+BOOL16 WINAPI SetDeskPattern16(void)
+{
+    return SystemParametersInfoA( SPI_SETDESKPATTERN, -1, NULL, FALSE );
+}
+
+
 /***********************************************************************
  *		GetSysColorBrush (USER.281)
  */
@@ -982,6 +991,15 @@ WORD WINAPI GetFreeSystemResources16( WORD resType )
 }
 
 
+/***********************************************************************
+ *           SetDeskWallPaper   (USER.285)
+ */
+BOOL16 WINAPI SetDeskWallPaper16( LPCSTR filename )
+{
+    return SetDeskWallPaper( filename );
+}
+
+
 /***********************************************************************
  *		GetClipCursor (USER.309)
  */
