@@ -184,7 +184,19 @@ static void winstation_init(void)
     if (buffer || !GetProcessWindowStation())
     {
         handle = CreateWindowStationW( winstation ? winstation : WinSta0, 0, WINSTA_ALL_ACCESS, NULL );
-        if (handle) SetProcessWindowStation( handle );
+        if (handle)
+        {
+            SetProcessWindowStation( handle );
+            /* only WinSta0 is visible */
+            if (!winstation || !strcmpiW( winstation, WinSta0 ))
+            {
+                USEROBJECTFLAGS flags;
+                flags.fInherit  = FALSE;
+                flags.fReserved = FALSE;
+                flags.dwFlags   = WSF_VISIBLE;
+                SetUserObjectInformationW( handle, UOI_FLAGS, &flags, sizeof(flags) );
+            }
+        }
     }
     if (buffer || !GetThreadDesktop( GetCurrentThreadId() ))
     {
@@ -251,6 +263,7 @@ static void thread_detach(void)
 
     WIN_DestroyThreadWindows( get_user_thread_info()->desktop );
     CloseHandle( get_user_thread_info()->server_queue );
+    HeapFree( GetProcessHeap(), 0, get_user_thread_info()->wmchar_data );
 
     exiting_thread_id = 0;
 }
@@ -318,3 +331,12 @@ BOOL WINAPI ExitWindowsEx( UINT flags, DWORD reason )
     CloseHandle( pi.hThread );
     return TRUE;
 }
+
+/***********************************************************************
+ *		RegisterServicesProcess (USER32.@)
+ */
+int WINAPI RegisterServicesProcess(DWORD ServicesProcessId)
+{
+    FIXME("(0x%x): stub\n", ServicesProcessId);
+    return 0;
+}
