@@ -34,6 +34,50 @@
 #include <winnls.h>
 #include <usp10.h>
 
+static void test_ScriptShape(HDC hdc)
+{
+    static const WCHAR test1[] = {'t', 'e', 's', 't',0};
+    BOOL ret;
+    HRESULT hr;
+    SCRIPT_CACHE sc = NULL;
+    WORD glyphs[4];
+    SCRIPT_VISATTR attrs[4];
+    SCRIPT_ITEM items[2];
+    int nb, widths[4];
+
+    hr = ScriptItemize(NULL, 4, 2, NULL, NULL, items, NULL);
+    ok(hr == E_INVALIDARG, "ScriptItemize should return E_INVALIDARG not %08x\n", hr);
+
+    hr = ScriptItemize(test1, 4, 2, NULL, NULL, NULL, NULL);
+    ok(hr == E_INVALIDARG, "ScriptItemize should return E_INVALIDARG not %08x\n", hr);
+
+    hr = ScriptItemize(test1, 4, 2, NULL, NULL, items, NULL);
+    ok(!hr, "ScriptItemize should return S_OK not %08x\n", hr);
+    ok(items[0].a.fNoGlyphIndex == FALSE, "fNoGlyphIndex TRUE\n");
+
+    hr = ScriptShape(hdc, &sc, test1, 4, 4, &items[0].a, glyphs, NULL, NULL, &nb);
+    ok(hr == E_INVALIDARG, "ScriptShape should return E_INVALIDARG not %08x\n", hr);
+
+    hr = ScriptShape(hdc, &sc, test1, 4, 4, &items[0].a, glyphs, NULL, attrs, NULL);
+    ok(hr == E_INVALIDARG, "ScriptShape should return E_INVALIDARG not %08x\n", hr);
+
+    hr = ScriptShape(hdc, &sc, test1, 4, 4, &items[0].a, glyphs, NULL, attrs, &nb);
+    ok(!hr, "ScriptShape should return S_OK not %08x\n", hr);
+    ok(items[0].a.fNoGlyphIndex == FALSE, "fNoGlyphIndex TRUE\n");
+
+    hr = ScriptPlace(hdc, &sc, glyphs, 4, NULL, &items[0].a, widths, NULL, NULL);
+    ok(hr == E_INVALIDARG, "ScriptPlace should return E_INVALIDARG not %08x\n", hr);
+
+    hr = ScriptPlace(hdc, &sc, glyphs, 4, attrs, &items[0].a, widths, NULL, NULL);
+    ok(!hr, "ScriptPlace should return S_OK not %08x\n", hr);
+    ok(items[0].a.fNoGlyphIndex == FALSE, "fNoGlyphIndex TRUE\n");
+
+    ret = ExtTextOutW(hdc, 1, 1, 0, NULL, glyphs, 4, widths);
+    ok(ret, "ExtTextOutW should return TRUE\n");
+
+    ScriptFreeCache(&sc);
+}
+
 static void test_ScriptItemIzeShapePlace(HDC hdc, unsigned short pwOutGlyphs[256])
 {
     HRESULT         hr;
@@ -152,7 +196,7 @@ static void test_ScriptItemIzeShapePlace(HDC hdc, unsigned short pwOutGlyphs[256
         cMaxItems = 255;
         hr = ScriptItemize(TestItem2, cInChars, cMaxItems, NULL, NULL, pItem, &pcItems);
         ok (hr == 0, "ScriptItemize should return 0, returned %08x\n", hr);
-        /*  This test is for the intertrim operation of ScriptItemize where only one SCRIPT_ITEM is *
+        /*  This test is for the interim operation of ScriptItemize where only one SCRIPT_ITEM is *
          *  returned.                                                                               */
         ok (pItem[0].iCharPos == 0 && pItem[1].iCharPos == cInChars,
                             "Start pos not = 0 (%d) or end pos not = %d (%d)\n",
@@ -279,6 +323,7 @@ static void test_ScriptGetCMap(HDC hdc, unsigned short pwOutGlyphs[256])
     ok( hr == S_OK, "ScriptGetCMap(NULL,&psc,NULL,0,0,NULL), expected S_OK, "
                     "got %08x\n", hr);
     ok( psc != NULL, "ScritpGetCMap expected psc to be not NULL\n");
+    ScriptFreeCache( &psc);
 
     /* Set psc to NULL, to be able to check if a pointer is returned in psc */
     psc = NULL;
@@ -336,10 +381,11 @@ static void test_ScriptGetFontProperties(HDC hdc)
     ok( hr == E_INVALIDARG, "(hdc,&psc,NULL), expected E_INVALIDARG, got %08x\n", hr);
     ok( psc == NULL, "Expected psc to be NULL, got %p\n", psc);
 
-    /* Pass an uninitialized sfp */
+    /* Pass an invalid sfp */
     psc = NULL;
+    sfp.cBytes = sizeof(SCRIPT_FONTPROPERTIES) - 1;
     hr = ScriptGetFontProperties(hdc,&psc,&sfp);
-    ok( hr == E_INVALIDARG, "(hdc,&psc,&sfp) partly uninitialized, expected E_INVALIDARG, got %08x\n", hr);
+    ok( hr == E_INVALIDARG, "(hdc,&psc,&sfp) invalid, expected E_INVALIDARG, got %08x\n", hr);
     ok( psc != NULL, "Expected a pointer in psc, got NULL\n");
     ScriptFreeCache(&psc);
     ok( psc == NULL, "Expected psc to be NULL, got %p\n", psc);
@@ -622,7 +668,8 @@ static void test_ScriptString(HDC hdc)
     int             MaxSel = 0;
     BOOL            Disabled = FALSE;
     const int      *clip_len;
-    UINT           *order, i;
+    int            i;
+    UINT           *order;
 
 
     Charset = -1;     /* this flag indicates unicode input */
@@ -637,6 +684,7 @@ static void test_ScriptString(HDC hdc)
                               ReqWidth, &Control, &State, Dx, &Tabdef,
                               &InClass, &ssa);
     ok(hr == S_OK, "ScriptStringAnalyse should return S_OK not %08x\n", hr);
+    ScriptStringFree(&ssa);
 
     /* test makes sure that a call with a valid pssa still works */
     hr = ScriptStringAnalyse( hdc, teststr, len, Glyphs, Charset, Flags,
@@ -697,7 +745,7 @@ static void test_ScriptStringXtoCP_CPtoX(HDC hdc)
     BOOL            fTrailing;
 
     /* Test with hdc, this should be a valid test
-     * Here we generrate an SCRIPT_STRING_ANALYSIS that will be used as input to the
+     * Here we generate an SCRIPT_STRING_ANALYSIS that will be used as input to the
      * following character positions to X and X to character position functions.
      */
     hr = ScriptStringAnalyse( hdc, String, String_len, Glyphs, Charset, Flags,
@@ -790,7 +838,7 @@ static void test_ScriptStringXtoCP_CPtoX(HDC hdc)
                                   iTrailing, X);
 
         /*
-         * Cleanup the the SSA for the next round of tests
+         * Cleanup the SSA for the next round of tests
          */
         hr = ScriptStringFree(&ssa);
         ok(hr == S_OK, "ScriptStringFree should return S_OK not %08x\n", hr);
@@ -841,8 +889,9 @@ static void test_ScriptCacheGetHeight(HDC hdc)
 
     hr = ScriptCacheGetHeight(hdc, &sc, &height);
     ok(hr == S_OK, "expected S_OK, got 0x%08x\n", hr);
-
     ok(height > 0, "expected height > 0\n");
+
+    ScriptFreeCache(&sc);
 }
 
 static void test_ScriptGetGlyphABCWidth(HDC hdc)
@@ -864,6 +913,8 @@ static void test_ScriptGetGlyphABCWidth(HDC hdc)
 
     hr = ScriptGetGlyphABCWidth(hdc, &sc, 'a', &abc);
     ok(hr == S_OK, "expected S_OK, got 0x%08x\n", hr);
+
+    ScriptFreeCache(&sc);
 }
 
 static void test_ScriptLayout(void)
@@ -1215,7 +1266,7 @@ START_TEST(usp10)
     hdc = GetDC(hwnd);                                      /* We now have a hdc             */
     ok( hdc != NULL, "HDC failed to be created %p\n", hdc);
 
-    memset(&lf, 0, sizeof(HFONT));
+    memset(&lf, 0, sizeof(LOGFONTA));
     lstrcpyA(lf.lfFaceName, "Symbol");
     lf.lfHeight = 10;
     lf.lfWeight = 3;
@@ -1227,6 +1278,7 @@ START_TEST(usp10)
     test_ScriptGetCMap(hdc, pwOutGlyphs);
     test_ScriptCacheGetHeight(hdc);
     test_ScriptGetGlyphABCWidth(hdc);
+    test_ScriptShape(hdc);
 
     test_ScriptGetFontProperties(hdc);
     test_ScriptTextOut(hdc);
