@@ -18,12 +18,13 @@
 
 #include <stdarg.h>
 #include <stdio.h>
+#include <assert.h>
 
-#include "wine/test.h"
 #include "windef.h"
 #include "winbase.h"
 #include "winerror.h"
 #include "winver.h"
+#include "wine/test.h"
 
 #define MY_LAST_ERROR ((DWORD)-1)
 #define EXPECT_BAD_PATH__NOT_FOUND \
@@ -43,6 +44,18 @@
 	"ERROR_PATH_NOT_FOUND (NT4)/ERROR_FILE_NOT_FOUND (2k3)" \
 	"expected, got %u\n", GetLastError());
 
+static void create_file(const CHAR *name)
+{
+    HANDLE file;
+    DWORD written;
+
+    file = CreateFileA(name, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "Failure to open file %s\n", name);
+    WriteFile(file, name, strlen(name), &written, NULL);
+    WriteFile(file, "\n", strlen("\n"), &written, NULL);
+    CloseHandle(file);
+}
+
 static void test_info_size(void)
 {   DWORD hdl, retval;
     char mypath[MAX_PATH] = "";
@@ -152,6 +165,19 @@ static void test_info_size(void)
     }
     else
 	trace("skipping GetModuleFileNameA(NULL,..) failed\n");
+
+    create_file("test.txt");
+
+    /* no version info */
+    SetLastError(0xdeadbeef);
+    hdl = 0xcafe;
+    retval = GetFileVersionInfoSizeA("test.txt", &hdl);
+    ok(retval == 0, "Expected 0, got %d\n", retval);
+    ok(hdl == 0, "Expected 0, got %d\n", hdl);
+    ok(GetLastError() == ERROR_RESOURCE_DATA_NOT_FOUND,
+       "Expected ERROR_RESOURCE_DATA_NOT_FOUND, got %d\n", GetLastError());
+
+    DeleteFileA("test.txt");
 }
 
 static void VersionDwordLong2String(DWORDLONG Version, LPSTR lpszVerString)
@@ -263,8 +289,8 @@ static void test_32bit_win(void)
     WCHAR FileDescriptionW[] = { '\\','S','t','r','i','n','g','F','i','l','e','I','n','f','o',
                                 '\\','0','4','0','9','0','4','E','4',
                                 '\\','F','i','l','e','D','e','s','c','r','i','p','t','i','o','n', 0 };
-    char WineFileDescriptionA[] = "Wine version test";
-    WCHAR WineFileDescriptionW[] = { 'W','i','n','e',' ','v','e','r','s','i','o','n',' ','t','e','s','t', 0 };
+    char WineFileDescriptionA[] = "FileDescription";
+    WCHAR WineFileDescriptionW[] = { 'F','i','l','e','D','e','s','c','r','i','p','t','i','o','n', 0 };
     BOOL is_unicode_enabled = TRUE;
 
     /* A copy from dlls/version/info.c */
@@ -289,6 +315,7 @@ static void test_32bit_win(void)
      */
 
     /* First get the versioninfo via the W versions */
+    SetLastError(0xdeadbeef);
     GetModuleFileNameW(NULL, mypathW, MAX_PATH);
     if (GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
     {
@@ -315,7 +342,7 @@ static void test_32bit_win(void)
         ok( !memcmp(pVersionInfoA, pVersionInfoW, retvalA), "Both structs should be the same, they aren't\n");
     }
 
-    /* The structs on Windows are bigger then just the struct for the basic information. The total struct
+    /* The structs on Windows are bigger than just the struct for the basic information. The total struct
      * contains also an empty part, which is used for converted strings. The converted strings are a result
      * of calling VerQueryValueA on a 32bit resource and calling VerQueryValueW on a 16bit resource.
      * The first WORD of the structure (wLength) shows the size of the base struct. The total struct size depends
@@ -388,18 +415,20 @@ static void test_32bit_win(void)
 
     retA = VerQueryValueA( pVersionInfoA, FileDescriptionA, (LPVOID *)&pBufA, &uiLengthA );
     ok (retA, "VerQueryValueA failed: GetLastError = %u\n", GetLastError());
-    ok( !lstrcmpA(WineFileDescriptionA, pBufA), "FileDescription should have been 'Wine version test'\n");
+    ok( !lstrcmpA(WineFileDescriptionA, pBufA), "expected '%s' got '%s'\n",
+        WineFileDescriptionA, pBufA);
 
     /* Test a second time */
     retA = VerQueryValueA( pVersionInfoA, FileDescriptionA, (LPVOID *)&pBufA, &uiLengthA );
     ok (retA, "VerQueryValueA failed: GetLastError = %u\n", GetLastError());
-    ok( !lstrcmpA(WineFileDescriptionA, pBufA), "FileDescription should have been 'Wine version test'\n");
+    ok( !lstrcmpA(WineFileDescriptionA, pBufA), "expected '%s' got '%s'\n",
+        WineFileDescriptionA, pBufA);
 
     if (is_unicode_enabled)
     { 
         retW = VerQueryValueW( pVersionInfoW, FileDescriptionW, (LPVOID *)&pBufW, &uiLengthW );
         ok (retW, "VerQueryValueW failed: GetLastError = %u\n", GetLastError());
-        ok( !lstrcmpW(WineFileDescriptionW, pBufW), "FileDescription should have been 'Wine version test' (unicode)\n");
+        ok( !lstrcmpW(WineFileDescriptionW, pBufW), "FileDescription should have been '%s'\n", WineFileDescriptionA);
     }
 
     HeapFree( GetProcessHeap(), 0, pVersionInfoA);
@@ -407,12 +436,127 @@ static void test_32bit_win(void)
         HeapFree( GetProcessHeap(), 0, pVersionInfoW);
 }
 
+static void test_VerQueryValue(void)
+{
+    static const char * const value_name[] = {
+        "Product", "CompanyName", "FileDescription", "Internal",
+        "ProductVersion", "InternalName", "File", "LegalCopyright",
+        "FileVersion", "Legal", "OriginalFilename", "ProductName",
+        "Company", "Original" };
+    char *ver, *p;
+    UINT len, ret, translation, i;
+    char buf[MAX_PATH];
+
+    ret = GetModuleFileName(NULL, buf, sizeof(buf));
+    assert(ret);
+
+    SetLastError(0xdeadbeef);
+    len = GetFileVersionInfoSize(buf, NULL);
+    ok(len, "GetFileVersionInfoSize(%s) error %u\n", buf, GetLastError());
+
+    ver = HeapAlloc(GetProcessHeap(), 0, len);
+    assert(ver);
+
+    SetLastError(0xdeadbeef);
+    ret = GetFileVersionInfo(buf, 0, len, ver);
+    ok(ret, "GetFileVersionInfo error %u\n", GetLastError());
+
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, "\\VarFileInfo\\Translation", (LPVOID*)&p, &len);
+    ok(ret, "VerQueryValue error %u\n", GetLastError());
+    ok(len == 4, "VerQueryValue returned %u, expected 4\n", len);
+
+    translation = *(UINT *)p;
+    translation = MAKELONG(HIWORD(translation), LOWORD(translation));
+
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, "String", (LPVOID*)&p, &len);
+    ok(!ret, "VerQueryValue should fail\n");
+    ok(GetLastError() == ERROR_RESOURCE_TYPE_NOT_FOUND ||
+       GetLastError() == 0xdeadbeef /* Win9x, NT4, W2K */,
+       "VerQueryValue returned %u\n", GetLastError());
+    ok(p == (char *)0xdeadbeef, "expected 0xdeadbeef got %p\n", p);
+    ok(len == 0, "expected 0 got %x\n", len);
+
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, "StringFileInfo", (LPVOID*)&p, &len);
+    ok(ret, "VerQueryValue error %u\n", GetLastError());
+todo_wine ok(len == 0, "VerQueryValue returned %u, expected 0\n", len);
+    ok(p != (char *)0xdeadbeef, "not expected 0xdeadbeef\n");
+
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, "\\StringFileInfo", (LPVOID*)&p, &len);
+    ok(ret, "VerQueryValue error %u\n", GetLastError());
+todo_wine ok(len == 0, "VerQueryValue returned %u, expected 0\n", len);
+    ok(p != (char *)0xdeadbeef, "not expected 0xdeadbeef\n");
+
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, "\\\\StringFileInfo", (LPVOID*)&p, &len);
+    ok(ret, "VerQueryValue error %u\n", GetLastError());
+todo_wine ok(len == 0, "VerQueryValue returned %u, expected 0\n", len);
+    ok(p != (char *)0xdeadbeef, "not expected 0xdeadbeef\n");
+
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, "\\StringFileInfo\\\\", (LPVOID*)&p, &len);
+    ok(ret, "VerQueryValue error %u\n", GetLastError());
+todo_wine ok(len == 0, "VerQueryValue returned %u, expected 0\n", len);
+    ok(p != (char *)0xdeadbeef, "not expected 0xdeadbeef\n");
+
+    sprintf(buf, "\\StringFileInfo\\%08x", translation);
+    p = (char *)0xdeadbeef;
+    len = 0xdeadbeef;
+    SetLastError(0xdeadbeef);
+    ret = VerQueryValue(ver, buf, (LPVOID*)&p, &len);
+    ok(ret, "VerQueryValue error %u\n", GetLastError());
+todo_wine ok(len == 0, "VerQueryValue returned %u, expected 0\n", len);
+    ok(p != (char *)0xdeadbeef, "not expected 0xdeadbeef\n");
+
+    for (i = 0; i < sizeof(value_name)/sizeof(value_name[0]); i++)
+    {
+	sprintf(buf, "\\StringFileInfo\\%08x\\%s", translation, value_name[i]);
+        p = (char *)0xdeadbeef;
+        len = 0xdeadbeef;
+        SetLastError(0xdeadbeef);
+        ret = VerQueryValue(ver, buf, (LPVOID*)&p, &len);
+        ok(ret, "VerQueryValue(%s) error %u\n", buf, GetLastError());
+        ok(len == strlen(value_name[i]) + 1, "VerQueryValue returned %u\n", len);
+        ok(!strcmp(value_name[i], p), "expected \"%s\", got \"%s\"\n",
+           value_name[i], p);
+
+        /* test partial value names */
+        len = lstrlen(buf);
+        buf[len - 2] = 0;
+        p = (char *)0xdeadbeef;
+        len = 0xdeadbeef;
+        SetLastError(0xdeadbeef);
+        ret = VerQueryValue(ver, buf, (LPVOID*)&p, &len);
+        ok(!ret, "VerQueryValue(%s) succeeded\n", buf);
+        ok(GetLastError() == ERROR_RESOURCE_TYPE_NOT_FOUND ||
+           GetLastError() == 0xdeadbeef /* Win9x, NT4, W2K */,
+           "VerQueryValue returned %u\n", GetLastError());
+        ok(p == (char *)0xdeadbeef, "expected 0xdeadbeef got %p\n", p);
+        ok(len == 0, "expected 0 got %x\n", len);
+    }
+
+    HeapFree(GetProcessHeap(), 0, ver);
+}
+
 START_TEST(info)
 {
     test_info_size();
     test_info();
-   
-    /* Test several AW-calls on a 32 bit windows executable */
-    trace("Testing 32 bit windows application\n");
     test_32bit_win();
+    test_VerQueryValue();
 }
