@@ -81,6 +81,8 @@ static int ALSA_TestDeviceForWine(int card, int device,  snd_pcm_stream_t stream
     const char *reason = NULL;
     unsigned int rrate;
 
+    hwparams = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, snd_pcm_hw_params_sizeof() );
+
     /* Note that the plug: device masks out a lot of info, we want to avoid that */
     sprintf(pcmname, "hw:%d,%d", card, device);
     retcode = snd_pcm_open(&pcm, pcmname, streamtype, SND_PCM_NONBLOCK);
@@ -92,8 +94,6 @@ static int ALSA_TestDeviceForWine(int card, int device,  snd_pcm_stream_t stream
         goto exit;
     }
 
-    snd_pcm_hw_params_alloca(&hwparams);
-
     retcode = snd_pcm_hw_params_any(pcm, hwparams);
     if (retcode < 0)
     {
@@ -136,6 +136,7 @@ static int ALSA_TestDeviceForWine(int card, int device,  snd_pcm_stream_t stream
 exit:
     if (pcm)
         snd_pcm_close(pcm);
+    HeapFree( GetProcessHeap(), 0, hwparams );
 
     if (retcode != 0 && retcode != (-1 * ENOENT))
         TRACE("Discarding card %d/device %d:  %s [%d(%s)]\n", card, device, reason, retcode, snd_strerror(retcode));
@@ -197,7 +198,7 @@ static int ALSA_RegGetBoolean(HKEY key, const char *value, BOOL *answer)
 }
 
 /*----------------------------------------------------------------------------
-** ALSA_RegGetBoolean
+** ALSA_RegGetInt
 **  Get a string and interpret it as a DWORD
 */
 static int ALSA_RegGetInt(HKEY key, const char *value, DWORD *answer)
@@ -252,21 +253,22 @@ static int ALSA_ComputeCaps(snd_ctl_t *ctl, snd_pcm_t *pcm,
     unsigned int ratemax = 0;
     unsigned int chmin = 0;
     unsigned int chmax = 0;
-    int dir = 0;
+    int rc, dir = 0;
 
-    snd_pcm_hw_params_alloca(&hw_params);
-    ALSA_RETURN_ONFAIL(snd_pcm_hw_params_any(pcm, hw_params));
+    hw_params = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, snd_pcm_hw_params_sizeof() );
+    fmask = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, snd_pcm_format_mask_sizeof() );
+    acmask = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, snd_pcm_access_mask_sizeof() );
+
+    if ((rc = snd_pcm_hw_params_any(pcm, hw_params)) < 0) goto done;
 
-    snd_pcm_format_mask_alloca(&fmask);
     snd_pcm_hw_params_get_format_mask(hw_params, fmask);
 
-    snd_pcm_access_mask_alloca(&acmask);
-    ALSA_RETURN_ONFAIL(snd_pcm_hw_params_get_access_mask(hw_params, acmask));
+    if ((rc = snd_pcm_hw_params_get_access_mask(hw_params, acmask)) < 0) goto done;
 
-    ALSA_RETURN_ONFAIL(snd_pcm_hw_params_get_rate_min(hw_params, &ratemin, &dir));
-    ALSA_RETURN_ONFAIL(snd_pcm_hw_params_get_rate_max(hw_params, &ratemax, &dir));
-    ALSA_RETURN_ONFAIL(snd_pcm_hw_params_get_channels_min(hw_params, &chmin));
-    ALSA_RETURN_ONFAIL(snd_pcm_hw_params_get_channels_max(hw_params, &chmax));
+    if ((rc = snd_pcm_hw_params_get_rate_min(hw_params, &ratemin, &dir)) < 0) goto done;
+    if ((rc = snd_pcm_hw_params_get_rate_max(hw_params, &ratemax, &dir)) < 0) goto done;
+    if ((rc = snd_pcm_hw_params_get_channels_min(hw_params, &chmin)) < 0) goto done;
+    if ((rc = snd_pcm_hw_params_get_channels_max(hw_params, &chmax)) < 0) goto done;
 
 #define X(r,v) \
     if ( (r) >= ratemin && ( (r) <= ratemax || ratemax == -1) ) \
@@ -357,7 +359,14 @@ static int ALSA_ComputeCaps(snd_ctl_t *ctl, snd_pcm_t *pcm,
                                WAVE_FORMAT_48S16 | WAVE_FORMAT_96S16) )
         *flags |= DSCAPS_PRIMARY16BIT;
 
-    return(0);
+    rc = 0;
+
+done:
+    if (rc < 0) ERR("failed: %s(%d)\n", snd_strerror(rc), rc);
+    HeapFree( GetProcessHeap(), 0, hw_params );
+    HeapFree( GetProcessHeap(), 0, fmask );
+    HeapFree( GetProcessHeap(), 0, acmask );
+    return rc;
 }
 
 /*----------------------------------------------------------------------------
@@ -375,14 +384,22 @@ static int ALSA_ComputeCaps(snd_ctl_t *ctl, snd_pcm_t *pcm,
 static int ALSA_AddCommonDevice(snd_ctl_t *ctl, snd_pcm_t *pcm, const char *pcmname, WINE_WAVEDEV *ww)
 {
     snd_pcm_info_t *infop;
+    int rc;
 
-    snd_pcm_info_alloca(&infop);
-    ALSA_RETURN_ONFAIL(snd_pcm_info(pcm, infop));
+    infop = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, snd_pcm_info_sizeof() );
+    if ((rc = snd_pcm_info(pcm, infop)) < 0)
+    {
+        HeapFree( GetProcessHeap(), 0, infop );
+        return rc;
+    }
 
     if (pcm && pcmname)
         ww->pcmname = ALSA_strdup(pcmname);
     else
+    {
+        HeapFree( GetProcessHeap(), 0, infop );
         return -1;
+    }
 
     if (ctl && snd_ctl_name(ctl))
         ww->ctlname = ALSA_strdup(snd_ctl_name(ctl));
@@ -401,6 +418,7 @@ static int ALSA_AddCommonDevice(snd_ctl_t *ctl, snd_pcm_t *pcm, const char *pcmn
     ww->ds_caps.dwMaxSecondarySampleRate = DSBFREQUENCY_MAX;
     ww->ds_caps.dwPrimaryBuffers = 1;
 
+    HeapFree( GetProcessHeap(), 0, infop );
     return 0;
 }
 
@@ -697,14 +715,9 @@ static int ALSA_ScanDevices(int directhw,
         ** Try to open a ctl handle; Wine doesn't absolutely require one,
         **  but it does allow for volume control and for device scanning
         **------------------------------------------------------------------*/
-        sprintf(ctlname, "default:%d", fixedctlcard == -1 ? card : fixedctlcard);
+        sprintf(ctlname, "hw:%d", fixedctlcard == -1 ? card : fixedctlcard);
         rc = snd_ctl_open(&ctl, ctlname, SND_CTL_NONBLOCK);
         if (rc < 0)
-        {
-            sprintf(ctlname, "hw:%d", fixedctlcard == -1 ? card : fixedctlcard);
-            rc = snd_ctl_open(&ctl, ctlname, SND_CTL_NONBLOCK);
-        }
-        if (rc < 0)
         {
             ctl = NULL;
             WARN("Unable to open an alsa ctl for [%s] (pcm card %d): %s; not scanning devices\n",
@@ -728,7 +741,7 @@ static int ALSA_ScanDevices(int directhw,
             char *pcmname = NULL;
             snd_pcm_t *pcm;
 
-            sprintf(defaultpcmname, "default:%d", card);
+            sprintf(defaultpcmname, "default");
             sprintf(plugpcmname,    "plughw:%d,%d", card, device);
             sprintf(hwpcmname,      "hw:%d,%d", card, device);
 
