@@ -7,7 +7,7 @@
  * Copyright 2002-2003 Raphael Junqueira
  * Copyright 2004 Christian Costa
  * Copyright 2005 Oliver Stieber
- * Copyright 2006-2007 Stefan Dsinger for CodeWeavers
+ * Copyright 2006-2008 Stefan Dsinger for CodeWeavers
  * Copyright 2007 Henri Verbeet
  * Copyright 2006-2007 Roderick Colenbrander
  *
@@ -197,30 +197,25 @@ HRESULT WINAPI IWineD3DBaseSurfaceImpl_SetPalette(IWineD3DSurface *iface, IWineD
     IWineD3DPaletteImpl *PalImpl = (IWineD3DPaletteImpl *) Pal;
     TRACE("(%p)->(%p)\n", This, Pal);
 
+    if(This->palette == PalImpl) {
+        TRACE("Nop palette change\n");
+        return WINED3D_OK;
+    }
+
     if(This->palette != NULL)
         if(This->resource.usage & WINED3DUSAGE_RENDERTARGET)
             This->palette->Flags &= ~WINEDDPCAPS_PRIMARYSURFACE;
 
-    if(PalImpl != NULL) {
-        if(This->resource.usage & WINED3DUSAGE_RENDERTARGET) {
-            /* Set the device's main palette if the palette
-            * wasn't a primary palette before
-            */
-            if(!(PalImpl->Flags & WINEDDPCAPS_PRIMARYSURFACE)) {
-                IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
-                unsigned int i;
-
-                for(i=0; i < 256; i++) {
-                    device->palettes[device->currentPalette][i] = PalImpl->palents[i];
-                }
-            }
-
-            (PalImpl)->Flags |= WINEDDPCAPS_PRIMARYSURFACE;
-        }
-    }
     This->palette = PalImpl;
 
-    return IWineD3DSurface_RealizePalette(iface);
+    if(PalImpl != NULL) {
+        if(This->resource.usage & WINED3DUSAGE_RENDERTARGET) {
+            (PalImpl)->Flags |= WINEDDPCAPS_PRIMARYSURFACE;
+        }
+
+        return IWineD3DSurface_RealizePalette(iface);
+    }
+    else return WINED3D_OK;
 }
 
 HRESULT WINAPI IWineD3DBaseSurfaceImpl_SetColorKey(IWineD3DSurface *iface, DWORD Flags, WINEDDCOLORKEY *CKey) {
@@ -287,45 +282,6 @@ HRESULT WINAPI IWineD3DBaseSurfaceImpl_GetPalette(IWineD3DSurface *iface, IWineD
     return WINED3D_OK;
 }
 
-HRESULT WINAPI IWineD3DBaseSurfaceImpl_RealizePalette(IWineD3DSurface *iface) {
-    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
-    RGBQUAD col[256];
-    IWineD3DPaletteImpl *pal = This->palette;
-    unsigned int n;
-    TRACE("(%p)\n", This);
-
-    if(This->resource.format == WINED3DFMT_P8 ||
-       This->resource.format == WINED3DFMT_A8P8)
-    {
-        if(!This->Flags & SFLAG_INSYSMEM) {
-            FIXME("Palette changed with surface that does not have an up to date system memory copy\n");
-        }
-        TRACE("Dirtifying surface\n");
-        This->Flags &= ~(SFLAG_INTEXTURE | SFLAG_INDRAWABLE);
-    }
-
-    if(This->Flags & SFLAG_DIBSECTION) {
-        TRACE("(%p): Updating the hdc's palette\n", This);
-        for (n=0; n<256; n++) {
-            if(pal) {
-                col[n].rgbRed   = pal->palents[n].peRed;
-                col[n].rgbGreen = pal->palents[n].peGreen;
-                col[n].rgbBlue  = pal->palents[n].peBlue;
-            } else {
-                IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
-                /* Use the default device palette */
-                col[n].rgbRed   = device->palettes[device->currentPalette][n].peRed;
-                col[n].rgbGreen = device->palettes[device->currentPalette][n].peGreen;
-                col[n].rgbBlue  = device->palettes[device->currentPalette][n].peBlue;
-            }
-            col[n].rgbReserved = 0;
-        }
-        SetDIBColorTable(This->hDC, 0, 256, col);
-    }
-
-    return WINED3D_OK;
-}
-
 DWORD WINAPI IWineD3DBaseSurfaceImpl_GetPitch(IWineD3DSurface *iface) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
     DWORD ret;
@@ -483,7 +439,6 @@ HRESULT IWineD3DBaseSurfaceImpl_CreateDIBSection(IWineD3DSurface *iface) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
     int extraline = 0;
     SYSTEM_INFO sysInfo;
-    void *oldmem = This->resource.allocatedMemory;
     BITMAPINFO* b_info;
     HDC ddc;
     DWORD *masks;
@@ -609,11 +564,117 @@ HRESULT IWineD3DBaseSurfaceImpl_CreateDIBSection(IWineD3DSurface *iface) {
 
     This->Flags |= SFLAG_DIBSECTION;
 
-    HeapFree(GetProcessHeap(), 0, oldmem);
+    HeapFree(GetProcessHeap(), 0, This->resource.heapMemory);
+    This->resource.heapMemory = NULL;
 
     return WINED3D_OK;
 }
 
+void convert_r32f_r16f(BYTE *src, BYTE *dst, DWORD pitch_in, DWORD pitch_out, unsigned int w, unsigned int h) {
+    unsigned int x, y;
+    float *src_f;
+    unsigned short *dst_s;
+
+    TRACE("Converting %dx%d pixels, pitches %d %d\n", w, h, pitch_in, pitch_out);
+    for(y = 0; y < h; y++) {
+        src_f = (float *) (src + y * pitch_in);
+        dst_s = (unsigned short *) (dst + y * pitch_out);
+        for(x = 0; x < w; x++) {
+            dst_s[x] = float_32_to_16(src_f + x);
+        }
+    }
+}
+
+struct d3dfmt_convertor_desc {
+    WINED3DFORMAT from, to;
+    void (*convert)(BYTE *src, BYTE *dst, DWORD pitch_in, DWORD pitch_out, unsigned int w, unsigned int h);
+};
+
+struct d3dfmt_convertor_desc convertors[] = {
+    {WINED3DFMT_R32F,       WINED3DFMT_R16F,        convert_r32f_r16f},
+};
+
+static inline struct d3dfmt_convertor_desc *find_convertor(WINED3DFORMAT from, WINED3DFORMAT to) {
+    unsigned int i;
+    for(i = 0; i < (sizeof(convertors) / sizeof(convertors[0])); i++) {
+        if(convertors[i].from == from && convertors[i].to == to) {
+            return &convertors[i];
+        }
+    }
+    return NULL;
+}
+
+/*****************************************************************************
+ * surface_convert_format
+ *
+ * Creates a duplicate of a surface in a different format. Is used by Blt to
+ * blit between surfaces with different formats
+ *
+ * Parameters
+ *  source: Source surface
+ *  fmt: Requested destination format
+ *
+ *****************************************************************************/
+IWineD3DSurfaceImpl *surface_convert_format(IWineD3DSurfaceImpl *source, WINED3DFORMAT to_fmt) {
+    IWineD3DSurface *ret = NULL;
+    struct d3dfmt_convertor_desc *conv;
+    WINED3DLOCKED_RECT lock_src, lock_dst;
+    HRESULT hr;
+
+    conv = find_convertor(source->resource.format, to_fmt);
+    if(!conv) {
+        FIXME("Cannot find a conversion function from format %s to %s\n",
+              debug_d3dformat(source->resource.format), debug_d3dformat(to_fmt));
+        return NULL;
+    }
+
+    IWineD3DDevice_CreateSurface((IWineD3DDevice *) source->resource.wineD3DDevice,
+                                 source->currentDesc.Width,
+                                 source->currentDesc.Height,
+                                 to_fmt,
+                                 TRUE,  /* lockable */
+                                 TRUE,  /* discard  */
+                                 0,     /* level */
+                                 &ret,
+                                 WINED3DRTYPE_SURFACE,
+                                 0,     /* usage */
+                                 WINED3DPOOL_SCRATCH,
+                                 WINED3DMULTISAMPLE_NONE,   /* TODO: Multisampled conversion */
+                                 0,     /* multisamplequality */
+                                 NULL,  /* sharedhandle */
+                                 IWineD3DSurface_GetImplType((IWineD3DSurface *) source),
+                                 NULL); /* parent */
+    if(!ret) {
+        ERR("Failed to create a destination surface for conversion\n");
+        return NULL;
+    }
+
+    memset(&lock_src, 0, sizeof(lock_src));
+    memset(&lock_dst, 0, sizeof(lock_dst));
+
+    hr = IWineD3DSurface_LockRect((IWineD3DSurface *) source, &lock_src, NULL, WINED3DLOCK_READONLY);
+    if(FAILED(hr)) {
+        ERR("Failed to lock the source surface\n");
+        IWineD3DSurface_Release(ret);
+        return NULL;
+    }
+    hr = IWineD3DSurface_LockRect(ret, &lock_dst, NULL, WINED3DLOCK_READONLY);
+    if(FAILED(hr)) {
+        ERR("Failed to lock the dest surface\n");
+        IWineD3DSurface_UnlockRect((IWineD3DSurface *) source);
+        IWineD3DSurface_Release(ret);
+        return NULL;
+    }
+
+    conv->convert(lock_src.pBits, lock_dst.pBits, lock_src.Pitch, lock_dst.Pitch,
+                  source->currentDesc.Width, source->currentDesc.Height);
+
+    IWineD3DSurface_UnlockRect(ret);
+    IWineD3DSurface_UnlockRect((IWineD3DSurface *) source);
+
+    return (IWineD3DSurfaceImpl *) ret;
+}
+
 /*****************************************************************************
  * _Blt_ColorFill
  *
@@ -651,7 +712,7 @@ static HRESULT
                 case 2: COLORFILL_ROW(WORD)
         case 3:
         {
-            BYTE *d = (BYTE *) buf;
+            BYTE *d = buf;
             for (x = 0; x < width; x++,d+=3)
             {
                 d[0] = (color    ) & 0xFF;
@@ -749,14 +810,22 @@ IWineD3DBaseSurfaceImpl_Blt(IWineD3DSurface *iface,
     }
     else
     {
+        dfmt = This->resource.format;
+        dEntry = getFormatDescEntry(dfmt, NULL, NULL);
         if (Src)
         {
-            IWineD3DSurface_LockRect(SrcSurface, &slock, NULL, WINED3DLOCK_READONLY);
+            if(This->resource.format != Src->resource.format) {
+                Src = surface_convert_format(Src, dfmt);
+                if(!Src) {
+                    /* The conv function writes a FIXME */
+                    WARN("Cannot convert source surface format to dest format\n");
+                    goto release;
+                }
+            }
+            IWineD3DSurface_LockRect((IWineD3DSurface *) Src, &slock, NULL, WINED3DLOCK_READONLY);
             sfmt = Src->resource.format;
         }
         sEntry = getFormatDescEntry(sfmt, NULL, NULL);
-        dfmt = This->resource.format;
-        dEntry = getFormatDescEntry(dfmt, NULL, NULL);
         IWineD3DSurface_LockRect(iface, &dlock,NULL,0);
     }
 
@@ -764,25 +833,13 @@ IWineD3DBaseSurfaceImpl_Blt(IWineD3DSurface *iface,
 
     if (sEntry->isFourcc && dEntry->isFourcc)
     {
-        if (sfmt != dfmt)
-        {
-            FIXME("FOURCC->FOURCC copy only supported for the same type of surface\n");
-            ret = WINED3DERR_WRONGTEXTUREFORMAT;
-            goto release;
-        }
         memcpy(dlock.pBits, slock.pBits, This->resource.size);
         goto release;
     }
 
-    if (sEntry->isFourcc && !dEntry->isFourcc)
-    {
-        FIXME("DXTC decompression not supported right now\n");
-        goto release;
-    }
-
     if (DestRect)
     {
-        memcpy(&xdst,DestRect,sizeof(xdst));
+        xdst = *DestRect;
     }
     else
     {
@@ -794,7 +851,7 @@ IWineD3DBaseSurfaceImpl_Blt(IWineD3DSurface *iface,
 
     if (SrcRect)
     {
-        memcpy(&xsrc,SrcRect,sizeof(xsrc));
+        xsrc = *SrcRect;
     }
     else
     {
@@ -1016,7 +1073,7 @@ IWineD3DBaseSurfaceImpl_Blt(IWineD3DSurface *iface,
                     sbuf = sbase;
 
                     /* check for overlapping surfaces */
-                    if (SrcSurface != iface || xdst.top < xsrc.top ||
+                    if (Src != This || xdst.top < xsrc.top ||
                         xdst.right <= xsrc.left || xsrc.right <= xdst.left)
                     {
                         /* no overlap, or dst above src, so copy from top downwards */
@@ -1304,7 +1361,9 @@ error:
 
 release:
     IWineD3DSurface_UnlockRect(iface);
-    if (SrcSurface && SrcSurface != iface) IWineD3DSurface_UnlockRect(SrcSurface);
+    if (Src && Src != This) IWineD3DSurface_UnlockRect((IWineD3DSurface *) Src);
+    /* Release the converted surface if any */
+    if (Src && SrcSurface != (IWineD3DSurface *) Src) IWineD3DSurface_Release((IWineD3DSurface *) Src);
     return ret;
 }
 
@@ -1427,8 +1486,8 @@ IWineD3DBaseSurfaceImpl_BltFast(IWineD3DSurface *iface,
 
         /* Since slock was originally copied from this surface's description, we can just reuse it */
         assert(This->resource.allocatedMemory != NULL);
-        sbuf = (BYTE *)This->resource.allocatedMemory + lock_src.top * pitch + lock_src.left * bpp;
-        dbuf = (BYTE *)This->resource.allocatedMemory + lock_dst.top * pitch + lock_dst.left * bpp;
+        sbuf = This->resource.allocatedMemory + lock_src.top * pitch + lock_src.left * bpp;
+        dbuf = This->resource.allocatedMemory + lock_dst.top * pitch + lock_dst.left * bpp;
         sEntry = getFormatDescEntry(Src->resource.format, NULL, NULL);
         dEntry = sEntry;
     }
@@ -1514,8 +1573,8 @@ IWineD3DBaseSurfaceImpl_BltFast(IWineD3DSurface *iface,
             {
                 BYTE *d, *s;
                 DWORD tmp;
-                s = (BYTE *) sbuf;
-                d = (BYTE *) dbuf;
+                s = sbuf;
+                d = dbuf;
                 for (y = 0; y < h; y++)
                 {
                     for (x = 0; x < w * 3; x += 3)
@@ -1544,13 +1603,28 @@ IWineD3DBaseSurfaceImpl_BltFast(IWineD3DSurface *iface,
     else
     {
         int width = w * bpp;
+        INT sbufpitch, dbufpitch;
+
         TRACE("NO color key copy\n");
+        /* Handle overlapping surfaces */
+        if (sbuf < dbuf)
+        {
+            sbuf += (h - 1) * slock.Pitch;
+            dbuf += (h - 1) * dlock.Pitch;
+            sbufpitch = -slock.Pitch;
+            dbufpitch = -dlock.Pitch;
+        }
+        else
+        {
+            sbufpitch = slock.Pitch;
+            dbufpitch = dlock.Pitch;
+        }
         for (y = 0; y < h; y++)
         {
             /* This is pretty easy, a line for line memcpy */
-            memcpy(dbuf, sbuf, width);
-            sbuf += slock.Pitch;
-            dbuf += dlock.Pitch;
+            memmove(dbuf, sbuf, width);
+            sbuf += sbufpitch;
+            dbuf += dbufpitch;
         }
         TRACE("Copy done\n");
     }
@@ -1568,3 +1642,68 @@ error:
 
     return ret;
 }
+
+HRESULT WINAPI IWineD3DBaseSurfaceImpl_LockRect(IWineD3DSurface *iface, WINED3DLOCKED_RECT* pLockedRect, CONST RECT* pRect, DWORD Flags)
+{
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
+
+    TRACE("(%p) : rect@%p flags(%08x), output lockedRect@%p, memory@%p\n",
+          This, pRect, Flags, pLockedRect, This->resource.allocatedMemory);
+
+    pLockedRect->Pitch = IWineD3DSurface_GetPitch(iface);
+
+    if (NULL == pRect)
+    {
+        pLockedRect->pBits = This->resource.allocatedMemory;
+        This->lockedRect.left   = 0;
+        This->lockedRect.top    = 0;
+        This->lockedRect.right  = This->currentDesc.Width;
+        This->lockedRect.bottom = This->currentDesc.Height;
+
+        TRACE("Locked Rect (%p) = l %d, t %d, r %d, b %d\n",
+              &This->lockedRect, This->lockedRect.left, This->lockedRect.top,
+              This->lockedRect.right, This->lockedRect.bottom);
+    }
+    else
+    {
+        TRACE("Lock Rect (%p) = l %d, t %d, r %d, b %d\n",
+              pRect, pRect->left, pRect->top, pRect->right, pRect->bottom);
+
+        /* DXTn textures are based on compressed blocks of 4x4 pixels, each
+         * 16 bytes large (8 bytes in case of DXT1). Because of that Pitch has
+         * slightly different meaning compared to regular textures. For DXTn
+         * textures Pitch is the size of a row of blocks, 4 high and "width"
+         * long. The x offset is calculated differently as well, since moving 4
+         * pixels to the right actually moves an entire 4x4 block to right, ie
+         * 16 bytes (8 in case of DXT1). */
+        if (This->resource.format == WINED3DFMT_DXT1)
+        {
+            pLockedRect->pBits = This->resource.allocatedMemory + (pLockedRect->Pitch * pRect->top / 4) + (pRect->left * 2);
+        }
+        else if (This->resource.format == WINED3DFMT_DXT2 || This->resource.format == WINED3DFMT_DXT3 ||
+                    This->resource.format == WINED3DFMT_DXT4 || This->resource.format == WINED3DFMT_DXT5)
+        {
+            pLockedRect->pBits = This->resource.allocatedMemory + (pLockedRect->Pitch * pRect->top / 4) + (pRect->left * 4);
+        }
+        else
+        {
+            pLockedRect->pBits = This->resource.allocatedMemory +
+                    (pLockedRect->Pitch * pRect->top) +
+                    (pRect->left * This->bytesPerPixel);
+        }
+        This->lockedRect.left   = pRect->left;
+        This->lockedRect.top    = pRect->top;
+        This->lockedRect.right  = pRect->right;
+        This->lockedRect.bottom = pRect->bottom;
+    }
+
+    /* No dirtifying is needed for this surface implementation */
+    TRACE("returning memory@%p, pitch(%d)\n", pLockedRect->pBits, pLockedRect->Pitch);
+
+    return WINED3D_OK;
+}
+
+void WINAPI IWineD3DBaseSurfaceImpl_BindTexture(IWineD3DSurface *iface) {
+    ERR("Should not be called on base texture\n");
+    return;
+}
