@@ -140,7 +140,7 @@ ULONG WINAPI IWineGDISurfaceImpl_Release(IWineD3DSurface *iface) {
     ULONG ref = InterlockedDecrement(&This->resource.ref);
     TRACE("(%p) : Releasing from %d\n", This, ref + 1);
     if (ref == 0) {
-        IWineD3DDeviceImpl *device = (IWineD3DDeviceImpl *) This->resource.wineD3DDevice;
+        IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
         TRACE("(%p) : cleaning up\n", This);
 
         if(This->Flags & SFLAG_DIBSECTION) {
@@ -182,6 +182,20 @@ IWineGDISurfaceImpl_PreLoad(IWineD3DSurface *iface)
     ERR("(%p): Please report to wine-devel\n", iface);
 }
 
+/*****************************************************************************
+ * IWineD3DSurface::Unoad, GDI version
+ *
+ * This call is unsupported on GDI surfaces, if it's called something went
+ * wrong in the parent library. Write an informative warning
+ *
+ *****************************************************************************/
+static void WINAPI IWineGDISurfaceImpl_UnLoad(IWineD3DSurface *iface)
+{
+    ERR("(%p): UnLoad is not supported on X11 surfaces!\n", iface);
+    ERR("(%p): Most likely the parent library did something wrong.\n", iface);
+    ERR("(%p): Please report to wine-devel\n", iface);
+}
+
 /*****************************************************************************
  * IWineD3DSurface::LockRect, GDI version
  *
@@ -212,73 +226,17 @@ IWineGDISurfaceImpl_LockRect(IWineD3DSurface *iface,
         /* What should I return here? */
         return WINED3DERR_INVALIDCALL;
     }
-
-    if (!(This->Flags & SFLAG_LOCKABLE))
-    {
-        /* This is some GL specific thing, see the OpenGL version of
-         * this method, but check for the flag and write a trace
-         */
-        TRACE("Warning: trying to lock unlockable surf@%p\n", This);
-    }
-
-    TRACE("(%p) : rect@%p flags(%08x), output lockedRect@%p, memory@%p\n",
-          This, pRect, Flags, pLockedRect, This->resource.allocatedMemory);
+    This->Flags |= SFLAG_LOCKED;
 
     if(!This->resource.allocatedMemory) {
-        HDC hdc;
-        HRESULT hr;
         /* This happens on gdi surfaces if the application set a user pointer and resets it.
          * Recreate the DIB section
          */
-        hr = IWineD3DSurface_GetDC(iface, &hdc);  /* will recursively call lockrect, do not set the LOCKED flag to this line */
-        if(hr != WINED3D_OK) return hr;
-        hr = IWineD3DSurface_ReleaseDC(iface, hdc);
-        if(hr != WINED3D_OK) return hr;
+        IWineD3DBaseSurfaceImpl_CreateDIBSection(iface);
+        This->resource.allocatedMemory = This->dib.bitmap_data;
     }
 
-    pLockedRect->Pitch = IWineD3DSurface_GetPitch(iface);
-
-    if (NULL == pRect)
-    {
-        pLockedRect->pBits = This->resource.allocatedMemory;
-        This->lockedRect.left   = 0;
-        This->lockedRect.top    = 0;
-        This->lockedRect.right  = This->currentDesc.Width;
-        This->lockedRect.bottom = This->currentDesc.Height;
-
-        TRACE("Locked Rect (%p) = l %d, t %d, r %d, b %d\n",
-        &This->lockedRect, This->lockedRect.left, This->lockedRect.top,
-        This->lockedRect.right, This->lockedRect.bottom);
-    }
-    else
-    {
-        TRACE("Lock Rect (%p) = l %d, t %d, r %d, b %d\n",
-              pRect, pRect->left, pRect->top, pRect->right, pRect->bottom);
-
-        if (This->resource.format == WINED3DFMT_DXT1)
-        {
-            /* DXT1 is half byte per pixel */
-            pLockedRect->pBits = This->resource.allocatedMemory +
-                                  (pLockedRect->Pitch * pRect->top) +
-                                  ((pRect->left * This->bytesPerPixel / 2));
-        }
-        else
-        {
-            pLockedRect->pBits = This->resource.allocatedMemory +
-                                 (pLockedRect->Pitch * pRect->top) +
-                                 (pRect->left * This->bytesPerPixel);
-        }
-        This->lockedRect.left   = pRect->left;
-        This->lockedRect.top    = pRect->top;
-        This->lockedRect.right  = pRect->right;
-        This->lockedRect.bottom = pRect->bottom;
-    }
-
-    /* No dirtifying is needed for this surface implementation */
-    TRACE("returning memory@%p, pitch(%d)\n", pLockedRect->pBits, pLockedRect->Pitch);
-
-    This->Flags |= SFLAG_LOCKED;
-    return WINED3D_OK;
+    return IWineD3DBaseSurfaceImpl_LockRect(iface, pLockedRect, pRect, Flags);
 }
 
 /*****************************************************************************
@@ -296,7 +254,7 @@ static HRESULT WINAPI
 IWineGDISurfaceImpl_UnlockRect(IWineD3DSurface *iface)
 {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
-    IWineD3DDeviceImpl *dev = (IWineD3DDeviceImpl *) This->resource.wineD3DDevice;
+    IWineD3DDeviceImpl *dev = This->resource.wineD3DDevice;
     TRACE("(%p)\n", This);
 
     if (!(This->Flags & SFLAG_LOCKED))
@@ -394,6 +352,13 @@ IWineGDISurfaceImpl_Flip(IWineD3DSurface *iface,
         tmp = This->resource.allocatedMemory;
         This->resource.allocatedMemory = Target->resource.allocatedMemory;
         Target->resource.allocatedMemory = tmp;
+
+        if(This->resource.heapMemory) {
+            ERR("GDI Surface %p has heap memory allocated\n", This);
+        }
+        if(Target->resource.heapMemory) {
+            ERR("GDI Surface %p has heap memory allocated\n", Target);
+        }
     }
 
     /* client_memory should not be different, but just in case */
@@ -527,7 +492,7 @@ const char* filename)
             table[i][2] = This->palette->palents[i].peBlue;
         }
         for (y = 0; y < This->pow2Height; y++) {
-            unsigned char *src = (unsigned char *) This->resource.allocatedMemory + (y * 1 * IWineD3DSurface_GetPitch(iface));
+            unsigned char *src = This->resource.allocatedMemory + (y * 1 * IWineD3DSurface_GetPitch(iface));
             for (x = 0; x < This->pow2Width; x++) {
                 unsigned char color = *src;
                 src += 1;
@@ -539,16 +504,17 @@ const char* filename)
             fwrite(output, 3 * This->pow2Width, 1, f);
         }
     } else {
-        int red_shift, green_shift, blue_shift, pix_width;
+        int red_shift, green_shift, blue_shift, pix_width, alpha_shift;
 
         pix_width = This->bytesPerPixel;
 
         red_shift = get_shift(formatEntry->redMask);
         green_shift = get_shift(formatEntry->greenMask);
         blue_shift = get_shift(formatEntry->blueMask);
+        alpha_shift = get_shift(formatEntry->alphaMask);
 
         for (y = 0; y < This->pow2Height; y++) {
-            unsigned char *src = (unsigned char *) This->resource.allocatedMemory + (y * 1 * IWineD3DSurface_GetPitch(iface));
+            unsigned char *src = This->resource.allocatedMemory + (y * 1 * IWineD3DSurface_GetPitch(iface));
             for (x = 0; x < This->pow2Width; x++) {	    
                 unsigned int color;
                 unsigned int comp;
@@ -564,8 +530,8 @@ const char* filename)
                 output[3 * x + 0] = red_shift > 0 ? comp >> red_shift : comp << -red_shift;
                 comp = color & formatEntry->greenMask;
                 output[3 * x + 1] = green_shift > 0 ? comp >> green_shift : comp << -green_shift;
-                comp = color & formatEntry->blueMask;
-                output[3 * x + 2] = blue_shift > 0 ? comp >> blue_shift : comp << -blue_shift;
+                comp = color & formatEntry->alphaMask;
+                output[3 * x + 2] = alpha_shift > 0 ? comp >> alpha_shift : comp << -alpha_shift;
             }
             fwrite(output, 3 * This->pow2Width, 1, f);
         }
@@ -613,28 +579,25 @@ HRESULT WINAPI IWineGDISurfaceImpl_GetDC(IWineD3DSurface *iface, HDC *pHDC) {
     if(This->resource.format == WINED3DFMT_P8 ||
        This->resource.format == WINED3DFMT_A8P8) {
         unsigned int n;
+        PALETTEENTRY *pal = NULL;
+
         if(This->palette) {
-            PALETTEENTRY ent[256];
-
-            GetPaletteEntries(This->palette->hpal, 0, 256, ent);
-            for (n=0; n<256; n++) {
-                col[n].rgbRed   = ent[n].peRed;
-                col[n].rgbGreen = ent[n].peGreen;
-                col[n].rgbBlue  = ent[n].peBlue;
-                col[n].rgbReserved = 0;
-            }
+            pal = This->palette->palents;
         } else {
-            IWineD3DDeviceImpl *device = This->resource.wineD3DDevice;
+            IWineD3DSurfaceImpl *dds_primary = (IWineD3DSurfaceImpl *)This->resource.wineD3DDevice->ddraw_primary;
+            if (dds_primary && dds_primary->palette)
+                pal = dds_primary->palette->palents;
+        }
 
+        if (pal) {
             for (n=0; n<256; n++) {
-                col[n].rgbRed   = device->palettes[device->currentPalette][n].peRed;
-                col[n].rgbGreen = device->palettes[device->currentPalette][n].peGreen;
-                col[n].rgbBlue  = device->palettes[device->currentPalette][n].peBlue;
+                col[n].rgbRed   = pal[n].peRed;
+                col[n].rgbGreen = pal[n].peGreen;
+                col[n].rgbBlue  = pal[n].peBlue;
                 col[n].rgbReserved = 0;
             }
-
+            SetDIBColorTable(This->hDC, 0, 256, col);
         }
-        SetDIBColorTable(This->hDC, 0, 256, col);
     }
 
     *pHDC = This->hDC;
@@ -665,6 +628,34 @@ HRESULT WINAPI IWineGDISurfaceImpl_ReleaseDC(IWineD3DSurface *iface, HDC hDC) {
     return WINED3D_OK;
 }
 
+HRESULT WINAPI IWineGDISurfaceImpl_RealizePalette(IWineD3DSurface *iface) {
+    IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *) iface;
+    RGBQUAD col[256];
+    IWineD3DPaletteImpl *pal = This->palette;
+    unsigned int n;
+    TRACE("(%p)\n", This);
+
+    if (!pal) return WINED3D_OK;
+
+    if(This->Flags & SFLAG_DIBSECTION) {
+        TRACE("(%p): Updating the hdc's palette\n", This);
+        for (n=0; n<256; n++) {
+            col[n].rgbRed   = pal->palents[n].peRed;
+            col[n].rgbGreen = pal->palents[n].peGreen;
+            col[n].rgbBlue  = pal->palents[n].peBlue;
+            col[n].rgbReserved = 0;
+        }
+        SetDIBColorTable(This->hDC, 0, 256, col);
+    }
+
+    /* Update the image because of the palette change. Some games like e.g Red Alert
+       call SetEntries a lot to implement fading. */
+    if(This->resource.usage & WINED3DUSAGE_RENDERTARGET)
+        x11_copy_to_screen(This, NULL);
+
+    return WINED3D_OK;
+}
+
 /*****************************************************************************
  * IWineD3DSurface::PrivateSetup, GDI version
  *
@@ -695,8 +686,9 @@ IWineGDISurfaceImpl_PrivateSetup(IWineD3DSurface *iface)
     /* Sysmem textures have memory already allocated -
      * release it, this avoids an unnecessary memcpy
      */
-    HeapFree(GetProcessHeap(), 0, This->resource.allocatedMemory);
+    HeapFree(GetProcessHeap(), 0, This->resource.heapMemory);
     This->resource.allocatedMemory = NULL;
+    This->resource.heapMemory = NULL;
 
     /* We don't mind the nonpow2 stuff in GDI */
     This->pow2Width = This->currentDesc.Width;
@@ -710,7 +702,16 @@ IWineGDISurfaceImpl_PrivateSetup(IWineD3DSurface *iface)
 
 void WINAPI IWineGDISurfaceImpl_SetGlTextureDesc(IWineD3DSurface *iface, UINT textureName, int target) {
     IWineD3DSurfaceImpl *This = (IWineD3DSurfaceImpl *)iface;
-    FIXME("(%p) : Should not be called on a GDI surface. textureName %u, target %i\n", This, textureName, target);
+
+    /* Ignore 0 textureName and target. D3D textures can be created with gdi surfaces as plain
+     * containers, but they're useless until the app creates a d3d device from a d3d point of
+     * view, it's not an implementation limitation. This avoids false warnings when the texture
+     * is destroyed and sets the description back to 0/0
+     */
+    if(textureName != 0 || target != 0) {
+        FIXME("(%p) : Should not be called on a GDI surface. textureName %u, target %i\n", This, textureName, target);
+        DebugBreak();
+    }
 }
 
 void WINAPI IWineGDISurfaceImpl_GetGlDesc(IWineD3DSurface *iface, glDescriptor **glDescription) {
@@ -774,6 +775,32 @@ HRESULT WINAPI IWineGDISurfaceImpl_SetMem(IWineD3DSurface *iface, void *Mem) {
     return WINED3D_OK;
 }
 
+/***************************
+ *
+ ***************************/
+static void WINAPI IWineGDISurfaceImpl_ModifyLocation(IWineD3DSurface *iface, DWORD flag, BOOL persistent) {
+    TRACE("(%p)->(%s, %s)\n", iface,
+          flag == SFLAG_INSYSMEM ? "SFLAG_INSYSMEM" : flag == SFLAG_INDRAWABLE ? "SFLAG_INDRAWABLE" : "SFLAG_INTEXTURE",
+          persistent ? "TRUE" : "FALSE");
+    /* GDI surfaces can be in system memory only */
+    if(flag != SFLAG_INSYSMEM) {
+        ERR("GDI Surface requested in gl %s memory\n", flag == SFLAG_INDRAWABLE ? "drawable" : "texture");
+    }
+}
+
+static HRESULT WINAPI IWineGDISurfaceImpl_LoadLocation(IWineD3DSurface *iface, DWORD flag, const RECT *rect) {
+    if(flag != SFLAG_INSYSMEM) {
+        ERR("GDI Surface requested to be copied to gl %s\n", flag == SFLAG_INTEXTURE ? "texture" : "drawable");
+    } else {
+        TRACE("Surface requested in surface memory\n");
+    }
+    return WINED3D_OK;
+}
+
+static WINED3DSURFTYPE WINAPI IWineGDISurfaceImpl_GetImplType(IWineD3DSurface *iface) {
+    return SURFACE_GDI;
+}
+
 /* FIXME: This vtable should not use any IWineD3DSurface* implementation functions,
  * only IWineD3DBaseSurface and IWineGDISurface ones.
  */
@@ -792,6 +819,7 @@ const IWineD3DSurfaceVtbl IWineGDISurface_Vtbl =
     IWineD3DBaseSurfaceImpl_SetPriority,
     IWineD3DBaseSurfaceImpl_GetPriority,
     IWineGDISurfaceImpl_PreLoad,
+    IWineGDISurfaceImpl_UnLoad,
     IWineD3DBaseSurfaceImpl_GetType,
     /* IWineD3DSurface */
     IWineD3DBaseSurfaceImpl_GetContainer,
@@ -809,7 +837,7 @@ const IWineD3DSurfaceVtbl IWineGDISurface_Vtbl =
     IWineD3DBaseSurfaceImpl_BltFast,
     IWineD3DBaseSurfaceImpl_GetPalette,
     IWineD3DBaseSurfaceImpl_SetPalette,
-    IWineD3DBaseSurfaceImpl_RealizePalette,
+    IWineGDISurfaceImpl_RealizePalette,
     IWineD3DBaseSurfaceImpl_SetColorKey,
     IWineD3DBaseSurfaceImpl_GetPitch,
     IWineGDISurfaceImpl_SetMem,
@@ -822,11 +850,15 @@ const IWineD3DSurfaceVtbl IWineGDISurface_Vtbl =
     /* Internal use: */
     IWineGDISurfaceImpl_AddDirtyRect,
     IWineGDISurfaceImpl_LoadTexture,
+    IWineD3DBaseSurfaceImpl_BindTexture,
     IWineGDISurfaceImpl_SaveSnapshot,
     IWineD3DBaseSurfaceImpl_SetContainer,
     IWineGDISurfaceImpl_SetGlTextureDesc,
     IWineGDISurfaceImpl_GetGlDesc,
     IWineD3DSurfaceImpl_GetData,
     IWineD3DBaseSurfaceImpl_SetFormat,
-    IWineGDISurfaceImpl_PrivateSetup
+    IWineGDISurfaceImpl_PrivateSetup,
+    IWineGDISurfaceImpl_ModifyLocation,
+    IWineGDISurfaceImpl_LoadLocation,
+    IWineGDISurfaceImpl_GetImplType
 };
