@@ -63,7 +63,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(wave);
 
-#ifdef HAVE_JACK_JACK_H
+#ifdef SONAME_LIBJACK
 
 #define MAKE_FUNCPTR(f) static typeof(f) * fp_##f = NULL;
 
@@ -489,7 +489,7 @@ int JACK_callback_wwo (nframes_t nframes, void *arg)
 /******************************************************************
  *		JACK_bufsize_wwo
  *
- *		Called whenever the jack server changes the the max number 
+ *		Called whenever the jack server changes the max number
  *		of frames passed to JACK_callback
  */
 int JACK_bufsize_wwo (nframes_t nframes, void *arg)
@@ -537,7 +537,7 @@ int JACK_bufsize_wwo (nframes_t nframes, void *arg)
 /******************************************************************
  *		JACK_bufsize_wwi
  *
- *		Called whenever the jack server changes the the max number 
+ *		Called whenever the jack server changes the max number
  *		of frames passed to JACK_callback
  */
 int JACK_bufsize_wwi (nframes_t nframes, void *arg)
@@ -1307,9 +1307,8 @@ static DWORD wodOpen(WORD wDevID, LPWAVEOPENDESC lpDesc, DWORD dwFlags)
 
     dwFlags &= ~WAVE_DIRECTSOUND;  /* direct sound not supported, ignore the flag */
 
-    wwo->wFlags = HIWORD(dwFlags & CALLBACK_TYPEMASK);
-    
-    memcpy(&wwo->waveDesc, lpDesc, 	     sizeof(WAVEOPENDESC));
+
+    wwo->waveDesc = *lpDesc;
     memcpy(&wwo->format,   lpDesc->lpFormat, sizeof(PCMWAVEFORMAT));
 
     /* open up jack ports for this device */
@@ -1805,7 +1804,8 @@ int JACK_callback_wwi (nframes_t nframes, void *arg)
 #if JACK_CLOSE_HACK
 	if(wwi->in_use == FALSE)
 	{
-	  /* do nothing if nothing is being recorded */
+            /* do nothing if nothing is being recorded */
+            LeaveCriticalSection(&wwi->access_crst);
 	    return 0;
 	}
 #endif
@@ -2110,8 +2110,8 @@ static DWORD widOpen(WORD wDevID, LPWAVEOPENDESC lpDesc, DWORD dwFlags)
     dwFlags &= ~WAVE_DIRECTSOUND;  /* direct sound not supported, ignore the flag */
 
     wwi->wFlags = HIWORD(dwFlags & CALLBACK_TYPEMASK);
-    
-    memcpy(&wwi->waveDesc, lpDesc, 	     sizeof(WAVEOPENDESC));
+
+    wwi->waveDesc = *lpDesc;
     memcpy(&wwi->format,   lpDesc->lpFormat, sizeof(PCMWAVEFORMAT));
 
     LeaveCriticalSection(&wwi->access_crst);
@@ -2385,7 +2385,7 @@ DWORD WINAPI JACK_widMessage(WORD wDevID, WORD wMsg, DWORD dwUser,
   return MMSYSERR_NOTSUPPORTED;
 }
 
-#else /* !HAVE_JACK_JACK_H */
+#else /* !SONAME_LIBJACK */
 
 /**************************************************************************
  * 				widMessage (WINEJACK.6)
@@ -2407,4 +2407,4 @@ DWORD WINAPI JACK_wodMessage(WORD wDevID, WORD wMsg, DWORD dwUser,
   return MMSYSERR_NOTENABLED;
 }
 
-#endif /* HAVE_JACK_JACK_H */
+#endif /* SONAME_LIBJACK */
