@@ -72,6 +72,7 @@
 # include <unistd.h>
 #endif
 #include <fcntl.h>
+#include <limits.h>
 #include <time.h>
 #include <assert.h>
 
@@ -1648,7 +1649,7 @@ HANDLE X11DRV_CLIPBOARD_ExportXAPIXMAP(Window requestor, Atom aTarget, Atom rpro
     memcpy(lpData, &lpdata->drvData, *lpBytes);
     GlobalUnlock(hData);
 
-    return (HANDLE) hData;
+    return hData;
 }
 
 
@@ -1666,8 +1667,7 @@ HANDLE X11DRV_CLIPBOARD_ExportMetaFilePict(Window requestor, Atom aTarget, Atom
         return 0;
     }
 
-    return X11DRV_CLIPBOARD_SerializeMetafile(CF_METAFILEPICT, (HANDLE)lpdata->hData32, 
-        lpBytes, TRUE);
+    return X11DRV_CLIPBOARD_SerializeMetafile(CF_METAFILEPICT, lpdata->hData32, lpBytes, TRUE);
 }
 
 
@@ -1685,8 +1685,7 @@ HANDLE X11DRV_CLIPBOARD_ExportEnhMetaFile(Window requestor, Atom aTarget, Atom r
         return 0;
     }
 
-    return X11DRV_CLIPBOARD_SerializeMetafile(CF_ENHMETAFILE, (HANDLE)lpdata->hData32, 
-        lpBytes, TRUE);
+    return X11DRV_CLIPBOARD_SerializeMetafile(CF_ENHMETAFILE, lpdata->hData32, lpBytes, TRUE);
 }
 
 
@@ -1903,8 +1902,8 @@ static int X11DRV_CLIPBOARD_QueryAvailableData(LPCLIPBOARDINFO lpcbinfo)
         * The TARGETS property should have returned us a list of atoms
         * corresponding to each selection target format supported.
         */
-       if ((atype == XA_ATOM || atype == x11drv_atom(TARGETS)) && aformat == 32)
-           X11DRV_CLIPBOARD_InsertSelectionProperties(display, targetList, cSelectionTargets);
+       if (atype == XA_ATOM || atype == x11drv_atom(TARGETS))
+           X11DRV_CLIPBOARD_InsertSelectionProperties(display, targetList, (cSelectionTargets * aformat / (8 * sizeof(Atom))));
 
        /* Free the list of targets */
        wine_tsx11_lock();
@@ -2008,71 +2007,55 @@ static BOOL X11DRV_CLIPBOARD_ReadProperty(Window w, Atom prop,
     Display *display = thread_display();
     Atom atype = AnyPropertyType;
     int aformat;
-    unsigned long total, nitems, remain, val_cnt;
-    long  reqlen, bwc;
-    unsigned char* val;
-    unsigned char* buffer;
+    unsigned long pos = 0, nitems, remain, count;
+    unsigned char *val = NULL, *buffer;
 
     if (prop == None)
         return FALSE;
 
-    TRACE("Reading property %d from X window %d\n",
-        (unsigned int)prop, (unsigned int)w);
+    TRACE("Reading property %lu from X window %lx\n", prop, w);
 
-    /*
-     * First request a zero length in order to figure out the request size.
-     */
-    wine_tsx11_lock();
-    if(XGetWindowProperty(display,w,prop,0,0,False, AnyPropertyType,
-        &atype, &aformat, &nitems, &remain, &buffer) != Success)
+    for (;;)
     {
+        wine_tsx11_lock();
+        if (XGetWindowProperty(display, w, prop, pos, INT_MAX / 4, False,
+                               AnyPropertyType, &atype, &aformat, &nitems, &remain, &buffer) != Success)
+        {
+            wine_tsx11_unlock();
+            WARN("Failed to read property\n");
+            HeapFree( GetProcessHeap(), 0, val );
+            return FALSE;
+        }
+
+        count = nitems * (aformat / 8);
+        if (!val) *data = HeapAlloc( GetProcessHeap(), 0, pos * sizeof(int) + count + 1 );
+        else *data = HeapReAlloc( GetProcessHeap(), 0, val, pos * sizeof(int) + count + 1 );
+
+        if (!*data)
+        {
+            XFree( buffer );
+            wine_tsx11_unlock();
+            HeapFree( GetProcessHeap(), 0, val );
+            return FALSE;
+        }
+        val = *data;
+        memcpy( (int *)val + pos, buffer, count );
+        XFree( buffer );
         wine_tsx11_unlock();
-        WARN("Failed to get property size\n");
-        return FALSE;
-    }
-
-    /* Free zero length return data if any */
-    if (buffer)
-    {
-       XFree(buffer);
-       buffer = NULL;
-    }
-
-    bwc = aformat/8;
-    reqlen = remain * bwc;
-
-    TRACE("Retrieving %ld bytes\n", reqlen);
-
-    val = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, reqlen);
-
-    /* Read property in 4K blocks */
-    for (total = 0, val_cnt = 0; remain;)
-    {
-       if (XGetWindowProperty(display, w, prop, (total / 4), 4096, False,
-           AnyPropertyType, &atype, &aformat, &nitems, &remain, &buffer) != Success)
-       {
-           wine_tsx11_unlock();
-           WARN("Failed to read property\n");
-           HeapFree(GetProcessHeap(), 0, val);
-           return FALSE;
-       }
-
-       bwc = aformat/8;
-       memcpy(&val[val_cnt], buffer, nitems * bwc);
-       val_cnt += nitems * bwc;
-       total += nitems*bwc;
-       XFree(buffer);
+        if (!remain)
+        {
+            *datasize = pos * sizeof(int) + count;
+            val[*datasize] = 0;
+            break;
+        }
+        pos += count / sizeof(int);
     }
 
     /* Delete the property on the window now that we are done
      * This will send a PropertyNotify event to the selection owner. */
+    wine_tsx11_lock();
     XDeleteProperty(display, w, prop);
-
     wine_tsx11_unlock();
-
-    *data = val;
-    *datasize = total;
-
     return TRUE;
 }
 
@@ -2136,7 +2119,7 @@ static HANDLE X11DRV_CLIPBOARD_SerializeMetafile(INT wformat, HANDLE hdata, LPDW
                 unsigned int wiresize, size;
                 LPMETAFILEPICT lpmfp = (LPMETAFILEPICT) GlobalLock(h);
 
-                memcpy(lpmfp, (LPVOID)hdata, sizeof(METAFILEPICT));
+                memcpy(lpmfp, hdata, sizeof(METAFILEPICT));
                 wiresize = *lpcbytes - sizeof(METAFILEPICT);
                 lpmfp->hMF = SetMetaFileBitsEx(wiresize,
                     ((const BYTE *)hdata) + sizeof(METAFILEPICT));
@@ -2146,7 +2129,7 @@ static HANDLE X11DRV_CLIPBOARD_SerializeMetafile(INT wformat, HANDLE hdata, LPDW
         }
         else if (wformat == CF_ENHMETAFILE)
         {
-            h = SetEnhMetaFileBits(*lpcbytes, (LPVOID)hdata);
+            h = SetEnhMetaFileBits(*lpcbytes, hdata);
         }
     }
 
@@ -2902,7 +2885,7 @@ static Atom X11DRV_SelectionRequest_MULTIPLE( HWND hWnd, XSelectionRequestEvent
                 }
 
                 /* Set up an XSelectionRequestEvent for this (target,property) pair */
-                memcpy( &event, pevent, sizeof(XSelectionRequestEvent) );
+                event = *pevent;
                 event.target = targetPropList[i];
                 event.property = targetPropList[i+1];
 
@@ -2990,7 +2973,7 @@ static void X11DRV_HandleSelectionRequest( HWND hWnd, XSelectionRequestEvent *ev
 
                     wine_tsx11_lock();
                     XChangeProperty(display, request, rprop, event->target,
-                                    8, PropModeReplace, (unsigned char *)lpClipData, cBytes);
+                                    8, PropModeReplace, lpClipData, cBytes);
                     wine_tsx11_unlock();
 
                     GlobalUnlock(hClipData);
