@@ -202,7 +202,7 @@ INT X11DRV_GetDeviceCaps( X11DRV_PDEVICE *physDev, INT cap )
     case DESKTOPVERTRES:
         return virtual_screen_rect.bottom - virtual_screen_rect.top;
     case BITSPIXEL:
-        return screen_depth;
+        return screen_bpp;
     case PLANES:
         return 1;
     case NUMBRUSHES:
@@ -251,8 +251,7 @@ INT X11DRV_GetDeviceCaps( X11DRV_PDEVICE *physDev, INT cap )
     case CAPS1:
         FIXME("(%p): CAPS1 is unimplemented, will return 0\n", physDev->hdc );
         /* please see wingdi.h for the possible bit-flag values that need
-           to be returned. also, see
-           http://msdn.microsoft.com/library/ddkdoc/win95ddk/graphcnt_1m0p.htm */
+           to be returned. */
         return 0;
     case SIZEPALETTE:
         return palette_size;
@@ -341,6 +340,8 @@ INT X11DRV_ExtEscape( X11DRV_PDEVICE *physDev, INT escape, INT in_count, LPCVOID
                     physDev->drawable = data->drawable;
                     physDev->drawable_rect = data->drawable_rect;
                     physDev->current_pf = pixelformat_from_fbconfig_id( data->fbconfig_id );
+                    physDev->gl_drawable = data->gl_drawable;
+                    physDev->pixmap = data->pixmap;
                     wine_tsx11_lock();
                     XSetSubwindowMode( gdi_display, physDev->gc, data->mode );
                     wine_tsx11_unlock();
@@ -406,19 +407,8 @@ INT X11DRV_ExtEscape( X11DRV_PDEVICE *physDev, INT escape, INT in_count, LPCVOID
                 }
                 break;
             case X11DRV_GET_DCE:
-                if (out_count >= sizeof(struct dce *))
-                {
-                    *(struct dce **)out_data = physDev->dce;
-                    return TRUE;
-                }
-                break;
             case X11DRV_SET_DCE:
-                if (in_count >= sizeof(struct x11drv_escape_set_dce))
-                {
-                    const struct x11drv_escape_set_dce *data = (const struct x11drv_escape_set_dce *)in_data;
-                    physDev->dce = data->dce;
-                    return TRUE;
-                }
+                FIXME( "%x escape no longer supported\n", *(const enum x11drv_escape_codes *)in_data );
                 break;
             case X11DRV_GET_GLX_DRAWABLE:
                 if (out_count >= sizeof(Drawable))
@@ -430,10 +420,13 @@ INT X11DRV_ExtEscape( X11DRV_PDEVICE *physDev, INT escape, INT in_count, LPCVOID
             case X11DRV_SYNC_PIXMAP:
                 if(physDev->bitmap)
                 {
-                    X11DRV_CoerceDIBSection(physDev, DIB_Status_GdiMod, FALSE);
+                    X11DRV_CoerceDIBSection(physDev, DIB_Status_GdiMod);
                     return TRUE;
                 }
                 return FALSE;
+            case X11DRV_FLUSH_GL_DRAWABLE:
+                flush_gl_drawable(physDev);
+                return TRUE;
             }
         }
         break;
