@@ -41,7 +41,6 @@ MAKE_FUNCPTR(XcursorImageLoadCursor);
 #include "winbase.h"
 #include "wine/winuser16.h"
 
-#include "win.h"
 #include "x11drv.h"
 #include "wine/server.h"
 #include "wine/library.h"
@@ -119,14 +118,22 @@ void X11DRV_Xcursor_Init(void)
  *
  * get the coordinates of a mouse event
  */
-static inline void get_coords( HWND hwnd, int x, int y, POINT *pt )
+static inline void get_coords( HWND hwnd, Window window, int x, int y, POINT *pt )
 {
     struct x11drv_win_data *data = X11DRV_get_win_data( hwnd );
 
     if (!data) return;
 
-    pt->x = x + data->whole_rect.left;
-    pt->y = y + data->whole_rect.top;
+    if (window == data->client_window)
+    {
+        pt->x = x + data->client_rect.left;
+        pt->y = y + data->client_rect.top;
+    }
+    else
+    {
+        pt->x = x + data->whole_rect.left;
+        pt->y = y + data->whole_rect.top;
+    }
 }
 
 /***********************************************************************
@@ -165,12 +172,7 @@ static void update_mouse_state( HWND hwnd, Window window, int x, int y, unsigned
 {
     struct x11drv_thread_data *data = x11drv_thread_data();
 
-    if (window == root_window)
-    {
-        x += virtual_screen_rect.left;
-        y += virtual_screen_rect.top;
-    }
-    get_coords( hwnd, x, y, pt );
+    get_coords( hwnd, window, x, y, pt );
 
     /* update the cursor */
 
@@ -283,6 +285,11 @@ void X11DRV_send_mouse_input( HWND hwnd, DWORD flags, DWORD x, DWORD y,
         {
             pt.x = x;
             pt.y = y;
+            wine_tsx11_lock();
+            if (cursor_pos.x == x && cursor_pos.y == y &&
+                (flags & ~(MOUSEEVENTF_MOVE | MOUSEEVENTF_ABSOLUTE)))
+                flags &= ~MOUSEEVENTF_MOVE;
+            wine_tsx11_unlock();
         }
     }
     else if (flags & MOUSEEVENTF_MOVE)
@@ -404,6 +411,7 @@ static XcursorImage *create_cursor_image( CURSORICONINFO *ptr )
     int and_width_bytes, xor_width_bytes;
     XcursorPixel *pixel_ptr;
     XcursorImage *image;
+    BOOL alpha_zero = TRUE;
 
     ymax = (ptr->nHeight > 32) ? 32 : ptr->nHeight;
     xmax = (ptr->nWidth > 32) ? 32 : ptr->nWidth;
@@ -420,6 +428,37 @@ static XcursorImage *create_cursor_image( CURSORICONINFO *ptr )
     image = pXcursorImageCreate( xmax, ymax );
     pixel_ptr = image->pixels;
 
+    /* Generally 32 bit bitmaps have an alpha channel which is used in favor
+     * of the AND mask. However, if all pixels have alpha = 0x00, the bitmap
+     * is treated like one without alpha and the masks are used. As soon as
+     * one pixel has alpha != 0x00, and the mask ignored as described in the
+     * docs.
+     *
+     * This is most likely for applications which create the bitmaps with
+     * CreateDIBitmap, which creates a device dependent bitmap, so the format
+     * that arrives when loading depends on the screen's bpp. Apps that were
+     * written at 8 / 16 bpp times do not know about the 32 bit alpha, so
+     * they would get a completely transparent cursor on 32 bit displays.
+     *
+     * Non-32 bit bitmaps always use the AND mask
+     */
+    if(ptr->bBitsPerPixel == 32)
+    {
+        for (y = 0; alpha_zero && y < ymax; ++y)
+        {
+            xor_ptr = xor_bits + (y * xor_width_bytes);
+            for (x = 0; x < xmax; ++x)
+            {
+                if (xor_ptr[3] != 0x00)
+                {
+                    alpha_zero = FALSE;
+                    break;
+                }
+                xor_ptr+=4;
+            }
+        }
+    }
+
     /* On windows, to calculate the color for a pixel, first an AND is done
      * with the background and the "and" bitmap, then an XOR with the "xor"
      * bitmap. This means that when the data in the "and" bitmap is 0, the
@@ -431,7 +470,7 @@ static XcursorImage *create_cursor_image( CURSORICONINFO *ptr )
      * background color.
      *
      * Since we can't support inverting colors, we map the grayscale value of
-     * the "xor" data to the alpha channel, and xor the the color with either
+     * the "xor" data to the alpha channel, and xor the color with either
      * black or white.
      */
     for (y = 0; y < ymax; ++y)
@@ -479,7 +518,7 @@ static XcursorImage *create_cursor_image( CURSORICONINFO *ptr )
                     return 0;
             }
 
-            if (ptr->bBitsPerPixel != 32)
+            if (alpha_zero)
             {
                 /* Alpha channel */
                 if (~*and_ptr & (1 << (7 - (x & 7)))) *pixel_ptr |= 0xff << 24;
@@ -529,8 +568,8 @@ static Cursor create_xcursor_cursor( Display *display, CURSORICONINFO *ptr )
     /* Make sure hotspot is valid */
     image->xhot = ptr->ptHotSpot.x;
     image->yhot = ptr->ptHotSpot.y;
-    if (image->xhot < 0 || image->xhot >= image->width ||
-        image->yhot < 0 || image->yhot >= image->height)
+    if (image->xhot >= image->width ||
+        image->yhot >= image->height)
     {
         image->xhot = image->width / 2;
         image->yhot = image->height / 2;
@@ -1071,6 +1110,7 @@ void X11DRV_EnterNotify( HWND hwnd, XEvent *xev )
 
     if (!hwnd) return;
     if (event->detail == NotifyVirtual || event->detail == NotifyNonlinearVirtual) return;
+    if (event->window == x11drv_thread_data()->grab_window) return;
 
     /* simulate a mouse motion event */
     update_mouse_state( hwnd, event->window, event->x, event->y, event->state, &pt );
