@@ -70,6 +70,7 @@ Screen *screen;
 Visual *visual;
 unsigned int screen_width;
 unsigned int screen_height;
+unsigned int screen_bpp;
 unsigned int screen_depth;
 RECT virtual_screen_rect;
 Window root_window;
@@ -122,12 +123,15 @@ static const char * const atom_names[NB_XATOMS - FIRST_XATOM] =
     "RAW_CAP_HEIGHT",
     "WM_PROTOCOLS",
     "WM_DELETE_WINDOW",
+    "WM_STATE",
     "WM_TAKE_FOCUS",
     "KWM_DOCKWINDOW",
     "DndProtocol",
     "DndSelection",
+    "_ICC_PROFILE",
     "_MOTIF_WM_HINTS",
     "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
+    "_NET_SUPPORTED",
     "_NET_SYSTEM_TRAY_OPCODE",
     "_NET_SYSTEM_TRAY_S0",
     "_NET_WM_MOVERESIZE",
@@ -135,7 +139,12 @@ static const char * const atom_names[NB_XATOMS - FIRST_XATOM] =
     "_NET_WM_PID",
     "_NET_WM_PING",
     "_NET_WM_STATE",
+    "_NET_WM_STATE_ABOVE",
     "_NET_WM_STATE_FULLSCREEN",
+    "_NET_WM_STATE_MAXIMIZED_HORZ",
+    "_NET_WM_STATE_MAXIMIZED_VERT",
+    "_NET_WM_STATE_SKIP_PAGER",
+    "_NET_WM_STATE_SKIP_TASKBAR",
     "_NET_WM_WINDOW_TYPE",
     "_NET_WM_WINDOW_TYPE_DIALOG",
     "_NET_WM_WINDOW_TYPE_NORMAL",
@@ -273,6 +282,37 @@ void wine_tsx11_unlock(void)
 }
 
 
+/***********************************************************************
+ *		depth_to_bpp
+ *
+ * Convert X11-reported depth to the BPP value that Windows apps expect to see.
+ */
+unsigned int depth_to_bpp( unsigned int depth )
+{
+    switch (depth)
+    {
+    case 1:
+    case 8:
+        return depth;
+    case 15:
+    case 16:
+        return 16;
+    case 24:
+        /* This is not necessarily right. X11 always has 24 bits per pixel, but it can run
+         * with 24 bit framebuffers and 32 bit framebuffers. It doesn't make any difference
+         * for windowing, but gl applications can get visuals with alpha channels. So we
+         * should check the framebuffer and/or opengl formats available to find out what the
+         * framebuffer actually does
+         */
+    case 32:
+        return 32;
+    default:
+        FIXME( "Unexpected X11 depth %d bpp, what to report to app?\n", depth );
+        return depth;
+    }
+}
+
+
 /***********************************************************************
  *		get_config_key
  *
@@ -441,7 +481,6 @@ sym_not_found:
 static BOOL process_attach(void)
 {
     Display *display;
-    XVisualInfo *desktop_vi = NULL;
     const char *env;
 
     setup_options();
@@ -478,24 +517,13 @@ static BOOL process_attach(void)
         }
     }
     if (!screen_depth) screen_depth = DefaultDepthOfScreen( screen );
-
-    /* If OpenGL is available, change the default visual, etc as necessary */
-    if ((desktop_vi = X11DRV_setup_opengl_visual( display )))
-    {
-        visual       = desktop_vi->visual;
-        screen       = ScreenOfDisplay(display, desktop_vi->screen);
-        screen_depth = desktop_vi->depth;
-        XFree(desktop_vi);
-    }
+    screen_bpp = depth_to_bpp( screen_depth );
 
     XInternAtoms( display, (char **)atom_names, NB_XATOMS - FIRST_XATOM, False, X11DRV_Atoms );
 
     if (TRACE_ON(synchronous)) XSynchronize( display, True );
 
-    screen_width  = WidthOfScreen( screen );
-    screen_height = HeightOfScreen( screen );
-
-    xinerama_init();
+    xinerama_init( WidthOfScreen(screen), HeightOfScreen(screen) );
     X11DRV_Settings_Init();
 
 #ifdef HAVE_LIBXXF86VM
@@ -510,8 +538,7 @@ static BOOL process_attach(void)
     X11DRV_XComposite_Init();
 #endif
 
-    X11DRV_ClipCursor( NULL );
-    X11DRV_InitKeyboard();
+    X11DRV_InitKeyboard( gdi_display );
     X11DRV_InitClipboard();
 
     return TRUE;
@@ -590,7 +617,7 @@ struct x11drv_thread_data *x11drv_init_thread_data(void)
 {
     struct x11drv_thread_data *data;
 
-    if (!(data = HeapAlloc( GetProcessHeap(), 0, sizeof(*data) )))
+    if (!(data = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*data) )))
     {
         ERR( "could not create data\n" );
         ExitProcess(1);
@@ -624,19 +651,12 @@ struct x11drv_thread_data *x11drv_init_thread_data(void)
     if (TRACE_ON(synchronous)) XSynchronize( data->display, True );
     wine_tsx11_unlock();
 
-    if (!use_xim)
-        data->xim = NULL;
-    else if (!(data->xim = X11DRV_SetupXIM( data->display, input_style )))
+    if (use_xim && !(data->xim = X11DRV_SetupXIM( data->display, input_style )))
         WARN("Input Method is not available\n");
 
     set_queue_display_fd( data->display );
-    data->process_event_count = 0;
-    data->cursor = None;
-    data->cursor_window = None;
-    data->grab_window = None;
-    data->last_focus = 0;
-    data->selection_wnd = 0;
     TlsSetValue( thread_data_tls_index, data );
+    X11DRV_SetCursor( NULL );
     return data;
 }
 
