@@ -687,7 +687,7 @@ BOOL	MMDRV_Init(void)
         char *next = strchr(p, ',');
         if (next) *next++ = 0;
         sprintf( filename, "wine%s.drv", p );
-        ret |= MMDRV_Install( filename, filename, FALSE );
+        if ((ret = MMDRV_Install( filename, filename, FALSE ))) break;
         p = next;
     }
 
