@@ -28,6 +28,7 @@
 #include "winerror.h"
 #include "winbase.h"
 #include "winuser.h"
+#include "winnls.h"
 
 #include "wintab.h"
 #include "wintab_internal.h"
@@ -45,66 +46,84 @@ static BOOL gLoaded;
 static LPOPENCONTEXT gOpenContexts;
 static HCTX gTopContext = (HCTX)0xc00;
 
-static char* DUMPBITS(int x, char* buf)
+static void LOGCONTEXTAtoW(const LOGCONTEXTA *in, LOGCONTEXTW *out)
 {
-    strcpy(buf,"{");
-   if (x&PK_CONTEXT) strcat(buf,"PK_CONTEXT ");
-   if (x&PK_STATUS) strcat(buf, "PK_STATUS ");
-   if (x&PK_TIME) strcat(buf, "PK_TIME ");
-   if (x&PK_CHANGED) strcat(buf, "PK_CHANGED ");
-   if (x&PK_SERIAL_NUMBER) strcat(buf, "PK_SERIAL_NUMBER ");
-   if (x&PK_CURSOR) strcat(buf, "PK_CURSOR ");
-   if (x&PK_BUTTONS) strcat(buf, "PK_BUTTONS ");
-   if (x&PK_X) strcat(buf, "PK_X ");
-   if (x&PK_Y) strcat(buf, "PK_Y ");
-   if (x&PK_Z) strcat(buf, "PK_Z ");
-   if (x&PK_NORMAL_PRESSURE) strcat(buf, "PK_NORMAL_PRESSURE ");
-   if (x&PK_TANGENT_PRESSURE) strcat(buf, "PK_TANGENT_PRESSURE ");
-   if (x&PK_ORIENTATION) strcat(buf, "PK_ORIENTATION ");
-   if (x&PK_ROTATION) strcat(buf, "PK_ROTATION ");
-    strcat(buf, "}");
-    return buf;
+    MultiByteToWideChar(CP_ACP, 0, in->lcName, -1, out->lcName, LCNAMELEN);
+    out->lcName[LCNAMELEN - 1] = 0;
+    /* we use the fact that the fields after lcName are the same in LOGCONTEXTA and W */
+    memcpy(&out->lcOptions, &in->lcOptions, sizeof(LOGCONTEXTA) - FIELD_OFFSET(LOGCONTEXTA, lcOptions));
+}
+
+static void LOGCONTEXTWtoA(const LOGCONTEXTW *in, LOGCONTEXTA *out)
+{
+    WideCharToMultiByte(CP_ACP, 0, in->lcName, LCNAMELEN, out->lcName, LCNAMELEN, NULL, NULL);
+    out->lcName[LCNAMELEN - 1] = 0;
+    /* we use the fact that the fields after lcName are the same in LOGCONTEXTA and W */
+    memcpy(&out->lcOptions, &in->lcOptions, sizeof(LOGCONTEXTW) - FIELD_OFFSET(LOGCONTEXTW, lcOptions));
+}
+
+static BOOL is_logcontext_category(UINT wCategory)
+{
+    return wCategory == WTI_DEFSYSCTX || wCategory == WTI_DEFCONTEXT || wCategory == WTI_DDCTXS;
+}
+
+static BOOL is_string_field(UINT wCategory, UINT nIndex)
+{
+    if (wCategory == WTI_INTERFACE && nIndex == IFC_WINTABID)
+        return TRUE;
+    if (is_logcontext_category(wCategory) && nIndex == CTX_NAME)
+        return TRUE;
+    if ((wCategory >= WTI_CURSORS && wCategory <= WTI_CURSORS + 9) &&
+            (nIndex == CSR_NAME || nIndex == CSR_BTNNAMES))
+        return TRUE;
+    if (wCategory == WTI_DEVICES && (nIndex == DVC_NAME || nIndex == DVC_PNPID))
+        return TRUE;
+    return FALSE;
+}
+
+static const char* DUMPBITS(int x)
+{
+    char buf[200];
+    buf[0] = 0;
+    if (x&PK_CONTEXT) strcat(buf,"PK_CONTEXT ");
+    if (x&PK_STATUS) strcat(buf, "PK_STATUS ");
+    if (x&PK_TIME) strcat(buf, "PK_TIME ");
+    if (x&PK_CHANGED) strcat(buf, "PK_CHANGED ");
+    if (x&PK_SERIAL_NUMBER) strcat(buf, "PK_SERIAL_NUMBER ");
+    if (x&PK_CURSOR) strcat(buf, "PK_CURSOR ");
+    if (x&PK_BUTTONS) strcat(buf, "PK_BUTTONS ");
+    if (x&PK_X) strcat(buf, "PK_X ");
+    if (x&PK_Y) strcat(buf, "PK_Y ");
+    if (x&PK_Z) strcat(buf, "PK_Z ");
+    if (x&PK_NORMAL_PRESSURE) strcat(buf, "PK_NORMAL_PRESSURE ");
+    if (x&PK_TANGENT_PRESSURE) strcat(buf, "PK_TANGENT_PRESSURE ");
+    if (x&PK_ORIENTATION) strcat(buf, "PK_ORIENTATION ");
+    if (x&PK_ROTATION) strcat(buf, "PK_ROTATION ");
+    return wine_dbg_sprintf("{%s}",buf);
 }
 
 static inline void DUMPPACKET(WTPACKET packet)
 {
     TRACE("pkContext: %p pkStatus: 0x%x pkTime : 0x%x pkChanged: 0x%x pkSerialNumber: 0x%x pkCursor : %i pkButtons: %x pkX: %i pkY: %i pkZ: %i pkNormalPressure: %i pkTangentPressure: %i pkOrientation: (%i,%i,%i) pkRotation: (%i,%i,%i)\n",
-          packet.pkContext,
-    (UINT)packet.pkStatus,
-    (UINT)packet.pkTime,
-    (UINT)packet.pkChanged,
-    packet.pkSerialNumber,
-    packet.pkCursor,
-    (UINT)packet.pkButtons,
-    packet.pkX,
-    packet.pkY,
-    packet.pkZ,
-    packet.pkNormalPressure,
-    packet.pkTangentPressure,
-    packet.pkOrientation.orAzimuth,
-        packet.pkOrientation.orAltitude, packet.pkOrientation.orTwist,
-    packet.pkRotation.roPitch,
-        packet.pkRotation.roRoll, packet.pkRotation.roYaw);
+          packet.pkContext, packet.pkStatus, packet.pkTime, packet.pkChanged, packet.pkSerialNumber,
+          packet.pkCursor, packet.pkButtons, packet.pkX, packet.pkY, packet.pkZ,
+          packet.pkNormalPressure, packet.pkTangentPressure,
+          packet.pkOrientation.orAzimuth, packet.pkOrientation.orAltitude, packet.pkOrientation.orTwist,
+          packet.pkRotation.roPitch, packet.pkRotation.roRoll, packet.pkRotation.roYaw);
 }
 
-static inline void DUMPCONTEXT(LOGCONTEXTA lc)
+static inline void DUMPCONTEXT(LOGCONTEXTW lc)
 {
-        CHAR mmsg[4000];
-        CHAR bits[100];
-        CHAR bits1[100];
-        CHAR bits2[100];
-
-        sprintf(mmsg,"%s, %x, %x, %x, %x, %x, %x, %x%s, %x%s, %x%s, %x, %x, %i, %i, %i, %i ,%i, %i, %i, %i, %i,%i, %i, %i, %i, %i, %i, %i, %i, %i, %i, %i %i %i",
-    debugstr_a(lc.lcName), lc.lcOptions, lc.lcStatus, lc.lcLocks, lc.lcMsgBase,
-lc.lcDevice, lc.lcPktRate, (UINT)lc.lcPktData, DUMPBITS(lc.lcPktData,bits),
-(UINT)lc.lcPktMode, DUMPBITS(lc.lcPktMode,bits1), (UINT)lc.lcMoveMask,
-DUMPBITS(lc.lcMoveMask,bits2), (INT)lc.lcBtnDnMask, (INT)lc.lcBtnUpMask,
-(INT)lc.lcInOrgX, (INT)lc.lcInOrgY, (INT)lc.lcInOrgZ, lc.lcInExtX, lc.lcInExtY,
-lc.lcInExtZ, lc.lcOutOrgX, lc.lcOutOrgY, lc.lcOutOrgZ, lc.lcOutExtX,
-lc.lcOutExtY, lc.lcOutExtZ, lc.lcSensX, lc.lcSensY, lc.lcSensZ, lc.lcSysMode,
-lc.lcSysOrgX, lc.lcSysOrgY, lc.lcSysExtX, lc.lcSysExtY, lc.lcSysSensX,
-lc.lcSysSensY);
-        TRACE("context: %s\n",mmsg);
+    TRACE("context: %s, %x, %x, %x, %x, %x, %x, %x%s, %x%s, %x%s, %x, %x, %i, %i, %i, %i ,%i, %i, %i, %i, %i,%i, %i, %i, %i, %i, %i, %i, %i, %i, %i, %i %i %i\n",
+          wine_dbgstr_w(lc.lcName), lc.lcOptions, lc.lcStatus, lc.lcLocks, lc.lcMsgBase,
+          lc.lcDevice, lc.lcPktRate, lc.lcPktData, DUMPBITS(lc.lcPktData),
+          lc.lcPktMode, DUMPBITS(lc.lcPktMode), lc.lcMoveMask,
+          DUMPBITS(lc.lcMoveMask), lc.lcBtnDnMask, lc.lcBtnUpMask,
+          lc.lcInOrgX, lc.lcInOrgY, lc.lcInOrgZ, lc.lcInExtX, lc.lcInExtY,
+          lc.lcInExtZ, lc.lcOutOrgX, lc.lcOutOrgY, lc.lcOutOrgZ, lc.lcOutExtX,
+          lc.lcOutExtY, lc.lcOutExtZ, lc.lcSensX, lc.lcSensY, lc.lcSensZ, lc.lcSysMode,
+          lc.lcSysOrgX, lc.lcSysOrgY, lc.lcSysExtX, lc.lcSysExtY, lc.lcSysSensX,
+          lc.lcSysSensY);
 }
 
 
@@ -138,8 +157,7 @@ int TABLET_PostTabletMessage(LPOPENCONTEXT newcontext, UINT msg, WPARAM wParam,
     return 0;
 }
 
-static inline DWORD ScaleForContext(DWORD In, DWORD InOrg, DWORD InExt, DWORD
-                                    OutOrg, DWORD OutExt)
+static inline DWORD ScaleForContext(DWORD In, LONG InOrg, LONG InExt, LONG OutOrg, LONG OutExt)
 {
     if (((InExt > 0 )&&(OutExt > 0)) || ((InExt<0) && (OutExt < 0)))
         return ((In - InOrg) * abs(OutExt) / abs(InExt)) + OutOrg;
@@ -147,21 +165,6 @@ static inline DWORD ScaleForContext(DWORD In, DWORD InOrg, DWORD InExt, DWORD
         return ((abs(InExt) - (In - InOrg))*abs(OutExt) / abs(InExt)) + OutOrg;
 }
 
-LPOPENCONTEXT FindOpenContext(HWND hwnd)
-{
-    LPOPENCONTEXT ptr;
-
-    EnterCriticalSection(&csTablet);
-    ptr = gOpenContexts;
-    while (ptr)
-    {
-        TRACE("Trying Context %p (%p %p)\n",ptr->handle,hwnd,ptr->hwndOwner);
-        if (ptr->hwndOwner == hwnd) break;
-    }
-    LeaveCriticalSection(&csTablet);
-    return ptr;
-}
-
 LPOPENCONTEXT AddPacketToContextQueue(LPWTPACKET packet, HWND hwnd)
 {
     LPOPENCONTEXT ptr=NULL;
@@ -211,8 +214,7 @@ LPOPENCONTEXT AddPacketToContextQueue(LPWTPACKET packet, HWND hwnd)
             else
             {
                 TRACE("Placed in queue %p index %i\n",ptr->handle,tgt);
-                memcpy(&ptr->PacketQueue[tgt], packet, sizeof
-                        (WTPACKET));
+                ptr->PacketQueue[tgt] = *packet;
                 ptr->PacketsQueued++;
 
                 if (ptr->ActiveCursor != packet->pkCursor)
@@ -236,12 +238,12 @@ LPOPENCONTEXT AddPacketToContextQueue(LPWTPACKET packet, HWND hwnd)
 /*
  * Flushes all packets from the queue.
  */
-inline static void TABLET_FlushQueue(LPOPENCONTEXT context)
+static inline void TABLET_FlushQueue(LPOPENCONTEXT context)
 {
     context->PacketsQueued = 0;
 }
 
-inline static int CopyTabletData(LPVOID target, LPVOID src, INT size)
+static inline int CopyTabletData(LPVOID target, LPVOID src, INT size)
 {
     memcpy(target,src,size);
     return(size);
@@ -270,10 +272,9 @@ static LPVOID TABLET_CopyPacketData(LPOPENCONTEXT context, LPVOID lpPkt,
                                     LPWTPACKET wtp)
 {
     LPBYTE ptr;
-    CHAR bits[100];
 
     ptr = lpPkt;
-    TRACE("Packet Bits %s\n",DUMPBITS(context->context.lcPktData,bits));
+    TRACE("Packet Bits %s\n",DUMPBITS(context->context.lcPktData));
 
     if (context->context.lcPktData & PK_CONTEXT)
         ptr+=CopyTabletData(ptr,&wtp->pkContext,sizeof(HCTX));
@@ -346,12 +347,11 @@ static VOID TABLET_BlankPacketData(LPOPENCONTEXT context, LPVOID lpPkt, INT n)
 }
 
 
-/***********************************************************************
- *		WTInfoA (WINTAB32.20)
- */
-UINT WINAPI WTInfoA(UINT wCategory, UINT nIndex, LPVOID lpOutput)
+UINT WINAPI WTInfoT(UINT wCategory, UINT nIndex, LPVOID lpOutput, BOOL bUnicode)
 {
     UINT result;
+
+    TRACE("(%d, %d, %p, %d)\n", wCategory, nIndex, lpOutput, bUnicode);
     if (gLoaded == FALSE)
          LoadTablet();
 
@@ -374,36 +374,64 @@ UINT WINAPI WTInfoA(UINT wCategory, UINT nIndex, LPVOID lpOutput)
         }
     }
 
-    result =  pWTInfoA( wCategory, nIndex, lpOutput );
-
-    /*
-     *  Handle system extents here, as we can use user32.dll code to set them.
-     */
-    if(wCategory == WTI_DEFSYSCTX && nIndex == 0)
+    if (is_logcontext_category(wCategory) && nIndex == 0)
     {
-        LPLOGCONTEXTA lpCtx = (LPLOGCONTEXTA)lpOutput;
-        lpCtx->lcSysExtX = GetSystemMetrics(SM_CXSCREEN);
-        lpCtx->lcSysExtY = GetSystemMetrics(SM_CYSCREEN);
+        if (lpOutput)
+        {
+            LOGCONTEXTW buf;
+            pWTInfoW(wCategory, nIndex, &buf);
+
+            /*  Handle system extents here, as we can use user32.dll code to set them */
+            if(wCategory == WTI_DEFSYSCTX && nIndex == 0)
+            {
+                buf.lcSysExtX = GetSystemMetrics(SM_CXSCREEN);
+                buf.lcSysExtY = GetSystemMetrics(SM_CYSCREEN);
+            }
+
+            if (bUnicode)
+                memcpy(lpOutput, &buf, sizeof(buf));
+            else
+                LOGCONTEXTWtoA(&buf, lpOutput);
+        }
+
+        result = bUnicode ? sizeof(LOGCONTEXTW) : sizeof(LOGCONTEXTA);
     }
+    else if (is_string_field(wCategory, nIndex) && !bUnicode)
+    {
+        int size = pWTInfoW(wCategory, nIndex, NULL);
+        WCHAR *buf = HeapAlloc(GetProcessHeap(), 0, size);
+        pWTInfoW(wCategory, nIndex, buf);
+        result = WideCharToMultiByte(CP_ACP, 0, buf, size/sizeof(WCHAR), lpOutput, lpOutput ? 2*size : 0, NULL, NULL);
+        HeapFree(GetProcessHeap(), 0, buf);
+    }
+    else
+        result =  pWTInfoW(wCategory, nIndex, lpOutput);
+
+    TRACE("returns %d\n", result);
     return result;
 }
 
+/***********************************************************************
+ *		WTInfoA (WINTAB32.20)
+ */
+UINT WINAPI WTInfoA(UINT wCategory, UINT nIndex, LPVOID lpOutput)
+{
+    return WTInfoT(wCategory, nIndex, lpOutput, FALSE);
+}
+
+
 /***********************************************************************
  *		WTInfoW (WINTAB32.1020)
  */
 UINT WINAPI WTInfoW(UINT wCategory, UINT nIndex, LPVOID lpOutput)
 {
-    FIXME("(%u, %u, %p): stub\n", wCategory, nIndex, lpOutput);
-
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-
-    return 0;
+    return WTInfoT(wCategory, nIndex, lpOutput, TRUE);
 }
 
 /***********************************************************************
- *		WTOpenA (WINTAB32.21)
+ *		WTOpenW (WINTAB32.2021)
  */
-HCTX WINAPI WTOpenA(HWND hWnd, LPLOGCONTEXTA lpLogCtx, BOOL fEnable)
+HCTX WINAPI WTOpenW(HWND hWnd, LPLOGCONTEXTW lpLogCtx, BOOL fEnable)
 {
     LPOPENCONTEXT newcontext;
 
@@ -411,7 +439,7 @@ HCTX WINAPI WTOpenA(HWND hWnd, LPLOGCONTEXTA lpLogCtx, BOOL fEnable)
     DUMPCONTEXT(*lpLogCtx);
 
     newcontext = HeapAlloc(GetProcessHeap(), 0 , sizeof(OPENCONTEXT));
-    memcpy(&(newcontext->context),lpLogCtx,sizeof(LOGCONTEXTA));
+    newcontext->context = *lpLogCtx;
     newcontext->hwndOwner = hWnd;
     newcontext->enabled = fEnable;
     newcontext->ActiveCursor = -1;
@@ -440,15 +468,14 @@ HCTX WINAPI WTOpenA(HWND hWnd, LPLOGCONTEXTA lpLogCtx, BOOL fEnable)
 }
 
 /***********************************************************************
- *		WTOpenW (WINTAB32.1021)
+ *		WTOpenA (WINTAB32.21)
  */
-HCTX WINAPI WTOpenW(HWND hWnd, LPLOGCONTEXTW lpLogCtx, BOOL fEnable)
+HCTX WINAPI WTOpenA(HWND hWnd, LPLOGCONTEXTA lpLogCtx, BOOL fEnable)
 {
-    FIXME("(%p, %p, %u): stub\n", hWnd, lpLogCtx, fEnable);
+    LOGCONTEXTW logCtxW;
 
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-
-    return NULL;
+    LOGCONTEXTAtoW(lpLogCtx, &logCtxW);
+    return WTOpenW(hWnd, &logCtxW, fEnable);
 }
 
 /***********************************************************************
@@ -635,7 +662,7 @@ BOOL WINAPI WTGetA(HCTX hCtx, LPLOGCONTEXTA lpLogCtx)
 
     EnterCriticalSection(&csTablet);
     context = TABLET_FindOpenContext(hCtx);
-    memmove(lpLogCtx,&context->context,sizeof(LOGCONTEXTA));
+    LOGCONTEXTWtoA(&context->context, lpLogCtx);
     LeaveCriticalSection(&csTablet);
 
     return TRUE;
@@ -646,11 +673,18 @@ BOOL WINAPI WTGetA(HCTX hCtx, LPLOGCONTEXTA lpLogCtx)
  */
 BOOL WINAPI WTGetW(HCTX hCtx, LPLOGCONTEXTW lpLogCtx)
 {
-    FIXME("(%p, %p): stub\n", hCtx, lpLogCtx);
+    LPOPENCONTEXT context;
 
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    TRACE("(%p, %p)\n", hCtx, lpLogCtx);
 
-    return FALSE;
+    if (!hCtx) return 0;
+
+    EnterCriticalSection(&csTablet);
+    context = TABLET_FindOpenContext(hCtx);
+    memmove(lpLogCtx,&context->context,sizeof(LOGCONTEXTW));
+    LeaveCriticalSection(&csTablet);
+
+    return TRUE;
 }
 
 /***********************************************************************
