@@ -17,14 +17,18 @@
 #include <stdarg.h>
 #endif
 
-#if !defined(_MSC_VER) && !defined(__int64)
-#define __int64 long long
-#endif
-
 #if defined(__x86_64__) && !defined(_WIN64)
 #define _WIN64
 #endif
 
+#if !defined(_MSC_VER) && !defined(__int64)
+# ifdef _WIN64
+#   define __int64 long
+# else
+#   define __int64 long long
+# endif
+#endif
+
 /* file._flag flags */
 #define _IOREAD          0x0001
 #define _IOWRT           0x0002
