@@ -25,6 +25,10 @@
 # error You must include config.h to use this header
 #endif
 
+#ifdef __WINE_BASETSD_H
+# error You must include port.h before all other headers
+#endif
+
 #define _FILE_OFFSET_BITS 64
 #define _GNU_SOURCE  /* for pread/pwrite */
 #include <fcntl.h>
@@ -122,6 +126,10 @@ struct statvfs
 #define pclose _pclose
 #endif
 
+#if !defined(HAVE_STRDUP) && defined(HAVE__STRDUP)
+#define strdup _strdup
+#endif
+
 #if !defined(HAVE_SNPRINTF) && defined(HAVE__SNPRINTF)
 #define snprintf _snprintf
 #endif
@@ -130,6 +138,14 @@ struct statvfs
 #define vsnprintf _vsnprintf
 #endif
 
+#if !defined(HAVE_STRTOLL) && defined(HAVE__STRTOI64)
+#define strtoll _strtoi64
+#endif
+
+#if !defined(HAVE_STRTOULL) && defined(HAVE__STRTOUI64)
+#define strtoull _strtoui64
+#endif
+
 #ifndef S_ISLNK
 # define S_ISLNK(mod) (0)
 #endif
@@ -272,7 +288,7 @@ int ffs( int x );
 
 #ifndef HAVE_FUTIMES
 struct timeval;
-int futimes(int fd, const struct timeval tv[2]);
+int futimes(int fd, const struct timeval *tv);
 #endif
 
 #ifndef HAVE_GETPAGESIZE
