@@ -57,6 +57,7 @@ extern void winetest_start_todo( const char* platform );
 extern int winetest_loop_todo(void);
 extern void winetest_end_todo( const char* platform );
 extern int winetest_get_mainargs( char*** pargv );
+extern void winetest_wait_child_process( HANDLE process );
 
 #ifdef STANDALONE
 #define START_TEST(name) \
@@ -340,12 +341,30 @@ int winetest_get_mainargs( char*** pargv )
     return winetest_argc;
 }
 
+void winetest_wait_child_process( HANDLE process )
+{
+    DWORD exit_code = 1;
+
+    if (WaitForSingleObject( process, 30000 ))
+        fprintf( stdout, "%s: child process wait failed\n", current_test->name );
+    else
+        GetExitCodeProcess( process, &exit_code );
+
+    if (exit_code)
+    {
+        fprintf( stdout, "%s: %u failures in child process\n",
+                 current_test->name, exit_code );
+        while (exit_code-- > 0)
+            InterlockedIncrement(&failures);
+    }
+}
+
 /* Find a test by name */
 static const struct test *find_test( const char *name )
 {
     const struct test *test;
     const char *p;
-    int len;
+    size_t len;
 
     if ((p = strrchr( name, '/' ))) name = p + 1;
     if ((p = strrchr( name, '\\' ))) name = p + 1;
