@@ -307,6 +307,70 @@ typedef struct _INTERNET_CACHE_GROUP_INFOW {
 DECL_WINELIB_TYPE_AW(INTERNET_CACHE_GROUP_INFO)
 DECL_WINELIB_TYPE_AW(LPINTERNET_CACHE_GROUP_INFO)
 
+typedef struct _INTERNET_PER_CONN_OPTIONA {
+    DWORD dwOption;
+    union {
+        DWORD    dwValue;
+        LPSTR    pszValue;
+        FILETIME ftValue;
+    } Value;
+} INTERNET_PER_CONN_OPTIONA, *LPINTERNET_PER_CONN_OPTIONA;
+
+typedef struct _INTERNET_PER_CONN_OPTIONW {
+    DWORD dwOption;
+    union {
+        DWORD    dwValue;
+        LPWSTR   pszValue;
+        FILETIME ftValue;
+    } Value;
+} INTERNET_PER_CONN_OPTIONW, *LPINTERNET_PER_CONN_OPTIONW;
+
+DECL_WINELIB_TYPE_AW(INTERNET_PER_CONN_OPTION)
+DECL_WINELIB_TYPE_AW(LPINTERNET_PER_CONN_OPTION)
+
+#define INTERNET_PER_CONN_FLAGS                        1
+#define INTERNET_PER_CONN_PROXY_SERVER                 2
+#define INTERNET_PER_CONN_PROXY_BYPASS                 3
+#define INTERNET_PER_CONN_AUTOCONFIG_URL               4
+#define INTERNET_PER_CONN_AUTODISCOVERY_FLAGS          5
+#define INTERNET_PER_CONN_AUTOCONFIG_SECONDARY_URL     6
+#define INTERNET_PER_CONN_AUTOCONFIG_RELOAD_DELAY_MINS 7
+#define INTERNET_PER_CONN_AUTOCONFIG_LAST_DETECT_TIME  8
+#define INTERNET_PER_CONN_AUTOCONFIG_LAST_DETECT_URL   9
+
+/* Values for INTERNET_PER_CONN_FLAGS */
+#define PROXY_TYPE_DIRECT         0x00000001
+#define PROXY_TYPE_PROXY          0x00000002
+#define PROXY_TYPE_AUTO_PROXY_URL 0x00000004
+#define PROXY_TYPE_AUTO_DETECT    0x00000008
+/* Values for INTERNET_PER_CONN_AUTODISCOVERY_FLAGS */
+#define AUTO_PROXY_FLAG_USER_SET                0x00000001
+#define AUTO_PROXY_FLAG_ALWAYS_DETECT           0x00000002
+#define AUTO_PROXY_FLAG_DETECTION_RUN           0x00000004
+#define AUTO_PROXY_FLAG_MIGRATED                0x00000008
+#define AUTO_PROXY_FLAG_DONT_CACHE_PROXY_RESULT 0x00000010
+#define AUTO_PROXY_FLAG_CACHE_INIT_RUN          0x00000020
+#define AUTO_PROXY_FLAG_DETECTION_SUSPECT       0x00000040
+
+typedef struct _INTERNET_PER_CONN_OPTION_LISTA {
+    DWORD                       dwSize;
+    LPSTR                       pszConnection;
+    DWORD                       dwOptionCount;
+    DWORD                       dwOptionError;
+    LPINTERNET_PER_CONN_OPTIONA pOptions;
+} INTERNET_PER_CONN_OPTION_LISTA, *LPINTERNET_PER_CONN_OPTION_LISTA;
+
+typedef struct _INTERNET_PER_CONN_OPTION_LISTW {
+    DWORD                       dwSize;
+    LPWSTR                      pszConnection;
+    DWORD                       dwOptionCount;
+    DWORD                       dwOptionError;
+    LPINTERNET_PER_CONN_OPTIONW pOptions;
+} INTERNET_PER_CONN_OPTION_LISTW, *LPINTERNET_PER_CONN_OPTION_LISTW;
+
+DECL_WINELIB_TYPE_AW(INTERNET_PER_CONN_OPTION_LIST)
+DECL_WINELIB_TYPE_AW(LPINTERNET_PER_CONN_OPTION_LIST)
+
 BOOLAPI InternetTimeFromSystemTimeA(CONST SYSTEMTIME *,DWORD ,LPSTR ,DWORD);
 BOOLAPI InternetTimeFromSystemTimeW(CONST SYSTEMTIME *,DWORD ,LPWSTR ,DWORD);
 #define InternetTimeFromSystemTime WINELIB_NAME_AW(InternetTimeFromSystemTime)
