@@ -498,7 +498,7 @@ typedef struct WS(timeval)
 #else
 #define WS_FD_CLR(fd, set)   __WS_FD_CLR((fd),(set), WS_fd_set)
 #define WS_FD_SET(fd, set)   __WS_FD_SET((fd),(set), WS_fd_set)
-#define WS_FD_ZERO(set)      ((WS_fd_set*)(set))->fd_count=0)
+#define WS_FD_ZERO(set)      (((WS_fd_set*)(set))->fd_count=0)
 #define WS_FD_ISSET(fd, set) __WSAFDIsSet((SOCKET)(fd), (WS_fd_set*)(set))
 #endif
 
