@@ -22,6 +22,8 @@
 
 #include "wine/unicode.h"
 
+extern WCHAR compose( const WCHAR *str );
+
 /* number of following bytes in sequence based on first byte value (for bytes above 0x7f) */
 static const char utf8_length[128] =
 {
@@ -82,7 +84,11 @@ static inline int get_length_wcs_utf8( int flags, const WCHAR *src, unsigned int
         if (val < 0x10000)  /* 0x800-0xffff: 3 bytes */
             len += 3;
         else   /* 0x10000-0x10ffff: 4 bytes */
+        {
             len += 4;
+            src++;
+            srclen--;
+        }
     }
     return len;
 }
@@ -144,15 +150,133 @@ int wine_utf8_wcstombs( int flags, const WCHAR *src, int srclen, char *dst, int
             val >>= 6;
             dst[0] = 0xf0 | val;
             dst += 4;
+            src++;
+            srclen--;
         }
     }
     return dstlen - len;
 }
 
-/* query necessary dst length for src string */
-inline static int get_length_mbs_utf8( int flags, const char *src, int srclen )
+/* helper for the various utf8 mbstowcs functions */
+static inline unsigned int decode_utf8_char( unsigned char ch, const char **str, const char *strend )
 {
-    int len, ret = 0;
+    unsigned int len = utf8_length[ch-0x80];
+    unsigned int res = ch & utf8_mask[len];
+    const char *end = *str + len;
+
+    if (end > strend) return ~0;
+    switch(len)
+    {
+    case 3:
+        if ((ch = end[-3] ^ 0x80) >= 0x40) break;
+        res = (res << 6) | ch;
+        (*str)++;
+    case 2:
+        if ((ch = end[-2] ^ 0x80) >= 0x40) break;
+        res = (res << 6) | ch;
+        (*str)++;
+    case 1:
+        if ((ch = end[-1] ^ 0x80) >= 0x40) break;
+        res = (res << 6) | ch;
+        (*str)++;
+        if (res < utf8_minval[len]) break;
+        return res;
+    }
+    return ~0;
+}
+
+/* query necessary dst length for src string with composition */
+static inline int get_length_mbs_utf8_compose( int flags, const char *src, int srclen )
+{
+    int ret = 0;
+    unsigned int res;
+    WCHAR composed[2];
+    const char *srcend = src + srclen;
+
+    composed[0] = 0;
+    while (src < srcend)
+    {
+        unsigned char ch = *src++;
+        if (ch < 0x80)  /* special fast case for 7-bit ASCII */
+        {
+            composed[0] = ch;
+            ret++;
+            continue;
+        }
+        if ((res = decode_utf8_char( ch, &src, srcend )) <= 0xffff)
+        {
+            if (composed[0])
+            {
+                composed[1] = res;
+                if ((composed[0] = compose( composed ))) continue;
+            }
+            composed[0] = res;
+            ret++;
+        }
+        else if (res <= 0x10ffff)
+        {
+            ret += 2;
+            composed[0] = 0;  /* no composition for surrogates */
+        }
+        else if (flags & MB_ERR_INVALID_CHARS) return -2;  /* bad char */
+        /* otherwise ignore it */
+    }
+    return ret;
+}
+
+/* UTF-8 to wide char string conversion with composition */
+/* return -1 on dst buffer overflow, -2 on invalid input char */
+static int utf8_mbstowcs_compose( int flags, const char *src, int srclen, WCHAR *dst, int dstlen )
+{
+    unsigned int res;
+    const char *srcend = src + srclen;
+    WCHAR composed[2];
+    WCHAR *dstend = dst + dstlen;
+
+    if (!dstlen) return get_length_mbs_utf8_compose( flags, src, srclen );
+
+    composed[0] = 0;
+    while (src < srcend)
+    {
+        unsigned char ch = *src++;
+        if (ch < 0x80)  /* special fast case for 7-bit ASCII */
+        {
+            if (dst >= dstend) return -1;  /* overflow */
+            *dst++ = composed[0] = ch;
+            continue;
+        }
+        if ((res = decode_utf8_char( ch, &src, srcend )) <= 0xffff)
+        {
+            if (composed[0])
+            {
+                composed[1] = res;
+                if ((composed[0] = compose( composed )))
+                {
+                    dst[-1] = composed[0];
+                    continue;
+                }
+            }
+            if (dst >= dstend) return -1;  /* overflow */
+            *dst++ = composed[0] = res;
+        }
+        else if (res <= 0x10ffff) /* we need surrogates */
+        {
+            if (dst >= dstend - 1) return -1;  /* overflow */
+            res -= 0x10000;
+            *dst++ = 0xd800 | (res >> 10);
+            *dst++ = 0xdc00 | (res & 0x3ff);
+            composed[0] = 0;  /* no composition for surrogates */
+        }
+        else if (flags & MB_ERR_INVALID_CHARS) return -2;  /* bad char */
+        /* otherwise ignore it */
+    }
+    return dstlen - (dstend - dst);
+}
+
+/* query necessary dst length for src string */
+static inline int get_length_mbs_utf8( int flags, const char *src, int srclen )
+{
+    int ret = 0;
     unsigned int res;
     const char *srcend = src + srclen;
 
@@ -164,32 +288,12 @@ inline static int get_length_mbs_utf8( int flags, const char *src, int srclen )
             ret++;
             continue;
         }
-        len = utf8_length[ch-0x80];
-        if (src + len > srcend) goto bad;
-        res = ch & utf8_mask[len];
-
-        switch(len)
+        if ((res = decode_utf8_char( ch, &src, srcend )) <= 0x10ffff)
         {
-        case 3:
-            if ((ch = *src ^ 0x80) >= 0x40) goto bad;
-            res = (res << 6) | ch;
-            src++;
-        case 2:
-            if ((ch = *src ^ 0x80) >= 0x40) goto bad;
-            res = (res << 6) | ch;
-            src++;
-        case 1:
-            if ((ch = *src ^ 0x80) >= 0x40) goto bad;
-            res = (res << 6) | ch;
-            src++;
-            if (res < utf8_minval[len]) goto bad;
-            if (res > 0x10ffff) goto bad;
             if (res > 0xffff) ret++;
             ret++;
-            continue;
         }
-    bad:
-        if (flags & MB_ERR_INVALID_CHARS) return -2;  /* bad char */
+        else if (flags & MB_ERR_INVALID_CHARS) return -2;  /* bad char */
         /* otherwise ignore it */
     }
     return ret;
@@ -199,11 +303,12 @@ inline static int get_length_mbs_utf8( int flags, const char *src, int srclen )
 /* return -1 on dst buffer overflow, -2 on invalid input char */
 int wine_utf8_mbstowcs( int flags, const char *src, int srclen, WCHAR *dst, int dstlen )
 {
-    int len;
     unsigned int res;
     const char *srcend = src + srclen;
     WCHAR *dstend = dst + dstlen;
 
+    if (flags & MB_COMPOSITE) return utf8_mbstowcs_compose( flags, src, srclen, dst, dstlen );
+
     if (!dstlen) return get_length_mbs_utf8( flags, src, srclen );
 
     while ((dst < dstend) && (src < srcend))
@@ -214,38 +319,18 @@ int wine_utf8_mbstowcs( int flags, const char *src, int srclen, WCHAR *dst, int
             *dst++ = ch;
             continue;
         }
-        len = utf8_length[ch-0x80];
-        if (src + len > srcend) goto bad;
-        res = ch & utf8_mask[len];
-
-        switch(len)
+        if ((res = decode_utf8_char( ch, &src, srcend )) <= 0xffff)
         {
-        case 3:
-            if ((ch = *src ^ 0x80) >= 0x40) goto bad;
-            res = (res << 6) | ch;
-            src++;
-        case 2:
-            if ((ch = *src ^ 0x80) >= 0x40) goto bad;
-            res = (res << 6) | ch;
-            src++;
-        case 1:
-            if ((ch = *src ^ 0x80) >= 0x40) goto bad;
-            res = (res << 6) | ch;
-            src++;
-            if (res < utf8_minval[len]) goto bad;
-            if (res > 0x10ffff) goto bad;
-            if (res <= 0xffff) *dst++ = res;
-            else /* we need surrogates */
-            {
-                if (dst == dstend - 1) return -1;  /* overflow */
-                res -= 0x10000;
-                *dst++ = 0xd800 | (res >> 10);
-                *dst++ = 0xdc00 | (res & 0x3ff);
-            }
-            continue;
+            *dst++ = res;
         }
-    bad:
-        if (flags & MB_ERR_INVALID_CHARS) return -2;  /* bad char */
+        else if (res <= 0x10ffff)  /* we need surrogates */
+        {
+            if (dst == dstend - 1) return -1;  /* overflow */
+            res -= 0x10000;
+            *dst++ = 0xd800 | (res >> 10);
+            *dst++ = 0xdc00 | (res & 0x3ff);
+        }
+        else if (flags & MB_ERR_INVALID_CHARS) return -2;  /* bad char */
         /* otherwise ignore it */
     }
     if (src < srcend) return -1;  /* overflow */
