@@ -171,7 +171,7 @@ void WCMD_copy (void) {
   if (outpath[strlenW(outpath) - 1] == '\\')
       outpath[strlenW(outpath) - 1] = '\0';
   attribs = GetFileAttributes(outpath);
-  if (attribs & FILE_ATTRIBUTE_DIRECTORY) {
+  if (attribs != INVALID_FILE_ATTRIBUTES && (attribs & FILE_ATTRIBUTE_DIRECTORY)) {
     strcatW (outpath, slashW);
     copyToDir = TRUE;
   }
@@ -998,13 +998,15 @@ void WCMD_part_execute(CMD_LIST **cmdList, WCHAR *firstcmd, WCHAR *variable,
       /* execute all appropriate commands */
       curPosition = *cmdList;
 
-      WINE_TRACE("Processing cmdList(%p) - &(%d) bd(%d / %d)\n",
+      WINE_TRACE("Processing cmdList(%p) - delim(%d) bd(%d / %d)\n",
                  *cmdList,
-                 (*cmdList)->isAmphersand,
+                 (*cmdList)->prevDelim,
                  (*cmdList)->bracketDepth, myDepth);
 
-      /* Execute any appended to the statement with &&'s */
-      if ((*cmdList)->isAmphersand) {
+      /* Execute any statements appended to the line */
+      /* FIXME: Only if previous call worked for && or failed for || */
+      if ((*cmdList)->prevDelim == CMD_ONFAILURE ||
+          (*cmdList)->prevDelim != CMD_ONSUCCESS) {
         if (processThese) {
           WCMD_execute ((*cmdList)->command, (*cmdList)->redirects, variable,
                         value, cmdList);
@@ -1525,7 +1527,7 @@ void WCMD_rename (void) {
        part otherwise use supplied name. This supports:
           ren *.fred *.jim
           ren jim.* fred.* etc
-       However, windows has a more complex algorithum supporting eg
+       However, windows has a more complex algorithm supporting eg
           ?'s and *'s mid name                                         */
     dotSrc = strchrW(fd.cFileName, '.');
 
@@ -1863,6 +1865,9 @@ void WCMD_setshow_default (WCHAR *command) {
       return;
     } else {
 
+      /* Save away the actual new directory, to store as current location */
+      GetCurrentDirectoryW (sizeof(string)/sizeof(WCHAR), string);
+
       /* Restore old directory if drive letter would change, and
            CD x:\directory /D (or pushd c:\directory) not supplied */
       if ((strstrW(quals, parmD) == NULL) &&
@@ -2278,6 +2283,7 @@ void WCMD_more (WCHAR *command) {
     HANDLE hConIn = CreateFile(conInW, GENERIC_READ | GENERIC_WRITE,
                          FILE_SHARE_READ, NULL, OPEN_EXISTING,
                          FILE_ATTRIBUTE_NORMAL, 0);
+    WINE_TRACE("No parms - working probably in pipe mode\n");
     SetStdHandle(STD_INPUT_HANDLE, hConIn);
 
     /* Warning: No easy way of ending the stream (ctrl+z on windows) so
@@ -2302,6 +2308,7 @@ void WCMD_more (WCHAR *command) {
     BOOL needsPause = FALSE;
 
     /* Loop through all args */
+    WINE_TRACE("Parms supplied - working through each file\n");
     WCMD_enter_paged_mode(moreStrPage);
 
     while (argN) {
@@ -2598,11 +2605,11 @@ void WCMD_assoc (WCHAR *command, BOOL assoc) {
                 WCMD_output_asis(keyValue);
               }
               WCMD_output_asis(newline);
+              RegCloseKey(readKey);
             }
           }
         }
       }
-      RegCloseKey(readKey);
 
     } else {
 
