@@ -64,6 +64,8 @@ struct icon
     HWND           tooltip;  /* Icon tooltip */
     UINT           id;       /* the unique id given by the app */
     UINT           callback_message;
+    BOOL           hidden;   /* icon display state */
+    WCHAR          tiptext[128]; /* Tooltip text. If empty => tooltip disabled */
 };
 
 static struct tray tray;
@@ -75,6 +77,317 @@ static BOOL hide_systray;
 /* space around icon (forces icon to center of KDE systray area) */
 #define ICON_BORDER  4
 
+
+/* Retrieves icon record by owner window and ID */
+static struct icon *get_icon(HWND owner, UINT id)
+{
+    struct icon *this;
+
+    /* search for the icon */
+    LIST_FOR_EACH_ENTRY( this, &tray.icons, struct icon, entry )
+        if ((this->id == id) && (this->owner == owner)) return this;
+
+    return NULL;
+}
+
+/* Creates tooltip window for icon. */
+static void create_tooltip(struct icon *icon)
+{
+    TTTOOLINFOW ti;
+    static BOOL tooltips_initialized = FALSE;
+
+    /* Register tooltip classes if this is the first icon */
+    if (!tooltips_initialized)
+    {
+        INITCOMMONCONTROLSEX init_tooltip;
+
+        init_tooltip.dwSize = sizeof(INITCOMMONCONTROLSEX);
+        init_tooltip.dwICC = ICC_TAB_CLASSES;
+
+        InitCommonControlsEx(&init_tooltip);
+        tooltips_initialized = TRUE;
+    }
+
+    icon->tooltip = CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, NULL,
+                                   WS_POPUP | TTS_ALWAYSTIP,
+                                   CW_USEDEFAULT, CW_USEDEFAULT,
+                                   CW_USEDEFAULT, CW_USEDEFAULT,
+                                   icon->window, NULL, NULL, NULL);
+
+    ZeroMemory(&ti, sizeof(ti));
+    ti.cbSize = sizeof(TTTOOLINFOW);
+    ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
+    ti.hwnd = icon->window;
+    ti.uId = (UINT_PTR)icon->window;
+    ti.lpszText = icon->tiptext;
+    SendMessageW(icon->tooltip, TTM_ADDTOOLW, 0, (LPARAM)&ti);
+}
+
+/* Synchronize tooltip text with tooltip window */
+static void update_tooltip_text(struct icon *icon)
+{
+    TTTOOLINFOW ti;
+
+    ZeroMemory(&ti, sizeof(ti));
+    ti.cbSize = sizeof(TTTOOLINFOW);
+    ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
+    ti.hwnd = icon->window;
+    ti.uId = (UINT_PTR)icon->window;
+    ti.lpszText = icon->tiptext;
+
+    SendMessageW(icon->tooltip, TTM_UPDATETIPTEXTW, 0, (LPARAM)&ti);
+}
+
+/*
+ * Sets visibility status of tray icon. Creates/deletes icon window.
+ * Does not create/delete icon record.
+ *
+ * The purpose is similar to ShowWindow function.
+ */
+static BOOL display_icon(struct icon *icon, BOOL hide)
+{
+    HMODULE x11drv = GetModuleHandleA("winex11.drv");
+    RECT rect;
+    static const WCHAR adaptor_windowname[] = /* Wine System Tray Adaptor */ {'W','i','n','e',' ','S','y','s','t','e','m',' ','T','r','a','y',' ','A','d','a','p','t','o','r',0};
+
+    WINE_TRACE("id=0x%x, hwnd=%p, hide=%d\n", icon->id, icon->owner, hide);
+
+    /* not a startup case and nothing to do */
+    if (icon->window && !icon->hidden == !hide)
+        return TRUE;
+
+    icon->hidden = hide;
+    if (hide)
+    {
+        /* At startup icon->hidden == FALSE and icon->window == NULL */
+        if (icon->window)
+        {
+            DestroyWindow(icon->window);
+            DestroyWindow(icon->tooltip);
+        }
+        return TRUE;
+    }
+
+    rect.left = 0;
+    rect.top = 0;
+    rect.right = GetSystemMetrics(SM_CXSMICON) + ICON_BORDER;
+    rect.bottom = GetSystemMetrics(SM_CYSMICON) + ICON_BORDER;
+    AdjustWindowRect(&rect, WS_CLIPSIBLINGS | WS_CAPTION, FALSE);
+
+    /* create the adaptor window */
+    icon->window = CreateWindowEx(0, adaptor_classname,
+                                  adaptor_windowname,
+                                  WS_CLIPSIBLINGS | WS_CAPTION,
+                                  CW_USEDEFAULT, CW_USEDEFAULT,
+                                  rect.right - rect.left,
+                                  rect.bottom - rect.top,
+                                  NULL, NULL, NULL, icon);
+    if (x11drv)
+    {
+        void (*make_systray_window)(HWND) = (void *)GetProcAddress(x11drv, "wine_make_systray_window");
+        if (make_systray_window) make_systray_window(icon->window);
+    }
+
+    if (!hide_systray)
+        ShowWindow(icon->window, SW_SHOWNA);
+
+    create_tooltip(icon);
+
+    return TRUE;
+}
+
+/* Modifies an existing icon record */
+static BOOL modify_icon(NOTIFYICONDATAW *nid)
+{
+    struct icon    *icon;
+
+    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
+
+    /* demarshal the request from the NID */
+    icon = get_icon(nid->hWnd, nid->uID);
+    if (!icon)
+    {
+        WINE_WARN("Invalid icon ID (0x%x) for HWND %p\n", nid->uID, nid->hWnd);
+        return FALSE;
+    }
+
+    if ((nid->uFlags & NIF_STATE) && (nid->dwStateMask & NIS_HIDDEN))
+        display_icon(icon, !!(nid->dwState & NIS_HIDDEN));
+
+    /* startup case*/
+    if ((!icon->window) && (!icon->hidden))
+        display_icon(icon, FALSE);
+
+    if (nid->uFlags & NIF_ICON)
+    {
+        if (icon->image) DestroyIcon(icon->image);
+        icon->image = CopyIcon(nid->hIcon);
+
+        if (!icon->hidden)
+            RedrawWindow(icon->window, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW);
+    }
+
+    if (nid->uFlags & NIF_MESSAGE)
+    {
+        icon->callback_message = nid->uCallbackMessage;
+    }
+    if (nid->uFlags & NIF_TIP)
+    {
+        lstrcpynW(icon->tiptext, nid->szTip, sizeof(icon->tiptext)/sizeof(WCHAR));
+        if (!icon->hidden)
+            update_tooltip_text(icon);
+    }
+    if (nid->uFlags & NIF_INFO && nid->cbSize >= NOTIFYICONDATAA_V2_SIZE)
+    {
+        WINE_FIXME("balloon tip title %s, message %s\n", wine_dbgstr_w(nid->szInfoTitle), wine_dbgstr_w(nid->szInfo));
+    }
+    return TRUE;
+}
+
+/* Adds a new icon record to the list */
+static BOOL add_icon(NOTIFYICONDATAW *nid)
+{
+    struct icon  *icon;
+
+    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
+
+    if ((icon = get_icon(nid->hWnd, nid->uID)))
+    {
+        WINE_WARN("duplicate tray icon add, buggy app?\n");
+        return FALSE;
+    }
+
+    if (!(icon = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*icon))))
+    {
+        WINE_ERR("out of memory\n");
+        return FALSE;
+    }
+
+    ZeroMemory(icon, sizeof(struct icon));
+    icon->id     = nid->uID;
+    icon->owner  = nid->hWnd;
+
+    list_add_tail(&tray.icons, &icon->entry);
+
+    /*
+     * Both icon->window and icon->hidden are zero. modify_icon function
+     * will treat this case as a startup, i.e. icon window will be created if
+     * NIS_HIDDEN flag is not set.
+     */
+
+    return modify_icon(nid);
+}
+
+/* Deletes tray icon window and icon record */
+static BOOL delete_icon_directly(struct icon *icon)
+{
+    display_icon(icon, TRUE);
+    list_remove(&icon->entry);
+    DestroyIcon(icon->image);
+    HeapFree(GetProcessHeap(), 0, icon);
+    return TRUE;
+}
+
+/* Deletes tray icon window and icon structure */
+static BOOL delete_icon(const NOTIFYICONDATAW *nid)
+{
+    struct icon *icon = get_icon(nid->hWnd, nid->uID);
+
+    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
+
+    if (!icon)
+    {
+        WINE_WARN("invalid tray icon ID specified: %u\n", nid->uID);
+        return FALSE;
+    }
+
+    return delete_icon_directly(icon);
+}
+
+static BOOL handle_incoming(HWND hwndSource, COPYDATASTRUCT *cds)
+{
+    NOTIFYICONDATAW nid;
+    DWORD cbSize;
+    BOOL ret = FALSE;
+
+    if (cds->cbData < NOTIFYICONDATAW_V1_SIZE) return FALSE;
+    cbSize = ((PNOTIFYICONDATA)cds->lpData)->cbSize;
+    if (cbSize < NOTIFYICONDATAW_V1_SIZE) return FALSE;
+
+    ZeroMemory(&nid, sizeof(nid));
+    memcpy(&nid, cds->lpData, min(sizeof(nid), cbSize));
+
+    /* FIXME: if statement only needed because we don't support interprocess
+     * icon handles */
+    if ((nid.uFlags & NIF_ICON) && (cds->cbData >= nid.cbSize + 2 * sizeof(BITMAP)))
+    {
+        LONG cbMaskBits;
+        LONG cbColourBits;
+        BITMAP bmMask;
+        BITMAP bmColour;
+        const char *buffer = cds->lpData;
+
+        buffer += nid.cbSize;
+
+        memcpy(&bmMask, buffer, sizeof(bmMask));
+        buffer += sizeof(bmMask);
+        memcpy(&bmColour, buffer, sizeof(bmColour));
+        buffer += sizeof(bmColour);
+
+        cbMaskBits = (bmMask.bmPlanes * bmMask.bmWidth * bmMask.bmHeight * bmMask.bmBitsPixel) / 8;
+        cbColourBits = (bmColour.bmPlanes * bmColour.bmWidth * bmColour.bmHeight * bmColour.bmBitsPixel) / 8;
+
+        if (cds->cbData < nid.cbSize + 2 * sizeof(BITMAP) + cbMaskBits + cbColourBits)
+        {
+            WINE_ERR("buffer underflow\n");
+            return FALSE;
+        }
+
+        /* sanity check */
+        if ((bmColour.bmWidth != bmMask.bmWidth) || (bmColour.bmHeight != bmMask.bmHeight))
+        {
+            WINE_ERR("colour and mask bitmaps aren't consistent\n");
+            return FALSE;
+        }
+
+        nid.hIcon = CreateIcon(NULL, bmColour.bmWidth, bmColour.bmHeight,
+                               bmColour.bmPlanes, bmColour.bmBitsPixel,
+                               buffer, buffer + cbMaskBits);
+    }
+
+    switch (cds->dwData)
+    {
+    case NIM_ADD:
+        ret = add_icon(&nid);
+        break;
+    case NIM_DELETE:
+        ret = delete_icon(&nid);
+        break;
+    case NIM_MODIFY:
+        ret = modify_icon(&nid);
+        break;
+    default:
+        WINE_FIXME("unhandled tray message: %ld\n", cds->dwData);
+        break;
+    }
+
+    /* FIXME: if statement only needed because we don't support interprocess
+     * icon handles */
+    if (nid.uFlags & NIF_ICON)
+        DestroyIcon(nid.hIcon);
+
+    return ret;
+}
+
+static LRESULT WINAPI listener_wndproc(HWND window, UINT msg,
+                                       WPARAM wparam, LPARAM lparam)
+{
+    if (msg == WM_COPYDATA)
+        return handle_incoming((HWND)wparam, (COPYDATASTRUCT *)lparam);
+
+    return DefWindowProc(window, msg, wparam, lparam);
+}
+
 static LRESULT WINAPI adaptor_wndproc(HWND window, UINT msg,
                                       WPARAM wparam, LPARAM lparam)
 {
@@ -126,20 +439,16 @@ static LRESULT WINAPI adaptor_wndproc(HWND window, UINT msg,
             /* notify the owner hwnd of the message */
             WINE_TRACE("relaying 0x%x\n", msg);
             ret = PostMessage(icon->owner, icon->callback_message, (WPARAM) icon->id, (LPARAM) msg);
-            if (!ret && (GetLastError() == ERROR_INVALID_HANDLE))
+            if (!ret && (GetLastError() == ERROR_INVALID_WINDOW_HANDLE))
             {
                 WINE_WARN("application window was destroyed without removing "
                           "notification icon, removing automatically\n");
-                DestroyWindow(window);
+                delete_icon_directly(icon);
             }
             break;
 
         case WM_NCDESTROY:
             SetWindowLongPtr(window, GWLP_USERDATA, 0);
-
-            list_remove(&icon->entry);
-            DestroyIcon(icon->image);
-            HeapFree(GetProcessHeap(), 0, icon);
             break;
 
         default:
@@ -150,236 +459,6 @@ static LRESULT WINAPI adaptor_wndproc(HWND window, UINT msg,
 }
 
 
-/* listener code */
-
-static struct icon *get_icon(HWND owner, UINT id)
-{
-    struct icon *this;
-
-    /* search for the icon */
-    LIST_FOR_EACH_ENTRY( this, &tray.icons, struct icon, entry )
-        if ((this->id == id) && (this->owner == owner)) return this;
-
-    return NULL;
-}
-
-static void set_tooltip(struct icon *icon, WCHAR *szTip, BOOL modify)
-{
-    TTTOOLINFOW ti;
-
-    ti.cbSize = sizeof(TTTOOLINFOW);
-    ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
-    ti.hwnd = icon->window;
-    ti.hinst = 0;
-    ti.uId = (UINT_PTR)icon->window;
-    ti.lpszText = szTip;
-    ti.lParam = 0;
-    ti.lpReserved = NULL;
-
-    if (modify)
-        SendMessageW(icon->tooltip, TTM_UPDATETIPTEXTW, 0, (LPARAM)&ti);
-    else
-        SendMessageW(icon->tooltip, TTM_ADDTOOLW, 0, (LPARAM)&ti);
-}
-
-static void modify_icon(NOTIFYICONDATAW *nid, BOOL modify_tooltip)
-{
-    struct icon    *icon;
-
-    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
-
-    /* demarshal the request from the NID */
-    icon = get_icon(nid->hWnd, nid->uID);
-    if (!icon)
-    {
-        WINE_WARN("Invalid icon ID (0x%x) for HWND %p\n", nid->uID, nid->hWnd);
-        return;
-    }
-
-    if (nid->uFlags & NIF_ICON)
-    {
-        if (icon->image) DestroyIcon(icon->image);
-        icon->image = CopyIcon(nid->hIcon);
-
-        RedrawWindow(icon->window, NULL, NULL, RDW_ERASE | RDW_INVALIDATE | RDW_UPDATENOW);
-    }
-
-    if (nid->uFlags & NIF_MESSAGE)
-    {
-        icon->callback_message = nid->uCallbackMessage;
-    }
-    if (nid->uFlags & NIF_TIP)
-    {
-        set_tooltip(icon, nid->szTip, modify_tooltip);
-    }
-}
-
-static void add_icon(NOTIFYICONDATAW *nid)
-{
-    RECT rect;
-    struct icon  *icon;
-    static const WCHAR adaptor_windowname[] = /* Wine System Tray Adaptor */ {'W','i','n','e',' ','S','y','s','t','e','m',' ','T','r','a','y',' ','A','d','a','p','t','o','r',0};
-    static BOOL tooltps_initialized = FALSE;
-
-    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
-
-    if ((icon = get_icon(nid->hWnd, nid->uID)))
-    {
-        WINE_WARN("duplicate tray icon add, buggy app?\n");
-        return;
-    }
-
-    if (!(icon = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*icon))))
-    {
-        WINE_ERR("out of memory\n");
-        return;
-    }
-
-    icon->id    = nid->uID;
-    icon->owner = nid->hWnd;
-    icon->image = NULL;
-
-    rect.left = 0;
-    rect.top = 0;
-    rect.right = GetSystemMetrics(SM_CXSMICON) + ICON_BORDER;
-    rect.bottom = GetSystemMetrics(SM_CYSMICON) + ICON_BORDER;
-    AdjustWindowRect(&rect, WS_CLIPSIBLINGS | WS_CAPTION, FALSE);
-
-    /* create the adaptor window */
-    icon->window = CreateWindowEx(WS_EX_TRAYWINDOW, adaptor_classname,
-                                  adaptor_windowname,
-                                  WS_CLIPSIBLINGS | WS_CAPTION,
-                                  CW_USEDEFAULT, CW_USEDEFAULT,
-                                  rect.right - rect.left,
-                                  rect.bottom - rect.top,
-                                  NULL, NULL, NULL, icon);
-
-    if (!hide_systray)
-        ShowWindow(icon->window, SW_SHOWNA);
-
-    /* create icon tooltip */
-
-    /* Register tooltip classes if this is the first icon */
-    if (!tooltps_initialized)
-    {
-        INITCOMMONCONTROLSEX init_tooltip;
-
-        init_tooltip.dwSize = sizeof(INITCOMMONCONTROLSEX);
-        init_tooltip.dwICC = ICC_TAB_CLASSES;
-
-        InitCommonControlsEx(&init_tooltip);
-        tooltps_initialized = TRUE;
-    }
-
-    icon->tooltip = CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, NULL,
-                                   WS_POPUP | TTS_ALWAYSTIP,
-                                   CW_USEDEFAULT, CW_USEDEFAULT,
-                                   CW_USEDEFAULT, CW_USEDEFAULT,
-                                   icon->window, NULL, NULL, NULL);
-
-    list_add_tail(&tray.icons, &icon->entry);
-
-    modify_icon(nid, FALSE);
-}
-
-static void delete_icon(const NOTIFYICONDATAW *nid)
-{
-    struct icon *icon = get_icon(nid->hWnd, nid->uID);
-
-    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
-   
-    if (!icon)
-    {
-        WINE_ERR("invalid tray icon ID specified: %u\n", nid->uID);
-        return;
-    }
-
-    DestroyWindow(icon->tooltip);
-    DestroyWindow(icon->window);
-}
-
-static void handle_incoming(HWND hwndSource, COPYDATASTRUCT *cds)
-{
-    NOTIFYICONDATAW nid;
-    DWORD cbSize;
-
-    if (cds->cbData < NOTIFYICONDATAW_V1_SIZE) return;
-    cbSize = ((PNOTIFYICONDATA)cds->lpData)->cbSize;
-    if (cbSize < NOTIFYICONDATAW_V1_SIZE) return;
-
-    ZeroMemory(&nid, sizeof(nid));
-    memcpy(&nid, cds->lpData, min(sizeof(nid), cbSize));
-
-    /* FIXME: if statement only needed because we don't support interprocess
-     * icon handles */
-    if ((nid.uFlags & NIF_ICON) && (cds->cbData >= nid.cbSize + 2 * sizeof(BITMAP)))
-    {
-        LONG cbMaskBits;
-        LONG cbColourBits;
-        BITMAP bmMask;
-        BITMAP bmColour;
-        const char *buffer = cds->lpData;
-
-        buffer += nid.cbSize;
-
-        memcpy(&bmMask, buffer, sizeof(bmMask));
-        buffer += sizeof(bmMask);
-        memcpy(&bmColour, buffer, sizeof(bmColour));
-        buffer += sizeof(bmColour);
-
-        cbMaskBits = (bmMask.bmPlanes * bmMask.bmWidth * bmMask.bmHeight * bmMask.bmBitsPixel) / 8;
-        cbColourBits = (bmColour.bmPlanes * bmColour.bmWidth * bmColour.bmHeight * bmColour.bmBitsPixel) / 8;
-
-        if (cds->cbData < nid.cbSize + 2 * sizeof(BITMAP) + cbMaskBits + cbColourBits)
-        {
-            WINE_ERR("buffer underflow\n");
-            return;
-        }
-
-        /* sanity check */
-        if ((bmColour.bmWidth != bmMask.bmWidth) || (bmColour.bmHeight != bmMask.bmHeight))
-        {
-            WINE_ERR("colour and mask bitmaps aren't consistent\n");
-            return;
-        }
-
-        nid.hIcon = CreateIcon(NULL, bmColour.bmWidth, bmColour.bmHeight,
-                               bmColour.bmPlanes, bmColour.bmBitsPixel,
-                               buffer, buffer + cbMaskBits);
-    }
-
-    switch (cds->dwData)
-    {
-    case NIM_ADD:
-        add_icon(&nid);
-        break;
-    case NIM_DELETE:
-        delete_icon(&nid);
-        break;
-    case NIM_MODIFY:
-        modify_icon(&nid, TRUE);
-        break;
-    default:
-        WINE_FIXME("unhandled tray message: %ld\n", cds->dwData);
-        break;
-    }
-
-    /* FIXME: if statement only needed because we don't support interprocess
-     * icon handles */
-    if (nid.uFlags & NIF_ICON)
-        DestroyIcon(nid.hIcon);
-}
-
-static LRESULT WINAPI listener_wndproc(HWND window, UINT msg,
-                                       WPARAM wparam, LPARAM lparam)
-{
-    if (msg == WM_COPYDATA)
-        handle_incoming((HWND)wparam, (COPYDATASTRUCT *)lparam);
-
-    return DefWindowProc(window, msg, wparam, lparam);
-}
-
-
 static BOOL is_systray_hidden(void)
 {
     const WCHAR show_systray_keyname[] = {'S','o','f','t','w','a','r','e','\\','W','i','n','e','\\',
@@ -402,7 +481,7 @@ static BOOL is_systray_hidden(void)
     return ret;
 }
 
-/* this function creates the the listener window */
+/* this function creates the listener window */
 void initialize_systray(void)
 {
     WNDCLASSEX class;
