@@ -4,6 +4,7 @@
  *  Copyright 1998,99 Marcel Baur <mbaur@g26.ethz.ch>
  *  Copyright 2002 Sylvain Petreolle <spetreolle@yahoo.fr>
  *  Copyright 2002 Andriy Palamarchuk
+ *  Copyright 2007 Rolf Kalbermatter
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,7 +33,7 @@
 #include "dialog.h"
 
 #define SPACES_IN_TAB 8
-#define PRINT_LEN_MAX 120
+#define PRINT_LEN_MAX 500
 
 static const WCHAR helpfileW[] = { 'n','o','t','e','p','a','d','.','h','l','p',0 };
 
@@ -187,10 +188,8 @@ BOOL DoCloseFile(void)
             case IDNO:      break;
 
             case IDCANCEL:  return(FALSE);
-                            break;
 
             default:        return(FALSE);
-                            break;
         } /* switch */
     } /* if */
 
@@ -218,7 +217,7 @@ void DoOpenFile(LPCWSTR szFileName)
 	OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
     if(hFile == INVALID_HANDLE_VALUE)
     {
-	ShowLastError();
+	AlertFileNotFound(szFileName);
 	return;
     }
 
@@ -266,9 +265,7 @@ void DoOpenFile(LPCWSTR szFileName)
     SendMessage(Globals.hEdit, EM_EMPTYUNDOBUFFER, 0, 0);
     SetFocus(Globals.hEdit);
     
-    /*  If the file starts with .LOG, add a time/date at the end and set cursor after
-     *  See http://support.microsoft.com/?kbid=260563
-     */
+    /*  If the file starts with .LOG, add a time/date at the end and set cursor after */
     if (GetWindowTextW(Globals.hEdit, log, sizeof(log)/sizeof(log[0])) && !lstrcmp(log, dotlog))
     {
 	static const WCHAR lfW[] = { '\r','\n',0 };
@@ -319,12 +316,8 @@ VOID DIALOG_FileOpen(VOID)
     openfilename.lpstrDefExt       = szDefaultExt;
 
 
-    if (GetOpenFileName(&openfilename)) {
-        if (FileExists(openfilename.lpstrFile))
-            DoOpenFile(openfilename.lpstrFile);
-        else
-            AlertFileNotFound(openfilename.lpstrFile);
-    }
+    if (GetOpenFileName(&openfilename))
+        DoOpenFile(openfilename.lpstrFile);
 }
 
 
@@ -367,40 +360,139 @@ VOID DIALOG_FileSaveAs(VOID)
     }
 }
 
+typedef struct {
+    LPWSTR mptr;
+    LPWSTR mend;
+    LPWSTR lptr;
+    DWORD len;
+} TEXTINFO, *LPTEXTINFO;
+
+static int notepad_print_header(HDC hdc, RECT *rc, BOOL dopage, BOOL header, int page, LPWSTR text)
+{
+    SIZE szMetric;
+
+    if (*text)
+    {
+        /* Write the header or footer */
+        GetTextExtentPoint32(hdc, text, lstrlen(text), &szMetric);
+        if (dopage)
+            ExtTextOut(hdc, (rc->left + rc->right - szMetric.cx) / 2,
+                       header ? rc->top : rc->bottom - szMetric.cy,
+                       ETO_CLIPPED, rc, text, lstrlen(text), NULL);
+        return 1;
+    }
+    return 0;
+}
+
+static BOOL notepad_print_page(HDC hdc, RECT *rc, BOOL dopage, int page, LPTEXTINFO tInfo)
+{
+    int b, y;
+    TEXTMETRIC tm;
+    SIZE szMetrics;
+
+    if (dopage)
+    {
+        if (StartPage(hdc) <= 0)
+        {
+            static const WCHAR failedW[] = { 'S','t','a','r','t','P','a','g','e',' ','f','a','i','l','e','d',0 };
+            static const WCHAR errorW[] = { 'P','r','i','n','t',' ','E','r','r','o','r',0 };
+            MessageBox(Globals.hMainWnd, failedW, errorW, MB_ICONEXCLAMATION);
+            return FALSE;
+        }
+    }
+
+    GetTextMetrics(hdc, &tm);
+    y = rc->top + notepad_print_header(hdc, rc, dopage, TRUE, page, Globals.szFileName) * tm.tmHeight;
+    b = rc->bottom - 2 * notepad_print_header(hdc, rc, FALSE, FALSE, page, Globals.szFooter) * tm.tmHeight;
+
+    do {
+        INT m, n;
+
+        if (!tInfo->len)
+        {
+            /* find the end of the line */
+            while (tInfo->mptr < tInfo->mend && *tInfo->mptr != '\n' && *tInfo->mptr != '\r')
+            {
+                if (*tInfo->mptr == '\t')
+                {
+                    /* replace tabs with spaces */
+                    for (m = 0; m < SPACES_IN_TAB; m++)
+                    {
+                        if (tInfo->len < PRINT_LEN_MAX)
+                            tInfo->lptr[tInfo->len++] = ' ';
+                        else if (Globals.bWrapLongLines)
+                            break;
+                    }
+                }
+                else if (tInfo->len < PRINT_LEN_MAX)
+                    tInfo->lptr[tInfo->len++] = *tInfo->mptr;
+
+                if (tInfo->len >= PRINT_LEN_MAX && Globals.bWrapLongLines)
+                     break;
+
+                tInfo->mptr++;
+            }
+        }
+
+        /* Find out how much we should print if line wrapping is enabled */
+        if (Globals.bWrapLongLines)
+        {
+            GetTextExtentExPoint(hdc, tInfo->lptr, tInfo->len, rc->right - rc->left, &n, NULL, &szMetrics);
+            if (n < tInfo->len && tInfo->lptr[n] != ' ')
+            {
+                m = n;
+                /* Don't wrap words unless it's a single word over the entire line */
+                while (m  && tInfo->lptr[m] != ' ') m--;
+                if (m > 0) n = m + 1;
+            }
+        }
+        else
+            n = tInfo->len;
+
+        if (dopage)
+            ExtTextOut(hdc, rc->left, y, ETO_CLIPPED, rc, tInfo->lptr, n, NULL);
+
+        tInfo->len -= n;
+
+        if (tInfo->len)
+        {
+            memcpy(tInfo->lptr, tInfo->lptr + n, tInfo->len * sizeof(WCHAR));
+            y += tm.tmHeight + tm.tmExternalLeading;
+        }
+        else
+        {
+            /* find the next line */
+            while (tInfo->mptr < tInfo->mend && y < b && (*tInfo->mptr == '\n' || *tInfo->mptr == '\r'))
+            {
+                if (*tInfo->mptr == '\n')
+                    y += tm.tmHeight + tm.tmExternalLeading;
+                tInfo->mptr++;
+            }
+        }
+    } while (tInfo->mptr < tInfo->mend && y < b);
+
+    notepad_print_header(hdc, rc, dopage, FALSE, page, Globals.szFooter);
+    if (dopage)
+    {
+        EndPage(hdc);
+    }
+    return TRUE;
+}
+
 VOID DIALOG_FilePrint(VOID)
 {
     DOCINFO di;
     PRINTDLG printer;
-    SIZE szMetric;
-    int cWidthPels, cHeightPels, border;
-    int xLeft, yTop, pagecount, dopage, copycount;
-    unsigned int i;
-    LOGFONT hdrFont;
-    HFONT font, old_font=0;
+    int page, dopage, copy;
+    LOGFONT lfFont;
+    HFONT hTextFont, old_font = 0;
     DWORD size;
+    BOOL ret = FALSE;
+    RECT rc;
     LPWSTR pTemp;
+    TEXTINFO tInfo;
     WCHAR cTemp[PRINT_LEN_MAX];
-    static const WCHAR print_fontW[] = { 'C','o','u','r','i','e','r',0 };
-    static const WCHAR letterM[] = { 'M',0 };
 
-    /* Get a small font and print some header info on each page */
-    hdrFont.lfHeight = -35;
-    hdrFont.lfWidth = 0;
-    hdrFont.lfEscapement = 0;
-    hdrFont.lfOrientation = 0;
-    hdrFont.lfWeight = FW_BOLD;
-    hdrFont.lfItalic = 0;
-    hdrFont.lfUnderline = 0;
-    hdrFont.lfStrikeOut = 0;
-    hdrFont.lfCharSet = ANSI_CHARSET;
-    hdrFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
-    hdrFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
-    hdrFont.lfQuality = PROOF_QUALITY;
-    hdrFont.lfPitchAndFamily = VARIABLE_PITCH | FF_ROMAN;
-    lstrcpy(hdrFont.lfFaceName, print_fontW);
-    
-    font = CreateFontIndirect(&hdrFont);
-    
     /* Get Current Settings */
     ZeroMemory(&printer, sizeof(printer));
     printer.lStructSize           = sizeof(printer);
@@ -408,7 +500,7 @@ VOID DIALOG_FilePrint(VOID)
     printer.hDevMode              = Globals.hDevMode;
     printer.hDevNames             = Globals.hDevNames;
     printer.hInstance             = Globals.hInstance;
-    
+
     /* Set some default flags */
     printer.Flags                 = PD_RETURNDC | PD_NOSELECTION;
     printer.nFromPage             = 0;
@@ -424,7 +516,7 @@ VOID DIALOG_FilePrint(VOID)
     Globals.hDevMode = printer.hDevMode;
     Globals.hDevNames = printer.hDevNames;
 
-    assert(printer.hDC != 0);
+    SetMapMode(printer.hDC, MM_TEXT);
 
     /* initialize DOCINFO */
     di.cbSize = sizeof(DOCINFO);
@@ -433,94 +525,72 @@ VOID DIALOG_FilePrint(VOID)
     di.lpszDatatype = NULL;
     di.fwType = 0; 
 
-    if (StartDoc(printer.hDC, &di) <= 0) return;
-    
-    /* Get the page dimensions in pixels. */
-    cWidthPels = GetDeviceCaps(printer.hDC, HORZRES);
-    cHeightPels = GetDeviceCaps(printer.hDC, VERTRES);
-
     /* Get the file text */
     size = GetWindowTextLength(Globals.hEdit) + 1;
     pTemp = HeapAlloc(GetProcessHeap(), 0, size * sizeof(WCHAR));
     if (!pTemp)
     {
-        ShowLastError();
-        return;
+       DeleteDC(printer.hDC);
+       ShowLastError();
+       return;
     }
     size = GetWindowText(Globals.hEdit, pTemp, size);
-    
-    border = 150;
-    old_font = SelectObject(printer.hDC, Globals.hFont);
-    GetTextExtentPoint32(printer.hDC, letterM, 1, &szMetric);
-    for (copycount=1; copycount <= printer.nCopies; copycount++) {
-        i = 0;
-        pagecount = 1;
-        do {
-            if (printer.Flags & PD_PAGENUMS) {
-                /* a specific range of pages is selected, so
-                 * skip pages that are not to be printed 
-                 */
-                if (pagecount > printer.nToPage)
-                    break;
-                else if (pagecount >= printer.nFromPage)
-                    dopage = 1;
-                else
-                    dopage = 0;
-            }
-            else
-                dopage = 1;
-                
-            if (dopage) {
-                if (StartPage(printer.hDC) <= 0) {
-                    static const WCHAR failedW[] = { 'S','t','a','r','t','P','a','g','e',' ','f','a','i','l','e','d',0 };
-                    static const WCHAR errorW[] = { 'P','r','i','n','t',' ','E','r','r','o','r',0 };
-                    MessageBox(Globals.hMainWnd, failedW, errorW, MB_ICONEXCLAMATION);
-                    return;
-                }
-                /* Write a rectangle and header at the top of each page */
-                SelectObject(printer.hDC, font);
-                Rectangle(printer.hDC, border, border, cWidthPels-border, border+szMetric.cy*2);
-                TextOut(printer.hDC, border*2, border+szMetric.cy/2, Globals.szFileTitle, lstrlen(Globals.szFileTitle));
-            }
-            
-            SelectObject(printer.hDC, Globals.hFont);
-            /* The starting point for the main text */
-            xLeft = border;
-            yTop = border+szMetric.cy*4;
-            
-            do {
-                int k=0, m;
-                /* find the end of the line */
-                while (i < size && pTemp[i] != '\n' && pTemp[i] != '\r') {
-                    if (pTemp[i] == '\t') {
-                        /* replace tabs with spaces */
-                        for (m=0; m<SPACES_IN_TAB; m++) {
-                            if (k <  PRINT_LEN_MAX)
-                                cTemp[k++] = ' ';
-                        }
-                    }
-                    else if (k <  PRINT_LEN_MAX)
-                        cTemp[k++] = pTemp[i];
-                    i++;
-                }
-                if (dopage)
-                    TextOut(printer.hDC, xLeft, yTop, cTemp, k);
-                /* find the next line */
-                while (i < size && (pTemp[i] == '\n' || pTemp[i] == '\r')) {
-                    if (pTemp[i] == '\n')
-                        yTop += szMetric.cy;
-                    i++;
-                }
-            } while (i<size && yTop<(cHeightPels-border*2));
-                
-            if (dopage)
-                EndPage(printer.hDC);
-            pagecount++;
-        } while (i<size);
-    }
-    SelectObject(printer.hDC, old_font);
 
-    EndDoc(printer.hDC);
+    if (StartDoc(printer.hDC, &di) > 0)
+    {
+        /* Get the page margins in pixels. */
+        rc.top =    MulDiv(Globals.iMarginTop, GetDeviceCaps(printer.hDC, LOGPIXELSY), 2540) -
+                    GetDeviceCaps(printer.hDC, PHYSICALOFFSETY);
+        rc.bottom = GetDeviceCaps(printer.hDC, PHYSICALHEIGHT) -
+                    MulDiv(Globals.iMarginBottom, GetDeviceCaps(printer.hDC, LOGPIXELSY), 2540);
+        rc.left =   MulDiv(Globals.iMarginLeft, GetDeviceCaps(printer.hDC, LOGPIXELSX), 2540) -
+                    GetDeviceCaps(printer.hDC, PHYSICALOFFSETX);
+        rc.right =  GetDeviceCaps(printer.hDC, PHYSICALWIDTH) -
+                    MulDiv(Globals.iMarginRight, GetDeviceCaps(printer.hDC, LOGPIXELSX), 2540);
+
+        /* Create a font for the printer resolution */
+        lfFont = Globals.lfFont;
+        lfFont.lfHeight = MulDiv(lfFont.lfHeight, GetDeviceCaps(printer.hDC, LOGPIXELSY), get_dpi());
+        /* Make the font a bit lighter */
+        lfFont.lfWeight -= 100;
+        hTextFont = CreateFontIndirect(&lfFont);
+        old_font = SelectObject(printer.hDC, hTextFont);
+
+        for (copy = 1; copy <= printer.nCopies; copy++)
+        {
+            page = 1;
+
+            tInfo.mptr = pTemp;
+            tInfo.mend = pTemp + size;
+            tInfo.lptr = cTemp;
+            tInfo.len = 0;
+
+            do {
+                if (printer.Flags & PD_PAGENUMS)
+                {
+                    /* a specific range of pages is selected, so
+                     * skip pages that are not to be printed
+                     */
+                    if (page > printer.nToPage)
+                        break;
+                    else if (page >= printer.nFromPage)
+                        dopage = 1;
+                    else
+                        dopage = 0;
+                }
+                else
+                    dopage = 1;
+
+                ret = notepad_print_page(printer.hDC, &rc, dopage, page, &tInfo);
+                page++;
+            } while (ret && tInfo.mptr < tInfo.mend);
+
+            if (!ret) break;
+        }
+        EndDoc(printer.hDC);
+        SelectObject(printer.hDC, old_font);
+        DeleteObject(hTextFont);
+    }
     DeleteDC(printer.hDC);
     HeapFree(GetProcessHeap(), 0, pTemp);
 }
@@ -695,29 +765,15 @@ VOID DIALOG_HelpHelp(VOID)
     WinHelp(Globals.hMainWnd, helpfileW, HELP_HELPONHELP, 0);
 }
 
-VOID DIALOG_HelpLicense(VOID)
+VOID DIALOG_HelpAboutNotepad(VOID)
 {
-    TCHAR cap[20], text[1024];
-    LoadString(Globals.hInstance, IDS_LICENSE, text, 1024);
-    LoadString(Globals.hInstance, IDS_LICENSE_CAPTION, cap, 20);
-    MessageBox(Globals.hMainWnd, text, cap, MB_ICONINFORMATION | MB_OK);
-}
-
-VOID DIALOG_HelpNoWarranty(VOID)
-{
-    TCHAR cap[20], text[1024];
-    LoadString(Globals.hInstance, IDS_WARRANTY, text, 1024);
-    LoadString(Globals.hInstance, IDS_WARRANTY_CAPTION, cap, 20);
-    MessageBox(Globals.hMainWnd, text, cap, MB_ICONEXCLAMATION | MB_OK);
-}
-
-VOID DIALOG_HelpAboutWine(VOID)
-{
-    static const WCHAR notepadW[] = { 'N','o','t','e','p','a','d','\n',0 };
+    static const WCHAR notepadW[] = { 'W','i','n','e',' ','N','o','t','e','p','a','d',0 };
     WCHAR szNotepad[MAX_STRING_LEN];
+    HICON icon = LoadImageW( Globals.hInstance, MAKEINTRESOURCE(IDI_NOTEPAD),
+                             IMAGE_ICON, 48, 48, LR_SHARED );
 
     LoadString(Globals.hInstance, STRING_NOTEPAD, szNotepad, SIZEOF(szNotepad));
-    ShellAbout(Globals.hMainWnd, szNotepad, notepadW, 0);
+    ShellAbout(Globals.hMainWnd, szNotepad, notepadW, icon);
 }
 
 
@@ -747,12 +803,13 @@ static INT_PTR WINAPI DIALOG_PAGESETUP_DlgProc(HWND hDlg, UINT msg, WPARAM wPara
         {
         case IDOK:
           /* save user input and close dialog */
-          GetDlgItemText(hDlg, 0x141, Globals.szHeader, SIZEOF(Globals.szHeader));
-          GetDlgItemText(hDlg, 0x143, Globals.szFooter, SIZEOF(Globals.szFooter));
-          GetDlgItemText(hDlg, 0x14A, Globals.szMarginTop, SIZEOF(Globals.szMarginTop));
-          GetDlgItemText(hDlg, 0x150, Globals.szMarginBottom, SIZEOF(Globals.szMarginBottom));
-          GetDlgItemText(hDlg, 0x147, Globals.szMarginLeft, SIZEOF(Globals.szMarginLeft));
-          GetDlgItemText(hDlg, 0x14D, Globals.szMarginRight, SIZEOF(Globals.szMarginRight));
+          GetDlgItemText(hDlg, IDC_PAGESETUP_HEADERVALUE, Globals.szHeader, SIZEOF(Globals.szHeader));
+          GetDlgItemText(hDlg, IDC_PAGESETUP_FOOTERVALUE, Globals.szFooter, SIZEOF(Globals.szFooter));
+
+          Globals.iMarginTop = GetDlgItemInt(hDlg, IDC_PAGESETUP_TOPVALUE, NULL, FALSE) * 100;
+          Globals.iMarginBottom = GetDlgItemInt(hDlg, IDC_PAGESETUP_BOTTOMVALUE, NULL, FALSE) * 100;
+          Globals.iMarginLeft = GetDlgItemInt(hDlg, IDC_PAGESETUP_LEFTVALUE, NULL, FALSE) * 100;
+          Globals.iMarginRight = GetDlgItemInt(hDlg, IDC_PAGESETUP_RIGHTVALUE, NULL, FALSE) * 100;
           EndDialog(hDlg, IDOK);
           return TRUE;
 
@@ -777,12 +834,12 @@ static INT_PTR WINAPI DIALOG_PAGESETUP_DlgProc(HWND hDlg, UINT msg, WPARAM wPara
 
     case WM_INITDIALOG:
        /* fetch last user input prior to display dialog */
-       SetDlgItemText(hDlg, 0x141, Globals.szHeader);
-       SetDlgItemText(hDlg, 0x143, Globals.szFooter);
-       SetDlgItemText(hDlg, 0x14A, Globals.szMarginTop);
-       SetDlgItemText(hDlg, 0x150, Globals.szMarginBottom);
-       SetDlgItemText(hDlg, 0x147, Globals.szMarginLeft);
-       SetDlgItemText(hDlg, 0x14D, Globals.szMarginRight);
+       SetDlgItemText(hDlg, IDC_PAGESETUP_HEADERVALUE, Globals.szHeader);
+       SetDlgItemText(hDlg, IDC_PAGESETUP_FOOTERVALUE, Globals.szFooter);
+       SetDlgItemInt(hDlg, IDC_PAGESETUP_TOPVALUE, Globals.iMarginTop / 100, FALSE);
+       SetDlgItemInt(hDlg, IDC_PAGESETUP_BOTTOMVALUE, Globals.iMarginBottom / 100, FALSE);
+       SetDlgItemInt(hDlg, IDC_PAGESETUP_LEFTVALUE, Globals.iMarginLeft / 100, FALSE);
+       SetDlgItemInt(hDlg, IDC_PAGESETUP_RIGHTVALUE, Globals.iMarginRight / 100, FALSE);
        break;
     }
 
