@@ -258,9 +258,8 @@ static TYPELIB_DATA *InitializeTLData(void)
 {
     TYPELIB_DATA *pTLData;
 
-    pTLData = HeapAlloc(GetProcessHeap(), 0, sizeof(TYPELIB_DATA));
+    pTLData = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(TYPELIB_DATA));
 
-    memset(pTLData, 0, sizeof(TYPELIB_DATA));
     pTLData->idl = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR));
     pTLData->idl[0] = '\0';
 
@@ -896,14 +895,18 @@ static void CreateInterfaceInfo(ITypeInfo *pTypeInfo, int cImplTypes, WCHAR *wsz
         AddToTLDataStrW(pTLData, wszSpace);
 
         ITypeInfo_GetRefTypeOfImplType(pTypeInfo, 0, &hRefType);
-        ITypeInfo_GetRefTypeInfo(pTypeInfo, hRefType, &pRefTypeInfo);
-        ITypeInfo_GetDocumentation(pRefTypeInfo, MEMBERID_NIL, &bstrName,
+        if (SUCCEEDED(ITypeInfo_GetRefTypeInfo(pTypeInfo, hRefType, &pRefTypeInfo)))
+        {
+            ITypeInfo_GetDocumentation(pRefTypeInfo, MEMBERID_NIL, &bstrName,
                 NULL, NULL, NULL);
-        AddToTLDataStrW(pTLData, bstrName);
-        AddToTLDataStrW(pTLData, wszSpace);
+            AddToTLDataStrW(pTLData, bstrName);
+            AddToTLDataStrW(pTLData, wszSpace);
 
-        SysFreeString(bstrName);
-        ITypeInfo_Release(pRefTypeInfo);
+            SysFreeString(bstrName);
+            ITypeInfo_Release(pRefTypeInfo);
+        }
+        else
+            AddToTLDataStrW(pTLData, wszFailed);
     }
     AddToTLDataStrW(pTLData, wszOpenBrackets3);
     AddToTLDataStrW(pTLData, wszNewLine);
