@@ -19,7 +19,7 @@
 /* Wine "bootup" handler application
  *
  * This app handles the various "hooks" windows allows for applications to perform
- * as part of the bootstrap process. Theses are roughly devided into three types.
+ * as part of the bootstrap process. These are roughly divided into three types.
  * Knowledge base articles that explain this are 137367, 179365, 232487 and 232509.
  * Also, 119941 has some info on grpconv.exe
  * The operations performed are (by order of execution):
@@ -29,7 +29,7 @@
  * - PendingRenameOperations (rename operations left in the registry - Win NT+ only)
  *
  * Startup (before the user logs in)
- * - Services (NT, ?semi-synchronous?, not implemented yet)
+ * - Services (NT)
  * - HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\RunServicesOnce (9x, asynch)
  * - HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\RunServices (9x, asynch)
  * 
@@ -61,6 +61,8 @@
 # include <getopt.h>
 #endif
 #include <windows.h>
+#include <wine/svcctl.h>
+#include <wine/unicode.h>
 #include <wine/debug.h>
 
 #define COBJMACROS
@@ -76,126 +78,60 @@ WINE_DEFAULT_DEBUG_CHANNEL(wineboot);
 extern BOOL shutdown_close_windows( BOOL force );
 extern void kill_processes( BOOL kill_desktop );
 
-static BOOL GetLine( HANDLE hFile, char *buf, size_t buflen )
-{
-    unsigned int i=0;
-    DWORD r;
-    buf[0]='\0';
-
-    do
-    {
-        DWORD read;
-        if( !ReadFile( hFile, buf, 1, &read, NULL ) || read!=1 )
-        {
-            return FALSE;
-        }
-
-    } while( isspace( *buf ) );
-
-    while( buf[i]!='\n' && i<=buflen &&
-            ReadFile( hFile, buf+i+1, 1, &r, NULL ) )
-    {
-        ++i;
-    }
-
-
-    if( buf[i]!='\n' )
-    {
-	return FALSE;
-    }
-
-    if( i>0 && buf[i-1]=='\r' )
-        --i;
-
-    buf[i]='\0';
-
-    return TRUE;
-}
+static WCHAR windowsdir[MAX_PATH];
 
 /* Performs the rename operations dictated in %SystemRoot%\Wininit.ini.
  * Returns FALSE if there was an error, or otherwise if all is ok.
  */
 static BOOL wininit(void)
 {
-    const char * const RENAME_FILE="wininit.ini";
-    const char * const RENAME_FILE_TO="wininit.bak";
-    const char * const RENAME_FILE_SECTION="[rename]";
-    char buffer[MAX_LINE_LENGTH];
-    HANDLE hFile;
+    static const WCHAR nulW[] = {'N','U','L',0};
+    static const WCHAR renameW[] = {'r','e','n','a','m','e',0};
+    static const WCHAR wininitW[] = {'w','i','n','i','n','i','t','.','i','n','i',0};
+    static const WCHAR wininitbakW[] = {'w','i','n','i','n','i','t','.','b','a','k',0};
+    WCHAR initial_buffer[1024];
+    WCHAR *str, *buffer = initial_buffer;
+    DWORD size = sizeof(initial_buffer)/sizeof(WCHAR);
+    DWORD res;
 
-
-    hFile=CreateFileA(RENAME_FILE, GENERIC_READ,
-		    FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
-		    NULL );
-    
-    if( hFile==INVALID_HANDLE_VALUE )
+    for (;;)
     {
-	DWORD err=GetLastError();
-	
-	if( err==ERROR_FILE_NOT_FOUND )
-	{
-	    /* No file - nothing to do. Great! */
-	    WINE_TRACE("Wininit.ini not present - no renaming to do\n");
-
-	    return TRUE;
-	}
-
-	WINE_ERR("There was an error in reading wininit.ini file - %d\n",
-		GetLastError() );
-
-	return FALSE;
+        if (!(res = GetPrivateProfileSectionW( renameW, buffer, size, wininitW ))) return TRUE;
+        if (res < size - 2) break;
+        if (buffer != initial_buffer) HeapFree( GetProcessHeap(), 0, buffer );
+        size *= 2;
+        if (!(buffer = HeapAlloc( GetProcessHeap(), 0, size * sizeof(WCHAR) ))) return FALSE;
     }
 
-    printf("Wine is finalizing your software installation. This may take a few minutes,\n");
-    printf("though it never actually does.\n");
-
-    while( GetLine( hFile, buffer, sizeof(buffer) ) &&
-	    lstrcmpiA(buffer,RENAME_FILE_SECTION)!=0  )
-	; /* Read the lines until we match the rename section */
-
-    while( GetLine( hFile, buffer, sizeof(buffer) ) && buffer[0]!='[' )
+    for (str = buffer; *str; str += strlenW(str) + 1)
     {
-	/* First, make sure this is not a comment */
-	if( buffer[0]!=';' && buffer[0]!='\0' )
-	{
-	    char * value;
+        WCHAR *value;
 
-	    value=strchr(buffer, '=');
+        if (*str == ';') continue;  /* comment */
+        if (!(value = strchrW( str, '=' ))) continue;
 
-	    if( value==NULL )
-	    {
-		WINE_WARN("Line with no \"=\" in it in wininit.ini - %s\n",
-			buffer);
-	    } else
-	    {
-		/* split the line into key and value */
-		*(value++)='\0';
+        /* split the line into key and value */
+        *value++ = 0;
 
-                if( lstrcmpiA( "NUL", buffer )==0 )
-                {
-                    WINE_TRACE("Deleting file \"%s\"\n", value );
-                    /* A file to delete */
-                    if( !DeleteFileA( value ) )
-                        WINE_WARN("Error deleting file \"%s\"\n", value);
-                } else
-                {
-                    WINE_TRACE("Renaming file \"%s\" to \"%s\"\n", value,
-                            buffer );
+        if (!lstrcmpiW( nulW, str ))
+        {
+            WINE_TRACE("Deleting file %s\n", wine_dbgstr_w(value) );
+            if( !DeleteFileW( value ) )
+                WINE_WARN("Error deleting file %s\n", wine_dbgstr_w(value) );
+        }
+        else
+        {
+            WINE_TRACE("Renaming file %s to %s\n", wine_dbgstr_w(value), wine_dbgstr_w(str) );
 
-                    if( !MoveFileExA(value, buffer, MOVEFILE_COPY_ALLOWED|
-                            MOVEFILE_REPLACE_EXISTING) )
-                    {
-                        WINE_WARN("Error renaming \"%s\" to \"%s\"\n", value,
-                                buffer );
-                    }
-                }
-	    }
-	}
+            if( !MoveFileExW(value, str, MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING) )
+                WINE_WARN("Error renaming %s to %s\n", wine_dbgstr_w(value), wine_dbgstr_w(str) );
+        }
+        str = value;
     }
 
-    CloseHandle( hFile );
+    if (buffer != initial_buffer) HeapFree( GetProcessHeap(), 0, buffer );
 
-    if( !MoveFileExA( RENAME_FILE, RENAME_FILE_TO, MOVEFILE_REPLACE_EXISTING) )
+    if( !MoveFileExW( wininitW, wininitbakW, MOVEFILE_REPLACE_EXISTING) )
     {
         WINE_ERR("Couldn't rename wininit.ini, error %d\n", GetLastError() );
 
@@ -225,18 +161,8 @@ static BOOL pendingRename(void)
     if( (res=RegOpenKeyExW( HKEY_LOCAL_MACHINE, SessionW, 0, KEY_ALL_ACCESS, &hSession ))
             !=ERROR_SUCCESS )
     {
-        if( res==ERROR_FILE_NOT_FOUND )
-        {
-            WINE_TRACE("The key was not found - skipping\n");
-            res=TRUE;
-        }
-        else
-        {
-            WINE_ERR("Couldn't open key, error %d\n", res );
-            res=FALSE;
-        }
-
-        goto end;
+        WINE_TRACE("The key was not found - skipping\n");
+        return TRUE;
     }
 
     res=RegQueryValueExW( hSession, ValueName, NULL, NULL /* The value type does not really interest us, as it is not
@@ -391,9 +317,7 @@ static DWORD runCmd(LPWSTR cmdline, LPCWSTR dir, BOOL wait, BOOL minimized)
 
     if( !CreateProcessW(NULL, cmdline, NULL, NULL, FALSE, 0, NULL, dir, &si, &info) )
     {
-        WINE_ERR("Failed to run command %s (%d)\n", wine_dbgstr_w(cmdline),
-                 GetLastError() );
-
+        WINE_WARN("Failed to run command %s (%d)\n", wine_dbgstr_w(cmdline), GetLastError() );
         return INVALID_RUNCMD_RETURN;
     }
 
@@ -406,6 +330,7 @@ static DWORD runCmd(LPWSTR cmdline, LPCWSTR dir, BOOL wait, BOOL minimized)
         GetExitCodeProcess(info.hProcess, &exit_code);
     }
 
+    CloseHandle( info.hThread );
     CloseHandle( info.hProcess );
 
     return exit_code;
@@ -426,8 +351,8 @@ static BOOL ProcessRunKeys( HKEY hkRoot, LPCWSTR szKeyName, BOOL bDelete,
     static const WCHAR WINKEY_NAME[]={'S','o','f','t','w','a','r','e','\\',
         'M','i','c','r','o','s','o','f','t','\\','W','i','n','d','o','w','s','\\',
         'C','u','r','r','e','n','t','V','e','r','s','i','o','n',0};
-    HKEY hkWin=NULL, hkRun=NULL;
-    DWORD res=ERROR_SUCCESS;
+    HKEY hkWin, hkRun;
+    DWORD res;
     DWORD i, nMaxCmdLine=0, nMaxValue=0;
     WCHAR *szCmdLine=NULL;
     WCHAR *szValue=NULL;
@@ -437,36 +362,19 @@ static BOOL ProcessRunKeys( HKEY hkRoot, LPCWSTR szKeyName, BOOL bDelete,
     else
         WINE_TRACE("processing %s entries under HKCU\n",wine_dbgstr_w(szKeyName) );
 
-    if( (res=RegOpenKeyExW( hkRoot, WINKEY_NAME, 0, KEY_READ, &hkWin ))!=ERROR_SUCCESS )
+    if (RegOpenKeyExW( hkRoot, WINKEY_NAME, 0, KEY_READ, &hkWin ) != ERROR_SUCCESS)
+        return TRUE;
+
+    if (RegOpenKeyExW( hkWin, szKeyName, 0, bDelete?KEY_ALL_ACCESS:KEY_READ, &hkRun ) != ERROR_SUCCESS)
     {
-        WINE_ERR("RegOpenKey failed on Software\\Microsoft\\Windows\\CurrentVersion (%d)\n",
-                res);
-
-        goto end;
+        RegCloseKey( hkWin );
+        return TRUE;
     }
+    RegCloseKey( hkWin );
 
-    if( (res=RegOpenKeyExW( hkWin, szKeyName, 0, bDelete?KEY_ALL_ACCESS:KEY_READ, &hkRun ))!=
-            ERROR_SUCCESS)
-    {
-        if( res==ERROR_FILE_NOT_FOUND )
-        {
-            WINE_TRACE("Key doesn't exist - nothing to be done\n");
-
-            res=ERROR_SUCCESS;
-        }
-        else
-            WINE_ERR("RegOpenKey failed on run key (%d)\n", res);
-
-        goto end;
-    }
-    
     if( (res=RegQueryInfoKeyW( hkRun, NULL, NULL, NULL, NULL, NULL, NULL, &i, &nMaxValue,
                     &nMaxCmdLine, NULL, NULL ))!=ERROR_SUCCESS )
-    {
-        WINE_ERR("Couldn't query key info (%d)\n", res );
-
         goto end;
-    }
 
     if( i==0 )
     {
@@ -530,14 +438,15 @@ static BOOL ProcessRunKeys( HKEY hkRoot, LPCWSTR szKeyName, BOOL bDelete,
     res=ERROR_SUCCESS;
 
 end:
+    HeapFree( GetProcessHeap(), 0, szValue );
+    HeapFree( GetProcessHeap(), 0, szCmdLine );
+
     if( hkRun!=NULL )
         RegCloseKey( hkRun );
-    if( hkWin!=NULL )
-        RegCloseKey( hkWin );
 
     WINE_TRACE("done\n");
 
-    return res==ERROR_SUCCESS?TRUE:FALSE;
+    return res==ERROR_SUCCESS;
 }
 
 /*
@@ -546,82 +455,182 @@ end:
  * known good versions. The only programs that should install into this dll
  * cache are Windows Updates and IE (which is treated like a Windows Update)
  *
- * Implementing this allows installing ie in win2k mode to actaully install the
+ * Implementing this allows installing ie in win2k mode to actually install the
  * system dlls that we expect and need
  */
 static int ProcessWindowsFileProtection(void)
 {
-    WIN32_FIND_DATA finddata;
-    LPSTR custom_dllcache = NULL;
-    static CHAR default_dllcache[] = "C:\\Windows\\System32\\dllcache";
+    static const WCHAR winlogonW[] = {'S','o','f','t','w','a','r','e','\\',
+                                      'M','i','c','r','o','s','o','f','t','\\',
+                                      'W','i','n','d','o','w','s',' ','N','T','\\',
+                                      'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
+                                      'W','i','n','l','o','g','o','n',0};
+    static const WCHAR cachedirW[] = {'S','F','C','D','l','l','C','a','c','h','e','D','i','r',0};
+    static const WCHAR dllcacheW[] = {'\\','d','l','l','c','a','c','h','e','\\','*',0};
+    static const WCHAR wildcardW[] = {'\\','*',0};
+    WIN32_FIND_DATAW finddata;
     HANDLE find_handle;
     BOOL find_rc;
     DWORD rc;
     HKEY hkey;
-    LPSTR dllcache;
-    CHAR find_string[MAX_PATH];
-    CHAR windowsdir[MAX_PATH];
+    LPWSTR dllcache = NULL;
 
-    rc = RegOpenKeyA( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon", &hkey );
-    if (rc == ERROR_SUCCESS)
+    if (!RegOpenKeyW( HKEY_LOCAL_MACHINE, winlogonW, &hkey ))
     {
         DWORD sz = 0;
-        rc = RegQueryValueEx( hkey, "SFCDllCacheDir", 0, NULL, NULL, &sz);
-        if (rc == ERROR_MORE_DATA)
+        if (!RegQueryValueExW( hkey, cachedirW, 0, NULL, NULL, &sz))
         {
-            sz++;
-            custom_dllcache = HeapAlloc(GetProcessHeap(),0,sz);
-            RegQueryValueEx( hkey, "SFCDllCacheDir", 0, NULL, (LPBYTE)custom_dllcache, &sz);
+            sz += sizeof(WCHAR);
+            dllcache = HeapAlloc(GetProcessHeap(),0,sz + sizeof(wildcardW));
+            RegQueryValueExW( hkey, cachedirW, 0, NULL, (LPBYTE)dllcache, &sz);
+            strcatW( dllcache, wildcardW );
         }
     }
     RegCloseKey(hkey);
 
-    if (custom_dllcache)
-        dllcache = custom_dllcache;
-    else
-        dllcache = default_dllcache;
+    if (!dllcache)
+    {
+        DWORD sz = GetSystemDirectoryW( NULL, 0 );
+        dllcache = HeapAlloc( GetProcessHeap(), 0, sz * sizeof(WCHAR) + sizeof(dllcacheW));
+        GetSystemDirectoryW( dllcache, sz );
+        strcatW( dllcache, dllcacheW );
+    }
 
-    strcpy(find_string,dllcache);
-    strcat(find_string,"\\*.*");
-
-    GetWindowsDirectory(windowsdir,MAX_PATH);
-
-    find_handle = FindFirstFile(find_string,&finddata);
+    find_handle = FindFirstFileW(dllcache,&finddata);
+    dllcache[ strlenW(dllcache) - 2] = 0; /* strip off wildcard */
     find_rc = find_handle != INVALID_HANDLE_VALUE;
     while (find_rc)
     {
-        CHAR targetpath[MAX_PATH];
-        CHAR currentpath[MAX_PATH];
+        static const WCHAR dotW[] = {'.',0};
+        static const WCHAR dotdotW[] = {'.','.',0};
+        WCHAR targetpath[MAX_PATH];
+        WCHAR currentpath[MAX_PATH];
         UINT sz;
         UINT sz2;
-        CHAR tempfile[MAX_PATH];
+        WCHAR tempfile[MAX_PATH];
 
-        if (strcmp(finddata.cFileName,".") == 0 ||
-            strcmp(finddata.cFileName,"..") == 0)
+        if (strcmpW(finddata.cFileName,dotW) == 0 || strcmpW(finddata.cFileName,dotdotW) == 0)
         {
-            find_rc = FindNextFile(find_handle,&finddata);
+            find_rc = FindNextFileW(find_handle,&finddata);
             continue;
         }
 
         sz = MAX_PATH;
         sz2 = MAX_PATH;
-        VerFindFile(VFFF_ISSHAREDFILE, finddata.cFileName, windowsdir, 
-                windowsdir, currentpath, &sz, targetpath,&sz2);
+        VerFindFileW(VFFF_ISSHAREDFILE, finddata.cFileName, windowsdir,
+                     windowsdir, currentpath, &sz, targetpath, &sz2);
         sz = MAX_PATH;
-        rc = VerInstallFile(0, finddata.cFileName, finddata.cFileName,
-                            dllcache, targetpath, currentpath, tempfile,&sz);
+        rc = VerInstallFileW(0, finddata.cFileName, finddata.cFileName,
+                             dllcache, targetpath, currentpath, tempfile, &sz);
         if (rc != ERROR_SUCCESS)
         {
-            WINE_ERR("WFP: %s error 0x%x\n",finddata.cFileName,rc);
-            DeleteFile(tempfile);
+            WINE_WARN("WFP: %s error 0x%x\n",wine_dbgstr_w(finddata.cFileName),rc);
+            DeleteFileW(tempfile);
         }
-        find_rc = FindNextFile(find_handle,&finddata);
+
+        /* now delete the source file so that we don't try to install it over and over again */
+        lstrcpynW( targetpath, dllcache, MAX_PATH - 1 );
+        sz = strlenW( targetpath );
+        targetpath[sz++] = '\\';
+        lstrcpynW( targetpath + sz, finddata.cFileName, MAX_PATH - sz );
+        if (!DeleteFileW( targetpath ))
+            WINE_WARN( "failed to delete %s: error %u\n", wine_dbgstr_w(targetpath), GetLastError() );
+
+        find_rc = FindNextFileW(find_handle,&finddata);
     }
     FindClose(find_handle);
-    HeapFree(GetProcessHeap(),0,custom_dllcache);
+    HeapFree(GetProcessHeap(),0,dllcache);
     return 1;
 }
 
+static BOOL start_services_process(void)
+{
+    static const WCHAR svcctl_started_event[] = SVCCTL_STARTED_EVENT;
+    static const WCHAR services[] = {'\\','s','e','r','v','i','c','e','s','.','e','x','e',0};
+    PROCESS_INFORMATION pi;
+    STARTUPINFOW si;
+    HANDLE wait_handles[2];
+    WCHAR path[MAX_PATH];
+
+    if (!GetSystemDirectoryW(path, MAX_PATH - strlenW(services)))
+        return FALSE;
+    strcatW(path, services);
+    ZeroMemory(&si, sizeof(si));
+    si.cb = sizeof(si);
+    if (!CreateProcessW(path, path, NULL, NULL, TRUE, 0, NULL, NULL, &si, &pi))
+    {
+        WINE_ERR("Couldn't start services.exe: error %u\n", GetLastError());
+        return FALSE;
+    }
+    CloseHandle(pi.hThread);
+
+    wait_handles[0] = CreateEventW(NULL, TRUE, FALSE, svcctl_started_event);
+    wait_handles[1] = pi.hProcess;
+
+    /* wait for the event to become available or the process to exit */
+    if ((WaitForMultipleObjects(2, wait_handles, FALSE, INFINITE)) == WAIT_OBJECT_0 + 1)
+    {
+        DWORD exit_code;
+        GetExitCodeProcess(pi.hProcess, &exit_code);
+        WINE_ERR("Unexpected termination of services.exe - exit code %d\n", exit_code);
+        CloseHandle(pi.hProcess);
+        CloseHandle(wait_handles[0]);
+        return FALSE;
+    }
+
+    CloseHandle(pi.hProcess);
+    CloseHandle(wait_handles[0]);
+    return TRUE;
+}
+
+/* start services */
+static void start_services(void)
+{
+    static const WCHAR servicesW[] = {'S','y','s','t','e','m','\\',
+                                      'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
+                                      'S','e','r','v','i','c','e','s',0};
+    static const WCHAR startW[] = {'S','t','a','r','t',0};
+    HKEY hkey, skey;
+    DWORD type, size, start, index = 0;
+    WCHAR name[MAX_PATH];
+    SC_HANDLE manager;
+
+    if (!start_services_process()) return;
+
+    /* FIXME: do all of this in services.exe instead */
+    if (RegOpenKeyW( HKEY_LOCAL_MACHINE, servicesW, &hkey )) return;
+
+    if (!(manager = OpenSCManagerW( NULL, NULL, SC_MANAGER_ALL_ACCESS )))
+    {
+        RegCloseKey( hkey );
+        return;
+    }
+
+    while (!RegEnumKeyW( hkey, index++, name, sizeof(name)/sizeof(name[0]) ))
+    {
+        if (RegOpenKeyW( hkey, name, &skey )) continue;
+        size = sizeof(start);
+        if (!RegQueryValueExW( skey, startW, NULL, &type, (LPBYTE)&start, &size ) && type == REG_DWORD)
+        {
+            if (start == SERVICE_BOOT_START ||
+                start == SERVICE_SYSTEM_START ||
+                start == SERVICE_AUTO_START)
+            {
+                SC_HANDLE handle = OpenServiceW( manager, name, SERVICE_ALL_ACCESS );
+                if (handle)
+                {
+                    WINE_TRACE( "starting service %s start %u\n", wine_dbgstr_w(name), start );
+                    StartServiceW( handle, 0, NULL );
+                    CloseServiceHandle( handle );
+                }
+            }
+        }
+        RegCloseKey( skey );
+    }
+    CloseServiceHandle( manager );
+    RegCloseKey( hkey );
+}
+
 /* Process items in the StartUp group of the user's Programs under the Start Menu. Some installers put
  * shell links here to restart themselves after boot. */
 static BOOL ProcessStartupItems(void)
@@ -687,7 +696,7 @@ static BOOL ProcessStartupItems(void)
 		WINE_TRACE("Unable to parse display name.\n");
 	    else
 		if ((iRet = (int)ShellExecuteW(NULL, NULL, wszCommand, NULL, NULL, SW_SHOWNORMAL)) <= 32)
-		    WINE_ERR("Error %d executing command %s.\n", iRet, wine_dbgstr_w(wszCommand));
+		    WINE_WARN("Error %d executing command %s.\n", iRet, wine_dbgstr_w(wszCommand));
 	}
 
 	IMalloc_Free(ppM, pidlItem);
@@ -711,76 +720,51 @@ static void usage(void)
     WINE_MESSAGE( "    -h,--help         Display this help message\n" );
     WINE_MESSAGE( "    -e,--end-session  End the current session cleanly\n" );
     WINE_MESSAGE( "    -f,--force        Force exit for processes that don't exit cleanly\n" );
+    WINE_MESSAGE( "    -i,--init         Perform initialization for first Wine instance\n" );
     WINE_MESSAGE( "    -k,--kill         Kill running processes without any cleanup\n" );
     WINE_MESSAGE( "    -r,--restart      Restart only, don't do normal startup operations\n" );
     WINE_MESSAGE( "    -s,--shutdown     Shutdown only, don't reboot\n" );
 }
 
-static const char short_options[] = "efhkrs";
+static const char short_options[] = "efhikrs";
 
 static const struct option long_options[] =
 {
     { "help",        0, 0, 'h' },
     { "end-session", 0, 0, 'e' },
     { "force",       0, 0, 'f' },
+    { "init" ,       0, 0, 'i' },
     { "kill",        0, 0, 'k' },
     { "restart",     0, 0, 'r' },
     { "shutdown",    0, 0, 's' },
     { NULL,          0, 0, 0 }
 };
 
-struct op_mask {
-    BOOL w9xonly; /* Perform only operations done on Windows 9x */
-    BOOL ntonly; /* Perform only operations done on Windows NT */
-    BOOL startup; /* Perform the operations that are performed every boot */
-    BOOL preboot; /* Perform file renames typically done before the system starts */
-    BOOL prelogin; /* Perform the operations typically done before the user logs in */
-    BOOL postlogin; /* Operations done after login */
-};
-
-static const struct op_mask SESSION_START={FALSE, FALSE, TRUE, TRUE, TRUE, TRUE},
-    SETUP={FALSE, FALSE, FALSE, TRUE, TRUE, TRUE};
-
 int main( int argc, char *argv[] )
 {
-    struct op_mask ops = SESSION_START; /* Which of the ops do we want to perform? */
+    extern HANDLE __wine_make_process_system(void);
+    static const WCHAR wineboot_eventW[] = {'_','_','w','i','n','e','b','o','o','t','_','e','v','e','n','t',0};
+
     /* First, set the current directory to SystemRoot */
-    TCHAR gen_path[MAX_PATH];
-    DWORD res;
     int optc;
-    int end_session = 0, force = 0, kill = 0, restart = 0, shutdown = 0;
+    int end_session = 0, force = 0, init = 0, kill = 0, restart = 0, shutdown = 0;
+    HANDLE event;
+    SECURITY_ATTRIBUTES sa;
 
-    res=GetWindowsDirectory( gen_path, sizeof(gen_path) );
-    
-    if( res==0 )
+    GetWindowsDirectoryW( windowsdir, MAX_PATH );
+    if( !SetCurrentDirectoryW( windowsdir ) )
     {
-	WINE_ERR("Couldn't get the windows directory - error %d\n",
-		GetLastError() );
-
-	return 100;
-    }
-
-    if( res>=sizeof(gen_path) )
-    {
-	WINE_ERR("Windows path too long (%d)\n", res );
-
-	return 100;
-    }
-
-    if( !SetCurrentDirectory( gen_path ) )
-    {
-        WINE_ERR("Cannot set the dir to %s (%d)\n", gen_path, GetLastError() );
-
+        WINE_ERR("Cannot set the dir to %s (%d)\n", wine_dbgstr_w(windowsdir), GetLastError() );
         return 100;
     }
 
-
     while ((optc = getopt_long(argc, argv, short_options, long_options, NULL )) != -1)
     {
         switch(optc)
         {
-        case 'e': end_session = 1; break;
+        case 'e': end_session = kill = 1; break;
         case 'f': force = 1; break;
+        case 'i': init = 1; break;
         case 'k': kill = 1; break;
         case 'r': restart = 1; break;
         case 's': shutdown = 1; break;
@@ -794,37 +778,38 @@ int main( int argc, char *argv[] )
         if (!shutdown_close_windows( force )) return 1;
     }
 
-    if (end_session || kill) kill_processes( shutdown );
+    if (kill) kill_processes( shutdown );
 
     if (shutdown) return 0;
 
-    if (restart) ops = SETUP;
+    sa.nLength = sizeof(sa);
+    sa.lpSecurityDescriptor = NULL;
+    sa.bInheritHandle = TRUE;  /* so that services.exe inherits it */
+    event = CreateEventW( &sa, TRUE, FALSE, wineboot_eventW );
 
-    /* Perform the ops by order, stopping if one fails, skipping if necessary */
-    /* Shachar: Sorry for the perl syntax */
-    res=(ops.ntonly || !ops.preboot || wininit())&&
-        (ops.w9xonly || !ops.preboot || pendingRename()) &&
-        (ops.ntonly || !ops.prelogin ||
-         ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUNSERVICESONCE],
-                TRUE, FALSE )) &&
-        (ops.ntonly || !ops.prelogin ||
-         ProcessWindowsFileProtection()) &&
-        (ops.ntonly || !ops.prelogin || !ops.startup ||
-         ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUNSERVICES],
-                FALSE, FALSE )) &&
-        (!ops.postlogin ||
-         ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUNONCE],
-                TRUE, TRUE )) &&
-        (!ops.postlogin || !ops.startup ||
-         ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUN],
-                FALSE, FALSE )) &&
-        (!ops.postlogin || !ops.startup ||
-         ProcessRunKeys( HKEY_CURRENT_USER, runkeys_names[RUNKEY_RUN],
-                FALSE, FALSE )) &&
-	(!ops.postlogin || !ops.startup ||
-	 ProcessStartupItems( ));
+    ResetEvent( event );  /* in case this is a restart */
+
+    wininit();
+    pendingRename();
+
+    ProcessWindowsFileProtection();
+    ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUNSERVICESONCE], TRUE, FALSE );
+    if (init || (kill && !restart))
+    {
+        ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUNSERVICES], FALSE, FALSE );
+        start_services();
+    }
+    ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUNONCE], TRUE, TRUE );
+
+    if (!init && !restart)
+    {
+        ProcessRunKeys( HKEY_LOCAL_MACHINE, runkeys_names[RUNKEY_RUN], FALSE, FALSE );
+        ProcessRunKeys( HKEY_CURRENT_USER, runkeys_names[RUNKEY_RUN], FALSE, FALSE );
+        ProcessStartupItems();
+    }
 
     WINE_TRACE("Operation done\n");
 
-    return res?0:101;
+    SetEvent( event );
+    return 0;
 }
