@@ -48,6 +48,8 @@ static const struct
     const char *szProductType;
 } win_versions[] =
 {
+    { "win2008", "Windows 2008",   6,  0, 0x1771,VER_PLATFORM_WIN32_NT, "Service Pack 1", 0, 0, "ServerNT"},
+    { "vista",   "Windows Vista",  6,  0, 0x1770,VER_PLATFORM_WIN32_NT, " ", 0, 0, "WinNT"},
     { "win2003", "Windows 2003",   5,  2, 0xECE, VER_PLATFORM_WIN32_NT, "Service Pack 1", 1, 0, "ServerNT"},
     { "winxp",   "Windows XP",     5,  1, 0xA28, VER_PLATFORM_WIN32_NT, "Service Pack 2", 2, 0, "WinNT"},
     { "win2k",   "Windows 2000",   5,  0, 0x893, VER_PLATFORM_WIN32_NT, "Service Pack 4", 4, 0, "WinNT"},
@@ -68,11 +70,18 @@ static const char szKeyNT[] = "Software\\Microsoft\\Windows NT\\CurrentVersion";
 
 static int get_registry_version(void)
 {
-    int i, best = -1, platform, major, minor = 0;
+    int i, best = -1, platform, major, minor, build = 0;
     char *p, *ver;
 
     if ((ver = get_reg_key( HKEY_LOCAL_MACHINE, szKeyNT, "CurrentVersion", NULL )))
+    {
+        char *build_str;
+
         platform = VER_PLATFORM_WIN32_NT;
+
+	build_str = get_reg_key( HKEY_LOCAL_MACHINE, szKeyNT, "CurrentBuildNumber", NULL );
+        build = atoi(build_str);
+    }
     else if ((ver = get_reg_key( HKEY_LOCAL_MACHINE, szKey9x, "VersionNumber", NULL )))
         platform = VER_PLATFORM_WIN32_WINDOWS;
     else
@@ -80,10 +89,15 @@ static int get_registry_version(void)
 
     if ((p = strchr( ver, '.' )))
     {
-        char *str = p;
-        *str++ = 0;
-        if ((p = strchr( str, '.' ))) *p = 0;
-        minor = atoi(str);
+        char *minor_str = p;
+        *minor_str++ = 0;
+        if ((p = strchr( minor_str, '.' )))
+        {
+            char *build_str = p;
+            *build_str++ = 0;
+            build = atoi(build_str);
+        }
+        minor = atoi(minor_str);
     }
     major = atoi(ver);
 
@@ -92,7 +106,9 @@ static int get_registry_version(void)
         if (win_versions[i].dwPlatformId != platform) continue;
         if (win_versions[i].dwMajorVersion != major) continue;
         best = i;
-        if (win_versions[i].dwMinorVersion == minor) return i;
+        if ((win_versions[i].dwMinorVersion == minor) &&
+            (win_versions[i].dwBuildNumber == build))
+            return i;
     }
     return best;
 }
@@ -106,7 +122,7 @@ static void update_comboboxes(HWND dialog)
     winver = get_reg_key(config_key, keypath(""), "Version", "");
     ver = get_registry_version();
 
-    if (*winver == '\0')
+    if (!winver || !winver[0])
     {
         HeapFree(GetProcessHeap(), 0, winver);
 
