@@ -24,7 +24,6 @@
  */
 
 #define IDC_STATIC                     -1
-#define IDS_WINE_VERSION                1
 #define IDS_TAB_APPLICATIONS            2
 #define IDS_TAB_DLLS                    3
 #define IDS_TAB_DRIVES                  4
@@ -152,6 +151,9 @@
 #define IDS_SHADER_MODE_HARDWARE        8100
 #define IDS_SHADER_MODE_NONE            8101
 
+#define IDC_RES_TRACKBAR                1107
+#define IDC_RES_DPIEDIT                 1108
+
 /* applications tab */
 #define IDC_APP_LISTVIEW                1200
 #define IDC_APP_ADDAPP                  1201
@@ -159,22 +161,24 @@
 
 /* audio tab */
 #define IDC_AUDIO_CONFIGURE             1300
-#define IDC_AUDIO_CONTROL_PANEL         1301
-#define IDC_DSOUND_HW_ACCEL             1302
-#define IDC_DSOUND_DRV_EMUL             1303
-#define IDC_AUDIO_TREE			1304
-#define IDR_WINECFG			1305
-#define IDB_CHECKBOX                    1306
-#define IDB_DEVICE                      1307
-#define IDS_AUDIO_MISSING               1308
-#define IDC_DSOUND_RATES                1309
-#define IDC_DSOUND_BITS                 1310
+#define IDC_AUDIO_TEST                  1301
+#define IDC_AUDIO_CONTROL_PANEL         1302
+#define IDC_DSOUND_HW_ACCEL             1303
+#define IDC_DSOUND_DRV_EMUL             1304
+#define IDC_AUDIO_TREE			1305
+#define IDR_WINECFG			1306
+#define IDB_CHECKBOX                    1307
+#define IDB_DEVICE                      1308
+#define IDS_AUDIO_MISSING               1309
+#define IDC_DSOUND_RATES                1310
+#define IDC_DSOUND_BITS                 1311
+#define IDW_TESTSOUND                   1312
 #define IDS_ACCEL_FULL                  8300
 #define IDS_ACCEL_STANDARD              8301
 #define IDS_ACCEL_BASIC                 8302
 #define IDS_ACCEL_EMULATION             8303
 #define IDS_DRIVER_ALSA                 8304
-#define IDS_DRIVER_ARTS                 8305
+
 #define IDS_DRIVER_ESOUND               8306
 #define IDS_DRIVER_OSS                  8307
 #define IDS_DRIVER_JACK                 8308
@@ -227,3 +231,19 @@
 #define IDC_SYSPARAMS_ACTIVE_TITLE_TEXT 8413
 #define IDC_SYSPARAMS_INACTIVE_TITLE    8414
 #define IDC_SYSPARAMS_INACTIVE_TITLE_TEXT 8415
+#define IDC_SYSPARAMS_MSGBOX_TEXT       8416
+#define IDC_SYSPARAMS_APPWORKSPACE      8417
+#define IDC_SYSPARAMS_WINDOW_FRAME      8418
+#define IDC_SYSPARAMS_ACTIVE_BORDER     8419
+#define IDC_SYSPARAMS_INACTIVE_BORDER   8420
+#define IDC_SYSPARAMS_BUTTON_SHADOW     8421
+#define IDC_SYSPARAMS_GRAY_TEXT         8422
+#define IDC_SYSPARAMS_BUTTON_HILIGHT    8423
+#define IDC_SYSPARAMS_BUTTON_DARK_SHADOW 8424
+#define IDC_SYSPARAMS_BUTTON_LIGHT      8425
+#define IDC_SYSPARAMS_BUTTON_ALTERNATE 8426
+#define IDC_SYSPARAMS_HOT_TRACKING      8427
+#define IDC_SYSPARAMS_ACTIVE_TITLE_GRADIENT 8428
+#define IDC_SYSPARAMS_INACTIVE_TITLE_GRADIENT 8429
+#define IDC_SYSPARAMS_MENU_HILIGHT      8430
+#define IDC_SYSPARAMS_MENUBAR           8431
