@@ -36,7 +36,14 @@
 WINE_DEFAULT_DEBUG_CHANNEL(winecfg);
 
 #define RES_MAXLEN 5 /* the maximum number of characters in a screen dimension. 5 digits should be plenty, what kind of crazy person runs their screen >10,000 pixels across? */
+#define MINDPI 96
+#define MAXDPI 480
+#define DEFDPI 96
 
+#define IDT_DPIEDIT 0x1234
+
+static const WCHAR logpixels_reg[] = {'S','y','s','t','e','m','\\','C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\','H','a','r','d','w','a','r','e',' ','P','r','o','f','i','l','e','s','\\','C','u','r','r','e','n','t','\\','S','o','f','t','w','a','r','e','\\','F','o','n','t','s',0};
+static const WCHAR logpixels[] = {'L','o','g','P','i','x','e','l','s',0};
 
 static struct SHADERMODE
 {
@@ -74,7 +81,7 @@ static void update_gui_for_desktop_mode(HWND dialog) {
         char* buf, *bufindex;
 	CheckDlgButton(dialog, IDC_ENABLE_DESKTOP, BST_CHECKED);
 
-        buf = get_reg_key(config_key, keypath("X11 Driver"), "Desktop", "640x480");
+        buf = get_reg_key(config_key, keypath("X11 Driver"), "Desktop", "800x600");
         /* note: this test must match the one in x11drv */
         if( buf[0] != 'n' &&  buf[0] != 'N' &&  buf[0] != 'F' &&  buf[0] != 'f'
                 &&  buf[0] != '0') {
@@ -92,8 +99,8 @@ static void update_gui_for_desktop_mode(HWND dialog) {
                 SetWindowText(GetDlgItem(dialog, IDC_DESKTOP_HEIGHT), bufindex);
             } else {
                 WINE_TRACE("Desktop registry entry is malformed\n");
-                SetWindowText(GetDlgItem(dialog, IDC_DESKTOP_WIDTH), "640");
-                SetWindowText(GetDlgItem(dialog, IDC_DESKTOP_HEIGHT), "480");
+                SetWindowText(GetDlgItem(dialog, IDC_DESKTOP_WIDTH), "800");
+                SetWindowText(GetDlgItem(dialog, IDC_DESKTOP_HEIGHT), "600");
             }
         }
         HeapFree(GetProcessHeap(), 0, buf);
@@ -179,12 +186,12 @@ static void set_from_desktop_edits(HWND dialog) {
 
     if (width == NULL || strcmp(width, "") == 0) {
         HeapFree(GetProcessHeap(), 0, width);
-        width = strdupA("640");
+        width = strdupA("800");
     }
     
     if (height == NULL || strcmp(height, "") == 0) {
         HeapFree(GetProcessHeap(), 0, height);
-        height = strdupA("480");
+        height = strdupA("600");
     }
 
     new = HeapAlloc(GetProcessHeap(), 0, strlen(width) + strlen(height) + 2 /* x + terminator */);
@@ -237,17 +244,100 @@ static void on_d3d_pshader_mode_clicked(HWND dialog) {
     else
         set_reg_key(config_key, keypath("Direct3D"), "PixelShaderMode", "disabled");
 }
+static INT read_logpixels_reg(void)
+{
+    DWORD dwLogPixels;
+    WCHAR *buf = get_reg_keyW(HKEY_LOCAL_MACHINE, logpixels_reg, logpixels, NULL);
+    dwLogPixels = buf ? *buf : DEFDPI;
+    HeapFree(GetProcessHeap(), 0, buf);
+    return dwLogPixels;
+}
+
+static void init_dpi_editbox(HWND hDlg)
+{
+    DWORD dwLogpixels;
+    char szLogpixels[MAXBUFLEN];
+
+    updating_ui = TRUE;
+
+    dwLogpixels = read_logpixels_reg();
+    WINE_TRACE("%u\n", dwLogpixels);
+
+    sprintf(szLogpixels, "%u", dwLogpixels);
+    SetDlgItemText(hDlg, IDC_RES_DPIEDIT, szLogpixels);
+
+    updating_ui = FALSE;
+}
+
+static void init_trackbar(HWND hDlg)
+{
+    HWND hTrackBar = GetDlgItem(hDlg, IDC_RES_TRACKBAR);
+    DWORD dwLogpixels;
+
+    updating_ui = TRUE;
+
+    dwLogpixels = read_logpixels_reg();
+
+    SendMessageW(hTrackBar, TBM_SETRANGE, TRUE, MAKELONG(MINDPI, MAXDPI));
+    SendMessageW(hTrackBar, TBM_SETPOS, TRUE, dwLogpixels);
+
+    updating_ui = FALSE;
+}
+
+static void update_dpi_trackbar_from_edit(HWND hDlg, BOOL fix)
+{
+    DWORD dpi;
+
+    updating_ui = TRUE;
+
+    dpi = GetDlgItemInt(hDlg, IDC_RES_DPIEDIT, NULL, FALSE);
+
+    if (fix)
+    {
+        DWORD fixed_dpi = dpi;
+
+        if (dpi < MINDPI) fixed_dpi = MINDPI;
+        if (dpi > MAXDPI) fixed_dpi = MAXDPI;
+
+        if (fixed_dpi != dpi)
+        {
+            char buf[16];
+
+            dpi = fixed_dpi;
+            sprintf(buf, "%u", dpi);
+            SetDlgItemText(hDlg, IDC_RES_DPIEDIT, buf);
+        }
+    }
+
+    if (dpi >= MINDPI && dpi <= MAXDPI)
+    {
+        SendDlgItemMessage(hDlg, IDC_RES_TRACKBAR, TBM_SETPOS, TRUE, dpi);
+        set_reg_key_dwordW(HKEY_LOCAL_MACHINE, logpixels_reg, logpixels, dpi);
+    }
+
+    updating_ui = FALSE;
+}
 
 INT_PTR CALLBACK
 GraphDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
     switch (uMsg) {
 	case WM_INITDIALOG:
+	    init_dpi_editbox(hDlg);
+	    init_trackbar(hDlg);
 	    break;
 
         case WM_SHOWWINDOW:
             set_window_title(hDlg);
             break;
+
+        case WM_TIMER:
+            if (wParam == IDT_DPIEDIT)
+            {
+                KillTimer(hDlg, IDT_DPIEDIT);
+                update_dpi_trackbar_from_edit(hDlg, TRUE);
+            }
+            break;
             
 	case WM_COMMAND:
 	    switch(HIWORD(wParam)) {
@@ -256,6 +346,11 @@ GraphDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 		    SendMessage(GetParent(hDlg), PSM_CHANGED, 0, 0);
 		    if ( ((LOWORD(wParam) == IDC_DESKTOP_WIDTH) || (LOWORD(wParam) == IDC_DESKTOP_HEIGHT)) && !updating_ui )
 			set_from_desktop_edits(hDlg);
+                    else if (LOWORD(wParam) == IDC_RES_DPIEDIT)
+                    {
+                        update_dpi_trackbar_from_edit(hDlg, FALSE);
+                        SetTimer(hDlg, IDT_DPIEDIT, 1500, NULL);
+                    }
 		    break;
 		}
 		case BN_CLICKED: {
@@ -301,6 +396,20 @@ GraphDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 	    }
 	    break;
 
+	case WM_HSCROLL:
+	    switch (wParam) {
+		default: {
+		    char buf[MAXBUFLEN];
+		    int i = SendMessageW(GetDlgItem(hDlg, IDC_RES_TRACKBAR), TBM_GETPOS, 0, 0);
+		    buf[0] = 0;
+		    sprintf(buf, "%d", i);
+		    SendMessage(GetDlgItem(hDlg, IDC_RES_DPIEDIT), WM_SETTEXT, 0, (LPARAM) buf);
+		    set_reg_key_dwordW(HKEY_LOCAL_MACHINE, logpixels_reg, logpixels, i);
+		    break;
+		}
+	    }
+	    break;
+
 	default:
 	    break;
     }
