@@ -242,6 +242,16 @@ const struct dbg_internal_var* dbg_get_internal_var(const char* name)
     return NULL;
 }
 
+unsigned         dbg_num_processes(void)
+{
+    struct dbg_process*	p;
+    unsigned            num = 0;
+
+    for (p = dbg_process_list; p; p = p->next)
+	num++;
+    return num;
+}
+
 struct dbg_process*     dbg_get_process(DWORD pid)
 {
     struct dbg_process*	p;
@@ -335,7 +345,7 @@ struct mod_loader_info
     IMAGEHLP_MODULE*    imh_mod;
 };
 
-static BOOL CALLBACK mod_loader_cb(PSTR mod_name, DWORD base, void* ctx)
+static BOOL CALLBACK mod_loader_cb(PCSTR mod_name, ULONG base, PVOID ctx)
 {
     struct mod_loader_info*     mli = (struct mod_loader_info*)ctx;
 
@@ -594,7 +604,7 @@ int main(int argc, char** argv)
     parser_handle(hFile);
 
     while (dbg_process_list)
-        dbg_process_list->process_io->close_process(dbg_process_list, TRUE);
+        dbg_process_list->process_io->close_process(dbg_process_list, FALSE);
 
     dbg_save_internal_vars();
 
