@@ -30,3 +30,6 @@
 #define STID_HELP_FILES_HLP	0x12C
 #define STID_DIALOG_TEST	0x12D
 #define STID_FILE_NOT_FOUND_s	0x12E
+
+#define IDD_INDEX               0x150
+#define IDC_INDEXLIST           0x151
