@@ -191,6 +191,7 @@ static const struct object_ops fd_ops =
 {
     sizeof(struct fd),        /* size */
     fd_dump,                  /* dump */
+    no_get_type,              /* get_type */
     no_add_queue,             /* add_queue */
     NULL,                     /* remove_queue */
     NULL,                     /* signaled */
@@ -198,6 +199,8 @@ static const struct object_ops fd_ops =
     no_signal,                /* signal */
     no_get_fd,                /* get_fd */
     no_map_access,            /* map_access */
+    default_get_sd,           /* get_sd */
+    default_set_sd,           /* set_sd */
     no_lookup_name,           /* lookup_name */
     no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
@@ -225,6 +228,7 @@ static const struct object_ops device_ops =
 {
     sizeof(struct device),    /* size */
     device_dump,              /* dump */
+    no_get_type,              /* get_type */
     no_add_queue,             /* add_queue */
     NULL,                     /* remove_queue */
     NULL,                     /* signaled */
@@ -232,6 +236,8 @@ static const struct object_ops device_ops =
     no_signal,                /* signal */
     no_get_fd,                /* get_fd */
     no_map_access,            /* map_access */
+    default_get_sd,           /* get_sd */
+    default_set_sd,           /* set_sd */
     no_lookup_name,           /* lookup_name */
     no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
@@ -258,6 +264,7 @@ static const struct object_ops inode_ops =
 {
     sizeof(struct inode),     /* size */
     inode_dump,               /* dump */
+    no_get_type,              /* get_type */
     no_add_queue,             /* add_queue */
     NULL,                     /* remove_queue */
     NULL,                     /* signaled */
@@ -265,6 +272,8 @@ static const struct object_ops inode_ops =
     no_signal,                /* signal */
     no_get_fd,                /* get_fd */
     no_map_access,            /* map_access */
+    default_get_sd,           /* get_sd */
+    default_set_sd,           /* set_sd */
     no_lookup_name,           /* lookup_name */
     no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
@@ -293,6 +302,7 @@ static const struct object_ops file_lock_ops =
 {
     sizeof(struct file_lock),   /* size */
     file_lock_dump,             /* dump */
+    no_get_type,                /* get_type */
     add_queue,                  /* add_queue */
     remove_queue,               /* remove_queue */
     file_lock_signaled,         /* signaled */
@@ -300,6 +310,8 @@ static const struct object_ops file_lock_ops =
     no_signal,                  /* signal */
     no_get_fd,                  /* get_fd */
     no_map_access,              /* map_access */
+    default_get_sd,             /* get_sd */
+    default_set_sd,             /* set_sd */
     no_lookup_name,             /* lookup_name */
     no_open_file,               /* open_file */
     no_close_handle,            /* close_handle */
@@ -420,6 +432,11 @@ static struct fd **freelist;                /* list of free entries in the array
 
 static int get_next_timeout(void);
 
+static inline void fd_poll_event( struct fd *fd, int event )
+{
+    fd->fd_ops->poll_event( fd, event );
+}
+
 #ifdef USE_EPOLL
 
 static int epoll_fd = -1;
@@ -1681,12 +1698,6 @@ int fd_close_handle( struct object *obj, struct process *process, obj_handle_t h
     return (!current || current->process == process);
 }
 
-/* callback for event happening in the main poll() loop */
-void fd_poll_event( struct fd *fd, int event )
-{
-    return fd->fd_ops->poll_event( fd, event );
-}
-
 /* check if events are pending and if yes return which one(s) */
 int check_fd_events( struct fd *fd, int events )
 {
@@ -1710,6 +1721,16 @@ int default_fd_signaled( struct object *obj, struct thread *thread )
     return ret;
 }
 
+/* default map_access() routine for objects that behave like an fd */
+unsigned int default_fd_map_access( struct object *obj, unsigned int access )
+{
+    if (access & GENERIC_READ)    access |= FILE_GENERIC_READ;
+    if (access & GENERIC_WRITE)   access |= FILE_GENERIC_WRITE;
+    if (access & GENERIC_EXECUTE) access |= FILE_GENERIC_EXECUTE;
+    if (access & GENERIC_ALL)     access |= FILE_ALL_ACCESS;
+    return access & ~(GENERIC_READ | GENERIC_WRITE | GENERIC_EXECUTE | GENERIC_ALL);
+}
+
 int default_fd_get_poll_events( struct fd *fd )
 {
     int events = 0;
@@ -1901,6 +1922,12 @@ static struct fd *get_handle_fd_obj( struct process *process, obj_handle_t handl
     return fd;
 }
 
+void fd_assign_completion( struct fd *fd, struct completion **p_port, unsigned long *p_key )
+{
+    *p_key = fd->comp_key;
+    *p_port = fd->completion ? (struct completion *)grab_object( fd->completion ) : NULL;
+}
+
 /* flush a file buffers */
 DECL_HANDLER(flush_file)
 {
@@ -1952,11 +1979,11 @@ DECL_HANDLER(get_handle_fd)
         int unix_fd = get_unix_fd( fd );
         if (unix_fd != -1)
         {
-            send_client_fd( current->process, unix_fd, req->handle );
             reply->type = fd->fd_ops->get_fd_type( fd );
             reply->removable = is_fd_removable(fd);
             reply->options = fd->options;
             reply->access = get_handle_access( current->process, req->handle );
+            send_client_fd( current->process, unix_fd, req->handle );
         }
         release_object( fd );
     }
@@ -2022,7 +2049,7 @@ DECL_HANDLER(set_completion_info)
 
     if (fd)
     {
-        if (!fd->completion)
+        if (!(fd->options & (FILE_SYNCHRONOUS_IO_ALERT | FILE_SYNCHRONOUS_IO_NONALERT)) && !fd->completion)
         {
             fd->completion = get_completion_obj( current->process, req->chandle, IO_COMPLETION_MODIFY_STATE );
             fd->comp_key = req->ckey;
@@ -2031,3 +2058,15 @@ DECL_HANDLER(set_completion_info)
         release_object( fd );
     }
 }
+
+/* push new completion msg into a completion queue attached to the fd */
+DECL_HANDLER(add_fd_completion)
+{
+    struct fd *fd = get_handle_fd_obj( current->process, req->handle, 0 );
+    if (fd)
+    {
+        if (fd->completion)
+            add_completion( fd->completion, fd->comp_key, req->cvalue, req->status, req->information );
+        release_object( fd );
+    }
+}
