@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <unistd.h>
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
@@ -66,13 +67,14 @@ const LUID SeCreateGlobalPrivilege         = { 30, 0 };
 static const SID world_sid = { SID_REVISION, 1, { SECURITY_WORLD_SID_AUTHORITY }, { SECURITY_WORLD_RID } };
 static const SID local_sid = { SID_REVISION, 1, { SECURITY_LOCAL_SID_AUTHORITY }, { SECURITY_LOCAL_RID } };
 static const SID interactive_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_INTERACTIVE_RID } };
+static const SID anonymous_logon_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_ANONYMOUS_LOGON_RID } };
 static const SID authenticated_user_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_AUTHENTICATED_USER_RID } };
 static const SID local_system_sid = { SID_REVISION, 1, { SECURITY_NT_AUTHORITY }, { SECURITY_LOCAL_SYSTEM_RID } };
-static const PSID security_world_sid = (PSID)&world_sid;
+const PSID security_world_sid = (PSID)&world_sid;
 static const PSID security_local_sid = (PSID)&local_sid;
 const PSID security_interactive_sid = (PSID)&interactive_sid;
 static const PSID security_authenticated_user_sid = (PSID)&authenticated_user_sid;
-static const PSID security_local_system_sid = (PSID)&local_system_sid;
+const PSID security_local_system_sid = (PSID)&local_system_sid;
 
 static luid_t prev_luid_value = { 1000, 0 };
 
@@ -88,7 +90,7 @@ struct token
     unsigned       primary;         /* is this a primary or impersonation token? */
     ACL           *default_dacl;    /* the default DACL to assign to objects created by this user */
     TOKEN_SOURCE   source;          /* source of the token */
-    SECURITY_IMPERSONATION_LEVEL impersonation_level; /* impersonation level this token is capable of if non-primary token */
+    int            impersonation_level; /* impersonation level this token is capable of if non-primary token */
 };
 
 struct privilege
@@ -120,6 +122,7 @@ static const struct object_ops token_ops =
 {
     sizeof(struct token),      /* size */
     token_dump,                /* dump */
+    no_get_type,               /* get_type */
     no_add_queue,              /* add_queue */
     NULL,                      /* remove_queue */
     NULL,                      /* signaled */
@@ -127,6 +130,8 @@ static const struct object_ops token_ops =
     no_signal,                 /* signal */
     no_get_fd,                 /* get_fd */
     token_map_access,          /* map_access */
+    default_get_sd,            /* get_sd */
+    default_set_sd,            /* set_sd */
     no_lookup_name,            /* lookup_name */
     no_open_file,              /* open_file */
     no_close_handle,           /* close_handle */
@@ -162,12 +167,6 @@ static SID *security_sid_alloc( const SID_IDENTIFIER_AUTHORITY *idauthority, int
     return sid;
 }
 
-static inline int security_equal_sid( const SID *sid1, const SID *sid2 )
-{
-    return ((sid1->SubAuthorityCount == sid2->SubAuthorityCount) &&
-        !memcmp( sid1, sid2, FIELD_OFFSET(SID, SubAuthority[sid1->SubAuthorityCount]) ));
-}
-
 void security_set_thread_token( struct thread *thread, obj_handle_t handle )
 {
     if (!handle)
@@ -191,9 +190,13 @@ void security_set_thread_token( struct thread *thread, obj_handle_t handle )
     }
 }
 
-static const ACE_HEADER *ace_next( const ACE_HEADER *ace )
+const SID *security_unix_uid_to_sid( uid_t uid )
 {
-    return (const ACE_HEADER *)((const char *)ace + ace->AceSize);
+    /* very simple mapping: either the current user or not the current user */
+    if (uid == getuid())
+        return &interactive_sid;
+    else
+        return &anonymous_logon_sid;
 }
 
 static int acl_is_valid( const ACL *acl, data_size_t size )
@@ -248,51 +251,9 @@ static int acl_is_valid( const ACL *acl, data_size_t size )
     return TRUE;
 }
 
-/* gets the discretionary access control list from a security descriptor */
-static inline const ACL *sd_get_dacl( const struct security_descriptor *sd, int *present )
-{
-    *present = (sd->control & SE_DACL_PRESENT ? TRUE : FALSE);
-
-    if (sd->dacl_len)
-        return (const ACL *)((const char *)(sd + 1) +
-            sd->owner_len + sd->group_len + sd->sacl_len);
-    else
-        return NULL;
-}
-
-/* gets the system access control list from a security descriptor */
-static inline const ACL *sd_get_sacl( const struct security_descriptor *sd, int *present )
-{
-    *present = (sd->control & SE_SACL_PRESENT ? TRUE : FALSE);
-
-    if (sd->sacl_len)
-        return (const ACL *)((const char *)(sd + 1) +
-            sd->owner_len + sd->group_len);
-    else
-        return NULL;
-}
-
-/* gets the owner from a security descriptor */
-static inline const SID *sd_get_owner( const struct security_descriptor *sd )
-{
-    if (sd->owner_len)
-        return (const SID *)(sd + 1);
-    else
-        return NULL;
-}
-
-/* gets the primary group from a security descriptor */
-static inline const SID *sd_get_group( const struct security_descriptor *sd )
-{
-    if (sd->group_len)
-        return (const SID *)((const char *)(sd + 1) + sd->owner_len);
-    else
-        return NULL;
-}
-
 /* checks whether all members of a security descriptor fit inside the size
  * of memory specified */
-static int sd_is_valid( const struct security_descriptor *sd, data_size_t size )
+int sd_is_valid( const struct security_descriptor *sd, data_size_t size )
 {
     size_t offset = sizeof(struct security_descriptor);
     const SID *group;
@@ -345,6 +306,30 @@ static int sd_is_valid( const struct security_descriptor *sd, data_size_t size )
     return TRUE;
 }
 
+/* determines whether an object_attributes struct is valid in a buffer
+ * and calls set_error appropriately */
+int objattr_is_valid( const struct object_attributes *objattr, data_size_t size )
+{
+    if ((size < sizeof(*objattr)) || (size - sizeof(*objattr) < objattr->sd_len) ||
+        (size - sizeof(*objattr) - objattr->sd_len < objattr->name_len))
+    {
+        set_error( STATUS_ACCESS_VIOLATION );
+        return FALSE;
+    }
+
+    if (objattr->sd_len)
+    {
+        const struct security_descriptor *sd = (const struct security_descriptor *)(objattr + 1);
+        if (!sd_is_valid( sd, objattr->sd_len ))
+        {
+            set_error( STATUS_INVALID_SECURITY_DESCR );
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
 /* maps from generic rights to specific rights as given by a mapping */
 static inline void map_generic_mask(unsigned int *mask, const GENERIC_MAPPING *mapping)
 {
@@ -436,7 +421,7 @@ static struct token *create_token( unsigned primary, const SID *user,
                                    const LUID_AND_ATTRIBUTES *privs, unsigned int priv_count,
                                    const ACL *default_dacl, TOKEN_SOURCE source,
                                    const luid_t *modified_id,
-                                   SECURITY_IMPERSONATION_LEVEL impersonation_level )
+                                   int impersonation_level )
 {
     struct token *token = alloc_object( &token_ops );
     if (token)
@@ -481,7 +466,7 @@ static struct token *create_token( unsigned primary, const SID *user,
             memcpy( &group->sid, groups[i].Sid, FIELD_OFFSET( SID, SubAuthority[((const SID *)groups[i].Sid)->SubAuthorityCount] ) );
             group->enabled = TRUE;
             group->def = TRUE;
-            group->logon = FALSE;
+            group->logon = (groups[i].Attributes & SE_GROUP_LOGON_ID) ? TRUE : FALSE;
             group->mandatory = (groups[i].Attributes & SE_GROUP_MANDATORY) ? TRUE : FALSE;
             group->owner = groups[i].Attributes & SE_GROUP_OWNER ? TRUE : FALSE;
             group->resource = FALSE;
@@ -521,7 +506,7 @@ static struct token *create_token( unsigned primary, const SID *user,
 }
 
 struct token *token_duplicate( struct token *src_token, unsigned primary,
-                               SECURITY_IMPERSONATION_LEVEL impersonation_level )
+                               int impersonation_level )
 {
     const luid_t *modified_id =
         primary || (impersonation_level == src_token->impersonation_level) ?
@@ -530,20 +515,19 @@ struct token *token_duplicate( struct token *src_token, unsigned primary,
     struct privilege *privilege;
     struct group *group;
 
-    if ((impersonation_level < SecurityAnonymous) ||
-        (impersonation_level > SecurityDelegation))
+    if (!primary &&
+        (impersonation_level < SecurityAnonymous ||
+         impersonation_level > SecurityDelegation ||
+         (!src_token->primary && (impersonation_level > src_token->impersonation_level))))
     {
         set_error( STATUS_BAD_IMPERSONATION_LEVEL );
         return NULL;
     }
 
-    if (primary || (impersonation_level <= src_token->impersonation_level))
-        token = create_token( primary, src_token->user, NULL, 0,
-                              NULL, 0, src_token->default_dacl,
-                              src_token->source, modified_id,
-                              impersonation_level );
-    else set_error( STATUS_BAD_IMPERSONATION_LEVEL );
-
+    token = create_token( primary, src_token->user, NULL, 0,
+                          NULL, 0, src_token->default_dacl,
+                          src_token->source, modified_id,
+                          impersonation_level );
     if (!token) return token;
 
     /* copy groups */
@@ -586,7 +570,7 @@ static ACL *create_default_dacl( const SID *user )
     default_dacl = mem_alloc( default_dacl_size );
     if (!default_dacl) return NULL;
 
-    default_dacl->AclRevision = MAX_ACL_REVISION;
+    default_dacl->AclRevision = ACL_REVISION;
     default_dacl->Sbz1 = 0;
     default_dacl->AclSize = default_dacl_size;
     default_dacl->AceCount = 2;
@@ -681,9 +665,7 @@ struct token *token_create_admin( void )
             { logon_sid, SE_GROUP_ENABLED|SE_GROUP_ENABLED_BY_DEFAULT|SE_GROUP_MANDATORY|SE_GROUP_LOGON_ID },
         };
         static const TOKEN_SOURCE admin_source = {"SeMgr", {0, 0}};
-        /* note: we just set the user sid to be the interactive builtin sid -
-         * we should really translate the UNIX user id to a sid */
-        token = create_token( TRUE, &interactive_sid,
+        token = create_token( TRUE, security_unix_uid_to_sid( getuid() ),
                             admin_groups, sizeof(admin_groups)/sizeof(admin_groups[0]),
                             admin_privs, sizeof(admin_privs)/sizeof(admin_privs[0]),
                             default_dacl, admin_source, NULL, -1 );
@@ -853,18 +835,12 @@ static unsigned int token_access_check( struct token *token,
     }
 
     /* 1: Grant desired access if the object is unprotected */
-    if (!dacl_present)
+    if (!dacl_present || !dacl)
     {
         *priv_count = 0;
         *granted_access = desired_access;
         return *status = STATUS_SUCCESS;
     }
-    if (!dacl)
-    {
-        *priv_count = 0;
-        *status = STATUS_ACCESS_DENIED;
-        return STATUS_SUCCESS;
-    }
 
     /* 2: Check if caller wants access to system security part. Note: access
      * is only granted if specifically asked for */
@@ -920,11 +896,15 @@ static unsigned int token_access_check( struct token *token,
 
     /* 4: Grant rights according to the DACL */
     ace = (const ACE_HEADER *)(dacl + 1);
-    for (i = 0; i < dacl->AceCount; i++)
+    for (i = 0; i < dacl->AceCount; i++, ace = ace_next( ace ))
     {
         const ACCESS_ALLOWED_ACE *aa_ace;
         const ACCESS_DENIED_ACE *ad_ace;
         const SID *sid;
+
+        if (ace->AceFlags & INHERIT_ONLY_ACE)
+            continue;
+
         switch (ace->AceType)
         {
         case ACCESS_DENIED_ACE_TYPE:
@@ -962,8 +942,6 @@ static unsigned int token_access_check( struct token *token,
             * rights we need */
         if (desired_access == *granted_access)
             break;
-
-        ace = ace_next( ace );
     }
 
 done:
@@ -984,91 +962,14 @@ const ACL *token_get_default_dacl( struct token *token )
     return token->default_dacl;
 }
 
-static void set_object_sd( struct object *obj, const struct security_descriptor *sd,
-                           unsigned int set_info )
+const SID *token_get_user( struct token *token )
 {
-    struct security_descriptor new_sd, *pnew_sd;
-    int present;
-    const SID *owner, *group;
-    const ACL *sacl, *dacl;
-    char *ptr;
+    return token->user;
+}
 
-    if (!set_info) return;
-
-    new_sd.control = sd->control & ~SE_SELF_RELATIVE;
-
-    owner = sd_get_owner( sd );
-    if (set_info & OWNER_SECURITY_INFORMATION && owner)
-        new_sd.owner_len = sd->owner_len;
-    else
-    {
-        owner = current->process->token->user;
-        new_sd.owner_len = FIELD_OFFSET(SID, SubAuthority[owner->SubAuthorityCount]);
-        new_sd.control |= SE_OWNER_DEFAULTED;
-    }
-
-    group = sd_get_group( sd );
-    if (set_info & GROUP_SECURITY_INFORMATION && group)
-        new_sd.group_len = sd->group_len;
-    else
-    {
-        group = current->process->token->primary_group;
-        new_sd.group_len = FIELD_OFFSET(SID, SubAuthority[group->SubAuthorityCount]);
-        new_sd.control |= SE_GROUP_DEFAULTED;
-    }
-
-    new_sd.control |= SE_SACL_PRESENT;
-    sacl = sd_get_sacl( sd, &present );
-    if (set_info & SACL_SECURITY_INFORMATION && present)
-        new_sd.sacl_len = sd->sacl_len;
-    else
-    {
-        if (obj->sd) sacl = sd_get_sacl( obj->sd, &present );
-
-        if (obj->sd && present)
-            new_sd.sacl_len = obj->sd->sacl_len;
-        else
-        {
-            new_sd.sacl_len = 0;
-            new_sd.control |= SE_SACL_DEFAULTED;
-        }
-    }
-
-    new_sd.control |= SE_DACL_PRESENT;
-    dacl = sd_get_dacl( sd, &present );
-    if (set_info & DACL_SECURITY_INFORMATION && present)
-        new_sd.dacl_len = sd->dacl_len;
-    else
-    {
-        if (obj->sd) dacl = sd_get_dacl( obj->sd, &present );
-
-        if (obj->sd && present)
-            new_sd.dacl_len = obj->sd->dacl_len;
-        else
-        {
-            dacl = token_get_default_dacl( current->process->token );
-            new_sd.dacl_len = dacl->AclSize;
-            new_sd.control |= SE_DACL_DEFAULTED;
-        }
-    }
-
-    ptr = mem_alloc( sizeof(new_sd) + new_sd.owner_len + new_sd.group_len +
-                     new_sd.sacl_len + new_sd.dacl_len );
-    if (!ptr) return;
-    pnew_sd = (struct security_descriptor*)ptr;
-
-    memcpy( ptr, &new_sd, sizeof(new_sd) );
-    ptr += sizeof(new_sd);
-    memcpy( ptr, owner, new_sd.owner_len );
-    ptr += new_sd.owner_len;
-    memcpy( ptr, group, new_sd.group_len );
-    ptr += new_sd.group_len;
-    memcpy( ptr, sacl, new_sd.sacl_len );
-    ptr += new_sd.sacl_len;
-    memcpy( ptr, dacl, new_sd.dacl_len );
-
-    free( obj->sd );
-    obj->sd = pnew_sd;
+const SID *token_get_primary_group( struct token *token )
+{
+    return token->primary_group;
 }
 
 int check_object_access(struct object *obj, unsigned int *access)
@@ -1111,7 +1012,7 @@ DECL_HANDLER(open_token)
         {
             if (thread->token)
             {
-                if (thread->token->impersonation_level <= SecurityAnonymous)
+                if (!thread->token->primary && thread->token->impersonation_level <= SecurityAnonymous)
                     set_error( STATUS_CANT_OPEN_ANONYMOUS );
                 else
                     reply->token = alloc_handle( current->process, thread->token,
@@ -1393,6 +1294,7 @@ DECL_HANDLER(get_token_groups)
                     if (group->owner) *attr_ptr |= SE_GROUP_OWNER;
                     if (group->deny_only) *attr_ptr |= SE_GROUP_USE_FOR_DENY_ONLY;
                     if (group->resource) *attr_ptr |= SE_GROUP_RESOURCE;
+                    if (group->logon) *attr_ptr |= SE_GROUP_LOGON_ID;
 
                     memcpy(sid_ptr, &group->sid, FIELD_OFFSET(SID, SubAuthority[group->sid.SubAuthorityCount]));
 
@@ -1442,30 +1344,3 @@ DECL_HANDLER(get_token_statistics)
         release_object( token );
     }
 }
-
-DECL_HANDLER(set_security_object)
-{
-    data_size_t sd_size = get_req_data_size();
-    const struct security_descriptor *sd = get_req_data();
-    struct object *obj;
-    unsigned int access = 0;
-
-    if (!sd_is_valid( sd, sd_size ))
-    {
-        set_error( STATUS_ACCESS_VIOLATION );
-        return;
-    }
-
-    if (req->security_info & OWNER_SECURITY_INFORMATION ||
-        req->security_info & GROUP_SECURITY_INFORMATION)
-        access |= WRITE_OWNER;
-    if (req->security_info & SACL_SECURITY_INFORMATION)
-        access |= ACCESS_SYSTEM_SECURITY;
-    if (req->security_info & DACL_SECURITY_INFORMATION)
-        access |= WRITE_DAC;
-
-    if (!(obj = get_handle_obj( current->process, req->handle, access, NULL ))) return;
-
-    set_object_sd( obj, sd, req->security_info );
-    release_object( obj );
-}
