@@ -26,11 +26,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 #include <string.h>
-#include <assert.h>
 #include <ctype.h>
-#include <signal.h>
 
 #define NONAMELESSUNION
 #define NONAMELESSSTRUCT
@@ -52,7 +52,7 @@ static typelib_t *typelib;
 
 type_t *duptype(type_t *t, int dupname)
 {
-  type_t *d = xmalloc(sizeof *d);
+  type_t *d = alloc_type();
 
   *d = *t;
   if (dupname && t->name)
@@ -252,6 +252,9 @@ void start_typelib(char *name, attr_list_t *attrs)
     typelib->attrs = attrs;
     list_init( &typelib->entries );
     list_init( &typelib->importlibs );
+
+    if (is_attr(attrs, ATTR_POINTERDEFAULT))
+        pointer_default = get_attrv(attrs, ATTR_POINTERDEFAULT);
 }
 
 void end_typelib(void)
@@ -260,6 +263,7 @@ void end_typelib(void)
     if (!typelib) return;
 
     create_msft_typelib(typelib);
+    pointer_default = RPC_FC_UP;
     return;
 }
 
