@@ -22,6 +22,7 @@
 #include "config.h"
 #include "wine/port.h"
 
+#include <errno.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -51,14 +52,17 @@
 
 static const char usage[] =
 "Usage: widl [options...] infile.idl\n"
+"   or: widl [options...] --dlldata-only name1 [name2...]\n"
 "   -c          Generate client stub\n"
 "   -C file     Name of client stub file (default is infile_c.c)\n"
 "   -d n        Set debug level to 'n'\n"
 "   -D id[=val] Define preprocessor identifier id=val\n"
+"   --dlldata=file  Name of the dlldata file (default is dlldata.c)\n"
 "   -E          Preprocess only\n"
 "   -h          Generate headers\n"
 "   -H file     Name of header file (default is infile.h)\n"
 "   -I path     Set include search dir to path (multiple -I allowed)\n"
+"   --local-stubs=file  Write empty stubs for call_as/local methods to file\n"
 "   -N          Do not preprocess input\n"
 "   --oldnames  Use old naming conventions\n"
 "   -p          Generate proxy\n"
@@ -99,13 +103,16 @@ int do_proxies = 0;
 int do_client = 0;
 int do_server = 0;
 int do_idfile = 0;
+int do_dlldata = 0;
 int no_preprocess = 0;
 int old_names = 0;
 
 char *input_name;
 char *header_name;
+char *local_stubs_name;
 char *header_token;
 char *typelib_name;
+char *dlldata_name;
 char *proxy_name;
 char *proxy_token;
 char *client_name;
@@ -121,6 +128,7 @@ const char *prefix_server = "";
 int line_number = 1;
 
 FILE *header;
+FILE *local_stubs;
 FILE *proxy;
 FILE *idfile;
 
@@ -128,6 +136,9 @@ time_t now;
 
 enum {
     OLDNAMES_OPTION = CHAR_MAX + 1,
+    DLLDATA_OPTION,
+    DLLDATA_ONLY_OPTION,
+    LOCAL_STUBS_OPTION,
     PREFIX_ALL_OPTION,
     PREFIX_CLIENT_OPTION,
     PREFIX_SERVER_OPTION
@@ -136,10 +147,13 @@ enum {
 static const char short_options[] =
     "cC:d:D:EhH:I:NpP:sS:tT:uU:VW";
 static const struct option long_options[] = {
-    { "oldnames", no_argument, 0, OLDNAMES_OPTION },
-    { "prefix-all", required_argument, 0, PREFIX_ALL_OPTION },
-    { "prefix-client", required_argument, 0, PREFIX_CLIENT_OPTION },
-    { "prefix-server", required_argument, 0, PREFIX_SERVER_OPTION },
+    { "dlldata", 1, 0, DLLDATA_OPTION },
+    { "dlldata-only", 0, 0, DLLDATA_ONLY_OPTION },
+    { "local-stubs", 1, 0, LOCAL_STUBS_OPTION },
+    { "oldnames", 0, 0, OLDNAMES_OPTION },
+    { "prefix-all", 1, 0, PREFIX_ALL_OPTION },
+    { "prefix-client", 1, 0, PREFIX_CLIENT_OPTION },
+    { "prefix-server", 1, 0, PREFIX_SERVER_OPTION },
     { 0, 0, 0, 0 }
 };
 
@@ -152,6 +166,9 @@ static char *make_token(const char *name)
   int i;
 
   slash = strrchr(name, '/');
+  if(!slash)
+    slash = strrchr(name, '\\');
+
   if (slash) name = slash + 1;
 
   token = xstrdup(name);
@@ -177,6 +194,145 @@ static void exit_on_signal( int sig )
     exit(1);  /* this will call the atexit functions */
 }
 
+static void set_everything(int x)
+{
+  do_header = x;
+  do_typelib = x;
+  do_proxies = x;
+  do_client = x;
+  do_server = x;
+  do_idfile = x;
+  do_dlldata = x;
+}
+
+static void start_cplusplus_guard(FILE *fp)
+{
+  fprintf(fp, "#ifdef __cplusplus\n");
+  fprintf(fp, "extern \"C\" {\n");
+  fprintf(fp, "#endif\n\n");
+}
+
+static void end_cplusplus_guard(FILE *fp)
+{
+  fprintf(fp, "#ifdef __cplusplus\n");
+  fprintf(fp, "}\n");
+  fprintf(fp, "#endif\n\n");
+}
+
+typedef struct
+{
+  char *filename;
+  struct list link;
+} filename_node_t;
+
+static void add_filename_node(struct list *list, const char *name)
+{
+  filename_node_t *node = xmalloc(sizeof *node);
+  node->filename = dup_basename( name, ".idl" );
+  list_add_tail(list, &node->link);
+}
+
+static void free_filename_nodes(struct list *list)
+{
+  filename_node_t *node, *next;
+  LIST_FOR_EACH_ENTRY_SAFE(node, next, list, filename_node_t, link) {
+    list_remove(&node->link);
+    free(node->filename);
+    free(node);
+  }
+}
+
+static void write_dlldata_list(struct list *filenames)
+{
+  FILE *dlldata;
+  filename_node_t *node;
+
+  dlldata = fopen(dlldata_name, "w");
+  if (!dlldata)
+    error("couldn't open %s: %s\n", dlldata_name, strerror(errno));
+
+  fprintf(dlldata, "/*** Autogenerated by WIDL %s ", PACKAGE_VERSION);
+  fprintf(dlldata, "- Do not edit ***/\n\n");
+  fprintf(dlldata, "#include <objbase.h>\n");
+  fprintf(dlldata, "#include <rpcproxy.h>\n\n");
+  start_cplusplus_guard(dlldata);
+
+  LIST_FOR_EACH_ENTRY(node, filenames, filename_node_t, link)
+    fprintf(dlldata, "EXTERN_PROXY_FILE(%s)\n", node->filename);
+
+  fprintf(dlldata, "\nPROXYFILE_LIST_START\n");
+  fprintf(dlldata, "/* Start of list */\n");
+  LIST_FOR_EACH_ENTRY(node, filenames, filename_node_t, link)
+    fprintf(dlldata, "  REFERENCE_PROXY_FILE(%s),\n", node->filename);
+  fprintf(dlldata, "/* End of list */\n");
+  fprintf(dlldata, "PROXYFILE_LIST_END\n\n");
+
+  fprintf(dlldata, "DLLDATA_ROUTINES(aProxyFileList, GET_DLL_CLSID)\n\n");
+  end_cplusplus_guard(dlldata);
+  fclose(dlldata);
+}
+
+static char *eat_space(char *s)
+{
+  while (isspace((unsigned char) *s))
+    ++s;
+  return s;
+}
+
+void write_dlldata(ifref_list_t *ifaces)
+{
+  struct list filenames = LIST_INIT(filenames);
+  filename_node_t *node;
+  FILE *dlldata;
+
+  if (!do_dlldata || !need_proxy_file(ifaces))
+    return;
+
+  dlldata = fopen(dlldata_name, "r");
+  if (dlldata) {
+    static char marker[] = "REFERENCE_PROXY_FILE";
+    char *line = NULL;
+    size_t len = 0;
+
+    while (widl_getline(&line, &len, dlldata)) {
+      char *start, *end;
+      start = eat_space(line);
+      if (strncmp(start, marker, sizeof marker - 1) == 0) {
+        start = eat_space(start + sizeof marker - 1);
+        if (*start != '(')
+          continue;
+        end = start = eat_space(start + 1);
+        while (*end && *end != ')')
+          ++end;
+        if (*end != ')')
+          continue;
+        while (isspace((unsigned char) end[-1]))
+          --end;
+        *end = '\0';
+        if (start < end)
+          add_filename_node(&filenames, start);
+      }
+    }
+
+    if (ferror(dlldata))
+      error("couldn't read from %s: %s\n", dlldata_name, strerror(errno));
+
+    free(line);
+    fclose(dlldata);
+  }
+
+  LIST_FOR_EACH_ENTRY(node, &filenames, filename_node_t, link)
+    if (strcmp(proxy_token, node->filename) == 0) {
+      /* We're already in the list, no need to regenerate this file.  */
+      free_filename_nodes(&filenames);
+      return;
+    }
+
+  add_filename_node(&filenames, proxy_token);
+  write_dlldata_list(&filenames);
+  free_filename_nodes(&filenames);
+}
+
 int main(int argc,char *argv[])
 {
   extern char* optarg;
@@ -195,6 +351,17 @@ int main(int argc,char *argv[])
 
   while((optc = getopt_long(argc, argv, short_options, long_options, &opti)) != EOF) {
     switch(optc) {
+    case DLLDATA_OPTION:
+      dlldata_name = xstrdup(optarg);
+      break;
+    case DLLDATA_ONLY_OPTION:
+      do_everything = 0;
+      do_dlldata = 1;
+      break;
+    case LOCAL_STUBS_OPTION:
+      do_everything = 0;
+      local_stubs_name = xstrdup(optarg);
+      break;
     case OLDNAMES_OPTION:
       old_names = 1;
       break;
@@ -267,25 +434,43 @@ int main(int argc,char *argv[])
       idfile_name = xstrdup(optarg);
       break;
     case 'V':
-      printf(version_string);
+      printf("%s", version_string);
       return 0;
     case 'W':
       pedantic = 1;
       break;
     default:
-      fprintf(stderr, usage);
+      fprintf(stderr, "%s", usage);
       return 1;
     }
   }
 
   if(do_everything) {
-      do_header = do_typelib = do_proxies = do_client = do_server = do_idfile = 1;
+    set_everything(TRUE);
   }
+
+  if (!dlldata_name && do_dlldata)
+    dlldata_name = xstrdup("dlldata.c");
+
   if(optind < argc) {
-    input_name = xstrdup(argv[optind]);
+    if (do_dlldata && !do_everything) {
+      struct list filenames = LIST_INIT(filenames);
+      for ( ; optind < argc; ++optind)
+        add_filename_node(&filenames, argv[optind]);
+
+      write_dlldata_list(&filenames);
+      free_filename_nodes(&filenames);
+      return 0;
+    }
+    else if (optind != argc - 1) {
+      fprintf(stderr, "%s", usage);
+      return 1;
+    }
+    else
+      input_name = xstrdup(argv[optind]);
   }
   else {
-    fprintf(stderr, usage);
+    fprintf(stderr, "%s", usage);
     return 1;
   }
 
@@ -378,9 +563,18 @@ int main(int argc,char *argv[])
     fprintf(header, "#include <rpcndr.h>\n\n" );
     fprintf(header, "#ifndef __WIDL_%s\n", header_token);
     fprintf(header, "#define __WIDL_%s\n", header_token);
-    fprintf(header, "#ifdef __cplusplus\n");
-    fprintf(header, "extern \"C\" {\n");
-    fprintf(header, "#endif\n");
+    start_cplusplus_guard(header);
+  }
+
+  if (local_stubs_name) {
+    local_stubs = fopen(local_stubs_name, "w");
+    if (!local_stubs) {
+      fprintf(stderr, "Could not open %s for output\n", local_stubs_name);
+      return 1;
+    }
+    fprintf(local_stubs, "/* call_as/local stubs for %s */\n\n", input_name);
+    fprintf(local_stubs, "#include <objbase.h>\n");
+    fprintf(local_stubs, "#include \"%s\"\n\n", header_name);
   }
 
   if (do_idfile) {
@@ -397,9 +591,7 @@ int main(int argc,char *argv[])
     fprintf(idfile, "#include <rpc.h>\n");
     fprintf(idfile, "#include <rpcndr.h>\n\n");
     fprintf(idfile, "#include <initguid.h>\n\n");
-    fprintf(idfile, "#ifdef __cplusplus\n");
-    fprintf(idfile, "extern \"C\" {\n");
-    fprintf(idfile, "#endif\n\n");
+    start_cplusplus_guard(idfile);
   }
 
   init_types();
@@ -409,22 +601,23 @@ int main(int argc,char *argv[])
     fprintf(header, "/* Begin additional prototypes for all interfaces */\n");
     fprintf(header, "\n");
     write_user_types();
+    write_generic_handle_routines();
     write_context_handle_rundowns();
     fprintf(header, "\n");
     fprintf(header, "/* End additional prototypes */\n");
     fprintf(header, "\n");
-    fprintf(header, "#ifdef __cplusplus\n");
-    fprintf(header, "}\n");
-    fprintf(header, "#endif\n");
+    end_cplusplus_guard(header);
     fprintf(header, "#endif /* __WIDL_%s */\n", header_token);
     fclose(header);
   }
 
+  if (local_stubs) {
+    fclose(local_stubs);
+  }
+
   if (do_idfile) {
     fprintf(idfile, "\n");
-    fprintf(idfile, "#ifdef __cplusplus\n");
-    fprintf(idfile, "}\n");
-    fprintf(idfile, "#endif\n");
+    end_cplusplus_guard(idfile);
 
     fclose(idfile);
   }
@@ -434,10 +627,11 @@ int main(int argc,char *argv[])
   if(ret) {
     exit(1);
   }
-  header_name = NULL;
-  client_name = NULL;
-  server_name = NULL;
-  idfile_name = NULL;
+
+  /* Everything has been done successfully, don't delete any files.  */
+  set_everything(FALSE);
+  local_stubs_name = NULL;
+
   return 0;
 }
 
@@ -446,10 +640,18 @@ static void rm_tempfile(void)
   abort_import();
   if(temp_name)
     unlink(temp_name);
-  if (header_name)
+  if (do_header)
     unlink(header_name);
-  if (client_name)
+  if (local_stubs_name)
+    unlink(local_stubs_name);
+  if (do_client)
     unlink(client_name);
-  if (server_name)
+  if (do_server)
     unlink(server_name);
+  if (do_idfile)
+    unlink(idfile_name);
+  if (do_proxies)
+    unlink(proxy_name);
+  if (do_typelib)
+    unlink(typelib_name);
 }
