@@ -1,14 +1,27 @@
 /*
- * Copyrignt 1998 Bertho A. Stultiens (BS)
+ * Copyright 1998 Bertho A. Stultiens (BS)
  *
- * 16-Apr-1998 BS	- Yeah, dump it to stdout.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#include "config.h"
+
+#include <assert.h>
 #include <stdio.h>
 #include <ctype.h>
 
-#include <config.h>
 #include "wrc.h"
 #include "dumpres.h"
 
@@ -23,7 +36,7 @@
  * Remarks	:
  *****************************************************************************
 */
-char *get_typename(resource_t* r)
+const char *get_typename(const resource_t* r)
 {
 	switch(r->type){
 	case res_acc:	return "ACCELERATOR";
@@ -42,6 +55,10 @@ char *get_typename(resource_t* r)
 	case res_usr:   return "UserResource";
 	case res_msg:	return "MESSAGETABLE";
 	case res_ver:	return "VERSIONINFO";
+	case res_dlginit: return "DLGINIT";
+	case res_toolbar: return "TOOLBAR";
+	case res_anicur:  return "CURSOR (animated)";
+	case res_aniico:  return "ICON (animated)";
 	default: 	return "Unknown";
 	}
 }
@@ -56,20 +73,20 @@ char *get_typename(resource_t* r)
  *	maxlen	- Max chars to copy
  * Output	: 'cs'
  * Description	: Copy a unicode string to ascii. Copying stops after the
- *		  first occuring '\0' or when maxlen-1 chars are copied. The
+ *		  first occurring '\0' or when maxlen-1 chars are copied. The
  *		  String is always nul terminated.
  * Remarks	: No codepage translation is done.
  *****************************************************************************
 */
-char *strncpyWtoA(char *cs, short *ws, int maxlen)
+static char *strncpyWtoA(char *cs, const WCHAR *ws, int maxlen)
 {
 	char *cptr = cs;
-	while(*ws && maxlen-1)
+	const WCHAR *wsMax = ws + maxlen - 1;
+	while(*ws && ws < wsMax)
 	{
-		if(*ws < -128 || *ws > 127)
-			printf("***Warning: Unicode string contains non-printable chars***");
+		if(*ws > 255)
+			fprintf(stderr, "***Warning: Unicode string contains non-printable chars***\n");
 		*cptr++ = (char)*ws++;
-		maxlen--;
 	}
 	*cptr = '\0';
 	return cs;
@@ -85,7 +102,7 @@ char *strncpyWtoA(char *cs, short *ws, int maxlen)
  * Remarks	:
  *****************************************************************************
 */
-void print_string(string_t *str)
+static void print_string(const string_t *str)
 {
 	char buffer[512];
 	if(!str)
@@ -102,7 +119,7 @@ void print_string(string_t *str)
 /*
  *****************************************************************************
  * Function	: get_nameid_str
- * Syntax	: char *get_nameid_str(name_id_t *n)
+ * Syntax	: const char *get_nameid_str(const name_id_t *n)
  * Input	:
  *	n	- nameid to convert to text
  * Output	: A pointer to the name.
@@ -110,7 +127,7 @@ void print_string(string_t *str)
  * Remarks	: Not reentrant because of static buffer
  *****************************************************************************
 */
-char *get_nameid_str(name_id_t *n)
+const char *get_nameid_str(const name_id_t *n)
 {
 	static char buffer[256];
 
@@ -147,7 +164,7 @@ char *get_nameid_str(name_id_t *n)
  * Remarks	:
  *****************************************************************************
 */
-void dump_memopt(DWORD memopt)
+static void dump_memopt(DWORD memopt)
 {
 	printf("Memory/load options: ");
 	if(memopt & 0x0040)
@@ -170,7 +187,7 @@ void dump_memopt(DWORD memopt)
 /*
  *****************************************************************************
  * Function	: dump_lvc
- * Syntax	: void dump_lvc(lvc_t *l)
+ * Syntax	: void dump_lvc(const lvc_t *l)
  * Input	:
  *	l	- pointer to lvc structure
  * Output	:
@@ -178,7 +195,7 @@ void dump_memopt(DWORD memopt)
  * Remarks	:
  *****************************************************************************
 */
-void dump_lvc(lvc_t *l)
+static void dump_lvc(const lvc_t *l)
 {
 	if(l->language)
 		printf("LANGUAGE %04x, %04x\n", l->language->id, l->language->sub);
@@ -186,12 +203,12 @@ void dump_lvc(lvc_t *l)
 		printf("LANGUAGE <not set>\n");
 
 	if(l->version)
-		printf("VERSION %08lx\n", *(l->version));
+		printf("VERSION %08x\n", *(l->version));
 	else
 		printf("VERSION <not set>\n");
 
 	if(l->characts)
-		printf("CHARACTERISTICS %08lx\n", *(l->characts));
+		printf("CHARACTERISTICS %08x\n", *(l->characts));
 	else
 		printf("CHARACTERISTICS <not set>\n");
 }
@@ -199,7 +216,7 @@ void dump_lvc(lvc_t *l)
 /*
  *****************************************************************************
  * Function	: dump_raw_data
- * Syntax	: void dump_raw_data(raw_data_t *d)
+ * Syntax	: void dump_raw_data(const raw_data_t *d)
  * Input	:
  *	d	- Raw data descriptor
  * Output	:
@@ -207,9 +224,9 @@ void dump_lvc(lvc_t *l)
  * Remarks	:
  *****************************************************************************
 */
-void dump_raw_data(raw_data_t *d)
+static void dump_raw_data(const raw_data_t *d)
 {
-	int n;
+	unsigned int n;
 	int i;
 	int j;
 
@@ -225,15 +242,17 @@ void dump_raw_data(raw_data_t *d)
 	for(n = 0; n < d->size; n++)
 	{
 		if((n % 16) == 0)
+                {
 			if(n)
 			{
 				printf("- ");
 				for(i = 0; i < 16; i++)
-					printf("%c", isprint(d->data[n-16+i]) ? d->data[n-16+i] : '.');
+					printf("%c", isprint(d->data[n-16+i] & 0xff) ? d->data[n-16+i] : '.');
 				printf("\n%08x: ", n);
 			}
 			else
 				printf("%08x: ", n);
+                }
 		printf("%02x ", d->data[n] & 0xff);
 	}
 	printf("- ");
@@ -241,14 +260,14 @@ void dump_raw_data(raw_data_t *d)
 	if(!j)
 		j = 16;
 	for(i = 0; i < j; i++)
-		printf("%c", isprint(d->data[n-j+i]) ? d->data[n-j+i] : '.');
+		printf("%c", isprint(d->data[n-j+i] & 0xff) ? d->data[n-j+i] : '.');
 	printf("\n");
 }
 
 /*
  *****************************************************************************
  * Function	: dump_accelerator
- * Syntax	: void dump_accelerator(resource_t *acc)
+ * Syntax	: void dump_accelerator(const accelerator_t *acc)
  * Input	:
  *	acc	- Accelerator resource descriptor
  * Output	: nop
@@ -256,7 +275,7 @@ void dump_raw_data(raw_data_t *d)
  * Remarks	:
  *****************************************************************************
 */
-void dump_accelerator(accelerator_t *acc)
+static void dump_accelerator(const accelerator_t *acc)
 {
 	event_t *ev = acc->events;
 
@@ -282,7 +301,7 @@ void dump_accelerator(accelerator_t *acc)
 /*
  *****************************************************************************
  * Function	: dump_cursor
- * Syntax	: void dump_cursor(cursor_t *cur)
+ * Syntax	: void dump_cursor(const cursor_t *cur)
  * Input	:
  *	cur	- Cursor resource descriptor
  * Output	: nop
@@ -290,7 +309,7 @@ void dump_accelerator(accelerator_t *acc)
  * Remarks	:
  *****************************************************************************
 */
-void dump_cursor(cursor_t *cur)
+static void dump_cursor(const cursor_t *cur)
 {
 	printf("Id: %d\n", cur->id);
 	printf("Width: %d\n", cur->width);
@@ -303,7 +322,7 @@ void dump_cursor(cursor_t *cur)
 /*
  *****************************************************************************
  * Function	: dump_cursor_group
- * Syntax	: void dump_cursor_group(cursor_group_t *cur)
+ * Syntax	: void dump_cursor_group(const cursor_group_t *cur)
  * Input	:
  *	cur	- Cursor group resource descriptor
  * Output	: nop
@@ -311,7 +330,7 @@ void dump_cursor(cursor_t *cur)
  * Remarks	:
  *****************************************************************************
 */
-void dump_cursor_group(cursor_group_t *curg)
+static void dump_cursor_group(const cursor_group_t *curg)
 {
 	dump_memopt(curg->memopt);
 	printf("There are %d cursors in this group\n", curg->ncursor);
@@ -320,7 +339,7 @@ void dump_cursor_group(cursor_group_t *curg)
 /*
  *****************************************************************************
  * Function	: dump_icon
- * Syntax	: void dump_icon(icon_t *ico)
+ * Syntax	: void dump_icon(const icon_t *ico)
  * Input	:
  *	ico	- Icon resource descriptor
  * Output	: nop
@@ -328,7 +347,7 @@ void dump_cursor_group(cursor_group_t *curg)
  * Remarks	:
  *****************************************************************************
 */
-void dump_icon(icon_t *ico)
+static void dump_icon(const icon_t *ico)
 {
 	printf("Id: %d\n", ico->id);
 	printf("Width: %d\n", ico->width);
@@ -342,7 +361,7 @@ void dump_icon(icon_t *ico)
 /*
  *****************************************************************************
  * Function	: dump_icon_group
- * Syntax	: void dump_icon_group(icon_group_t *ico)
+ * Syntax	: void dump_icon_group(const icon_group_t *ico)
  * Input	:
  *	ico	- Icon group resource descriptor
  * Output	: nop
@@ -350,16 +369,34 @@ void dump_icon(icon_t *ico)
  * Remarks	:
  *****************************************************************************
 */
-void dump_icon_group(icon_group_t *icog)
+static void dump_icon_group(const icon_group_t *icog)
 {
 	dump_memopt(icog->memopt);
 	printf("There are %d icons in this group\n", icog->nicon);
 }
 
+/*
+ *****************************************************************************
+ * Function	: dump_ani_curico
+ * Syntax	: void dump_ani_curico(const ani_curico_t *ani)
+ * Input	:
+ *	ani	- Animated object resource descriptor
+ * Output	: nop
+ * Description	:
+ * Remarks	:
+ *****************************************************************************
+*/
+static void dump_ani_curico(const ani_curico_t *ani)
+{
+	dump_memopt(ani->memopt);
+	dump_lvc(&ani->data->lvc);
+	dump_raw_data(ani->data);
+}
+
 /*
  *****************************************************************************
  * Function	: dump_font
- * Syntax	: void dump_font(font_t *fnt)
+ * Syntax	: void dump_font(const font_t *fnt)
  * Input	:
  *	fnt	- Font resource descriptor
  * Output	: nop
@@ -367,16 +404,17 @@ void dump_icon_group(icon_group_t *icog)
  * Remarks	:
  *****************************************************************************
 */
-void dump_font(font_t *fnt)
+static void dump_font(const font_t *fnt)
 {
 	dump_memopt(fnt->memopt);
+	dump_lvc(&(fnt->data->lvc));
 	dump_raw_data(fnt->data);
 }
 
 /*
  *****************************************************************************
  * Function	: dump_bitmap
- * Syntax	: void dump_bitmap(bitmap_t *bmp)
+ * Syntax	: void dump_bitmap(const bitmap_t *bmp)
  * Input	:
  *	bmp	- Bitmap resource descriptor
  * Output	: nop
@@ -384,16 +422,17 @@ void dump_font(font_t *fnt)
  * Remarks	:
  *****************************************************************************
 */
-void dump_bitmap(bitmap_t *bmp)
+static void dump_bitmap(const bitmap_t *bmp)
 {
 	dump_memopt(bmp->memopt);
+	dump_lvc(&(bmp->data->lvc));
 	dump_raw_data(bmp->data);
 }
 
 /*
  *****************************************************************************
  * Function	: dump_rcdata
- * Syntax	: void dump_rcdata(rcdata_t *rdt)
+ * Syntax	: void dump_rcdata(const rcdata_t *rdt)
  * Input	:
  *	rdt	- RCData resource descriptor
  * Output	: nop
@@ -401,16 +440,17 @@ void dump_bitmap(bitmap_t *bmp)
  * Remarks	:
  *****************************************************************************
 */
-void dump_rcdata(rcdata_t *rdt)
+static void dump_rcdata(const rcdata_t *rdt)
 {
 	dump_memopt(rdt->memopt);
+	dump_lvc(&(rdt->data->lvc));
 	dump_raw_data(rdt->data);
 }
 
 /*
  *****************************************************************************
  * Function	: dump_user
- * Syntax	: void dump_user(user_t *usr)
+ * Syntax	: void dump_user(const user_t *usr)
  * Input	:
  *	usr	- User resource descriptor
  * Output	: nop
@@ -418,9 +458,10 @@ void dump_rcdata(rcdata_t *rdt)
  * Remarks	:
  *****************************************************************************
 */
-void dump_user(user_t *usr)
+static void dump_user(const user_t *usr)
 {
 	dump_memopt(usr->memopt);
+	dump_lvc(&(usr->data->lvc));
 	printf("Class %s\n", get_nameid_str(usr->type));
 	dump_raw_data(usr->data);
 }
@@ -428,7 +469,7 @@ void dump_user(user_t *usr)
 /*
  *****************************************************************************
  * Function	: dump_messagetable
- * Syntax	: void dump_messagetable(messagetable_t *msg)
+ * Syntax	: void dump_messagetable(const messagetable_t *msg)
  * Input	:
  *	msg	- Messagetable resource descriptor
  * Output	: nop
@@ -436,15 +477,17 @@ void dump_user(user_t *usr)
  * Remarks	:
  *****************************************************************************
 */
-void dump_messagetable(messagetable_t *msg)
+static void dump_messagetable(const messagetable_t *msg)
 {
+	dump_memopt(msg->memopt);
+	dump_lvc(&(msg->data->lvc));
 	dump_raw_data(msg->data);
 }
 
 /*
  *****************************************************************************
  * Function	: dump_stringtable
- * Syntax	: void dump_stringtable(stringtable_t *stt)
+ * Syntax	: void dump_stringtable(const stringtable_t *stt)
  * Input	:
  *	stt	- Stringtable resource descriptor
  * Output	: nop
@@ -452,7 +495,7 @@ void dump_messagetable(messagetable_t *msg)
  * Remarks	:
  *****************************************************************************
 */
-void dump_stringtable(stringtable_t *stt)
+static void dump_stringtable(const stringtable_t *stt)
 {
 	int i;
 	for(; stt; stt = stt->next)
@@ -476,7 +519,7 @@ void dump_stringtable(stringtable_t *stt)
 /*
  *****************************************************************************
  * Function	: dump_control
- * Syntax	: void dump_control(control_t *ctrl)
+ * Syntax	: void dump_control(const control_t *ctrl)
  * Input	:
  *	ctrl	- Control resource descriptor
  * Output	:
@@ -484,18 +527,26 @@ void dump_stringtable(stringtable_t *stt)
  * Remarks	:
  *****************************************************************************
 */
-void dump_control(control_t *ctrl)
+static void dump_control(const control_t *ctrl)
 {
 	printf("Control {\n\tClass: %s\n", get_nameid_str(ctrl->ctlclass));
-	printf("\tText: "); print_string(ctrl->title); printf("\n");
+	printf("\tText: "); get_nameid_str(ctrl->title); printf("\n");
 	printf("\tId: %d\n", ctrl->id);
 	printf("\tx, y, w, h: %d, %d, %d, %d\n", ctrl->x, ctrl->y, ctrl->width, ctrl->height);
 	if(ctrl->gotstyle)
-		printf("\tStyle: %08lx\n", ctrl->style);
+	{
+		assert(ctrl->style != NULL);
+		assert(ctrl->style->and_mask == 0);
+		printf("\tStyle: %08x\n", ctrl->style->or_mask);
+	}
 	if(ctrl->gotexstyle)
-		printf("\tExStyle: %08lx\n", ctrl->exstyle);
+	{
+		assert(ctrl->exstyle != NULL);
+		assert(ctrl->exstyle->and_mask == 0);
+		printf("\tExStyle: %08x\n", ctrl->exstyle->or_mask);
+	}
 	if(ctrl->gothelpid)
-		printf("\tHelpid: %ld\n", ctrl->helpid);
+		printf("\tHelpid: %d\n", ctrl->helpid);
 	if(ctrl->extra)
 	{
 		printf("\t");
@@ -507,7 +558,7 @@ void dump_control(control_t *ctrl)
 /*
  *****************************************************************************
  * Function	: dump_dialog
- * Syntax	: void dump_dialog(dialog_t *dlg)
+ * Syntax	: void dump_dialog(const dialog_t *dlg)
  * Input	:
  *	dlg	- Dialog resource descriptor
  * Output	:
@@ -515,7 +566,7 @@ void dump_control(control_t *ctrl)
  * Remarks	:
  *****************************************************************************
 */
-void dump_dialog(dialog_t *dlg)
+static void dump_dialog(const dialog_t *dlg)
 {
 	control_t *c = dlg->controls;
 
@@ -523,9 +574,18 @@ void dump_dialog(dialog_t *dlg)
 	dump_lvc(&(dlg->lvc));
 	printf("x, y, w, h: %d, %d, %d, %d\n", dlg->x, dlg->y, dlg->width, dlg->height);
 	if(dlg->gotstyle)
-		printf("Style: %08lx\n", dlg->style);
+	{
+		assert(dlg->style != NULL);
+		assert(dlg->style->and_mask == 0);
+		printf("Style: %08x\n", dlg->style->or_mask);
+
+	}
 	if(dlg->gotexstyle)
-		printf("ExStyle: %08lx\n", dlg->exstyle);
+	{
+		assert(dlg->exstyle != NULL);
+		assert(dlg->exstyle->and_mask == 0);
+		printf("ExStyle: %08x\n", dlg->exstyle->or_mask);
+	}
 	printf("Menu: %s\n", get_nameid_str(dlg->menu));
 	printf("Class: %s\n", get_nameid_str(dlg->dlgclass));
 	printf("Title: "); print_string(dlg->title); printf("\n");
@@ -548,7 +608,7 @@ void dump_dialog(dialog_t *dlg)
 /*
  *****************************************************************************
  * Function	: dump_dialogex
- * Syntax	: void dump_dialogex(dialogex_t *dlgex)
+ * Syntax	: void dump_dialogex(const dialogex_t *dlgex)
  * Input	:
  *	dlgex	- DialogEx resource descriptor
  * Output	:
@@ -556,19 +616,27 @@ void dump_dialog(dialog_t *dlg)
  * Remarks	:
  *****************************************************************************
 */
-void dump_dialogex(dialogex_t *dlgex)
+static void dump_dialogex(const dialogex_t *dlgex)
 {
-	control_t *c = dlgex->controls;
+	const control_t *c = dlgex->controls;
 
 	dump_memopt(dlgex->memopt);
 	dump_lvc(&(dlgex->lvc));
 	printf("x, y, w, h: %d, %d, %d, %d\n", dlgex->x, dlgex->y, dlgex->width, dlgex->height);
 	if(dlgex->gotstyle)
-		printf("Style: %08lx\n", dlgex->style);
+	{
+		assert(dlgex->style != NULL);
+		assert(dlgex->style->and_mask == 0);
+		printf("Style: %08x\n", dlgex->style->or_mask);
+	}
 	if(dlgex->gotexstyle)
-		printf("ExStyle: %08lx\n", dlgex->exstyle);
+	{
+		assert(dlgex->exstyle != NULL);
+		assert(dlgex->exstyle->and_mask == 0);
+		printf("ExStyle: %08x\n", dlgex->exstyle->or_mask);
+	}
 	if(dlgex->gothelpid)
-		printf("Helpid: %ld\n", dlgex->helpid);
+		printf("Helpid: %d\n", dlgex->helpid);
 	printf("Menu: %s\n", get_nameid_str(dlgex->menu));
 	printf("Class: %s\n", get_nameid_str(dlgex->dlgclass));
 	printf("Title: "); print_string(dlgex->title); printf("\n");
@@ -591,14 +659,14 @@ void dump_dialogex(dialogex_t *dlgex)
 /*
  *****************************************************************************
  * Function	: dump_menu_item
- * Syntax	: void dump_menu_item(menu_item_t *item)
+ * Syntax	: void dump_menu_item(const menu_item_t *item)
  * Input	:
  * Output	:
  * Description	:
  * Remarks	:
  *****************************************************************************
 */
-void dump_menu_item(menu_item_t *item)
+static void dump_menu_item(const menu_item_t *item)
 {
 	while(item)
 	{
@@ -615,7 +683,7 @@ void dump_menu_item(menu_item_t *item)
 			if(item->name)
 			{
 				print_string(item->name);
-				printf(", %d, %08lx", item->id, item->state);
+				printf(", %d, %08x", item->id, item->state);
 			}
 			else
 				printf("SEPARATOR");
@@ -628,7 +696,7 @@ void dump_menu_item(menu_item_t *item)
 /*
  *****************************************************************************
  * Function	: dump_menu
- * Syntax	: void dump_menu(menu_t *men)
+ * Syntax	: void dump_menu(const menu_t *men)
  * Input	:
  *	men	- Menu resource descriptor
  * Output	:
@@ -636,7 +704,7 @@ void dump_menu_item(menu_item_t *item)
  * Remarks	:
  *****************************************************************************
 */
-void dump_menu(menu_t *men)
+static void dump_menu(const menu_t *men)
 {
 	dump_memopt(men->memopt);
 	dump_lvc(&(men->lvc));
@@ -646,14 +714,14 @@ void dump_menu(menu_t *men)
 /*
  *****************************************************************************
  * Function	: dump_menuex_item
- * Syntax	: void dump_menuex_item(menuex_item_t *item)
+ * Syntax	: void dump_menuex_item(const menuex_item_t *item)
  * Input	:
  * Output	:
  * Description	:
  * Remarks	:
  *****************************************************************************
 */
-void dump_menuex_item(menuex_item_t *item)
+static void dump_menuex_item(const menuex_item_t *item)
 {
 	while(item)
 	{
@@ -664,9 +732,9 @@ void dump_menuex_item(menuex_item_t *item)
 			if(item->gotid)
 				printf(", Id=%d", item->id);
 			if(item->gottype)
-				printf(", Type=%ld", item->type);
+				printf(", Type=%d", item->type);
 			if(item->gotstate)
-				printf(", State=%08lx", item->state);
+				printf(", State=%08x", item->state);
 			if(item->gothelpid)
 				printf(", HelpId=%d", item->helpid);
 			printf("\n");
@@ -681,9 +749,9 @@ void dump_menuex_item(menuex_item_t *item)
 				if(item->gotid)
 					printf(", Id=%d", item->id);
 				if(item->gottype)
-					printf(", Type=%ld", item->type);
+					printf(", Type=%d", item->type);
 				if(item->gotstate)
-					printf(", State=%08lx", item->state);
+					printf(", State=%08x", item->state);
 				if(item->gothelpid)
 					printf(", HelpId=%d", item->helpid);
 			}
@@ -698,7 +766,7 @@ void dump_menuex_item(menuex_item_t *item)
 /*
  *****************************************************************************
  * Function	: dump_menuex
- * Syntax	: void dump_menuex(dialogex_t *menex)
+ * Syntax	: void dump_menuex(const menuex_t *menex)
  * Input	:
  *	menex	- MenuEx resource descriptor
  * Output	:
@@ -706,7 +774,7 @@ void dump_menuex_item(menuex_item_t *item)
  * Remarks	:
  *****************************************************************************
 */
-void dump_menuex(menuex_t *menex)
+static void dump_menuex(const menuex_t *menex)
 {
 	dump_memopt(menex->memopt);
 	dump_lvc(&(menex->lvc));
@@ -716,16 +784,17 @@ void dump_menuex(menuex_t *menex)
 /*
  *****************************************************************************
  * Function	: dump_ver_value
- * Syntax	: void dump_ver_value(ver_value_t *val)
+ * Syntax	: void dump_ver_value(const ver_value_t *val)
  * Input	:
  * Output	:
  * Description	:
  * Remarks	:
  *****************************************************************************
 */
-void dump_ver_value(ver_value_t *val)
+static void dump_ver_block(const ver_block_t *);	/* Forward ref */
+
+static void dump_ver_value(const ver_value_t *val)
 {
-	extern void dump_ver_block(ver_block_t *);
 	if(val->type == val_str)
 	{
 		printf("VALUE ");
@@ -752,16 +821,16 @@ void dump_ver_value(ver_value_t *val)
 /*
  *****************************************************************************
  * Function	: dump_ver_block
- * Syntax	: void dump_ver_block(ver_block_t *blk)
+ * Syntax	: void dump_ver_block(const ver_block_t *blk)
  * Input	:
  * Output	:
  * Description	:
  * Remarks	:
  *****************************************************************************
 */
-void dump_ver_block(ver_block_t *blk)
+static void dump_ver_block(const ver_block_t *blk)
 {
-	ver_value_t *val = blk->values;
+	const ver_value_t *val = blk->values;
 	printf("BLOCK ");
 	print_string(blk->name);
 	printf("\n{\n");
@@ -776,7 +845,7 @@ void dump_ver_block(ver_block_t *blk)
 /*
  *****************************************************************************
  * Function	: dump_versioninfo
- * Syntax	: void dump_versioninfo(versioninfo_t *ver)
+ * Syntax	: void dump_versioninfo(const versioninfo_t *ver)
  * Input	:
  *	ver	- Versioninfo resource descriptor
  * Output	:
@@ -784,9 +853,11 @@ void dump_ver_block(ver_block_t *blk)
  * Remarks	:
  *****************************************************************************
 */
-void dump_versioninfo(versioninfo_t *ver)
+static void dump_versioninfo(const versioninfo_t *ver)
 {
-	ver_block_t *blk = ver->blocks;
+	const ver_block_t *blk = ver->blocks;
+
+	dump_lvc(&(ver->lvc));
 
 	if(ver->gotit.fv)
 		printf("FILEVERSION %04x, %04x, %04x, %04x\n",
@@ -819,18 +890,69 @@ void dump_versioninfo(versioninfo_t *ver)
 
 /*
  *****************************************************************************
- * Function	:
- * Syntax	:
+ * Function	: dump_toolbar_item
+ * Syntax	: void dump_toolbar_item(const toolbar_item_t *item)
  * Input	:
  * Output	:
  * Description	:
  * Remarks	:
  *****************************************************************************
 */
+static void dump_toolbar_items(const toolbar_item_t *items)
+{
+	while(items)
+	{
+	        if(items->id)
+			printf("   BUTTON %d", items->id );
+		else
+	      		printf("   SEPARATOR");
+
+		printf("\n");
+
+		items = items->next;
+	}
+}
+
+/*
+ *****************************************************************************
+ * Function	: dump_toolbar
+ * Syntax	: void dump_toolbar(const toolbar_t *toolbar)
+ * Input	:
+ *	toolbar	- Toolbar resource descriptor
+ * Output	:
+ * Description	:
+ * Remarks	:
+ *****************************************************************************
+*/
+static void dump_toolbar(const toolbar_t *toolbar)
+{
+	dump_memopt(toolbar->memopt);
+	dump_lvc(&(toolbar->lvc));
+	dump_toolbar_items(toolbar->items);
+}
+
+/*
+ *****************************************************************************
+ * Function	: dump_dlginit
+ * Syntax	: void dump_dlginit(const dlginit_t *dit)
+ * Input	:
+ *	dit	- DlgInit resource descriptor
+ * Output	:
+ * Description	:
+ * Remarks	:
+ *****************************************************************************
+*/
+static void dump_dlginit(const dlginit_t *dit)
+{
+	dump_memopt(dit->memopt);
+	dump_lvc(&(dit->data->lvc));
+	dump_raw_data(dit->data);
+}
+
 /*
  *****************************************************************************
  * Function	: dump_resources
- * Syntax	: void dump_resources(resource_t *top)
+ * Syntax	: void dump_resources(const resource_t *top)
  * Input	:
  *	top	- Top of the resource tree
  * Output	:
@@ -839,7 +961,7 @@ void dump_versioninfo(versioninfo_t *ver)
  * Remarks	:
  *****************************************************************************
 */
-void dump_resources(resource_t *top)
+void dump_resources(const resource_t *top)
 {
 	printf("Internal resource-tree dump:\n");
 	while(top)
@@ -897,6 +1019,16 @@ void dump_resources(resource_t *top)
 		case res_ver:
 			dump_versioninfo(top->res.ver);
 			break;
+		case res_dlginit:
+			dump_dlginit(top->res.dlgi);
+			break;
+		case res_toolbar:
+			dump_toolbar(top->res.tbt);
+			break;
+		case res_anicur:
+		case res_aniico:
+			dump_ani_curico(top->res.ani);
+			break;
 		default:
 			printf("Report this: Unknown resource type parsed %08x\n", top->type);
 		}
@@ -904,4 +1036,3 @@ void dump_resources(resource_t *top)
 		top = top->next;
 	}
 }
-
