@@ -272,7 +272,7 @@ static void string_to_upper(string_t *str)
     }
     else
     {
-        internal_error(__FILE__, __LINE__, "Invalid string type %d", str->type);
+        internal_error(__FILE__, __LINE__, "Invalid string type %d\n", str->type);
     }
 }
 
@@ -313,7 +313,7 @@ static void put_string(res_t *res, const string_t *str, enum str_e type, int ist
         {
             if (!check_unicode_conversion( str, newstr, codepage ))
                 error( "String %s does not convert identically to Unicode and back in codepage %d. "
-                       "Try using a Unicode string instead.", str->str.cstr, codepage );
+                       "Try using a Unicode string instead\n", str->str.cstr, codepage );
         }
         if (!isterm) put_word(res, newstr->size);
         for(cnt = 0; cnt < newstr->size; cnt++)
@@ -366,7 +366,7 @@ static void put_name_id(res_t *res, name_id_t *nid, int upcase, const language_t
 	}
 	else
 	{
-		internal_error(__FILE__, __LINE__, "Invalid name_id type %d", nid->type);
+		internal_error(__FILE__, __LINE__, "Invalid name_id type %d\n", nid->type);
 	}
 }
 
@@ -597,7 +597,7 @@ static res_t *dialog2res(name_id_t *name, dialog_t *dlg)
 			if(ctrl->ctlclass)
 				put_name_id(res, ctrl->ctlclass, TRUE, dlg->lvc.language);
 			else
-				internal_error(__FILE__, __LINE__, "Control has no control-class");
+				internal_error(__FILE__, __LINE__, "Control has no control-class\n");
 			if(ctrl->title)
 				put_name_id(res, ctrl->title, FALSE, dlg->lvc.language);
 			else
@@ -665,10 +665,10 @@ static res_t *dialog2res(name_id_t *name, dialog_t *dlg)
 				else if(ctrl->ctlclass->type == name_str)
 					put_name_id(res, ctrl->ctlclass, FALSE, NULL);
 				else
-					error("Unknown control-class %04x", ctrl->ctlclass->name.i_name);
+					error("Unknown control-class %04x\n", ctrl->ctlclass->name.i_name);
 			}
 			else
-				internal_error(__FILE__, __LINE__, "Control has no control-class");
+				internal_error(__FILE__, __LINE__, "Control has no control-class\n");
 			if(ctrl->title)
 				put_name_id(res, ctrl->title, FALSE, NULL);
 			else
@@ -716,7 +716,7 @@ static res_t *dialogex2res(name_id_t *name, dialogex_t *dlgex)
 	{
 		restag = put_res_header(res, WRC_RT_DIALOG, NULL, name, dlgex->memopt, &(dlgex->lvc));
 
-		/* FIXME: MS doc says thet the first word must contain 0xffff
+		/* FIXME: MS doc says that the first word must contain 0xffff
 		 * and the second 0x0001 to signal a DLGTEMPLATEEX. Borland's
 		 * compiler reverses the two words.
 		 * I don't know which one to choose, but I write it as Mr. B
@@ -749,8 +749,8 @@ static res_t *dialogex2res(name_id_t *name, dialogex_t *dlgex)
 		{
 			put_word(res, dlgex->font->size);
 			put_word(res, dlgex->font->weight);
-			/* FIXME: ? TRUE should be sufficient to say that its
-			 * italic, but Borland's compiler says its 0x0101.
+			/* FIXME: ? TRUE should be sufficient to say that it's
+			 * italic, but Borland's compiler says it's 0x0101.
 			 * I just copy it here, and hope for the best.
 			 */
 			put_word(res, dlgex->font->italic ? 0x0101 : 0);
@@ -772,7 +772,7 @@ static res_t *dialogex2res(name_id_t *name, dialogex_t *dlgex)
 			if(ctrl->ctlclass)
 				put_name_id(res, ctrl->ctlclass, TRUE, dlgex->lvc.language);
 			else
-				internal_error(__FILE__, __LINE__, "Control has no control-class");
+				internal_error(__FILE__, __LINE__, "Control has no control-class\n");
 			if(ctrl->title)
 				put_name_id(res, ctrl->title, FALSE, dlgex->lvc.language);
 			else
@@ -1020,7 +1020,7 @@ static res_t *cursorgroup2res(name_id_t *name, cursor_group_t *curg)
 			put_word(res, cur->planes);
 			put_word(res, cur->bits);
 			/* FIXME: The +4 is the hotspot in the cursor resource.
-			 * However, I cound not find this in the documentation.
+			 * However, I could not find this in the documentation.
 			 * The hotspot bytes must either be included or MS
 			 * doesn't care.
 			 */
@@ -1057,7 +1057,7 @@ static res_t *cursorgroup2res(name_id_t *name, cursor_group_t *curg)
 			put_word(res, cur->planes);
 			put_word(res, cur->bits);
 			/* FIXME: The +4 is the hotspot in the cursor resource.
-			 * However, I cound not find this in the documentation.
+			 * However, I could not find this in the documentation.
 			 * The hotspot bytes must either be included or MS
 			 * doesn't care.
 			 */
@@ -1454,7 +1454,7 @@ static res_t *stringtable2res(stringtable_t *stt)
 	{
 		if(!stt->nentries)
 		{
-			warning("Empty internal stringtable");
+			warning("Empty internal stringtable\n");
 			continue;
 		}
 		name.type = name_ord;
@@ -1591,7 +1591,7 @@ static void versionblock2res(res_t *res, ver_block_t *blk, int level, const lang
 		}
 		else
 		{
-			internal_error(__FILE__, __LINE__, "Invalid value indicator %d in VERSIONINFO", val->type);
+			internal_error(__FILE__, __LINE__, "Invalid value indicator %d in VERSIONINFO\n", val->type);
 		}
 	}
 
@@ -1794,7 +1794,7 @@ char *prep_nid_for_label(const name_id_t *nid)
 			if((unsigned)*sptr < 0x80 && isprint(*sptr & 0xff))
 				buf[i] = *sptr++;
 			else
-				warning("Resourcename (str_unicode) contain unprintable characters or invalid translation, ignored");
+				warning("Resourcename (str_unicode) contain unprintable characters or invalid translation, ignored\n");
 		}
 		buf[i] = '\0';
 	}
@@ -1809,7 +1809,7 @@ char *prep_nid_for_label(const name_id_t *nid)
 			if((unsigned)*cptr < 0x80 && isprint(*cptr & 0xff))
 				buf[i] = *cptr++;
 			else
-				warning("Resourcename (str_char) contain unprintable characters, ignored");
+				warning("Resourcename (str_char) contain unprintable characters, ignored\n");
 		}
 		buf[i] = '\0';
 	}
@@ -1819,7 +1819,7 @@ char *prep_nid_for_label(const name_id_t *nid)
 	}
 	else
 	{
-		internal_error(__FILE__, __LINE__, "Resource name_id with invalid type %d", nid->type);
+		internal_error(__FILE__, __LINE__, "Resource name_id with invalid type %d\n", nid->type);
 	}
 	return buf;
 }
@@ -2000,7 +2000,7 @@ void resources2res(resource_t *top)
 			    top->binres = anicurico2res(top->name, top->res.ani, top->type);
 			break;
 		default:
-			internal_error(__FILE__, __LINE__, "Unknown resource type encountered %d in binary res generation", top->type);
+			internal_error(__FILE__, __LINE__, "Unknown resource type encountered %d in binary res generation\n", top->type);
 		}
 		top->c_name = make_c_name(get_c_typename(top->type), top->name, top->lan);
 		top = top->next;
